[^:]*: Assembler messages:
.*: Error: symbol definition loop encountered at `aaa'
#...
.*: Error: symbol definition loop encountered at `iii'
#...
.*: Error: symbol definition loop encountered at `xxx'
#pass
