[^:]*: Assembler messages:
[^:]*:4: Error: selected processor does not support `mrrc [^']*' in (ARM|Thumb) mode
[^:]*:5: Error: selected processor does not support `mcrr [^']*' in (ARM|Thumb) mode
[^:]*:6: Error: selected processor does not support `mrrc [^']*' in (ARM|Thumb) mode
[^:]*:7: Error: selected processor does not support `mcrr [^']*' in (ARM|Thumb) mode
