.*: Assembler messages:
.*.s:6: Error: [Ii]ndex must be a general register
.*.s:7: Error: [Ii]ndex must be a general register
.*.s:8: Error: [Ii]ndex must be a general register
.*.s:9: Error: [Ii]ndex must be a general register
.*.s:13: Error: [Ii]ndirect register index must be a general register
.*.s:14: Error: [Ii]ndirect register index must be a general register
.*.s:15: Error: [Ii]ndirect register index must be a general register
.*.s:16: Error: [Ii]ndirect register index must be a general register
.*.s:20: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:21: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:22: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:23: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:24: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:25: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:27: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:28: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:29: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:30: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:31: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:32: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:37: Error: [Rr]otating register index must be a non-negative constant
.*.s:39: Error: [Ii]ndex out of range 0\.\.[[:digit:]]+
.*.s:40: Error: [Rr]otating register index must be a non-negative constant
.*.s:41: Error: [Rr]otating register index must be a non-negative constant
.*.s:42: Error: [Rr]otating register index must be a non-negative constant
.*.s:44: Error: [Ii]ndirect register index must be a general register
.*.s:45: Error: [Ii]ndirect register index must be a general register
.*.s:46: Error: [Ii]ndirect register index must be a general register
.*.s:47: Error: [Ii]ndirect register index must be a general register
.*.s:51: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:52: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:53: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:54: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:55: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:56: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:58: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:59: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:60: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:61: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:62: Error: [Ii]ndex can only be applied to rotating or indirect registers
.*.s:63: Error: [Ii]ndex can only be applied to rotating or indirect registers
