.*: Assembler messages:
.*:6: Error: invalid unextended operand value
.*:8: Error: invalid unextended operand value
.*:10: Error: invalid unextended operand value
.*:13: Error: invalid unextended operand value
.*:15: Error: invalid unextended operand value `lb \$4,%hi\(baz\)\(\$5\)'
.*:16: Error: invalid unextended operand value
.*:23: Error: invalid unextended operand value
.*:25: Error: invalid unextended operand value `sd \$31,%lo\(baz\)\(\$29\)'
.*:26: Error: invalid unextended operand value
.*:28: Error: invalid unextended operand value `sd \$4,%lo\(baz\)\(\$29\)'
.*:31: Error: invalid unextended operand value
.*:33: Error: invalid unextended operand value
.*:35: Error: invalid unextended operand value `lh \$6,%lo\(baz\)\(\$7\)'
.*:36: Error: invalid unextended operand value
.*:38: Error: invalid unextended operand value `addiu \$29,%lo\(baz\)'
.*:39: Error: invalid unextended operand value
.*:41: Error: invalid unextended operand value `cmpi \$2,%hi\(baz\)'
.*:42: Error: invalid unextended operand value
.*:44: Error: invalid unextended operand value `addiu \$3,\$pc,%lo\(baz\)'
.*:45: Error: invalid unextended operand value
.*:47: Error: invalid unextended operand value `daddiu \$4,\$pc,%lo\(baz\)'
.*:49: Error: invalid unextended operand value
.*:51: Error: invalid unextended operand value `daddiu \$5,%lo\(baz\)'
.*:52: Error: invalid unextended operand value
.*:54: Error: invalid unextended operand value `addiu \$2,%lo\(baz\)'
.*:7: Error: invalid unextended operand value
.*:9: Error: invalid unextended operand value
.*:11: Error: invalid unextended operand value
.*:14: Error: invalid unextended operand value
.*:17: Error: invalid unextended operand value
.*:19: Error: invalid unextended operand value
.*:20: Error: invalid unextended operand value
.*:21: Error: invalid unextended operand value
.*:22: Error: invalid unextended operand value
.*:24: Error: invalid unextended operand value
.*:27: Error: invalid unextended operand value
.*:29: Error: invalid unextended operand value
.*:30: Error: invalid unextended operand value
.*:32: Error: invalid unextended operand value
.*:34: Error: invalid unextended operand value
.*:37: Error: invalid unextended operand value
.*:40: Error: invalid unextended operand value
.*:43: Error: invalid unextended operand value
.*:46: Error: invalid unextended operand value
.*:50: Error: invalid unextended operand value
.*:53: Error: invalid unextended operand value
.*:55: Error: invalid unextended operand value
.*:56: Error: invalid unextended operand value
