[^:]*: Assembler messages:
[^:]*:6: Error: r0-r12, lr or APSR expected -- `clrm {}'
[^:]*:7: Error: r0-r12, lr or APSR expected -- `clrm {sp}'
[^:]*:8: Error: r0-r12, lr or APSR expected -- `clrm {pc}'
[^:]*:10: Error: VFP single precision register or VPR expected -- `vscclrm {}'
[^:]*:11: Error: VPR expected last -- `vscclrm {s0}'
[^:]*:12: Error: VFP single precision register or VPR expected -- `vscclrm {s1,d1,VPR}'
[^:]*:13: Error: VFP single precision register expected -- `vscclrm {s1-d1,VPR}'
[^:]*:15: Error: syntax error -- `vldr APSR,\[r2\]'
[^:]*:20: Error: syntax error -- `vstr APSR,\[r2\]'
[^:]*:16: Error: co-processor offset out of range
[^:]*:17: Error: co-processor offset out of range
[^:]*:18: Error: co-processor offset out of range
[^:]*:21: Error: co-processor offset out of range
[^:]*:22: Error: co-processor offset out of range
[^:]*:23: Error: co-processor offset out of range
