! Copyright 2010-2022 Free Software Foundation, Inc.
! 
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 3 of the License, or
! (at your option) any later version.
! 
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <http://www.gnu.org/licenses/>.

        use lib
	use libmany, only: var_j
        if (var_i .ne. 1) call abort
	call lib_func
        if (var_i .ne. 2) call abort
        if (var_j .ne. 3) call abort
        if (var_i .eq. 2) write (*,*) 'nop'          ! i-is-2-in-main
end
