# name: ARC object attribute names
# source: attr-name.s
# as:
# readelf: -A

Attribute Section: ARC
File Attributes
  Tag_ARC_PCS_config: Bare metal/mwdt
  Tag_ARC_CPU_base: ARC6xx
  Tag_ARC_CPU_variation: Core1
  Tag_ARC_CPU_name: "random-cpu"
  Tag_ARC_ABI_rf16: yes
  Tag_ARC_ABI_osver: v3
  Tag_ARC_ABI_sda: GNU
  Tag_ARC_ABI_pic: GNU
  Tag_ARC_ABI_tls: r25
  Tag_ARC_ABI_enumsize: default
  Tag_ARC_ABI_exceptions: OPTFP
  Tag_ARC_ABI_double_size: 8
  Tag_ARC_ISA_config: "CD,FPUDA"
  Tag_ARC_ISA_apex: "QUARKSE"
  Tag_ARC_ISA_mpy_option: 6
  Tag_ARC_ATR_version: 1
