[^:]*: Assembler messages:
[^:]*:24: Warning: instruction is UNPREDICTABLE with SP operand
[^:]*:25: Warning: instruction is UNPREDICTABLE with PC operand
[^:]*:26: Error: bad type in SIMD instruction -- `vfma.f64 q0,q1,q2'
[^:]*:27: Error: bad type in SIMD instruction -- `vfma.32 q0,q1,q2'
[^:]*:28: Error: bad type in SIMD instruction -- `vfms.f64 q0,q1,q2'
[^:]*:29: Error: bad type in SIMD instruction -- `vfms.32 q0,q1,q2'
[^:]*:31: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:31: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:31: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:31: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:31: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:31: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:32: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:32: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:32: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:32: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:32: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:32: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:33: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:33: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:33: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:33: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:33: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:33: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:35: Error: syntax error -- `vfmaeq.f16 q0,q1,q2'
[^:]*:36: Error: syntax error -- `vfmaeq.f16 q0,q1,q2'
[^:]*:38: Error: syntax error -- `vfmaeq.f16 q0,q1,q2'
[^:]*:39: Error: vector predicated instruction should be in VPT/VPST block -- `vfmat.f16 q0,q1,q2'
[^:]*:41: Error: instruction missing MVE vector predication code -- `vfma.f16 q0,q1,q2'
[^:]*:43: Error: syntax error -- `vfmseq.f16 q0,q1,q2'
[^:]*:44: Error: syntax error -- `vfmseq.f16 q0,q1,q2'
[^:]*:46: Error: syntax error -- `vfmseq.f16 q0,q1,q2'
[^:]*:47: Error: vector predicated instruction should be in VPT/VPST block -- `vfmst.f16 q0,q1,q2'
[^:]*:49: Error: instruction missing MVE vector predication code -- `vfms.f16 q0,q1,q2'
