.*: Assembler messages:
.*:71: Warning: no .cprestore pseudo-op used in PIC code
.*:69: Warning: relaxed out-of-range branch into a jump
.*:73: Warning: relaxed out-of-range branch into a jump
.*:75: Warning: relaxed out-of-range branch into a jump
.*:77: Warning: relaxed out-of-range branch into a jump
.*:80: Warning: relaxed out-of-range branch into a jump
.*:83: Warning: relaxed out-of-range branch into a jump
.*:85: Warning: relaxed out-of-range branch into a jump
.*:87: Warning: relaxed out-of-range branch into a jump
.*:89: Warning: relaxed out-of-range branch into a jump
.*:91: Warning: relaxed out-of-range branch into a jump
.*:93: Warning: relaxed out-of-range branch into a jump
.*:95: Warning: relaxed out-of-range branch into a jump
.*:97: Warning: relaxed out-of-range branch into a jump
.*:99: Warning: relaxed out-of-range branch into a jump
.*:101: Warning: relaxed out-of-range branch into a jump
.*:104: Warning: relaxed out-of-range branch into a jump
.*:106: Warning: relaxed out-of-range branch into a jump
.*:109: Warning: relaxed out-of-range branch into a jump
.*:111: Warning: relaxed out-of-range branch into a jump
.*:113: Warning: relaxed out-of-range branch into a jump
.*:115: Warning: relaxed out-of-range branch into a jump
.*:117: Warning: relaxed out-of-range branch into a jump
.*:119: Warning: relaxed out-of-range branch into a jump
.*:121: Warning: relaxed out-of-range branch into a jump
.*:123: Warning: relaxed out-of-range branch into a jump
.*:125: Warning: relaxed out-of-range branch into a jump
.*:127: Warning: relaxed out-of-range branch into a jump
.*:129: Warning: relaxed out-of-range branch into a jump
.*:131: Warning: relaxed out-of-range branch into a jump
.*:133: Warning: relaxed out-of-range branch into a jump
.*:135: Warning: relaxed out-of-range branch into a jump
.*:137: Warning: relaxed out-of-range branch into a jump
.*:139: Warning: relaxed out-of-range branch into a jump
.*:141: Warning: relaxed out-of-range branch into a jump
.*:143: Warning: relaxed out-of-range branch into a jump
.*:145: Warning: relaxed out-of-range branch into a jump
.*:147: Warning: relaxed out-of-range branch into a jump
.*:149: Warning: relaxed out-of-range branch into a jump
.*:151: Warning: relaxed out-of-range branch into a jump
.*:153: Warning: relaxed out-of-range branch into a jump
.*:155: Warning: relaxed out-of-range branch into a jump
.*:157: Warning: relaxed out-of-range branch into a jump
.*:159: Warning: relaxed out-of-range branch into a jump
.*:161: Warning: relaxed out-of-range branch into a jump
.*:163: Warning: relaxed out-of-range branch into a jump
.*:165: Warning: relaxed out-of-range branch into a jump
.*:167: Warning: relaxed out-of-range branch into a jump
.*:169: Warning: relaxed out-of-range branch into a jump
