#objdump: -d

.*: +file format .*arc.*


Disassembly of section .text:

[0-9a-f]+ <start>:
   0:	3921 0080           	insn1	r0,r1,r2
   4:	382f 0061           	insn2	r0,r1
   8:	392f 407f           	insn3	r1
   c:	396f 403f           	insn4
