; Test C64x+ dind compact instruction format
	.text
	.nocmp
dind:
	nop
	.align 16
	nop
	.align 16
	.short 0x0404
	.short 0x1404 
	.short 0x0604
	.short 0x040c 
	.short 0x0405
	.short 0x1405 
	.short 0x0605
	.short 0x040d
	.short 0x1405 
	.short 0x0405
	.short 0x160d 
	.short 0x260d
	.short 0x361d 
	.short 0x261d
	.word  0xefe00000
	.short 0x0404
	.short 0x1404 
	.short 0x0604
	.short 0x040c 
	.short 0x0405
	.short 0x1405 
	.short 0x0605
	.short 0x040d
	.short 0x1405 
	.short 0x0605
	.short 0x160d 
	.short 0x260d
	.short 0x361d 
	.short 0x261d
	.word  0xefe8c000
	.short 0x0404
	.short 0x1404 
	.short 0x0604
	.short 0x040c 
	.short 0x0405
	.short 0x1405 
	.short 0x0605
	.short 0x040d
	.short 0x1405 
	.short 0x0605
	.short 0x160d 
	.short 0x260d
	.short 0x361d 
	.short 0x261d
	.word  0xefe9c000
	.short 0x0404
	.short 0x1404 
	.short 0x0604
	.short 0x040c 
	.short 0x0405
	.short 0x1405 
	.short 0x0605
	.short 0x040d
	.short 0x1405 
	.short 0x0605
	.short 0x160d 
	.short 0x260d
	.short 0x361d 
	.short 0x261d
	.word  0xefeac000
	.short 0x0604
	.short 0x1604 
	.short 0x0604
	.short 0x040c 
	.short 0x0405
	.short 0x1405 
	.short 0x0605
	.short 0x040d
	.short 0x1405 
	.short 0x0605
	.short 0x160d 
	.short 0x260d
	.short 0x361d 
	.short 0x261d
	.word  0xefebc000
	.short 0x0404
	.short 0x1404 
	.short 0x0604
	.short 0x040c 
	.short 0x0405
	.short 0x1405 
	.short 0x0605
	.short 0x040d
	.short 0x1405 
	.short 0x0605
	.short 0x160d 
	.short 0x260d
	.short 0x361d 
	.short 0x261d
	.word  0xefecc000
	.short 0x0404
	.short 0x1404 
	.short 0x0604
	.short 0x040c 
	.short 0x0405
	.short 0x1405 
	.short 0x0605
	.short 0x040d
	.short 0x1405 
	.short 0x0605
	.short 0x160d 
	.short 0x260d
	.short 0x361d 
	.short 0x261d
	.word  0xefedc000
	.short 0x0404
	.short 0x1404 
	.short 0x0604
	.short 0x040c 
	.short 0x0405
	.short 0x1405 
	.short 0x0605
	.short 0x040d
	.short 0x1405 
	.short 0x0605
	.short 0x160d 
	.short 0x260d
	.short 0x361d 
	.short 0x261d
	.word  0xefeec000
	.short 0x0404
	.short 0x1404 
	.short 0x0604
	.short 0x040c 
	.short 0x0405
	.short 0x1405 
	.short 0x0605
	.short 0x040d
	.short 0x1405 
	.short 0x0605
	.short 0x160d 
	.short 0x260d
	.short 0x361d 
	.short 0x261d
	.word  0xefefc000
	.short 0x0414
	.short 0x1414 
	.short 0x0614
	.short 0x041c 
	.short 0x0415
	.short 0x1415 
	.short 0x0615
	.short 0x041d
	.short 0x1415 
	.short 0x0615
	.short 0x161d 
	.short 0x261d
	.short 0x361d 
	.short 0x261d
	.word  0xefefc000
