.*: Assembler messages:
.*:2: Error: opcode not supported on this processor: .* \(.*\) `daddi \$23,\$24,1023'
.*:3: Error: invalid operands `ddiv \$2,\$4'
.*:4: Error: invalid operands `ddivu \$2,\$4'
.*:5: Error: opcode not supported on this processor: .* \(.*\) `dmult \$2,\$3'
.*:6: Error: opcode not supported on this processor: .* \(.*\) `dmultu \$2,\$3'
.*:7: Error: opcode not supported on this processor: .* \(.*\) `ldl \$2,1\(\$3\)'
.*:8: Error: opcode not supported on this processor: .* \(.*\) `ldr \$2,1\(\$3\)'
.*:9: Error: opcode not supported on this processor: .* \(.*\) `sdl \$2,1\(\$3\)'
.*:10: Error: opcode not supported on this processor: .* \(.*\) `sdr \$2,1\(\$3\)'
