! Copyright 2009-2022 Free Software Foundation, Inc.
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 3 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <http://www.gnu.org/licenses/>.

module mod1
  integer :: var_i = 1
  integer :: var_const
  parameter (var_const = 20)
contains
  subroutine check_all
    if (var_i .ne. 1) call abort
    if (var_const .ne. 20) call abort
  end subroutine check_all
end module mod1

module mod2
  integer :: var_i = 2
contains
    subroutine check_var_i
    if (var_i .ne. 2) call abort
  end subroutine check_var_i
end module mod2
