	.text
	.syntax unified
	stmfd	sp!, {r0}
	stmfd	sp!, {r1}
	stmfd	sp!, {r2}
	stmfd	sp!, {r3}
	stmfd	sp!, {r4}
	stmfd	sp!, {r5}
	stmfd	sp!, {r6}
	stmfd	sp!, {r7}
	stmfd	sp!, {r8}
	stmfd	sp!, {r9}
	stmfd	sp!, {sl}
	stmfd	sp!, {fp}
	stmfd	sp!, {ip}
	stmfd	sp!, {sp}
	stmfd	sp!, {lr}
	stmfd	sp!, {pc}
	stmfd	sp!, {r1, r2, r3}
	ldmia sp!, {r1, r2, r3}
	ldmia sp!, {r0}
	ldmia sp!, {r1}
	ldmia sp!, {r2}
	ldmia sp!, {r3}
	ldmia sp!, {r4}
	ldmia sp!, {r5}
	ldmia sp!, {r6}
	ldmia sp!, {r7}
	ldmia sp!, {r8}
	ldmia sp!, {r9}
	ldmia sp!, {sl}
	ldmia sp!, {fp}
	ldmia sp!, {ip}
	ldmia sp!, {sp}
	ldmia sp!, {lr}
	ldmia sp!, {pc}
