.*illegal.s: Assembler messages:
.*illegal.s:5: Error: unknown register r56
.*illegal.s:8: Error: expecting \( near 0x1000
.*illegal.s:8: Error: missing argument
.*illegal.s:9: Error: expecting \) near r5
.*illegal.s:10: Error: expecting \( near 0x1000r5\)
.*illegal.s:10: Error: missing argument
.*illegal.s:11: Error: expecting \( near 0x1000,r5
.*illegal.s:11: Error: missing argument
.*illegal.s:12: Error: unknown register 0x1000
.*illegal.s:14: Error: unrecognised instruction fop
.*illegal.s:16: Error: too many arguments
.*illegal.s:17: Error: too many arguments
.*illegal.s:17: Error: unknown register r2,r4
