/* Copyright 2016-2022 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

struct foo
{
  int a;
  int x[5];
  struct foo *y;
};

void
func01 (void)
{
  struct foo *ptr;

  asm ("func01_label: .globl func01_label");
  ptr = (struct foo *) 0x0;
  return;
}

int
main ()
{
  asm ("main_label: .globl main_label");
  func01 ();
  return 0;
}
