.*Assembler messages:
.*Error: unrecognized opcode `r,0x00000013'
.*Error: values must be constant `0x4,rs1'
.*Error: unrecognized values `0x4 0x5'
.*Error: unrecognized values `0x4,0x5,0x6'
.*Error: value conflicts with instruction length `0x4,0x0001'
.*Error: value conflicts with instruction length `0x2,0x00000013'
