	hfence.vvma
	hfence.vvma	a0
	hfence.vvma	x0, a1
	hfence.vvma	a1, a2
	hfence.gvma
	hfence.gvma	a0
	hfence.gvma	x0, a1
	hfence.gvma	a1, a2

	hlv.b   a0, (a1)
	hlv.b   a0,0(a1)
	hlv.bu  a0, (a1)
	hlv.h   a1, (a2)
	hlv.hu  a1, (a1)
	hlvx.hu a1, (a2)
	hlv.w   a2, (a2)
	hlv.wu  a2, (a3)
	hlvx.wu a2, (a3)
	hlv.d   a3, (a4)
	hsv.b   a0, (a1)
	hsv.b   a0,0(a1)
	hsv.h   a0, (a1)
	hsv.w   a0, (a1)
	hsv.d   a0, (a1)
