[^:]*: Assembler messages:
[^:]*:13: Error: bad type in SIMD instruction -- `vrintn.i16 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrintn.f64 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrintx.i16 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrintx.f64 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrinta.i16 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrinta.f64 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrintz.i16 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrintz.f64 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrintm.i16 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrintm.f64 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrintp.i16 q0,q1'
[^:]*:13: Error: bad type in SIMD instruction -- `vrintp.f64 q0,q1'
[^:]*:14: Error: VFP single, double or Neon quad precision register expected -- `vrintr.f16 q0,q1'
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Error: syntax error -- `vrintneq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintneq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintneq.f16 q0,q1'
[^:]*:25: Error: vector predicated instruction should be in VPT/VPST block -- `vrintnt.f16 q0,q1'
[^:]*:25: Error: instruction missing MVE vector predication code -- `vrintn.f16 q0,q1'
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Error: syntax error -- `vrintxeq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintxeq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintxeq.f16 q0,q1'
[^:]*:25: Error: vector predicated instruction should be in VPT/VPST block -- `vrintxt.f16 q0,q1'
[^:]*:25: Error: instruction missing MVE vector predication code -- `vrintx.f16 q0,q1'
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Error: syntax error -- `vrintaeq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintaeq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintaeq.f16 q0,q1'
[^:]*:25: Error: vector predicated instruction should be in VPT/VPST block -- `vrintat.f16 q0,q1'
[^:]*:25: Error: instruction missing MVE vector predication code -- `vrinta.f16 q0,q1'
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Error: syntax error -- `vrintzeq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintzeq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintzeq.f16 q0,q1'
[^:]*:25: Error: vector predicated instruction should be in VPT/VPST block -- `vrintzt.f16 q0,q1'
[^:]*:25: Error: instruction missing MVE vector predication code -- `vrintz.f16 q0,q1'
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Error: syntax error -- `vrintmeq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintmeq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintmeq.f16 q0,q1'
[^:]*:25: Error: vector predicated instruction should be in VPT/VPST block -- `vrintmt.f16 q0,q1'
[^:]*:25: Error: instruction missing MVE vector predication code -- `vrintm.f16 q0,q1'
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:25: Error: syntax error -- `vrintpeq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintpeq.f16 q0,q1'
[^:]*:25: Error: syntax error -- `vrintpeq.f16 q0,q1'
[^:]*:25: Error: vector predicated instruction should be in VPT/VPST block -- `vrintpt.f16 q0,q1'
[^:]*:25: Error: instruction missing MVE vector predication code -- `vrintp.f16 q0,q1'
