       .arch   armv6t2

foo:
       srsdb   r13, #13
       srsdb   r13!, #13
       srsia   r13, #13
       srsia   r13!, #13
       srsdb   r4, #13
       srsia   r4, #13
       srsea   r13, #13
       srsea   r13!, #13
       srsfd   r13, #13
       srsfd   r13!, #13
       srs     r13, #13
       srs     r13!, #13
       srsea   r4, #13
       srsfd   r4, #13
       srs     r4, #13

