// Copyright (C) March 2023-2024 Linaro Limited (or its affiliates). All rights reserved.

// Copyright (C) 1995-2023 Arm Limited (or its affiliates). All rights reserved.

// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
#include <cstdio>

class MyClass
{
public:
  MyClass()
  {
    printf("this: %p\n", this); /* BP-1 */
  }

  void method()
  {
    printf("this: %p\n", this); /* BP-3 */
  }
};

int
main (void)
{
  MyClass myClass; 
  printf("&myClass: %p\n", &myClass); /* BP-2 */
  myClass.method();
  return 0;
}
