.*: Assembler messages:
.*:6: Error: branch out of range or not a multiple of 2
.*:7: Error: branch out of range or not a multiple of 2
.*:8: Error: branch out of range or not a multiple of 2
.*:9: Error: branch out of range or not a multiple of 2
.*:11: Error: r15 not allowed here -- `bfx 2,PC'
.*:12: Error: r13 not allowed here -- `bflx 2,SP'
