	.allow_index_reg
	.text
	
_start:
    BEXTR    $0x67,(%edx,%esi,8),%ebx
    BEXTR    $0x0,%esi,%eax
    BEXTR    $0x7FFFFFFF,%eax,%edi
    BEXTR    $0x35B2,(%esi),%esp
    BEXTR    $0x9C86,%edi,%ebp
    BEXTR    $0x3,%ecx,%ecx
    BEXTR    $0xEE,-0x3(%ebx,%eax,2),%esi
    BEXTR    $0x55,(%ebx),%esp
    BEXTR    $0x4EE8,(%edx),%edx
    BEXTR    $0x0,%ebx,%edi
    BEXTR    $0xDC,%esp,%esi
    BEXTR    $0xA9,(%eax),%eax
    BEXTR    $0x189,%edx,%ebp
    BEXTR    $0x84,0x0(%ecx,%eax,2),%ecx
    BEXTR    $0xCAFE,(%ecx,%eax),%eax
    BEXTR    $0xDEAD,0x7109(%esi,%edi),%edi
    BLCFILL  (%ecx),%eax
    BLCFILL  %esi,%edi
    BLCFILL  %eax,%ecx
    BLCFILL  %edi,%esi
    BLCFILL  (%esi),%esp
    BLCFILL  (%ebx),%ebp
    BLCFILL  0x1A95(%ebx,%eax),%edx
    BLCFILL  (%edx),%edi
    BLCFILL  %ebx,%edi
    BLCFILL  0xCE(%eax,%esi),%eax
    BLCFILL  -0xCAFE(,%ebx,1),%eax
    BLCFILL  -0xAE5F(,%eax),%ebx
    BLCFILL  %ecx,%edi
    BLCFILL  %esp,%eax
    BLCFILL  %ebp,%edi
    BLCFILL  (%esi,%ecx,2),%eax
    BLCI     %eax,%ecx
    BLCI     %ecx,%ebx
    BLCI     0x12B0(,%eax,2),%eax
    BLCI     (%eax),%edi
    BLCI     %edi,%esi
    BLCI     %esp,%edx
    BLCI     %esi,%ebp
    BLCI     %edx,%eax
    BLCI     -0x72A9(%ebx,%eax,4),%esp
    BLCI     (%esi),%ebx
    BLCI     (%ebx,%esi,2),%eax
    BLCI     (%ebx),%edx
    BLCI     %ebx,%eax
    BLCI     0xE0A2(%ebx,%edx,4),%ecx
    BLCI     (%edi),%edi
    BLCI     0x3FFFFFFF(,%eax,2),%eax
    BLCIC    %edi,%ecx
    BLCIC    %eax,%edi
    BLCIC    (%eax),%ebx
    BLCIC    %ecx,%edx
    BLCIC    %esi,%esp
    BLCIC    -0xCAFE(,%ebx),%ebp
    BLCIC    %ebp,%eax
    BLCIC    (%esi),%esi
    BLCIC    %esp,%ebx
    BLCIC    0x0(%edi,%edi,1),%esi
    BLCIC    -0x3FFFFFFF(,%esi),%ebp
    BLCIC    (%ebx),%edi
    BLCIC    0x8(%edi,%eax,8),%eax
    BLCIC    0x3A574AD1(%ecx),%edi
    BLCIC    %esp,%edi
    BLCIC    %edx,%edi
    BLCMSK   0xC(%eax),%edi
    BLCMSK   (%esi,%edx),%ebp
    BLCMSK   -0x1DC2DE00(%edi),%ecx
    BLCMSK   %eax,%esp
    BLCMSK   0x0(%edi,%edx,2),%eax
    BLCMSK   (%ebx),%edx
    BLCMSK   (%edx),%edi
    BLCMSK   %esi,%esi
    BLCMSK   %esp,%edi
    BLCMSK   %edi,%esp
    BLCMSK   -0x0(%ebx,%eax,8),%ebx
    BLCMSK   (%edi),%eax
    BLCMSK   %edx,%eax
    BLCMSK   0x67(%ebx,%edi),%edi
    BLCMSK   -0x55ED2760(,%eax),%edi
    BLCMSK   0x1(,%eax),%eax
    BLCS     %edx,%esi
    BLCS     (%ebx),%eax
    BLCS     %eax,%edi
    BLCS     0xCAFE(%ecx,%eax),%esp
    BLCS     %edi,%ebp
    BLCS     (%edx),%ecx
    BLCS     (%edi),%edi
    BLCS     -0xCAFE(%ebx),%ebx
    BLCS     %esp,%ecx
    BLCS     %esi,%edx
    BLCS     (%eax),%edi
    BLCS     0x1(,%ecx,1),%edi
    BLCS     %ecx,%eax
    BLCS     (%ebx,%edx),%edi
    BLCS     -0xDEAD(%eax,%eax),%eax
    BLCS     0x0(%ebx,%edx),%edi
    BLSFILL  %eax,%eax
    BLSFILL  %ecx,%esi
    BLSFILL  (%eax),%edi
    BLSFILL  %ebx,%esp
    BLSFILL  %edx,%edx
    BLSFILL  (%ecx),%ecx
    BLSFILL  %edi,%edi
    BLSFILL  %ebp,%ebp
    BLSFILL  (%edi),%edi
    BLSFILL  (%ebx),%ebx
    BLSFILL  (%esi),%eax
    BLSFILL  (%eax,%eax,4),%eax
    BLSFILL  %esi,%edi
    BLSFILL  0xA221(%eax,%ebx),%edi
    BLSFILL  (%eax,%eax,1),%eax
    BLSFILL  -0x8(,%ebx,2),%ecx
    BLSIC    %eax,%edi
    BLSIC    (%esi),%ebx
    BLSIC    (,%ebx,2),%ebp
    BLSIC    (%ecx,%eax,2),%eax
    BLSIC    (%edi),%esp
    BLSIC    (%ebx),%eax
    BLSIC    %edi,%ecx
    BLSIC    0x51(%eax,%ebx,1),%edi
    BLSIC    %esp,%edx
    BLSIC    -0x67(%esi,%edi),%edx
    BLSIC    (%ecx),%edi
    BLSIC    0x67(%esi,%ecx,4),%esi
    BLSIC    0x81(%ebx,%edx,8),%edi
    BLSIC    0xE(%ecx,%edx),%edi
    BLSIC    0x3B(%eax),%esp
    BLSIC    %ecx,%edi
    T1MSKC   %eax,%eax
    T1MSKC   %edi,%edi
    T1MSKC   (%ecx),%ecx
    T1MSKC   (%ebx,%esi,1),%esi
    T1MSKC   %edx,%ebp
    T1MSKC   0x0(,%ecx,1),%edx
    T1MSKC   (,%esi,4),%esp
    T1MSKC   %ebx,%ecx
    T1MSKC   (%ebx),%ebx
    T1MSKC   %esp,%edi
    T1MSKC   (%eax),%edi
    T1MSKC   %ecx,%eax
    T1MSKC   0xDEAD(%eax),%edi
    T1MSKC   %ecx,%edx
    T1MSKC   0xDEAD(,%edx),%ebx
    T1MSKC   (%edx),%edi
    TZMSK    (%ebx),%esp
    TZMSK    %edi,%eax
    TZMSK    -0xCAFE(%edi),%esi
    TZMSK    (,%edi,1),%edx
    TZMSK    %eax,%ebp
    TZMSK    %ebp,%ebx
    TZMSK    (%esi),%edi
    TZMSK    (%ecx),%ecx
    TZMSK    (,%eax,2),%edi
    TZMSK    %edi,%edi
    TZMSK    %esp,%edx
    TZMSK    (%eax),%ecx
    TZMSK    (%edx,%edi),%eax
    TZMSK    (%ebx),%eax
    TZMSK    0x2A2AC6D9(%ebx),%eax
    TZMSK    -0x16B9(%ecx,%eax,1),%ecx
