[^:]*: Assembler messages:
[^:]*:6: Error: registers may not be the same -- `mrrc p0,#1,r1,r1,c4'
[^:]*:7: Error: registers may not be the same -- `mrrc2 p0,#1,r1,r1,c4'
[^:]*:20: Error: registers may not be the same -- `mrrc p0,#1,r1,r1,c4'
[^:]*:21: Error: registers may not be the same -- `mrrc2 p0,#1,r1,r1,c4'
