[^:]*: Assembler messages:
[^:]*:10: Error: selected FPU does not support instruction -- `vmul.f16 q0,q1,q2'
[^:]*:11: Error: selected FPU does not support instruction -- `vmul.f16 q0,q1,r2'
[^:]*:12: Error: selected FPU does not support instruction -- `vmul.f32 q0,q1,q2'
[^:]*:13: Error: selected FPU does not support instruction -- `vmul.f32 q0,q1,r2'
[^:]*:14: Error: bad type in SIMD instruction -- `vmul.i64 q0,q1,q2'
[^:]*:15: Error: bad type in SIMD instruction -- `vmul.i64 q0,q1,r2'
[^:]*:16: Warning: instruction is UNPREDICTABLE with PC operand
[^:]*:17: Warning: instruction is UNPREDICTABLE with SP operand
[^:]*:18: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:18: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:18: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:18: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:18: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:18: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:19: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:19: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:19: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:19: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:19: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:19: Warning: instruction is UNPREDICTABLE in an IT block
[^:]*:21: Error: syntax error -- `vmuleq.i32 q0,q1,q2'
[^:]*:22: Error: syntax error -- `vmuleq.i32 q0,q1,q2'
[^:]*:24: Error: syntax error -- `vmuleq.i32 q0,q1,q2'
[^:]*:25: Error: vector predicated instruction should be in VPT/VPST block -- `vmult.i32 q0,q1,q2'
[^:]*:27: Error: instruction missing MVE vector predication code -- `vmul.i32 q0,q1,q2'
[^:]*:29: Error: syntax error -- `vmuleq.i32 q0,q1,r2'
[^:]*:30: Error: syntax error -- `vmuleq.i32 q0,q1,r2'
[^:]*:32: Error: syntax error -- `vmuleq.i32 q0,q1,r2'
[^:]*:33: Error: vector predicated instruction should be in VPT/VPST block -- `vmult.i32 q0,q1,r2'
[^:]*:35: Error: instruction missing MVE vector predication code -- `vmul.i32 q0,q1,r2'
