[^:]*: Assembler messages:
[^:]*:7: Error: constant expression required
[^:]*:10: Error: width suffixes are invalid in ARM mode
[^:]*:11: Error: width suffixes are invalid in ARM mode
[^:]*:14: Error: cannot determine Thumb instruction size. Use .inst.n/.inst.w instead
[^:]*:15: Error: .inst.n operand too big. Use .inst.w instead
