        .text
        D         ADD       D0Re0,D0Re0,D0Re0
        DO        ADD       D0Re0,D0Re0,D0Re0
        D         ADD       D0Re0,D0Re0,A1LbP
        DM        ADD       D0Re0,D0Re0,A1.7
        DO        MOV       D0Re0,D0.7
        D         ADD       D0Re0,D0Re0,D1Re0
        D         ADD       D0Re0,D0Re0,[D0AR.0]
        DL        ADD       D0Re0,D0Re0,[D0AR.0]
        DO        ADD       D0Re0,D0Re0,RD
        DLO       ADD       D0Re0,D0Re0,RD
        DM        ADD       D0Re0,D0Re0,AC0.0
        DMO       ADD       D0Re0,D0Re0,AC0.0
        DM        ADD       D0Re0,D0Re0,[D0AR.0++]
        DMO       ADD       D0Re0,D0Re0,[D0AR.0++]
        DM        ADD       D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DMO       ADD       D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DO        ADD       D0Re0,D0Re0,A0FrP
        DL        ADD       D0Re0,D0.7,D0Re0
        DO        ADD       D0Re0,D0.7,A1LbP
        DO        ADD       D0Re0,D0.7,D0.7
        DO        ADD       D0Re0,D0.7,D1Re0
        DM        ADD       D0Re0,D0.7,[D0AR.0]
        DMO       ADD       D0Re0,D0.7,[D0AR.0]
        DLM       ADD       D0Re0,D0.7,AC0.0
        DLM       ADD       D0Re0,D0.7,[D0AR.0++]
        DLM       ADD       D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DMO       ADD       D0Re0,D0.7,A0FrP
        DLM       ADD       D0Re0,[D0AR.0],D0Re0
        DMO       ADD       D0Re0,[D0AR.0],A1LbP
        DO        ADD       D0Re0,[D0AR.0],A1.7
        DMO       ADD       D0Re0,[D0AR.0],D1Re0
        DM        ADD       D0Re0,[D0AR.0],RD
        DMO       ADD       D0Re0,[D0AR.0],RD
        DO        ADD       D0Re0,[D0AR.0],AC0.0
        DO        ADD       D0Re0,[D0AR.0],[D0AR.0++]
        DO        ADD       D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ADD       D0Re0,[D0AR.0],A0.7
        DO        ADD       D0Re0,[D0AR.0++],D0Re0
        D         ADD       D0Re0,[D0AR.0++],D0.7
        DMO       ADD       D0Re0,[D0AR.0++],D0.7
        D         ADD       D0Re0,[D0AR.0++],D1.7
        DL        ADD       D0Re0,[D0AR.0++],[D0AR.0]
        DLO       ADD       D0Re0,[D0AR.0++],[D0AR.0]
        DO        ADD       D0Re0,[D0AR.0++],AC0.0
        DO        ADD       D0Re0,[D0AR.0++],[D0AR.0++]
        DO        ADD       D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ADD       D0Re0,[D0AR.0++],A0.7
        DO        ADD       D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         ADD       D0Re0,[D0AR.0+D0ARI.0++],D0.7
        DMO       ADD       D0Re0,[D0AR.0+D0ARI.0++],D0.7
        D         ADD       D0Re0,[D0AR.0+D0ARI.0++],D1.7
        DL        ADD       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DLO       ADD       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DO        ADD       D0Re0,[D0AR.0+D0ARI.0++],AC0.0
        DO        ADD       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DO        ADD       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ADD       D0Re0,[D0AR.0+D0ARI.0++],A0.7
        DLT       ADD       D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T0
        DLOT      ADD       D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T0
        DMT       ADD       D0Re0,D0Re0,AC0.0 [A0FrP+A0FrP++],T0
        DMOT      ADD       D0Re0,D0Re0,AC0.0 [A0FrP+A0FrP++],T0
        DMT       ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0FrP++]
        DMOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0FrP++]
        DT        ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0FrP++]
        DOT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0FrP++]
        DT        ADD       D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T0
        DOT       ADD       D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T0
        DT        ADD       AC0.0,AC0.0,AC1.0 [A0FrP+A0.7++],T0
        DLMT      ADD       D0Re0,D0Re0,AC0.0 [A0FrP+A0.7++],T0
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 [A0FrP+A0.7++],T0
        DLMT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0.7++]
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0.7++]
        DLT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0.7++]
        DLOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0.7++]
        DLT       ADD       D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T0
        DLOT      ADD       D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T0
        DMT       ADD       D0Re0,D0Re0,AC0.0 [A0FrP+A0.8++],T0
        DMOT      ADD       D0Re0,D0Re0,AC0.0 [A0FrP+A0.8++],T0
        DMT       ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0.8++]
        DMOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0.8++]
        DT        ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0.8++]
        DOT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0.8++]
        DT        ADD       D0Re0,D0Re0,D0Re0 [A0FrP],T0
        DOT       ADD       D0Re0,D0Re0,D0Re0 [A0FrP],T0
        DT        ADD       AC0.0,AC0.0,AC1.0 [A0FrP],T0
        DLMT      ADD       D0Re0,D0Re0,AC0.0 [A0FrP],T0
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 [A0FrP],T0
        DLMT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP]
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP]
        DLT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP]
        DLOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP]
        DLT       ADD       D0Re0,D0Re0,D0Re0 [A0FrP++],T0
        DLOT      ADD       D0Re0,D0Re0,D0Re0 [A0FrP++],T0
        DMT       ADD       D0Re0,D0Re0,AC0.0 [A0FrP++],T0
        DMOT      ADD       D0Re0,D0Re0,AC0.0 [A0FrP++],T0
        DMT       ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP++]
        DMOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP++]
        DT        ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP++]
        DOT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP++]
        DT        ADD       D0Re0,D0Re0,D0Re0 [A0FrP--],T0
        DOT       ADD       D0Re0,D0Re0,D0Re0 [A0FrP--],T0
        DT        ADD       AC0.0,AC0.0,AC1.0 [A0FrP--],T0
        DLMT      ADD       D0Re0,D0Re0,AC0.0 [A0FrP--],T0
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 [A0FrP--],T0
        DLMT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP--]
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0FrP--]
        DLT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP--]
        DLOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0FrP--]
        DL        ADD       D0.7,D0Re0,D0Re0
        DLO       ADD       D0.7,D0Re0,D0Re0
        D         ADD       D0.7,D0Re0,D0.7
        DLM       ADD       D0.7,D0Re0,D0.7
        DLO       ADD       D0.7,D0Re0,D0.7
        D         ADD       D0.7,D0Re0,D1.7
        DM        ADD       D0.7,D0Re0,[D0AR.0]
        DLM       ADD       D0.7,D0Re0,[D0AR.0]
        DMO       ADD       D0.7,D0Re0,[D0AR.0]
        DLMO      ADD       D0.7,D0Re0,[D0AR.0]
        DL        ADD       D0.7,D0Re0,AC0.0
        DLO       ADD       D0.7,D0Re0,AC0.0
        DL        ADD       D0.7,D0Re0,[D0AR.0++]
        DLO       ADD       D0.7,D0Re0,[D0AR.0++]
        DL        ADD       D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DLO       ADD       D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         ADD       D0.7,D0Re0,A0.7
        DLT       ADD       D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T0
        DLOT      ADD       D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T0
        DMT       ADD       D0Re0,D0Re0,AC0.0 [A0.7+A0FrP++],T0
        DMOT      ADD       D0Re0,D0Re0,AC0.0 [A0.7+A0FrP++],T0
        DMT       ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7+A0FrP++]
        DMOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7+A0FrP++]
        DT        ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7+A0FrP++]
        DOT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7+A0FrP++]
        D         ADD       D0.7,D0.7,D0Re0
        DLWT      ADD       D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T0
        DMOT      ADD       D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T0
        DT        ADD       AC0.0,AC0.0,AC1.0 [A0.7+A0.7++],T0
        DLMT      ADD       D0Re0,D0Re0,AC0.0 [A0.7+A0.7++],T0
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 [A0.7+A0.7++],T0
        DM        ADD       D0.7,D0.7,D0.7
        DLO       ADD       D0.7,D0.7,D0.7
        DO        ADD       D0.7,D0.7,D1.7
        DMT       ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.7++]
        DLMT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.7++]
        DMOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.7++]
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.7++]
        DMT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7+A0.7++]
        DO        ADD       D0.7,D0.7,AC0.0
        DLOWT     ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7+A0.7++]
        DO        ADD       D0.7,D0.7,[D0AR.0++]
        DO        ADD       D0.7,D0.7,[D0AR.0+D0ARI.0++]
        D         ADD       D0.7,D0.7,A0.7
        DLT       ADD       D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T0
        DLOT      ADD       D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T0
        DMT       ADD       D0Re0,D0Re0,AC0.0 [A0.7+A0.8++],T0
        DMOT      ADD       D0Re0,D0Re0,AC0.0 [A0.7+A0.8++],T0
        DMT       ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.8++]
        DMOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.8++]
        DT        ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7+A0.8++]
        DOT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7+A0.8++]
        D         ADD       D0.7,[D0AR.0],D0Re0
        DLWT      ADD       D0Re0,D0Re0,D0Re0 [A0.7],T0
        DMOT      ADD       D0Re0,D0Re0,D0Re0 [A0.7],T0
        DT        ADD       AC0.0,AC0.0,AC1.0 [A0.7],T0
        DLMT      ADD       D0Re0,D0Re0,AC0.0 [A0.7],T0
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 [A0.7],T0
        DM        ADD       D0.7,[D0AR.0],D0.7
        DLO       ADD       D0.7,[D0AR.0],D0.7
        DO        ADD       D0.7,[D0AR.0],D1.7
        DMT       ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7]
        DLMT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7]
        DMOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7]
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7]
        DMT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7]
        DO        ADD       D0.7,[D0AR.0],AC0.0
        DLOWT     ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7]
        DO        ADD       D0.7,[D0AR.0],[D0AR.0++]
        DO        ADD       D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ADD       D0.7,[D0AR.0],A0.7
        DMT       ADD       D0Re0,D0Re0,D0Re0 [A0.7++],T0
        DO        ADD       D0.7,[D0AR.0++],D0Re0
        DLOWT     ADD       D0Re0,D0Re0,D0Re0 [A0.7++],T0
        DMT       ADD       D0Re0,D0Re0,AC0.0 [A0.7++],T0
        DMOT      ADD       D0Re0,D0Re0,AC0.0 [A0.7++],T0
        DO        ADD       D0.7,[D0AR.0++],A1LbP
        DO        ADD       D0.7,[D0AR.0++],D0.7
        DO        ADD       D0.7,[D0AR.0++],D1Re0
        DT        ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7++]
        DLT       ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7++]
        DOT       ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7++]
        DLOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7++]
        D         ADD       D0.7,[D0AR.0++],AC0.0
        DLWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7++]
        DMOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7++]
        D         ADD       D0.7,[D0AR.0++],[D0AR.0++]
        D         ADD       D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ADD       D0.7,[D0AR.0++],A0FrP
        D         ADD       D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         ADD       D0.7,[D0AR.0+D0ARI.0++],A1LbP
        DL        ADD       D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         ADD       D0.7,[D0AR.0+D0ARI.0++],D1Re0
        D         ADD       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        DO        ADD       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        DM        ADD       D0.7,[D0AR.0+D0ARI.0++],AC0.0
        DM        ADD       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DM        ADD       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DM        ADD       D0.7,[D0AR.0+D0ARI.0++],A0FrP
        DWT       ADD       D0Re0,D0Re0,D0Re0 [A0.7--],T0
        DOWT      ADD       D0Re0,D0Re0,D0Re0 [A0.7--],T0
        DWT       ADD       AC0.0,AC0.0,AC1.0 [A0.7--],T0
        DLMWT     ADD       D0Re0,D0Re0,AC0.0 [A0.7--],T0
        DLMOWT    ADD       D0Re0,D0Re0,AC0.0 [A0.7--],T0
        DLMWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7--]
        DLMOWT    ADD       D0Re0,D0Re0,D0Re0 T0,[A0.7--]
        DLWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7--]
        DLOWT     ADD       D0Re0,D0Re0,AC0.0 T0,[A0.7--]
        DLWT      ADD       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T0
        DLOWT     ADD       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T0
        DMWT      ADD       D0Re0,D0Re0,AC0.0 [A0.8+A0FrP++],T0
        DMOWT     ADD       D0Re0,D0Re0,AC0.0 [A0.8+A0FrP++],T0
        DMWT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8+A0FrP++]
        DMOWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8+A0FrP++]
        DWT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8+A0FrP++]
        DOWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8+A0FrP++]
        DWT       ADD       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T0
        DOWT      ADD       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T0
        DWT       ADD       AC0.0,AC0.0,AC1.0 [A0.8+A0.7++],T0
        DLMWT     ADD       D0Re0,D0Re0,AC0.0 [A0.8+A0.7++],T0
        DLMOWT    ADD       D0Re0,D0Re0,AC0.0 [A0.8+A0.7++],T0
        DLMWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8+A0.7++]
        DLMOWT    ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8+A0.7++]
        DLWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8+A0.7++]
        DLOWT     ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8+A0.7++]
        DLWT      ADD       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T0
        DLOWT     ADD       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T0
        DMWT      ADD       D0Re0,D0Re0,AC0.0 [A0.8+A0.8++],T0
        DMOWT     ADD       D0Re0,D0Re0,AC0.0 [A0.8+A0.8++],T0
        DMWT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8+A0.8++]
        DMOWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8+A0.8++]
        DWT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8+A0.8++]
        DOWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8+A0.8++]
        DWT       ADD       D0Re0,D0Re0,D0Re0 [A0.8],T0
        DOWT      ADD       D0Re0,D0Re0,D0Re0 [A0.8],T0
        DWT       ADD       AC0.0,AC0.0,AC1.0 [A0.8],T0
        DLMWT     ADD       D0Re0,D0Re0,AC0.0 [A0.8],T0
        DLMOWT    ADD       D0Re0,D0Re0,AC0.0 [A0.8],T0
        DLMWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8]
        DLMOWT    ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8]
        DLWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8]
        DLOWT     ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8]
        DLWT      ADD       D0Re0,D0Re0,D0Re0 [A0.8++],T0
        DLOWT     ADD       D0Re0,D0Re0,D0Re0 [A0.8++],T0
        DMWT      ADD       D0Re0,D0Re0,AC0.0 [A0.8++],T0
        DMOWT     ADD       D0Re0,D0Re0,AC0.0 [A0.8++],T0
        DMWT      ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8++]
        DMOWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8++]
        DWT       ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8++]
        DOWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8++]
        DWT       ADD       D0Re0,D0Re0,D0Re0 [A0.8--],T0
        DOWT      ADD       D0Re0,D0Re0,D0Re0 [A0.8--],T0
        DWT       ADD       AC0.0,AC0.0,AC1.0 [A0.8--],T0
        DLMWT     ADD       D0Re0,D0Re0,AC0.0 [A0.8--],T0
        DLMOWT    ADD       D0Re0,D0Re0,AC0.0 [A0.8--],T0
        DLMWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8--]
        DLMOWT    ADD       D0Re0,D0Re0,D0Re0 T0,[A0.8--]
        DLWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8--]
        DLOWT     ADD       D0Re0,D0Re0,AC0.0 T0,[A0.8--]
        DL        MOV       [D0AW.0],D0Re0
        DLO       MOV       [D0AW.0],D0Re0
        D         ADD       [D0AW.0],D0Re0,D0.7
        DLM       ADD       [D0AW.0],D0Re0,D0.7
        DLO       ADD       [D0AW.0],D0Re0,D0.7
        D         ADD       [D0AW.0],D0Re0,D1.7
        DM        ADD       [D0AW.0],D0Re0,[D0AR.0]
        DLM       ADD       [D0AW.0],D0Re0,[D0AR.0]
        DMO       ADD       [D0AW.0],D0Re0,[D0AR.0]
        DLMO      ADD       [D0AW.0],D0Re0,[D0AR.0]
        DL        MOV       [D0AW.0],AC0.0
        DLO       MOV       [D0AW.0],AC0.0
        DL        MOV       [D0AW.0],[D0AR.0++]
        DLO       MOV       [D0AW.0],[D0AR.0++]
        DL        MOV       [D0AW.0],[D0AR.0+D0ARI.0++]
        DLO       MOV       [D0AW.0],[D0AR.0+D0ARI.0++]
        DM        ADD       [D0AW.0],D0Re0,A0.7
        DLMO      ADD       [D0AW.0],D0.7,D0Re0
        DM        ADD       [D0AW.0],D0.7,A1.7
        DLMO      ADD       [D0AW.0],D0.7,D0.7
        DMO       ADD       [D0AW.0],D0.7,D1.7
        DLM       ADD       [D0AW.0],D0.7,RD
        D         ADD       [D0AW.0],D0.7,AC0.0
        D         ADD       [D0AW.0],D0.7,[D0AR.0++]
        D         ADD       [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        D         ADD       [D0AW.0],D0.7,A0FrP
        D         ADD       [D0AW.0],[D0AR.0],D0Re0
        D         ADD       [D0AW.0],[D0AR.0],A1LbP
        DL        ADD       [D0AW.0],[D0AR.0],D0.7
        D         ADD       [D0AW.0],[D0AR.0],D1Re0
        D         ADD       [D0AW.0],[D0AR.0],[D0AR.0]
        DO        ADD       [D0AW.0],[D0AR.0],[D0AR.0]
        DM        ADD       [D0AW.0],[D0AR.0],AC0.0
        DM        ADD       [D0AW.0],[D0AR.0],[D0AR.0++]
        DM        ADD       [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DM        ADD       [D0AW.0],[D0AR.0],A0FrP
        DM        ADD       [D0AW.0],[D0AR.0++],D0Re0
        DM        ADD       [D0AW.0],[D0AR.0++],A1LbP
        DLM       ADD       [D0AW.0],[D0AR.0++],D0.7
        DM        ADD       [D0AW.0],[D0AR.0++],D1Re0
        D         ADD       [D0AW.0],[D0AR.0++],RD
        DO        ADD       [D0AW.0],[D0AR.0++],RD
        DM        ADD       [D0AW.0],[D0AR.0++],AC0.0
        DM        ADD       [D0AW.0],[D0AR.0++],[D0AR.0++]
        DM        ADD       [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DM        ADD       [D0AW.0],[D0AR.0++],A0FrP
        DM        ADD       [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DM        ADD       [D0AW.0],[D0AR.0+D0ARI.0++],A1LbP
        DLM       ADD       [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DM        ADD       [D0AW.0],[D0AR.0+D0ARI.0++],D1Re0
        D         ADD       [D0AW.0],[D0AR.0+D0ARI.0++],RD
        DO        ADD       [D0AW.0],[D0AR.0+D0ARI.0++],RD
        DM        ADD       [D0AW.0],[D0AR.0+D0ARI.0++],AC0.0
        DM        ADD       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DM        ADD       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DM        ADD       [D0AW.0],[D0AR.0+D0ARI.0++],A0FrP
        DMT       ADD       D0Re0,D0Re0,D0Re0 [D0AW.0],T0
        DT        ADD       D0Re0,D0Re0,AC0.0 [D0AW.0],T0
        DT        ADD       D0Re0,D0Re0,D0Re0 T0,[D0AR.0]
        DT        ADD       AC0.0,AC0.0,AC1.0 T0,[D0AR.0]
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 T0,[D0AR.0]
        DLM       MOV       [D0AW.0++],D0Re0
        DLMO      MOV       [D0AW.0++],D0Re0
        DM        ADD       [D0AW.0++],D0Re0,D0.7
        DO        ADD       [D0AW.0++],D0Re0,A1.7
        DLMO      MOV       [D0AW.0++],D0.7
        DMO       ADD       [D0AW.0++],D0Re0,D1.7
        DM        MOV       [D0AW.0++],RD
        DLM       MOV       [D0AW.0++],RD
        DMO       MOV       [D0AW.0++],RD
        D         ADD       [D0AW.0++],D0Re0,AC0.0
        DO        ADD       [D0AW.0++],D0Re0,AC0.0
        D         ADD       [D0AW.0++],D0Re0,[D0AR.0++]
        DO        ADD       [D0AW.0++],D0Re0,[D0AR.0++]
        D         ADD       [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DO        ADD       [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        D         ADD       [D0AW.0++],D0Re0,A0FrP
        DT        ADD       D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T0
        DOT       ADD       D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T0
        DT        ADD       AC0.0,AC0.0,AC1.0 [A1LbP+A1LbP++],T0
        DLMT      ADD       D0Re0,D0Re0,AC0.0 [A1LbP+A1LbP++],T0
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 [A1LbP+A1LbP++],T0
        DLMT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1LbP++]
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1LbP++]
        DLT       ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1LbP++]
        DLOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1LbP++]
        DMT       ADD       D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T0
        DO        ADD       [D0AW.0++],D0.7,D0Re0
        DLOWT     ADD       D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T0
        DMT       ADD       D0Re0,D0Re0,AC0.0 [A1LbP+A1.7++],T0
        DMOT      ADD       D0Re0,D0Re0,AC0.0 [A1LbP+A1.7++],T0
        DO        ADD       [D0AW.0++],D0.7,A1LbP
        DO        ADD       [D0AW.0++],D0.7,D0.7
        DO        ADD       [D0AW.0++],D0.7,D1Re0
        DT        ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1.7++]
        DLT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1.7++]
        DOT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1.7++]
        DLOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1.7++]
        DT        ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1.7++]
        DLM       ADD       [D0AW.0++],D0.7,AC0.0
        DMOWT     ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1.7++]
        DM        ADD       [D0AW.0++],D0.7,[D0AR.0++]
        DM        ADD       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DM        ADD       [D0AW.0++],D0.7,A0FrP
        DWT       ADD       D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T0
        DOWT      ADD       D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T0
        DWT       ADD       AC0.0,AC0.0,AC1.0 [A1LbP+A1.8++],T0
        DLMWT     ADD       D0Re0,D0Re0,AC0.0 [A1LbP+A1.8++],T0
        DLMOWT    ADD       D0Re0,D0Re0,AC0.0 [A1LbP+A1.8++],T0
        DLMWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1.8++]
        DLMOWT    ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1.8++]
        DLWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1.8++]
        DLOWT     ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1.8++]
        DMWT      ADD       D0Re0,D0Re0,D0Re0 [A1LbP],T0
        DOT       ADD       D0Re0,D0Re0,D0Re0 [A1LbP],T0
        DLMO      ADD       [D0AW.0++],[D0AR.0],D0Re0
        DMWT      ADD       D0Re0,D0Re0,AC0.0 [A1LbP],T0
        DMOWT     ADD       D0Re0,D0Re0,AC0.0 [A1LbP],T0
        DMO       ADD       [D0AW.0++],[D0AR.0],A1LbP
        DO        ADD       [D0AW.0++],[D0AR.0],A1.7
        DMO       ADD       [D0AW.0++],[D0AR.0],D1Re0
        DWT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP]
        DLWT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP]
        DOWT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP]
        DLOWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP]
        DT        ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP]
        DLM       ADD       [D0AW.0++],[D0AR.0],AC0.0
        DMOWT     ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP]
        DM        ADD       [D0AW.0++],[D0AR.0],[D0AR.0++]
        DM        ADD       [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DM        ADD       [D0AW.0++],[D0AR.0],A0FrP
        DT        ADD       D0Re0,D0Re0,D0Re0 [A1LbP++],T0
        DLM       ADD       [D0AW.0++],[D0AR.0++],D0Re0
        DMOWT     ADD       D0Re0,D0Re0,D0Re0 [A1LbP++],T0
        DWT       ADD       AC0.0,AC0.0,AC1.0 [A1LbP++],T0
        DLMWT     ADD       D0Re0,D0Re0,AC0.0 [A1LbP++],T0
        DLMOWT    ADD       D0Re0,D0Re0,AC0.0 [A1LbP++],T0
        DM        ADD       [D0AW.0++],[D0AR.0++],A1.7
        DLMO      ADD       [D0AW.0++],[D0AR.0++],D0.7
        DMO       ADD       [D0AW.0++],[D0AR.0++],D1.7
        DMWT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP++]
        DLMWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP++]
        DMOWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP++]
        DLMOWT    ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP++]
        DMWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP++]
        DOT       ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP++]
        DLMO      ADD       [D0AW.0++],[D0AR.0++],AC0.0
        DMO       ADD       [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DMO       ADD       [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DM        ADD       [D0AW.0++],[D0AR.0++],A0.7
        DMO       ADD       [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         ADD       [D0AW.0++],[D0AR.0+D0ARI.0++],A1.7
        DMO       ADD       [D0AW.0++],[D0AR.0+D0ARI.0++],A1.7
        DM        ADD       [D0AW.0++],[D0AR.0+D0ARI.0++],D1.7
        DL        ADD       [D0AW.0++],[D0AR.0+D0ARI.0++],RD
        DLO       ADD       [D0AW.0++],[D0AR.0+D0ARI.0++],RD
        DMO       ADD       [D0AW.0++],[D0AR.0+D0ARI.0++],AC0.0
        DMO       ADD       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DMO       ADD       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DM        ADD       [D0AW.0++],[D0AR.0+D0ARI.0++],A0.7
        DLWT      ADD       D0Re0,D0Re0,D0Re0 [A1LbP--],T0
        DLOWT     ADD       D0Re0,D0Re0,D0Re0 [A1LbP--],T0
        DMWT      ADD       D0Re0,D0Re0,AC0.0 [A1LbP--],T0
        DMOWT     ADD       D0Re0,D0Re0,AC0.0 [A1LbP--],T0
        DMWT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP--]
        DMOWT     ADD       D0Re0,D0Re0,D0Re0 T0,[A1LbP--]
        DWT       ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP--]
        DOWT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1LbP--]
        DMT       ADD       D0Re0,D0Re0,D0Re0 [D0AW.0++],T0
        DT        ADD       D0Re0,D0Re0,AC0.0 [D0AW.0++],T0
        DT        ADD       D0Re0,D0Re0,D0Re0 T0,[D0AR.0++]
        DT        ADD       AC0.0,AC0.0,AC1.0 T0,[D0AR.0++]
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 T0,[D0AR.0++]
        DLM       MOV       [D0AW.0+D0AWI.0++],D0Re0
        DLMO      MOV       [D0AW.0+D0AWI.0++],D0Re0
        DM        ADD       [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DO        ADD       [D0AW.0+D0AWI.0++],D0Re0,A1.7
        DLMO      MOV       [D0AW.0+D0AWI.0++],D0.7
        DMO       ADD       [D0AW.0+D0AWI.0++],D0Re0,D1.7
        DM        MOV       [D0AW.0+D0AWI.0++],RD
        DLM       MOV       [D0AW.0+D0AWI.0++],RD
        DMO       MOV       [D0AW.0+D0AWI.0++],RD
        DLMO      MOV       [D0AW.0+D0AWI.0++],RD
        DLM       MOV       [D0AW.0+D0AWI.0++],AC0.0
        DLMO      MOV       [D0AW.0+D0AWI.0++],AC0.0
        DLM       MOV       [D0AW.0+D0AWI.0++],[D0AR.0++]
        DLMO      MOV       [D0AW.0+D0AWI.0++],[D0AR.0++]
        DLM       MOV       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++]
        DLMO      MOV       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++]
        DMO       ADD       [D0AW.0+D0AWI.0++],D0Re0,A0.7
        DLMO      ADD       [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DM        ADD       [D0AW.0+D0AWI.0++],D0.7,A1.7
        DLMO      ADD       [D0AW.0+D0AWI.0++],D0.7,D0.7
        DMO       ADD       [D0AW.0+D0AWI.0++],D0.7,D1.7
        DLM       ADD       [D0AW.0+D0AWI.0++],D0.7,RD
        D         ADD       [D0AW.0+D0AWI.0++],D0.7,AC0.0
        D         ADD       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        D         ADD       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        D         ADD       [D0AW.0+D0AWI.0++],D0.7,A0FrP
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0],A1LbP
        DL        ADD       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0],D1Re0
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DO        ADD       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0],AC0.0
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0],A0FrP
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0++],A1LbP
        DL        ADD       [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0++],D1Re0
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DO        ADD       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0++],AC0.0
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0++],A0FrP
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A1LbP
        DL        ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D1Re0
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DO        ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],AC0.0
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A0FrP
        DT        ADD       D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T0
        DT        ADD       AC0.0,AC0.0,AC1.0 [D0AW.0+D0AWI.0++],T0
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 [D0AW.0+D0AWI.0++],T0
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 T0,[D0AR.0+D0ARI.0++]
        DLOT      ADD       D0Re0,D0Re0,AC0.0 T0,[D0AR.0+D0ARI.0++]
        DLMT      ADD       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T0
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T0
        DLT       ADD       D0Re0,D0Re0,AC0.0 [A1.7+A1LbP++],T0
        DLOT      ADD       D0Re0,D0Re0,AC0.0 [A1.7+A1LbP++],T0
        DLT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7+A1LbP++]
        DLOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7+A1LbP++]
        DMT       ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7+A1LbP++]
        DMOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7+A1LbP++]
        DMT       ADD       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T0
        DMOT      ADD       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T0
        DT        ADD       D0Re0,D0Re0,AC0.0 [A1.7+A1.7++],T0
        DOT       ADD       D0Re0,D0Re0,AC0.0 [A1.7+A1.7++],T0
        DT        ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7+A1.7++]
        DOT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7+A1.7++]
        DT        ADD       AC0.0,AC0.0,AC1.0 T0,[A1.7+A1.7++]
        DLMT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7+A1.7++]
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7+A1.7++]
        DLMT      ADD       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T0
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T0
        DLT       ADD       D0Re0,D0Re0,AC0.0 [A1.7+A1.8++],T0
        DLOT      ADD       D0Re0,D0Re0,AC0.0 [A1.7+A1.8++],T0
        DLT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7+A1.8++]
        DLOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7+A1.8++]
        DMT       ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7+A1.8++]
        DMOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7+A1.8++]
        DMT       ADD       D0Re0,D0Re0,D0Re0 [A1.7],T0
        DMOT      ADD       D0Re0,D0Re0,D0Re0 [A1.7],T0
        DT        ADD       D0Re0,D0Re0,AC0.0 [A1.7],T0
        DOT       ADD       D0Re0,D0Re0,AC0.0 [A1.7],T0
        DT        ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7]
        DOT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7]
        DT        ADD       AC0.0,AC0.0,AC1.0 T0,[A1.7]
        DLMT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7]
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7]
        DLMT      ADD       D0Re0,D0Re0,D0Re0 [A1.7++],T0
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 [A1.7++],T0
        DLT       ADD       D0Re0,D0Re0,AC0.0 [A1.7++],T0
        DLOT      ADD       D0Re0,D0Re0,AC0.0 [A1.7++],T0
        DLT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7++]
        DLOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7++]
        DMT       ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7++]
        DMOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7++]
        DMT       ADD       D0Re0,D0Re0,D0Re0 [A1.7--],T0
        DMOT      ADD       D0Re0,D0Re0,D0Re0 [A1.7--],T0
        DT        ADD       D0Re0,D0Re0,AC0.0 [A1.7--],T0
        DOT       ADD       D0Re0,D0Re0,AC0.0 [A1.7--],T0
        DT        ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7--]
        DOT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.7--]
        DT        ADD       AC0.0,AC0.0,AC1.0 T0,[A1.7--]
        DLMT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7--]
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 T0,[A1.7--]
        DLMT      ADD       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T0
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T0
        DLT       ADD       D0Re0,D0Re0,AC0.0 [A1.8+A1LbP++],T0
        DLOT      ADD       D0Re0,D0Re0,AC0.0 [A1.8+A1LbP++],T0
        DLT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8+A1LbP++]
        DLOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8+A1LbP++]
        DMT       ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8+A1LbP++]
        DMOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8+A1LbP++]
        DMT       ADD       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T0
        DMOT      ADD       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T0
        DT        ADD       D0Re0,D0Re0,AC0.0 [A1.8+A1.7++],T0
        DOT       ADD       D0Re0,D0Re0,AC0.0 [A1.8+A1.7++],T0
        DT        ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8+A1.7++]
        DOT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8+A1.7++]
        DT        ADD       AC0.0,AC0.0,AC1.0 T0,[A1.8+A1.7++]
        DLMT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8+A1.7++]
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8+A1.7++]
        DLMT      ADD       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T0
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T0
        DLT       ADD       D0Re0,D0Re0,AC0.0 [A1.8+A1.8++],T0
        DLOT      ADD       D0Re0,D0Re0,AC0.0 [A1.8+A1.8++],T0
        DLT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8+A1.8++]
        DLOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8+A1.8++]
        DMT       ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8+A1.8++]
        DMOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8+A1.8++]
        DMT       ADD       D0Re0,D0Re0,D0Re0 [A1.8],T0
        DMOT      ADD       D0Re0,D0Re0,D0Re0 [A1.8],T0
        DT        ADD       D0Re0,D0Re0,AC0.0 [A1.8],T0
        DOT       ADD       D0Re0,D0Re0,AC0.0 [A1.8],T0
        DT        ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8]
        DOT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8]
        DT        ADD       AC0.0,AC0.0,AC1.0 T0,[A1.8]
        DLMT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8]
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8]
        DLMT      ADD       D0Re0,D0Re0,D0Re0 [A1.8++],T0
        DLMOT     ADD       D0Re0,D0Re0,D0Re0 [A1.8++],T0
        DLT       ADD       D0Re0,D0Re0,AC0.0 [A1.8++],T0
        DLOT      ADD       D0Re0,D0Re0,AC0.0 [A1.8++],T0
        DLT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8++]
        DLOT      ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8++]
        DMT       ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8++]
        DMOT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8++]
        DMT       ADD       D0Re0,D0Re0,D0Re0 [A1.8--],T0
        DMOT      ADD       D0Re0,D0Re0,D0Re0 [A1.8--],T0
        DT        ADD       D0Re0,D0Re0,AC0.0 [A1.8--],T0
        DOT       ADD       D0Re0,D0Re0,AC0.0 [A1.8--],T0
        DT        ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8--]
        DOT       ADD       D0Re0,D0Re0,D0Re0 T0,[A1.8--]
        DT        ADD       AC0.0,AC0.0,AC1.0 T0,[A1.8--]
        DLMT      ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8--]
        DLMOT     ADD       D0Re0,D0Re0,AC0.0 T0,[A1.8--]
        DLM       ADD       D1Re0,D1Re0,D1Re0
        DLMO      ADD       D1Re0,D1Re0,D1Re0
        D         MOV       D1Re0,D1.7
        DO        ADD       D1Re0,D1Re0,D1.7
        DLMO      ADD       D1Re0,D1Re0,D1.7
        DO        ADD       D1Re0,D1Re0,D0.7
        DLM       ADD       D1Re0,D1Re0,RD
        DLMO      ADD       D1Re0,D1Re0,RD
        DLM       ADD       D1Re0,D1Re0,AC1.0
        DLMO      ADD       D1Re0,D1Re0,AC1.0
        DM        ADD       D1Re0,D1Re0,[D1BR.1]
        DMO       ADD       D1Re0,D1Re0,[D1BR.1]
        DM        ADD       D1Re0,D1Re0,[D1BR.1++]
        DMO       ADD       D1Re0,D1Re0,[D1BR.1++]
        D         MOV       D1Re0,[D1BR.1+D1BRI.1++]
        DO        ADD       D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DLMO      ADD       D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DLO       ADD       D1Re0,D1.7,D1Re0
        DM        ADD       D1Re0,D1.7,A1.7
        DLMO      ADD       D1Re0,D1.7,D1.7
        DMO       ADD       D1Re0,D1.7,D0.7
        D         ADD       D1Re0,D1.7,AC1.0
        D         ADD       D1Re0,D1.7,A0FrP
        DO        ADD       D1Re0,D1.7,[D1BR.1]
        DO        ADD       D1Re0,D1.7,[D1BR.1++]
        DL        ADD       D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        D         ADD       D1Re0,[D1BR.1],D1Re0
        D         ADD       D1Re0,[D1BR.1],A1LbP
        DL        ADD       D1Re0,[D1BR.1],D1.7
        D         ADD       D1Re0,[D1BR.1],D0Re0
        D         ADD       D1Re0,[D1BR.1],RD
        D         ADD       D1Re0,[D1BR.1],AC1.0
        D         ADD       D1Re0,[D1BR.1],A0FrP
        DO        ADD       D1Re0,[D1BR.1],[D1BR.1]
        DO        ADD       D1Re0,[D1BR.1],[D1BR.1++]
        DL        ADD       D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ADD       D1Re0,[D1BR.1++],D1Re0
        D         ADD       D1Re0,[D1BR.1++],A1LbP
        DL        ADD       D1Re0,[D1BR.1++],D1.7
        D         ADD       D1Re0,[D1BR.1++],D0Re0
        D         ADD       D1Re0,[D1BR.1++],RD
        D         ADD       D1Re0,[D1BR.1++],AC1.0
        D         ADD       D1Re0,[D1BR.1++],A0FrP
        DO        ADD       D1Re0,[D1BR.1++],[D1BR.1]
        DO        ADD       D1Re0,[D1BR.1++],[D1BR.1++]
        DL        ADD       D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ADD       D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         ADD       D1Re0,[D1BR.1+D1BRI.1++],A1LbP
        DL        ADD       D1Re0,[D1BR.1+D1BRI.1++],D1.7
        D         ADD       D1Re0,[D1BR.1+D1BRI.1++],D0Re0
        D         ADD       D1Re0,[D1BR.1+D1BRI.1++],RD
        D         ADD       D1Re0,[D1BR.1+D1BRI.1++],AC1.0
        D         ADD       D1Re0,[D1BR.1+D1BRI.1++],A0FrP
        DO        ADD       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DO        ADD       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        ADD       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DT        ADD       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T0
        DOT       ADD       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T0
        DT        ADD       AC1.0,AC1.0,AC0.0 [A0FrP+A0FrP++],T0
        DLMT      ADD       D1Re0,D1Re0,AC1.0 [A0FrP+A0FrP++],T0
        DLMOT     ADD       D1Re0,D1Re0,AC1.0 [A0FrP+A0FrP++],T0
        DLMT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0FrP++]
        DLMOT     ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0FrP++]
        DLT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0FrP++]
        DLOT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0FrP++]
        DLT       ADD       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T0
        DLOT      ADD       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T0
        DMT       ADD       D1Re0,D1Re0,AC1.0 [A0FrP+A0.7++],T0
        DMOT      ADD       D1Re0,D1Re0,AC1.0 [A0FrP+A0.7++],T0
        DMT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0.7++]
        DMOT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0.7++]
        DT        ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0.7++]
        DOT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0.7++]
        DT        ADD       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T0
        DOT       ADD       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T0
        DT        ADD       AC1.0,AC1.0,AC0.0 [A0FrP+A0.8++],T0
        DLMT      ADD       D1Re0,D1Re0,AC1.0 [A0FrP+A0.8++],T0
        DLMOT     ADD       D1Re0,D1Re0,AC1.0 [A0FrP+A0.8++],T0
        DLMT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0.8++]
        DLMOT     ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0.8++]
        DLT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0.8++]
        DLOT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0.8++]
        DLT       ADD       D1Re0,D1Re0,D1Re0 [A0FrP],T0
        DLOT      ADD       D1Re0,D1Re0,D1Re0 [A0FrP],T0
        DMT       ADD       D1Re0,D1Re0,AC1.0 [A0FrP],T0
        DMOT      ADD       D1Re0,D1Re0,AC1.0 [A0FrP],T0
        DMT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP]
        DMOT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP]
        DT        ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP]
        DOT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP]
        DT        ADD       D1Re0,D1Re0,D1Re0 [A0FrP++],T0
        DOT       ADD       D1Re0,D1Re0,D1Re0 [A0FrP++],T0
        DT        ADD       AC1.0,AC1.0,AC0.0 [A0FrP++],T0
        DLMT      ADD       D1Re0,D1Re0,AC1.0 [A0FrP++],T0
        DLMOT     ADD       D1Re0,D1Re0,AC1.0 [A0FrP++],T0
        DLMT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP++]
        DLMOT     ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP++]
        DLT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP++]
        DLOT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP++]
        DLT       ADD       D1Re0,D1Re0,D1Re0 [A0FrP--],T0
        DLOT      ADD       D1Re0,D1Re0,D1Re0 [A0FrP--],T0
        DMT       ADD       D1Re0,D1Re0,AC1.0 [A0FrP--],T0
        DMOT      ADD       D1Re0,D1Re0,AC1.0 [A0FrP--],T0
        DMT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP--]
        DMOT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0FrP--]
        DT        ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP--]
        DOT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0FrP--]
        D         ADD       D1.7,D1Re0,D1Re0
        DO        ADD       D1.7,D1Re0,D1Re0
        D         ADD       D1.7,D1Re0,A1LbP
        DM        ADD       D1.7,D1Re0,A1.7
        DO        MOV       D1.7,D1.7
        D         ADD       D1.7,D1Re0,D0Re0
        D         ADD       D1.7,D1Re0,RD
        DO        ADD       D1.7,D1Re0,RD
        D         ADD       D1.7,D1Re0,AC1.0
        DO        ADD       D1.7,D1Re0,AC1.0
        D         ADD       D1.7,D1Re0,A0FrP
        DL        ADD       D1.7,D1Re0,[D1BR.1]
        DLO       ADD       D1.7,D1Re0,[D1BR.1]
        DL        ADD       D1.7,D1Re0,[D1BR.1++]
        DLO       ADD       D1.7,D1Re0,[D1BR.1++]
        DM        ADD       D1.7,D1Re0,A0.7
        DO        MOV       D1.7,[D1BR.1+D1BRI.1++]
        DT        ADD       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T0
        DOT       ADD       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T0
        DT        ADD       AC1.0,AC1.0,AC0.0 [A0.7+A0FrP++],T0
        DLMT      ADD       D1Re0,D1Re0,AC1.0 [A0.7+A0FrP++],T0
        DLMOT     ADD       D1Re0,D1Re0,AC1.0 [A0.7+A0FrP++],T0
        DLMT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7+A0FrP++]
        DLMOT     ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7+A0FrP++]
        DLT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7+A0FrP++]
        DLOT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7+A0FrP++]
        DMT       ADD       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T0
        DO        ADD       D1.7,D1.7,D1Re0
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T0
        DMT       ADD       D1Re0,D1Re0,AC1.0 [A0.7+A0.7++],T0
        DMOT      ADD       D1Re0,D1Re0,AC1.0 [A0.7+A0.7++],T0
        DO        ADD       D1.7,D1.7,A1LbP
        DO        ADD       D1.7,D1.7,D1.7
        DO        ADD       D1.7,D1.7,D0Re0
        DWT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7+A0.7++]
        DO        ADD       D1.7,D1.7,RD
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7+A0.7++]
        DWT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7+A0.7++]
        DLMT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7+A0.7++]
        DLO       ADD       D1.7,D1.7,AC1.0
        DO        ADD       D1.7,D1.7,A0FrP
        DLO       ADD       D1.7,D1.7,[D1BR.1]
        DLO       ADD       D1.7,D1.7,[D1BR.1++]
        DO        ADD       D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DMT       ADD       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T0
        DMOT      ADD       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T0
        DT        ADD       D1Re0,D1Re0,AC1.0 [A0.7+A0.8++],T0
        DOT       ADD       D1Re0,D1Re0,AC1.0 [A0.7+A0.8++],T0
        DT        ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7+A0.8++]
        DOT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7+A0.8++]
        DT        ADD       AC1.0,AC1.0,AC0.0 T0,[A0.7+A0.8++]
        DLMT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7+A0.8++]
        DLMOT     ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7+A0.8++]
        DLMT      ADD       D1Re0,D1Re0,D1Re0 [A0.7],T0
        DLMOT     ADD       D1Re0,D1Re0,D1Re0 [A0.7],T0
        DLT       ADD       D1Re0,D1Re0,AC1.0 [A0.7],T0
        DLOT      ADD       D1Re0,D1Re0,AC1.0 [A0.7],T0
        DLT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7]
        DLOT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7]
        DMT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7]
        DMOT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7]
        DMT       ADD       D1Re0,D1Re0,D1Re0 [A0.7++],T0
        DMOT      ADD       D1Re0,D1Re0,D1Re0 [A0.7++],T0
        DT        ADD       D1Re0,D1Re0,AC1.0 [A0.7++],T0
        DOT       ADD       D1Re0,D1Re0,AC1.0 [A0.7++],T0
        DT        ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7++]
        DOT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7++]
        DT        ADD       AC1.0,AC1.0,AC0.0 T0,[A0.7++]
        DLMT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7++]
        DLMOT     ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7++]
        DLMT      ADD       D1Re0,D1Re0,D1Re0 [A0.7--],T0
        DLMOT     ADD       D1Re0,D1Re0,D1Re0 [A0.7--],T0
        DLT       ADD       D1Re0,D1Re0,AC1.0 [A0.7--],T0
        DLOT      ADD       D1Re0,D1Re0,AC1.0 [A0.7--],T0
        DLT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7--]
        DLOT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0.7--]
        DMT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7--]
        DMOT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.7--]
        DL        ADD       D1.7,[D1BR.1],D1Re0
        DO        ADD       D1.7,[D1BR.1],A1LbP
        DO        ADD       D1.7,[D1BR.1],D1.7
        DO        ADD       D1.7,[D1BR.1],D0Re0
        DL        ADD       D1.7,[D1BR.1],RD
        DL        ADD       D1.7,[D1BR.1],AC1.0
        DO        ADD       D1.7,[D1BR.1],A0FrP
        DLO       ADD       D1.7,[D1BR.1],[D1BR.1]
        DLO       ADD       D1.7,[D1BR.1],[D1BR.1++]
        DO        ADD       D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        ADD       D1.7,[D1BR.1++],D1Re0
        DO        ADD       D1.7,[D1BR.1++],A1LbP
        DO        ADD       D1.7,[D1BR.1++],D1.7
        DO        ADD       D1.7,[D1BR.1++],D0Re0
        DL        ADD       D1.7,[D1BR.1++],RD
        DL        ADD       D1.7,[D1BR.1++],AC1.0
        DO        ADD       D1.7,[D1BR.1++],A0FrP
        DLO       ADD       D1.7,[D1BR.1++],[D1BR.1]
        DLO       ADD       D1.7,[D1BR.1++],[D1BR.1++]
        DO        ADD       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        ADD       D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DO        ADD       D1.7,[D1BR.1+D1BRI.1++],A1LbP
        DO        ADD       D1.7,[D1BR.1+D1BRI.1++],D1.7
        DO        ADD       D1.7,[D1BR.1+D1BRI.1++],D0Re0
        DL        ADD       D1.7,[D1BR.1+D1BRI.1++],RD
        DL        ADD       D1.7,[D1BR.1+D1BRI.1++],AC1.0
        DO        ADD       D1.7,[D1BR.1+D1BRI.1++],A0FrP
        DLO       ADD       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLO       ADD       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DO        ADD       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DMT       ADD       D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T0
        DMOT      ADD       D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T0
        DT        ADD       D1Re0,D1Re0,AC1.0 [A0.8+A0FrP++],T0
        DOT       ADD       D1Re0,D1Re0,AC1.0 [A0.8+A0FrP++],T0
        DT        ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8+A0FrP++]
        DOT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8+A0FrP++]
        DT        ADD       AC1.0,AC1.0,AC0.0 T0,[A0.8+A0FrP++]
        DLMT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8+A0FrP++]
        DLMOT     ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8+A0FrP++]
        DLMT      ADD       D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T0
        DLMOT     ADD       D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T0
        DLT       ADD       D1Re0,D1Re0,AC1.0 [A0.8+A0.7++],T0
        DLOT      ADD       D1Re0,D1Re0,AC1.0 [A0.8+A0.7++],T0
        DLT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8+A0.7++]
        DLOT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8+A0.7++]
        DMT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8+A0.7++]
        DMOT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8+A0.7++]
        DMT       ADD       D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T0
        DMOT      ADD       D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T0
        DT        ADD       D1Re0,D1Re0,AC1.0 [A0.8+A0.8++],T0
        DOT       ADD       D1Re0,D1Re0,AC1.0 [A0.8+A0.8++],T0
        DT        ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8+A0.8++]
        DOT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8+A0.8++]
        DT        ADD       AC1.0,AC1.0,AC0.0 T0,[A0.8+A0.8++]
        DLMT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8+A0.8++]
        DLMOT     ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8+A0.8++]
        DLMT      ADD       D1Re0,D1Re0,D1Re0 [A0.8],T0
        DLMOT     ADD       D1Re0,D1Re0,D1Re0 [A0.8],T0
        DLT       ADD       D1Re0,D1Re0,AC1.0 [A0.8],T0
        DLOT      ADD       D1Re0,D1Re0,AC1.0 [A0.8],T0
        DLT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8]
        DLOT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8]
        DMT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8]
        DMOT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8]
        DMT       ADD       D1Re0,D1Re0,D1Re0 [A0.8++],T0
        DMOT      ADD       D1Re0,D1Re0,D1Re0 [A0.8++],T0
        DT        ADD       D1Re0,D1Re0,AC1.0 [A0.8++],T0
        DOT       ADD       D1Re0,D1Re0,AC1.0 [A0.8++],T0
        DT        ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8++]
        DOT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8++]
        DT        ADD       AC1.0,AC1.0,AC0.0 T0,[A0.8++]
        DLMT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8++]
        DLMOT     ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8++]
        DLMT      ADD       D1Re0,D1Re0,D1Re0 [A0.8--],T0
        DLMOT     ADD       D1Re0,D1Re0,D1Re0 [A0.8--],T0
        DLT       ADD       D1Re0,D1Re0,AC1.0 [A0.8--],T0
        DLOT      ADD       D1Re0,D1Re0,AC1.0 [A0.8--],T0
        DLT       ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8--]
        DLOT      ADD       D1Re0,D1Re0,D1Re0 T0,[A0.8--]
        DMT       ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8--]
        DMOT      ADD       D1Re0,D1Re0,AC1.0 T0,[A0.8--]
        DWT       ADD       D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T0
        DOWT      ADD       D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T0
        DWT       ADD       AC1.0,AC1.0,AC0.0 [A1LbP+A1LbP++],T0
        DLMWT     ADD       D1Re0,D1Re0,AC1.0 [A1LbP+A1LbP++],T0
        DLMOWT    ADD       D1Re0,D1Re0,AC1.0 [A1LbP+A1LbP++],T0
        DLMWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1LbP++]
        DLMOWT    ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1LbP++]
        DLWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1LbP++]
        DLOWT     ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1LbP++]
        DLWT      ADD       D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T0
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T0
        DMWT      ADD       D1Re0,D1Re0,AC1.0 [A1LbP+A1.7++],T0
        DMOWT     ADD       D1Re0,D1Re0,AC1.0 [A1LbP+A1.7++],T0
        DMWT      ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1.7++]
        DMOWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1.7++]
        DWT       ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1.7++]
        DOWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1.7++]
        DWT       ADD       D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T0
        DOWT      ADD       D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T0
        DWT       ADD       AC1.0,AC1.0,AC0.0 [A1LbP+A1.8++],T0
        DLMWT     ADD       D1Re0,D1Re0,AC1.0 [A1LbP+A1.8++],T0
        DLMOWT    ADD       D1Re0,D1Re0,AC1.0 [A1LbP+A1.8++],T0
        DLMWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1.8++]
        DLMOWT    ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1.8++]
        DLWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1.8++]
        DLOWT     ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1.8++]
        DLWT      ADD       D1Re0,D1Re0,D1Re0 [A1LbP],T0
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 [A1LbP],T0
        DMWT      ADD       D1Re0,D1Re0,AC1.0 [A1LbP],T0
        DMOWT     ADD       D1Re0,D1Re0,AC1.0 [A1LbP],T0
        DMWT      ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP]
        DMOWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP]
        DWT       ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP]
        DOWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP]
        DWT       ADD       D1Re0,D1Re0,D1Re0 [A1LbP++],T0
        DOWT      ADD       D1Re0,D1Re0,D1Re0 [A1LbP++],T0
        DWT       ADD       AC1.0,AC1.0,AC0.0 [A1LbP++],T0
        DLMWT     ADD       D1Re0,D1Re0,AC1.0 [A1LbP++],T0
        DLMOWT    ADD       D1Re0,D1Re0,AC1.0 [A1LbP++],T0
        DLMWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP++]
        DLMOWT    ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP++]
        DLWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP++]
        DLOWT     ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP++]
        DLWT      ADD       D1Re0,D1Re0,D1Re0 [A1LbP--],T0
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 [A1LbP--],T0
        DMWT      ADD       D1Re0,D1Re0,AC1.0 [A1LbP--],T0
        DMOWT     ADD       D1Re0,D1Re0,AC1.0 [A1LbP--],T0
        DMWT      ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP--]
        DMOWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1LbP--]
        DWT       ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP--]
        DOWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1LbP--]
        DWT       ADD       D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T0
        DOWT      ADD       D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T0
        DWT       ADD       AC1.0,AC1.0,AC0.0 [A1.7+A1LbP++],T0
        DLMWT     ADD       D1Re0,D1Re0,AC1.0 [A1.7+A1LbP++],T0
        DLMOWT    ADD       D1Re0,D1Re0,AC1.0 [A1.7+A1LbP++],T0
        DLMWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7+A1LbP++]
        DLMOWT    ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7+A1LbP++]
        DLWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7+A1LbP++]
        DLOWT     ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7+A1LbP++]
        DLWT      ADD       D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T0
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T0
        DMWT      ADD       D1Re0,D1Re0,AC1.0 [A1.7+A1.7++],T0
        DMOWT     ADD       D1Re0,D1Re0,AC1.0 [A1.7+A1.7++],T0
        DMWT      ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7+A1.7++]
        DMOWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7+A1.7++]
        DWT       ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7+A1.7++]
        DOWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7+A1.7++]
        DWT       ADD       D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T0
        DOWT      ADD       D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T0
        DWT       ADD       AC1.0,AC1.0,AC0.0 [A1.7+A1.8++],T0
        DLMWT     ADD       D1Re0,D1Re0,AC1.0 [A1.7+A1.8++],T0
        DLMOWT    ADD       D1Re0,D1Re0,AC1.0 [A1.7+A1.8++],T0
        DLMWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7+A1.8++]
        DLMOWT    ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7+A1.8++]
        DLWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7+A1.8++]
        DLOWT     ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7+A1.8++]
        DLWT      ADD       D1Re0,D1Re0,D1Re0 [A1.7],T0
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 [A1.7],T0
        DMWT      ADD       D1Re0,D1Re0,AC1.0 [A1.7],T0
        DMOWT     ADD       D1Re0,D1Re0,AC1.0 [A1.7],T0
        DMWT      ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7]
        DMOWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7]
        DWT       ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7]
        DOWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7]
        DWT       ADD       D1Re0,D1Re0,D1Re0 [A1.7++],T0
        DOWT      ADD       D1Re0,D1Re0,D1Re0 [A1.7++],T0
        DWT       ADD       AC1.0,AC1.0,AC0.0 [A1.7++],T0
        DLMWT     ADD       D1Re0,D1Re0,AC1.0 [A1.7++],T0
        DLMOWT    ADD       D1Re0,D1Re0,AC1.0 [A1.7++],T0
        DLMWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7++]
        DLMOWT    ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7++]
        DLWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7++]
        DLOWT     ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7++]
        DLWT      ADD       D1Re0,D1Re0,D1Re0 [A1.7--],T0
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 [A1.7--],T0
        DMWT      ADD       D1Re0,D1Re0,AC1.0 [A1.7--],T0
        DMOWT     ADD       D1Re0,D1Re0,AC1.0 [A1.7--],T0
        DMWT      ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7--]
        DMOWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.7--]
        DWT       ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7--]
        DOWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.7--]
        DWT       ADD       D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T0
        DOWT      ADD       D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T0
        DWT       ADD       AC1.0,AC1.0,AC0.0 [A1.8+A1LbP++],T0
        DLMWT     ADD       D1Re0,D1Re0,AC1.0 [A1.8+A1LbP++],T0
        DLMOWT    ADD       D1Re0,D1Re0,AC1.0 [A1.8+A1LbP++],T0
        DLMWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8+A1LbP++]
        DLMOWT    ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8+A1LbP++]
        DLWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8+A1LbP++]
        DLOWT     ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8+A1LbP++]
        DLWT      ADD       D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T0
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T0
        DMWT      ADD       D1Re0,D1Re0,AC1.0 [A1.8+A1.7++],T0
        DMOWT     ADD       D1Re0,D1Re0,AC1.0 [A1.8+A1.7++],T0
        DMWT      ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8+A1.7++]
        DMOWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8+A1.7++]
        DWT       ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8+A1.7++]
        DOWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8+A1.7++]
        DWT       ADD       D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T0
        DOWT      ADD       D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T0
        DWT       ADD       AC1.0,AC1.0,AC0.0 [A1.8+A1.8++],T0
        DLMWT     ADD       D1Re0,D1Re0,AC1.0 [A1.8+A1.8++],T0
        DLMOWT    ADD       D1Re0,D1Re0,AC1.0 [A1.8+A1.8++],T0
        DLMWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8+A1.8++]
        DLMOWT    ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8+A1.8++]
        DLWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8+A1.8++]
        DLOWT     ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8+A1.8++]
        DLWT      ADD       D1Re0,D1Re0,D1Re0 [A1.8],T0
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 [A1.8],T0
        DMWT      ADD       D1Re0,D1Re0,AC1.0 [A1.8],T0
        DMOWT     ADD       D1Re0,D1Re0,AC1.0 [A1.8],T0
        DMWT      ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8]
        DMOWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8]
        DWT       ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8]
        DOWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8]
        DWT       ADD       D1Re0,D1Re0,D1Re0 [A1.8++],T0
        DOWT      ADD       D1Re0,D1Re0,D1Re0 [A1.8++],T0
        DWT       ADD       AC1.0,AC1.0,AC0.0 [A1.8++],T0
        DLMWT     ADD       D1Re0,D1Re0,AC1.0 [A1.8++],T0
        DLMOWT    ADD       D1Re0,D1Re0,AC1.0 [A1.8++],T0
        DLMWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8++]
        DLMOWT    ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8++]
        DLWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8++]
        DLOWT     ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8++]
        DLWT      ADD       D1Re0,D1Re0,D1Re0 [A1.8--],T0
        DLOWT     ADD       D1Re0,D1Re0,D1Re0 [A1.8--],T0
        DMWT      ADD       D1Re0,D1Re0,AC1.0 [A1.8--],T0
        DMOWT     ADD       D1Re0,D1Re0,AC1.0 [A1.8--],T0
        DMWT      ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8--]
        DMOWT     ADD       D1Re0,D1Re0,D1Re0 T0,[A1.8--]
        DWT       ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8--]
        DOWT      ADD       D1Re0,D1Re0,AC1.0 T0,[A1.8--]
        D         MOV       [D1BW.1],D1Re0
        DO        MOV       [D1BW.1],D1Re0
        DM        ADD       [D1BW.1],D1Re0,A1LbP
        DM        MOV       [D1BW.1],D1.7
        DMO       ADD       [D1BW.1],D1Re0,D1.7
        DM        ADD       [D1BW.1],D1Re0,D0Re0
        D         MOV       [D1BW.1],RD
        DO        MOV       [D1BW.1],RD
        D         MOV       [D1BW.1],AC1.0
        DO        MOV       [D1BW.1],AC1.0
        DM        ADD       [D1BW.1],D1Re0,A0FrP
        DL        MOV       [D1BW.1],[D1BR.1]
        DLO       MOV       [D1BW.1],[D1BR.1]
        DL        MOV       [D1BW.1],[D1BR.1++]
        DLO       MOV       [D1BW.1],[D1BR.1++]
        DM        MOV       [D1BW.1],[D1BR.1+D1BRI.1++]
        DMO       ADD       [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DM        ADD       [D1BW.1],D1.7,D1Re0
        DM        ADD       [D1BW.1],D1.7,A1LbP
        DLM       ADD       [D1BW.1],D1.7,D1.7
        DM        ADD       [D1BW.1],D1.7,D0Re0
        DM        ADD       [D1BW.1],D1.7,RD
        DLM       ADD       [D1BW.1],D1.7,AC1.0
        DMO       ADD       [D1BW.1],D1.7,A0FrP
        DLMO      ADD       [D1BW.1],D1.7,[D1BR.1]
        DLMO      ADD       [D1BW.1],D1.7,[D1BR.1++]
        DO        ADD       [D1BW.1],D1.7,A0.7
        DLMT      ADD       D1Re0,D1Re0,D1Re0 [D1BW.1],T0
        DLT       ADD       D1Re0,D1Re0,AC1.0 [D1BW.1],T0
        DLT       ADD       D1Re0,D1Re0,D1Re0 T0,[D1BR.1]
        DMT       ADD       D1Re0,D1Re0,AC1.0 T0,[D1BR.1]
        DM        ADD       [D1BW.1],[D1BR.1],D1Re0
        DM        ADD       [D1BW.1],[D1BR.1],A1LbP
        DLM       ADD       [D1BW.1],[D1BR.1],D1.7
        DM        ADD       [D1BW.1],[D1BR.1],D0Re0
        DM        ADD       [D1BW.1],[D1BR.1],RD
        DM        ADD       [D1BW.1],[D1BR.1],AC1.0
        DM        ADD       [D1BW.1],[D1BR.1],A0FrP
        DMO       ADD       [D1BW.1],[D1BR.1],[D1BR.1]
        DMO       ADD       [D1BW.1],[D1BR.1],[D1BR.1++]
        DLM       ADD       [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DM        ADD       [D1BW.1],[D1BR.1++],D1Re0
        DM        ADD       [D1BW.1],[D1BR.1++],A1LbP
        DLM       ADD       [D1BW.1],[D1BR.1++],D1.7
        DM        ADD       [D1BW.1],[D1BR.1++],D0Re0
        DM        ADD       [D1BW.1],[D1BR.1++],RD
        DM        ADD       [D1BW.1],[D1BR.1++],AC1.0
        DM        ADD       [D1BW.1],[D1BR.1++],A0FrP
        DMO       ADD       [D1BW.1],[D1BR.1++],[D1BR.1]
        DMO       ADD       [D1BW.1],[D1BR.1++],[D1BR.1++]
        DLM       ADD       [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DM        ADD       [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DM        ADD       [D1BW.1],[D1BR.1+D1BRI.1++],A1LbP
        DLM       ADD       [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DM        ADD       [D1BW.1],[D1BR.1+D1BRI.1++],D0Re0
        DM        ADD       [D1BW.1],[D1BR.1+D1BRI.1++],RD
        DM        ADD       [D1BW.1],[D1BR.1+D1BRI.1++],AC1.0
        DM        ADD       [D1BW.1],[D1BR.1+D1BRI.1++],A0FrP
        DMO       ADD       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DMO       ADD       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLM       ADD       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         MOV       [D1BW.1++],D1Re0
        DO        MOV       [D1BW.1++],D1Re0
        DM        ADD       [D1BW.1++],D1Re0,A1LbP
        DM        MOV       [D1BW.1++],D1.7
        DMO       ADD       [D1BW.1++],D1Re0,D1.7
        DM        ADD       [D1BW.1++],D1Re0,D0Re0
        D         MOV       [D1BW.1++],RD
        DO        MOV       [D1BW.1++],RD
        D         MOV       [D1BW.1++],AC1.0
        DO        MOV       [D1BW.1++],AC1.0
        DM        ADD       [D1BW.1++],D1Re0,A0FrP
        DL        MOV       [D1BW.1++],[D1BR.1]
        DLO       MOV       [D1BW.1++],[D1BR.1]
        DL        MOV       [D1BW.1++],[D1BR.1++]
        DLO       MOV       [D1BW.1++],[D1BR.1++]
        DM        MOV       [D1BW.1++],[D1BR.1+D1BRI.1++]
        DMO       ADD       [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DM        ADD       [D1BW.1++],D1.7,D1Re0
        DM        ADD       [D1BW.1++],D1.7,A1LbP
        DLM       ADD       [D1BW.1++],D1.7,D1.7
        DM        ADD       [D1BW.1++],D1.7,D0Re0
        DM        ADD       [D1BW.1++],D1.7,RD
        DM        ADD       [D1BW.1++],D1.7,AC1.0
        DM        ADD       [D1BW.1++],D1.7,A0FrP
        DMO       ADD       [D1BW.1++],D1.7,[D1BR.1]
        DMO       ADD       [D1BW.1++],D1.7,[D1BR.1++]
        DLM       ADD       [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DMT       ADD       D1Re0,D1Re0,D1Re0 [D1BW.1++],T0
        DT        ADD       D1Re0,D1Re0,AC1.0 [D1BW.1++],T0
        DT        ADD       D1Re0,D1Re0,D1Re0 T0,[D1BR.1++]
        DT        ADD       AC1.0,AC1.0,AC0.0 T0,[D1BR.1++]
        DLMOT     ADD       D1Re0,D1Re0,AC1.0 T0,[D1BR.1++]
        DLMO      ADD       [D1BW.1++],[D1BR.1],D1Re0
        DM        ADD       [D1BW.1++],[D1BR.1],A1.7
        DLMO      ADD       [D1BW.1++],[D1BR.1],D1.7
        DMO       ADD       [D1BW.1++],[D1BR.1],D0.7
        DLMO      ADD       [D1BW.1++],[D1BR.1],RD
        DLMO      ADD       [D1BW.1++],[D1BR.1],AC1.0
        DLM       ADD       [D1BW.1++],[D1BR.1],[D1BR.1]
        DLM       ADD       [D1BW.1++],[D1BR.1],[D1BR.1++]
        DM        ADD       [D1BW.1++],[D1BR.1],A0.7
        DLMO      ADD       [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLMO      ADD       [D1BW.1++],[D1BR.1++],D1Re0
        DM        ADD       [D1BW.1++],[D1BR.1++],A1.7
        DLMO      ADD       [D1BW.1++],[D1BR.1++],D1.7
        DMO       ADD       [D1BW.1++],[D1BR.1++],D0.7
        DLMO      ADD       [D1BW.1++],[D1BR.1++],RD
        DLMO      ADD       [D1BW.1++],[D1BR.1++],AC1.0
        DLM       ADD       [D1BW.1++],[D1BR.1++],[D1BR.1]
        DLM       ADD       [D1BW.1++],[D1BR.1++],[D1BR.1++]
        DM        ADD       [D1BW.1++],[D1BR.1++],A0.7
        DLMO      ADD       [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLMO      ADD       [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        DM        ADD       [D1BW.1++],[D1BR.1+D1BRI.1++],A1.7
        DLMO      ADD       [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        DMO       ADD       [D1BW.1++],[D1BR.1+D1BRI.1++],D0.7
        DLMO      ADD       [D1BW.1++],[D1BR.1+D1BRI.1++],RD
        DLMO      ADD       [D1BW.1++],[D1BR.1+D1BRI.1++],AC1.0
        DLM       ADD       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLM       ADD       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DM        ADD       [D1BW.1++],[D1BR.1+D1BRI.1++],A0.7
        DLMO      ADD       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLM       MOV       [D1BW.1+D1BWI.1++],D1Re0
        DLMO      MOV       [D1BW.1+D1BWI.1++],D1Re0
        DM        ADD       [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DO        ADD       [D1BW.1+D1BWI.1++],D1Re0,A1.7
        DLMO      MOV       [D1BW.1+D1BWI.1++],D1.7
        DMO       ADD       [D1BW.1+D1BWI.1++],D1Re0,D0.7
        DLM       MOV       [D1BW.1+D1BWI.1++],RD
        DLMO      MOV       [D1BW.1+D1BWI.1++],RD
        DLM       MOV       [D1BW.1+D1BWI.1++],AC1.0
        DLMO      MOV       [D1BW.1+D1BWI.1++],AC1.0
        DM        MOV       [D1BW.1+D1BWI.1++],[D1BR.1]
        DMO       MOV       [D1BW.1+D1BWI.1++],[D1BR.1]
        DM        MOV       [D1BW.1+D1BWI.1++],[D1BR.1++]
        DMO       MOV       [D1BW.1+D1BWI.1++],[D1BR.1++]
        DM        ADD       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DO        ADD       [D1BW.1+D1BWI.1++],D1Re0,A0.7
        DLMO      MOV       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++]
        DLMO      ADD       [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DM        ADD       [D1BW.1+D1BWI.1++],D1.7,A1.7
        DLMO      ADD       [D1BW.1+D1BWI.1++],D1.7,D1.7
        DMO       ADD       [D1BW.1+D1BWI.1++],D1.7,D0.7
        DLMO      ADD       [D1BW.1+D1BWI.1++],D1.7,RD
        DLMO      ADD       [D1BW.1+D1BWI.1++],D1.7,AC1.0
        DLM       ADD       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        DLM       ADD       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        DM        ADD       [D1BW.1+D1BWI.1++],D1.7,A0.7
        DLMO      ADD       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DLMOT     ADD       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T0
        DLOT      ADD       D1Re0,D1Re0,AC1.0 [D1BW.1+D1BWI.1++],T0
        DLOT      ADD       D1Re0,D1Re0,D1Re0 T0,[D1BR.1+D1BRI.1++]
        DMOT      ADD       D1Re0,D1Re0,AC1.0 T0,[D1BR.1+D1BRI.1++]
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        D         ADD       [D1BW.1+D1BWI.1++],[D1BR.1],A1.7
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1],A1.7
        DM        ADD       [D1BW.1+D1BWI.1++],[D1BR.1],D0.7
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1],RD
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1],AC1.0
        DM        ADD       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        DM        ADD       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        D         ADD       [D1BW.1+D1BWI.1++],[D1BR.1],A0.7
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1],A0.7
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        D         ADD       [D1BW.1+D1BWI.1++],[D1BR.1++],A1.7
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1++],A1.7
        DM        ADD       [D1BW.1+D1BWI.1++],[D1BR.1++],D0.7
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1++],RD
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1++],AC1.0
        DM        ADD       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        DM        ADD       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        D         ADD       [D1BW.1+D1BWI.1++],[D1BR.1++],A0.7
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1++],A0.7
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         ADD       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1.7
        DMO       ADD       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1.7
        DM        ADD       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D0.7
        D         ADD       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],AC1.0
        D         ADD       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0FrP
        DO        ADD       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DO        ADD       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        ADD       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ADD       D0Re0,D0Re0,#0x7fff
        D         ADD       D0.7,D0.7,#-0x8000
        D         ADD       [D0AW.0],[D0AR.0],#-0x1
        D         ADD       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x7fff
        D         ADD       D1Re0,D1Re0,#-0x8000
        D         ADD       D1.7,D1.7,#-0x1
        D         ADD       [D1BW.1++],[D1BR.1++],#0x7fff
        D         ADD       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#-0x8000
        DHNE      ADDPL     D0Re0,D0Re0,D0.7
        DLLS      ADDLS     D0Re0,D0Re0,[D0AR.0]
        DECS      ADDLE     D0Re0,D0Re0,A0FrP
        DECS      ADDLE     D0Re0,D0.7,D0.7
        DLNE      ADDNE     D0Re0,D0.7,[D0AR.0]
        DECS      ADDLE     D0Re0,D0.7,A0FrP
        D         ADDNV     D0Re0,[D0AR.0],D1Re0
        D         ADDNV     D0Re0,[D0AR.0++],D0Re0
        D         ADDNV     D0Re0,[D0AR.0++],[D0AR.0++]
        D         ADDNV     D0Re0,[D0AR.0+D0ARI.0++],D1Re0
        DLNE      ADDNE     D0.7,D0Re0,D0Re0
        D         ADDNV     D0.7,D0Re0,D1Re0
        D         ADDNV     D0.7,D0Re0,[D0AR.0++]
        D         ADDNV     D0.7,D0.7,D0Re0
        DECS      ADDLE     D0.7,D0.7,D1.7
        DECS      ADDLE     D0.7,D0.7,A0FrP
        D         ADDNV     D0.7,[D0AR.0],D1Re0
        D         ADDNV     D0.7,[D0AR.0++],D0Re0
        D         ADDNV     D0.7,[D0AR.0++],[D0AR.0++]
        D         ADDNV     D0.7,[D0AR.0+D0ARI.0++],D1Re0
        DLLS      ADDLS     D0.16,D0Re0,D0Re0
        D         ADDNV     D0.16,D0Re0,D1Re0
        DECS      ADDLE     D0.16,D0Re0,A0FrP
        DECS      ADDLE     D0.16,D0.7,D0.7
        DLNE      ADDNE     D0.16,D0.7,[D0AR.0]
        D         ADDNV     D0.16,D0.7,A0FrP
        D         ADDNV     D0.16,[D0AR.0],D1.7
        D         ADDNV     D0.16,[D0AR.0++],A1LbP
        DHCS      ADDVS     D0.16,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ADDNV     D0.16,[D0AR.0+D0ARI.0++],D1.7
        D         ADDNV     D1Re0,D1Re0,D1Re0
        D         ADDNV     D1Re0,D1Re0,D0Re0
        D         ADDNV     D1Re0,D1Re0,[D1BR.1]
        D         ADDNV     D1Re0,D1.7,D1Re0
        D         ADDNV     D1Re0,D1.7,D0Re0
        DECS      ADDLE     D1Re0,D1.7,[D1BR.1++]
        D         ADDNV     D1Re0,[D1BR.1],D1.7
        D         ADDNV     D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ADDNV     D1Re0,[D1BR.1++],RD
        D         ADDNV     D1Re0,[D1BR.1+D1BRI.1++],D1.7
        D         ADDNV     D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ADDNV     D1.7,D1Re0,D1.7
        DECS      ADDLE     D1.7,D1Re0,A0FrP
        D         ADDNV     D1.7,D1Re0,A0.7
        D         ADDNV     D1.7,D1.7,A1.7
        D         ADDNV     D1.7,D1.7,[D1BR.1]
        D         ADDNV     D1.7,[D1BR.1],A1LbP
        D         ADDNV     D1.7,[D1BR.1],[D1BR.1++]
        D         ADDNV     D1.7,[D1BR.1++],D0.7
        D         ADDNV     D1.7,[D1BR.1+D1BRI.1++],A1LbP
        D         ADDNV     D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DECS      ADDLE     D1.16,D1Re0,A1.7
        D         ADDNV     D1.16,D1Re0,RD
        DECS      ADDLE     D1.16,D1Re0,A0.7
        DECS      ADDLE     D1.16,D1.7,A1.7
        D         ADDNV     D1.16,D1.7,RD
        D         ADDNV     D1.16,D1.7,[D1BR.1+D1BRI.1++]
        D         ADDNV     D1.16,[D1BR.1],RD
        D         ADDNV     D1.16,[D1BR.1++],D1.7
        D         ADDNV     D1.16,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ADDNV     D1.16,[D1BR.1+D1BRI.1++],RD
        DM        ADDS      D0Re0,D0Re0,D0Re0
        DMO       ADDS      D0Re0,D0Re0,D0Re0
        DO        ADDS      D0Re0,D0Re0,A1LbP
        DL        ADDS      D0Re0,D0Re0,D0.7
        DMO       ADDS      D0Re0,D0Re0,A1.7
        DO        ADDS      D0Re0,D0Re0,D1Re0
        D         MOVS      D0Re0,[D0AR.0]
        DL        MOVS      D0Re0,RD
        DMO       ADDS      D0Re0,D0Re0,[D0AR.0]
        DLMO      ADDS      D0Re0,D0Re0,[D0AR.0]
        DL        MOVS      D0Re0,AC0.0
        DLO       MOVS      D0Re0,AC0.0
        DL        MOVS      D0Re0,[D0AR.0++]
        DLO       MOVS      D0Re0,[D0AR.0++]
        DL        MOVS      D0Re0,[D0AR.0+D0ARI.0++]
        DLO       MOVS      D0Re0,[D0AR.0+D0ARI.0++]
        DM        ADDS      D0Re0,D0Re0,A0.7
        DMO       ADDS      D0Re0,D0.7,D0Re0
        D         ADDS      D0Re0,D0.7,A1.7
        DMO       ADDS      D0Re0,D0.7,A1.7
        DM        ADDS      D0Re0,D0.7,D1.7
        DLM       ADDS      D0Re0,D0.7,[D0AR.0]
        D         ADDS      D0Re0,D0.7,AC0.0
        D         ADDS      D0Re0,D0.7,[D0AR.0++]
        D         ADDS      D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        D         ADDS      D0Re0,D0.7,A0FrP
        D         ADDS      D0Re0,[D0AR.0],D0Re0
        D         ADDS      D0Re0,[D0AR.0],A1LbP
        DL        ADDS      D0Re0,[D0AR.0],D0.7
        D         ADDS      D0Re0,[D0AR.0],D1Re0
        D         ADDS      D0Re0,[D0AR.0],[D0AR.0]
        DMO       ADDS      D0Re0,[D0AR.0],[D0AR.0]
        DLM       ADDS      D0Re0,[D0AR.0],AC0.0
        DLM       ADDS      D0Re0,[D0AR.0],[D0AR.0++]
        DLM       ADDS      D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        DMO       ADDS      D0Re0,[D0AR.0],A0FrP
        DLM       ADDS      D0Re0,[D0AR.0++],D0Re0
        DMO       ADDS      D0Re0,[D0AR.0++],A1LbP
        DO        ADDS      D0Re0,[D0AR.0++],A1.7
        DMO       ADDS      D0Re0,[D0AR.0++],D1Re0
        DM        ADDS      D0Re0,[D0AR.0++],RD
        DLO       ADDS      D0Re0,[D0AR.0++],RD
        DLO       ADDS      D0Re0,[D0AR.0++],AC0.0
        DLO       ADDS      D0Re0,[D0AR.0++],[D0AR.0++]
        DLO       ADDS      D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DO        ADDS      D0Re0,[D0AR.0++],A0.7
        DLO       ADDS      D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        DM        ADDS      D0Re0,[D0AR.0+D0ARI.0++],D0.7
        DLO       ADDS      D0Re0,[D0AR.0+D0ARI.0++],D0.7
        DO        ADDS      D0Re0,[D0AR.0+D0ARI.0++],D1.7
        DO        ADDS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DM        ADDS      D0Re0,[D0AR.0+D0ARI.0++],AC0.0
        DM        ADDS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DM        ADDS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DM        ADDS      D0Re0,[D0AR.0+D0ARI.0++],A0FrP
        DWT       ADDS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T0
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T0
        DWT       ADDS      D0Re0,D0Re0,AC0.0 [A0FrP+A0FrP++],T0
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 [A0FrP+A0FrP++],T0
        DWT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0FrP++]
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0FrP++]
        DWT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0FrP++]
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0FrP++]
        DWT       ADDS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T0
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T0
        DWT       ADDS      D0Re0,D0Re0,AC0.0 [A0FrP+A0.7++],T0
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 [A0FrP+A0.7++],T0
        DWT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0.7++]
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0.7++]
        DWT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0.7++]
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0.7++]
        DWT       ADDS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T0
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T0
        DWT       ADDS      D0Re0,D0Re0,AC0.0 [A0FrP+A0.8++],T0
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 [A0FrP+A0.8++],T0
        DWT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0.8++]
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP+A0.8++]
        DWT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0.8++]
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP+A0.8++]
        DWT       ADDS      D0Re0,D0Re0,D0Re0 [A0FrP],T0
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 [A0FrP],T0
        DWT       ADDS      D0Re0,D0Re0,AC0.0 [A0FrP],T0
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 [A0FrP],T0
        DWT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP]
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP]
        DWT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP]
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP]
        DWT       ADDS      D0Re0,D0Re0,D0Re0 [A0FrP++],T0
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 [A0FrP++],T0
        DWT       ADDS      D0Re0,D0Re0,AC0.0 [A0FrP++],T0
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 [A0FrP++],T0
        DWT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP++]
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP++]
        DWT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP++]
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP++]
        DWT       ADDS      D0Re0,D0Re0,D0Re0 [A0FrP--],T0
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 [A0FrP--],T0
        DWT       ADDS      D0Re0,D0Re0,AC0.0 [A0FrP--],T0
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 [A0FrP--],T0
        DWT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP--]
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0FrP--]
        DWT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP--]
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A0FrP--]
        D         MOVS      D0.7,D0Re0
        DO        MOVS      D0.7,D0Re0
        DM        ADDS      D0.7,D0Re0,A1LbP
        DM        MOVS      D0.7,D0.7
        DMO       ADDS      D0.7,D0Re0,D0.7
        DM        ADDS      D0.7,D0Re0,D1Re0
        D         ADDS      D0.7,D0Re0,RD
        DL        MOVS      D0.7,[D0AR.0]
        DO        MOVS      D0.7,RD
        DLO       MOVS      D0.7,RD
        DL        ADDS      D0.7,D0Re0,AC0.0
        DLO       ADDS      D0.7,D0Re0,AC0.0
        DL        ADDS      D0.7,D0Re0,[D0AR.0++]
        DLO       ADDS      D0.7,D0Re0,[D0AR.0++]
        DL        ADDS      D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DLO       ADDS      D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         ADDS      D0.7,D0Re0,A0.7
        DLT       ADDS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T0
        DLOT      ADDS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T0
        DLT       ADDS      D0Re0,D0Re0,AC0.0 [A0.7+A0FrP++],T0
        DLOT      ADDS      D0Re0,D0Re0,AC0.0 [A0.7+A0FrP++],T0
        DLT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7+A0FrP++]
        DLOT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7+A0FrP++]
        DLT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7+A0FrP++]
        DLOT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7+A0FrP++]
        DMT       ADDS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T0
        DO        ADDS      D0.7,D0.7,D0Re0
        DLOWT     ADDS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T0
        DLT       ADDS      D0Re0,D0Re0,AC0.0 [A0.7+A0.7++],T0
        DLOT      ADDS      D0Re0,D0Re0,AC0.0 [A0.7+A0.7++],T0
        D         ADDS      D0.7,D0.7,D0.7
        DMO       ADDS      D0.7,D0.7,D0.7
        D         ADDS      D0.7,D0.7,D1.7
        DM        ADDS      D0.7,D0.7,[D0AR.0]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.7++]
        DMOT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.7++]
        DLMOWT    ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.7++]
        DLT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7+A0.7++]
        DMO       ADDS      D0.7,D0.7,AC0.0
        DLMOWT    ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7+A0.7++]
        DLMO      ADDS      D0.7,D0.7,[D0AR.0++]
        DLMO      ADDS      D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DMO       ADDS      D0.7,D0.7,A0.7
        DLMWT     ADDS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T0
        DLMOWT    ADDS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T0
        DLMWT     ADDS      D0Re0,D0Re0,AC0.0 [A0.7+A0.8++],T0
        DLMOWT    ADDS      D0Re0,D0Re0,AC0.0 [A0.7+A0.8++],T0
        DLMWT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.8++]
        DLMOWT    ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7+A0.8++]
        DLMWT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7+A0.8++]
        DLMOWT    ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7+A0.8++]
        DLT       ADDS      D0Re0,D0Re0,D0Re0 [A0.7],T0
        DMO       ADDS      D0.7,[D0AR.0],D0Re0
        DLMOWT    ADDS      D0Re0,D0Re0,D0Re0 [A0.7],T0
        DLMWT     ADDS      D0Re0,D0Re0,AC0.0 [A0.7],T0
        DLMOWT    ADDS      D0Re0,D0Re0,AC0.0 [A0.7],T0
        DM        ADDS      D0.7,[D0AR.0],A1.7
        DLMO      ADDS      D0.7,[D0AR.0],D0.7
        DMO       ADDS      D0.7,[D0AR.0],D1.7
        DMWT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7]
        DO        ADDS      D0.7,[D0AR.0],RD
        DLO       ADDS      D0.7,[D0AR.0],RD
        DWT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7]
        DLO       ADDS      D0.7,[D0AR.0],AC0.0
        DL        ADDS      D0.7,[D0AR.0],[D0AR.0++]
        DL        ADDS      D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DO        ADDS      D0.7,[D0AR.0],A0FrP
        DWT       ADDS      D0Re0,D0Re0,D0Re0 [A0.7++],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A0.7++],T0
        DLO       ADDS      D0.7,[D0AR.0++],D0Re0
        DMT       ADDS      D0Re0,D0Re0,AC0.0 [A0.7++],T0
        DMOT      ADDS      D0Re0,D0Re0,AC0.0 [A0.7++],T0
        DO        ADDS      D0.7,[D0AR.0++],A1LbP
        DO        ADDS      D0.7,[D0AR.0++],D0.7
        DO        ADDS      D0.7,[D0AR.0++],D1Re0
        DT        ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7++]
        DLWT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7++]
        DMO       ADDS      D0.7,[D0AR.0++],[D0AR.0]
        DLMO      ADDS      D0.7,[D0AR.0++],[D0AR.0]
        DMWT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7++]
        DOT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7++]
        DLMO      ADDS      D0.7,[D0AR.0++],AC0.0
        DMO       ADDS      D0.7,[D0AR.0++],[D0AR.0++]
        DMO       ADDS      D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DM        ADDS      D0.7,[D0AR.0++],A0.7
        DMO       ADDS      D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         ADDS      D0.7,[D0AR.0+D0ARI.0++],A1.7
        DMO       ADDS      D0.7,[D0AR.0+D0ARI.0++],A1.7
        DM        ADDS      D0.7,[D0AR.0+D0ARI.0++],D1.7
        DLM       ADDS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ADDS      D0.7,[D0AR.0+D0ARI.0++],AC0.0
        D         ADDS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ADDS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ADDS      D0.7,[D0AR.0+D0ARI.0++],A0FrP
        DT        ADDS      D0Re0,D0Re0,D0Re0 [A0.7--],T0
        DOT       ADDS      D0Re0,D0Re0,D0Re0 [A0.7--],T0
        DT        ADDS      D0Re0,D0Re0,AC0.0 [A0.7--],T0
        DOT       ADDS      D0Re0,D0Re0,AC0.0 [A0.7--],T0
        DT        ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7--]
        DOT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.7--]
        DT        ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7--]
        DOT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.7--]
        DT        ADDS      D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T0
        DOT       ADDS      D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T0
        DT        ADDS      D0Re0,D0Re0,AC0.0 [A0.8+A0FrP++],T0
        DOT       ADDS      D0Re0,D0Re0,AC0.0 [A0.8+A0FrP++],T0
        DT        ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8+A0FrP++]
        DOT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8+A0FrP++]
        DT        ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8+A0FrP++]
        DOT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8+A0FrP++]
        DT        ADDS      D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T0
        DOT       ADDS      D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T0
        DT        ADDS      D0Re0,D0Re0,AC0.0 [A0.8+A0.7++],T0
        DOT       ADDS      D0Re0,D0Re0,AC0.0 [A0.8+A0.7++],T0
        DT        ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8+A0.7++]
        DOT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8+A0.7++]
        DT        ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8+A0.7++]
        DOT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8+A0.7++]
        DT        ADDS      D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T0
        DOT       ADDS      D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T0
        DT        ADDS      D0Re0,D0Re0,AC0.0 [A0.8+A0.8++],T0
        DOT       ADDS      D0Re0,D0Re0,AC0.0 [A0.8+A0.8++],T0
        DT        ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8+A0.8++]
        DOT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8+A0.8++]
        DT        ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8+A0.8++]
        DOT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8+A0.8++]
        DT        ADDS      D0Re0,D0Re0,D0Re0 [A0.8],T0
        DOT       ADDS      D0Re0,D0Re0,D0Re0 [A0.8],T0
        DT        ADDS      D0Re0,D0Re0,AC0.0 [A0.8],T0
        DOT       ADDS      D0Re0,D0Re0,AC0.0 [A0.8],T0
        DT        ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8]
        DOT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8]
        DT        ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8]
        DOT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8]
        DT        ADDS      D0Re0,D0Re0,D0Re0 [A0.8++],T0
        DOT       ADDS      D0Re0,D0Re0,D0Re0 [A0.8++],T0
        DT        ADDS      D0Re0,D0Re0,AC0.0 [A0.8++],T0
        DOT       ADDS      D0Re0,D0Re0,AC0.0 [A0.8++],T0
        DT        ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8++]
        DOT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8++]
        DT        ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8++]
        DOT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8++]
        DT        ADDS      D0Re0,D0Re0,D0Re0 [A0.8--],T0
        DOT       ADDS      D0Re0,D0Re0,D0Re0 [A0.8--],T0
        DT        ADDS      D0Re0,D0Re0,AC0.0 [A0.8--],T0
        DOT       ADDS      D0Re0,D0Re0,AC0.0 [A0.8--],T0
        DT        ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8--]
        DOT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A0.8--]
        DT        ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8--]
        DOT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A0.8--]
        D         ADDS      [D0AW.0],D0Re0,D0Re0
        DO        ADDS      [D0AW.0],D0Re0,D0Re0
        D         ADDS      [D0AW.0],D0Re0,A1LbP
        DM        ADDS      [D0AW.0],D0Re0,A1.7
        DO        MOVS      [D0AW.0],D0.7
        D         ADDS      [D0AW.0],D0Re0,D1Re0
        D         ADDS      [D0AW.0],D0Re0,[D0AR.0]
        DL        ADDS      [D0AW.0],D0Re0,[D0AR.0]
        DO        MOVS      [D0AW.0],[D0AR.0]
        DLO       MOVS      [D0AW.0],[D0AR.0]
        DM        MOVS      [D0AW.0],AC0.0
        DMO       MOVS      [D0AW.0],AC0.0
        DM        MOVS      [D0AW.0],[D0AR.0++]
        DMO       MOVS      [D0AW.0],[D0AR.0++]
        DM        MOVS      [D0AW.0],[D0AR.0+D0ARI.0++]
        DMO       MOVS      [D0AW.0],[D0AR.0+D0ARI.0++]
        DMO       ADDS      [D0AW.0],D0Re0,A0FrP
        DLM       ADDS      [D0AW.0],D0.7,D0Re0
        DMO       ADDS      [D0AW.0],D0.7,A1LbP
        DO        ADDS      [D0AW.0],D0.7,A1.7
        DMO       ADDS      [D0AW.0],D0.7,D1Re0
        DM        ADDS      [D0AW.0],D0.7,RD
        DLMO      ADDS      [D0AW.0],D0.7,[D0AR.0]
        DLMO      ADDS      [D0AW.0],D0.7,AC0.0
        DLMO      ADDS      [D0AW.0],D0.7,[D0AR.0++]
        DLMO      ADDS      [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        DMO       ADDS      [D0AW.0],D0.7,A0.7
        DLMO      ADDS      [D0AW.0],[D0AR.0],D0Re0
        DM        ADDS      [D0AW.0],[D0AR.0],A1.7
        DLMO      ADDS      [D0AW.0],[D0AR.0],D0.7
        DMO       ADDS      [D0AW.0],[D0AR.0],D1.7
        DO        ADDS      [D0AW.0],[D0AR.0],RD
        DLM       ADDS      [D0AW.0],[D0AR.0],AC0.0
        DLM       ADDS      [D0AW.0],[D0AR.0],[D0AR.0++]
        DLM       ADDS      [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DMO       ADDS      [D0AW.0],[D0AR.0],A0FrP
        DLM       ADDS      [D0AW.0],[D0AR.0++],D0Re0
        DMO       ADDS      [D0AW.0],[D0AR.0++],A1LbP
        DO        ADDS      [D0AW.0],[D0AR.0++],A1.7
        DMO       ADDS      [D0AW.0],[D0AR.0++],D1Re0
        DM        ADDS      [D0AW.0],[D0AR.0++],RD
        DLMO      ADDS      [D0AW.0],[D0AR.0++],[D0AR.0]
        DLMO      ADDS      [D0AW.0],[D0AR.0++],AC0.0
        DLMO      ADDS      [D0AW.0],[D0AR.0++],[D0AR.0++]
        DLMO      ADDS      [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DMO       ADDS      [D0AW.0],[D0AR.0++],A0.7
        DLMO      ADDS      [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DM        ADDS      [D0AW.0],[D0AR.0+D0ARI.0++],A1.7
        DLMO      ADDS      [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DMO       ADDS      [D0AW.0],[D0AR.0+D0ARI.0++],D1.7
        DO        ADDS      [D0AW.0],[D0AR.0+D0ARI.0++],RD
        DLM       ADDS      [D0AW.0],[D0AR.0+D0ARI.0++],AC0.0
        DLM       ADDS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLM       ADDS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DMO       ADDS      [D0AW.0],[D0AR.0+D0ARI.0++],A0FrP
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [D0AW.0],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [D0AW.0],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[D0AR.0]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[D0AR.0]
        DM        MOVS      [D0AW.0++],D0Re0
        DMO       MOVS      [D0AW.0++],D0Re0
        DMO       ADDS      [D0AW.0++],D0Re0,A1LbP
        DL        MOVS      [D0AW.0++],D0.7
        DMO       MOVS      [D0AW.0++],D0.7
        DMO       ADDS      [D0AW.0++],D0Re0,D1Re0
        D         MOVS      [D0AW.0++],RD
        DLM       ADDS      [D0AW.0++],D0Re0,[D0AR.0]
        DMO       ADDS      [D0AW.0++],D0Re0,RD
        DLMO      MOVS      [D0AW.0++],RD
        DLM       MOVS      [D0AW.0++],AC0.0
        DLMO      MOVS      [D0AW.0++],AC0.0
        DLM       MOVS      [D0AW.0++],[D0AR.0++]
        DLMO      MOVS      [D0AW.0++],[D0AR.0++]
        DLM       MOVS      [D0AW.0++],[D0AR.0+D0ARI.0++]
        DLMO      MOVS      [D0AW.0++],[D0AR.0+D0ARI.0++]
        DMO       ADDS      [D0AW.0++],D0Re0,A0.7
        DLMWT     ADDS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T0
        DLMOWT    ADDS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T0
        DLMWT     ADDS      D0Re0,D0Re0,AC0.0 [A1LbP+A1LbP++],T0
        DLMOWT    ADDS      D0Re0,D0Re0,AC0.0 [A1LbP+A1LbP++],T0
        DLMWT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1LbP++]
        DLMOWT    ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1LbP++]
        DLMWT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1LbP++]
        DLMOWT    ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1LbP++]
        DLT       ADDS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T0
        DMO       ADDS      [D0AW.0++],D0.7,D0Re0
        DLMOWT    ADDS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T0
        DLMWT     ADDS      D0Re0,D0Re0,AC0.0 [A1LbP+A1.7++],T0
        DLMOWT    ADDS      D0Re0,D0Re0,AC0.0 [A1LbP+A1.7++],T0
        DM        ADDS      [D0AW.0++],D0.7,A1.7
        DLMO      ADDS      [D0AW.0++],D0.7,D0.7
        DMO       ADDS      [D0AW.0++],D0.7,D1.7
        DMWT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1.7++]
        DO        ADDS      [D0AW.0++],D0.7,RD
        DLO       ADDS      [D0AW.0++],D0.7,RD
        DWT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1.7++]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1.7++]
        DLO       ADDS      [D0AW.0++],D0.7,AC0.0
        DL        ADDS      [D0AW.0++],D0.7,[D0AR.0++]
        DL        ADDS      [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DO        ADDS      [D0AW.0++],D0.7,A0FrP
        DMT       ADDS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T0
        DMOT      ADDS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T0
        DMT       ADDS      D0Re0,D0Re0,AC0.0 [A1LbP+A1.8++],T0
        DMOT      ADDS      D0Re0,D0Re0,AC0.0 [A1LbP+A1.8++],T0
        DMT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1.8++]
        DMOT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP+A1.8++]
        DMT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1.8++]
        DMOT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP+A1.8++]
        DWT       ADDS      D0Re0,D0Re0,D0Re0 [A1LbP],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1LbP],T0
        DLO       ADDS      [D0AW.0++],[D0AR.0],D0Re0
        DMT       ADDS      D0Re0,D0Re0,AC0.0 [A1LbP],T0
        DMOT      ADDS      D0Re0,D0Re0,AC0.0 [A1LbP],T0
        DO        ADDS      [D0AW.0++],[D0AR.0],A1LbP
        DO        ADDS      [D0AW.0++],[D0AR.0],D0.7
        DO        ADDS      [D0AW.0++],[D0AR.0],D1Re0
        DT        ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP]
        DLWT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP]
        DMO       ADDS      [D0AW.0++],[D0AR.0],[D0AR.0]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP]
        DL        ADDS      [D0AW.0++],[D0AR.0],AC0.0
        DOWT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP]
        DLO       ADDS      [D0AW.0++],[D0AR.0],[D0AR.0++]
        DLO       ADDS      [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DO        ADDS      [D0AW.0++],[D0AR.0],A0.7
        DL        ADDS      [D0AW.0++],[D0AR.0++],D0Re0
        DOWT      ADDS      D0Re0,D0Re0,D0Re0 [A1LbP++],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1LbP++],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1LbP++],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1LbP++],T0
        DM        ADDS      [D0AW.0++],[D0AR.0++],D0.7
        DLO       ADDS      [D0AW.0++],[D0AR.0++],D0.7
        DO        ADDS      [D0AW.0++],[D0AR.0++],D1.7
        DMT       ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP++]
        DO        ADDS      [D0AW.0++],[D0AR.0++],[D0AR.0]
        DLO       ADDS      [D0AW.0++],[D0AR.0++],[D0AR.0]
        DWT       ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP++]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP++]
        DLO       ADDS      [D0AW.0++],[D0AR.0++],AC0.0
        DL        ADDS      [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DL        ADDS      [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DO        ADDS      [D0AW.0++],[D0AR.0++],A0FrP
        DL        ADDS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DO        ADDS      [D0AW.0++],[D0AR.0+D0ARI.0++],A1LbP
        DO        ADDS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DO        ADDS      [D0AW.0++],[D0AR.0+D0ARI.0++],D1Re0
        DM        ADDS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLO       ADDS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLO       ADDS      [D0AW.0++],[D0AR.0+D0ARI.0++],AC0.0
        DLO       ADDS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLO       ADDS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DO        ADDS      [D0AW.0++],[D0AR.0+D0ARI.0++],A0.7
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1LbP--],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1LbP--],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1LbP--],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1LbP--],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP--]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1LbP--]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP--]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1LbP--]
        DLOT      ADDS      D0Re0,D0Re0,D0Re0 [D0AW.0++],T0
        DLOT      ADDS      D0Re0,D0Re0,AC0.0 [D0AW.0++],T0
        DLOT      ADDS      D0Re0,D0Re0,D0Re0 T0,[D0AR.0++]
        DLOT      ADDS      D0Re0,D0Re0,AC0.0 T0,[D0AR.0++]
        DLM       ADDS      [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DLMO      ADDS      [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        D         MOVS      [D0AW.0+D0AWI.0++],D0.7
        DO        ADDS      [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DLMO      ADDS      [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DO        ADDS      [D0AW.0+D0AWI.0++],D0Re0,D1.7
        DM        MOVS      [D0AW.0+D0AWI.0++],[D0AR.0]
        DO        ADDS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DLO       ADDS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DM        ADDS      [D0AW.0+D0AWI.0++],D0Re0,AC0.0
        DMO       ADDS      [D0AW.0+D0AWI.0++],D0Re0,AC0.0
        DM        ADDS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DMO       ADDS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DM        ADDS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DMO       ADDS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DO        ADDS      [D0AW.0+D0AWI.0++],D0Re0,A0FrP
        DL        ADDS      [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DO        ADDS      [D0AW.0+D0AWI.0++],D0.7,A1LbP
        DO        ADDS      [D0AW.0+D0AWI.0++],D0.7,D0.7
        DO        ADDS      [D0AW.0+D0AWI.0++],D0.7,D1Re0
        DM        ADDS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DLO       ADDS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DLO       ADDS      [D0AW.0+D0AWI.0++],D0.7,AC0.0
        DLO       ADDS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DLO       ADDS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DO        ADDS      [D0AW.0+D0AWI.0++],D0.7,A0.7
        DLO       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DM        ADDS      [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DLO       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DO        ADDS      [D0AW.0+D0AWI.0++],[D0AR.0],D1.7
        DLM       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        D         ADDS      [D0AW.0+D0AWI.0++],[D0AR.0],AC0.0
        D         ADDS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        D         ADDS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ADDS      [D0AW.0+D0AWI.0++],[D0AR.0],A0FrP
        D         ADDS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         ADDS      [D0AW.0+D0AWI.0++],[D0AR.0++],A1LbP
        DL        ADDS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        D         ADDS      [D0AW.0+D0AWI.0++],[D0AR.0++],D1Re0
        D         ADDS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DO        ADDS      [D0AW.0+D0AWI.0++],[D0AR.0++],RD
        DLM       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0++],AC0.0
        DLM       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DLM       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DMO       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0++],A0FrP
        DLM       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DMO       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A1LbP
        DO        ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A1.7
        DMO       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D1Re0
        DM        ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],RD
        DLO       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLO       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],AC0.0
        DLO       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLO       ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DO        ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A0.7
        DLOT      ADDS      D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T0
        DLOT      ADDS      D0Re0,D0Re0,AC0.0 [D0AW.0+D0AWI.0++],T0
        DLOT      ADDS      D0Re0,D0Re0,D0Re0 T0,[D0AR.0+D0ARI.0++]
        DLOT      ADDS      D0Re0,D0Re0,AC0.0 T0,[D0AR.0+D0ARI.0++]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.7+A1LbP++],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.7+A1LbP++],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7+A1LbP++]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7+A1LbP++]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7+A1LbP++]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7+A1LbP++]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.7+A1.7++],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.7+A1.7++],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7+A1.7++]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7+A1.7++]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7+A1.7++]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7+A1.7++]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.7+A1.8++],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.7+A1.8++],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7+A1.8++]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7+A1.8++]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7+A1.8++]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7+A1.8++]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.7],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.7],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.7],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.7],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.7++],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.7++],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.7++],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.7++],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7++]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7++]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7++]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7++]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.7--],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.7--],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.7--],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.7--],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7--]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.7--]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7--]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.7--]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.8+A1LbP++],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.8+A1LbP++],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8+A1LbP++]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8+A1LbP++]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8+A1LbP++]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8+A1LbP++]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.8+A1.7++],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.8+A1.7++],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8+A1.7++]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8+A1.7++]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8+A1.7++]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8+A1.7++]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.8+A1.8++],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.8+A1.8++],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8+A1.8++]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8+A1.8++]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8+A1.8++]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8+A1.8++]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.8],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.8],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.8],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.8],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.8++],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.8++],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.8++],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.8++],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8++]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8++]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8++]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8++]
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 [A1.8--],T0
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 [A1.8--],T0
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 [A1.8--],T0
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 [A1.8--],T0
        DLMT      ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8--]
        DLMOT     ADDS      D0Re0,D0Re0,D0Re0 T0,[A1.8--]
        DLMT      ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8--]
        DLMOT     ADDS      D0Re0,D0Re0,AC0.0 T0,[A1.8--]
        DLM       ADDS      D1Re0,D1Re0,D1Re0
        DLMO      ADDS      D1Re0,D1Re0,D1Re0
        D         MOVS      D1Re0,D1.7
        DO        ADDS      D1Re0,D1Re0,D1.7
        DLMO      ADDS      D1Re0,D1Re0,D1.7
        DO        ADDS      D1Re0,D1Re0,D0.7
        DLM       MOVS      D1Re0,RD
        D         MOVS      D1Re0,AC1.0
        DO        MOVS      D1Re0,AC1.0
        DM        ADDS      D1Re0,D1Re0,A0FrP
        DL        MOVS      D1Re0,[D1BR.1]
        DLO       MOVS      D1Re0,[D1BR.1]
        DL        MOVS      D1Re0,[D1BR.1++]
        DLO       MOVS      D1Re0,[D1BR.1++]
        DM        MOVS      D1Re0,[D1BR.1+D1BRI.1++]
        DMO       ADDS      D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DM        ADDS      D1Re0,D1.7,D1Re0
        DM        ADDS      D1Re0,D1.7,A1LbP
        DLM       ADDS      D1Re0,D1.7,D1.7
        DM        ADDS      D1Re0,D1.7,D0Re0
        DM        ADDS      D1Re0,D1.7,RD
        DO        ADDS      D1Re0,D1.7,AC1.0
        D         ADDS      D1Re0,D1.7,[D1BR.1]
        D         ADDS      D1Re0,D1.7,[D1BR.1++]
        D         ADDS      D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DMO       ADDS      D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DO        ADDS      D1Re0,[D1BR.1],D1Re0
        D         ADDS      D1Re0,[D1BR.1],D1.7
        DMO       ADDS      D1Re0,[D1BR.1],D1.7
        D         ADDS      D1Re0,[D1BR.1],D0.7
        DMO       ADDS      D1Re0,[D1BR.1],RD
        DLMO      ADDS      D1Re0,[D1BR.1],AC1.0
        DLM       ADDS      D1Re0,[D1BR.1],[D1BR.1]
        DLM       ADDS      D1Re0,[D1BR.1],[D1BR.1++]
        DM        ADDS      D1Re0,[D1BR.1],A0.7
        DLMO      ADDS      D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLMO      ADDS      D1Re0,[D1BR.1++],D1Re0
        DM        ADDS      D1Re0,[D1BR.1++],A1.7
        DLMO      ADDS      D1Re0,[D1BR.1++],D1.7
        DMO       ADDS      D1Re0,[D1BR.1++],D0.7
        DL        ADDS      D1Re0,[D1BR.1++],AC1.0
        DO        ADDS      D1Re0,[D1BR.1++],A0FrP
        DLO       ADDS      D1Re0,[D1BR.1++],[D1BR.1]
        DLO       ADDS      D1Re0,[D1BR.1++],[D1BR.1++]
        DO        ADDS      D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        ADDS      D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DO        ADDS      D1Re0,[D1BR.1+D1BRI.1++],A1LbP
        DO        ADDS      D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DO        ADDS      D1Re0,[D1BR.1+D1BRI.1++],D0Re0
        DLM       ADDS      D1Re0,[D1BR.1+D1BRI.1++],RD
        DMO       ADDS      D1Re0,[D1BR.1+D1BRI.1++],AC1.0
        DM        ADDS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DM        ADDS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ADDS      D1Re0,[D1BR.1+D1BRI.1++],A0.7
        DMO       ADDS      D1Re0,[D1BR.1+D1BRI.1++],A0.7
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T0
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T0
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 [A0FrP+A0FrP++],T0
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 [A0FrP+A0FrP++],T0
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0FrP++]
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0FrP++]
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0FrP++]
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0FrP++]
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T0
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T0
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 [A0FrP+A0.7++],T0
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 [A0FrP+A0.7++],T0
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0.7++]
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0.7++]
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0.7++]
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0.7++]
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T0
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T0
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 [A0FrP+A0.8++],T0
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 [A0FrP+A0.8++],T0
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0.8++]
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP+A0.8++]
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0.8++]
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP+A0.8++]
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 [A0FrP],T0
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 [A0FrP],T0
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 [A0FrP],T0
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 [A0FrP],T0
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP]
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP]
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP]
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP]
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 [A0FrP++],T0
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 [A0FrP++],T0
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 [A0FrP++],T0
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 [A0FrP++],T0
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP++]
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP++]
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP++]
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP++]
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 [A0FrP--],T0
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 [A0FrP--],T0
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 [A0FrP--],T0
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 [A0FrP--],T0
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP--]
        DLOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0FrP--]
        DLWT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP--]
        DLOWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0FrP--]
        DL        MOVS      D1.7,D1Re0
        DLO       MOVS      D1.7,D1Re0
        D         ADDS      D1.7,D1Re0,A1.7
        DLM       MOVS      D1.7,D1.7
        DLO       MOVS      D1.7,D1.7
        DM        ADDS      D1.7,D1Re0,D0.7
        DLM       ADDS      D1.7,D1Re0,RD
        D         ADDS      D1.7,D1Re0,AC1.0
        DO        ADDS      D1.7,D1Re0,AC1.0
        D         ADDS      D1.7,D1Re0,A0FrP
        DL        ADDS      D1.7,D1Re0,[D1BR.1]
        DLO       ADDS      D1.7,D1Re0,[D1BR.1]
        DL        ADDS      D1.7,D1Re0,[D1BR.1++]
        DLO       ADDS      D1.7,D1Re0,[D1BR.1++]
        DM        ADDS      D1.7,D1Re0,A0.7
        DO        MOVS      D1.7,[D1BR.1+D1BRI.1++]
        DT        ADDS      D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T0
        DOT       ADDS      D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T0
        DT        ADDS      D1Re0,D1Re0,AC1.0 [A0.7+A0FrP++],T0
        DOT       ADDS      D1Re0,D1Re0,AC1.0 [A0.7+A0FrP++],T0
        DT        ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7+A0FrP++]
        DOT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7+A0FrP++]
        DT        ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7+A0FrP++]
        DOT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7+A0FrP++]
        D         ADDS      D1.7,D1.7,D1Re0
        DLWT      ADDS      D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T0
        DMOT      ADDS      D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T0
        DT        ADDS      D1Re0,D1Re0,AC1.0 [A0.7+A0.7++],T0
        DOT       ADDS      D1Re0,D1Re0,AC1.0 [A0.7+A0.7++],T0
        D         ADDS      D1.7,D1.7,A1LbP
        DL        ADDS      D1.7,D1.7,D1.7
        D         ADDS      D1.7,D1.7,D0Re0
        D         ADDS      D1.7,D1.7,RD
        DLM       ADDS      D1.7,D1.7,RD
        DMOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7+A0.7++]
        DM        ADDS      D1.7,D1.7,AC1.0
        DLMWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7+A0.7++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7+A0.7++]
        DMO       ADDS      D1.7,D1.7,A0FrP
        DLMO      ADDS      D1.7,D1.7,[D1BR.1]
        DLMO      ADDS      D1.7,D1.7,[D1BR.1++]
        DO        ADDS      D1.7,D1.7,A0.7
        DMWT      ADDS      D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T0
        DMOWT     ADDS      D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T0
        DMWT      ADDS      D1Re0,D1Re0,AC1.0 [A0.7+A0.8++],T0
        DMOWT     ADDS      D1Re0,D1Re0,AC1.0 [A0.7+A0.8++],T0
        DMWT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7+A0.8++]
        DMOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7+A0.8++]
        DMWT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7+A0.8++]
        DMOWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7+A0.8++]
        DMWT      ADDS      D1Re0,D1Re0,D1Re0 [A0.7],T0
        DMOWT     ADDS      D1Re0,D1Re0,D1Re0 [A0.7],T0
        DMWT      ADDS      D1Re0,D1Re0,AC1.0 [A0.7],T0
        DMOWT     ADDS      D1Re0,D1Re0,AC1.0 [A0.7],T0
        DMWT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7]
        DMOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7]
        DMWT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7]
        DMOWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7]
        DMWT      ADDS      D1Re0,D1Re0,D1Re0 [A0.7++],T0
        DMOWT     ADDS      D1Re0,D1Re0,D1Re0 [A0.7++],T0
        DMWT      ADDS      D1Re0,D1Re0,AC1.0 [A0.7++],T0
        DMOWT     ADDS      D1Re0,D1Re0,AC1.0 [A0.7++],T0
        DMWT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7++]
        DMOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7++]
        DMWT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7++]
        DMOWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7++]
        DMWT      ADDS      D1Re0,D1Re0,D1Re0 [A0.7--],T0
        DMOWT     ADDS      D1Re0,D1Re0,D1Re0 [A0.7--],T0
        DMWT      ADDS      D1Re0,D1Re0,AC1.0 [A0.7--],T0
        DMOWT     ADDS      D1Re0,D1Re0,AC1.0 [A0.7--],T0
        DMWT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7--]
        DMOWT     ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.7--]
        DMWT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7--]
        DMOWT     ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.7--]
        DLM       ADDS      D1.7,[D1BR.1],D1Re0
        DMO       ADDS      D1.7,[D1BR.1],A1LbP
        DO        ADDS      D1.7,[D1BR.1],A1.7
        DMO       ADDS      D1.7,[D1BR.1],D0Re0
        DO        ADDS      D1.7,[D1BR.1],RD
        DLO       ADDS      D1.7,[D1BR.1],AC1.0
        DL        ADDS      D1.7,[D1BR.1],[D1BR.1]
        DL        ADDS      D1.7,[D1BR.1],[D1BR.1++]
        DM        ADDS      D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLO       ADDS      D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLO       ADDS      D1.7,[D1BR.1++],D1Re0
        DM        ADDS      D1.7,[D1BR.1++],D1.7
        DLO       ADDS      D1.7,[D1BR.1++],D1.7
        DO        ADDS      D1.7,[D1BR.1++],D0.7
        DM        ADDS      D1.7,[D1BR.1++],AC1.0
        DM        ADDS      D1.7,[D1BR.1++],A0FrP
        DMO       ADDS      D1.7,[D1BR.1++],[D1BR.1]
        DMO       ADDS      D1.7,[D1BR.1++],[D1BR.1++]
        DLM       ADDS      D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DM        ADDS      D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DM        ADDS      D1.7,[D1BR.1+D1BRI.1++],A1LbP
        DLM       ADDS      D1.7,[D1BR.1+D1BRI.1++],D1.7
        DM        ADDS      D1.7,[D1BR.1+D1BRI.1++],D0Re0
        DM        ADDS      D1.7,[D1BR.1+D1BRI.1++],RD
        DO        ADDS      D1.7,[D1BR.1+D1BRI.1++],AC1.0
        D         ADDS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ADDS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ADDS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DMO       ADDS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A0.8+A0FrP++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A0.8+A0FrP++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8+A0FrP++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8+A0FrP++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8+A0FrP++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8+A0FrP++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A0.8+A0.7++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A0.8+A0.7++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8+A0.7++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8+A0.7++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8+A0.7++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8+A0.7++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A0.8+A0.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A0.8+A0.8++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8+A0.8++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8+A0.8++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8+A0.8++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8+A0.8++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A0.8],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A0.8],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A0.8],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A0.8],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A0.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A0.8++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A0.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A0.8++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A0.8--],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A0.8--],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A0.8--],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A0.8--],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8--]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A0.8--]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8--]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A0.8--]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1LbP+A1LbP++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1LbP+A1LbP++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1LbP++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1LbP++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1LbP++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1LbP++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1LbP+A1.7++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1LbP+A1.7++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1.7++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1.7++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1.7++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1.7++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1LbP+A1.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1LbP+A1.8++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1.8++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP+A1.8++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1.8++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP+A1.8++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1LbP],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1LbP],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1LbP],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1LbP],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1LbP++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1LbP++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1LbP++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1LbP++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1LbP--],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1LbP--],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1LbP--],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1LbP--],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP--]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1LbP--]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP--]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1LbP--]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.7+A1LbP++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.7+A1LbP++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7+A1LbP++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7+A1LbP++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7+A1LbP++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7+A1LbP++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.7+A1.7++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.7+A1.7++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7+A1.7++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7+A1.7++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7+A1.7++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7+A1.7++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.7+A1.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.7+A1.8++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7+A1.8++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7+A1.8++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7+A1.8++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7+A1.8++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.7],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.7],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.7],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.7],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.7++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.7++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.7++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.7++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.7--],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.7--],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.7--],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.7--],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7--]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.7--]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7--]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.7--]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.8+A1LbP++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.8+A1LbP++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8+A1LbP++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8+A1LbP++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8+A1LbP++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8+A1LbP++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.8+A1.7++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.8+A1.7++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8+A1.7++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8+A1.7++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8+A1.7++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8+A1.7++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.8+A1.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.8+A1.8++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8+A1.8++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8+A1.8++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8+A1.8++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8+A1.8++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.8],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.8],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.8],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.8],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.8++],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.8++],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.8++],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8++]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8++]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8++]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [A1.8--],T0
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 [A1.8--],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [A1.8--],T0
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 [A1.8--],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8--]
        DLOT      ADDS      D1Re0,D1Re0,D1Re0 T0,[A1.8--]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8--]
        DLOT      ADDS      D1Re0,D1Re0,AC1.0 T0,[A1.8--]
        DL        ADDS      [D1BW.1],D1Re0,D1Re0
        DLO       ADDS      [D1BW.1],D1Re0,D1Re0
        D         ADDS      [D1BW.1],D1Re0,D1.7
        DLM       ADDS      [D1BW.1],D1Re0,D1.7
        DLO       ADDS      [D1BW.1],D1Re0,D1.7
        D         ADDS      [D1BW.1],D1Re0,D0.7
        DL        MOVS      [D1BW.1],RD
        DLMO      MOVS      [D1BW.1],RD
        DLM       MOVS      [D1BW.1],AC1.0
        DLMO      MOVS      [D1BW.1],AC1.0
        DM        MOVS      [D1BW.1],[D1BR.1]
        DMO       MOVS      [D1BW.1],[D1BR.1]
        DM        MOVS      [D1BW.1],[D1BR.1++]
        DMO       MOVS      [D1BW.1],[D1BR.1++]
        DM        ADDS      [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DO        ADDS      [D1BW.1],D1Re0,A0.7
        DLMO      MOVS      [D1BW.1],[D1BR.1+D1BRI.1++]
        DLMO      ADDS      [D1BW.1],D1.7,D1Re0
        DM        ADDS      [D1BW.1],D1.7,A1.7
        DLMO      ADDS      [D1BW.1],D1.7,D1.7
        DMO       ADDS      [D1BW.1],D1.7,D0.7
        DL        ADDS      [D1BW.1],D1.7,AC1.0
        DO        ADDS      [D1BW.1],D1.7,A0FrP
        DLO       ADDS      [D1BW.1],D1.7,[D1BR.1]
        DLO       ADDS      [D1BW.1],D1.7,[D1BR.1++]
        DO        ADDS      [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DLT       ADDS      D1Re0,D1Re0,D1Re0 [D1BW.1],T0
        DLT       ADDS      D1Re0,D1Re0,AC1.0 [D1BW.1],T0
        DLT       ADDS      D1Re0,D1Re0,D1Re0 T0,[D1BR.1]
        DLT       ADDS      D1Re0,D1Re0,AC1.0 T0,[D1BR.1]
        DL        ADDS      [D1BW.1],[D1BR.1],D1Re0
        DO        ADDS      [D1BW.1],[D1BR.1],A1LbP
        DO        ADDS      [D1BW.1],[D1BR.1],D1.7
        DO        ADDS      [D1BW.1],[D1BR.1],D0Re0
        DLM       ADDS      [D1BW.1],[D1BR.1],RD
        DMO       ADDS      [D1BW.1],[D1BR.1],AC1.0
        DM        ADDS      [D1BW.1],[D1BR.1],[D1BR.1]
        DM        ADDS      [D1BW.1],[D1BR.1],[D1BR.1++]
        D         ADDS      [D1BW.1],[D1BR.1],A0.7
        DMO       ADDS      [D1BW.1],[D1BR.1],A0.7
        DMO       ADDS      [D1BW.1],[D1BR.1++],D1Re0
        D         ADDS      [D1BW.1],[D1BR.1++],A1.7
        DMO       ADDS      [D1BW.1],[D1BR.1++],A1.7
        DM        ADDS      [D1BW.1],[D1BR.1++],D0.7
        DMO       ADDS      [D1BW.1],[D1BR.1++],RD
        DLMO      ADDS      [D1BW.1],[D1BR.1++],AC1.0
        DLM       ADDS      [D1BW.1],[D1BR.1++],[D1BR.1]
        DLM       ADDS      [D1BW.1],[D1BR.1++],[D1BR.1++]
        DM        ADDS      [D1BW.1],[D1BR.1++],A0.7
        DLMO      ADDS      [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLMO      ADDS      [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DM        ADDS      [D1BW.1],[D1BR.1+D1BRI.1++],A1.7
        DLMO      ADDS      [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DMO       ADDS      [D1BW.1],[D1BR.1+D1BRI.1++],D0.7
        DLM       ADDS      [D1BW.1],[D1BR.1+D1BRI.1++],AC1.0
        DMO       ADDS      [D1BW.1],[D1BR.1+D1BRI.1++],A0FrP
        DLMO      ADDS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLMO      ADDS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DO        ADDS      [D1BW.1],[D1BR.1+D1BRI.1++],A0.7
        DM        MOVS      [D1BW.1++],D1Re0
        DMO       MOVS      [D1BW.1++],D1Re0
        DMO       ADDS      [D1BW.1++],D1Re0,A1LbP
        DL        MOVS      [D1BW.1++],D1.7
        DMO       MOVS      [D1BW.1++],D1.7
        DMO       ADDS      [D1BW.1++],D1Re0,D0Re0
        DM        MOVS      [D1BW.1++],RD
        DLMO      MOVS      [D1BW.1++],RD
        DLM       MOVS      [D1BW.1++],AC1.0
        DLMO      MOVS      [D1BW.1++],AC1.0
        DM        MOVS      [D1BW.1++],[D1BR.1]
        DMO       MOVS      [D1BW.1++],[D1BR.1]
        DM        MOVS      [D1BW.1++],[D1BR.1++]
        DMO       MOVS      [D1BW.1++],[D1BR.1++]
        DM        ADDS      [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DO        ADDS      [D1BW.1++],D1Re0,A0.7
        DLMO      MOVS      [D1BW.1++],[D1BR.1+D1BRI.1++]
        DLMO      ADDS      [D1BW.1++],D1.7,D1Re0
        DM        ADDS      [D1BW.1++],D1.7,A1.7
        DLMO      ADDS      [D1BW.1++],D1.7,D1.7
        DMO       ADDS      [D1BW.1++],D1.7,D0.7
        DLM       ADDS      [D1BW.1++],D1.7,AC1.0
        DMO       ADDS      [D1BW.1++],D1.7,A0FrP
        DLMO      ADDS      [D1BW.1++],D1.7,[D1BR.1]
        DLMO      ADDS      [D1BW.1++],D1.7,[D1BR.1++]
        DO        ADDS      [D1BW.1++],D1.7,A0.7
        DLMT      ADDS      D1Re0,D1Re0,D1Re0 [D1BW.1++],T0
        DLMT      ADDS      D1Re0,D1Re0,AC1.0 [D1BW.1++],T0
        DLMT      ADDS      D1Re0,D1Re0,D1Re0 T0,[D1BR.1++]
        DLMT      ADDS      D1Re0,D1Re0,AC1.0 T0,[D1BR.1++]
        DLM       ADDS      [D1BW.1++],[D1BR.1],D1Re0
        DMO       ADDS      [D1BW.1++],[D1BR.1],A1LbP
        DO        ADDS      [D1BW.1++],[D1BR.1],A1.7
        DMO       ADDS      [D1BW.1++],[D1BR.1],D0Re0
        DMO       ADDS      [D1BW.1++],[D1BR.1],RD
        DLMO      ADDS      [D1BW.1++],[D1BR.1],AC1.0
        DLM       ADDS      [D1BW.1++],[D1BR.1],[D1BR.1]
        DLM       ADDS      [D1BW.1++],[D1BR.1],[D1BR.1++]
        DM        ADDS      [D1BW.1++],[D1BR.1],A0.7
        DLMO      ADDS      [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLMO      ADDS      [D1BW.1++],[D1BR.1++],D1Re0
        DM        ADDS      [D1BW.1++],[D1BR.1++],A1.7
        DLMO      ADDS      [D1BW.1++],[D1BR.1++],D1.7
        DMO       ADDS      [D1BW.1++],[D1BR.1++],D0.7
        DLM       ADDS      [D1BW.1++],[D1BR.1++],AC1.0
        DMO       ADDS      [D1BW.1++],[D1BR.1++],A0FrP
        DLMO      ADDS      [D1BW.1++],[D1BR.1++],[D1BR.1]
        DLMO      ADDS      [D1BW.1++],[D1BR.1++],[D1BR.1++]
        DO        ADDS      [D1BW.1++],[D1BR.1++],A0.7
        DLM       ADDS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        DMO       ADDS      [D1BW.1++],[D1BR.1+D1BRI.1++],A1LbP
        DO        ADDS      [D1BW.1++],[D1BR.1+D1BRI.1++],A1.7
        DMO       ADDS      [D1BW.1++],[D1BR.1+D1BRI.1++],D0Re0
        DMO       ADDS      [D1BW.1++],[D1BR.1+D1BRI.1++],RD
        DLMO      ADDS      [D1BW.1++],[D1BR.1+D1BRI.1++],AC1.0
        DLM       ADDS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLM       ADDS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DM        ADDS      [D1BW.1++],[D1BR.1+D1BRI.1++],A0.7
        DLMO      ADDS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLM       MOVS      [D1BW.1+D1BWI.1++],D1Re0
        DLMO      MOVS      [D1BW.1+D1BWI.1++],D1Re0
        DM        ADDS      [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DO        ADDS      [D1BW.1+D1BWI.1++],D1Re0,A1.7
        DLMO      MOVS      [D1BW.1+D1BWI.1++],D1.7
        DMO       ADDS      [D1BW.1+D1BWI.1++],D1Re0,D0.7
        DO        MOVS      [D1BW.1+D1BWI.1++],RD
        DM        MOVS      [D1BW.1+D1BWI.1++],AC1.0
        DMO       MOVS      [D1BW.1+D1BWI.1++],AC1.0
        DMO       ADDS      [D1BW.1+D1BWI.1++],D1Re0,A0FrP
        DLM       MOVS      [D1BW.1+D1BWI.1++],[D1BR.1]
        DLMO      MOVS      [D1BW.1+D1BWI.1++],[D1BR.1]
        DLM       MOVS      [D1BW.1+D1BWI.1++],[D1BR.1++]
        DLMO      MOVS      [D1BW.1+D1BWI.1++],[D1BR.1++]
        DL        MOVS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++]
        DMO       MOVS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++]
        DLM       ADDS      [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DMO       ADDS      [D1BW.1+D1BWI.1++],D1.7,A1LbP
        DO        ADDS      [D1BW.1+D1BWI.1++],D1.7,A1.7
        DMO       ADDS      [D1BW.1+D1BWI.1++],D1.7,D0Re0
        DMO       ADDS      [D1BW.1+D1BWI.1++],D1.7,RD
        DLMO      ADDS      [D1BW.1+D1BWI.1++],D1.7,AC1.0
        DLM       ADDS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        DLM       ADDS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        DM        ADDS      [D1BW.1+D1BWI.1++],D1.7,A0.7
        DLMO      ADDS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DLMOT     ADDS      D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T0
        DLMOT     ADDS      D1Re0,D1Re0,AC1.0 [D1BW.1+D1BWI.1++],T0
        DLMOT     ADDS      D1Re0,D1Re0,D1Re0 T0,[D1BR.1+D1BRI.1++]
        DLMOT     ADDS      D1Re0,D1Re0,AC1.0 T0,[D1BR.1+D1BRI.1++]
        DLMO      ADDS      [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        DM        ADDS      [D1BW.1+D1BWI.1++],[D1BR.1],A1.7
        DLMO      ADDS      [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        DMO       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1],D0.7
        DLM       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1],AC1.0
        DMO       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1],A0FrP
        DLMO      ADDS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        DLMO      ADDS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        DO        ADDS      [D1BW.1+D1BWI.1++],[D1BR.1],A0.7
        DLM       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        DMO       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1++],A1LbP
        DO        ADDS      [D1BW.1+D1BWI.1++],[D1BR.1++],A1.7
        DMO       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1++],D0Re0
        DMO       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1++],RD
        DLMO      ADDS      [D1BW.1+D1BWI.1++],[D1BR.1++],AC1.0
        DLM       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        DLM       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        DM        ADDS      [D1BW.1+D1BWI.1++],[D1BR.1++],A0.7
        DLMO      ADDS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLMO      ADDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        DM        ADDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1.7
        DLMO      ADDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        DMO       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D0.7
        DLM       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],AC1.0
        DMO       ADDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0FrP
        DLMO      ADDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLMO      ADDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DO        ADDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0.7
        DL        ADDS      D0Re0,D0Re0,#-0x8000
        DL        ADDS      D0.7,D0.7,#-0x1
        DL        ADDS      [D0AW.0++],[D0AR.0++],#0x7fff
        DL        ADDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#-0x8000
        DL        ADDS      D1Re0,D1Re0,#-0x1
        DL        ADDS      [D1BW.1],[D1BR.1],#0x7fff
        DL        ADDS      [D1BW.1++],[D1BR.1++],#-0x8000
        DL        ADDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#-0x1
        D         ADDSNV    D0Re0,D0Re0,A1.7
        D         ADDSNV    D0Re0,D0Re0,[D0AR.0++]
        D         ADDSNV    D0Re0,D0.7,D0Re0
        D         ADDSNV    D0Re0,D0.7,D1Re0
        D         ADDSNV    D0Re0,D0.7,[D0AR.0++]
        D         ADDSNV    D0Re0,[D0AR.0],A1LbP
        D         ADDSNV    D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ADDSNV    D0Re0,[D0AR.0++],D1.7
        D         ADDSNV    D0Re0,[D0AR.0+D0ARI.0++],A1LbP
        D         ADDSNV    D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DECS      ADDSLE    D0.7,D0Re0,A1.7
        DHNE      ADDSPL    D0.7,D0Re0,RD
        D         ADDSNV    D0.7,D0Re0,A0FrP
        D         ADDSNV    D0.7,D0.7,D0.7
        DHNE      ADDSPL    D0.7,D0.7,[D0AR.0++]
        D         ADDSNV    D0.7,[D0AR.0],D0Re0
        D         ADDSNV    D0.7,[D0AR.0],[D0AR.0++]
        D         ADDSNV    D0.7,[D0AR.0++],D1Re0
        D         ADDSNV    D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         ADDSNV    D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DECS      ADDSLE    D0.16,D0Re0,A1.7
        D         ADDSNV    D0.16,D0Re0,RD
        D         ADDSNV    D0.16,D0Re0,A0.7
        D         ADDSNV    D0.16,D0.7,A1.7
        D         ADDSNV    D0.16,D0.7,RD
        D         ADDSNV    D0.16,D0.7,A0.7
        D         ADDSNV    D0.16,[D0AR.0],RD
        D         ADDSNV    D0.16,[D0AR.0++],A1.7
        D         ADDSNV    D0.16,[D0AR.0++],A0.7
        D         ADDSNV    D0.16,[D0AR.0+D0ARI.0++],RD
        DHNE      ADDSPL    D1Re0,D1Re0,D1.7
        DECS      ADDSLE    D1Re0,D1Re0,A0FrP
        D         ADDSNV    D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        D         ADDSNV    D1Re0,D1.7,D1.7
        DECS      ADDSLE    D1Re0,D1.7,A0FrP
        D         ADDSNV    D1Re0,D1.7,A0.7
        D         ADDSNV    D1Re0,[D1BR.1],A0FrP
        D         ADDSNV    D1Re0,[D1BR.1++],A1.7
        D         ADDSNV    D1Re0,[D1BR.1++],A0.7
        D         ADDSNV    D1Re0,[D1BR.1+D1BRI.1++],A0FrP
        D         ADDSNV    D1.7,D1Re0,A1LbP
        D         ADDSNV    D1.7,D1Re0,RD
        D         ADDSNV    D1.7,D1Re0,A0.7
        D         ADDSNV    D1.7,D1.7,D0.7
        DHNE      ADDSPL    D1.7,D1.7,[D1BR.1+D1BRI.1++]
        D         ADDSNV    D1.7,[D1BR.1],D0.7
        D         ADDSNV    D1.7,[D1BR.1++],A1LbP
        D         ADDSNV    D1.7,[D1BR.1++],[D1BR.1++]
        D         ADDSNV    D1.7,[D1BR.1+D1BRI.1++],D0.7
        D         ADDSNV    D1.16,D1Re0,D1Re0
        D         ADDSNV    D1.16,D1Re0,RD
        D         ADDSNV    D1.16,D1Re0,A0.7
        DHNE      ADDSPL    D1.16,D1.7,D0Re0
        D         ADDSNV    D1.16,D1.7,[D1BR.1]
        D         ADDSNV    D1.16,[D1BR.1],D1.7
        D         ADDSNV    D1.16,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ADDSNV    D1.16,[D1BR.1++],RD
        D         ADDSNV    D1.16,[D1BR.1+D1BRI.1++],D1.7
        D         ADDSNV    D1.16,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLM       SUB       D0Re0,D0Re0,D0Re0
        DLMO      SUB       D0Re0,D0Re0,D0Re0
        DO        SUB       D0Re0,D0Re0,A1LbP
        D         NEG       D0Re0,D0.7
        DL        NEG       D0Re0,A1.7
        DMO       SUB       D0Re0,D0Re0,D0.7
        DLMO      NEG       D0Re0,D0.7
        DO        SUB       D0Re0,D0Re0,D1Re0
        DM        SUB       D0Re0,D0Re0,D1.7
        DLO       NEG       D0Re0,D1.7
        DM        NEG       D0Re0,[D0AR.0]
        DO        SUB       D0Re0,D0Re0,[D0AR.0]
        DLO       SUB       D0Re0,D0Re0,[D0AR.0]
        DM        SUB       D0Re0,D0Re0,AC0.0
        DMO       SUB       D0Re0,D0Re0,AC0.0
        DM        SUB       D0Re0,D0Re0,[D0AR.0++]
        DMO       SUB       D0Re0,D0Re0,[D0AR.0++]
        DM        SUB       D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DMO       SUB       D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DM        SUB       D0Re0,D0Re0,A0FrP
        DLO       NEG       D0Re0,A0FrP
        DO        SUB       D0Re0,D0Re0,A0.7
        DL        SUB       D0Re0,D0.7,D0Re0
        DO        SUB       D0Re0,D0.7,A1LbP
        DO        SUB       D0Re0,D0.7,D0.7
        DO        SUB       D0Re0,D0.7,D1Re0
        DM        SUB       D0Re0,D0.7,[D0AR.0]
        DLO       SUB       D0Re0,D0.7,[D0AR.0]
        DLO       SUB       D0Re0,D0.7,AC0.0
        DLO       SUB       D0Re0,D0.7,[D0AR.0++]
        DLO       SUB       D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DO        SUB       D0Re0,D0.7,A0.7
        DLO       SUB       D0Re0,[D0AR.0],D0Re0
        DM        SUB       D0Re0,[D0AR.0],D0.7
        DLO       SUB       D0Re0,[D0AR.0],D0.7
        DO        SUB       D0Re0,[D0AR.0],D1.7
        DO        SUB       D0Re0,[D0AR.0],[D0AR.0]
        DL        SUB       D0Re0,[D0AR.0],AC0.0
        DL        SUB       D0Re0,[D0AR.0],[D0AR.0++]
        DL        SUB       D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        DO        SUB       D0Re0,[D0AR.0],A0FrP
        DL        SUB       D0Re0,[D0AR.0++],D0Re0
        DO        SUB       D0Re0,[D0AR.0++],A1LbP
        DO        SUB       D0Re0,[D0AR.0++],D0.7
        DO        SUB       D0Re0,[D0AR.0++],D1Re0
        DM        SUB       D0Re0,[D0AR.0++],[D0AR.0]
        DLO       SUB       D0Re0,[D0AR.0++],[D0AR.0]
        DLO       SUB       D0Re0,[D0AR.0++],AC0.0
        DLO       SUB       D0Re0,[D0AR.0++],[D0AR.0++]
        DLO       SUB       D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DO        SUB       D0Re0,[D0AR.0++],A0.7
        DLO       SUB       D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        DM        SUB       D0Re0,[D0AR.0+D0ARI.0++],D0.7
        DLO       SUB       D0Re0,[D0AR.0+D0ARI.0++],D0.7
        DO        SUB       D0Re0,[D0AR.0+D0ARI.0++],D1.7
        DO        SUB       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        SUB       D0Re0,[D0AR.0+D0ARI.0++],AC0.0
        DL        SUB       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DL        SUB       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DO        SUB       D0Re0,[D0AR.0+D0ARI.0++],A0FrP
        DMT       SUB       D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T1
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [A0FrP+A0FrP++],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [A0FrP+A0FrP++],T1
        DT        SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0FrP++]
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0FrP++]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[A0FrP+A0FrP++]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0FrP++]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0FrP++]
        DLMT      SUB       D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T1
        DLMOT     SUB       D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T1
        DLT       SUB       D0Re0,D0Re0,AC0.0 [A0FrP+A0.7++],T1
        DLOT      SUB       D0Re0,D0Re0,AC0.0 [A0FrP+A0.7++],T1
        DLT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0.7++]
        DLOT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0.7++]
        DMT       SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0.7++]
        DMOT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0.7++]
        DMT       SUB       D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T1
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [A0FrP+A0.8++],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [A0FrP+A0.8++],T1
        DT        SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0.8++]
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0.8++]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[A0FrP+A0.8++]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0.8++]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0.8++]
        DLMT      SUB       D0Re0,D0Re0,D0Re0 [A0FrP],T1
        DLMOT     SUB       D0Re0,D0Re0,D0Re0 [A0FrP],T1
        DLT       SUB       D0Re0,D0Re0,AC0.0 [A0FrP],T1
        DLOT      SUB       D0Re0,D0Re0,AC0.0 [A0FrP],T1
        DLT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP]
        DLOT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP]
        DMT       SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP]
        DMOT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP]
        DMT       SUB       D0Re0,D0Re0,D0Re0 [A0FrP++],T1
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [A0FrP++],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [A0FrP++],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [A0FrP++],T1
        DT        SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP++]
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP++]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[A0FrP++]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP++]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP++]
        DLMT      SUB       D0Re0,D0Re0,D0Re0 [A0FrP--],T1
        DLMOT     SUB       D0Re0,D0Re0,D0Re0 [A0FrP--],T1
        DLT       SUB       D0Re0,D0Re0,AC0.0 [A0FrP--],T1
        DLOT      SUB       D0Re0,D0Re0,AC0.0 [A0FrP--],T1
        DLT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP--]
        DLOT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0FrP--]
        DMT       SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP--]
        DMOT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0FrP--]
        DM        SUB       D0.7,D0Re0,D0Re0
        DMO       SUB       D0.7,D0Re0,D0Re0
        DM        SUB       D0.7,D0Re0,A1LbP
        DLO       NEG       D0.7,A1LbP
        DM        NEG       D0.7,D0.7
        DO        SUB       D0.7,D0Re0,D0.7
        DLO       SUB       D0.7,D0Re0,D0.7
        DM        SUB       D0.7,D0Re0,D1Re0
        DLO       NEG       D0.7,D1Re0
        DO        SUB       D0.7,D0Re0,D1.7
        D         NEG       D0.7,[D0AR.0]
        DL        NEG       D0.7,RD
        DMO       SUB       D0.7,D0Re0,[D0AR.0]
        DLMO      NEG       D0.7,[D0AR.0]
        DLM       SUB       D0.7,D0Re0,AC0.0
        DLMO      SUB       D0.7,D0Re0,AC0.0
        DLM       SUB       D0.7,D0Re0,[D0AR.0++]
        DLMO      SUB       D0.7,D0Re0,[D0AR.0++]
        DLM       SUB       D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DLMO      SUB       D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DO        SUB       D0.7,D0Re0,A0FrP
        DM        SUB       D0.7,D0Re0,A0.7
        DLO       NEG       D0.7,A0.7
        DLMT      SUB       D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T1
        DLMOT     SUB       D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T1
        DLT       SUB       D0Re0,D0Re0,AC0.0 [A0.7+A0FrP++],T1
        DLOT      SUB       D0Re0,D0Re0,AC0.0 [A0.7+A0FrP++],T1
        DLT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7+A0FrP++]
        DLOT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7+A0FrP++]
        DMT       SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7+A0FrP++]
        DMOT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7+A0FrP++]
        DL        SUB       D0.7,D0.7,D0Re0
        DWT       SUB       D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T1
        DO        SUB       D0.7,D0.7,A1LbP
        DLMOT     SUB       D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T1
        DLT       SUB       D0Re0,D0Re0,AC0.0 [A0.7+A0.7++],T1
        DLOT      SUB       D0Re0,D0Re0,AC0.0 [A0.7+A0.7++],T1
        DL        SUB       D0.7,D0.7,D0.7
        D         SUB       D0.7,D0.7,D1Re0
        D         SUB       D0.7,D0.7,[D0AR.0]
        DMO       SUB       D0.7,D0.7,[D0AR.0]
        DO        SUB       D0.7,D0.7,AC0.0
        DMT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7+A0.7++]
        DO        SUB       D0.7,D0.7,[D0AR.0++]
        DLOWT     SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7+A0.7++]
        DMT       SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7+A0.7++]
        DMOT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7+A0.7++]
        DL        SUB       D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DO        SUB       D0.7,D0.7,A0FrP
        DMT       SUB       D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T1
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [A0.7+A0.8++],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [A0.7+A0.8++],T1
        DT        SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7+A0.8++]
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7+A0.8++]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[A0.7+A0.8++]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7+A0.8++]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7+A0.8++]
        DLO       SUB       D0.7,[D0AR.0],D0Re0
        DLT       SUB       D0Re0,D0Re0,D0Re0 [A0.7],T1
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [A0.7],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [A0.7],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [A0.7],T1
        D         SUB       D0.7,[D0AR.0],D0.7
        DMO       SUB       D0.7,[D0AR.0],D0.7
        D         SUB       D0.7,[D0AR.0],D1.7
        DL        SUB       D0.7,[D0AR.0],[D0AR.0]
        D         SUB       D0.7,[D0AR.0],AC0.0
        D         SUB       D0.7,[D0AR.0],[D0AR.0++]
        DLWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7]
        DMOT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[A0.7]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7]
        DLO       SUB       D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DO        SUB       D0.7,[D0AR.0],A0.7
        DLO       SUB       D0.7,[D0AR.0++],D0Re0
        DLT       SUB       D0Re0,D0Re0,D0Re0 [A0.7++],T1
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [A0.7++],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [A0.7++],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [A0.7++],T1
        D         SUB       D0.7,[D0AR.0++],D0.7
        DMO       SUB       D0.7,[D0AR.0++],D0.7
        D         SUB       D0.7,[D0AR.0++],D1.7
        DL        SUB       D0.7,[D0AR.0++],[D0AR.0]
        D         SUB       D0.7,[D0AR.0++],AC0.0
        D         SUB       D0.7,[D0AR.0++],[D0AR.0++]
        DLWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7++]
        DMOT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7++]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[A0.7++]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7++]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7++]
        DLO       SUB       D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DO        SUB       D0.7,[D0AR.0++],A0.7
        DLO       SUB       D0.7,[D0AR.0+D0ARI.0++],D0Re0
        DM        SUB       D0.7,[D0AR.0+D0ARI.0++],D0.7
        DLO       SUB       D0.7,[D0AR.0+D0ARI.0++],D0.7
        DO        SUB       D0.7,[D0AR.0+D0ARI.0++],D1.7
        DO        SUB       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        SUB       D0.7,[D0AR.0+D0ARI.0++],AC0.0
        DL        SUB       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DL        SUB       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DO        SUB       D0.7,[D0AR.0+D0ARI.0++],A0FrP
        DMT       SUB       D0Re0,D0Re0,D0Re0 [A0.7--],T1
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [A0.7--],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [A0.7--],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [A0.7--],T1
        DT        SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7--]
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0.7--]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[A0.7--]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7--]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[A0.7--]
        DLMT      SUB       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T1
        DLMOT     SUB       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T1
        DLT       SUB       D0Re0,D0Re0,AC0.0 [A0.8+A0FrP++],T1
        DLOT      SUB       D0Re0,D0Re0,AC0.0 [A0.8+A0FrP++],T1
        DLT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8+A0FrP++]
        DLOT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8+A0FrP++]
        DMT       SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8+A0FrP++]
        DMOT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8+A0FrP++]
        DMT       SUB       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T1
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [A0.8+A0.7++],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [A0.8+A0.7++],T1
        DT        SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8+A0.7++]
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8+A0.7++]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[A0.8+A0.7++]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8+A0.7++]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8+A0.7++]
        DLMT      SUB       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T1
        DLMOT     SUB       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T1
        DLT       SUB       D0Re0,D0Re0,AC0.0 [A0.8+A0.8++],T1
        DLOT      SUB       D0Re0,D0Re0,AC0.0 [A0.8+A0.8++],T1
        DLT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8+A0.8++]
        DLOT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8+A0.8++]
        DMT       SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8+A0.8++]
        DMOT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8+A0.8++]
        DMT       SUB       D0Re0,D0Re0,D0Re0 [A0.8],T1
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [A0.8],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [A0.8],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [A0.8],T1
        DT        SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8]
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[A0.8]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8]
        DLMT      SUB       D0Re0,D0Re0,D0Re0 [A0.8++],T1
        DLMOT     SUB       D0Re0,D0Re0,D0Re0 [A0.8++],T1
        DLT       SUB       D0Re0,D0Re0,AC0.0 [A0.8++],T1
        DLOT      SUB       D0Re0,D0Re0,AC0.0 [A0.8++],T1
        DLT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8++]
        DLOT      SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8++]
        DMT       SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8++]
        DMOT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8++]
        DMT       SUB       D0Re0,D0Re0,D0Re0 [A0.8--],T1
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [A0.8--],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [A0.8--],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [A0.8--],T1
        DT        SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8--]
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[A0.8--]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[A0.8--]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8--]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[A0.8--]
        DLM       SUB       [D0AW.0],D0Re0,D0Re0
        DLMO      SUB       [D0AW.0],D0Re0,D0Re0
        DLM       NEG       [D0AW.0],A1LbP
        D         SUB       [D0AW.0],D0Re0,A1.7
        DL        NEG       [D0AW.0],D0.7
        DO        NEG       [D0AW.0],A1.7
        DLMO      SUB       [D0AW.0],D0Re0,D0.7
        DLM       NEG       [D0AW.0],D1Re0
        D         NEG       [D0AW.0],D1.7
        DMO       NEG       [D0AW.0],D1.7
        DM        SUB       [D0AW.0],D0Re0,RD
        DLM       NEG       [D0AW.0],RD
        DMO       NEG       [D0AW.0],RD
        D         NEG       [D0AW.0],AC0.0
        DO        NEG       [D0AW.0],AC0.0
        D         NEG       [D0AW.0],[D0AR.0++]
        DO        NEG       [D0AW.0],[D0AR.0++]
        D         NEG       [D0AW.0],[D0AR.0+D0ARI.0++]
        DO        NEG       [D0AW.0],[D0AR.0+D0ARI.0++]
        D         NEG       [D0AW.0],A0FrP
        DMO       NEG       [D0AW.0],A0FrP
        DLM       NEG       [D0AW.0],A0.7
        DM        SUB       [D0AW.0],D0.7,D0Re0
        DM        SUB       [D0AW.0],D0.7,A1LbP
        DLM       SUB       [D0AW.0],D0.7,D0.7
        DM        SUB       [D0AW.0],D0.7,D1Re0
        D         SUB       [D0AW.0],D0.7,RD
        DMO       SUB       [D0AW.0],D0.7,RD
        DMO       SUB       [D0AW.0],D0.7,AC0.0
        DMO       SUB       [D0AW.0],D0.7,[D0AR.0++]
        DMO       SUB       [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        DM        SUB       [D0AW.0],D0.7,A0.7
        DMO       SUB       [D0AW.0],[D0AR.0],D0Re0
        D         SUB       [D0AW.0],[D0AR.0],A1.7
        DMO       SUB       [D0AW.0],[D0AR.0],A1.7
        DM        SUB       [D0AW.0],[D0AR.0],D1.7
        DLM       SUB       [D0AW.0],[D0AR.0],[D0AR.0]
        DM        SUB       [D0AW.0],[D0AR.0],AC0.0
        DM        SUB       [D0AW.0],[D0AR.0],[D0AR.0++]
        DM        SUB       [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DM        SUB       [D0AW.0],[D0AR.0],A0FrP
        DM        SUB       [D0AW.0],[D0AR.0++],D0Re0
        DM        SUB       [D0AW.0],[D0AR.0++],A1LbP
        DLM       SUB       [D0AW.0],[D0AR.0++],D0.7
        DM        SUB       [D0AW.0],[D0AR.0++],D1Re0
        D         SUB       [D0AW.0],[D0AR.0++],RD
        DMO       SUB       [D0AW.0],[D0AR.0++],RD
        DMO       SUB       [D0AW.0],[D0AR.0++],AC0.0
        DMO       SUB       [D0AW.0],[D0AR.0++],[D0AR.0++]
        DMO       SUB       [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DM        SUB       [D0AW.0],[D0AR.0++],A0.7
        DMO       SUB       [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        D         SUB       [D0AW.0],[D0AR.0+D0ARI.0++],A1.7
        DMO       SUB       [D0AW.0],[D0AR.0+D0ARI.0++],A1.7
        DM        SUB       [D0AW.0],[D0AR.0+D0ARI.0++],D1.7
        DLM       SUB       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DM        SUB       [D0AW.0],[D0AR.0+D0ARI.0++],AC0.0
        DM        SUB       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DM        SUB       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DM        SUB       [D0AW.0],[D0AR.0+D0ARI.0++],A0FrP
        DMT       SUB       D0Re0,D0Re0,D0Re0 [D0AW.0],T1
        DT        SUB       D0Re0,D0Re0,AC0.0 [D0AW.0],T1
        DT        SUB       D0Re0,D0Re0,D0Re0 T1,[D0AR.0]
        DT        SUB       AC0.0,AC0.0,AC1.0 T1,[D0AR.0]
        DLMOT     SUB       D0Re0,D0Re0,AC0.0 T1,[D0AR.0]
        DLM       NEG       [D0AW.0++],D0Re0
        DLMO      NEG       [D0AW.0++],D0Re0
        DO        NEG       [D0AW.0++],A1LbP
        D         NEG       [D0AW.0++],A1.7
        DLM       SUB       [D0AW.0++],D0Re0,D0.7
        DMO       SUB       [D0AW.0++],D0Re0,A1.7
        DLMO      NEG       [D0AW.0++],A1.7
        DO        NEG       [D0AW.0++],D1Re0
        DM        NEG       [D0AW.0++],D1.7
        DLMO      NEG       [D0AW.0++],D1.7
        DM        NEG       [D0AW.0++],RD
        DO        SUB       [D0AW.0++],D0Re0,RD
        DLO       NEG       [D0AW.0++],[D0AR.0]
        DM        NEG       [D0AW.0++],AC0.0
        DMO       NEG       [D0AW.0++],AC0.0
        DM        NEG       [D0AW.0++],[D0AR.0++]
        DMO       NEG       [D0AW.0++],[D0AR.0++]
        DM        NEG       [D0AW.0++],[D0AR.0+D0ARI.0++]
        DMO       NEG       [D0AW.0++],[D0AR.0+D0ARI.0++]
        DM        NEG       [D0AW.0++],A0FrP
        DLMO      NEG       [D0AW.0++],A0FrP
        DO        NEG       [D0AW.0++],A0.7
        DMWT      SUB       D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T1
        DMOWT     SUB       D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T1
        DWT       SUB       D0Re0,D0Re0,AC0.0 [A1LbP+A1LbP++],T1
        DOWT      SUB       D0Re0,D0Re0,AC0.0 [A1LbP+A1LbP++],T1
        DWT       SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1LbP++]
        DOWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1LbP++]
        DWT       SUB       AC0.0,AC0.0,AC1.0 T1,[A1LbP+A1LbP++]
        DLMWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1LbP++]
        DLMOWT    SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1LbP++]
        DLMO      SUB       [D0AW.0++],D0.7,D0Re0
        DLWT      SUB       D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T1
        DMOWT     SUB       D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T1
        DWT       SUB       D0Re0,D0Re0,AC0.0 [A1LbP+A1.7++],T1
        DOWT      SUB       D0Re0,D0Re0,AC0.0 [A1LbP+A1.7++],T1
        D         SUB       [D0AW.0++],D0.7,A1.7
        DMO       SUB       [D0AW.0++],D0.7,A1.7
        DM        SUB       [D0AW.0++],D0.7,D1.7
        DLM       SUB       [D0AW.0++],D0.7,[D0AR.0]
        DM        SUB       [D0AW.0++],D0.7,AC0.0
        DT        SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1.7++]
        DLM       SUB       [D0AW.0++],D0.7,[D0AR.0++]
        DMOWT     SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1.7++]
        DWT       SUB       AC0.0,AC0.0,AC1.0 T1,[A1LbP+A1.7++]
        DLMWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1.7++]
        DLMOWT    SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1.7++]
        DLMO      SUB       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DMO       SUB       [D0AW.0++],D0.7,A0.7
        DLMWT     SUB       D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T1
        DLMOWT    SUB       D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T1
        DLWT      SUB       D0Re0,D0Re0,AC0.0 [A1LbP+A1.8++],T1
        DLOWT     SUB       D0Re0,D0Re0,AC0.0 [A1LbP+A1.8++],T1
        DLWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1.8++]
        DLOWT     SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1.8++]
        DMWT      SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1.8++]
        DMOWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1.8++]
        DLM       SUB       [D0AW.0++],[D0AR.0],D0Re0
        DM        SUB       [D0AW.0++],[D0AR.0],A1LbP
        DOT       SUB       D0Re0,D0Re0,D0Re0 [A1LbP],T1
        DLMOWT    SUB       D0Re0,D0Re0,D0Re0 [A1LbP],T1
        DLWT      SUB       D0Re0,D0Re0,AC0.0 [A1LbP],T1
        DLOWT     SUB       D0Re0,D0Re0,AC0.0 [A1LbP],T1
        DLM       SUB       [D0AW.0++],[D0AR.0],D0.7
        DM        SUB       [D0AW.0++],[D0AR.0],D1Re0
        D         SUB       [D0AW.0++],[D0AR.0],RD
        DMO       SUB       [D0AW.0++],[D0AR.0],RD
        DMO       SUB       [D0AW.0++],[D0AR.0],AC0.0
        DMWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP]
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP]
        DLMO      SUB       [D0AW.0++],[D0AR.0],[D0AR.0++]
        DMWT      SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP]
        DMOWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP]
        DLM       SUB       [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DMO       SUB       [D0AW.0++],[D0AR.0],A0FrP
        DLM       SUB       [D0AW.0++],[D0AR.0++],D0Re0
        DM        SUB       [D0AW.0++],[D0AR.0++],A1LbP
        DOT       SUB       D0Re0,D0Re0,D0Re0 [A1LbP++],T1
        DLMOWT    SUB       D0Re0,D0Re0,D0Re0 [A1LbP++],T1
        DLWT      SUB       D0Re0,D0Re0,AC0.0 [A1LbP++],T1
        DLOWT     SUB       D0Re0,D0Re0,AC0.0 [A1LbP++],T1
        DLM       SUB       [D0AW.0++],[D0AR.0++],D0.7
        DM        SUB       [D0AW.0++],[D0AR.0++],D1Re0
        D         SUB       [D0AW.0++],[D0AR.0++],RD
        DMO       SUB       [D0AW.0++],[D0AR.0++],RD
        DMO       SUB       [D0AW.0++],[D0AR.0++],AC0.0
        DMWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP++]
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP++]
        DLMO      SUB       [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DMWT      SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP++]
        DMOWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP++]
        DLM       SUB       [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DMO       SUB       [D0AW.0++],[D0AR.0++],A0FrP
        DLM       SUB       [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DMO       SUB       [D0AW.0++],[D0AR.0+D0ARI.0++],A1LbP
        DO        SUB       [D0AW.0++],[D0AR.0+D0ARI.0++],A1.7
        DMO       SUB       [D0AW.0++],[D0AR.0+D0ARI.0++],D1Re0
        DM        SUB       [D0AW.0++],[D0AR.0+D0ARI.0++],RD
        DLMO      SUB       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLMO      SUB       [D0AW.0++],[D0AR.0+D0ARI.0++],AC0.0
        DLMO      SUB       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLMO      SUB       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DMO       SUB       [D0AW.0++],[D0AR.0+D0ARI.0++],A0.7
        DLMWT     SUB       D0Re0,D0Re0,D0Re0 [A1LbP--],T1
        DLMOWT    SUB       D0Re0,D0Re0,D0Re0 [A1LbP--],T1
        DLWT      SUB       D0Re0,D0Re0,AC0.0 [A1LbP--],T1
        DLOWT     SUB       D0Re0,D0Re0,AC0.0 [A1LbP--],T1
        DLWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP--]
        DLOWT     SUB       D0Re0,D0Re0,D0Re0 T1,[A1LbP--]
        DMWT      SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP--]
        DMOWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1LbP--]
        DLMT      SUB       D0Re0,D0Re0,D0Re0 [D0AW.0++],T1
        DLT       SUB       D0Re0,D0Re0,AC0.0 [D0AW.0++],T1
        DLT       SUB       D0Re0,D0Re0,D0Re0 T1,[D0AR.0++]
        DMT       SUB       D0Re0,D0Re0,AC0.0 T1,[D0AR.0++]
        D         NEG       [D0AW.0+D0AWI.0++],D0Re0
        DO        NEG       [D0AW.0+D0AWI.0++],D0Re0
        D         NEG       [D0AW.0+D0AWI.0++],A1LbP
        DMO       NEG       [D0AW.0+D0AWI.0++],A1LbP
        DM        SUB       [D0AW.0+D0AWI.0++],D0Re0,A1.7
        DLM       NEG       [D0AW.0+D0AWI.0++],A1.7
        DMO       NEG       [D0AW.0+D0AWI.0++],A1.7
        D         NEG       [D0AW.0+D0AWI.0++],D1Re0
        DMO       NEG       [D0AW.0+D0AWI.0++],D1Re0
        DLM       NEG       [D0AW.0+D0AWI.0++],D1.7
        D         SUB       [D0AW.0+D0AWI.0++],D0Re0,RD
        DL        NEG       [D0AW.0+D0AWI.0++],[D0AR.0]
        DO        NEG       [D0AW.0+D0AWI.0++],RD
        DLMO      SUB       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DL        NEG       [D0AW.0+D0AWI.0++],AC0.0
        DLO       NEG       [D0AW.0+D0AWI.0++],AC0.0
        DL        NEG       [D0AW.0+D0AWI.0++],[D0AR.0++]
        DLO       NEG       [D0AW.0+D0AWI.0++],[D0AR.0++]
        DL        NEG       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++]
        DLO       NEG       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++]
        DLM       NEG       [D0AW.0+D0AWI.0++],A0FrP
        D         NEG       [D0AW.0+D0AWI.0++],A0.7
        DMO       NEG       [D0AW.0+D0AWI.0++],A0.7
        DMO       SUB       [D0AW.0+D0AWI.0++],D0.7,D0Re0
        D         SUB       [D0AW.0+D0AWI.0++],D0.7,A1.7
        DMO       SUB       [D0AW.0+D0AWI.0++],D0.7,A1.7
        DM        SUB       [D0AW.0+D0AWI.0++],D0.7,D1.7
        DLM       SUB       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DM        SUB       [D0AW.0+D0AWI.0++],D0.7,AC0.0
        DM        SUB       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DM        SUB       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DM        SUB       [D0AW.0+D0AWI.0++],D0.7,A0FrP
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0],A1LbP
        DLM       SUB       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0],D1Re0
        D         SUB       [D0AW.0+D0AWI.0++],[D0AR.0],RD
        DMO       SUB       [D0AW.0+D0AWI.0++],[D0AR.0],RD
        DMO       SUB       [D0AW.0+D0AWI.0++],[D0AR.0],AC0.0
        DMO       SUB       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        DMO       SUB       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0],A0.7
        DMO       SUB       [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         SUB       [D0AW.0+D0AWI.0++],[D0AR.0++],A1.7
        DMO       SUB       [D0AW.0+D0AWI.0++],[D0AR.0++],A1.7
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0++],D1.7
        DLM       SUB       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0++],AC0.0
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0++],A0FrP
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A1LbP
        DLM       SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D1Re0
        D         SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],RD
        DMO       SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],RD
        DMO       SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],AC0.0
        DMO       SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DMO       SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DM        SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A0.7
        DMOT      SUB       D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T1
        DOT       SUB       D0Re0,D0Re0,AC0.0 [D0AW.0+D0AWI.0++],T1
        DOT       SUB       D0Re0,D0Re0,D0Re0 T1,[D0AR.0+D0ARI.0++]
        DLMT      SUB       D0Re0,D0Re0,AC0.0 T1,[D0AR.0+D0ARI.0++]
        DMWT      SUB       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T1
        DMOWT     SUB       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T1
        DWT       SUB       D0Re0,D0Re0,AC0.0 [A1.7+A1LbP++],T1
        DOWT      SUB       D0Re0,D0Re0,AC0.0 [A1.7+A1LbP++],T1
        DWT       SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7+A1LbP++]
        DOWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7+A1LbP++]
        DWT       SUB       AC0.0,AC0.0,AC1.0 T1,[A1.7+A1LbP++]
        DLMWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7+A1LbP++]
        DLMOWT    SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7+A1LbP++]
        DLMWT     SUB       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T1
        DLMOWT    SUB       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T1
        DLWT      SUB       D0Re0,D0Re0,AC0.0 [A1.7+A1.7++],T1
        DLOWT     SUB       D0Re0,D0Re0,AC0.0 [A1.7+A1.7++],T1
        DLWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7+A1.7++]
        DLOWT     SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7+A1.7++]
        DMWT      SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7+A1.7++]
        DMOWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7+A1.7++]
        DMWT      SUB       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T1
        DMOWT     SUB       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T1
        DWT       SUB       D0Re0,D0Re0,AC0.0 [A1.7+A1.8++],T1
        DOWT      SUB       D0Re0,D0Re0,AC0.0 [A1.7+A1.8++],T1
        DWT       SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7+A1.8++]
        DOWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7+A1.8++]
        DWT       SUB       AC0.0,AC0.0,AC1.0 T1,[A1.7+A1.8++]
        DLMWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7+A1.8++]
        DLMOWT    SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7+A1.8++]
        DLMWT     SUB       D0Re0,D0Re0,D0Re0 [A1.7],T1
        DLMOWT    SUB       D0Re0,D0Re0,D0Re0 [A1.7],T1
        DLWT      SUB       D0Re0,D0Re0,AC0.0 [A1.7],T1
        DLOWT     SUB       D0Re0,D0Re0,AC0.0 [A1.7],T1
        DLWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7]
        DLOWT     SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7]
        DMWT      SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7]
        DMOWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7]
        DMWT      SUB       D0Re0,D0Re0,D0Re0 [A1.7++],T1
        DMOWT     SUB       D0Re0,D0Re0,D0Re0 [A1.7++],T1
        DWT       SUB       D0Re0,D0Re0,AC0.0 [A1.7++],T1
        DOWT      SUB       D0Re0,D0Re0,AC0.0 [A1.7++],T1
        DWT       SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7++]
        DOWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7++]
        DWT       SUB       AC0.0,AC0.0,AC1.0 T1,[A1.7++]
        DLMWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7++]
        DLMOWT    SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7++]
        DLMWT     SUB       D0Re0,D0Re0,D0Re0 [A1.7--],T1
        DLMOWT    SUB       D0Re0,D0Re0,D0Re0 [A1.7--],T1
        DLWT      SUB       D0Re0,D0Re0,AC0.0 [A1.7--],T1
        DLOWT     SUB       D0Re0,D0Re0,AC0.0 [A1.7--],T1
        DLWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7--]
        DLOWT     SUB       D0Re0,D0Re0,D0Re0 T1,[A1.7--]
        DMWT      SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7--]
        DMOWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.7--]
        DMWT      SUB       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T1
        DMOWT     SUB       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T1
        DWT       SUB       D0Re0,D0Re0,AC0.0 [A1.8+A1LbP++],T1
        DOWT      SUB       D0Re0,D0Re0,AC0.0 [A1.8+A1LbP++],T1
        DWT       SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8+A1LbP++]
        DOWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8+A1LbP++]
        DWT       SUB       AC0.0,AC0.0,AC1.0 T1,[A1.8+A1LbP++]
        DLMWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8+A1LbP++]
        DLMOWT    SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8+A1LbP++]
        DLMWT     SUB       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T1
        DLMOWT    SUB       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T1
        DLWT      SUB       D0Re0,D0Re0,AC0.0 [A1.8+A1.7++],T1
        DLOWT     SUB       D0Re0,D0Re0,AC0.0 [A1.8+A1.7++],T1
        DLWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8+A1.7++]
        DLOWT     SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8+A1.7++]
        DMWT      SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8+A1.7++]
        DMOWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8+A1.7++]
        DMWT      SUB       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T1
        DMOWT     SUB       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T1
        DWT       SUB       D0Re0,D0Re0,AC0.0 [A1.8+A1.8++],T1
        DOWT      SUB       D0Re0,D0Re0,AC0.0 [A1.8+A1.8++],T1
        DWT       SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8+A1.8++]
        DOWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8+A1.8++]
        DWT       SUB       AC0.0,AC0.0,AC1.0 T1,[A1.8+A1.8++]
        DLMWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8+A1.8++]
        DLMOWT    SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8+A1.8++]
        DLMWT     SUB       D0Re0,D0Re0,D0Re0 [A1.8],T1
        DLMOWT    SUB       D0Re0,D0Re0,D0Re0 [A1.8],T1
        DLWT      SUB       D0Re0,D0Re0,AC0.0 [A1.8],T1
        DLOWT     SUB       D0Re0,D0Re0,AC0.0 [A1.8],T1
        DLWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8]
        DLOWT     SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8]
        DMWT      SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8]
        DMOWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8]
        DMWT      SUB       D0Re0,D0Re0,D0Re0 [A1.8++],T1
        DMOWT     SUB       D0Re0,D0Re0,D0Re0 [A1.8++],T1
        DWT       SUB       D0Re0,D0Re0,AC0.0 [A1.8++],T1
        DOWT      SUB       D0Re0,D0Re0,AC0.0 [A1.8++],T1
        DWT       SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8++]
        DOWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8++]
        DWT       SUB       AC0.0,AC0.0,AC1.0 T1,[A1.8++]
        DLMWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8++]
        DLMOWT    SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8++]
        DLMWT     SUB       D0Re0,D0Re0,D0Re0 [A1.8--],T1
        DLMOWT    SUB       D0Re0,D0Re0,D0Re0 [A1.8--],T1
        DLWT      SUB       D0Re0,D0Re0,AC0.0 [A1.8--],T1
        DLOWT     SUB       D0Re0,D0Re0,AC0.0 [A1.8--],T1
        DLWT      SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8--]
        DLOWT     SUB       D0Re0,D0Re0,D0Re0 T1,[A1.8--]
        DMWT      SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8--]
        DMOWT     SUB       D0Re0,D0Re0,AC0.0 T1,[A1.8--]
        DM        NEG       D1Re0,D1Re0
        DMO       NEG       D1Re0,D1Re0
        DM        NEG       D1Re0,A1LbP
        DLMO      NEG       D1Re0,A1LbP
        DM        NEG       D1Re0,A1.7
        DO        SUB       D1Re0,D1Re0,A1.7
        DLO       NEG       D1Re0,D1.7
        DM        NEG       D1Re0,D0Re0
        DLMO      NEG       D1Re0,D0Re0
        DO        NEG       D1Re0,D0.7
        DM        NEG       D1Re0,RD
        DLMO      NEG       D1Re0,RD
        DLM       NEG       D1Re0,AC1.0
        DLMO      NEG       D1Re0,AC1.0
        DO        NEG       D1Re0,A0FrP
        DM        NEG       D1Re0,[D1BR.1]
        DMO       NEG       D1Re0,[D1BR.1]
        DM        NEG       D1Re0,[D1BR.1++]
        DMO       NEG       D1Re0,[D1BR.1++]
        D         NEG       D1Re0,A0.7
        DLM       SUB       D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DMO       SUB       D1Re0,D1Re0,A0.7
        DLMO      NEG       D1Re0,A0.7
        DLMO      SUB       D1Re0,D1.7,D1Re0
        DM        SUB       D1Re0,D1.7,A1.7
        DLMO      SUB       D1Re0,D1.7,D1.7
        DMO       SUB       D1Re0,D1.7,D0.7
        DLM       SUB       D1Re0,D1.7,AC1.0
        DMO       SUB       D1Re0,D1.7,A0FrP
        DLMO      SUB       D1Re0,D1.7,[D1BR.1]
        DLMO      SUB       D1Re0,D1.7,[D1BR.1++]
        DO        SUB       D1Re0,D1.7,A0.7
        DLM       SUB       D1Re0,[D1BR.1],D1Re0
        DMO       SUB       D1Re0,[D1BR.1],A1LbP
        DO        SUB       D1Re0,[D1BR.1],A1.7
        DMO       SUB       D1Re0,[D1BR.1],D0Re0
        DMO       SUB       D1Re0,[D1BR.1],RD
        DLMO      SUB       D1Re0,[D1BR.1],AC1.0
        DLM       SUB       D1Re0,[D1BR.1],[D1BR.1]
        DLM       SUB       D1Re0,[D1BR.1],[D1BR.1++]
        DM        SUB       D1Re0,[D1BR.1],A0.7
        DLMO      SUB       D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLMO      SUB       D1Re0,[D1BR.1++],D1Re0
        DM        SUB       D1Re0,[D1BR.1++],A1.7
        DLMO      SUB       D1Re0,[D1BR.1++],D1.7
        DMO       SUB       D1Re0,[D1BR.1++],D0.7
        DLM       SUB       D1Re0,[D1BR.1++],AC1.0
        DMO       SUB       D1Re0,[D1BR.1++],A0FrP
        DLMO      SUB       D1Re0,[D1BR.1++],[D1BR.1]
        DLMO      SUB       D1Re0,[D1BR.1++],[D1BR.1++]
        DO        SUB       D1Re0,[D1BR.1++],A0.7
        DLM       SUB       D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DMO       SUB       D1Re0,[D1BR.1+D1BRI.1++],A1LbP
        DO        SUB       D1Re0,[D1BR.1+D1BRI.1++],A1.7
        DMO       SUB       D1Re0,[D1BR.1+D1BRI.1++],D0Re0
        DMO       SUB       D1Re0,[D1BR.1+D1BRI.1++],RD
        DLMO      SUB       D1Re0,[D1BR.1+D1BRI.1++],AC1.0
        DLM       SUB       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLM       SUB       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DM        SUB       D1Re0,[D1BR.1+D1BRI.1++],A0.7
        DLMO      SUB       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T1
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T1
        DLWT      SUB       D1Re0,D1Re0,AC1.0 [A0FrP+A0FrP++],T1
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 [A0FrP+A0FrP++],T1
        DLWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0FrP++]
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0FrP++]
        DMWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0FrP++]
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0FrP++]
        DMWT      SUB       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T1
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T1
        DWT       SUB       D1Re0,D1Re0,AC1.0 [A0FrP+A0.7++],T1
        DOWT      SUB       D1Re0,D1Re0,AC1.0 [A0FrP+A0.7++],T1
        DWT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0.7++]
        DOWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0.7++]
        DWT       SUB       AC1.0,AC1.0,AC0.0 T1,[A0FrP+A0.7++]
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0.7++]
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0.7++]
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T1
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T1
        DLWT      SUB       D1Re0,D1Re0,AC1.0 [A0FrP+A0.8++],T1
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 [A0FrP+A0.8++],T1
        DLWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0.8++]
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0.8++]
        DMWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0.8++]
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0.8++]
        DMWT      SUB       D1Re0,D1Re0,D1Re0 [A0FrP],T1
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 [A0FrP],T1
        DWT       SUB       D1Re0,D1Re0,AC1.0 [A0FrP],T1
        DOWT      SUB       D1Re0,D1Re0,AC1.0 [A0FrP],T1
        DWT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP]
        DOWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP]
        DWT       SUB       AC1.0,AC1.0,AC0.0 T1,[A0FrP]
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP]
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP]
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 [A0FrP++],T1
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 [A0FrP++],T1
        DLWT      SUB       D1Re0,D1Re0,AC1.0 [A0FrP++],T1
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 [A0FrP++],T1
        DLWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP++]
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP++]
        DMWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP++]
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP++]
        DMWT      SUB       D1Re0,D1Re0,D1Re0 [A0FrP--],T1
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 [A0FrP--],T1
        DWT       SUB       D1Re0,D1Re0,AC1.0 [A0FrP--],T1
        DOWT      SUB       D1Re0,D1Re0,AC1.0 [A0FrP--],T1
        DWT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP--]
        DOWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0FrP--]
        DWT       SUB       AC1.0,AC1.0,AC0.0 T1,[A0FrP--]
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP--]
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 T1,[A0FrP--]
        DLM       NEG       D1.7,D1Re0
        DLMO      NEG       D1.7,D1Re0
        DO        NEG       D1.7,A1LbP
        D         NEG       D1.7,A1.7
        DLM       SUB       D1.7,D1Re0,D1.7
        DMO       SUB       D1.7,D1Re0,A1.7
        DLMO      NEG       D1.7,A1.7
        DO        NEG       D1.7,D0Re0
        DM        NEG       D1.7,D0.7
        DLMO      NEG       D1.7,D0.7
        DO        NEG       D1.7,RD
        DM        NEG       D1.7,AC1.0
        DMO       NEG       D1.7,AC1.0
        DM        NEG       D1.7,A0FrP
        DLMO      NEG       D1.7,A0FrP
        DLM       NEG       D1.7,[D1BR.1]
        DLMO      NEG       D1.7,[D1BR.1]
        DLM       NEG       D1.7,[D1BR.1++]
        DLMO      NEG       D1.7,[D1BR.1++]
        DM        NEG       D1.7,A0.7
        DO        SUB       D1.7,D1Re0,A0.7
        DLO       NEG       D1.7,[D1BR.1+D1BRI.1++]
        DMWT      SUB       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T1
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T1
        DWT       SUB       D1Re0,D1Re0,AC1.0 [A0.7+A0FrP++],T1
        DOWT      SUB       D1Re0,D1Re0,AC1.0 [A0.7+A0FrP++],T1
        DWT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7+A0FrP++]
        DOWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7+A0FrP++]
        DWT       SUB       AC1.0,AC1.0,AC0.0 T1,[A0.7+A0FrP++]
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7+A0FrP++]
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7+A0FrP++]
        DLMO      SUB       D1.7,D1.7,D1Re0
        DLWT      SUB       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T1
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T1
        DWT       SUB       D1Re0,D1Re0,AC1.0 [A0.7+A0.7++],T1
        DOWT      SUB       D1Re0,D1Re0,AC1.0 [A0.7+A0.7++],T1
        D         SUB       D1.7,D1.7,A1.7
        DMO       SUB       D1.7,D1.7,A1.7
        DM        SUB       D1.7,D1.7,D0.7
        DM        SUB       D1.7,D1.7,AC1.0
        DWT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7+A0.7++]
        DOWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7+A0.7++]
        DWT       SUB       AC1.0,AC1.0,AC0.0 T1,[A0.7+A0.7++]
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7+A0.7++]
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7+A0.7++]
        DLM       SUB       D1.7,D1.7,[D1BR.1]
        DLM       SUB       D1.7,D1.7,[D1BR.1++]
        DM        SUB       D1.7,D1.7,A0.7
        DLMO      SUB       D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T1
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T1
        DLWT      SUB       D1Re0,D1Re0,AC1.0 [A0.7+A0.8++],T1
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 [A0.7+A0.8++],T1
        DLWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7+A0.8++]
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7+A0.8++]
        DMWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7+A0.8++]
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7+A0.8++]
        DMWT      SUB       D1Re0,D1Re0,D1Re0 [A0.7],T1
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 [A0.7],T1
        DWT       SUB       D1Re0,D1Re0,AC1.0 [A0.7],T1
        DOWT      SUB       D1Re0,D1Re0,AC1.0 [A0.7],T1
        DWT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7]
        DOWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7]
        DWT       SUB       AC1.0,AC1.0,AC0.0 T1,[A0.7]
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7]
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7]
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 [A0.7++],T1
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 [A0.7++],T1
        DLWT      SUB       D1Re0,D1Re0,AC1.0 [A0.7++],T1
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 [A0.7++],T1
        DLWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7++]
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7++]
        DMWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7++]
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7++]
        DMWT      SUB       D1Re0,D1Re0,D1Re0 [A0.7--],T1
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 [A0.7--],T1
        DWT       SUB       D1Re0,D1Re0,AC1.0 [A0.7--],T1
        DOWT      SUB       D1Re0,D1Re0,AC1.0 [A0.7--],T1
        DWT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7--]
        DOWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0.7--]
        DWT       SUB       AC1.0,AC1.0,AC0.0 T1,[A0.7--]
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7--]
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 T1,[A0.7--]
        DLMO      SUB       D1.7,[D1BR.1],D1Re0
        DM        SUB       D1.7,[D1BR.1],A1.7
        DLMO      SUB       D1.7,[D1BR.1],D1.7
        DMO       SUB       D1.7,[D1BR.1],D0.7
        DLM       SUB       D1.7,[D1BR.1],AC1.0
        DMO       SUB       D1.7,[D1BR.1],A0FrP
        DLMO      SUB       D1.7,[D1BR.1],[D1BR.1]
        DLMO      SUB       D1.7,[D1BR.1],[D1BR.1++]
        DO        SUB       D1.7,[D1BR.1],A0.7
        DLM       SUB       D1.7,[D1BR.1++],D1Re0
        DMO       SUB       D1.7,[D1BR.1++],A1LbP
        DO        SUB       D1.7,[D1BR.1++],A1.7
        DMO       SUB       D1.7,[D1BR.1++],D0Re0
        DMO       SUB       D1.7,[D1BR.1++],RD
        DLO       SUB       D1.7,[D1BR.1++],AC1.0
        DL        SUB       D1.7,[D1BR.1++],[D1BR.1]
        DL        SUB       D1.7,[D1BR.1++],[D1BR.1++]
        DM        SUB       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLO       SUB       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLO       SUB       D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DM        SUB       D1.7,[D1BR.1+D1BRI.1++],D1.7
        DLO       SUB       D1.7,[D1BR.1+D1BRI.1++],D1.7
        DO        SUB       D1.7,[D1BR.1+D1BRI.1++],D0.7
        DL        SUB       D1.7,[D1BR.1+D1BRI.1++],AC1.0
        DO        SUB       D1.7,[D1BR.1+D1BRI.1++],A0FrP
        DLO       SUB       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLO       SUB       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DO        SUB       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DMT       SUB       D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T1
        DMOT      SUB       D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T1
        DT        SUB       D1Re0,D1Re0,AC1.0 [A0.8+A0FrP++],T1
        DOT       SUB       D1Re0,D1Re0,AC1.0 [A0.8+A0FrP++],T1
        DT        SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8+A0FrP++]
        DOT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8+A0FrP++]
        DT        SUB       AC1.0,AC1.0,AC0.0 T1,[A0.8+A0FrP++]
        DLMT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8+A0FrP++]
        DLMOT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8+A0FrP++]
        DLMT      SUB       D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T1
        DLMOT     SUB       D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T1
        DLT       SUB       D1Re0,D1Re0,AC1.0 [A0.8+A0.7++],T1
        DLOT      SUB       D1Re0,D1Re0,AC1.0 [A0.8+A0.7++],T1
        DLT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8+A0.7++]
        DLOT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8+A0.7++]
        DMT       SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8+A0.7++]
        DMOT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8+A0.7++]
        DMT       SUB       D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T1
        DMOT      SUB       D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T1
        DT        SUB       D1Re0,D1Re0,AC1.0 [A0.8+A0.8++],T1
        DOT       SUB       D1Re0,D1Re0,AC1.0 [A0.8+A0.8++],T1
        DT        SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8+A0.8++]
        DOT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8+A0.8++]
        DT        SUB       AC1.0,AC1.0,AC0.0 T1,[A0.8+A0.8++]
        DLMT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8+A0.8++]
        DLMOT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8+A0.8++]
        DLMT      SUB       D1Re0,D1Re0,D1Re0 [A0.8],T1
        DLMOT     SUB       D1Re0,D1Re0,D1Re0 [A0.8],T1
        DLT       SUB       D1Re0,D1Re0,AC1.0 [A0.8],T1
        DLOT      SUB       D1Re0,D1Re0,AC1.0 [A0.8],T1
        DLT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8]
        DLOT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8]
        DMT       SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8]
        DMOT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8]
        DMT       SUB       D1Re0,D1Re0,D1Re0 [A0.8++],T1
        DMOT      SUB       D1Re0,D1Re0,D1Re0 [A0.8++],T1
        DT        SUB       D1Re0,D1Re0,AC1.0 [A0.8++],T1
        DOT       SUB       D1Re0,D1Re0,AC1.0 [A0.8++],T1
        DT        SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8++]
        DOT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8++]
        DT        SUB       AC1.0,AC1.0,AC0.0 T1,[A0.8++]
        DLMT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8++]
        DLMOT     SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8++]
        DLMT      SUB       D1Re0,D1Re0,D1Re0 [A0.8--],T1
        DLMOT     SUB       D1Re0,D1Re0,D1Re0 [A0.8--],T1
        DLT       SUB       D1Re0,D1Re0,AC1.0 [A0.8--],T1
        DLOT      SUB       D1Re0,D1Re0,AC1.0 [A0.8--],T1
        DLT       SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8--]
        DLOT      SUB       D1Re0,D1Re0,D1Re0 T1,[A0.8--]
        DMT       SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8--]
        DMOT      SUB       D1Re0,D1Re0,AC1.0 T1,[A0.8--]
        DWT       SUB       D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T1
        DOWT      SUB       D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T1
        DWT       SUB       AC1.0,AC1.0,AC0.0 [A1LbP+A1LbP++],T1
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 [A1LbP+A1LbP++],T1
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 [A1LbP+A1LbP++],T1
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1LbP++]
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1LbP++]
        DLWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1LbP++]
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1LbP++]
        DLWT      SUB       D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T1
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T1
        DMWT      SUB       D1Re0,D1Re0,AC1.0 [A1LbP+A1.7++],T1
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 [A1LbP+A1.7++],T1
        DMWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1.7++]
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1.7++]
        DWT       SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1.7++]
        DOWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1.7++]
        DWT       SUB       D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T1
        DOWT      SUB       D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T1
        DWT       SUB       AC1.0,AC1.0,AC0.0 [A1LbP+A1.8++],T1
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 [A1LbP+A1.8++],T1
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 [A1LbP+A1.8++],T1
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1.8++]
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1.8++]
        DLWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1.8++]
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1.8++]
        DLWT      SUB       D1Re0,D1Re0,D1Re0 [A1LbP],T1
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 [A1LbP],T1
        DMWT      SUB       D1Re0,D1Re0,AC1.0 [A1LbP],T1
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 [A1LbP],T1
        DMWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP]
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP]
        DWT       SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP]
        DOWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP]
        DWT       SUB       D1Re0,D1Re0,D1Re0 [A1LbP++],T1
        DOWT      SUB       D1Re0,D1Re0,D1Re0 [A1LbP++],T1
        DWT       SUB       AC1.0,AC1.0,AC0.0 [A1LbP++],T1
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 [A1LbP++],T1
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 [A1LbP++],T1
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP++]
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP++]
        DLWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP++]
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP++]
        DLWT      SUB       D1Re0,D1Re0,D1Re0 [A1LbP--],T1
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 [A1LbP--],T1
        DMWT      SUB       D1Re0,D1Re0,AC1.0 [A1LbP--],T1
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 [A1LbP--],T1
        DMWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP--]
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1LbP--]
        DWT       SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP--]
        DOWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1LbP--]
        DWT       SUB       D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T1
        DOWT      SUB       D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T1
        DWT       SUB       AC1.0,AC1.0,AC0.0 [A1.7+A1LbP++],T1
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 [A1.7+A1LbP++],T1
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 [A1.7+A1LbP++],T1
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7+A1LbP++]
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7+A1LbP++]
        DLWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7+A1LbP++]
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7+A1LbP++]
        DLWT      SUB       D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T1
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T1
        DMWT      SUB       D1Re0,D1Re0,AC1.0 [A1.7+A1.7++],T1
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 [A1.7+A1.7++],T1
        DMWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7+A1.7++]
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7+A1.7++]
        DWT       SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7+A1.7++]
        DOWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7+A1.7++]
        DWT       SUB       D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T1
        DOWT      SUB       D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T1
        DWT       SUB       AC1.0,AC1.0,AC0.0 [A1.7+A1.8++],T1
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 [A1.7+A1.8++],T1
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 [A1.7+A1.8++],T1
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7+A1.8++]
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7+A1.8++]
        DLWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7+A1.8++]
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7+A1.8++]
        DLWT      SUB       D1Re0,D1Re0,D1Re0 [A1.7],T1
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 [A1.7],T1
        DMWT      SUB       D1Re0,D1Re0,AC1.0 [A1.7],T1
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 [A1.7],T1
        DMWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7]
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7]
        DWT       SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7]
        DOWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7]
        DWT       SUB       D1Re0,D1Re0,D1Re0 [A1.7++],T1
        DOWT      SUB       D1Re0,D1Re0,D1Re0 [A1.7++],T1
        DWT       SUB       AC1.0,AC1.0,AC0.0 [A1.7++],T1
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 [A1.7++],T1
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 [A1.7++],T1
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7++]
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7++]
        DLWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7++]
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7++]
        DLWT      SUB       D1Re0,D1Re0,D1Re0 [A1.7--],T1
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 [A1.7--],T1
        DMWT      SUB       D1Re0,D1Re0,AC1.0 [A1.7--],T1
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 [A1.7--],T1
        DMWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7--]
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.7--]
        DWT       SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7--]
        DOWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.7--]
        DWT       SUB       D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T1
        DOWT      SUB       D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T1
        DWT       SUB       AC1.0,AC1.0,AC0.0 [A1.8+A1LbP++],T1
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 [A1.8+A1LbP++],T1
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 [A1.8+A1LbP++],T1
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8+A1LbP++]
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8+A1LbP++]
        DLWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8+A1LbP++]
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8+A1LbP++]
        DLWT      SUB       D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T1
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T1
        DMWT      SUB       D1Re0,D1Re0,AC1.0 [A1.8+A1.7++],T1
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 [A1.8+A1.7++],T1
        DMWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8+A1.7++]
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8+A1.7++]
        DWT       SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8+A1.7++]
        DOWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8+A1.7++]
        DWT       SUB       D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T1
        DOWT      SUB       D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T1
        DWT       SUB       AC1.0,AC1.0,AC0.0 [A1.8+A1.8++],T1
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 [A1.8+A1.8++],T1
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 [A1.8+A1.8++],T1
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8+A1.8++]
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8+A1.8++]
        DLWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8+A1.8++]
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8+A1.8++]
        DLWT      SUB       D1Re0,D1Re0,D1Re0 [A1.8],T1
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 [A1.8],T1
        DMWT      SUB       D1Re0,D1Re0,AC1.0 [A1.8],T1
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 [A1.8],T1
        DMWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8]
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8]
        DWT       SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8]
        DOWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8]
        DWT       SUB       D1Re0,D1Re0,D1Re0 [A1.8++],T1
        DOWT      SUB       D1Re0,D1Re0,D1Re0 [A1.8++],T1
        DWT       SUB       AC1.0,AC1.0,AC0.0 [A1.8++],T1
        DLMWT     SUB       D1Re0,D1Re0,AC1.0 [A1.8++],T1
        DLMOWT    SUB       D1Re0,D1Re0,AC1.0 [A1.8++],T1
        DLMWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8++]
        DLMOWT    SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8++]
        DLWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8++]
        DLOWT     SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8++]
        DLWT      SUB       D1Re0,D1Re0,D1Re0 [A1.8--],T1
        DLOWT     SUB       D1Re0,D1Re0,D1Re0 [A1.8--],T1
        DMWT      SUB       D1Re0,D1Re0,AC1.0 [A1.8--],T1
        DMOWT     SUB       D1Re0,D1Re0,AC1.0 [A1.8--],T1
        DMWT      SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8--]
        DMOWT     SUB       D1Re0,D1Re0,D1Re0 T1,[A1.8--]
        DWT       SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8--]
        DOWT      SUB       D1Re0,D1Re0,AC1.0 T1,[A1.8--]
        D         NEG       [D1BW.1],D1Re0
        DO        NEG       [D1BW.1],D1Re0
        D         NEG       [D1BW.1],A1LbP
        DMO       NEG       [D1BW.1],A1LbP
        DM        SUB       [D1BW.1],D1Re0,A1.7
        DLM       NEG       [D1BW.1],A1.7
        DMO       NEG       [D1BW.1],A1.7
        D         NEG       [D1BW.1],D0Re0
        DMO       NEG       [D1BW.1],D0Re0
        DLM       NEG       [D1BW.1],D0.7
        D         NEG       [D1BW.1],RD
        DMO       NEG       [D1BW.1],RD
        DL        NEG       [D1BW.1],AC1.0
        DLO       NEG       [D1BW.1],AC1.0
        DLM       NEG       [D1BW.1],A0FrP
        D         NEG       [D1BW.1],[D1BR.1]
        DO        NEG       [D1BW.1],[D1BR.1]
        D         NEG       [D1BW.1],[D1BR.1++]
        DO        NEG       [D1BW.1],[D1BR.1++]
        D         SUB       [D1BW.1],D1Re0,A0.7
        DL        NEG       [D1BW.1],[D1BR.1+D1BRI.1++]
        DO        NEG       [D1BW.1],A0.7
        DLMO      SUB       [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DMO       SUB       [D1BW.1],D1.7,D1Re0
        D         SUB       [D1BW.1],D1.7,A1.7
        DMO       SUB       [D1BW.1],D1.7,A1.7
        DM        SUB       [D1BW.1],D1.7,D0.7
        DM        SUB       [D1BW.1],D1.7,AC1.0
        DM        SUB       [D1BW.1],D1.7,A0FrP
        DMO       SUB       [D1BW.1],D1.7,[D1BR.1]
        DMO       SUB       [D1BW.1],D1.7,[D1BR.1++]
        DLM       SUB       [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DMT       SUB       D1Re0,D1Re0,D1Re0 [D1BW.1],T1
        DT        SUB       D1Re0,D1Re0,AC1.0 [D1BW.1],T1
        DT        SUB       D1Re0,D1Re0,D1Re0 T1,[D1BR.1]
        DT        SUB       AC1.0,AC1.0,AC0.0 T1,[D1BR.1]
        DLMOT     SUB       D1Re0,D1Re0,AC1.0 T1,[D1BR.1]
        DLMO      SUB       [D1BW.1],[D1BR.1],D1Re0
        DM        SUB       [D1BW.1],[D1BR.1],A1.7
        DLMO      SUB       [D1BW.1],[D1BR.1],D1.7
        DMO       SUB       [D1BW.1],[D1BR.1],D0.7
        DLM       SUB       [D1BW.1],[D1BR.1],AC1.0
        DMO       SUB       [D1BW.1],[D1BR.1],A0FrP
        DLMO      SUB       [D1BW.1],[D1BR.1],[D1BR.1]
        DLMO      SUB       [D1BW.1],[D1BR.1],[D1BR.1++]
        DO        SUB       [D1BW.1],[D1BR.1],A0.7
        DLM       SUB       [D1BW.1],[D1BR.1++],D1Re0
        DMO       SUB       [D1BW.1],[D1BR.1++],A1LbP
        DO        SUB       [D1BW.1],[D1BR.1++],A1.7
        DMO       SUB       [D1BW.1],[D1BR.1++],D0Re0
        DMO       SUB       [D1BW.1],[D1BR.1++],RD
        DLMO      SUB       [D1BW.1],[D1BR.1++],AC1.0
        DLM       SUB       [D1BW.1],[D1BR.1++],[D1BR.1]
        DLM       SUB       [D1BW.1],[D1BR.1++],[D1BR.1++]
        DM        SUB       [D1BW.1],[D1BR.1++],A0.7
        DLMO      SUB       [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLMO      SUB       [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DM        SUB       [D1BW.1],[D1BR.1+D1BRI.1++],A1.7
        DLMO      SUB       [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DMO       SUB       [D1BW.1],[D1BR.1+D1BRI.1++],D0.7
        DLM       SUB       [D1BW.1],[D1BR.1+D1BRI.1++],AC1.0
        DMO       SUB       [D1BW.1],[D1BR.1+D1BRI.1++],A0FrP
        DLMO      SUB       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLMO      SUB       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DO        SUB       [D1BW.1],[D1BR.1+D1BRI.1++],A0.7
        DM        NEG       [D1BW.1++],D1Re0
        DMO       NEG       [D1BW.1++],D1Re0
        DM        NEG       [D1BW.1++],A1LbP
        DLMO      NEG       [D1BW.1++],A1LbP
        DM        NEG       [D1BW.1++],A1.7
        DO        SUB       [D1BW.1++],D1Re0,A1.7
        DLO       NEG       [D1BW.1++],D1.7
        DM        NEG       [D1BW.1++],D0Re0
        DLMO      NEG       [D1BW.1++],D0Re0
        DO        NEG       [D1BW.1++],D0.7
        DM        NEG       [D1BW.1++],RD
        DLMO      NEG       [D1BW.1++],RD
        DLM       NEG       [D1BW.1++],AC1.0
        DLMO      NEG       [D1BW.1++],AC1.0
        DO        NEG       [D1BW.1++],A0FrP
        DM        NEG       [D1BW.1++],[D1BR.1]
        DMO       NEG       [D1BW.1++],[D1BR.1]
        DM        NEG       [D1BW.1++],[D1BR.1++]
        DMO       NEG       [D1BW.1++],[D1BR.1++]
        D         NEG       [D1BW.1++],A0.7
        DLM       SUB       [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DMO       SUB       [D1BW.1++],D1Re0,A0.7
        DLMO      NEG       [D1BW.1++],A0.7
        DLMO      SUB       [D1BW.1++],D1.7,D1Re0
        DM        SUB       [D1BW.1++],D1.7,A1.7
        DLMO      SUB       [D1BW.1++],D1.7,D1.7
        DMO       SUB       [D1BW.1++],D1.7,D0.7
        DLM       SUB       [D1BW.1++],D1.7,AC1.0
        DMO       SUB       [D1BW.1++],D1.7,A0FrP
        DLMO      SUB       [D1BW.1++],D1.7,[D1BR.1]
        DLMO      SUB       [D1BW.1++],D1.7,[D1BR.1++]
        DO        SUB       [D1BW.1++],D1.7,A0.7
        DLMT      SUB       D1Re0,D1Re0,D1Re0 [D1BW.1++],T1
        DLT       SUB       D1Re0,D1Re0,AC1.0 [D1BW.1++],T1
        DLT       SUB       D1Re0,D1Re0,D1Re0 T1,[D1BR.1++]
        DMT       SUB       D1Re0,D1Re0,AC1.0 T1,[D1BR.1++]
        DM        SUB       [D1BW.1++],[D1BR.1],D1Re0
        DM        SUB       [D1BW.1++],[D1BR.1],A1LbP
        DLM       SUB       [D1BW.1++],[D1BR.1],D1.7
        DM        SUB       [D1BW.1++],[D1BR.1],D0Re0
        DM        SUB       [D1BW.1++],[D1BR.1],RD
        DMO       SUB       [D1BW.1++],[D1BR.1],AC1.0
        DM        SUB       [D1BW.1++],[D1BR.1],[D1BR.1]
        DM        SUB       [D1BW.1++],[D1BR.1],[D1BR.1++]
        D         SUB       [D1BW.1++],[D1BR.1],A0.7
        DMO       SUB       [D1BW.1++],[D1BR.1],A0.7
        DMO       SUB       [D1BW.1++],[D1BR.1++],D1Re0
        D         SUB       [D1BW.1++],[D1BR.1++],A1.7
        DMO       SUB       [D1BW.1++],[D1BR.1++],A1.7
        DM        SUB       [D1BW.1++],[D1BR.1++],D0.7
        DM        SUB       [D1BW.1++],[D1BR.1++],AC1.0
        DM        SUB       [D1BW.1++],[D1BR.1++],A0FrP
        DMO       SUB       [D1BW.1++],[D1BR.1++],[D1BR.1]
        DMO       SUB       [D1BW.1++],[D1BR.1++],[D1BR.1++]
        DLM       SUB       [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DM        SUB       [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        DM        SUB       [D1BW.1++],[D1BR.1+D1BRI.1++],A1LbP
        DLM       SUB       [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        DM        SUB       [D1BW.1++],[D1BR.1+D1BRI.1++],D0Re0
        DM        SUB       [D1BW.1++],[D1BR.1+D1BRI.1++],RD
        DMO       SUB       [D1BW.1++],[D1BR.1+D1BRI.1++],AC1.0
        DM        SUB       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DM        SUB       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         SUB       [D1BW.1++],[D1BR.1+D1BRI.1++],A0.7
        DMO       SUB       [D1BW.1++],[D1BR.1+D1BRI.1++],A0.7
        DL        NEG       [D1BW.1+D1BWI.1++],D1Re0
        DLO       NEG       [D1BW.1+D1BWI.1++],D1Re0
        DLM       NEG       [D1BW.1+D1BWI.1++],A1LbP
        D         SUB       [D1BW.1+D1BWI.1++],D1Re0,A1.7
        DL        NEG       [D1BW.1+D1BWI.1++],D1.7
        DO        NEG       [D1BW.1+D1BWI.1++],A1.7
        DLMO      SUB       [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DLM       NEG       [D1BW.1+D1BWI.1++],D0Re0
        D         NEG       [D1BW.1+D1BWI.1++],D0.7
        DMO       NEG       [D1BW.1+D1BWI.1++],D0.7
        DLM       NEG       [D1BW.1+D1BWI.1++],RD
        D         NEG       [D1BW.1+D1BWI.1++],AC1.0
        DO        NEG       [D1BW.1+D1BWI.1++],AC1.0
        D         NEG       [D1BW.1+D1BWI.1++],A0FrP
        DMO       NEG       [D1BW.1+D1BWI.1++],A0FrP
        DL        NEG       [D1BW.1+D1BWI.1++],[D1BR.1]
        DLO       NEG       [D1BW.1+D1BWI.1++],[D1BR.1]
        DL        NEG       [D1BW.1+D1BWI.1++],[D1BR.1++]
        DLO       NEG       [D1BW.1+D1BWI.1++],[D1BR.1++]
        DM        SUB       [D1BW.1+D1BWI.1++],D1Re0,A0.7
        DLM       NEG       [D1BW.1+D1BWI.1++],A0.7
        DMO       NEG       [D1BW.1+D1BWI.1++],A0.7
        DM        SUB       [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DM        SUB       [D1BW.1+D1BWI.1++],D1.7,A1LbP
        DLM       SUB       [D1BW.1+D1BWI.1++],D1.7,D1.7
        DM        SUB       [D1BW.1+D1BWI.1++],D1.7,D0Re0
        DM        SUB       [D1BW.1+D1BWI.1++],D1.7,RD
        DO        SUB       [D1BW.1+D1BWI.1++],D1.7,AC1.0
        D         SUB       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        D         SUB       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        D         SUB       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DMO       SUB       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DOT       SUB       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T1
        DLMT      SUB       D1Re0,D1Re0,AC1.0 [D1BW.1+D1BWI.1++],T1
        DLMT      SUB       D1Re0,D1Re0,D1Re0 T1,[D1BR.1+D1BRI.1++]
        DLT       SUB       D1Re0,D1Re0,AC1.0 T1,[D1BR.1+D1BRI.1++]
        DL        SUB       [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        DO        SUB       [D1BW.1+D1BWI.1++],[D1BR.1],A1LbP
        DO        SUB       [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        DO        SUB       [D1BW.1+D1BWI.1++],[D1BR.1],D0Re0
        DO        SUB       [D1BW.1+D1BWI.1++],[D1BR.1],RD
        DMO       SUB       [D1BW.1+D1BWI.1++],[D1BR.1],AC1.0
        DM        SUB       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        DM        SUB       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        D         SUB       [D1BW.1+D1BWI.1++],[D1BR.1],A0.7
        DMO       SUB       [D1BW.1+D1BWI.1++],[D1BR.1],A0.7
        DMO       SUB       [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        D         SUB       [D1BW.1+D1BWI.1++],[D1BR.1++],A1.7
        DMO       SUB       [D1BW.1+D1BWI.1++],[D1BR.1++],A1.7
        DM        SUB       [D1BW.1+D1BWI.1++],[D1BR.1++],D0.7
        D         SUB       [D1BW.1+D1BWI.1++],[D1BR.1++],AC1.0
        D         SUB       [D1BW.1+D1BWI.1++],[D1BR.1++],A0FrP
        DO        SUB       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        DO        SUB       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        DL        SUB       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1LbP
        DL        SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D0Re0
        D         SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],RD
        DLM       SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],AC1.0
        DMO       SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0FrP
        DLMO      SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLMO      SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DO        SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0.7
        DL        SUB       D0Re0,D0Re0,#-0x8000
        DL        SUB       D0.7,D0.7,#-0x1
        DL        SUB       [D0AW.0++],[D0AR.0++],#0x7fff
        DL        SUB       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#-0x8000
        DL        SUB       D1Re0,D1Re0,#-0x1
        DL        SUB       [D1BW.1],[D1BR.1],#0x7fff
        DL        SUB       [D1BW.1++],[D1BR.1++],#-0x8000
        DL        SUB       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#-0x1
        D         SUBNV     D0Re0,D0Re0,A1.7
        D         SUBNV     D0Re0,D0Re0,RD
        D         SUBNV     D0Re0,D0Re0,A0.7
        D         SUBNV     D0Re0,D0.7,A1.7
        D         SUBNV     D0Re0,D0.7,RD
        D         SUBNV     D0Re0,D0.7,A0.7
        D         SUBNV     D0Re0,[D0AR.0],RD
        D         SUBNV     D0Re0,[D0AR.0++],A1.7
        D         SUBNV     D0Re0,[D0AR.0++],A0.7
        D         SUBNV     D0Re0,[D0AR.0+D0ARI.0++],RD
        D         SUBNV     D0.7,D0Re0,A1LbP
        D         SUBNV     D0.7,D0Re0,D1.7
        DHCS      SUBVS     D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         SUBNV     D0.7,D0.7,A1LbP
        D         SUBNV     D0.7,D0.7,D1.7
        D         SUBNV     D0.7,D0.7,[D0AR.0+D0ARI.0++]
        D         SUBNV     D0.7,[D0AR.0],A1.7
        D         SUBNV     D0.7,[D0AR.0],A0.7
        D         SUBNV     D0.7,[D0AR.0++],RD
        D         SUBNV     D0.7,[D0AR.0+D0ARI.0++],A1.7
        D         SUBNV     D0.7,[D0AR.0+D0ARI.0++],A0.7
        D         SUBNV     D0.16,D0Re0,A1.7
        D         SUBNV     D0.16,D0Re0,RD
        D         SUBNV     D0.16,D0Re0,A0.7
        D         SUBNV     D0.16,D0.7,A1.7
        D         SUBNV     D0.16,D0.7,RD
        D         SUBNV     D0.16,D0.7,A0.7
        D         SUBNV     D0.16,[D0AR.0],RD
        D         SUBNV     D0.16,[D0AR.0++],A1.7
        D         SUBNV     D0.16,[D0AR.0++],A0.7
        D         SUBNV     D0.16,[D0AR.0+D0ARI.0++],RD
        D         SUBNV     D1Re0,D1Re0,A1LbP
        D         SUBNV     D1Re0,D1Re0,D0.7
        D         SUBNV     D1Re0,D1Re0,[D1BR.1++]
        D         SUBNV     D1Re0,D1.7,A1LbP
        D         SUBNV     D1Re0,D1.7,D0.7
        D         SUBNV     D1Re0,D1.7,[D1BR.1++]
        D         SUBNV     D1Re0,[D1BR.1],A1.7
        D         SUBNV     D1Re0,[D1BR.1],A0.7
        D         SUBNV     D1Re0,[D1BR.1++],A0FrP
        D         SUBNV     D1Re0,[D1BR.1+D1BRI.1++],A1.7
        D         SUBNV     D1Re0,[D1BR.1+D1BRI.1++],A0.7
        D         SUBNV     D1.7,D1Re0,A1.7
        D         SUBNV     D1.7,D1Re0,A0FrP
        D         SUBNV     D1.7,D1Re0,A0.7
        D         SUBNV     D1.7,D1.7,A1.7
        D         SUBNV     D1.7,D1.7,A0FrP
        D         SUBNV     D1.7,D1.7,A0.7
        D         SUBNV     D1.7,[D1BR.1],A0FrP
        D         SUBNV     D1.7,[D1BR.1++],A1.7
        D         SUBNV     D1.7,[D1BR.1++],A0.7
        D         SUBNV     D1.7,[D1BR.1+D1BRI.1++],A0FrP
        D         SUBNV     D1.16,D1Re0,A1LbP
        D         SUBNV     D1.16,D1Re0,D0.7
        D         SUBNV     D1.16,D1Re0,[D1BR.1++]
        D         SUBNV     D1.16,D1.7,A1LbP
        D         SUBNV     D1.16,D1.7,D0.7
        D         SUBNV     D1.16,D1.7,[D1BR.1++]
        D         SUBNV     D1.16,[D1BR.1],A1.7
        D         SUBNV     D1.16,[D1BR.1],A0.7
        D         SUBNV     D1.16,[D1BR.1++],A0FrP
        D         SUBNV     D1.16,[D1BR.1+D1BRI.1++],A1.7
        D         SUBNV     D1.16,[D1BR.1+D1BRI.1++],A0.7
        DLM       NEGS      D0Re0,D0Re0
        DLMO      NEGS      D0Re0,D0Re0
        DO        NEGS      D0Re0,A1LbP
        D         NEGS      D0Re0,A1.7
        DLM       SUBS      D0Re0,D0Re0,D0.7
        DMO       SUBS      D0Re0,D0Re0,A1.7
        DLMO      NEGS      D0Re0,A1.7
        DO        NEGS      D0Re0,D1Re0
        DM        NEGS      D0Re0,D1.7
        DLMO      NEGS      D0Re0,D1.7
        DM        NEGS      D0Re0,RD
        DO        SUBS      D0Re0,D0Re0,RD
        DLO       SUBS      D0Re0,D0Re0,RD
        DM        SUBS      D0Re0,D0Re0,AC0.0
        DMO       SUBS      D0Re0,D0Re0,AC0.0
        DM        SUBS      D0Re0,D0Re0,[D0AR.0++]
        DMO       SUBS      D0Re0,D0Re0,[D0AR.0++]
        DM        SUBS      D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DMO       SUBS      D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DM        SUBS      D0Re0,D0Re0,A0FrP
        DLO       NEGS      D0Re0,A0FrP
        DO        SUBS      D0Re0,D0Re0,A0.7
        DL        SUBS      D0Re0,D0.7,D0Re0
        DO        SUBS      D0Re0,D0.7,A1LbP
        DO        SUBS      D0Re0,D0.7,D0.7
        DO        SUBS      D0Re0,D0.7,D1Re0
        DM        SUBS      D0Re0,D0.7,[D0AR.0]
        DLO       SUBS      D0Re0,D0.7,[D0AR.0]
        DMO       SUBS      D0Re0,D0.7,AC0.0
        DMO       SUBS      D0Re0,D0.7,[D0AR.0++]
        DMO       SUBS      D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DM        SUBS      D0Re0,D0.7,A0.7
        DMO       SUBS      D0Re0,[D0AR.0],D0Re0
        D         SUBS      D0Re0,[D0AR.0],A1.7
        DMO       SUBS      D0Re0,[D0AR.0],A1.7
        DM        SUBS      D0Re0,[D0AR.0],D1.7
        DLM       SUBS      D0Re0,[D0AR.0],[D0AR.0]
        D         SUBS      D0Re0,[D0AR.0],AC0.0
        D         SUBS      D0Re0,[D0AR.0],[D0AR.0++]
        D         SUBS      D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         SUBS      D0Re0,[D0AR.0],A0FrP
        D         SUBS      D0Re0,[D0AR.0++],D0Re0
        D         SUBS      D0Re0,[D0AR.0++],A1LbP
        DL        SUBS      D0Re0,[D0AR.0++],D0.7
        D         SUBS      D0Re0,[D0AR.0++],D1Re0
        D         SUBS      D0Re0,[D0AR.0++],[D0AR.0]
        DMO       SUBS      D0Re0,[D0AR.0++],[D0AR.0]
        DLM       SUBS      D0Re0,[D0AR.0++],AC0.0
        DLM       SUBS      D0Re0,[D0AR.0++],[D0AR.0++]
        DLM       SUBS      D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DMO       SUBS      D0Re0,[D0AR.0++],A0FrP
        DLM       SUBS      D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        DMO       SUBS      D0Re0,[D0AR.0+D0ARI.0++],A1LbP
        DO        SUBS      D0Re0,[D0AR.0+D0ARI.0++],A1.7
        DMO       SUBS      D0Re0,[D0AR.0+D0ARI.0++],D1Re0
        DM        SUBS      D0Re0,[D0AR.0+D0ARI.0++],RD
        DLO       SUBS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DMO       SUBS      D0Re0,[D0AR.0+D0ARI.0++],AC0.0
        DMO       SUBS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DMO       SUBS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DM        SUBS      D0Re0,[D0AR.0+D0ARI.0++],A0.7
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T1
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T1
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 [A0FrP+A0FrP++],T1
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 [A0FrP+A0FrP++],T1
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0FrP++]
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0FrP++]
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0FrP++]
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0FrP++]
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T1
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T1
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 [A0FrP+A0.7++],T1
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 [A0FrP+A0.7++],T1
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0.7++]
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0.7++]
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0.7++]
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0.7++]
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T1
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T1
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 [A0FrP+A0.8++],T1
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 [A0FrP+A0.8++],T1
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0.8++]
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP+A0.8++]
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0.8++]
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP+A0.8++]
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 [A0FrP],T1
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 [A0FrP],T1
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 [A0FrP],T1
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 [A0FrP],T1
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP]
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP]
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP]
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP]
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 [A0FrP++],T1
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 [A0FrP++],T1
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 [A0FrP++],T1
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 [A0FrP++],T1
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP++]
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP++]
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP++]
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP++]
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 [A0FrP--],T1
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 [A0FrP--],T1
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 [A0FrP--],T1
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 [A0FrP--],T1
        DLWT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP--]
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A0FrP--]
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP--]
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A0FrP--]
        DL        NEGS      D0.7,D0Re0
        DLO       NEGS      D0.7,D0Re0
        DLM       NEGS      D0.7,A1LbP
        D         SUBS      D0.7,D0Re0,A1.7
        DL        NEGS      D0.7,D0.7
        DO        NEGS      D0.7,A1.7
        DLMO      SUBS      D0.7,D0Re0,D0.7
        DLM       NEGS      D0.7,D1Re0
        D         NEGS      D0.7,D1.7
        DMO       NEGS      D0.7,D1.7
        DM        SUBS      D0.7,D0Re0,RD
        DLM       NEGS      D0.7,RD
        DMO       NEGS      D0.7,RD
        D         SUBS      D0.7,D0Re0,AC0.0
        DO        SUBS      D0.7,D0Re0,AC0.0
        D         SUBS      D0.7,D0Re0,[D0AR.0++]
        DO        SUBS      D0.7,D0Re0,[D0AR.0++]
        D         SUBS      D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DO        SUBS      D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         SUBS      D0.7,D0Re0,A0FrP
        DMO       SUBS      D0.7,D0Re0,A0FrP
        DL        NEGS      D0.7,A0.7
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A0.7+A0FrP++],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A0.7+A0FrP++],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7+A0FrP++]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7+A0FrP++]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7+A0FrP++]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7+A0FrP++]
        D         SUBS      D0.7,D0.7,D0Re0
        D         SUBS      D0.7,D0.7,A1LbP
        DLMT      SUBS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T1
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T1
        DLT       SUBS      D0Re0,D0Re0,AC0.0 [A0.7+A0.7++],T1
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 [A0.7+A0.7++],T1
        DL        SUBS      D0.7,D0.7,D0.7
        D         SUBS      D0.7,D0.7,D1Re0
        D         SUBS      D0.7,D0.7,[D0AR.0]
        DMO       SUBS      D0.7,D0.7,[D0AR.0]
        DLM       SUBS      D0.7,D0.7,AC0.0
        DM        SUBS      D0.7,D0.7,[D0AR.0++]
        DLMWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7+A0.7++]
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7+A0.7++]
        DMWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7+A0.7++]
        DMOWT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7+A0.7++]
        DLM       SUBS      D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DMO       SUBS      D0.7,D0.7,A0FrP
        DMWT      SUBS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T1
        DMOWT     SUBS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T1
        DMWT      SUBS      D0Re0,D0Re0,AC0.0 [A0.7+A0.8++],T1
        DMOWT     SUBS      D0Re0,D0Re0,AC0.0 [A0.7+A0.8++],T1
        DMWT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7+A0.8++]
        DMOWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7+A0.8++]
        DMWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7+A0.8++]
        DMOWT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7+A0.8++]
        DLM       SUBS      D0.7,[D0AR.0],D0Re0
        DM        SUBS      D0.7,[D0AR.0],A1LbP
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A0.7],T1
        DLMOWT    SUBS      D0Re0,D0Re0,D0Re0 [A0.7],T1
        DLMWT     SUBS      D0Re0,D0Re0,AC0.0 [A0.7],T1
        DLMOWT    SUBS      D0Re0,D0Re0,AC0.0 [A0.7],T1
        DO        SUBS      D0.7,[D0AR.0],A1.7
        DMO       SUBS      D0.7,[D0AR.0],D1Re0
        DM        SUBS      D0.7,[D0AR.0],RD
        DLO       SUBS      D0.7,[D0AR.0],RD
        DLO       SUBS      D0.7,[D0AR.0],AC0.0
        DL        SUBS      D0.7,[D0AR.0],[D0AR.0++]
        DOWT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7]
        DLMOT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7]
        DLMT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7]
        DLMOT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7]
        DLO       SUBS      D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DO        SUBS      D0.7,[D0AR.0],A0.7
        DLO       SUBS      D0.7,[D0AR.0++],D0Re0
        DLT       SUBS      D0Re0,D0Re0,D0Re0 [A0.7++],T1
        DMOT      SUBS      D0Re0,D0Re0,D0Re0 [A0.7++],T1
        DMT       SUBS      D0Re0,D0Re0,AC0.0 [A0.7++],T1
        DMOT      SUBS      D0Re0,D0Re0,AC0.0 [A0.7++],T1
        DM        SUBS      D0.7,[D0AR.0++],D0.7
        DLO       SUBS      D0.7,[D0AR.0++],D0.7
        DO        SUBS      D0.7,[D0AR.0++],D1.7
        DO        SUBS      D0.7,[D0AR.0++],[D0AR.0]
        DM        SUBS      D0.7,[D0AR.0++],AC0.0
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7++]
        DLM       SUBS      D0.7,[D0AR.0++],[D0AR.0++]
        DMOWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7++]
        DWT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7++]
        DOWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7++]
        DM        SUBS      D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DM        SUBS      D0.7,[D0AR.0++],A0FrP
        DM        SUBS      D0.7,[D0AR.0+D0ARI.0++],D0Re0
        DM        SUBS      D0.7,[D0AR.0+D0ARI.0++],A1LbP
        DLM       SUBS      D0.7,[D0AR.0+D0ARI.0++],D0.7
        DM        SUBS      D0.7,[D0AR.0+D0ARI.0++],D1Re0
        D         SUBS      D0.7,[D0AR.0+D0ARI.0++],RD
        DMO       SUBS      D0.7,[D0AR.0+D0ARI.0++],RD
        DO        SUBS      D0.7,[D0AR.0+D0ARI.0++],AC0.0
        DO        SUBS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DO        SUBS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         SUBS      D0.7,[D0AR.0+D0ARI.0++],A0.7
        DLT       SUBS      D0Re0,D0Re0,D0Re0 [A0.7--],T1
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 [A0.7--],T1
        DLT       SUBS      D0Re0,D0Re0,AC0.0 [A0.7--],T1
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 [A0.7--],T1
        DLT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7--]
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.7--]
        DLT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7--]
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.7--]
        DLT       SUBS      D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T1
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T1
        DLT       SUBS      D0Re0,D0Re0,AC0.0 [A0.8+A0FrP++],T1
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 [A0.8+A0FrP++],T1
        DLT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8+A0FrP++]
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8+A0FrP++]
        DLT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8+A0FrP++]
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8+A0FrP++]
        DLT       SUBS      D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T1
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T1
        DLT       SUBS      D0Re0,D0Re0,AC0.0 [A0.8+A0.7++],T1
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 [A0.8+A0.7++],T1
        DLT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8+A0.7++]
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8+A0.7++]
        DLT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8+A0.7++]
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8+A0.7++]
        DLT       SUBS      D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T1
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T1
        DLT       SUBS      D0Re0,D0Re0,AC0.0 [A0.8+A0.8++],T1
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 [A0.8+A0.8++],T1
        DLT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8+A0.8++]
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8+A0.8++]
        DLT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8+A0.8++]
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8+A0.8++]
        DLT       SUBS      D0Re0,D0Re0,D0Re0 [A0.8],T1
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 [A0.8],T1
        DLT       SUBS      D0Re0,D0Re0,AC0.0 [A0.8],T1
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 [A0.8],T1
        DLT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8]
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8]
        DLT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8]
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8]
        DLT       SUBS      D0Re0,D0Re0,D0Re0 [A0.8++],T1
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 [A0.8++],T1
        DLT       SUBS      D0Re0,D0Re0,AC0.0 [A0.8++],T1
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 [A0.8++],T1
        DLT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8++]
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8++]
        DLT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8++]
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8++]
        DLT       SUBS      D0Re0,D0Re0,D0Re0 [A0.8--],T1
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 [A0.8--],T1
        DLT       SUBS      D0Re0,D0Re0,AC0.0 [A0.8--],T1
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 [A0.8--],T1
        DLT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8--]
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A0.8--]
        DLT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8--]
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A0.8--]
        DL        SUBS      [D0AW.0],D0Re0,D0Re0
        DLO       SUBS      [D0AW.0],D0Re0,D0Re0
        DL        NEGS      [D0AW.0],A1LbP
        D         SUBS      [D0AW.0],D0Re0,D0.7
        DL        SUBS      [D0AW.0],D0Re0,D0.7
        DO        NEGS      [D0AW.0],D0.7
        DLO       NEGS      [D0AW.0],A1.7
        DL        NEGS      [D0AW.0],D1Re0
        D         SUBS      [D0AW.0],D0Re0,D1.7
        DMO       SUBS      [D0AW.0],D0Re0,D1.7
        DM        SUBS      [D0AW.0],D0Re0,[D0AR.0]
        DLM       NEGS      [D0AW.0],[D0AR.0]
        DMO       NEGS      [D0AW.0],[D0AR.0]
        DLMO      NEGS      [D0AW.0],RD
        DLM       NEGS      [D0AW.0],AC0.0
        DLMO      NEGS      [D0AW.0],AC0.0
        DLM       NEGS      [D0AW.0],[D0AR.0++]
        DLMO      NEGS      [D0AW.0],[D0AR.0++]
        DLM       NEGS      [D0AW.0],[D0AR.0+D0ARI.0++]
        DLMO      NEGS      [D0AW.0],[D0AR.0+D0ARI.0++]
        DO        NEGS      [D0AW.0],A0FrP
        DM        NEGS      [D0AW.0],A0.7
        DLMO      NEGS      [D0AW.0],A0.7
        DLMO      SUBS      [D0AW.0],D0.7,D0Re0
        DM        SUBS      [D0AW.0],D0.7,A1.7
        DLMO      SUBS      [D0AW.0],D0.7,D0.7
        DMO       SUBS      [D0AW.0],D0.7,D1.7
        DO        SUBS      [D0AW.0],D0.7,RD
        DL        SUBS      [D0AW.0],D0.7,AC0.0
        DL        SUBS      [D0AW.0],D0.7,[D0AR.0++]
        DL        SUBS      [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        DO        SUBS      [D0AW.0],D0.7,A0FrP
        DL        SUBS      [D0AW.0],[D0AR.0],D0Re0
        DO        SUBS      [D0AW.0],[D0AR.0],A1LbP
        DO        SUBS      [D0AW.0],[D0AR.0],D0.7
        DO        SUBS      [D0AW.0],[D0AR.0],D1Re0
        DM        SUBS      [D0AW.0],[D0AR.0],[D0AR.0]
        DLO       SUBS      [D0AW.0],[D0AR.0],[D0AR.0]
        DMO       SUBS      [D0AW.0],[D0AR.0],AC0.0
        DMO       SUBS      [D0AW.0],[D0AR.0],[D0AR.0++]
        DMO       SUBS      [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DM        SUBS      [D0AW.0],[D0AR.0],A0.7
        DMO       SUBS      [D0AW.0],[D0AR.0++],D0Re0
        D         SUBS      [D0AW.0],[D0AR.0++],A1.7
        DMO       SUBS      [D0AW.0],[D0AR.0++],A1.7
        DM        SUBS      [D0AW.0],[D0AR.0++],D1.7
        DLM       SUBS      [D0AW.0],[D0AR.0++],[D0AR.0]
        D         SUBS      [D0AW.0],[D0AR.0++],AC0.0
        D         SUBS      [D0AW.0],[D0AR.0++],[D0AR.0++]
        D         SUBS      [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         SUBS      [D0AW.0],[D0AR.0++],A0FrP
        D         SUBS      [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        D         SUBS      [D0AW.0],[D0AR.0+D0ARI.0++],A1LbP
        DL        SUBS      [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        D         SUBS      [D0AW.0],[D0AR.0+D0ARI.0++],D1Re0
        D         SUBS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DMO       SUBS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLM       SUBS      [D0AW.0],[D0AR.0+D0ARI.0++],AC0.0
        DLM       SUBS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLM       SUBS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DMO       SUBS      [D0AW.0],[D0AR.0+D0ARI.0++],A0FrP
        DLMT      SUBS      D0Re0,D0Re0,D0Re0 [D0AW.0],T1
        DLMT      SUBS      D0Re0,D0Re0,AC0.0 [D0AW.0],T1
        DLMT      SUBS      D0Re0,D0Re0,D0Re0 T1,[D0AR.0]
        DLMT      SUBS      D0Re0,D0Re0,AC0.0 T1,[D0AR.0]
        DM        NEGS      [D0AW.0++],D0Re0
        DMO       NEGS      [D0AW.0++],D0Re0
        DM        NEGS      [D0AW.0++],A1LbP
        DLMO      NEGS      [D0AW.0++],A1LbP
        DM        NEGS      [D0AW.0++],A1.7
        DO        SUBS      [D0AW.0++],D0Re0,A1.7
        DLO       NEGS      [D0AW.0++],D0.7
        DM        NEGS      [D0AW.0++],D1Re0
        DLMO      NEGS      [D0AW.0++],D1Re0
        DO        NEGS      [D0AW.0++],D1.7
        D         NEGS      [D0AW.0++],RD
        DLM       SUBS      [D0AW.0++],D0Re0,[D0AR.0]
        DMO       SUBS      [D0AW.0++],D0Re0,RD
        DLMO      NEGS      [D0AW.0++],[D0AR.0]
        DLM       SUBS      [D0AW.0++],D0Re0,AC0.0
        DLMO      SUBS      [D0AW.0++],D0Re0,AC0.0
        DLM       SUBS      [D0AW.0++],D0Re0,[D0AR.0++]
        DLMO      SUBS      [D0AW.0++],D0Re0,[D0AR.0++]
        DLM       SUBS      [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DLMO      SUBS      [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DO        SUBS      [D0AW.0++],D0Re0,A0FrP
        DM        SUBS      [D0AW.0++],D0Re0,A0.7
        DLO       NEGS      [D0AW.0++],A0.7
        DLMT      SUBS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T1
        DLMOT     SUBS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T1
        DLMT      SUBS      D0Re0,D0Re0,AC0.0 [A1LbP+A1LbP++],T1
        DLMOT     SUBS      D0Re0,D0Re0,AC0.0 [A1LbP+A1LbP++],T1
        DLMT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1LbP++]
        DLMOT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1LbP++]
        DLMT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1LbP++]
        DLMOT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1LbP++]
        DLO       SUBS      [D0AW.0++],D0.7,D0Re0
        DLT       SUBS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T1
        DMOT      SUBS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T1
        DMT       SUBS      D0Re0,D0Re0,AC0.0 [A1LbP+A1.7++],T1
        DMOT      SUBS      D0Re0,D0Re0,AC0.0 [A1LbP+A1.7++],T1
        DM        SUBS      [D0AW.0++],D0.7,D0.7
        DLO       SUBS      [D0AW.0++],D0.7,D0.7
        DO        SUBS      [D0AW.0++],D0.7,D1.7
        DO        SUBS      [D0AW.0++],D0.7,[D0AR.0]
        DM        SUBS      [D0AW.0++],D0.7,AC0.0
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1.7++]
        DLM       SUBS      [D0AW.0++],D0.7,[D0AR.0++]
        DMOWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1.7++]
        DWT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1.7++]
        DOWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1.7++]
        DM        SUBS      [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DM        SUBS      [D0AW.0++],D0.7,A0FrP
        DWT       SUBS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T1
        DOWT      SUBS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T1
        DWT       SUBS      D0Re0,D0Re0,AC0.0 [A1LbP+A1.8++],T1
        DOWT      SUBS      D0Re0,D0Re0,AC0.0 [A1LbP+A1.8++],T1
        DWT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1.8++]
        DOWT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP+A1.8++]
        DWT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1.8++]
        DOWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP+A1.8++]
        DM        SUBS      [D0AW.0++],[D0AR.0],D0Re0
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1LbP],T1
        DLMWT     SUBS      D0Re0,D0Re0,D0Re0 [A1LbP],T1
        DLOWT     SUBS      D0Re0,D0Re0,D0Re0 [A1LbP],T1
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 [A1LbP],T1
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 [A1LbP],T1
        DLM       SUBS      [D0AW.0++],[D0AR.0],D0.7
        DM        SUBS      [D0AW.0++],[D0AR.0],D1Re0
        D         SUBS      [D0AW.0++],[D0AR.0],RD
        DMO       SUBS      [D0AW.0++],[D0AR.0],RD
        DMO       SUBS      [D0AW.0++],[D0AR.0],AC0.0
        DMWT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP]
        DLMO      SUBS      [D0AW.0++],[D0AR.0],[D0AR.0++]
        DLWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP]
        DLOWT     SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP]
        DMO       SUBS      [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DM        SUBS      [D0AW.0++],[D0AR.0],A0.7
        DMO       SUBS      [D0AW.0++],[D0AR.0++],D0Re0
        DMWT      SUBS      D0Re0,D0Re0,D0Re0 [A1LbP++],T1
        DMO       SUBS      [D0AW.0++],[D0AR.0++],A1LbP
        DWT       SUBS      D0Re0,D0Re0,AC0.0 [A1LbP++],T1
        DOWT      SUBS      D0Re0,D0Re0,AC0.0 [A1LbP++],T1
        D         SUBS      [D0AW.0++],[D0AR.0++],A1.7
        DMO       SUBS      [D0AW.0++],[D0AR.0++],A1.7
        DM        SUBS      [D0AW.0++],[D0AR.0++],D1.7
        DLM       SUBS      [D0AW.0++],[D0AR.0++],[D0AR.0]
        DM        SUBS      [D0AW.0++],[D0AR.0++],AC0.0
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP++]
        DLM       SUBS      [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DMOWT     SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP++]
        DWT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP++]
        DOWT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP++]
        DM        SUBS      [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DM        SUBS      [D0AW.0++],[D0AR.0++],A0FrP
        DM        SUBS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DM        SUBS      [D0AW.0++],[D0AR.0+D0ARI.0++],A1LbP
        DLM       SUBS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DM        SUBS      [D0AW.0++],[D0AR.0+D0ARI.0++],D1Re0
        D         SUBS      [D0AW.0++],[D0AR.0+D0ARI.0++],RD
        DMO       SUBS      [D0AW.0++],[D0AR.0+D0ARI.0++],RD
        DO        SUBS      [D0AW.0++],[D0AR.0+D0ARI.0++],AC0.0
        DO        SUBS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DO        SUBS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         SUBS      [D0AW.0++],[D0AR.0+D0ARI.0++],A0.7
        DLT       SUBS      D0Re0,D0Re0,D0Re0 [A1LbP--],T1
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 [A1LbP--],T1
        DLT       SUBS      D0Re0,D0Re0,AC0.0 [A1LbP--],T1
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 [A1LbP--],T1
        DLT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP--]
        DLOT      SUBS      D0Re0,D0Re0,D0Re0 T1,[A1LbP--]
        DLT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP--]
        DLOT      SUBS      D0Re0,D0Re0,AC0.0 T1,[A1LbP--]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [D0AW.0++],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [D0AW.0++],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[D0AR.0++]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[D0AR.0++]
        DL        SUBS      [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DLO       SUBS      [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DL        NEGS      [D0AW.0+D0AWI.0++],A1LbP
        D         SUBS      [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DL        SUBS      [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DO        NEGS      [D0AW.0+D0AWI.0++],D0.7
        DLO       NEGS      [D0AW.0+D0AWI.0++],A1.7
        DL        NEGS      [D0AW.0+D0AWI.0++],D1Re0
        D         SUBS      [D0AW.0+D0AWI.0++],D0Re0,D1.7
        DMO       SUBS      [D0AW.0+D0AWI.0++],D0Re0,D1.7
        DM        SUBS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DLM       NEGS      [D0AW.0+D0AWI.0++],[D0AR.0]
        DMO       NEGS      [D0AW.0+D0AWI.0++],[D0AR.0]
        D         SUBS      [D0AW.0+D0AWI.0++],D0Re0,AC0.0
        DO        SUBS      [D0AW.0+D0AWI.0++],D0Re0,AC0.0
        D         SUBS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DO        SUBS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        D         SUBS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DO        SUBS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        D         SUBS      [D0AW.0+D0AWI.0++],D0Re0,A0FrP
        DMO       SUBS      [D0AW.0+D0AWI.0++],D0Re0,A0FrP
        DL        NEGS      [D0AW.0+D0AWI.0++],A0.7
        D         SUBS      [D0AW.0+D0AWI.0++],D0.7,D0Re0
        D         SUBS      [D0AW.0+D0AWI.0++],D0.7,A1LbP
        DL        SUBS      [D0AW.0+D0AWI.0++],D0.7,D0.7
        D         SUBS      [D0AW.0+D0AWI.0++],D0.7,D1Re0
        D         SUBS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DMO       SUBS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DO        SUBS      [D0AW.0+D0AWI.0++],D0.7,AC0.0
        DO        SUBS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DO        SUBS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        D         SUBS      [D0AW.0+D0AWI.0++],D0.7,A0.7
        DO        SUBS      [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DMO       SUBS      [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0],D1.7
        DL        SUBS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0],AC0.0
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0],A0FrP
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0++],A1LbP
        DL        SUBS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0++],D1Re0
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DMO       SUBS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DO        SUBS      [D0AW.0+D0AWI.0++],[D0AR.0++],AC0.0
        DO        SUBS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DO        SUBS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0++],A0.7
        DO        SUBS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DMO       SUBS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D1.7
        DL        SUBS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],AC0.0
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         SUBS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A0FrP
        DT        SUBS      D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [D0AW.0+D0AWI.0++],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[D0AR.0+D0ARI.0++]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[D0AR.0+D0ARI.0++]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.7+A1LbP++],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.7+A1LbP++],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7+A1LbP++]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7+A1LbP++]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7+A1LbP++]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7+A1LbP++]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.7+A1.7++],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.7+A1.7++],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7+A1.7++]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7+A1.7++]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7+A1.7++]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7+A1.7++]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.7+A1.8++],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.7+A1.8++],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7+A1.8++]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7+A1.8++]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7+A1.8++]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7+A1.8++]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.7],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.7],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.7],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.7],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.7++],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.7++],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.7++],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.7++],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7++]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7++]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7++]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7++]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.7--],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.7--],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.7--],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.7--],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7--]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.7--]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7--]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.7--]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.8+A1LbP++],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.8+A1LbP++],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8+A1LbP++]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8+A1LbP++]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8+A1LbP++]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8+A1LbP++]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.8+A1.7++],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.8+A1.7++],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8+A1.7++]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8+A1.7++]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8+A1.7++]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8+A1.7++]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.8+A1.8++],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.8+A1.8++],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8+A1.8++]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8+A1.8++]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8+A1.8++]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8+A1.8++]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.8],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.8],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.8],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.8],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.8++],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.8++],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.8++],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.8++],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8++]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8++]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8++]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8++]
        DT        SUBS      D0Re0,D0Re0,D0Re0 [A1.8--],T1
        DOT       SUBS      D0Re0,D0Re0,D0Re0 [A1.8--],T1
        DT        SUBS      D0Re0,D0Re0,AC0.0 [A1.8--],T1
        DOT       SUBS      D0Re0,D0Re0,AC0.0 [A1.8--],T1
        DT        SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8--]
        DOT       SUBS      D0Re0,D0Re0,D0Re0 T1,[A1.8--]
        DT        SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8--]
        DOT       SUBS      D0Re0,D0Re0,AC0.0 T1,[A1.8--]
        D         SUBS      D1Re0,D1Re0,D1Re0
        DO        SUBS      D1Re0,D1Re0,D1Re0
        D         SUBS      D1Re0,D1Re0,A1LbP
        DMO       SUBS      D1Re0,D1Re0,A1LbP
        DM        SUBS      D1Re0,D1Re0,D1.7
        DLM       NEGS      D1Re0,D1.7
        DMO       NEGS      D1Re0,D1.7
        D         SUBS      D1Re0,D1Re0,D0Re0
        DMO       SUBS      D1Re0,D1Re0,D0Re0
        DL        NEGS      D1Re0,D0.7
        D         SUBS      D1Re0,D1Re0,RD
        DMO       SUBS      D1Re0,D1Re0,RD
        DM        NEGS      D1Re0,AC1.0
        DMO       NEGS      D1Re0,AC1.0
        DM        NEGS      D1Re0,A0FrP
        DLMO      NEGS      D1Re0,A0FrP
        DLM       NEGS      D1Re0,[D1BR.1]
        DLMO      NEGS      D1Re0,[D1BR.1]
        DLM       NEGS      D1Re0,[D1BR.1++]
        DLMO      NEGS      D1Re0,[D1BR.1++]
        DM        NEGS      D1Re0,A0.7
        DO        SUBS      D1Re0,D1Re0,A0.7
        DLO       NEGS      D1Re0,[D1BR.1+D1BRI.1++]
        DLM       SUBS      D1Re0,D1.7,D1Re0
        DMO       SUBS      D1Re0,D1.7,A1LbP
        DO        SUBS      D1Re0,D1.7,A1.7
        DMO       SUBS      D1Re0,D1.7,D0Re0
        DMO       SUBS      D1Re0,D1.7,RD
        DLMO      SUBS      D1Re0,D1.7,AC1.0
        DLM       SUBS      D1Re0,D1.7,[D1BR.1]
        DLM       SUBS      D1Re0,D1.7,[D1BR.1++]
        DM        SUBS      D1Re0,D1.7,A0.7
        DLMO      SUBS      D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DLMO      SUBS      D1Re0,[D1BR.1],D1Re0
        DM        SUBS      D1Re0,[D1BR.1],A1.7
        DLMO      SUBS      D1Re0,[D1BR.1],D1.7
        DMO       SUBS      D1Re0,[D1BR.1],D0.7
        DLM       SUBS      D1Re0,[D1BR.1],AC1.0
        DMO       SUBS      D1Re0,[D1BR.1],A0FrP
        DLMO      SUBS      D1Re0,[D1BR.1],[D1BR.1]
        DLMO      SUBS      D1Re0,[D1BR.1],[D1BR.1++]
        DO        SUBS      D1Re0,[D1BR.1],A0.7
        DLM       SUBS      D1Re0,[D1BR.1++],D1Re0
        DMO       SUBS      D1Re0,[D1BR.1++],A1LbP
        DO        SUBS      D1Re0,[D1BR.1++],A1.7
        DMO       SUBS      D1Re0,[D1BR.1++],D0Re0
        DMO       SUBS      D1Re0,[D1BR.1++],RD
        DLMO      SUBS      D1Re0,[D1BR.1++],AC1.0
        DLM       SUBS      D1Re0,[D1BR.1++],[D1BR.1]
        DLM       SUBS      D1Re0,[D1BR.1++],[D1BR.1++]
        DM        SUBS      D1Re0,[D1BR.1++],A0.7
        DLMO      SUBS      D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLMO      SUBS      D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DM        SUBS      D1Re0,[D1BR.1+D1BRI.1++],A1.7
        DLMO      SUBS      D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DMO       SUBS      D1Re0,[D1BR.1+D1BRI.1++],D0.7
        DLM       SUBS      D1Re0,[D1BR.1+D1BRI.1++],AC1.0
        DMO       SUBS      D1Re0,[D1BR.1+D1BRI.1++],A0FrP
        DLMO      SUBS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLMO      SUBS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DO        SUBS      D1Re0,[D1BR.1+D1BRI.1++],A0.7
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T1
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T1
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 [A0FrP+A0FrP++],T1
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 [A0FrP+A0FrP++],T1
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0FrP++]
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0FrP++]
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0FrP++]
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0FrP++]
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T1
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T1
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 [A0FrP+A0.7++],T1
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 [A0FrP+A0.7++],T1
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0.7++]
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0.7++]
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0.7++]
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0.7++]
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T1
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T1
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 [A0FrP+A0.8++],T1
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 [A0FrP+A0.8++],T1
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0.8++]
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP+A0.8++]
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0.8++]
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP+A0.8++]
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 [A0FrP],T1
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 [A0FrP],T1
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 [A0FrP],T1
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 [A0FrP],T1
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP]
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP]
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP]
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP]
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 [A0FrP++],T1
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 [A0FrP++],T1
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 [A0FrP++],T1
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 [A0FrP++],T1
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP++]
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP++]
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP++]
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP++]
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 [A0FrP--],T1
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 [A0FrP--],T1
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 [A0FrP--],T1
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 [A0FrP--],T1
        DMWT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP--]
        DMOWT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0FrP--]
        DMWT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP--]
        DMOWT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0FrP--]
        DM        NEGS      D1.7,D1Re0
        DMO       NEGS      D1.7,D1Re0
        DM        NEGS      D1.7,A1LbP
        DLMO      NEGS      D1.7,A1LbP
        DM        NEGS      D1.7,A1.7
        DO        SUBS      D1.7,D1Re0,A1.7
        DLO       NEGS      D1.7,D1.7
        DM        NEGS      D1.7,D0Re0
        DLMO      NEGS      D1.7,D0Re0
        DO        NEGS      D1.7,D0.7
        DM        NEGS      D1.7,RD
        DLO       NEGS      D1.7,RD
        DLM       SUBS      D1.7,D1Re0,AC1.0
        DLMO      SUBS      D1.7,D1Re0,AC1.0
        DO        SUBS      D1.7,D1Re0,A0FrP
        DM        SUBS      D1.7,D1Re0,[D1BR.1]
        DMO       SUBS      D1.7,D1Re0,[D1BR.1]
        DM        SUBS      D1.7,D1Re0,[D1BR.1++]
        DMO       SUBS      D1.7,D1Re0,[D1BR.1++]
        D         NEGS      D1.7,[D1BR.1+D1BRI.1++]
        DL        NEGS      D1.7,A0.7
        DMO       SUBS      D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DLMO      NEGS      D1.7,[D1BR.1+D1BRI.1++]
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T1
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T1
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 [A0.7+A0FrP++],T1
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 [A0.7+A0FrP++],T1
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7+A0FrP++]
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7+A0FrP++]
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7+A0FrP++]
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7+A0FrP++]
        DLO       SUBS      D1.7,D1.7,D1Re0
        DLT       SUBS      D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A0.7+A0.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A0.7+A0.7++],T1
        DM        SUBS      D1.7,D1.7,D1.7
        DLO       SUBS      D1.7,D1.7,D1.7
        DO        SUBS      D1.7,D1.7,D0.7
        DLO       SUBS      D1.7,D1.7,RD
        DLO       SUBS      D1.7,D1.7,AC1.0
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7+A0.7++]
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7+A0.7++]
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7+A0.7++]
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7+A0.7++]
        DL        SUBS      D1.7,D1.7,[D1BR.1]
        DL        SUBS      D1.7,D1.7,[D1BR.1++]
        DM        SUBS      D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DLO       SUBS      D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T1
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T1
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 [A0.7+A0.8++],T1
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 [A0.7+A0.8++],T1
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7+A0.8++]
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7+A0.8++]
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7+A0.8++]
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7+A0.8++]
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 [A0.7],T1
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 [A0.7],T1
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 [A0.7],T1
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 [A0.7],T1
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7]
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7]
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7]
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7]
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 [A0.7++],T1
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 [A0.7++],T1
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 [A0.7++],T1
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 [A0.7++],T1
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7++]
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7++]
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7++]
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7++]
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 [A0.7--],T1
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 [A0.7--],T1
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 [A0.7--],T1
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 [A0.7--],T1
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7--]
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.7--]
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7--]
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.7--]
        DLO       SUBS      D1.7,[D1BR.1],D1Re0
        DM        SUBS      D1.7,[D1BR.1],D1.7
        DLO       SUBS      D1.7,[D1BR.1],D1.7
        DO        SUBS      D1.7,[D1BR.1],D0.7
        DL        SUBS      D1.7,[D1BR.1],AC1.0
        DO        SUBS      D1.7,[D1BR.1],A0FrP
        DLO       SUBS      D1.7,[D1BR.1],[D1BR.1]
        DLO       SUBS      D1.7,[D1BR.1],[D1BR.1++]
        DO        SUBS      D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        SUBS      D1.7,[D1BR.1++],D1Re0
        DO        SUBS      D1.7,[D1BR.1++],A1LbP
        DO        SUBS      D1.7,[D1BR.1++],D1.7
        DO        SUBS      D1.7,[D1BR.1++],D0Re0
        DO        SUBS      D1.7,[D1BR.1++],RD
        DLO       SUBS      D1.7,[D1BR.1++],AC1.0
        DL        SUBS      D1.7,[D1BR.1++],[D1BR.1]
        DL        SUBS      D1.7,[D1BR.1++],[D1BR.1++]
        DM        SUBS      D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLO       SUBS      D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLO       SUBS      D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DM        SUBS      D1.7,[D1BR.1+D1BRI.1++],D1.7
        DLO       SUBS      D1.7,[D1BR.1+D1BRI.1++],D1.7
        DO        SUBS      D1.7,[D1BR.1+D1BRI.1++],D0.7
        DL        SUBS      D1.7,[D1BR.1+D1BRI.1++],AC1.0
        DO        SUBS      D1.7,[D1BR.1+D1BRI.1++],A0FrP
        DLO       SUBS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLO       SUBS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DO        SUBS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A0.8+A0FrP++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A0.8+A0FrP++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8+A0FrP++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8+A0FrP++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8+A0FrP++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8+A0FrP++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A0.8+A0.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A0.8+A0.7++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8+A0.7++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8+A0.7++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8+A0.7++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8+A0.7++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A0.8+A0.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A0.8+A0.8++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8+A0.8++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8+A0.8++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8+A0.8++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8+A0.8++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A0.8],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A0.8],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A0.8],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A0.8],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A0.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A0.8++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A0.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A0.8++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A0.8--],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A0.8--],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A0.8--],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A0.8--],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8--]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A0.8--]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8--]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A0.8--]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1LbP+A1LbP++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1LbP+A1LbP++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1LbP++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1LbP++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1LbP++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1LbP++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1LbP+A1.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1LbP+A1.7++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1.7++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1.7++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1.7++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1.7++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1LbP+A1.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1LbP+A1.8++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1.8++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP+A1.8++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1.8++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP+A1.8++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1LbP],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1LbP],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1LbP],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1LbP],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1LbP++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1LbP++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1LbP++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1LbP++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1LbP--],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1LbP--],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1LbP--],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1LbP--],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP--]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1LbP--]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP--]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1LbP--]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.7+A1LbP++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.7+A1LbP++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7+A1LbP++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7+A1LbP++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7+A1LbP++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7+A1LbP++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.7+A1.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.7+A1.7++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7+A1.7++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7+A1.7++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7+A1.7++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7+A1.7++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.7+A1.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.7+A1.8++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7+A1.8++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7+A1.8++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7+A1.8++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7+A1.8++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.7],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.7],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.7],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.7],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.7++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.7++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.7--],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.7--],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.7--],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.7--],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7--]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.7--]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7--]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.7--]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.8+A1LbP++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.8+A1LbP++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8+A1LbP++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8+A1LbP++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8+A1LbP++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8+A1LbP++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.8+A1.7++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.8+A1.7++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8+A1.7++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8+A1.7++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8+A1.7++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8+A1.7++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.8+A1.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.8+A1.8++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8+A1.8++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8+A1.8++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8+A1.8++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8+A1.8++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.8],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.8],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.8],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.8],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.8++],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.8++],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.8++],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8++]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8++]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8++]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8++]
        DMT       SUBS      D1Re0,D1Re0,D1Re0 [A1.8--],T1
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 [A1.8--],T1
        DMT       SUBS      D1Re0,D1Re0,AC1.0 [A1.8--],T1
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 [A1.8--],T1
        DMT       SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8--]
        DMOT      SUBS      D1Re0,D1Re0,D1Re0 T1,[A1.8--]
        DMT       SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8--]
        DMOT      SUBS      D1Re0,D1Re0,AC1.0 T1,[A1.8--]
        DM        SUBS      [D1BW.1],D1Re0,D1Re0
        DMO       SUBS      [D1BW.1],D1Re0,D1Re0
        DM        SUBS      [D1BW.1],D1Re0,A1LbP
        DLO       NEGS      [D1BW.1],A1LbP
        DM        NEGS      [D1BW.1],D1.7
        DO        SUBS      [D1BW.1],D1Re0,D1.7
        DLO       SUBS      [D1BW.1],D1Re0,D1.7
        DM        SUBS      [D1BW.1],D1Re0,D0Re0
        DLO       NEGS      [D1BW.1],D0Re0
        DO        SUBS      [D1BW.1],D1Re0,D0.7
        DM        SUBS      [D1BW.1],D1Re0,RD
        DLO       NEGS      [D1BW.1],RD
        DLM       SUBS      [D1BW.1],D1Re0,AC1.0
        DLMO      SUBS      [D1BW.1],D1Re0,AC1.0
        DO        SUBS      [D1BW.1],D1Re0,A0FrP
        DM        SUBS      [D1BW.1],D1Re0,[D1BR.1]
        DMO       SUBS      [D1BW.1],D1Re0,[D1BR.1]
        DM        SUBS      [D1BW.1],D1Re0,[D1BR.1++]
        DMO       SUBS      [D1BW.1],D1Re0,[D1BR.1++]
        D         NEGS      [D1BW.1],[D1BR.1+D1BRI.1++]
        DL        NEGS      [D1BW.1],A0.7
        DMO       SUBS      [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DLMO      NEGS      [D1BW.1],[D1BR.1+D1BRI.1++]
        DLO       SUBS      [D1BW.1],D1.7,D1Re0
        DM        SUBS      [D1BW.1],D1.7,D1.7
        DLO       SUBS      [D1BW.1],D1.7,D1.7
        DO        SUBS      [D1BW.1],D1.7,D0.7
        DL        SUBS      [D1BW.1],D1.7,AC1.0
        DO        SUBS      [D1BW.1],D1.7,A0FrP
        DLO       SUBS      [D1BW.1],D1.7,[D1BR.1]
        DLO       SUBS      [D1BW.1],D1.7,[D1BR.1++]
        DO        SUBS      [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DLT       SUBS      D1Re0,D1Re0,D1Re0 [D1BW.1],T1
        DLT       SUBS      D1Re0,D1Re0,AC1.0 [D1BW.1],T1
        DLT       SUBS      D1Re0,D1Re0,D1Re0 T1,[D1BR.1]
        DLT       SUBS      D1Re0,D1Re0,AC1.0 T1,[D1BR.1]
        DL        SUBS      [D1BW.1],[D1BR.1],D1Re0
        DO        SUBS      [D1BW.1],[D1BR.1],A1LbP
        DO        SUBS      [D1BW.1],[D1BR.1],D1.7
        DO        SUBS      [D1BW.1],[D1BR.1],D0Re0
        DO        SUBS      [D1BW.1],[D1BR.1],RD
        DLO       SUBS      [D1BW.1],[D1BR.1],AC1.0
        DL        SUBS      [D1BW.1],[D1BR.1],[D1BR.1]
        DL        SUBS      [D1BW.1],[D1BR.1],[D1BR.1++]
        DM        SUBS      [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLO       SUBS      [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLO       SUBS      [D1BW.1],[D1BR.1++],D1Re0
        DM        SUBS      [D1BW.1],[D1BR.1++],D1.7
        DLO       SUBS      [D1BW.1],[D1BR.1++],D1.7
        DO        SUBS      [D1BW.1],[D1BR.1++],D0.7
        DL        SUBS      [D1BW.1],[D1BR.1++],AC1.0
        DO        SUBS      [D1BW.1],[D1BR.1++],A0FrP
        DLO       SUBS      [D1BW.1],[D1BR.1++],[D1BR.1]
        DLO       SUBS      [D1BW.1],[D1BR.1++],[D1BR.1++]
        DO        SUBS      [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        SUBS      [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DO        SUBS      [D1BW.1],[D1BR.1+D1BRI.1++],A1LbP
        DO        SUBS      [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DO        SUBS      [D1BW.1],[D1BR.1+D1BRI.1++],D0Re0
        DMO       SUBS      [D1BW.1],[D1BR.1+D1BRI.1++],RD
        DLMO      SUBS      [D1BW.1],[D1BR.1+D1BRI.1++],AC1.0
        DLM       SUBS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLM       SUBS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DM        SUBS      [D1BW.1],[D1BR.1+D1BRI.1++],A0.7
        DLMO      SUBS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLM       NEGS      [D1BW.1++],D1Re0
        DLMO      NEGS      [D1BW.1++],D1Re0
        DO        NEGS      [D1BW.1++],A1LbP
        D         NEGS      [D1BW.1++],A1.7
        DLM       SUBS      [D1BW.1++],D1Re0,D1.7
        DMO       SUBS      [D1BW.1++],D1Re0,A1.7
        DLMO      NEGS      [D1BW.1++],A1.7
        DO        NEGS      [D1BW.1++],D0Re0
        DM        NEGS      [D1BW.1++],D0.7
        DLMO      NEGS      [D1BW.1++],D0.7
        DO        NEGS      [D1BW.1++],RD
        DM        NEGS      [D1BW.1++],AC1.0
        DMO       NEGS      [D1BW.1++],AC1.0
        DM        NEGS      [D1BW.1++],A0FrP
        DLMO      NEGS      [D1BW.1++],A0FrP
        DLM       NEGS      [D1BW.1++],[D1BR.1]
        DLMO      NEGS      [D1BW.1++],[D1BR.1]
        DLM       NEGS      [D1BW.1++],[D1BR.1++]
        DLMO      NEGS      [D1BW.1++],[D1BR.1++]
        DM        NEGS      [D1BW.1++],A0.7
        DO        SUBS      [D1BW.1++],D1Re0,A0.7
        DLO       NEGS      [D1BW.1++],[D1BR.1+D1BRI.1++]
        DLM       SUBS      [D1BW.1++],D1.7,D1Re0
        DMO       SUBS      [D1BW.1++],D1.7,A1LbP
        DO        SUBS      [D1BW.1++],D1.7,A1.7
        DMO       SUBS      [D1BW.1++],D1.7,D0Re0
        DMO       SUBS      [D1BW.1++],D1.7,RD
        DLMO      SUBS      [D1BW.1++],D1.7,AC1.0
        DLM       SUBS      [D1BW.1++],D1.7,[D1BR.1]
        DLM       SUBS      [D1BW.1++],D1.7,[D1BR.1++]
        DM        SUBS      [D1BW.1++],D1.7,A0.7
        DLMO      SUBS      [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 [D1BW.1++],T1
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 [D1BW.1++],T1
        DLMOT     SUBS      D1Re0,D1Re0,D1Re0 T1,[D1BR.1++]
        DLMOT     SUBS      D1Re0,D1Re0,AC1.0 T1,[D1BR.1++]
        DLMO      SUBS      [D1BW.1++],[D1BR.1],D1Re0
        DM        SUBS      [D1BW.1++],[D1BR.1],A1.7
        DLMO      SUBS      [D1BW.1++],[D1BR.1],D1.7
        DMO       SUBS      [D1BW.1++],[D1BR.1],D0.7
        DLM       SUBS      [D1BW.1++],[D1BR.1],AC1.0
        DMO       SUBS      [D1BW.1++],[D1BR.1],A0FrP
        DLMO      SUBS      [D1BW.1++],[D1BR.1],[D1BR.1]
        DLMO      SUBS      [D1BW.1++],[D1BR.1],[D1BR.1++]
        DO        SUBS      [D1BW.1++],[D1BR.1],A0.7
        DLM       SUBS      [D1BW.1++],[D1BR.1++],D1Re0
        DMO       SUBS      [D1BW.1++],[D1BR.1++],A1LbP
        DO        SUBS      [D1BW.1++],[D1BR.1++],A1.7
        DMO       SUBS      [D1BW.1++],[D1BR.1++],D0Re0
        DMO       SUBS      [D1BW.1++],[D1BR.1++],RD
        DLMO      SUBS      [D1BW.1++],[D1BR.1++],AC1.0
        DLM       SUBS      [D1BW.1++],[D1BR.1++],[D1BR.1]
        DLM       SUBS      [D1BW.1++],[D1BR.1++],[D1BR.1++]
        DM        SUBS      [D1BW.1++],[D1BR.1++],A0.7
        DLMO      SUBS      [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLMO      SUBS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        DM        SUBS      [D1BW.1++],[D1BR.1+D1BRI.1++],A1.7
        DLMO      SUBS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        DMO       SUBS      [D1BW.1++],[D1BR.1+D1BRI.1++],D0.7
        DLM       SUBS      [D1BW.1++],[D1BR.1+D1BRI.1++],AC1.0
        DMO       SUBS      [D1BW.1++],[D1BR.1+D1BRI.1++],A0FrP
        DLMO      SUBS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLMO      SUBS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DO        SUBS      [D1BW.1++],[D1BR.1+D1BRI.1++],A0.7
        DM        NEGS      [D1BW.1+D1BWI.1++],D1Re0
        DMO       NEGS      [D1BW.1+D1BWI.1++],D1Re0
        DM        NEGS      [D1BW.1+D1BWI.1++],A1LbP
        DLMO      NEGS      [D1BW.1+D1BWI.1++],A1LbP
        DM        NEGS      [D1BW.1+D1BWI.1++],A1.7
        DO        SUBS      [D1BW.1+D1BWI.1++],D1Re0,A1.7
        DLO       NEGS      [D1BW.1+D1BWI.1++],D1.7
        DM        NEGS      [D1BW.1+D1BWI.1++],D0Re0
        DLMO      NEGS      [D1BW.1+D1BWI.1++],D0Re0
        DO        NEGS      [D1BW.1+D1BWI.1++],D0.7
        DM        NEGS      [D1BW.1+D1BWI.1++],RD
        DLMO      NEGS      [D1BW.1+D1BWI.1++],RD
        DLM       NEGS      [D1BW.1+D1BWI.1++],AC1.0
        DLMO      NEGS      [D1BW.1+D1BWI.1++],AC1.0
        DO        NEGS      [D1BW.1+D1BWI.1++],A0FrP
        DM        NEGS      [D1BW.1+D1BWI.1++],[D1BR.1]
        DMO       NEGS      [D1BW.1+D1BWI.1++],[D1BR.1]
        DM        NEGS      [D1BW.1+D1BWI.1++],[D1BR.1++]
        DMO       NEGS      [D1BW.1+D1BWI.1++],[D1BR.1++]
        D         NEGS      [D1BW.1+D1BWI.1++],A0.7
        DLM       SUBS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DMO       SUBS      [D1BW.1+D1BWI.1++],D1Re0,A0.7
        DLMO      NEGS      [D1BW.1+D1BWI.1++],A0.7
        DLMO      SUBS      [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DM        SUBS      [D1BW.1+D1BWI.1++],D1.7,A1.7
        DLMO      SUBS      [D1BW.1+D1BWI.1++],D1.7,D1.7
        DMO       SUBS      [D1BW.1+D1BWI.1++],D1.7,D0.7
        DLM       SUBS      [D1BW.1+D1BWI.1++],D1.7,AC1.0
        DMO       SUBS      [D1BW.1+D1BWI.1++],D1.7,A0FrP
        DLMO      SUBS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        DLMO      SUBS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        DO        SUBS      [D1BW.1+D1BWI.1++],D1.7,A0.7
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T1
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 [D1BW.1+D1BWI.1++],T1
        DLMT      SUBS      D1Re0,D1Re0,D1Re0 T1,[D1BR.1+D1BRI.1++]
        DLMT      SUBS      D1Re0,D1Re0,AC1.0 T1,[D1BR.1+D1BRI.1++]
        DLM       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        DMO       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1],A1LbP
        DO        SUBS      [D1BW.1+D1BWI.1++],[D1BR.1],A1.7
        DMO       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1],D0Re0
        DMO       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1],RD
        DLMO      SUBS      [D1BW.1+D1BWI.1++],[D1BR.1],AC1.0
        DLM       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        DLM       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        DM        SUBS      [D1BW.1+D1BWI.1++],[D1BR.1],A0.7
        DLMO      SUBS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLMO      SUBS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        DM        SUBS      [D1BW.1+D1BWI.1++],[D1BR.1++],A1.7
        DLMO      SUBS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        DMO       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1++],D0.7
        DLM       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1++],AC1.0
        DMO       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1++],A0FrP
        DLMO      SUBS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        DLMO      SUBS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        DO        SUBS      [D1BW.1+D1BWI.1++],[D1BR.1++],A0.7
        DLM       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        DMO       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1LbP
        DO        SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1.7
        DMO       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D0Re0
        DMO       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],RD
        DLMO      SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],AC1.0
        DLM       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLM       SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DM        SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0.7
        DLMO      SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DL        SUBS      D0.7,D0.7,#0x7fff
        DL        SUBS      [D0AW.0],[D0AR.0],#-0x8000
        DL        SUBS      [D0AW.0++],[D0AR.0++],#-0x1
        DL        SUBS      D1Re0,D1Re0,#0x7fff
        DL        SUBS      D1.7,D1.7,#-0x8000
        DL        SUBS      [D1BW.1],[D1BR.1],#-0x1
        DL        SUBS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x7fff
        D         SUBSNV    D0Re0,D0Re0,A1LbP
        D         SUBSNV    D0Re0,D0Re0,D1.7
        D         SUBSNV    D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        D         SUBSNV    D0Re0,D0.7,A1LbP
        D         SUBSNV    D0Re0,D0.7,D1.7
        DHCS      SUBSVS    D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        D         SUBSNV    D0Re0,[D0AR.0],A1.7
        D         SUBSNV    D0Re0,[D0AR.0],A0.7
        D         SUBSNV    D0Re0,[D0AR.0++],RD
        D         SUBSNV    D0Re0,[D0AR.0+D0ARI.0++],A1.7
        D         SUBSNV    D0Re0,[D0AR.0+D0ARI.0++],A0.7
        D         SUBSNV    D0.7,D0Re0,A1.7
        D         SUBSNV    D0.7,D0Re0,RD
        D         SUBSNV    D0.7,D0Re0,A0.7
        D         SUBSNV    D0.7,D0.7,A1.7
        D         SUBSNV    D0.7,D0.7,RD
        D         SUBSNV    D0.7,D0.7,A0.7
        D         SUBSNV    D0.7,[D0AR.0],RD
        D         SUBSNV    D0.7,[D0AR.0++],A1.7
        D         SUBSNV    D0.7,[D0AR.0++],A0.7
        D         SUBSNV    D0.7,[D0AR.0+D0ARI.0++],RD
        D         SUBSNV    D0.16,D0Re0,A1LbP
        DLLS      SUBSLS    D0.16,D0Re0,[D0AR.0]
        DECS      SUBSLE    D0.16,D0Re0,A0FrP
        DECS      SUBSLE    D0.16,D0.7,D0.7
        DLNE      SUBSNE    D0.16,D0.7,[D0AR.0]
        DECS      SUBSLE    D0.16,D0.7,A0FrP
        D         SUBSNV    D0.16,[D0AR.0],D1Re0
        D         SUBSNV    D0.16,[D0AR.0++],D0Re0
        D         SUBSNV    D0.16,[D0AR.0++],[D0AR.0++]
        D         SUBSNV    D0.16,[D0AR.0+D0ARI.0++],D1Re0
        DLLS      SUBSLS    D1Re0,D1Re0,D1Re0
        DECS      SUBSLE    D1Re0,D1Re0,D0Re0
        DLNE      SUBSNE    D1Re0,D1Re0,[D1BR.1]
        DLNE      SUBSNE    D1Re0,D1.7,D1Re0
        DHNE      SUBSPL    D1Re0,D1.7,D0Re0
        DLLS      SUBSLS    D1Re0,D1.7,[D1BR.1]
        D         SUBSNV    D1Re0,[D1BR.1],D1Re0
        D         SUBSNV    D1Re0,[D1BR.1],[D1BR.1]
        D         SUBSNV    D1Re0,[D1BR.1++],D0Re0
        D         SUBSNV    D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         SUBSNV    D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DHNE      SUBSPL    D1.7,D1Re0,D1.7
        DHNE      SUBSPL    D1.7,D1Re0,RD
        DECS      SUBSLE    D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DECS      SUBSLE    D1.7,D1.7,D1.7
        DECS      SUBSLE    D1.7,D1.7,RD
        DHNE      SUBSPL    D1.7,D1.7,[D1BR.1+D1BRI.1++]
        D         SUBSNV    D1.7,[D1BR.1],D0Re0
        D         SUBSNV    D1.7,[D1BR.1++],D1Re0
        D         SUBSNV    D1.7,[D1BR.1++],[D1BR.1]
        D         SUBSNV    D1.7,[D1BR.1+D1BRI.1++],D0Re0
        DLLS      SUBSLS    D1.16,D1Re0,D1Re0
        DECS      SUBSLE    D1.16,D1Re0,D0Re0
        DLNE      SUBSNE    D1.16,D1Re0,[D1BR.1]
        DLNE      SUBSNE    D1.16,D1.7,D1Re0
        DHNE      SUBSPL    D1.16,D1.7,D0Re0
        DLLS      SUBSLS    D1.16,D1.7,[D1BR.1]
        D         SUBSNV    D1.16,[D1BR.1],D1Re0
        D         SUBSNV    D1.16,[D1BR.1],[D1BR.1]
        D         SUBSNV    D1.16,[D1BR.1++],D0Re0
        D         SUBSNV    D1.16,[D1BR.1+D1BRI.1++],D1Re0
        D         SUBSNV    D1.16,[D1BR.1+D1BRI.1++],[D1BR.1]
        DL        ADDB8     D0Re0,D0Re0,D0Re0
        DLM       ADDT8     D0Re0,D0Re0,D0Re0
        D         ADDT8     D0Re0,D0Re0,D0.7
        DM        ADDB8     D0Re0,D0Re0,D0.7
        D         AND       D0Re0,D0Re0,[D0AR.0]
        DLQ       ANDQ      D0Re0,D0Re0,RD,A0.2
        DL        ANDQ      D0Re0,D0Re0,[D0AR.0+D0ARI.0++],A0.2
        D         ADDB8     D0Re0,D0.7,D0Re0
        DM        ADDB8     D0Re0,D0.7,D0Re0
        D         AND       D0Re0,D0.7,D0.7
        D         ANDQ      D0Re0,D0.7,A1.7,A0.2
        D         ANDQ      D0Re0,D0.7,D1Re0,A0.2
        DL        ANDQ      D0Re0,D0.7,[D0AR.0],A0.2
        D         AND       D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        D         AND       D0Re0,D0.7,A0.7
        D         ANDQ      D0Re0,[D0AR.0],D0Re0,A0.2
        D         ANDQ      D0Re0,[D0AR.0],A1LbP,A0.2
        DL        ADDB8     D0Re0,[D0AR.0],D0.7
        DLM       ADDB8     D0Re0,D0.16,D0.7
        DL        AND       D0Re0,[D0AR.0],[D0AR.0]
        DL        AND       D0Re0,[D0AR.0],[D0AR.0++]
        D         ANDQ      D0Re0,[D0AR.0],A0FrP,A0.2
        DL        AND       D0Re0,[D0AR.0++],D0Re0
        D         AND       D0Re0,[D0AR.0++],A1.3
        DL        ADDT8     D0Re0,[D0AR.0++],D0.7
        D         ANDQ      D0Re0,[D0AR.0++],D1.7,A0.2
        DLQ       ANDQ      D0Re0,[D0AR.0++],D0.16,A0.2
        D         ANDQ      D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        D         AND       D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        DLQ       ANDQ      D0Re0,[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         ADDT8     D0Re0,[D0AR.0+D0ARI.0++],D0.7
        D         AND       D0Re0,[D0AR.0+D0ARI.0++],D1Re0
        D         ANDQ      D0Re0,[D0AR.0+D0ARI.0++],RD,A0.2
        DL        ANDQ      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++],A0.2
        D         ANDQ      D0Re0,[D0AR.0+D0ARI.0++],A0.2,A0.2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0FrP++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0FrP++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0FrP++]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.2++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.2++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.2++]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.7++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.7++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.7++]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.8++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.8++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.8++]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0FrP],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0FrP],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0FrP]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0FrP++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0FrP++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0FrP++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP++]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0FrP--],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0FrP--],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0FrP--]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP--]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0FrP--]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0.2+A0FrP++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2+A0FrP++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2+A0FrP++]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.2++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.2++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.2++]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.7++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.7++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.7++]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.8++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.8++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.8++]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.2],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0.2],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0.2]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.2++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0.2++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0.2++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2++]
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.2--],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0.2--],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0.2--]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2--]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.2--]
        DL        ADDT8     D0.7,D0Re0,D0Re0
        D         AND       D0.7,D0Re0,A1LbP
        DL        AND       D0.7,D0Re0,D0.7
        DM        ADDT8     D0.7,D0Re0,D0.7
        D         AND       D0.7,D0Re0,RD
        D         AND       D0.7,D0Re0,[D0AR.0++]
        D         AND       D0.7,D0Re0,A0FrP
        DT        ADDB8     D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T2
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T2
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T2
        DLT       AND       D0Re0,D0Re0,D0Re0 T2,[A0.7+A0FrP++]
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.7+A0FrP++]
        DT        ADDB8     D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T2
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T2
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T2
        DLT       AND       D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.2++]
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.2++]
        D         ADDT8     D0.7,D0.7,D0Re0
        DM        ADDT8     D0.7,D0.7,D0Re0
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T2
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T2
        DL        ADDB8     D0.7,D0.7,D0.7
        DLM       ADDB8     D0.7,D0.7,D0.7
        DL        AND       D0.7,D0.7,[D0AR.0]
        DL        AND       D0.7,D0.7,[D0AR.0++]
        DT        ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.7++]
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.7++]
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.7++]
        D         ANDQ      D0.7,D0.7,A0.7,A0.2
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T2
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T2
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.8++]
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.8++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.8++]
        DL        ANDQ      D0.7,[D0AR.0],D0Re0,A0.2
        DT        AND       D0Re0,D0Re0,D0Re0 [A0.7],T2
        DLT       ADDB8     D0Re0,D0Re0,D0Re0 [A0.7],T2
        DLMT      ADDB8     D0Re0,D0Re0,D0Re0 [A0.7],T2
        D         ADDB8     D0.7,[D0AR.0],D0.7
        DLQ       ANDQ      D0.7,[D0AR.0],D0.7,A0.2
        D         ANDQ      D0.7,[D0AR.0],D1.7,A0.2
        DLQ       ANDQ      D0.7,[D0AR.0],D0.16,A0.2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A0.7]
        DLT       ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.7]
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.7]
        D         AND       D0.7,[D0AR.0],A0.2
        DL        ADDB8     D0.7,[D0AR.0++],D0Re0
        DWT       AND       D0Re0,D0Re0,D0Re0 [A0.7++],T2
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 [A0.7++],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A0.7++],T2
        D         ADDT8     D0.7,[D0AR.0++],D0.7
        D         AND       D0.7,[D0AR.0++],D1Re0
        D         ANDQ      D0.7,[D0AR.0++],RD,A0.2
        DL        ANDQ      D0.7,[D0AR.0++],[D0AR.0++],A0.2
        DL        AND       D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DL        ANDQ      D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.7++]
        D         ADDB8     D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         AND       D0.7,[D0AR.0+D0ARI.0++],A1LbP
        DL        AND       D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         ANDQ      D0.7,[D0AR.0+D0ARI.0++],D1Re0,A0.2
        DL        ANDQ      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0],A0.2
        D         AND       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         AND       D0.7,[D0AR.0+D0ARI.0++],A0.7
        DLT       AND       D0Re0,D0Re0,D0Re0 [A0.7--],T2
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.7--],T2
        DT        ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.7--]
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.7--]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.7--]
        DLT       AND       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T2
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T2
        DT        ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0FrP++]
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0FrP++]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0FrP++]
        DLT       AND       D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T2
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T2
        DT        ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.2++]
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.2++]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.2++]
        DLT       AND       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T2
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T2
        DT        ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.7++]
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.7++]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.7++]
        DLT       AND       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T2
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T2
        DT        ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.8++]
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.8++]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.8++]
        DLT       AND       D0Re0,D0Re0,D0Re0 [A0.8],T2
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.8],T2
        DT        ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.8]
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8]
        DLT       AND       D0Re0,D0Re0,D0Re0 [A0.8++],T2
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.8++],T2
        DT        ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.8++]
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8++]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8++]
        DLT       AND       D0Re0,D0Re0,D0Re0 [A0.8--],T2
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 [A0.8--],T2
        DT        ADDB8     D0Re0,D0Re0,D0Re0 T2,[A0.8--]
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8--]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A0.8--]
        D         ANDQ      [D0AW.0],D0Re0,D0Re0,A0.2
        D         ANDQ      [D0AW.0],D0Re0,A1LbP,A0.2
        DL        ADDB8     [D0AW.0],D0Re0,D0.7
        DLM       ADDB8     [D0AW.0],D0Re0,D0.7
        DL        AND       [D0AW.0],D0Re0,[D0AR.0]
        DL        AND       [D0AW.0],D0Re0,[D0AR.0++]
        D         ANDQ      [D0AW.0],D0Re0,A0FrP,A0.2
        DL        AND       [D0AW.0],D0.7,D0Re0
        DLM       ADDB8     [D0AW.0],D0.7,D0Re0
        D         ADDB8     [D0AW.0],D0.7,D0.7
        DLQ       ANDQ      [D0AW.0],D0.7,D0.7,A0.2
        D         ANDQ      [D0AW.0],D0.7,D1.7,A0.2
        DLQ       ANDQ      [D0AW.0],D0.7,D0.16,A0.2
        D         ANDQ      [D0AW.0],D0.7,[D0AR.0+D0ARI.0++],A0.2
        D         AND       [D0AW.0],[D0AR.0],D0Re0
        DLQ       ANDQ      [D0AW.0],[D0AR.0],D0Re0,A0.2
        D         ANDQ      [D0AW.0],[D0AR.0],A1.3,A0.2
        D         ANDQ      [D0AW.0],[D0AR.0],D0.7,A0.2
        D         AND       [D0AW.0],[D0AR.0],D1Re0
        D         ANDQ      [D0AW.0],[D0AR.0],RD,A0.2
        DL        ANDQ      [D0AW.0],[D0AR.0],[D0AR.0++],A0.2
        D         ANDQ      [D0AW.0],[D0AR.0],A0.2,A0.2
        DL        ADDT8     [D0AW.0],[D0AR.0++],D0Re0
        D         AND       [D0AW.0],[D0AR.0++],D0.7
        D         ANDQ      [D0AW.0],[D0AR.0++],A1.7,A0.2
        D         AND       [D0AW.0],[D0AR.0++],RD
        D         AND       [D0AW.0],[D0AR.0++],[D0AR.0++]
        D         AND       [D0AW.0],[D0AR.0++],A0FrP
        D         ADDT8     [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        D         ANDQ      [D0AW.0],[D0AR.0+D0ARI.0++],A1LbP,A0.2
        DL        ADDB8     [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        D         AND       [D0AW.0],[D0AR.0+D0ARI.0++],D1.7
        DL        ANDQ      [D0AW.0],[D0AR.0+D0ARI.0++],RD,A0.2
        DL        AND       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ANDQ      [D0AW.0],[D0AR.0+D0ARI.0++],A0.7,A0.2
        DLT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [D0AW.0],T2
        DT        ADDT8     D0Re0,D0Re0,D0Re0 T2,[D0AR.0]
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[D0AR.0]
        DL        ADDT8     [D0AW.0++],D0Re0,D0Re0
        D         AND       [D0AW.0++],D0Re0,A1LbP
        DL        AND       [D0AW.0++],D0Re0,D0.7
        DM        ADDT8     [D0AW.0++],D0Re0,D0.7
        D         AND       [D0AW.0++],D0Re0,RD
        D         AND       [D0AW.0++],D0Re0,[D0AR.0++]
        D         AND       [D0AW.0++],D0Re0,A0FrP
        DT        ADDB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T2
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T2
        DMT       ADDB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T2
        DT        AND       D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1LbP++]
        DLT       ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1LbP++]
        DLQT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP+A1LbP++]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1LbP++]
        DLT       AND       D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T2
        DLT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.3++],T2
        DLMT      ADDB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T2
        DT        ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.3++]
        DT        ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP+A1.3++]
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.3++]
        D         ADDT8     [D0AW.0++],D0.7,D0Re0
        DM        ADDT8     [D0AW.0++],D0.7,D0Re0
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T2
        D         ANDQ      [D0AW.0++],D0.7,A1.3,A0.2
        D         ANDQ      [D0AW.0++],D0.7,D0.7,A0.2
        D         AND       [D0AW.0++],D0.7,D1Re0
        D         ANDQ      [D0AW.0++],D0.7,RD,A0.2
        DL        ANDQ      [D0AW.0++],D0.7,[D0AR.0++],A0.2
        DL        AND       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DT        ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP+A1.7++]
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.7++]
        D         ANDQ      [D0AW.0++],D0.7,A0FrP,A0.2
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.8++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.8++]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP+A1.8++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.8++]
        DL        ANDQ      [D0AW.0++],[D0AR.0],D0Re0,A0.2
        DT        AND       D0Re0,D0Re0,D0Re0 [A1LbP],T2
        DLT       ADDB8     D0Re0,D0Re0,D0Re0 [A1LbP],T2
        DLQT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP],T2
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 [A1LbP],T2
        DL        AND       [D0AW.0++],[D0AR.0],D0.7
        DM        ADDT8     [D0AW.0++],D0.16,D0.7
        D         AND       [D0AW.0++],[D0AR.0],RD
        D         AND       [D0AW.0++],[D0AR.0],[D0AR.0++]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1LbP]
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP]
        DLQT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP]
        D         AND       [D0AW.0++],[D0AR.0++],D0Re0
        DLQ       ANDQ      [D0AW.0++],[D0AR.0++],D0Re0,A0.2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 [A1LbP++],T2
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP++],T2
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1LbP++],T2
        D         AND       [D0AW.0++],[D0AR.0++],A1.7
        DL        ANDQ      [D0AW.0++],[D0AR.0++],D0.7,A0.2
        DL        AND       [D0AW.0++],[D0AR.0++],[D0AR.0]
        DL        AND       [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DT        ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP++]
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP++]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP++]
        D         ADDB8     [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         AND       [D0AW.0++],[D0AR.0+D0ARI.0++],A1LbP
        DL        AND       [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        D         ANDQ      [D0AW.0++],[D0AR.0+D0ARI.0++],D1Re0,A0.2
        DL        ANDQ      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0],A0.2
        D         AND       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         AND       [D0AW.0++],[D0AR.0+D0ARI.0++],A0.7
        DLT       AND       D0Re0,D0Re0,D0Re0 [A1LbP--],T2
        DLT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP--],T2
        DLMT      ADDB8     D0Re0,D0Re0,D0Re0 [A1LbP--],T2
        DT        ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP--]
        DT        ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP--]
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1LbP--]
        DT        ADDT8     D0Re0,D0Re0,D0Re0 [D0AW.0++],T2
        DMT       ADDT8     D0Re0,D0Re0,D0Re0 [D0AW.0++],T2
        DLT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[D0AR.0++]
        D         AND       [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DLQ       ANDQ      [D0AW.0+D0AWI.0++],D0Re0,D0Re0,A0.2
        D         ANDQ      [D0AW.0+D0AWI.0++],D0Re0,A1.3,A0.2
        D         ANDQ      [D0AW.0+D0AWI.0++],D0Re0,D0.7,A0.2
        D         AND       [D0AW.0+D0AWI.0++],D0Re0,D1Re0
        D         ANDQ      [D0AW.0+D0AWI.0++],D0Re0,RD,A0.2
        DL        ANDQ      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++],A0.2
        D         ANDQ      [D0AW.0+D0AWI.0++],D0Re0,A0.2,A0.2
        DL        ADDT8     [D0AW.0+D0AWI.0++],D0.7,D0Re0
        D         AND       [D0AW.0+D0AWI.0++],D0.7,A1LbP
        DL        AND       [D0AW.0+D0AWI.0++],D0.7,D0.7
        DM        ADDT8     [D0AW.0+D0AWI.0++],D0.7,D0.7
        D         AND       [D0AW.0+D0AWI.0++],D0.7,RD
        D         AND       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        D         AND       [D0AW.0+D0AWI.0++],D0.7,A0FrP
        D         ADDT8     [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DM        ADDT8     [D0AW.0+D0AWI.0++],D0.16,D0Re0
        D         AND       [D0AW.0+D0AWI.0++],[D0AR.0],A1.7
        DL        ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0],D0.7,A0.2
        D         AND       [D0AW.0+D0AWI.0++],[D0AR.0],D1.7
        DL        ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0],RD,A0.2
        DL        AND       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0],A0.7,A0.2
        DL        ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0,A0.2
        D         ADDB8     [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        DLQ       ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7,A0.2
        D         ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0],A0.2
        D         ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++],A0.2
        D         AND       [D0AW.0+D0AWI.0++],[D0AR.0++],A0.2
        DL        ADDB8     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A1.3,A0.2
        D         ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7,A0.2
        D         AND       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLQ       ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],RD,A0.2
        DL        ANDQ      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++],A0.2
        DT        ADDB8     D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T2
        DMT       ADDB8     D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T2
        DLT       ADDB8     D0Re0,D0Re0,D0Re0 T2,[D0AR.0+D0ARI.0++]
        DLMT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[D0AR.0+D0ARI.0++]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1LbP++],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3+A1LbP++]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.3+A1LbP++]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3+A1LbP++]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.3++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.3++]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.3+A1.3++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.3++]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.7++],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.7++]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.3+A1.7++]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.7++]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.8++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.8++]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.3+A1.8++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.8++]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.3],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.3],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.3]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1.3++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1.3++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1.3++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1.3++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1.3++]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.3++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3++]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.3--],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3--],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.3--],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3--]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.3--]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.3--]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1.7+A1LbP++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1.7+A1LbP++]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7+A1LbP++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.7+A1LbP++]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.3++],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.3++]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7+A1.3++]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.3++]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.7++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.7++]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7+A1.7++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.7++]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.8++],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.8++]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7+A1.8++]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.8++]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1.7],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1.7],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1.7],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1.7]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1.7]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.7]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.7++],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7++],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.7++],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.7++]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7++]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.7++]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1.7--],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1.7--],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1.7--],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1.7--]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1.7--]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7--]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.7--]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1LbP++],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8+A1LbP++]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.8+A1LbP++]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8+A1LbP++]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.3++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.3++]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.8+A1.3++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.3++]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.7++],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.7++]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.8+A1.7++]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.7++]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.8++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.8++]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.8+A1.8++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.8++]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.8],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.8],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.8]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8]
        DWT       ADDB8     D0Re0,D0Re0,D0Re0 [A1.8++],T2
        DLWT      ADDT8     D0Re0,D0Re0,D0Re0 [A1.8++],T2
        DMWT      ADDB8     D0Re0,D0Re0,D0Re0 [A1.8++],T2
        DWT       AND       D0Re0,D0Re0,D0Re0 T2,[A1.8++]
        DLWT      ADDB8     D0Re0,D0Re0,D0Re0 T2,[A1.8++]
        DLQWT     ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.8++]
        DLMWT     ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8++]
        DLWT      AND       D0Re0,D0Re0,D0Re0 [A1.8--],T2
        DLWT      ANDQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8--],T2
        DLMWT     ADDB8     D0Re0,D0Re0,D0Re0 [A1.8--],T2
        DWT       ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8--]
        DWT       ANDQ      D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.8--]
        DMWT      ADDT8     D0Re0,D0Re0,D0Re0 T2,[A1.8--]
        DL        AND       D1Re0,D1Re0,D1Re0
        DL        ANDQ      D1Re0,D1Re0,D1Re0,A1.3
        D         ADDB8     D1Re0,D1Re0,D1.7
        DLM       ADDT8     D1Re0,D1Re0,D1.7
        D         ANDQ      D1Re0,D1Re0,D0.7,A1.3
        D         AND       D1Re0,D1Re0,A0.2
        D         ANDQ      D1Re0,D1Re0,[D1BR.1++],A1.3
        D         AND       D1Re0,D1.7,D1Re0
        DLM       ADDB8     D1Re0,D1.7,D1Re0
        D         ANDQ      D1Re0,D1.7,A1.3,A1.3
        DM        ADDB8     D1Re0,D1.7,D1.7
        D         AND       D1Re0,D1.7,D0Re0
        DLQ       ANDQ      D1Re0,D1.7,RD,A1.3
        DL        ANDQ      D1Re0,D1.7,[D1BR.1],A1.3
        D         ANDQ      D1Re0,D1.7,[D1BR.1+D1BRI.1++],A1.3
        DM        ADDT8     D1Re0,D1.16,D1.7
        DL        ADDT8     D1Re0,[D1BR.1],D1Re0
        D         AND       D1Re0,[D1BR.1],D1.7
        D         ANDQ      D1Re0,[D1BR.1],A1.7,A1.3
        D         ANDQ      D1Re0,[D1BR.1],RD,A1.3
        D         AND       D1Re0,[D1BR.1],[D1BR.1]
        D         AND       D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ADDT8     D1Re0,[D1BR.1++],D1Re0
        D         ANDQ      D1Re0,[D1BR.1++],A1LbP,A1.3
        DL        ADDB8     D1Re0,[D1BR.1++],D1.7
        D         AND       D1Re0,[D1BR.1++],D0.7
        D         ANDQ      D1Re0,[D1BR.1++],A0FrP,A1.3
        DL        AND       D1Re0,[D1BR.1++],[D1BR.1++]
        DL        ANDQ      D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        ANDQ      D1Re0,[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         ADDB8     D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DLQ       ANDQ      D1Re0,[D1BR.1+D1BRI.1++],D1.7,A1.3
        DLQ       ANDQ      D1Re0,[D1BR.1+D1BRI.1++],D1.16,A1.3
        D         ANDQ      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1],A1.3
        DL        AND       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DT        ADDT8     D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T2
        DMT       ADDB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T2
        DT        ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0FrP++],T2
        DT        ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0FrP++]
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0FrP++]
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0FrP++]
        DT        AND       D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T2
        DLT       ADDB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T2
        DLMT      ADDB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T2
        DLQT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.2++],T2
        DLT       AND       D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.2++]
        DMT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.2++]
        DLT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0FrP+A0.2++]
        DT        ADDT8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T2
        DMT       ADDB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T2
        DT        ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.7++],T2
        DT        ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.7++]
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.7++]
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.7++]
        DT        AND       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T2
        DLT       ADDB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T2
        DLMT      ADDB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T2
        DLQT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.8++],T2
        DLT       AND       D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.8++]
        DMT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.8++]
        DLT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0FrP+A0.8++]
        DT        ADDT8     D1Re0,D1Re0,D1Re0 [A0FrP],T2
        DMT       ADDB8     D1Re0,D1Re0,D1Re0 [A0FrP],T2
        DT        ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP],T2
        DT        ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0FrP]
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP]
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP]
        DT        AND       D1Re0,D1Re0,D1Re0 [A0FrP++],T2
        DLT       ADDB8     D1Re0,D1Re0,D1Re0 [A0FrP++],T2
        DLMT      ADDB8     D1Re0,D1Re0,D1Re0 [A0FrP++],T2
        DLQT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP++],T2
        DLT       AND       D1Re0,D1Re0,D1Re0 T2,[A0FrP++]
        DMT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP++]
        DLT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0FrP++]
        DT        ADDT8     D1Re0,D1Re0,D1Re0 [A0FrP--],T2
        DMT       ADDB8     D1Re0,D1Re0,D1Re0 [A0FrP--],T2
        DT        ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP--],T2
        DT        ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0FrP--]
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP--]
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0FrP--]
        DT        AND       D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T2
        DLT       ADDB8     D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T2
        DLMT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T2
        DLQT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0FrP++],T2
        DLT       AND       D1Re0,D1Re0,D1Re0 T2,[A0.2+A0FrP++]
        DMT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.2+A0FrP++]
        DLT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2+A0FrP++]
        DT        ADDT8     D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T2
        DMT       ADDB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T2
        DT        ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.2++],T2
        DT        ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.2++]
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.2++]
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.2++]
        DT        AND       D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T2
        DLT       ADDB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T2
        DLMT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T2
        DLQT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.7++],T2
        DLT       AND       D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.7++]
        DMT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.7++]
        DLT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2+A0.7++]
        DT        ADDT8     D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T2
        DMT       ADDB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T2
        DT        ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.8++],T2
        DT        ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.8++]
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.8++]
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.8++]
        DT        AND       D1Re0,D1Re0,D1Re0 [A0.2],T2
        DLT       ADDB8     D1Re0,D1Re0,D1Re0 [A0.2],T2
        DLMT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.2],T2
        DLQT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2],T2
        DLT       AND       D1Re0,D1Re0,D1Re0 T2,[A0.2]
        DMT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.2]
        DLT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2]
        DT        ADDT8     D1Re0,D1Re0,D1Re0 [A0.2++],T2
        DMT       ADDB8     D1Re0,D1Re0,D1Re0 [A0.2++],T2
        DT        ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2++],T2
        DT        ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.2++]
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.2++]
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.2++]
        DT        AND       D1Re0,D1Re0,D1Re0 [A0.2--],T2
        DLT       ADDB8     D1Re0,D1Re0,D1Re0 [A0.2--],T2
        DLMT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.2--],T2
        DLQT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2--],T2
        DLT       AND       D1Re0,D1Re0,D1Re0 T2,[A0.2--]
        DMT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.2--]
        DLT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2--]
        DL        ADDB8     D1.7,D1Re0,D1Re0
        DLQ       ANDQ      D1.7,D1Re0,D1Re0,A1.3
        D         ADDT8     D1.7,D1Re0,D1.7
        D         ANDQ      D1.7,D1Re0,D1.7,A1.3
        D         AND       D1.7,D1Re0,RD
        D         ANDQ      D1.7,D1Re0,A0.2,A1.3
        DL        ANDQ      D1.7,D1Re0,[D1BR.1++],A1.3
        DWT       AND       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T2
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T2
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T2
        DLQWT     ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0FrP++],T2
        DLWT      AND       D1Re0,D1Re0,D1Re0 T2,[A0.7+A0FrP++]
        DMWT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.7+A0FrP++]
        DLWT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7+A0FrP++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T2
        DWT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.2++],T2
        DWT       ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.2++]
        DLWT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.2++]
        DLMWT     ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.2++]
        D         ADDB8     D1.7,D1.7,D1Re0
        DLM       ADDT8     D1.7,D1.7,D1Re0
        DT        ADDB8     D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T2
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T2
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T2
        D         AND       D1.7,D1.7,A1.3
        DL        ADDT8     D1.7,D1.7,D1.7
        DLQ       ANDQ      D1.7,D1.7,D1.7,A1.3
        DLQ       ANDQ      D1.7,D1.7,D1.16,A1.3
        DLT       AND       D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.7++]
        DMT       ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.7++]
        DLT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7+A0.7++]
        D         AND       D1.7,D1.7,[D1BR.1]
        D         AND       D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DT        ADDB8     D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T2
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T2
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T2
        DT        AND       D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.8++]
        DLT       ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.8++]
        DLMT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.8++]
        DLQT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7+A0.8++]
        DT        ADDB8     D1Re0,D1Re0,D1Re0 [A0.7],T2
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 [A0.7],T2
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 [A0.7],T2
        DM        ADDB8     D1.7,D1.16,D1.7
        DT        ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.7]
        DMT       ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.7]
        DT        ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7]
        DT        ADDB8     D1Re0,D1Re0,D1Re0 [A0.7++],T2
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 [A0.7++],T2
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 [A0.7++],T2
        DT        AND       D1Re0,D1Re0,D1Re0 T2,[A0.7++]
        DLT       ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.7++]
        DLMT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.7++]
        DLQT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7++]
        DLT       AND       D1Re0,D1Re0,D1Re0 [A0.7--],T2
        DMT       ADDT8     D1Re0,D1Re0,D1Re0 [A0.7--],T2
        DLT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7--],T2
        DT        ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.7--]
        DMT       ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.7--]
        DT        ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7--]
        D         ADDT8     D1.7,[D1BR.1],D1Re0
        D         ANDQ      D1.7,[D1BR.1],A1LbP,A1.3
        DL        ADDB8     D1.7,[D1BR.1],D1.7
        D         AND       D1.7,[D1BR.1],D0.7
        D         ANDQ      D1.7,[D1BR.1],A0FrP,A1.3
        DL        AND       D1.7,[D1BR.1],[D1BR.1++]
        DL        ANDQ      D1.7,[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        DL        ANDQ      D1.7,[D1BR.1++],D1Re0,A1.3
        D         ADDB8     D1.7,[D1BR.1++],D1.7
        DLQ       ANDQ      D1.7,[D1BR.1++],D1.7,A1.3
        DLQ       ANDQ      D1.7,[D1BR.1++],D1.16,A1.3
        D         ANDQ      D1.7,[D1BR.1++],[D1BR.1],A1.3
        DL        AND       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        ADDB8     D1.7,[D1BR.1+D1BRI.1++],D1Re0
        D         ANDQ      D1.7,[D1BR.1+D1BRI.1++],A1.3,A1.3
        D         ANDQ      D1.7,[D1BR.1+D1BRI.1++],D1.7,A1.3
        D         AND       D1.7,[D1BR.1+D1BRI.1++],RD
        D         ANDQ      D1.7,[D1BR.1+D1BRI.1++],A0.2,A1.3
        DL        ANDQ      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++],A1.3
        DWT       AND       D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T2
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T2
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T2
        DLQWT     ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0FrP++],T2
        DLWT      AND       D1Re0,D1Re0,D1Re0 T2,[A0.8+A0FrP++]
        DMWT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.8+A0FrP++]
        DLWT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8+A0FrP++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T2
        DWT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.2++],T2
        DWT       ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.2++]
        DLWT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.2++]
        DLMWT     ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.2++]
        DWT       AND       D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T2
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T2
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T2
        DLQWT     ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.7++],T2
        DLWT      AND       D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.7++]
        DMWT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.7++]
        DLWT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8+A0.7++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T2
        DWT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.8++],T2
        DWT       ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.8++]
        DLWT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.8++]
        DLMWT     ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.8++]
        DWT       AND       D1Re0,D1Re0,D1Re0 [A0.8],T2
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.8],T2
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 [A0.8],T2
        DLQWT     ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8],T2
        DLWT      AND       D1Re0,D1Re0,D1Re0 T2,[A0.8]
        DMWT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.8]
        DLWT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A0.8++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.8++],T2
        DWT       ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8++],T2
        DWT       ADDB8     D1Re0,D1Re0,D1Re0 T2,[A0.8++]
        DLWT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.8++]
        DLMWT     ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.8++]
        DWT       AND       D1Re0,D1Re0,D1Re0 [A0.8--],T2
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 [A0.8--],T2
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 [A0.8--],T2
        DLQWT     ANDQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8--],T2
        DLWT      AND       D1Re0,D1Re0,D1Re0 T2,[A0.8--]
        DMWT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[A0.8--]
        DLWT      ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8--]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1LbP++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1LbP++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1LbP++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.3++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.3++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.3++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.7++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.7++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.7++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.8++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.8++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.8++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1LbP],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1LbP],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1LbP]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1LbP++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1LbP++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1LbP++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1LbP--],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1LbP--],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1LbP--]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP--]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1LbP--]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.3+A1LbP++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3+A1LbP++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3+A1LbP++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.3++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.3++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.3++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.7++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.7++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.7++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.8++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.8++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.8++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.3],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.3],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.3]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.3++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.3++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.3++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.3--],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.3--],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.3--]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3--]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.3--]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.7+A1LbP++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7+A1LbP++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7+A1LbP++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.3++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.3++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.3++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.7++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.7++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.7++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.8++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.8++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.8++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.7],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.7],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.7]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.7++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.7++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.7++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.7--],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.7--],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.7--]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7--]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.7--]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.8+A1LbP++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8+A1LbP++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8+A1LbP++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.3++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.3++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.3++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.7++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.7++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.7++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.8++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.8++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.8++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.8],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.8],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.8]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.8++],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.8++],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.8++]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8++]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8++]
        DWT       ADDT8     D1Re0,D1Re0,D1Re0 [A1.8--],T2
        DMWT      ADDB8     D1Re0,D1Re0,D1Re0 [A1.8--],T2
        DWT       AND       D1Re0,D1Re0,D1Re0 T2,[A1.8--]
        DLWT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8--]
        DLMWT     ADDB8     D1Re0,D1Re0,D1Re0 T2,[A1.8--]
        DL        ADDT8     [D1BW.1],D1Re0,D1Re0
        D         AND       [D1BW.1],D1Re0,A1LbP
        DL        AND       [D1BW.1],D1Re0,D1.7
        D         ANDQ      [D1BW.1],D1Re0,A1.7,A1.3
        D         ANDQ      [D1BW.1],D1Re0,RD,A1.3
        D         AND       [D1BW.1],D1Re0,[D1BR.1]
        D         AND       [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        D         ADDT8     [D1BW.1],D1.7,D1Re0
        D         ANDQ      [D1BW.1],D1.7,D1Re0,A1.3
        D         AND       [D1BW.1],D1.7,A1.7
        DLM       ADDB8     [D1BW.1],D1.7,D1.7
        D         AND       [D1BW.1],D1.7,D0.7
        D         ANDQ      [D1BW.1],D1.7,A0FrP,A1.3
        DL        AND       [D1BW.1],D1.7,[D1BR.1++]
        DL        ANDQ      [D1BW.1],D1.7,[D1BR.1+D1BRI.1++],A1.3
        DLM       ADDT8     [D1BW.1],D1.16,D1.7
        DMT       ADDT8     D1Re0,D1Re0,D1Re0 [D1BW.1],T2
        DT        ADDT8     D1Re0,D1Re0,D1Re0 T2,[D1BR.1]
        DT        ANDQ      D1Re0,D1Re0,D1Re0,A1.3 T2,[D1BR.1]
        DL        ADDT8     [D1BW.1],[D1BR.1],D1Re0
        D         AND       [D1BW.1],[D1BR.1],D1.7
        D         ANDQ      [D1BW.1],[D1BR.1],A1.7,A1.3
        D         ANDQ      [D1BW.1],[D1BR.1],RD,A1.3
        D         AND       [D1BW.1],[D1BR.1],[D1BR.1]
        D         AND       [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ADDT8     [D1BW.1],[D1BR.1++],D1Re0
        D         ANDQ      [D1BW.1],[D1BR.1++],A1LbP,A1.3
        DL        ADDB8     [D1BW.1],[D1BR.1++],D1.7
        D         AND       [D1BW.1],[D1BR.1++],D0.7
        D         ANDQ      [D1BW.1],[D1BR.1++],A0FrP,A1.3
        DL        AND       [D1BW.1],[D1BR.1++],[D1BR.1++]
        DL        ANDQ      [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        ANDQ      [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         ADDB8     [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DLQ       ANDQ      [D1BW.1],[D1BR.1+D1BRI.1++],D1.7,A1.3
        DLQ       ANDQ      [D1BW.1],[D1BR.1+D1BRI.1++],D1.16,A1.3
        D         ANDQ      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1],A1.3
        DL        AND       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DL        ADDB8     [D1BW.1++],D1Re0,D1Re0
        DLQ       ANDQ      [D1BW.1++],D1Re0,D1Re0,A1.3
        D         ADDT8     [D1BW.1++],D1Re0,D1.7
        D         ANDQ      [D1BW.1++],D1Re0,D1.7,A1.3
        D         AND       [D1BW.1++],D1Re0,RD
        D         ANDQ      [D1BW.1++],D1Re0,A0.2,A1.3
        DL        ANDQ      [D1BW.1++],D1Re0,[D1BR.1++],A1.3
        D         ADDB8     [D1BW.1++],D1.7,D1Re0
        DLM       ADDT8     [D1BW.1++],D1.7,D1Re0
        D         AND       [D1BW.1++],D1.7,D1.7
        DM        ADDT8     [D1BW.1++],D1.7,D1.7
        D         ANDQ      [D1BW.1++],D1.7,D0Re0,A1.3
        D         AND       [D1BW.1++],D1.7,A0FrP
        D         AND       [D1BW.1++],D1.7,[D1BR.1++]
        D         ANDQ      [D1BW.1++],D1.7,A0.7,A1.3
        DLM       ADDB8     [D1BW.1++],D1.16,D1.7
        DMT       ADDB8     D1Re0,D1Re0,D1Re0 [D1BW.1++],T2
        DT        ADDB8     D1Re0,D1Re0,D1Re0 T2,[D1BR.1++]
        DLMT      ADDT8     D1Re0,D1Re0,D1Re0 T2,[D1BR.1++]
        DL        ADDB8     [D1BW.1++],[D1BR.1],D1Re0
        D         ANDQ      [D1BW.1++],[D1BR.1],A1.3,A1.3
        D         ANDQ      [D1BW.1++],[D1BR.1],D1.7,A1.3
        D         AND       [D1BW.1++],[D1BR.1],RD
        D         ANDQ      [D1BW.1++],[D1BR.1],A0.2,A1.3
        DL        ANDQ      [D1BW.1++],[D1BR.1],[D1BR.1++],A1.3
        D         ADDB8     [D1BW.1++],[D1BR.1++],D1Re0
        D         AND       [D1BW.1++],[D1BR.1++],A1LbP
        DL        AND       [D1BW.1++],[D1BR.1++],D1.7
        D         ANDQ      [D1BW.1++],[D1BR.1++],D0Re0,A1.3
        D         AND       [D1BW.1++],[D1BR.1++],A0FrP
        D         AND       [D1BW.1++],[D1BR.1++],[D1BR.1++]
        D         ANDQ      [D1BW.1++],[D1BR.1++],A0.7,A1.3
        D         ANDQ      [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         AND       [D1BW.1++],[D1BR.1+D1BRI.1++],A1.7
        DL        ANDQ      [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7,A1.3
        DL        ANDQ      [D1BW.1++],[D1BR.1+D1BRI.1++],RD,A1.3
        DL        AND       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         AND       [D1BW.1++],[D1BR.1+D1BRI.1++],A0.7
        DL        AND       [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        DL        ANDQ      [D1BW.1+D1BWI.1++],D1Re0,D1Re0,A1.3
        D         ADDB8     [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DLM       ADDT8     [D1BW.1+D1BWI.1++],D1Re0,D1.7
        D         ANDQ      [D1BW.1+D1BWI.1++],D1Re0,D0.7,A1.3
        D         AND       [D1BW.1+D1BWI.1++],D1Re0,A0.2
        D         ANDQ      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++],A1.3
        D         AND       [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DLM       ADDB8     [D1BW.1+D1BWI.1++],D1.7,D1Re0
        D         ANDQ      [D1BW.1+D1BWI.1++],D1.7,A1.3,A1.3
        DM        ADDB8     [D1BW.1+D1BWI.1++],D1.7,D1.7
        D         AND       [D1BW.1+D1BWI.1++],D1.7,D0Re0
        DLQ       ANDQ      [D1BW.1+D1BWI.1++],D1.7,RD,A1.3
        DL        ANDQ      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1],A1.3
        D         ANDQ      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++],A1.3
        DM        ADDT8     [D1BW.1+D1BWI.1++],D1.16,D1.7
        DLT       ADDT8     D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T2
        DT        AND       D1Re0,D1Re0,D1Re0 T2,[D1BR.1+D1BRI.1++]
        DLMT      ADDB8     D1Re0,D1Re0,D1Re0 T2,[D1BR.1+D1BRI.1++]
        DL        AND       [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        D         AND       [D1BW.1+D1BWI.1++],[D1BR.1],A1.3
        DL        ADDT8     [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        D         ANDQ      [D1BW.1+D1BWI.1++],[D1BR.1],D0.7,A1.3
        D         AND       [D1BW.1+D1BWI.1++],[D1BR.1],A0.2
        D         ANDQ      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++],A1.3
        D         AND       [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        DLQ       ANDQ      [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0,A1.3
        D         ADDT8     [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        D         AND       [D1BW.1+D1BWI.1++],[D1BR.1++],D0Re0
        DLQ       ANDQ      [D1BW.1+D1BWI.1++],[D1BR.1++],RD,A1.3
        DL        ANDQ      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1],A1.3
        D         ANDQ      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        ADDT8     [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         AND       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         ANDQ      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1.7,A1.3
        D         ANDQ      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],RD,A1.3
        D         AND       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         AND       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         AND       D0Re0,D0Re0,#-0x8000
        D         AND       D0.7,D0.7,#-0x1
        D         AND       [D0AW.0++],[D0AR.0++],#0x7fff
        D         AND       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#-0x8000
        D         AND       D1Re0,D1Re0,#-0x1
        D         AND       [D1BW.1],[D1BR.1],#0x7fff
        D         AND       [D1BW.1++],[D1BR.1++],#-0x8000
        D         AND       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#-0x1
        DLNE      ANDQNE    D0Re0,D0Re0,D0.7,A0.2
        DLLS      ANDQLS    D0Re0,D0Re0,[D0AR.0],A0.2
        DHNE      ANDQPL    D0Re0,D0Re0,A0FrP,A0.2
        DHNE      ANDQPL    D0Re0,D0.7,A1LbP,A0.2
        DECS      ANDQLE    D0Re0,D0.7,D1Re0,A0.2
        DLNE      ANDQNE    D0Re0,D0.7,[D0AR.0++],A0.2
        DHNE      ANDQPL    D0Re0,D0.7,A0.7,A0.2
        D         ANDQNV    D0Re0,[D0AR.0],D1.7,A0.2
        D         ANDQNV    D0Re0,[D0AR.0++],D0Re0,A0.2
        D         ANDQNV    D0Re0,[D0AR.0++],RD,A0.2
        D         ANDQNV    D0Re0,[D0AR.0+D0ARI.0++],A1.3,A0.2
        D         ANDQNV    D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++],A0.2
        DHNE      ANDQPL    D0.7,D0Re0,A1.3,A0.2
        DHNE      ANDQPL    D0.7,D0Re0,D1.7,A0.2
        DLNE      ANDQNE    D0.7,D0Re0,[D0AR.0+D0ARI.0++],A0.2
        DLNE      ANDQNE    D0.7,D0.7,D0Re0,A0.2
        D         ANDQNV    D0.7,D0.7,D0.7,A0.2
        D         ANDQNV    D0.7,D0.7,[D0AR.0],A0.2
        DHNE      ANDQPL    D0.7,D0.7,A0.2,A0.2
        D         ANDQNV    D0.7,[D0AR.0],A1.7,A0.2
        D         ANDQNV    D0.7,[D0AR.0],A0.2,A0.2
        D         ANDQNV    D0.7,[D0AR.0++],D1.7,A0.2
        D         ANDQNV    D0.7,[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         ANDQNV    D0.7,[D0AR.0+D0ARI.0++],RD,A0.2
        DHNE      ANDQPL    D0.16,D0Re0,A1LbP,A0.2
        DECS      ANDQLE    D0.16,D0Re0,D1Re0,A0.2
        DLNE      ANDQNE    D0.16,D0Re0,[D0AR.0++],A0.2
        DHNE      ANDQPL    D0.16,D0Re0,A0.7,A0.2
        DLNE      ANDQNE    D0.16,D0.7,D0.7,A0.2
        DLLS      ANDQLS    D0.16,D0.7,[D0AR.0],A0.2
        DHNE      ANDQPL    D0.16,D0.7,A0FrP,A0.2
        D         ANDQNV    D0.16,[D0AR.0],A1.3,A0.2
        D         ANDQNV    D0.16,[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        D         ANDQNV    D0.16,[D0AR.0++],A1.7,A0.2
        D         ANDQNV    D0.16,[D0AR.0++],A0.2,A0.2
        D         ANDQNV    D0.16,[D0AR.0+D0ARI.0++],D1.7,A0.2
        DECS      ANDQLE    D1Re0,D1Re0,D1Re0,A1.3
        D         ANDQNV    D1Re0,D1Re0,D1.7,A1.3
        DECS      ANDQLE    D1Re0,D1Re0,A0FrP,A1.3
        DHNE      ANDQPL    D1Re0,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DECS      ANDQLE    D1Re0,D1.7,A1.3,A1.3
        DECS      ANDQLE    D1Re0,D1.7,D0.7,A1.3
        DECS      ANDQLE    D1Re0,D1.7,[D1BR.1],A1.3
        D         ANDQNV    D1Re0,[D1BR.1],D1Re0,A1.3
        D         ANDQNV    D1Re0,[D1BR.1],A0FrP,A1.3
        D         ANDQNV    D1Re0,[D1BR.1++],A1.3,A1.3
        D         ANDQNV    D1Re0,[D1BR.1++],[D1BR.1],A1.3
        D         ANDQNV    D1Re0,[D1BR.1+D1BRI.1++],A1.7,A1.3
        D         ANDQNV    D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        DHNE      ANDQPL    D1.7,D1Re0,D1.7,A1.3
        DECS      ANDQLE    D1.7,D1Re0,RD,A1.3
        DHNE      ANDQPL    D1.7,D1Re0,[D1BR.1++],A1.3
        DECS      ANDQLE    D1.7,D1.7,A1LbP,A1.3
        DECS      ANDQLE    D1.7,D1.7,D0Re0,A1.3
        DECS      ANDQLE    D1.7,D1.7,A0.2,A1.3
        D         ANDQNV    D1.7,D1.7,[D1BR.1+D1BRI.1++],A1.3
        D         ANDQNV    D1.7,[D1BR.1],D0.7,A1.3
        D         ANDQNV    D1.7,[D1BR.1++],D1Re0,A1.3
        D         ANDQNV    D1.7,[D1BR.1++],A0FrP,A1.3
        D         ANDQNV    D1.7,[D1BR.1+D1BRI.1++],A1.3,A1.3
        D         ANDQNV    D1.7,[D1BR.1+D1BRI.1++],[D1BR.1],A1.3
        DECS      ANDQLE    D1.16,D1Re0,A1.3,A1.3
        DECS      ANDQLE    D1.16,D1Re0,D0.7,A1.3
        DECS      ANDQLE    D1.16,D1Re0,[D1BR.1],A1.3
        DECS      ANDQLE    D1.16,D1.7,D1Re0,A1.3
        D         ANDQNV    D1.16,D1.7,D1.7,A1.3
        DECS      ANDQLE    D1.16,D1.7,A0FrP,A1.3
        DHNE      ANDQPL    D1.16,D1.7,[D1BR.1+D1BRI.1++],A1.3
        D         ANDQNV    D1.16,[D1BR.1],A1.7,A1.3
        D         ANDQNV    D1.16,[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        D         ANDQNV    D1.16,[D1BR.1++],D0.7,A1.3
        D         ANDQNV    D1.16,[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         ANDQNV    D1.16,[D1BR.1+D1BRI.1++],A0FrP,A1.3
        D         ADDST8    D0Re0,D0Re0,D0Re0
        DM        ADDST8    D0Re0,D0Re0,D0Re0
        D         ANDS      D0Re0,D0Re0,A1.7
        DL        ANDSQ     D0Re0,D0Re0,D0.7,A0.2
        D         ANDS      D0Re0,D0Re0,D1.7
        DL        ANDSQ     D0Re0,D0Re0,RD,A0.2
        DL        ANDS      D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        D         ANDSQ     D0Re0,D0Re0,A0.7,A0.2
        DL        ANDSQ     D0Re0,D0.7,D0Re0,A0.2
        D         ANDS      D0Re0,D0.7,A1.3
        DL        ADDST8    D0Re0,D0.7,D0.7
        DLM       ADDST8    D0Re0,D0.7,D0.7
        D         ANDSQ     D0Re0,D0.7,[D0AR.0],A0.2
        D         ANDSQ     D0Re0,D0.7,[D0AR.0++],A0.2
        D         ANDS      D0Re0,D0.7,A0.2
        DL        ADDSB8    D0Re0,[D0AR.0],D0Re0
        DLM       ADDST8    D0Re0,D0.16,D0Re0
        D         ADDST8    D0Re0,[D0AR.0],D0.7
        DM        ADDSB8    D0Re0,D0.16,D0.7
        D         ANDS      D0Re0,[D0AR.0],[D0AR.0]
        DLQ       ANDSQ     D0Re0,[D0AR.0],RD,A0.2
        DL        ANDSQ     D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        D         ADDSB8    D0Re0,[D0AR.0++],D0Re0
        D         ANDS      D0Re0,[D0AR.0++],A1LbP
        DL        ANDS      D0Re0,[D0AR.0++],D0.7
        D         ANDSQ     D0Re0,[D0AR.0++],D1Re0,A0.2
        DL        ANDSQ     D0Re0,[D0AR.0++],[D0AR.0],A0.2
        D         ANDS      D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ANDS      D0Re0,[D0AR.0++],A0.7
        D         ANDSQ     D0Re0,[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         ANDS      D0Re0,[D0AR.0+D0ARI.0++],A1.7
        DL        ANDSQ     D0Re0,[D0AR.0+D0ARI.0++],D0.7,A0.2
        DL        ANDS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        ANDS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ANDSQ     D0Re0,[D0AR.0+D0ARI.0++],A0FrP,A0.2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0FrP++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0FrP++]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.2++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.2++]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.7++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.7++]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.8++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0FrP+A0.8++]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0FrP],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0FrP]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0FrP]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0FrP++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0FrP++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0FrP++]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0FrP--],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP--],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0FrP--],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0FrP--]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0FrP--]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.2+A0FrP++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.2+A0FrP++]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.2++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.2++]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.7++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.7++]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.8++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.2+A0.8++]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.2],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0.2],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0.2],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.2]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.2]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.2++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0.2++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0.2++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.2++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.2++]
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.2--],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0.2--],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A0.2--],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.2--]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.2--]
        DL        ANDS      D0.7,D0Re0,D0Re0
        DLM       ADDSB8    D0.7,D0Re0,D0Re0
        D         ADDSB8    D0.7,D0Re0,D0.7
        DLQ       ANDSQ     D0.7,D0Re0,D0.7,A0.2
        D         ANDSQ     D0.7,D0Re0,D1.7,A0.2
        DLQ       ANDSQ     D0.7,D0Re0,D0.16,A0.2
        D         ANDSQ     D0.7,D0Re0,[D0AR.0+D0ARI.0++],A0.2
        DT        ANDS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T2
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T2
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T2
        DT        ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.7+A0FrP++]
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.7+A0FrP++]
        DT        ANDS      D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T2
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T2
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T2
        DT        ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.2++]
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.2++]
        D         ANDS      D0.7,D0.7,D0Re0
        DLQ       ANDSQ     D0.7,D0.7,D0Re0,A0.2
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T2
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T2
        D         ADDST8    D0.7,D0.7,D0.7
        DM        ADDSB8    D0.7,D0.7,D0.7
        D         ANDS      D0.7,D0.7,[D0AR.0]
        DLQ       ANDSQ     D0.7,D0.7,RD,A0.2
        DT        ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.7++]
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.7++]
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.7++]
        D         ANDSQ     D0.7,D0.7,A0.2,A0.2
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T2
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T2
        DWT       ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.8++]
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.8++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.7+A0.8++]
        DL        ADDST8    D0.7,[D0AR.0],D0Re0
        D         ANDS      D0.7,[D0AR.0],A1LbP
        DLT       ANDS      D0Re0,D0Re0,D0Re0 [A0.7],T2
        DMT       ADDST8    D0Re0,D0Re0,D0Re0 [A0.7],T2
        D         ANDS      D0.7,[D0AR.0],D0.7
        D         ANDSQ     D0.7,[D0AR.0],A1.7,A0.2
        D         ANDSQ     D0.7,[D0AR.0],D1Re0,A0.2
        DL        ANDSQ     D0.7,[D0AR.0],[D0AR.0],A0.2
        D         ANDS      D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DLT       ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.7]
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.7]
        D         ANDS      D0.7,[D0AR.0],A0FrP
        D         ADDST8    D0.7,[D0AR.0++],D0Re0
        D         ANDSQ     D0.7,[D0AR.0++],A1LbP,A0.2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 [A0.7++],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A0.7++],T2
        D         ANDS      D0.7,[D0AR.0++],A1.7
        DL        ANDSQ     D0.7,[D0AR.0++],D0.7,A0.2
        DL        ANDS      D0.7,[D0AR.0++],[D0AR.0]
        DL        ANDS      D0.7,[D0AR.0++],[D0AR.0++]
        DT        ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.7++]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A0.7++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.7++]
        D         ANDSQ     D0.7,[D0AR.0++],A0.7,A0.2
        DL        ANDSQ     D0.7,[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         ADDSB8    D0.7,[D0AR.0+D0ARI.0++],D0.7
        DLQ       ANDSQ     D0.7,[D0AR.0+D0ARI.0++],D0.7,A0.2
        D         ANDSQ     D0.7,[D0AR.0+D0ARI.0++],[D0AR.0],A0.2
        D         ANDSQ     D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++],A0.2
        D         ANDS      D0.7,[D0AR.0+D0ARI.0++],A0.2
        DT        ADDST8    D0Re0,D0Re0,D0Re0 [A0.7--],T2
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.7--],T2
        DT        ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.7--]
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.7--]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.7--]
        DT        ADDST8    D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T2
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T2
        DT        ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.8+A0FrP++]
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8+A0FrP++]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8+A0FrP++]
        DT        ADDST8    D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T2
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T2
        DT        ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.2++]
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.2++]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.2++]
        DT        ADDST8    D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T2
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T2
        DT        ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.7++]
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.7++]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.7++]
        DT        ADDST8    D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T2
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T2
        DT        ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.8++]
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.8++]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8+A0.8++]
        DT        ADDST8    D0Re0,D0Re0,D0Re0 [A0.8],T2
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.8],T2
        DT        ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.8]
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8]
        DT        ADDST8    D0Re0,D0Re0,D0Re0 [A0.8++],T2
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.8++],T2
        DT        ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.8++]
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8++]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8++]
        DT        ADDST8    D0Re0,D0Re0,D0Re0 [A0.8--],T2
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 [A0.8--],T2
        DT        ANDS      D0Re0,D0Re0,D0Re0 T2,[A0.8--]
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8--]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A0.8--]
        DL        ADDSB8    [D0AW.0],D0Re0,D0Re0
        DLM       ADDST8    [D0AW.0],D0Re0,D0Re0
        D         ADDST8    [D0AW.0],D0Re0,D0.7
        DM        ADDSB8    [D0AW.0],D0Re0,D0.7
        D         ANDS      [D0AW.0],D0Re0,[D0AR.0]
        DLQ       ANDSQ     [D0AW.0],D0Re0,RD,A0.2
        DL        ANDSQ     [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++],A0.2
        D         ADDSB8    [D0AW.0],D0.7,D0Re0
        DM        ADDSB8    [D0AW.0],D0.7,D0Re0
        D         ANDS      [D0AW.0],D0.7,D0.7
        D         ANDSQ     [D0AW.0],D0.7,A1.7,A0.2
        D         ANDSQ     [D0AW.0],D0.7,D1Re0,A0.2
        DL        ANDSQ     [D0AW.0],D0.7,[D0AR.0],A0.2
        D         ANDS      [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        D         ANDS      [D0AW.0],D0.7,A0.7
        D         ANDSQ     [D0AW.0],[D0AR.0],D0Re0,A0.2
        D         ANDSQ     [D0AW.0],[D0AR.0],A1LbP,A0.2
        DL        ADDSB8    [D0AW.0],[D0AR.0],D0.7
        DLM       ADDSB8    [D0AW.0],D0.16,D0.7
        DL        ANDS      [D0AW.0],[D0AR.0],[D0AR.0]
        DL        ANDS      [D0AW.0],[D0AR.0],[D0AR.0++]
        D         ANDSQ     [D0AW.0],[D0AR.0],A0FrP,A0.2
        DL        ANDS      [D0AW.0],[D0AR.0++],D0Re0
        D         ANDS      [D0AW.0],[D0AR.0++],A1.3
        DL        ADDST8    [D0AW.0],[D0AR.0++],D0.7
        D         ANDSQ     [D0AW.0],[D0AR.0++],D1.7,A0.2
        DLQ       ANDSQ     [D0AW.0],[D0AR.0++],D0.16,A0.2
        D         ANDSQ     [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        D         ANDS      [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DLQ       ANDSQ     [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         ADDST8    [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        D         ANDS      [D0AW.0],[D0AR.0+D0ARI.0++],D1Re0
        D         ANDSQ     [D0AW.0],[D0AR.0+D0ARI.0++],RD,A0.2
        DL        ANDSQ     [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++],A0.2
        D         ANDSQ     [D0AW.0],[D0AR.0+D0ARI.0++],A0.2,A0.2
        DLT       ADDST8    D0Re0,D0Re0,D0Re0 [D0AW.0],T2
        DT        ANDS      D0Re0,D0Re0,D0Re0 T2,[D0AR.0]
        DLQT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[D0AR.0]
        DL        ANDS      [D0AW.0++],D0Re0,D0Re0
        DLM       ADDSB8    [D0AW.0++],D0Re0,D0Re0
        D         ADDSB8    [D0AW.0++],D0Re0,D0.7
        DLQ       ANDSQ     [D0AW.0++],D0Re0,D0.7,A0.2
        D         ANDSQ     [D0AW.0++],D0Re0,D1.7,A0.2
        DLQ       ANDSQ     [D0AW.0++],D0Re0,D0.16,A0.2
        D         ANDSQ     [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++],A0.2
        DT        ANDS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T2
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T2
        DLQT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1LbP++],T2
        DLMT      ADDST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T2
        DLT       ANDS      D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1LbP++]
        DLT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP+A1LbP++]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1LbP++]
        DT        ADDST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T2
        DT        ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.3++],T2
        DMT       ADDST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T2
        DT        ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.3++]
        DLT       ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.3++]
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.3++]
        D         ANDS      [D0AW.0++],D0.7,D0Re0
        DLQ       ANDSQ     [D0AW.0++],D0.7,D0Re0,A0.2
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.7++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T2
        DL        ADDSB8    [D0AW.0++],D0.7,D0.7
        DLM       ADDSB8    [D0AW.0++],D0.7,D0.7
        DL        ANDS      [D0AW.0++],D0.7,[D0AR.0]
        DL        ANDS      [D0AW.0++],D0.7,[D0AR.0++]
        DT        ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.7++]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.7++]
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP+A1.7++]
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.7++]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.8++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.8++]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP+A1.8++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP+A1.8++]
        DL        ADDST8    [D0AW.0++],[D0AR.0],D0Re0
        D         ANDS      [D0AW.0++],[D0AR.0],A1LbP
        DLT       ANDS      D0Re0,D0Re0,D0Re0 [A1LbP],T2
        DLT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1LbP],T2
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1LbP],T2
        D         ADDSB8    [D0AW.0++],[D0AR.0],D0.7
        DLQ       ANDSQ     [D0AW.0++],[D0AR.0],D0.7,A0.2
        D         ANDSQ     [D0AW.0++],[D0AR.0],D1.7,A0.2
        DLQ       ANDSQ     [D0AW.0++],[D0AR.0],D0.16,A0.2
        DWT       ANDS      D0Re0,D0Re0,D0Re0 T2,[A1LbP]
        DLT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP]
        DLT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP]
        D         ANDS      [D0AW.0++],[D0AR.0],A0.7
        D         ANDSQ     [D0AW.0++],[D0AR.0++],D0Re0,A0.2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 [A1LbP++],T2
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1LbP++],T2
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1LbP++],T2
        D         ANDSQ     [D0AW.0++],[D0AR.0++],A1.3,A0.2
        D         ANDSQ     [D0AW.0++],[D0AR.0++],D0.7,A0.2
        D         ANDS      [D0AW.0++],[D0AR.0++],[D0AR.0]
        DLQ       ANDSQ     [D0AW.0++],[D0AR.0++],RD,A0.2
        DT        ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP++]
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP++]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1LbP++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP++]
        D         ANDSQ     [D0AW.0++],[D0AR.0++],A0.7,A0.2
        DL        ANDSQ     [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         ADDSB8    [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DLQ       ANDSQ     [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7,A0.2
        D         ANDSQ     [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0],A0.2
        D         ANDSQ     [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++],A0.2
        D         ANDS      [D0AW.0++],[D0AR.0+D0ARI.0++],A0.2
        DT        ADDST8    D0Re0,D0Re0,D0Re0 [A1LbP--],T2
        DT        ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1LbP--],T2
        DMT       ADDST8    D0Re0,D0Re0,D0Re0 [A1LbP--],T2
        DT        ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP--]
        DLT       ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1LbP--]
        DMT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1LbP--]
        DT        ANDS      D0Re0,D0Re0,D0Re0 [D0AW.0++],T2
        DLQT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [D0AW.0++],T2
        DLT       ANDS      D0Re0,D0Re0,D0Re0 T2,[D0AR.0++]
        DLMT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[D0AR.0++]
        D         ANDSQ     [D0AW.0+D0AWI.0++],D0Re0,D0Re0,A0.2
        D         ANDSQ     [D0AW.0+D0AWI.0++],D0Re0,A1LbP,A0.2
        DL        ADDSB8    [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DLM       ADDSB8    [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DL        ANDS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DL        ANDS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        D         ANDSQ     [D0AW.0+D0AWI.0++],D0Re0,A0FrP,A0.2
        DL        ANDS      [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DLM       ADDSB8    [D0AW.0+D0AWI.0++],D0.7,D0Re0
        D         ADDSB8    [D0AW.0+D0AWI.0++],D0.7,D0.7
        DLQ       ANDSQ     [D0AW.0+D0AWI.0++],D0.7,D0.7,A0.2
        D         ANDSQ     [D0AW.0+D0AWI.0++],D0.7,D1.7,A0.2
        DLQ       ANDSQ     [D0AW.0+D0AWI.0++],D0.7,D0.16,A0.2
        D         ANDSQ     [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++],A0.2
        D         ANDS      [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DLQ       ANDSQ     [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0,A0.2
        D         ANDSQ     [D0AW.0+D0AWI.0++],[D0AR.0],A1.3,A0.2
        D         ANDSQ     [D0AW.0+D0AWI.0++],[D0AR.0],D0.7,A0.2
        D         ANDS      [D0AW.0+D0AWI.0++],[D0AR.0],D1Re0
        D         ANDSQ     [D0AW.0+D0AWI.0++],[D0AR.0],RD,A0.2
        DL        ANDSQ     [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++],A0.2
        D         ANDSQ     [D0AW.0+D0AWI.0++],[D0AR.0],A0.2,A0.2
        DL        ADDST8    [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         ANDS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        D         ANDSQ     [D0AW.0+D0AWI.0++],[D0AR.0++],A1.7,A0.2
        D         ANDS      [D0AW.0+D0AWI.0++],[D0AR.0++],RD
        D         ANDS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        D         ANDS      [D0AW.0+D0AWI.0++],[D0AR.0++],A0FrP
        D         ADDST8    [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         ANDSQ     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A1LbP,A0.2
        DL        ADDSB8    [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        D         ANDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D1.7
        DL        ANDSQ     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],RD,A0.2
        DL        ANDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ANDSQ     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A0.7,A0.2
        DLT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [D0AW.0+D0AWI.0++],T2
        DT        ADDST8    D0Re0,D0Re0,D0Re0 T2,[D0AR.0+D0ARI.0++]
        DMT       ADDST8    D0Re0,D0Re0,D0Re0 T2,[D0AR.0+D0ARI.0++]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1LbP++],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3+A1LbP++]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.3+A1LbP++]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3+A1LbP++]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.3++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.3++]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.3+A1.3++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.3++]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.7++],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.7++]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.7++]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.7++]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.8++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.8++]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.3+A1.8++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3+A1.8++]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.3],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.3],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.3],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.3]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1.3++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1.3++],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.3++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1.3++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1.3++]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.3++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3++]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.3--],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.3--],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.3--],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3--]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.3--]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.3--]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1LbP++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1.7+A1LbP++]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7+A1LbP++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.7+A1LbP++]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.3++],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.3++]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.3++]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.3++]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.7++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.7++]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7+A1.7++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.7++]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.8++],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.8++]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.8++]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.7+A1.8++]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1.7],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1.7],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1.7],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1.7]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.7]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.7++],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7++],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.7++],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.7++]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.7++]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.7++]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1.7--],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1.7--],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7--],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1.7--],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1.7--]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.7--]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.7--]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1LbP++],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8+A1LbP++]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.8+A1LbP++]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8+A1LbP++]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.3++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.3++]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.8+A1.3++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.3++]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.7++],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.7++]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.7++]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.7++]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.8++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.8++]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.8+A1.8++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8+A1.8++]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.8],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.8],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.8],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.8]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8]
        DWT       ANDS      D0Re0,D0Re0,D0Re0 [A1.8++],T2
        DLWT      ADDSB8    D0Re0,D0Re0,D0Re0 [A1.8++],T2
        DLQWT     ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.8++],T2
        DLMWT     ADDST8    D0Re0,D0Re0,D0Re0 [A1.8++],T2
        DLWT      ANDS      D0Re0,D0Re0,D0Re0 T2,[A1.8++]
        DLWT      ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 T2,[A1.8++]
        DLMWT     ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8++]
        DWT       ADDST8    D0Re0,D0Re0,D0Re0 [A1.8--],T2
        DWT       ANDSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.8--],T2
        DMWT      ADDST8    D0Re0,D0Re0,D0Re0 [A1.8--],T2
        DWT       ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8--]
        DLWT      ADDST8    D0Re0,D0Re0,D0Re0 T2,[A1.8--]
        DMWT      ADDSB8    D0Re0,D0Re0,D0Re0 T2,[A1.8--]
        D         ADDSB8    D1Re0,D1Re0,D1Re0
        DLM       ADDST8    D1Re0,D1Re0,D1Re0
        D         ANDS      D1Re0,D1Re0,D1.7
        DM        ADDST8    D1Re0,D1Re0,D1.7
        D         ANDSQ     D1Re0,D1Re0,D0Re0,A1.3
        D         ANDS      D1Re0,D1Re0,A0FrP
        D         ANDS      D1Re0,D1Re0,[D1BR.1++]
        D         ANDSQ     D1Re0,D1Re0,A0.7,A1.3
        DM        ADDSB8    D1Re0,D1.7,D1Re0
        D         ANDSQ     D1Re0,D1.7,A1LbP,A1.3
        DL        ADDSB8    D1Re0,D1.7,D1.7
        DL        ANDSQ     D1Re0,D1.7,D1.7,A1.3
        DL        ANDSQ     D1Re0,D1.7,RD,A1.3
        DL        ANDS      D1Re0,D1.7,[D1BR.1]
        D         ANDS      D1Re0,D1.7,A0.7
        DLM       ADDST8    D1Re0,D1.16,D1Re0
        DL        ANDS      D1Re0,[D1BR.1],D1Re0
        D         ANDS      D1Re0,[D1BR.1],A1.3
        DL        ADDST8    D1Re0,[D1BR.1],D1.7
        D         ANDSQ     D1Re0,[D1BR.1],D0.7,A1.3
        D         ANDS      D1Re0,[D1BR.1],A0.2
        D         ANDSQ     D1Re0,[D1BR.1],[D1BR.1++],A1.3
        D         ANDS      D1Re0,[D1BR.1++],D1Re0
        DLQ       ANDSQ     D1Re0,[D1BR.1++],D1Re0,A1.3
        D         ADDST8    D1Re0,[D1BR.1++],D1.7
        D         ANDS      D1Re0,[D1BR.1++],D0Re0
        DLQ       ANDSQ     D1Re0,[D1BR.1++],RD,A1.3
        DL        ANDSQ     D1Re0,[D1BR.1++],[D1BR.1],A1.3
        D         ANDSQ     D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        ADDST8    D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         ANDS      D1Re0,[D1BR.1+D1BRI.1++],D1.7
        D         ANDSQ     D1Re0,[D1BR.1+D1BRI.1++],A1.7,A1.3
        D         ANDSQ     D1Re0,[D1BR.1+D1BRI.1++],RD,A1.3
        D         ANDS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ANDS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DT        ADDSB8    D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T2
        DLT       ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T2
        DLMT      ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T2
        DT        ANDS      D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0FrP++]
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0FrP++]
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0FrP++]
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0FrP+A0FrP++]
        DLT       ANDS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T2
        DMT       ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T2
        DLT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.2++],T2
        DT        ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.2++]
        DMT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.2++]
        DT        ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0FrP+A0.2++]
        DT        ADDSB8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T2
        DLT       ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T2
        DLMT      ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T2
        DT        ANDS      D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.7++]
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.7++]
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.7++]
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0FrP+A0.7++]
        DLT       ANDS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T2
        DMT       ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T2
        DLT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.8++],T2
        DT        ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.8++]
        DMT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP+A0.8++]
        DT        ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0FrP+A0.8++]
        DT        ADDSB8    D1Re0,D1Re0,D1Re0 [A0FrP],T2
        DLT       ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP],T2
        DLMT      ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP],T2
        DT        ANDS      D1Re0,D1Re0,D1Re0 T2,[A0FrP]
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP]
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP]
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0FrP]
        DLT       ANDS      D1Re0,D1Re0,D1Re0 [A0FrP++],T2
        DMT       ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP++],T2
        DLT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0FrP++],T2
        DT        ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0FrP++]
        DMT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP++]
        DT        ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0FrP++]
        DT        ADDSB8    D1Re0,D1Re0,D1Re0 [A0FrP--],T2
        DLT       ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP--],T2
        DLMT      ADDST8    D1Re0,D1Re0,D1Re0 [A0FrP--],T2
        DT        ANDS      D1Re0,D1Re0,D1Re0 T2,[A0FrP--]
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP--]
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0FrP--]
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0FrP--]
        DLT       ANDS      D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T2
        DMT       ADDST8    D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T2
        DLT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0FrP++],T2
        DT        ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.2+A0FrP++]
        DMT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.2+A0FrP++]
        DT        ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2+A0FrP++]
        DT        ADDSB8    D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T2
        DLT       ADDST8    D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T2
        DLMT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T2
        DT        ANDS      D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.2++]
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.2++]
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.2++]
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2+A0.2++]
        DLT       ANDS      D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T2
        DMT       ADDST8    D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T2
        DLT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.7++],T2
        DT        ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.7++]
        DMT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.7++]
        DT        ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2+A0.7++]
        DT        ADDSB8    D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T2
        DLT       ADDST8    D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T2
        DLMT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T2
        DT        ANDS      D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.8++]
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.8++]
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.2+A0.8++]
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2+A0.8++]
        DLT       ANDS      D1Re0,D1Re0,D1Re0 [A0.2],T2
        DMT       ADDST8    D1Re0,D1Re0,D1Re0 [A0.2],T2
        DLT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2],T2
        DT        ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.2]
        DMT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.2]
        DT        ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2]
        DT        ADDSB8    D1Re0,D1Re0,D1Re0 [A0.2++],T2
        DLT       ADDST8    D1Re0,D1Re0,D1Re0 [A0.2++],T2
        DLMT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.2++],T2
        DT        ANDS      D1Re0,D1Re0,D1Re0 T2,[A0.2++]
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.2++]
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.2++]
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2++]
        DLT       ANDS      D1Re0,D1Re0,D1Re0 [A0.2--],T2
        DMT       ADDST8    D1Re0,D1Re0,D1Re0 [A0.2--],T2
        DLT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2--],T2
        DT        ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.2--]
        DMT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.2--]
        DT        ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.2--]
        D         ADDST8    D1.7,D1Re0,D1Re0
        D         ANDSQ     D1.7,D1Re0,D1Re0,A1.3
        D         ANDS      D1.7,D1Re0,A1.7
        DLM       ADDSB8    D1.7,D1Re0,D1.7
        D         ANDS      D1.7,D1Re0,D0.7
        D         ANDSQ     D1.7,D1Re0,A0FrP,A1.3
        DL        ANDS      D1.7,D1Re0,[D1BR.1++]
        DL        ANDSQ     D1.7,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T2
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T2
        DLWT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0FrP++],T2
        DWT       ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.7+A0FrP++]
        DMWT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.7+A0FrP++]
        DWT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7+A0FrP++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T2
        DWT       ANDS      D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.2++]
        DLWT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.2++]
        DLMWT     ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.2++]
        DLQWT     ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7+A0.2++]
        DM        ADDST8    D1.7,D1.7,D1Re0
        DT        ANDS      D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T2
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T2
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T2
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.7++],T2
        DL        ANDS      D1.7,D1.7,D1.7
        D         ANDSQ     D1.7,D1.7,A1.7,A1.3
        D         ANDSQ     D1.7,D1.7,RD,A1.3
        DT        ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.7++]
        DMT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.7++]
        DT        ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7+A0.7++]
        D         ANDS      D1.7,D1.7,A0.2
        D         ANDSQ     D1.7,D1.7,[D1BR.1++],A1.3
        DT        ANDS      D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T2
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T2
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T2
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.8++],T2
        DLT       ANDS      D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.8++]
        DMT       ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.7+A0.8++]
        DLT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7+A0.8++]
        DT        ANDS      D1Re0,D1Re0,D1Re0 [A0.7],T2
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 [A0.7],T2
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 [A0.7],T2
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7],T2
        DT        ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.7]
        DLT       ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.7]
        DLMT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.7]
        DT        ANDS      D1Re0,D1Re0,D1Re0 [A0.7++],T2
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 [A0.7++],T2
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 [A0.7++],T2
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7++],T2
        DLT       ANDS      D1Re0,D1Re0,D1Re0 T2,[A0.7++]
        DMT       ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.7++]
        DLT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.7++]
        DT        ADDST8    D1Re0,D1Re0,D1Re0 [A0.7--],T2
        DMT       ADDSB8    D1Re0,D1Re0,D1Re0 [A0.7--],T2
        DT        ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7--],T2
        DT        ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.7--]
        DLT       ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.7--]
        DLMT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.7--]
        D         ANDS      D1.7,[D1BR.1],D1Re0
        DLQ       ANDSQ     D1.7,[D1BR.1],D1Re0,A1.3
        D         ADDST8    D1.7,[D1BR.1],D1.7
        D         ANDS      D1.7,[D1BR.1],D0Re0
        DLQ       ANDSQ     D1.7,[D1BR.1],RD,A1.3
        DL        ANDSQ     D1.7,[D1BR.1],[D1BR.1],A1.3
        D         ANDSQ     D1.7,[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        DL        ADDST8    D1.7,[D1BR.1++],D1Re0
        D         ANDS      D1.7,[D1BR.1++],D1.7
        D         ANDSQ     D1.7,[D1BR.1++],A1.7,A1.3
        D         ANDSQ     D1.7,[D1BR.1++],RD,A1.3
        D         ANDS      D1.7,[D1BR.1++],[D1BR.1]
        D         ANDS      D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ADDST8    D1.7,[D1BR.1+D1BRI.1++],D1Re0
        D         ANDSQ     D1.7,[D1BR.1+D1BRI.1++],A1LbP,A1.3
        DL        ADDSB8    D1.7,[D1BR.1+D1BRI.1++],D1.7
        D         ANDS      D1.7,[D1BR.1+D1BRI.1++],D0.7
        D         ANDSQ     D1.7,[D1BR.1+D1BRI.1++],A0FrP,A1.3
        DL        ANDS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        ANDSQ     D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T2
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T2
        DLWT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0FrP++],T2
        DWT       ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.8+A0FrP++]
        DMWT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.8+A0FrP++]
        DWT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8+A0FrP++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T2
        DWT       ANDS      D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.2++]
        DLWT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.2++]
        DLMWT     ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.2++]
        DLQWT     ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8+A0.2++]
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T2
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T2
        DLWT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.7++],T2
        DWT       ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.7++]
        DMWT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.7++]
        DWT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8+A0.7++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T2
        DWT       ANDS      D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.8++]
        DLWT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.8++]
        DLMWT     ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.8+A0.8++]
        DLQWT     ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8+A0.8++]
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 [A0.8],T2
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.8],T2
        DLWT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8],T2
        DWT       ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.8]
        DMWT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.8]
        DWT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A0.8++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.8++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A0.8++],T2
        DWT       ANDS      D1Re0,D1Re0,D1Re0 T2,[A0.8++]
        DLWT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.8++]
        DLMWT     ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.8++]
        DLQWT     ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8++]
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 [A0.8--],T2
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 [A0.8--],T2
        DLWT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8--],T2
        DWT       ADDST8    D1Re0,D1Re0,D1Re0 T2,[A0.8--]
        DMWT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[A0.8--]
        DWT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 T2,[A0.8--]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1LbP++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1LbP++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.3++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.3++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.7++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.7++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.8++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1LbP+A1.8++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1LbP],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1LbP]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1LbP]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1LbP++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1LbP++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1LbP++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1LbP--],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP--],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1LbP--],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1LbP--]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1LbP--]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.3+A1LbP++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.3+A1LbP++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.3++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.3++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.7++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.7++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.8++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.3+A1.8++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.3],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.3],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.3],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.3]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.3]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.3++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.3++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.3++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.3++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.3++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.3--],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.3--],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.3--],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.3--]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.3--]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.7+A1LbP++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.7+A1LbP++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.3++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.3++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.7++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.7++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.8++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.7+A1.8++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.7],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.7],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.7],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.7]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.7]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.7++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.7++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.7++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.7++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.7++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.7--],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.7--],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.7--],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.7--]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.7--]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.8+A1LbP++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.8+A1LbP++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.3++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.3++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.7++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.7++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.8++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.8+A1.8++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.8],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.8],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.8],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.8]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.8]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.8++],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.8++],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.8++],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.8++]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.8++]
        DWT       ADDSB8    D1Re0,D1Re0,D1Re0 [A1.8--],T2
        DLWT      ADDST8    D1Re0,D1Re0,D1Re0 [A1.8--],T2
        DLMWT     ADDST8    D1Re0,D1Re0,D1Re0 [A1.8--],T2
        DLWT      ANDS      D1Re0,D1Re0,D1Re0 T2,[A1.8--]
        DMWT      ADDST8    D1Re0,D1Re0,D1Re0 T2,[A1.8--]
        DL        ANDS      [D1BW.1],D1Re0,D1Re0
        DL        ANDSQ     [D1BW.1],D1Re0,D1Re0,A1.3
        D         ADDSB8    [D1BW.1],D1Re0,D1.7
        DLM       ADDST8    [D1BW.1],D1Re0,D1.7
        D         ANDSQ     [D1BW.1],D1Re0,D0.7,A1.3
        D         ANDS      [D1BW.1],D1Re0,A0.2
        D         ANDSQ     [D1BW.1],D1Re0,[D1BR.1++],A1.3
        D         ANDS      [D1BW.1],D1.7,D1Re0
        DLM       ADDSB8    [D1BW.1],D1.7,D1Re0
        D         ANDSQ     [D1BW.1],D1.7,A1.3,A1.3
        DM        ADDSB8    [D1BW.1],D1.7,D1.7
        D         ANDS      [D1BW.1],D1.7,D0Re0
        DLQ       ANDSQ     [D1BW.1],D1.7,RD,A1.3
        DL        ANDSQ     [D1BW.1],D1.7,[D1BR.1],A1.3
        D         ANDSQ     [D1BW.1],D1.7,[D1BR.1+D1BRI.1++],A1.3
        DM        ADDST8    [D1BW.1],D1.16,D1.7
        DLT       ADDST8    D1Re0,D1Re0,D1Re0 [D1BW.1],T2
        DT        ANDS      D1Re0,D1Re0,D1Re0 T2,[D1BR.1]
        DLMT      ADDSB8    D1Re0,D1Re0,D1Re0 T2,[D1BR.1]
        DL        ANDS      [D1BW.1],[D1BR.1],D1Re0
        D         ANDS      [D1BW.1],[D1BR.1],A1.3
        DL        ADDST8    [D1BW.1],[D1BR.1],D1.7
        D         ANDSQ     [D1BW.1],[D1BR.1],D0.7,A1.3
        D         ANDS      [D1BW.1],[D1BR.1],A0.2
        D         ANDSQ     [D1BW.1],[D1BR.1],[D1BR.1++],A1.3
        D         ANDS      [D1BW.1],[D1BR.1++],D1Re0
        DLQ       ANDSQ     [D1BW.1],[D1BR.1++],D1Re0,A1.3
        D         ADDST8    [D1BW.1],[D1BR.1++],D1.7
        D         ANDS      [D1BW.1],[D1BR.1++],D0Re0
        DLQ       ANDSQ     [D1BW.1],[D1BR.1++],RD,A1.3
        DL        ANDSQ     [D1BW.1],[D1BR.1++],[D1BR.1],A1.3
        D         ANDSQ     [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        ADDST8    [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        D         ANDS      [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        D         ANDSQ     [D1BW.1],[D1BR.1+D1BRI.1++],A1.7,A1.3
        D         ANDSQ     [D1BW.1],[D1BR.1+D1BRI.1++],RD,A1.3
        D         ANDS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ANDS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ADDST8    [D1BW.1++],D1Re0,D1Re0
        D         ANDSQ     [D1BW.1++],D1Re0,D1Re0,A1.3
        D         ANDS      [D1BW.1++],D1Re0,A1.7
        DLM       ADDSB8    [D1BW.1++],D1Re0,D1.7
        D         ANDS      [D1BW.1++],D1Re0,D0.7
        D         ANDSQ     [D1BW.1++],D1Re0,A0FrP,A1.3
        DL        ANDS      [D1BW.1++],D1Re0,[D1BR.1++]
        DL        ANDSQ     [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DM        ADDST8    [D1BW.1++],D1.7,D1Re0
        D         ANDS      [D1BW.1++],D1.7,A1.3
        DL        ADDST8    [D1BW.1++],D1.7,D1.7
        DLQ       ANDSQ     [D1BW.1++],D1.7,D1.7,A1.3
        DLQ       ANDSQ     [D1BW.1++],D1.7,D1.16,A1.3
        D         ANDSQ     [D1BW.1++],D1.7,[D1BR.1],A1.3
        DL        ANDS      [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DM        ADDSB8    [D1BW.1++],D1.16,D1.7
        DLT       ADDSB8    D1Re0,D1Re0,D1Re0 [D1BW.1++],T2
        DLQT      ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [D1BW.1++],T2
        DMT       ADDST8    D1Re0,D1Re0,D1Re0 T2,[D1BR.1++]
        D         ADDST8    [D1BW.1++],[D1BR.1],D1Re0
        D         ANDSQ     [D1BW.1++],[D1BR.1],A1LbP,A1.3
        DL        ADDSB8    [D1BW.1++],[D1BR.1],D1.7
        D         ANDS      [D1BW.1++],[D1BR.1],D0.7
        D         ANDSQ     [D1BW.1++],[D1BR.1],A0FrP,A1.3
        DL        ANDS      [D1BW.1++],[D1BR.1],[D1BR.1++]
        DL        ANDSQ     [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        DL        ANDSQ     [D1BW.1++],[D1BR.1++],D1Re0,A1.3
        D         ADDSB8    [D1BW.1++],[D1BR.1++],D1.7
        DLQ       ANDSQ     [D1BW.1++],[D1BR.1++],D1.7,A1.3
        DLQ       ANDSQ     [D1BW.1++],[D1BR.1++],D1.16,A1.3
        D         ANDSQ     [D1BW.1++],[D1BR.1++],[D1BR.1],A1.3
        DL        ANDS      [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        ADDSB8    [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         ANDSQ     [D1BW.1++],[D1BR.1+D1BRI.1++],A1.3,A1.3
        D         ANDSQ     [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7,A1.3
        D         ANDS      [D1BW.1++],[D1BR.1+D1BRI.1++],RD
        D         ANDSQ     [D1BW.1++],[D1BR.1+D1BRI.1++],A0.2,A1.3
        DL        ANDSQ     [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++],A1.3
        D         ADDSB8    [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        DLM       ADDST8    [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        D         ANDS      [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DM        ADDST8    [D1BW.1+D1BWI.1++],D1Re0,D1.7
        D         ANDSQ     [D1BW.1+D1BWI.1++],D1Re0,D0Re0,A1.3
        D         ANDS      [D1BW.1+D1BWI.1++],D1Re0,A0FrP
        D         ANDS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        D         ANDSQ     [D1BW.1+D1BWI.1++],D1Re0,A0.7,A1.3
        DM        ADDSB8    [D1BW.1+D1BWI.1++],D1.7,D1Re0
        D         ANDSQ     [D1BW.1+D1BWI.1++],D1.7,A1LbP,A1.3
        DL        ADDSB8    [D1BW.1+D1BWI.1++],D1.7,D1.7
        DL        ANDSQ     [D1BW.1+D1BWI.1++],D1.7,D1.7,A1.3
        DL        ANDSQ     [D1BW.1+D1BWI.1++],D1.7,RD,A1.3
        DL        ANDS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        D         ANDS      [D1BW.1+D1BWI.1++],D1.7,A0.7
        DLM       ADDST8    [D1BW.1+D1BWI.1++],D1.16,D1Re0
        DLT       ANDS      D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T2
        DLT       ANDSQ     D1Re0,D1Re0,D1Re0,A1.3 [D1BW.1+D1BWI.1++],T2
        DMT       ADDSB8    D1Re0,D1Re0,D1Re0 T2,[D1BR.1+D1BRI.1++]
        D         ADDSB8    [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        D         ANDS      [D1BW.1+D1BWI.1++],[D1BR.1],A1LbP
        DL        ANDS      [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        D         ANDSQ     [D1BW.1+D1BWI.1++],[D1BR.1],D0Re0,A1.3
        D         ANDS      [D1BW.1+D1BWI.1++],[D1BR.1],A0FrP
        D         ANDS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        D         ANDSQ     [D1BW.1+D1BWI.1++],[D1BR.1],A0.7,A1.3
        D         ANDSQ     [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0,A1.3
        D         ANDS      [D1BW.1+D1BWI.1++],[D1BR.1++],A1.7
        DL        ANDSQ     [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7,A1.3
        DL        ANDSQ     [D1BW.1+D1BWI.1++],[D1BR.1++],RD,A1.3
        DL        ANDS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        D         ANDS      [D1BW.1+D1BWI.1++],[D1BR.1++],A0.7
        DL        ANDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         ANDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        ADDST8    [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         ANDSQ     [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D0.7,A1.3
        D         ANDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0.2
        D         ANDSQ     [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++],A1.3
        D         ANDS      D0Re0,D0Re0,#0x7fff
        D         ANDS      D0.7,D0.7,#-0x8000
        D         ANDS      [D0AW.0],[D0AR.0],#-0x1
        D         ANDS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x7fff
        D         ANDS      D1Re0,D1Re0,#-0x8000
        D         ANDS      D1.7,D1.7,#-0x1
        D         ANDS      [D1BW.1++],[D1BR.1++],#0x7fff
        D         ANDS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#-0x8000
        DHNE      ANDSQPL   D0Re0,D0Re0,A1.3,A0.2
        DHNE      ANDSQPL   D0Re0,D0Re0,D1.7,A0.2
        DLLS      ANDSQLS   D0Re0,D0Re0,[D0AR.0+D0ARI.0++],A0.2
        DLLS      ANDSQLS   D0Re0,D0.7,D0Re0,A0.2
        D         ANDSQNV   D0Re0,D0.7,D0.7,A0.2
        D         ANDSQNV   D0Re0,D0.7,[D0AR.0],A0.2
        DECS      ANDSQLE   D0Re0,D0.7,A0.2,A0.2
        D         ANDSQNV   D0Re0,[D0AR.0],A1.7,A0.2
        D         ANDSQNV   D0Re0,[D0AR.0],A0.2,A0.2
        D         ANDSQNV   D0Re0,[D0AR.0++],D1.7,A0.2
        D         ANDSQNV   D0Re0,[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         ANDSQNV   D0Re0,[D0AR.0+D0ARI.0++],RD,A0.2
        DHNE      ANDSQPL   D0.7,D0Re0,A1LbP,A0.2
        DHNE      ANDSQPL   D0.7,D0Re0,D1Re0,A0.2
        DLNE      ANDSQNE   D0.7,D0Re0,[D0AR.0++],A0.2
        DHNE      ANDSQPL   D0.7,D0Re0,A0.7,A0.2
        DLNE      ANDSQNE   D0.7,D0.7,D0.7,A0.2
        DLNE      ANDSQNE   D0.7,D0.7,[D0AR.0],A0.2
        DHNE      ANDSQPL   D0.7,D0.7,A0FrP,A0.2
        D         ANDSQNV   D0.7,[D0AR.0],A1.3,A0.2
        DHCS      ANDSQVS   D0.7,[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        D         ANDSQNV   D0.7,[D0AR.0++],A1.7,A0.2
        D         ANDSQNV   D0.7,[D0AR.0++],A0.2,A0.2
        D         ANDSQNV   D0.7,[D0AR.0+D0ARI.0++],D1.7,A0.2
        DLLS      ANDSQLS   D0.16,D0Re0,D0Re0,A0.2
        D         ANDSQNV   D0.16,D0Re0,D0.7,A0.2
        D         ANDSQNV   D0.16,D0Re0,[D0AR.0],A0.2
        DECS      ANDSQLE   D0.16,D0Re0,A0.2,A0.2
        DHNE      ANDSQPL   D0.16,D0.7,A1.3,A0.2
        DHNE      ANDSQPL   D0.16,D0.7,D1.7,A0.2
        DLLS      ANDSQLS   D0.16,D0.7,[D0AR.0+D0ARI.0++],A0.2
        D         ANDSQNV   D0.16,[D0AR.0],D0Re0,A0.2
        D         ANDSQNV   D0.16,[D0AR.0],RD,A0.2
        D         ANDSQNV   D0.16,[D0AR.0++],A1.3,A0.2
        D         ANDSQNV   D0.16,[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        D         ANDSQNV   D0.16,[D0AR.0+D0ARI.0++],A1.7,A0.2
        D         ANDSQNV   D0.16,[D0AR.0+D0ARI.0++],A0.2,A0.2
        DHNE      ANDSQPL   D1Re0,D1Re0,D1.7,A1.3
        DECS      ANDSQLE   D1Re0,D1Re0,RD,A1.3
        DHNE      ANDSQPL   D1Re0,D1Re0,[D1BR.1++],A1.3
        DECS      ANDSQLE   D1Re0,D1.7,A1LbP,A1.3
        DECS      ANDSQLE   D1Re0,D1.7,D0Re0,A1.3
        DECS      ANDSQLE   D1Re0,D1.7,A0.2,A1.3
        D         ANDSQNV   D1Re0,D1.7,[D1BR.1+D1BRI.1++],A1.3
        D         ANDSQNV   D1Re0,[D1BR.1],D0.7,A1.3
        D         ANDSQNV   D1Re0,[D1BR.1++],D1Re0,A1.3
        D         ANDSQNV   D1Re0,[D1BR.1++],A0FrP,A1.3
        D         ANDSQNV   D1Re0,[D1BR.1+D1BRI.1++],A1.3,A1.3
        D         ANDSQNV   D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1],A1.3
        DECS      ANDSQLE   D1.7,D1Re0,A1.3,A1.3
        DECS      ANDSQLE   D1.7,D1Re0,D0.7,A1.3
        DHNE      ANDSQPL   D1.7,D1Re0,[D1BR.1],A1.3
        DHNE      ANDSQPL   D1.7,D1.7,D1Re0,A1.3
        D         ANDSQNV   D1.7,D1.7,D1.7,A1.3
        DECS      ANDSQLE   D1.7,D1.7,A0FrP,A1.3
        DHNE      ANDSQPL   D1.7,D1.7,[D1BR.1+D1BRI.1++],A1.3
        D         ANDSQNV   D1.7,[D1BR.1],A1.7,A1.3
        D         ANDSQNV   D1.7,[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        D         ANDSQNV   D1.7,[D1BR.1++],D0.7,A1.3
        D         ANDSQNV   D1.7,[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         ANDSQNV   D1.7,[D1BR.1+D1BRI.1++],A0FrP,A1.3
        DECS      ANDSQLE   D1.16,D1Re0,A1LbP,A1.3
        DECS      ANDSQLE   D1.16,D1Re0,D0Re0,A1.3
        DECS      ANDSQLE   D1.16,D1Re0,A0.2,A1.3
        D         ANDSQNV   D1.16,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DHNE      ANDSQPL   D1.16,D1.7,D1.7,A1.3
        DECS      ANDSQLE   D1.16,D1.7,RD,A1.3
        DHNE      ANDSQPL   D1.16,D1.7,[D1BR.1++],A1.3
        D         ANDSQNV   D1.16,[D1BR.1],A1.3,A1.3
        D         ANDSQNV   D1.16,[D1BR.1],[D1BR.1],A1.3
        D         ANDSQNV   D1.16,[D1BR.1++],A1.7,A1.3
        D         ANDSQNV   D1.16,[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        D         ANDSQNV   D1.16,[D1BR.1+D1BRI.1++],D0.7,A1.3
        D         OR        D0Re0,D0Re0,D0Re0
        DLQ       ORQ       D0Re0,D0Re0,D0Re0,A0.2
        D         ORQ       D0Re0,D0Re0,A1.3,A0.2
        D         ORQ       D0Re0,D0Re0,D0.7,A0.2
        D         OR        D0Re0,D0Re0,D1Re0
        D         ORQ       D0Re0,D0Re0,RD,A0.2
        DL        ORQ       D0Re0,D0Re0,[D0AR.0++],A0.2
        D         ORQ       D0Re0,D0Re0,A0.2,A0.2
        DL        SUBT8     D0Re0,D0.7,D0Re0
        D         OR        D0Re0,D0.7,A1LbP
        DL        OR        D0Re0,D0.7,D0.7
        DM        SUBT8     D0Re0,D0.7,D0.7
        D         OR        D0Re0,D0.7,RD
        D         OR        D0Re0,D0.7,[D0AR.0++]
        D         OR        D0Re0,D0.7,A0FrP
        D         SUBT8     D0Re0,[D0AR.0],D0Re0
        DM        SUBT8     D0Re0,D0.16,D0Re0
        D         OR        D0Re0,[D0AR.0],A1.7
        DL        ORQ       D0Re0,[D0AR.0],D0.7,A0.2
        D         OR        D0Re0,[D0AR.0],D1.7
        DL        ORQ       D0Re0,[D0AR.0],RD,A0.2
        DL        OR        D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ORQ       D0Re0,[D0AR.0],A0.7,A0.2
        DL        ORQ       D0Re0,[D0AR.0++],D0Re0,A0.2
        D         SUBB8     D0Re0,[D0AR.0++],D0.7
        DLQ       ORQ       D0Re0,[D0AR.0++],D0.7,A0.2
        D         ORQ       D0Re0,[D0AR.0++],[D0AR.0],A0.2
        D         ORQ       D0Re0,[D0AR.0++],[D0AR.0++],A0.2
        D         OR        D0Re0,[D0AR.0++],A0.2
        DL        SUBB8     D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         ORQ       D0Re0,[D0AR.0+D0ARI.0++],A1.3,A0.2
        D         ORQ       D0Re0,[D0AR.0+D0ARI.0++],D0.7,A0.2
        D         OR        D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DLQ       ORQ       D0Re0,[D0AR.0+D0ARI.0++],RD,A0.2
        DL        ORQ       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++],A0.2
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0FrP++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0FrP++]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.2++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.2++]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.7++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.7++]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.8++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.8++]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0FrP],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0FrP]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0FrP]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0FrP++],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0FrP++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0FrP++]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0FrP--],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP--],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0FrP--],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0FrP--]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0FrP--]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.2+A0FrP++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.2+A0FrP++]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.2++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.2++]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.7++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.7++]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.8++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.8++]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0.2],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0.2],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0.2],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.2]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.2]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0.2++],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0.2++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0.2++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.2++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.2++]
        DWT       OR        D0Re0,D0Re0,D0Re0 [A0.2--],T3
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0.2--],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A0.2--],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.2--]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.2--]
        D         SUBB8     D0.7,D0Re0,D0Re0
        DM        SUBB8     D0.7,D0Re0,D0Re0
        D         OR        D0.7,D0Re0,D0.7
        D         ORQ       D0.7,D0Re0,A1.7,A0.2
        D         ORQ       D0.7,D0Re0,D1Re0,A0.2
        DL        ORQ       D0.7,D0Re0,[D0AR.0],A0.2
        D         OR        D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         OR        D0.7,D0Re0,A0.7
        DLT       OR        D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T3
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T3
        DT        SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.7+A0FrP++]
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.7+A0FrP++]
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.7+A0FrP++]
        DLT       OR        D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T3
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T3
        DT        SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.2++]
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.2++]
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.2++]
        D         ORQ       D0.7,D0.7,D0Re0,A0.2
        D         ORQ       D0.7,D0.7,A1LbP,A0.2
        DLT       OR        D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T3
        D         OR        D0.7,D0.7,A1.7
        DL        ORQ       D0.7,D0.7,D0.7,A0.2
        D         OR        D0.7,D0.7,D1.7
        DL        ORQ       D0.7,D0.7,RD,A0.2
        DL        OR        D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.7++]
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.7++]
        D         ORQ       D0.7,D0.7,A0FrP,A0.2
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T3
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T3
        DLWT      OR        D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.8++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.8++]
        DL        OR        D0.7,[D0AR.0],D0Re0
        DLM       SUBB8     D0.7,D0.16,D0Re0
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A0.7],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.7],T3
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.7],T3
        DL        SUBT8     D0.7,[D0AR.0],D0.7
        DLM       SUBT8     D0.7,D0.16,D0.7
        D         ORQ       D0.7,[D0AR.0],[D0AR.0],A0.2
        D         ORQ       D0.7,[D0AR.0],[D0AR.0++],A0.2
        DT        SUBB8     D0Re0,D0Re0,D0Re0 T3,[A0.7]
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.7]
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.7]
        D         OR        D0.7,[D0AR.0++],D0Re0
        DLQ       ORQ       D0.7,[D0AR.0++],D0Re0,A0.2
        DT        SUBT8     D0Re0,D0Re0,D0Re0 [A0.7++],T3
        D         ORQ       D0.7,[D0AR.0++],A1.3,A0.2
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 [A0.7++],T3
        D         ORQ       D0.7,[D0AR.0++],D0.7,A0.2
        D         OR        D0.7,[D0AR.0++],[D0AR.0]
        DLQ       ORQ       D0.7,[D0AR.0++],RD,A0.2
        DL        ORQ       D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        DLWT      OR        D0Re0,D0Re0,D0Re0 T3,[A0.7++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.7++]
        D         ORQ       D0.7,[D0AR.0++],A0.2,A0.2
        DL        SUBT8     D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         OR        D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         ORQ       D0.7,[D0AR.0+D0ARI.0++],A1.7,A0.2
        D         OR        D0.7,[D0AR.0+D0ARI.0++],RD
        D         OR        D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         OR        D0.7,[D0AR.0+D0ARI.0++],A0FrP
        DT        SUBB8     D0Re0,D0Re0,D0Re0 [A0.7--],T3
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [A0.7--],T3
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.7--],T3
        DLT       OR        D0Re0,D0Re0,D0Re0 T3,[A0.7--]
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.7--]
        DT        SUBB8     D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T3
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T3
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T3
        DLT       OR        D0Re0,D0Re0,D0Re0 T3,[A0.8+A0FrP++]
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.8+A0FrP++]
        DT        SUBB8     D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T3
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T3
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T3
        DLT       OR        D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.2++]
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.2++]
        DT        SUBB8     D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T3
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T3
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T3
        DLT       OR        D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.7++]
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.7++]
        DT        SUBB8     D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T3
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T3
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T3
        DLT       OR        D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.8++]
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.8++]
        DT        SUBB8     D0Re0,D0Re0,D0Re0 [A0.8],T3
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [A0.8],T3
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.8],T3
        DLT       OR        D0Re0,D0Re0,D0Re0 T3,[A0.8]
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.8]
        DT        SUBB8     D0Re0,D0Re0,D0Re0 [A0.8++],T3
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [A0.8++],T3
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.8++],T3
        DLT       OR        D0Re0,D0Re0,D0Re0 T3,[A0.8++]
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.8++]
        DT        SUBB8     D0Re0,D0Re0,D0Re0 [A0.8--],T3
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [A0.8--],T3
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 [A0.8--],T3
        DLT       OR        D0Re0,D0Re0,D0Re0 T3,[A0.8--]
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A0.8--]
        D         SUBT8     [D0AW.0],D0Re0,D0Re0
        DM        SUBT8     [D0AW.0],D0Re0,D0Re0
        D         OR        [D0AW.0],D0Re0,A1.7
        DL        ORQ       [D0AW.0],D0Re0,D0.7,A0.2
        D         OR        [D0AW.0],D0Re0,D1.7
        DL        ORQ       [D0AW.0],D0Re0,RD,A0.2
        DL        OR        [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        D         ORQ       [D0AW.0],D0Re0,A0.7,A0.2
        DL        ORQ       [D0AW.0],D0.7,D0Re0,A0.2
        D         OR        [D0AW.0],D0.7,A1.3
        DL        SUBT8     [D0AW.0],D0.7,D0.7
        DLM       SUBT8     [D0AW.0],D0.7,D0.7
        D         ORQ       [D0AW.0],D0.7,[D0AR.0],A0.2
        D         ORQ       [D0AW.0],D0.7,[D0AR.0++],A0.2
        D         OR        [D0AW.0],D0.7,A0.2
        DL        SUBB8     [D0AW.0],[D0AR.0],D0Re0
        DLM       SUBT8     [D0AW.0],D0.16,D0Re0
        D         SUBT8     [D0AW.0],[D0AR.0],D0.7
        DM        SUBB8     [D0AW.0],D0.16,D0.7
        D         OR        [D0AW.0],[D0AR.0],[D0AR.0]
        DLQ       ORQ       [D0AW.0],[D0AR.0],RD,A0.2
        DL        ORQ       [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        D         SUBB8     [D0AW.0],[D0AR.0++],D0Re0
        D         OR        [D0AW.0],[D0AR.0++],A1LbP
        DL        OR        [D0AW.0],[D0AR.0++],D0.7
        D         ORQ       [D0AW.0],[D0AR.0++],D1Re0,A0.2
        DL        ORQ       [D0AW.0],[D0AR.0++],[D0AR.0],A0.2
        D         OR        [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         OR        [D0AW.0],[D0AR.0++],A0.7
        D         ORQ       [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         OR        [D0AW.0],[D0AR.0+D0ARI.0++],A1.7
        DL        ORQ       [D0AW.0],[D0AR.0+D0ARI.0++],D0.7,A0.2
        DL        OR        [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        OR        [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ORQ       [D0AW.0],[D0AR.0+D0ARI.0++],A0FrP,A0.2
        DLT       OR        D0Re0,D0Re0,D0Re0 [D0AW.0],T3
        DLMT      SUBB8     D0Re0,D0Re0,D0Re0 [D0AW.0],T3
        DT        ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[D0AR.0]
        D         SUBB8     [D0AW.0++],D0Re0,D0Re0
        DM        SUBB8     [D0AW.0++],D0Re0,D0Re0
        D         OR        [D0AW.0++],D0Re0,D0.7
        D         ORQ       [D0AW.0++],D0Re0,A1.7,A0.2
        D         ORQ       [D0AW.0++],D0Re0,D1Re0,A0.2
        DL        ORQ       [D0AW.0++],D0Re0,[D0AR.0],A0.2
        D         OR        [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        D         OR        [D0AW.0++],D0Re0,A0.7
        DLT       OR        D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T3
        DLT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1LbP++],T3
        DLMT      SUBB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T3
        DT        SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1LbP++]
        DT        ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1LbP+A1LbP++]
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1LbP++]
        DT        SUBB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T3
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T3
        DMT       SUBB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T3
        DT        OR        D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.3++]
        DLT       SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.3++]
        DLQT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1LbP+A1.3++]
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.3++]
        D         ORQ       [D0AW.0++],D0.7,D0Re0,A0.2
        D         ORQ       [D0AW.0++],D0.7,A1LbP,A0.2
        DLT       OR        D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T3
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.7++],T3
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T3
        D         SUBT8     [D0AW.0++],D0.7,D0.7
        DM        SUBB8     [D0AW.0++],D0.7,D0.7
        D         OR        [D0AW.0++],D0.7,[D0AR.0]
        DLQ       ORQ       [D0AW.0++],D0.7,RD,A0.2
        DL        ORQ       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++],A0.2
        DLWT      OR        D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.7++]
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1LbP+A1.7++]
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.7++]
        D         ORQ       [D0AW.0++],D0.7,A0.7,A0.2
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.8++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.8++]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1LbP+A1.8++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.8++]
        DL        OR        [D0AW.0++],[D0AR.0],D0Re0
        DLM       SUBB8     [D0AW.0++],D0.16,D0Re0
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1LbP],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1LbP],T3
        DMT       SUBB8     D0Re0,D0Re0,D0Re0 [A1LbP],T3
        D         OR        [D0AW.0++],[D0AR.0],D0.7
        D         ORQ       [D0AW.0++],[D0AR.0],A1.7,A0.2
        D         ORQ       [D0AW.0++],[D0AR.0],D1Re0,A0.2
        DL        ORQ       [D0AW.0++],[D0AR.0],[D0AR.0],A0.2
        D         OR        [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DT        SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1LbP]
        DT        ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1LbP]
        DMT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1LbP]
        D         OR        [D0AW.0++],[D0AR.0],A0.2
        DL        SUBB8     [D0AW.0++],[D0AR.0++],D0Re0
        DT        OR        D0Re0,D0Re0,D0Re0 [A1LbP++],T3
        DLT       SUBB8     D0Re0,D0Re0,D0Re0 [A1LbP++],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1LbP++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1LbP++],T3
        DL        SUBB8     [D0AW.0++],[D0AR.0++],D0.7
        D         OR        [D0AW.0++],[D0AR.0++],D1.7
        DL        ORQ       [D0AW.0++],[D0AR.0++],RD,A0.2
        DL        OR        [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1LbP++]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1LbP++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1LbP++]
        D         ORQ       [D0AW.0++],[D0AR.0++],A0.2,A0.2
        DL        SUBT8     [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         OR        [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        D         ORQ       [D0AW.0++],[D0AR.0+D0ARI.0++],A1.7,A0.2
        D         OR        [D0AW.0++],[D0AR.0+D0ARI.0++],RD
        D         OR        [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         OR        [D0AW.0++],[D0AR.0+D0ARI.0++],A0FrP
        DT        SUBB8     D0Re0,D0Re0,D0Re0 [A1LbP--],T3
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [A1LbP--],T3
        DMT       SUBB8     D0Re0,D0Re0,D0Re0 [A1LbP--],T3
        DT        OR        D0Re0,D0Re0,D0Re0 T3,[A1LbP--]
        DLT       SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1LbP--]
        DLQT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1LbP--]
        DLMT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1LbP--]
        DT        ORQ       D0Re0,D0Re0,D0Re0,A0.2 [D0AW.0++],T3
        DT        SUBB8     D0Re0,D0Re0,D0Re0 T3,[D0AR.0++]
        DMT       SUBB8     D0Re0,D0Re0,D0Re0 T3,[D0AR.0++]
        DL        SUBB8     [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DLM       SUBT8     [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        D         SUBT8     [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DM        SUBB8     [D0AW.0+D0AWI.0++],D0Re0,D0.7
        D         OR        [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DLQ       ORQ       [D0AW.0+D0AWI.0++],D0Re0,RD,A0.2
        DL        ORQ       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++],A0.2
        D         SUBB8     [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DM        SUBB8     [D0AW.0+D0AWI.0++],D0.7,D0Re0
        D         OR        [D0AW.0+D0AWI.0++],D0.7,D0.7
        D         ORQ       [D0AW.0+D0AWI.0++],D0.7,A1.7,A0.2
        D         ORQ       [D0AW.0+D0AWI.0++],D0.7,D1Re0,A0.2
        DL        ORQ       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0],A0.2
        D         OR        [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        D         OR        [D0AW.0+D0AWI.0++],D0.7,A0.7
        D         ORQ       [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0,A0.2
        D         ORQ       [D0AW.0+D0AWI.0++],[D0AR.0],A1LbP,A0.2
        DL        SUBB8     [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DLM       SUBB8     [D0AW.0+D0AWI.0++],D0.16,D0.7
        DL        OR        [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DL        OR        [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        D         ORQ       [D0AW.0+D0AWI.0++],[D0AR.0],A0FrP,A0.2
        DL        OR        [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         OR        [D0AW.0+D0AWI.0++],[D0AR.0++],A1.3
        DL        SUBT8     [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        D         ORQ       [D0AW.0+D0AWI.0++],[D0AR.0++],D1.7,A0.2
        DLQ       ORQ       [D0AW.0+D0AWI.0++],[D0AR.0++],D0.16,A0.2
        D         ORQ       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        D         OR        [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLQ       ORQ       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         SUBT8     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        D         OR        [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D1Re0
        D         ORQ       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],RD,A0.2
        DL        ORQ       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++],A0.2
        D         ORQ       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A0.2,A0.2
        DLT       SUBT8     D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T3
        DT        OR        D0Re0,D0Re0,D0Re0 T3,[D0AR.0+D0ARI.0++]
        DLQT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[D0AR.0+D0ARI.0++]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.3+A1LbP++]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.3+A1LbP++]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3+A1LbP++]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.3+A1LbP++]
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.3++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.3++]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3+A1.3++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.3++]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.7++]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.7++]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3+A1.7++]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.7++]
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.8++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.8++]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3+A1.8++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.8++]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.3],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.3],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.3],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.3]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.3]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.3]
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1.3++],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1.3++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1.3++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.3++]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.3++]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.3--],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.3--],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.3--],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.3--]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.3--]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3--]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.3--]
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1LbP++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7+A1LbP++]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.7+A1LbP++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7+A1LbP++]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.3++]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.3++]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.7+A1.3++]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.3++]
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.7++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.7++]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.7+A1.7++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.7++]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.8++]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.8++]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.7+A1.8++]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.8++]
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1.7],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1.7],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1.7],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.7]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.7++],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.7++],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.7++],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.7++]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.7++]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.7++]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7++]
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1.7--],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1.7--],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1.7--],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7--]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.7--]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.7--]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.8+A1LbP++]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.8+A1LbP++]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8+A1LbP++]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.8+A1LbP++]
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.3++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.3++]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8+A1.3++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.3++]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.7++]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.7++]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8+A1.7++]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.7++]
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.8++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.8++]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8+A1.8++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.8++]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.8],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.8],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.8],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.8]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.8]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.8]
        DLWT      OR        D0Re0,D0Re0,D0Re0 [A1.8++],T3
        DLWT      ORQ       D0Re0,D0Re0,D0Re0,A0.2 [A1.8++],T3
        DLMWT     SUBB8     D0Re0,D0Re0,D0Re0 [A1.8++],T3
        DWT       SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.8++]
        DWT       ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8++]
        DMWT      SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.8++]
        DWT       SUBB8     D0Re0,D0Re0,D0Re0 [A1.8--],T3
        DLWT      SUBT8     D0Re0,D0Re0,D0Re0 [A1.8--],T3
        DMWT      SUBB8     D0Re0,D0Re0,D0Re0 [A1.8--],T3
        DWT       OR        D0Re0,D0Re0,D0Re0 T3,[A1.8--]
        DLWT      SUBB8     D0Re0,D0Re0,D0Re0 T3,[A1.8--]
        DLQWT     ORQ       D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8--]
        DLMWT     SUBT8     D0Re0,D0Re0,D0Re0 T3,[A1.8--]
        DM        SUBT8     D1Re0,D1Re0,D1Re0
        D         OR        D1Re0,D1Re0,A1.3
        DL        SUBT8     D1Re0,D1Re0,D1.7
        DLQ       ORQ       D1Re0,D1Re0,D1.7,A1.3
        DLQ       ORQ       D1Re0,D1Re0,D1.16,A1.3
        D         ORQ       D1Re0,D1Re0,[D1BR.1],A1.3
        DL        OR        D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DL        SUBB8     D1Re0,D1.7,D1Re0
        DLQ       ORQ       D1Re0,D1.7,D1Re0,A1.3
        D         SUBT8     D1Re0,D1.7,D1.7
        D         ORQ       D1Re0,D1.7,D1.7,A1.3
        D         OR        D1Re0,D1.7,RD
        D         ORQ       D1Re0,D1.7,A0.2,A1.3
        DL        ORQ       D1Re0,D1.7,[D1BR.1++],A1.3
        DM        SUBT8     D1Re0,D1.16,D1Re0
        D         SUBB8     D1Re0,[D1BR.1],D1Re0
        D         OR        D1Re0,[D1BR.1],A1LbP
        DL        OR        D1Re0,[D1BR.1],D1.7
        D         ORQ       D1Re0,[D1BR.1],D0Re0,A1.3
        D         OR        D1Re0,[D1BR.1],A0FrP
        D         OR        D1Re0,[D1BR.1],[D1BR.1++]
        D         ORQ       D1Re0,[D1BR.1],A0.7,A1.3
        D         ORQ       D1Re0,[D1BR.1++],D1Re0,A1.3
        D         OR        D1Re0,[D1BR.1++],A1.7
        DL        ORQ       D1Re0,[D1BR.1++],D1.7,A1.3
        DL        ORQ       D1Re0,[D1BR.1++],RD,A1.3
        DL        OR        D1Re0,[D1BR.1++],[D1BR.1]
        D         OR        D1Re0,[D1BR.1++],A0.7
        DL        OR        D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         OR        D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DL        SUBT8     D1Re0,[D1BR.1+D1BRI.1++],D1.7
        D         ORQ       D1Re0,[D1BR.1+D1BRI.1++],D0.7,A1.3
        D         OR        D1Re0,[D1BR.1+D1BRI.1++],A0.2
        D         ORQ       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++],A1.3
        DT        OR        D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T3
        DLT       SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T3
        DLMT      SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T3
        DLQT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0FrP++],T3
        DLT       OR        D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0FrP++]
        DMT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0FrP++]
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP+A0FrP++]
        DT        SUBT8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T3
        DMT       SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T3
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.2++],T3
        DT        SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.2++]
        DLT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.2++]
        DLMT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.2++]
        DT        OR        D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T3
        DLT       SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T3
        DLMT      SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T3
        DLQT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.7++],T3
        DLT       OR        D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.7++]
        DMT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.7++]
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP+A0.7++]
        DT        SUBT8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T3
        DMT       SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T3
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.8++],T3
        DT        SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.8++]
        DLT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.8++]
        DLMT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.8++]
        DT        OR        D1Re0,D1Re0,D1Re0 [A0FrP],T3
        DLT       SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP],T3
        DLMT      SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP],T3
        DLQT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0FrP],T3
        DLT       OR        D1Re0,D1Re0,D1Re0 T3,[A0FrP]
        DMT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0FrP]
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP]
        DT        SUBT8     D1Re0,D1Re0,D1Re0 [A0FrP++],T3
        DMT       SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP++],T3
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0FrP++],T3
        DT        SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0FrP++]
        DLT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0FrP++]
        DLMT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0FrP++]
        DT        OR        D1Re0,D1Re0,D1Re0 [A0FrP--],T3
        DLT       SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP--],T3
        DLMT      SUBB8     D1Re0,D1Re0,D1Re0 [A0FrP--],T3
        DLQT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0FrP--],T3
        DLT       OR        D1Re0,D1Re0,D1Re0 T3,[A0FrP--]
        DMT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0FrP--]
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP--]
        DT        SUBT8     D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T3
        DMT       SUBB8     D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T3
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0FrP++],T3
        DT        SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.2+A0FrP++]
        DLT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2+A0FrP++]
        DLMT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2+A0FrP++]
        DT        OR        D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T3
        DLT       SUBB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T3
        DLMT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T3
        DLQT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.2++],T3
        DLT       OR        D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.2++]
        DMT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.2++]
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.2+A0.2++]
        DT        SUBT8     D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T3
        DMT       SUBB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T3
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.7++],T3
        DT        SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.7++]
        DLT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.7++]
        DLMT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.7++]
        DT        OR        D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T3
        DLT       SUBB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T3
        DLMT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T3
        DLQT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.8++],T3
        DLT       OR        D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.8++]
        DMT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.8++]
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.2+A0.8++]
        DT        SUBT8     D1Re0,D1Re0,D1Re0 [A0.2],T3
        DMT       SUBB8     D1Re0,D1Re0,D1Re0 [A0.2],T3
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.2],T3
        DT        SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.2]
        DLT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2]
        DLMT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2]
        DT        OR        D1Re0,D1Re0,D1Re0 [A0.2++],T3
        DLT       SUBB8     D1Re0,D1Re0,D1Re0 [A0.2++],T3
        DLMT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.2++],T3
        DLQT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.2++],T3
        DLT       OR        D1Re0,D1Re0,D1Re0 T3,[A0.2++]
        DMT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2++]
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.2++]
        DT        SUBT8     D1Re0,D1Re0,D1Re0 [A0.2--],T3
        DMT       SUBB8     D1Re0,D1Re0,D1Re0 [A0.2--],T3
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.2--],T3
        DT        SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.2--]
        DLT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2--]
        DLMT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.2--]
        D         OR        D1.7,D1Re0,D1Re0
        DLM       SUBB8     D1.7,D1Re0,D1Re0
        D         ORQ       D1.7,D1Re0,A1.3,A1.3
        DM        SUBB8     D1.7,D1Re0,D1.7
        D         OR        D1.7,D1Re0,D0Re0
        DLQ       ORQ       D1.7,D1Re0,RD,A1.3
        DL        ORQ       D1.7,D1Re0,[D1BR.1],A1.3
        D         ORQ       D1.7,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T3
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T3
        DWT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0FrP++],T3
        DWT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.7+A0FrP++]
        DLWT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.7+A0FrP++]
        DLMWT     SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.7+A0FrP++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T3
        DLQWT     ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.2++],T3
        DLWT      OR        D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.2++]
        DMWT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.2++]
        DLWT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.7+A0.2++]
        DL        SUBT8     D1.7,D1.7,D1Re0
        D         OR        D1.7,D1.7,A1LbP
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T3
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T3
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.7++],T3
        D         SUBB8     D1.7,D1.7,D1.7
        DLM       SUBT8     D1.7,D1.7,D1.7
        D         ORQ       D1.7,D1.7,D0.7,A1.3
        DT        SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.7++]
        DLT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.7++]
        DLMT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.7++]
        D         OR        D1.7,D1.7,A0FrP
        D         OR        D1.7,D1.7,[D1BR.1++]
        D         ORQ       D1.7,D1.7,A0.7,A1.3
        DLT       OR        D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T3
        DMT       SUBT8     D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T3
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.8++],T3
        DT        SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.8++]
        DMT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.8++]
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.7+A0.8++]
        DLM       SUBB8     D1.7,D1.16,D1Re0
        DLT       OR        D1Re0,D1Re0,D1Re0 [A0.7],T3
        DMT       SUBT8     D1Re0,D1Re0,D1Re0 [A0.7],T3
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.7],T3
        DT        OR        D1Re0,D1Re0,D1Re0 T3,[A0.7]
        DLT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.7]
        DLMT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.7]
        DLQT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.7]
        DLT       OR        D1Re0,D1Re0,D1Re0 [A0.7++],T3
        DMT       SUBT8     D1Re0,D1Re0,D1Re0 [A0.7++],T3
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.7++],T3
        DT        SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.7++]
        DMT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.7++]
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.7++]
        DT        SUBB8     D1Re0,D1Re0,D1Re0 [A0.7--],T3
        DLT       SUBT8     D1Re0,D1Re0,D1Re0 [A0.7--],T3
        DLMT      SUBT8     D1Re0,D1Re0,D1Re0 [A0.7--],T3
        DT        OR        D1Re0,D1Re0,D1Re0 T3,[A0.7--]
        DLT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.7--]
        DLMT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.7--]
        DLQT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.7--]
        D         ORQ       D1.7,[D1BR.1],D1Re0,A1.3
        D         OR        D1.7,[D1BR.1],A1.7
        DL        ORQ       D1.7,[D1BR.1],D1.7,A1.3
        DL        ORQ       D1.7,[D1BR.1],RD,A1.3
        DL        OR        D1.7,[D1BR.1],[D1BR.1]
        D         OR        D1.7,[D1BR.1],A0.7
        DL        OR        D1.7,[D1BR.1++],D1Re0
        D         OR        D1.7,[D1BR.1++],A1.3
        DL        SUBT8     D1.7,[D1BR.1++],D1.7
        D         ORQ       D1.7,[D1BR.1++],D0.7,A1.3
        D         OR        D1.7,[D1BR.1++],A0.2
        D         ORQ       D1.7,[D1BR.1++],[D1BR.1++],A1.3
        D         OR        D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DLQ       ORQ       D1.7,[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         SUBT8     D1.7,[D1BR.1+D1BRI.1++],D1.7
        D         OR        D1.7,[D1BR.1+D1BRI.1++],D0Re0
        DLQ       ORQ       D1.7,[D1BR.1+D1BRI.1++],RD,A1.3
        DL        ORQ       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1],A1.3
        D         ORQ       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T3
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T3
        DWT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0FrP++],T3
        DWT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.8+A0FrP++]
        DLWT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8+A0FrP++]
        DLMWT     SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8+A0FrP++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T3
        DLQWT     ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.2++],T3
        DLWT      OR        D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.2++]
        DMWT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.2++]
        DLWT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.8+A0.2++]
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T3
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T3
        DWT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.7++],T3
        DWT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.7++]
        DLWT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.7++]
        DLMWT     SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.7++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T3
        DLQWT     ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.8++],T3
        DLWT      OR        D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.8++]
        DMWT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.8++]
        DLWT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.8+A0.8++]
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 [A0.8],T3
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.8],T3
        DWT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.8],T3
        DWT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.8]
        DLWT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8]
        DLMWT     SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A0.8++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.8++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A0.8++],T3
        DLQWT     ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.8++],T3
        DLWT      OR        D1Re0,D1Re0,D1Re0 T3,[A0.8++]
        DMWT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8++]
        DLWT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.8++]
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 [A0.8--],T3
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 [A0.8--],T3
        DWT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 [A0.8--],T3
        DWT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[A0.8--]
        DLWT      SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8--]
        DLMWT     SUBT8     D1Re0,D1Re0,D1Re0 T3,[A0.8--]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1LbP++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1LbP++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.3++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.3++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.7++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.7++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.8++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.8++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1LbP],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1LbP]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1LbP]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1LbP++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1LbP++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1LbP++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1LbP--],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP--],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1LbP--],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1LbP--]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1LbP--]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.3+A1LbP++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.3+A1LbP++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.3++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.3++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.7++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.7++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.8++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.8++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.3],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.3],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.3],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.3]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.3]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.3++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.3++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.3++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.3++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.3++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.3--],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.3--],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.3--],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.3--]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.3--]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.7+A1LbP++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.7+A1LbP++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.3++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.3++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.7++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.7++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.8++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.8++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.7],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.7],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.7],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.7]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.7]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.7++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.7++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.7++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.7++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.7++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.7--],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.7--],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.7--],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.7--]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.7--]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.8+A1LbP++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.8+A1LbP++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.3++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.3++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.7++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.7++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.8++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.8++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.8],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.8],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.8],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.8]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.8]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.8++],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.8++],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.8++],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.8++]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.8++]
        DWT       OR        D1Re0,D1Re0,D1Re0 [A1.8--],T3
        DLWT      SUBB8     D1Re0,D1Re0,D1Re0 [A1.8--],T3
        DLMWT     SUBB8     D1Re0,D1Re0,D1Re0 [A1.8--],T3
        DWT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[A1.8--]
        DMWT      SUBB8     D1Re0,D1Re0,D1Re0 T3,[A1.8--]
        D         SUBB8     [D1BW.1],D1Re0,D1Re0
        DLM       SUBT8     [D1BW.1],D1Re0,D1Re0
        D         OR        [D1BW.1],D1Re0,D1.7
        DM        SUBT8     [D1BW.1],D1Re0,D1.7
        D         ORQ       [D1BW.1],D1Re0,D0Re0,A1.3
        D         OR        [D1BW.1],D1Re0,A0FrP
        D         OR        [D1BW.1],D1Re0,[D1BR.1++]
        D         ORQ       [D1BW.1],D1Re0,A0.7,A1.3
        DM        SUBB8     [D1BW.1],D1.7,D1Re0
        D         ORQ       [D1BW.1],D1.7,A1LbP,A1.3
        DL        SUBB8     [D1BW.1],D1.7,D1.7
        DL        ORQ       [D1BW.1],D1.7,D1.7,A1.3
        DL        ORQ       [D1BW.1],D1.7,RD,A1.3
        DL        OR        [D1BW.1],D1.7,[D1BR.1]
        D         OR        [D1BW.1],D1.7,A0.7
        DLM       SUBT8     [D1BW.1],D1.16,D1Re0
        DLT       OR        D1Re0,D1Re0,D1Re0 [D1BW.1],T3
        DLT       ORQ       D1Re0,D1Re0,D1Re0,A1.3 [D1BW.1],T3
        DMT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[D1BR.1]
        D         SUBB8     [D1BW.1],[D1BR.1],D1Re0
        D         OR        [D1BW.1],[D1BR.1],A1LbP
        DL        OR        [D1BW.1],[D1BR.1],D1.7
        D         ORQ       [D1BW.1],[D1BR.1],D0Re0,A1.3
        D         OR        [D1BW.1],[D1BR.1],A0FrP
        D         OR        [D1BW.1],[D1BR.1],[D1BR.1++]
        D         ORQ       [D1BW.1],[D1BR.1],A0.7,A1.3
        D         ORQ       [D1BW.1],[D1BR.1++],D1Re0,A1.3
        D         OR        [D1BW.1],[D1BR.1++],A1.7
        DL        ORQ       [D1BW.1],[D1BR.1++],D1.7,A1.3
        DL        ORQ       [D1BW.1],[D1BR.1++],RD,A1.3
        DL        OR        [D1BW.1],[D1BR.1++],[D1BR.1]
        D         OR        [D1BW.1],[D1BR.1++],A0.7
        DL        OR        [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        D         OR        [D1BW.1],[D1BR.1+D1BRI.1++],A1.3
        DL        SUBT8     [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        D         ORQ       [D1BW.1],[D1BR.1+D1BRI.1++],D0.7,A1.3
        D         OR        [D1BW.1],[D1BR.1+D1BRI.1++],A0.2
        D         ORQ       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++],A1.3
        D         OR        [D1BW.1++],D1Re0,D1Re0
        DLM       SUBB8     [D1BW.1++],D1Re0,D1Re0
        D         ORQ       [D1BW.1++],D1Re0,A1.3,A1.3
        DM        SUBB8     [D1BW.1++],D1Re0,D1.7
        D         OR        [D1BW.1++],D1Re0,D0Re0
        DLQ       ORQ       [D1BW.1++],D1Re0,RD,A1.3
        DL        ORQ       [D1BW.1++],D1Re0,[D1BR.1],A1.3
        D         ORQ       [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DL        SUBT8     [D1BW.1++],D1.7,D1Re0
        D         OR        [D1BW.1++],D1.7,A1LbP
        DL        OR        [D1BW.1++],D1.7,D1.7
        D         ORQ       [D1BW.1++],D1.7,A1.7,A1.3
        D         ORQ       [D1BW.1++],D1.7,RD,A1.3
        D         OR        [D1BW.1++],D1.7,[D1BR.1]
        D         OR        [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DLM       SUBB8     [D1BW.1++],D1.16,D1Re0
        DT        SUBT8     D1Re0,D1Re0,D1Re0 [D1BW.1++],T3
        DT        ORQ       D1Re0,D1Re0,D1Re0,A1.3 [D1BW.1++],T3
        DLT       SUBT8     D1Re0,D1Re0,D1Re0 T3,[D1BR.1++]
        D         OR        [D1BW.1++],[D1BR.1],D1Re0
        DLQ       ORQ       [D1BW.1++],[D1BR.1],D1Re0,A1.3
        D         SUBT8     [D1BW.1++],[D1BR.1],D1.7
        D         OR        [D1BW.1++],[D1BR.1],D0Re0
        DLQ       ORQ       [D1BW.1++],[D1BR.1],RD,A1.3
        DL        ORQ       [D1BW.1++],[D1BR.1],[D1BR.1],A1.3
        D         ORQ       [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        DL        SUBT8     [D1BW.1++],[D1BR.1++],D1Re0
        D         OR        [D1BW.1++],[D1BR.1++],D1.7
        D         ORQ       [D1BW.1++],[D1BR.1++],A1.7,A1.3
        D         ORQ       [D1BW.1++],[D1BR.1++],RD,A1.3
        D         OR        [D1BW.1++],[D1BR.1++],[D1BR.1]
        D         OR        [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         SUBT8     [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         ORQ       [D1BW.1++],[D1BR.1+D1BRI.1++],A1LbP,A1.3
        DL        SUBB8     [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        D         OR        [D1BW.1++],[D1BR.1+D1BRI.1++],D0.7
        D         ORQ       [D1BW.1++],[D1BR.1+D1BRI.1++],A0FrP,A1.3
        DL        OR        [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        ORQ       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        DM        SUBT8     [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        D         OR        [D1BW.1+D1BWI.1++],D1Re0,A1.3
        DL        SUBT8     [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DLQ       ORQ       [D1BW.1+D1BWI.1++],D1Re0,D1.7,A1.3
        DLQ       ORQ       [D1BW.1+D1BWI.1++],D1Re0,D1.16,A1.3
        D         ORQ       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1],A1.3
        DL        OR        [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DL        SUBB8     [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DLQ       ORQ       [D1BW.1+D1BWI.1++],D1.7,D1Re0,A1.3
        D         SUBT8     [D1BW.1+D1BWI.1++],D1.7,D1.7
        D         ORQ       [D1BW.1+D1BWI.1++],D1.7,D1.7,A1.3
        D         OR        [D1BW.1+D1BWI.1++],D1.7,RD
        D         ORQ       [D1BW.1+D1BWI.1++],D1.7,A0.2,A1.3
        DL        ORQ       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++],A1.3
        DM        SUBT8     [D1BW.1+D1BWI.1++],D1.16,D1Re0
        DT        SUBB8     D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T3
        DLMT      SUBT8     D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T3
        DLT       SUBB8     D1Re0,D1Re0,D1Re0 T3,[D1BR.1+D1BRI.1++]
        DLQT      ORQ       D1Re0,D1Re0,D1Re0,A1.3 T3,[D1BR.1+D1BRI.1++]
        DL        ORQ       [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0,A1.3
        D         SUBB8     [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        DLQ       ORQ       [D1BW.1+D1BWI.1++],[D1BR.1],D1.7,A1.3
        DLQ       ORQ       [D1BW.1+D1BWI.1++],[D1BR.1],D1.16,A1.3
        D         ORQ       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1],A1.3
        DL        OR        [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        SUBB8     [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        D         ORQ       [D1BW.1+D1BWI.1++],[D1BR.1++],A1.3,A1.3
        D         ORQ       [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7,A1.3
        D         OR        [D1BW.1+D1BWI.1++],[D1BR.1++],RD
        D         ORQ       [D1BW.1+D1BWI.1++],[D1BR.1++],A0.2,A1.3
        DL        ORQ       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++],A1.3
        D         SUBB8     [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         OR        [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1LbP
        DL        OR        [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         ORQ       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D0Re0,A1.3
        D         OR        [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0FrP
        D         OR        [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ORQ       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0.7,A1.3
        D         OR        D0.7,D0.7,#0x7fff
        D         OR        [D0AW.0],[D0AR.0],#-0x8000
        D         OR        [D0AW.0++],[D0AR.0++],#-0x1
        D         OR        D1Re0,D1Re0,#0x7fff
        D         OR        D1.7,D1.7,#-0x8000
        D         OR        [D1BW.1],[D1BR.1],#-0x1
        D         OR        [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x7fff
        DHNE      ORQPL     D0Re0,D0Re0,A1LbP,A0.2
        DECS      ORQLE     D0Re0,D0Re0,D1Re0,A0.2
        DLNE      ORQNE     D0Re0,D0Re0,[D0AR.0++],A0.2
        DHNE      ORQPL     D0Re0,D0Re0,A0.7,A0.2
        DLNE      ORQNE     D0Re0,D0.7,D0.7,A0.2
        DLLS      ORQLS     D0Re0,D0.7,[D0AR.0],A0.2
        DHNE      ORQPL     D0Re0,D0.7,A0FrP,A0.2
        D         ORQNV     D0Re0,[D0AR.0],A1.3,A0.2
        D         ORQNV     D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        D         ORQNV     D0Re0,[D0AR.0++],A1.7,A0.2
        D         ORQNV     D0Re0,[D0AR.0++],A0.2,A0.2
        D         ORQNV     D0Re0,[D0AR.0+D0ARI.0++],D1.7,A0.2
        DLNE      ORQNE     D0.7,D0Re0,D0Re0,A0.2
        D         ORQNV     D0.7,D0Re0,D0.7,A0.2
        D         ORQNV     D0.7,D0Re0,[D0AR.0],A0.2
        DHNE      ORQPL     D0.7,D0Re0,A0.2,A0.2
        DHNE      ORQPL     D0.7,D0.7,A1.3,A0.2
        DHNE      ORQPL     D0.7,D0.7,D1.7,A0.2
        DLNE      ORQNE     D0.7,D0.7,[D0AR.0+D0ARI.0++],A0.2
        D         ORQNV     D0.7,[D0AR.0],D0Re0,A0.2
        D         ORQNV     D0.7,[D0AR.0],RD,A0.2
        D         ORQNV     D0.7,[D0AR.0++],A1.3,A0.2
        DHCS      ORQVS     D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        D         ORQNV     D0.7,[D0AR.0+D0ARI.0++],A1.7,A0.2
        D         ORQNV     D0.7,[D0AR.0+D0ARI.0++],A0.2,A0.2
        DLNE      ORQNE     D0.16,D0Re0,D0.7,A0.2
        DLLS      ORQLS     D0.16,D0Re0,[D0AR.0],A0.2
        DHNE      ORQPL     D0.16,D0Re0,A0FrP,A0.2
        DHNE      ORQPL     D0.16,D0.7,A1LbP,A0.2
        DECS      ORQLE     D0.16,D0.7,D1Re0,A0.2
        DLNE      ORQNE     D0.16,D0.7,[D0AR.0++],A0.2
        DHNE      ORQPL     D0.16,D0.7,A0.7,A0.2
        D         ORQNV     D0.16,[D0AR.0],D1.7,A0.2
        D         ORQNV     D0.16,[D0AR.0++],D0Re0,A0.2
        D         ORQNV     D0.16,[D0AR.0++],RD,A0.2
        D         ORQNV     D0.16,[D0AR.0+D0ARI.0++],A1.3,A0.2
        D         ORQNV     D0.16,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++],A0.2
        DECS      ORQLE     D1Re0,D1Re0,A1.3,A1.3
        DECS      ORQLE     D1Re0,D1Re0,D0.7,A1.3
        DECS      ORQLE     D1Re0,D1Re0,[D1BR.1],A1.3
        DECS      ORQLE     D1Re0,D1.7,D1Re0,A1.3
        D         ORQNV     D1Re0,D1.7,D1.7,A1.3
        DECS      ORQLE     D1Re0,D1.7,A0FrP,A1.3
        DHNE      ORQPL     D1Re0,D1.7,[D1BR.1+D1BRI.1++],A1.3
        D         ORQNV     D1Re0,[D1BR.1],A1.7,A1.3
        D         ORQNV     D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        D         ORQNV     D1Re0,[D1BR.1++],D0.7,A1.3
        D         ORQNV     D1Re0,[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         ORQNV     D1Re0,[D1BR.1+D1BRI.1++],A0FrP,A1.3
        DECS      ORQLE     D1.7,D1Re0,A1LbP,A1.3
        DECS      ORQLE     D1.7,D1Re0,D0Re0,A1.3
        DECS      ORQLE     D1.7,D1Re0,A0.2,A1.3
        D         ORQNV     D1.7,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DHNE      ORQPL     D1.7,D1.7,D1.7,A1.3
        DECS      ORQLE     D1.7,D1.7,RD,A1.3
        DHNE      ORQPL     D1.7,D1.7,[D1BR.1++],A1.3
        D         ORQNV     D1.7,[D1BR.1],A1.3,A1.3
        D         ORQNV     D1.7,[D1BR.1],[D1BR.1],A1.3
        D         ORQNV     D1.7,[D1BR.1++],A1.7,A1.3
        D         ORQNV     D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        D         ORQNV     D1.7,[D1BR.1+D1BRI.1++],D0.7,A1.3
        DECS      ORQLE     D1.16,D1Re0,D1Re0,A1.3
        D         ORQNV     D1.16,D1Re0,D1.7,A1.3
        DECS      ORQLE     D1.16,D1Re0,A0FrP,A1.3
        DHNE      ORQPL     D1.16,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DECS      ORQLE     D1.16,D1.7,A1.3,A1.3
        DECS      ORQLE     D1.16,D1.7,D0.7,A1.3
        DECS      ORQLE     D1.16,D1.7,[D1BR.1],A1.3
        D         ORQNV     D1.16,[D1BR.1],D1Re0,A1.3
        D         ORQNV     D1.16,[D1BR.1],A0FrP,A1.3
        D         ORQNV     D1.16,[D1BR.1++],A1.3,A1.3
        D         ORQNV     D1.16,[D1BR.1++],[D1BR.1],A1.3
        D         ORQNV     D1.16,[D1BR.1+D1BRI.1++],A1.7,A1.3
        D         ORQNV     D1.16,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        D         ORSQ      D0Re0,D0Re0,D0Re0,A0.2
        D         ORSQ      D0Re0,D0Re0,A1LbP,A0.2
        DL        SUBSB8    D0Re0,D0Re0,D0.7
        DLM       SUBSB8    D0Re0,D0Re0,D0.7
        DL        ORS       D0Re0,D0Re0,[D0AR.0]
        DL        ORS       D0Re0,D0Re0,[D0AR.0++]
        D         ORSQ      D0Re0,D0Re0,A0FrP,A0.2
        DL        ORS       D0Re0,D0.7,D0Re0
        DLM       SUBSB8    D0Re0,D0.7,D0Re0
        D         SUBSB8    D0Re0,D0.7,D0.7
        DLQ       ORSQ      D0Re0,D0.7,D0.7,A0.2
        D         ORSQ      D0Re0,D0.7,D1.7,A0.2
        DLQ       ORSQ      D0Re0,D0.7,D0.16,A0.2
        D         ORSQ      D0Re0,D0.7,[D0AR.0+D0ARI.0++],A0.2
        D         ORS       D0Re0,[D0AR.0],D0Re0
        DLQ       ORSQ      D0Re0,[D0AR.0],D0Re0,A0.2
        D         ORSQ      D0Re0,[D0AR.0],A1.3,A0.2
        D         ORSQ      D0Re0,[D0AR.0],D0.7,A0.2
        D         ORS       D0Re0,[D0AR.0],D1Re0
        D         ORSQ      D0Re0,[D0AR.0],RD,A0.2
        DL        ORSQ      D0Re0,[D0AR.0],[D0AR.0++],A0.2
        D         ORSQ      D0Re0,[D0AR.0],A0.2,A0.2
        DL        SUBST8    D0Re0,[D0AR.0++],D0Re0
        D         ORS       D0Re0,[D0AR.0++],D0.7
        D         ORSQ      D0Re0,[D0AR.0++],A1.7,A0.2
        D         ORS       D0Re0,[D0AR.0++],RD
        D         ORS       D0Re0,[D0AR.0++],[D0AR.0++]
        D         ORS       D0Re0,[D0AR.0++],A0FrP
        D         SUBST8    D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         ORSQ      D0Re0,[D0AR.0+D0ARI.0++],A1LbP,A0.2
        DL        SUBSB8    D0Re0,[D0AR.0+D0ARI.0++],D0.7
        D         ORS       D0Re0,[D0AR.0+D0ARI.0++],D1.7
        DL        ORSQ      D0Re0,[D0AR.0+D0ARI.0++],RD,A0.2
        DL        ORS       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ORSQ      D0Re0,[D0AR.0+D0ARI.0++],A0.7,A0.2
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0FrP++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0FrP++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0FrP++]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.2++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.2++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.2++]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.7++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.7++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.7++]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.8++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.8++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP+A0.8++]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0FrP],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0FrP],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0FrP]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0FrP++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0FrP++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0FrP++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP++]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0FrP--],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0FrP--],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0FrP--]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP--]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0FrP--]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0FrP++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0FrP++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0FrP++]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.2++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.2++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.2++]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.7++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.7++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.7++]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.8++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.8++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2+A0.8++]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0.2],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0.2],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.2]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0.2++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0.2++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.2++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2++]
        DLWT      ORS       D0Re0,D0Re0,D0Re0 [A0.2--],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A0.2--],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.2--]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2--]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.2--]
        DL        ORSQ      D0.7,D0Re0,D0Re0,A0.2
        D         ORS       D0.7,D0Re0,A1.3
        DL        SUBST8    D0.7,D0Re0,D0.7
        DLM       SUBST8    D0.7,D0Re0,D0.7
        D         ORSQ      D0.7,D0Re0,[D0AR.0],A0.2
        D         ORSQ      D0.7,D0Re0,[D0AR.0++],A0.2
        D         ORS       D0.7,D0Re0,A0.2
        DT        SUBST8    D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T3
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T3
        DT        ORS       D0Re0,D0Re0,D0Re0 T3,[A0.7+A0FrP++]
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.7+A0FrP++]
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.7+A0FrP++]
        DT        SUBST8    D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T3
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T3
        DT        ORS       D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.2++]
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.2++]
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.2++]
        DL        SUBSB8    D0.7,D0.7,D0Re0
        DLM       SUBST8    D0.7,D0.7,D0Re0
        DT        SUBST8    D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T3
        D         ORSQ      D0.7,D0.7,A1.3,A0.2
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T3
        D         ORSQ      D0.7,D0.7,D0.7,A0.2
        D         ORS       D0.7,D0.7,D1Re0
        D         ORSQ      D0.7,D0.7,RD,A0.2
        DL        ORSQ      D0.7,D0.7,[D0AR.0++],A0.2
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.7++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.7++]
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.7++]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T3
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T3
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.8++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.7+A0.8++]
        D         SUBSB8    D0.7,[D0AR.0],D0Re0
        DM        SUBSB8    D0.7,D0.16,D0Re0
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A0.7],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.7],T3
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.7],T3
        DL        ORS       D0.7,[D0AR.0],D0.7
        DM        SUBST8    D0.7,D0.16,D0.7
        D         ORS       D0.7,[D0AR.0],RD
        D         ORS       D0.7,[D0AR.0],[D0AR.0++]
        DT        ORS       D0Re0,D0Re0,D0Re0 T3,[A0.7]
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.7]
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.7]
        D         ORS       D0.7,[D0AR.0],A0.7
        D         ORSQ      D0.7,[D0AR.0++],D0Re0,A0.2
        DT        SUBSB8    D0Re0,D0Re0,D0Re0 [A0.7++],T3
        DLT       SUBST8    D0Re0,D0Re0,D0Re0 [A0.7++],T3
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 [A0.7++],T3
        DL        SUBSB8    D0.7,[D0AR.0++],D0.7
        D         ORS       D0.7,[D0AR.0++],D1.7
        DL        ORSQ      D0.7,[D0AR.0++],RD,A0.2
        DL        ORS       D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.7++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.7++]
        D         ORSQ      D0.7,[D0AR.0++],A0FrP,A0.2
        DL        ORS       D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         ORS       D0.7,[D0AR.0+D0ARI.0++],A1.3
        DL        SUBST8    D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         ORSQ      D0.7,[D0AR.0+D0ARI.0++],D1.7,A0.2
        DLQ       ORSQ      D0.7,[D0AR.0+D0ARI.0++],D0.16,A0.2
        D         ORSQ      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++],A0.2
        DT        ORS       D0Re0,D0Re0,D0Re0 [A0.7--],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [A0.7--],T3
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.7--],T3
        DT        SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.7--]
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.7--]
        DT        ORS       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T3
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T3
        DT        SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.8+A0FrP++]
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.8+A0FrP++]
        DT        ORS       D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T3
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T3
        DT        SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.2++]
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.2++]
        DT        ORS       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T3
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T3
        DT        SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.7++]
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.7++]
        DT        ORS       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T3
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T3
        DT        SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.8++]
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.8+A0.8++]
        DT        ORS       D0Re0,D0Re0,D0Re0 [A0.8],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8],T3
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8],T3
        DT        SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.8]
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.8]
        DT        ORS       D0Re0,D0Re0,D0Re0 [A0.8++],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8++],T3
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8++],T3
        DT        SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.8++]
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.8++]
        DT        ORS       D0Re0,D0Re0,D0Re0 [A0.8--],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8--],T3
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 [A0.8--],T3
        DT        SUBST8    D0Re0,D0Re0,D0Re0 T3,[A0.8--]
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A0.8--]
        D         ORS       [D0AW.0],D0Re0,D0Re0
        DLQ       ORSQ      [D0AW.0],D0Re0,D0Re0,A0.2
        D         ORSQ      [D0AW.0],D0Re0,A1.3,A0.2
        D         ORSQ      [D0AW.0],D0Re0,D0.7,A0.2
        D         ORS       [D0AW.0],D0Re0,D1Re0
        D         ORSQ      [D0AW.0],D0Re0,RD,A0.2
        DL        ORSQ      [D0AW.0],D0Re0,[D0AR.0++],A0.2
        D         ORSQ      [D0AW.0],D0Re0,A0.2,A0.2
        DL        SUBST8    [D0AW.0],D0.7,D0Re0
        D         ORS       [D0AW.0],D0.7,A1LbP
        DL        ORS       [D0AW.0],D0.7,D0.7
        DM        SUBST8    [D0AW.0],D0.7,D0.7
        D         ORS       [D0AW.0],D0.7,RD
        D         ORS       [D0AW.0],D0.7,[D0AR.0++]
        D         ORS       [D0AW.0],D0.7,A0FrP
        D         SUBST8    [D0AW.0],[D0AR.0],D0Re0
        DM        SUBST8    [D0AW.0],D0.16,D0Re0
        D         ORS       [D0AW.0],[D0AR.0],A1.7
        DL        ORSQ      [D0AW.0],[D0AR.0],D0.7,A0.2
        D         ORS       [D0AW.0],[D0AR.0],D1.7
        DL        ORSQ      [D0AW.0],[D0AR.0],RD,A0.2
        DL        ORS       [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ORSQ      [D0AW.0],[D0AR.0],A0.7,A0.2
        DL        ORSQ      [D0AW.0],[D0AR.0++],D0Re0,A0.2
        D         SUBSB8    [D0AW.0],[D0AR.0++],D0.7
        DLQ       ORSQ      [D0AW.0],[D0AR.0++],D0.7,A0.2
        D         ORSQ      [D0AW.0],[D0AR.0++],[D0AR.0],A0.2
        D         ORSQ      [D0AW.0],[D0AR.0++],[D0AR.0++],A0.2
        D         ORS       [D0AW.0],[D0AR.0++],A0.2
        DL        SUBSB8    [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        D         ORSQ      [D0AW.0],[D0AR.0+D0ARI.0++],A1.3,A0.2
        D         ORSQ      [D0AW.0],[D0AR.0+D0ARI.0++],D0.7,A0.2
        D         ORS       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLQ       ORSQ      [D0AW.0],[D0AR.0+D0ARI.0++],RD,A0.2
        DL        ORSQ      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++],A0.2
        DT        SUBSB8    D0Re0,D0Re0,D0Re0 [D0AW.0],T3
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 [D0AW.0],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[D0AR.0]
        DLMT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[D0AR.0]
        DL        ORSQ      [D0AW.0++],D0Re0,D0Re0,A0.2
        D         ORS       [D0AW.0++],D0Re0,A1.3
        DL        SUBST8    [D0AW.0++],D0Re0,D0.7
        DLM       SUBST8    [D0AW.0++],D0Re0,D0.7
        D         ORSQ      [D0AW.0++],D0Re0,[D0AR.0],A0.2
        D         ORSQ      [D0AW.0++],D0Re0,[D0AR.0++],A0.2
        D         ORS       [D0AW.0++],D0Re0,A0.2
        DT        SUBST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T3
        DT        ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1LbP++],T3
        DMT       SUBST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T3
        DT        SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1LbP++]
        DLT       SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1LbP++]
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1LbP++]
        DT        ORS       D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T3
        DLQT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.3++],T3
        DLMT      SUBST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T3
        DLT       ORS       D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.3++]
        DLT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1LbP+A1.3++]
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.3++]
        DL        SUBSB8    [D0AW.0++],D0.7,D0Re0
        DLM       SUBST8    [D0AW.0++],D0.7,D0Re0
        DT        SUBST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T3
        D         ORSQ      [D0AW.0++],D0.7,A1.3,A0.2
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T3
        D         ORS       [D0AW.0++],D0.7,A1.7
        DL        ORSQ      [D0AW.0++],D0.7,D0.7,A0.2
        D         ORS       [D0AW.0++],D0.7,D1.7
        DL        ORSQ      [D0AW.0++],D0.7,RD,A0.2
        DL        ORS       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.7++]
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1LbP+A1.7++]
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.7++]
        D         ORSQ      [D0AW.0++],D0.7,A0.2,A0.2
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.8++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.8++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.8++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1LbP+A1.8++]
        D         SUBSB8    [D0AW.0++],[D0AR.0],D0Re0
        DM        SUBSB8    [D0AW.0++],D0.16,D0Re0
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1LbP],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1LbP],T3
        DLQT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP],T3
        DLMT      SUBST8    D0Re0,D0Re0,D0Re0 [A1LbP],T3
        DL        SUBST8    [D0AW.0++],[D0AR.0],D0.7
        DLM       SUBST8    [D0AW.0++],D0.16,D0.7
        D         ORSQ      [D0AW.0++],[D0AR.0],[D0AR.0],A0.2
        D         ORSQ      [D0AW.0++],[D0AR.0],[D0AR.0++],A0.2
        DT        SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1LbP]
        DLT       SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1LbP]
        DMT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1LbP]
        D         ORS       [D0AW.0++],[D0AR.0],A0FrP
        D         SUBST8    [D0AW.0++],[D0AR.0++],D0Re0
        D         ORSQ      [D0AW.0++],[D0AR.0++],A1LbP,A0.2
        DLT       ORS       D0Re0,D0Re0,D0Re0 [A1LbP++],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1LbP++],T3
        D         SUBST8    [D0AW.0++],[D0AR.0++],D0.7
        D         ORS       [D0AW.0++],[D0AR.0++],D1Re0
        D         ORSQ      [D0AW.0++],[D0AR.0++],RD,A0.2
        DL        ORSQ      [D0AW.0++],[D0AR.0++],[D0AR.0++],A0.2
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1LbP++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1LbP++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1LbP++]
        D         ORSQ      [D0AW.0++],[D0AR.0++],A0FrP,A0.2
        DL        ORS       [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         ORS       [D0AW.0++],[D0AR.0+D0ARI.0++],A1.3
        DL        SUBST8    [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        D         ORSQ      [D0AW.0++],[D0AR.0+D0ARI.0++],D1.7,A0.2
        DLQ       ORSQ      [D0AW.0++],[D0AR.0+D0ARI.0++],D0.16,A0.2
        D         ORSQ      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++],A0.2
        DT        ORS       D0Re0,D0Re0,D0Re0 [A1LbP--],T3
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [A1LbP--],T3
        DLQT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP--],T3
        DLMT      SUBST8    D0Re0,D0Re0,D0Re0 [A1LbP--],T3
        DLT       ORS       D0Re0,D0Re0,D0Re0 T3,[A1LbP--]
        DLT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1LbP--]
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1LbP--]
        DLT       SUBSB8    D0Re0,D0Re0,D0Re0 [D0AW.0++],T3
        DLMT      SUBST8    D0Re0,D0Re0,D0Re0 [D0AW.0++],T3
        DLT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[D0AR.0++]
        D         SUBST8    [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DM        SUBST8    [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        D         ORS       [D0AW.0+D0AWI.0++],D0Re0,A1.7
        DL        ORSQ      [D0AW.0+D0AWI.0++],D0Re0,D0.7,A0.2
        D         ORS       [D0AW.0+D0AWI.0++],D0Re0,D1.7
        DL        ORSQ      [D0AW.0+D0AWI.0++],D0Re0,RD,A0.2
        DL        ORS       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        D         ORSQ      [D0AW.0+D0AWI.0++],D0Re0,A0.7,A0.2
        DL        ORSQ      [D0AW.0+D0AWI.0++],D0.7,D0Re0,A0.2
        D         ORS       [D0AW.0+D0AWI.0++],D0.7,A1.3
        DL        SUBST8    [D0AW.0+D0AWI.0++],D0.7,D0.7
        DLM       SUBST8    [D0AW.0+D0AWI.0++],D0.7,D0.7
        D         ORSQ      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0],A0.2
        D         ORSQ      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++],A0.2
        D         ORS       [D0AW.0+D0AWI.0++],D0.7,A0.2
        DL        SUBSB8    [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DLM       SUBST8    [D0AW.0+D0AWI.0++],D0.16,D0Re0
        D         SUBST8    [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DM        SUBSB8    [D0AW.0+D0AWI.0++],D0.16,D0.7
        D         ORS       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DLQ       ORSQ      [D0AW.0+D0AWI.0++],[D0AR.0],RD,A0.2
        DL        ORSQ      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        D         SUBSB8    [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         ORS       [D0AW.0+D0AWI.0++],[D0AR.0++],A1LbP
        DL        ORS       [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        D         ORSQ      [D0AW.0+D0AWI.0++],[D0AR.0++],D1Re0,A0.2
        DL        ORSQ      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0],A0.2
        D         ORS       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ORS       [D0AW.0+D0AWI.0++],[D0AR.0++],A0.7
        D         ORSQ      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         ORS       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A1.7
        DL        ORSQ      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7,A0.2
        DL        ORS       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        ORS       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ORSQ      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A0FrP,A0.2
        DLT       ORS       D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T3
        DLMT      SUBSB8    D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T3
        DT        ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[D0AR.0+D0ARI.0++]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1LbP++],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.3+A1LbP++]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3+A1LbP++]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.3+A1LbP++]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.3++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.3++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.3++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.3++]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.7++],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.7++]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3+A1.7++]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.7++]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.8++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.8++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.8++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.3+A1.8++]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.3],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.3],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.3],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.3]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.3]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1.3++],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1.3++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.3++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1.3++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.3++]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.3--],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.3--],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3--],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.3--],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.3--]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.3--]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.3--]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1LbP++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7+A1LbP++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1.7+A1LbP++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7+A1LbP++]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.3++],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.3++]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.7+A1.3++]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.3++]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.7++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.7++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.7++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.7++]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.8++],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.8++]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.7+A1.8++]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7+A1.8++]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1.7],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1.7],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1.7]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.7++],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.7++],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7++],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.7++],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.7++]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.7++]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7++]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1.7--],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7--],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1.7--],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7--]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1.7--]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.7--]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1LbP++],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.8+A1LbP++]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8+A1LbP++]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.8+A1LbP++]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.3++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.3++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.3++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.3++]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.7++],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.7++]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8+A1.7++]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.7++]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.8++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.8++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.8++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.8+A1.8++]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.8],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.8],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.8],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.8]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.8]
        DWT       SUBST8    D0Re0,D0Re0,D0Re0 [A1.8++],T3
        DWT       ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8++],T3
        DMWT      SUBST8    D0Re0,D0Re0,D0Re0 [A1.8++],T3
        DWT       SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.8++]
        DLWT      SUBST8    D0Re0,D0Re0,D0Re0 T3,[A1.8++]
        DMWT      SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.8++]
        DWT       ORS       D0Re0,D0Re0,D0Re0 [A1.8--],T3
        DLWT      SUBSB8    D0Re0,D0Re0,D0Re0 [A1.8--],T3
        DLQWT     ORSQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8--],T3
        DLMWT     SUBST8    D0Re0,D0Re0,D0Re0 [A1.8--],T3
        DLWT      ORS       D0Re0,D0Re0,D0Re0 T3,[A1.8--]
        DLWT      ORSQ      D0Re0,D0Re0,D0Re0,A0.2 T3,[A1.8--]
        DLMWT     SUBSB8    D0Re0,D0Re0,D0Re0 T3,[A1.8--]
        DL        SUBST8    D1Re0,D1Re0,D1Re0
        D         ORS       D1Re0,D1Re0,A1LbP
        DL        ORS       D1Re0,D1Re0,D1.7
        D         ORSQ      D1Re0,D1Re0,A1.7,A1.3
        D         ORSQ      D1Re0,D1Re0,RD,A1.3
        D         ORS       D1Re0,D1Re0,[D1BR.1]
        D         ORS       D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        D         SUBST8    D1Re0,D1.7,D1Re0
        D         ORSQ      D1Re0,D1.7,D1Re0,A1.3
        D         ORS       D1Re0,D1.7,A1.7
        DLM       SUBSB8    D1Re0,D1.7,D1.7
        D         ORS       D1Re0,D1.7,D0.7
        D         ORSQ      D1Re0,D1.7,A0FrP,A1.3
        DL        ORS       D1Re0,D1.7,[D1BR.1++]
        DL        ORSQ      D1Re0,D1.7,[D1BR.1+D1BRI.1++],A1.3
        DLM       SUBST8    D1Re0,D1.16,D1.7
        DL        ORSQ      D1Re0,[D1BR.1],D1Re0,A1.3
        D         SUBSB8    D1Re0,[D1BR.1],D1.7
        DLQ       ORSQ      D1Re0,[D1BR.1],D1.7,A1.3
        DLQ       ORSQ      D1Re0,[D1BR.1],D1.16,A1.3
        D         ORSQ      D1Re0,[D1BR.1],[D1BR.1],A1.3
        DL        ORS       D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        SUBSB8    D1Re0,[D1BR.1++],D1Re0
        D         ORSQ      D1Re0,[D1BR.1++],A1.3,A1.3
        D         ORSQ      D1Re0,[D1BR.1++],D1.7,A1.3
        D         ORS       D1Re0,[D1BR.1++],RD
        D         ORSQ      D1Re0,[D1BR.1++],A0.2,A1.3
        DL        ORSQ      D1Re0,[D1BR.1++],[D1BR.1++],A1.3
        D         SUBSB8    D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         ORS       D1Re0,[D1BR.1+D1BRI.1++],A1LbP
        DL        ORS       D1Re0,[D1BR.1+D1BRI.1++],D1.7
        D         ORSQ      D1Re0,[D1BR.1+D1BRI.1++],D0Re0,A1.3
        D         ORS       D1Re0,[D1BR.1+D1BRI.1++],A0FrP
        D         ORS       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ORSQ      D1Re0,[D1BR.1+D1BRI.1++],A0.7,A1.3
        DLT       ORS       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T3
        DMT       SUBST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T3
        DLT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0FrP++],T3
        DT        SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0FrP++]
        DMT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0FrP++]
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP+A0FrP++]
        DT        SUBSB8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T3
        DLT       SUBST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T3
        DLMT      SUBST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T3
        DT        ORS       D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.2++]
        DLT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.2++]
        DLMT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.2++]
        DLQT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP+A0.2++]
        DLT       ORS       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T3
        DMT       SUBST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T3
        DLT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.7++],T3
        DT        SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.7++]
        DMT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.7++]
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP+A0.7++]
        DT        SUBSB8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T3
        DLT       SUBST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T3
        DLMT      SUBST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T3
        DT        ORS       D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.8++]
        DLT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.8++]
        DLMT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0FrP+A0.8++]
        DLQT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP+A0.8++]
        DLT       ORS       D1Re0,D1Re0,D1Re0 [A0FrP],T3
        DMT       SUBST8    D1Re0,D1Re0,D1Re0 [A0FrP],T3
        DLT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP],T3
        DT        SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0FrP]
        DMT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0FrP]
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP]
        DT        SUBSB8    D1Re0,D1Re0,D1Re0 [A0FrP++],T3
        DLT       SUBST8    D1Re0,D1Re0,D1Re0 [A0FrP++],T3
        DLMT      SUBST8    D1Re0,D1Re0,D1Re0 [A0FrP++],T3
        DT        ORS       D1Re0,D1Re0,D1Re0 T3,[A0FrP++]
        DLT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0FrP++]
        DLMT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0FrP++]
        DLQT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP++]
        DLT       ORS       D1Re0,D1Re0,D1Re0 [A0FrP--],T3
        DMT       SUBST8    D1Re0,D1Re0,D1Re0 [A0FrP--],T3
        DLT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP--],T3
        DT        SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0FrP--]
        DMT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0FrP--]
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0FrP--]
        DT        SUBSB8    D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T3
        DLT       SUBST8    D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T3
        DLMT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T3
        DT        ORS       D1Re0,D1Re0,D1Re0 T3,[A0.2+A0FrP++]
        DLT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2+A0FrP++]
        DLMT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2+A0FrP++]
        DLQT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.2+A0FrP++]
        DLT       ORS       D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T3
        DMT       SUBST8    D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T3
        DLT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.2++],T3
        DT        SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.2++]
        DMT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.2++]
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.2+A0.2++]
        DT        SUBSB8    D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T3
        DLT       SUBST8    D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T3
        DLMT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T3
        DT        ORS       D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.7++]
        DLT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.7++]
        DLMT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.7++]
        DLQT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.2+A0.7++]
        DLT       ORS       D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T3
        DMT       SUBST8    D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T3
        DLT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.8++],T3
        DT        SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.8++]
        DMT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2+A0.8++]
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.2+A0.8++]
        DT        SUBSB8    D1Re0,D1Re0,D1Re0 [A0.2],T3
        DLT       SUBST8    D1Re0,D1Re0,D1Re0 [A0.2],T3
        DLMT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.2],T3
        DT        ORS       D1Re0,D1Re0,D1Re0 T3,[A0.2]
        DLT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2]
        DLMT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2]
        DLQT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.2]
        DLT       ORS       D1Re0,D1Re0,D1Re0 [A0.2++],T3
        DMT       SUBST8    D1Re0,D1Re0,D1Re0 [A0.2++],T3
        DLT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2++],T3
        DT        SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.2++]
        DMT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2++]
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.2++]
        DT        SUBSB8    D1Re0,D1Re0,D1Re0 [A0.2--],T3
        DLT       SUBST8    D1Re0,D1Re0,D1Re0 [A0.2--],T3
        DLMT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.2--],T3
        DT        ORS       D1Re0,D1Re0,D1Re0 T3,[A0.2--]
        DLT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2--]
        DLMT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.2--]
        DLQT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.2--]
        DM        SUBSB8    D1.7,D1Re0,D1Re0
        D         ORSQ      D1.7,D1Re0,A1LbP,A1.3
        DL        SUBSB8    D1.7,D1Re0,D1.7
        DL        ORSQ      D1.7,D1Re0,D1.7,A1.3
        DL        ORSQ      D1.7,D1Re0,RD,A1.3
        DL        ORS       D1.7,D1Re0,[D1BR.1]
        D         ORS       D1.7,D1Re0,A0.7
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T3
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T3
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T3
        DWT       ORS       D1Re0,D1Re0,D1Re0 T3,[A0.7+A0FrP++]
        DLWT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.7+A0FrP++]
        DLMWT     SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.7+A0FrP++]
        DLQWT     ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.7+A0FrP++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T3
        DLWT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.2++],T3
        DWT       SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.2++]
        DMWT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.2++]
        DWT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.7+A0.2++]
        DL        ORS       D1.7,D1.7,D1Re0
        DL        ORSQ      D1.7,D1.7,D1Re0,A1.3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T3
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T3
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T3
        D         ORS       D1.7,D1.7,D1.7
        DM        SUBST8    D1.7,D1.7,D1.7
        D         ORSQ      D1.7,D1.7,D0Re0,A1.3
        DT        ORS       D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.7++]
        DLT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.7++]
        DLMT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.7++]
        DLQT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.7+A0.7++]
        D         ORSQ      D1.7,D1.7,[D1BR.1],A1.3
        DL        ORS       D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DT        SUBST8    D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T3
        DMT       SUBSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T3
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.8++],T3
        DT        SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.8++]
        DLT       SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.8++]
        DLMT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.7+A0.8++]
        DM        SUBSB8    D1.7,D1.16,D1Re0
        DT        SUBST8    D1Re0,D1Re0,D1Re0 [A0.7],T3
        DMT       SUBSB8    D1Re0,D1Re0,D1Re0 [A0.7],T3
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7],T3
        DLM       SUBSB8    D1.7,D1.16,D1.7
        DLT       ORS       D1Re0,D1Re0,D1Re0 T3,[A0.7]
        DMT       SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.7]
        DLT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.7]
        DT        SUBST8    D1Re0,D1Re0,D1Re0 [A0.7++],T3
        DMT       SUBSB8    D1Re0,D1Re0,D1Re0 [A0.7++],T3
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7++],T3
        DT        SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.7++]
        DLT       SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.7++]
        DLMT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.7++]
        DT        ORS       D1Re0,D1Re0,D1Re0 [A0.7--],T3
        DLT       SUBSB8    D1Re0,D1Re0,D1Re0 [A0.7--],T3
        DLMT      SUBSB8    D1Re0,D1Re0,D1Re0 [A0.7--],T3
        DLQT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7--],T3
        DLT       ORS       D1Re0,D1Re0,D1Re0 T3,[A0.7--]
        DMT       SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.7--]
        DLT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.7--]
        DL        SUBSB8    D1.7,[D1BR.1],D1Re0
        D         ORSQ      D1.7,[D1BR.1],A1.3,A1.3
        D         ORSQ      D1.7,[D1BR.1],D1.7,A1.3
        D         ORS       D1.7,[D1BR.1],RD
        D         ORSQ      D1.7,[D1BR.1],A0.2,A1.3
        DL        ORSQ      D1.7,[D1BR.1],[D1BR.1++],A1.3
        D         SUBSB8    D1.7,[D1BR.1++],D1Re0
        D         ORS       D1.7,[D1BR.1++],A1LbP
        DL        ORS       D1.7,[D1BR.1++],D1.7
        D         ORSQ      D1.7,[D1BR.1++],D0Re0,A1.3
        D         ORS       D1.7,[D1BR.1++],A0FrP
        D         ORS       D1.7,[D1BR.1++],[D1BR.1++]
        D         ORSQ      D1.7,[D1BR.1++],A0.7,A1.3
        D         ORSQ      D1.7,[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         ORS       D1.7,[D1BR.1+D1BRI.1++],A1.7
        DL        ORSQ      D1.7,[D1BR.1+D1BRI.1++],D1.7,A1.3
        DL        ORSQ      D1.7,[D1BR.1+D1BRI.1++],RD,A1.3
        DL        ORS       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ORS       D1.7,[D1BR.1+D1BRI.1++],A0.7
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T3
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T3
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T3
        DWT       ORS       D1Re0,D1Re0,D1Re0 T3,[A0.8+A0FrP++]
        DLWT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8+A0FrP++]
        DLMWT     SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8+A0FrP++]
        DLQWT     ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.8+A0FrP++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T3
        DLWT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.2++],T3
        DWT       SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.2++]
        DMWT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.2++]
        DWT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.8+A0.2++]
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T3
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T3
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T3
        DWT       ORS       D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.7++]
        DLWT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.7++]
        DLMWT     SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.7++]
        DLQWT     ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.8+A0.7++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T3
        DLWT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.8++],T3
        DWT       SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.8++]
        DMWT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8+A0.8++]
        DWT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.8+A0.8++]
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 [A0.8],T3
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.8],T3
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 [A0.8],T3
        DWT       ORS       D1Re0,D1Re0,D1Re0 T3,[A0.8]
        DLWT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8]
        DLMWT     SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8]
        DLQWT     ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.8]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A0.8++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.8++],T3
        DLWT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8++],T3
        DWT       SUBST8    D1Re0,D1Re0,D1Re0 T3,[A0.8++]
        DMWT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8++]
        DWT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.8++]
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 [A0.8--],T3
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 [A0.8--],T3
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 [A0.8--],T3
        DWT       ORS       D1Re0,D1Re0,D1Re0 T3,[A0.8--]
        DLWT      SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8--]
        DLMWT     SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A0.8--]
        DLQWT     ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[A0.8--]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1LbP++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1LbP++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1LbP++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.3++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.3++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.3++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.7++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.7++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.7++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.8++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.8++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP+A1.8++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1LbP],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1LbP],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1LbP]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1LbP++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1LbP++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1LbP++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1LbP--],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1LbP--],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1LbP--]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP--]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1LbP--]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1LbP++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1LbP++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1LbP++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.3++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.3++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.3++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.7++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.7++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.7++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.8++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.8++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3+A1.8++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.3],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.3],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.3]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.3++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.3++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.3++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.3--],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.3--],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.3--]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3--]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.3--]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1LbP++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1LbP++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1LbP++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.3++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.3++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.3++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.7++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.7++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.7++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.8++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.8++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7+A1.8++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.7],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.7],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.7]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.7++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.7++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.7++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.7--],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.7--],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.7--]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7--]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.7--]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1LbP++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1LbP++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1LbP++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.3++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.3++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.3++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.7++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.7++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.7++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.8++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.8++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8+A1.8++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.8],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.8],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.8]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.8++],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.8++],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.8++]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8++]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8++]
        DLWT      ORS       D1Re0,D1Re0,D1Re0 [A1.8--],T3
        DMWT      SUBST8    D1Re0,D1Re0,D1Re0 [A1.8--],T3
        DWT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[A1.8--]
        DLWT      SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8--]
        DLMWT     SUBST8    D1Re0,D1Re0,D1Re0 T3,[A1.8--]
        DM        SUBST8    [D1BW.1],D1Re0,D1Re0
        D         ORS       [D1BW.1],D1Re0,A1.3
        DL        SUBST8    [D1BW.1],D1Re0,D1.7
        DLQ       ORSQ      [D1BW.1],D1Re0,D1.7,A1.3
        DLQ       ORSQ      [D1BW.1],D1Re0,D1.16,A1.3
        D         ORSQ      [D1BW.1],D1Re0,[D1BR.1],A1.3
        DL        ORS       [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DL        SUBSB8    [D1BW.1],D1.7,D1Re0
        DLQ       ORSQ      [D1BW.1],D1.7,D1Re0,A1.3
        D         SUBST8    [D1BW.1],D1.7,D1.7
        D         ORSQ      [D1BW.1],D1.7,D1.7,A1.3
        D         ORS       [D1BW.1],D1.7,RD
        D         ORSQ      [D1BW.1],D1.7,A0.2,A1.3
        DL        ORSQ      [D1BW.1],D1.7,[D1BR.1++],A1.3
        DM        SUBST8    [D1BW.1],D1.16,D1Re0
        DT        SUBSB8    D1Re0,D1Re0,D1Re0 [D1BW.1],T3
        DLMT      SUBST8    D1Re0,D1Re0,D1Re0 [D1BW.1],T3
        DLT       SUBSB8    D1Re0,D1Re0,D1Re0 T3,[D1BR.1]
        DLQT      ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[D1BR.1]
        DL        ORSQ      [D1BW.1],[D1BR.1],D1Re0,A1.3
        D         SUBSB8    [D1BW.1],[D1BR.1],D1.7
        DLQ       ORSQ      [D1BW.1],[D1BR.1],D1.7,A1.3
        DLQ       ORSQ      [D1BW.1],[D1BR.1],D1.16,A1.3
        D         ORSQ      [D1BW.1],[D1BR.1],[D1BR.1],A1.3
        DL        ORS       [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        SUBSB8    [D1BW.1],[D1BR.1++],D1Re0
        D         ORSQ      [D1BW.1],[D1BR.1++],A1.3,A1.3
        D         ORSQ      [D1BW.1],[D1BR.1++],D1.7,A1.3
        D         ORS       [D1BW.1],[D1BR.1++],RD
        D         ORSQ      [D1BW.1],[D1BR.1++],A0.2,A1.3
        DL        ORSQ      [D1BW.1],[D1BR.1++],[D1BR.1++],A1.3
        D         SUBSB8    [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        D         ORS       [D1BW.1],[D1BR.1+D1BRI.1++],A1LbP
        DL        ORS       [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        D         ORSQ      [D1BW.1],[D1BR.1+D1BRI.1++],D0Re0,A1.3
        D         ORS       [D1BW.1],[D1BR.1+D1BRI.1++],A0FrP
        D         ORS       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ORSQ      [D1BW.1],[D1BR.1+D1BRI.1++],A0.7,A1.3
        DM        SUBSB8    [D1BW.1++],D1Re0,D1Re0
        D         ORSQ      [D1BW.1++],D1Re0,A1LbP,A1.3
        DL        SUBSB8    [D1BW.1++],D1Re0,D1.7
        DL        ORSQ      [D1BW.1++],D1Re0,D1.7,A1.3
        DL        ORSQ      [D1BW.1++],D1Re0,RD,A1.3
        DL        ORS       [D1BW.1++],D1Re0,[D1BR.1]
        D         ORS       [D1BW.1++],D1Re0,A0.7
        DL        ORS       [D1BW.1++],D1.7,D1Re0
        DL        ORSQ      [D1BW.1++],D1.7,D1Re0,A1.3
        D         SUBSB8    [D1BW.1++],D1.7,D1.7
        DLM       SUBST8    [D1BW.1++],D1.7,D1.7
        D         ORSQ      [D1BW.1++],D1.7,D0.7,A1.3
        D         ORS       [D1BW.1++],D1.7,A0.2
        D         ORSQ      [D1BW.1++],D1.7,[D1BR.1++],A1.3
        DM        SUBSB8    [D1BW.1++],D1.16,D1Re0
        DT        ORS       D1Re0,D1Re0,D1Re0 [D1BW.1++],T3
        DLMT      SUBSB8    D1Re0,D1Re0,D1Re0 [D1BW.1++],T3
        DLT       ORS       D1Re0,D1Re0,D1Re0 T3,[D1BR.1++]
        DLT       ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[D1BR.1++]
        D         ORSQ      [D1BW.1++],[D1BR.1],D1Re0,A1.3
        D         ORS       [D1BW.1++],[D1BR.1],A1.7
        DL        ORSQ      [D1BW.1++],[D1BR.1],D1.7,A1.3
        DL        ORSQ      [D1BW.1++],[D1BR.1],RD,A1.3
        DL        ORS       [D1BW.1++],[D1BR.1],[D1BR.1]
        D         ORS       [D1BW.1++],[D1BR.1],A0.7
        DL        ORS       [D1BW.1++],[D1BR.1++],D1Re0
        D         ORS       [D1BW.1++],[D1BR.1++],A1.3
        DL        SUBST8    [D1BW.1++],[D1BR.1++],D1.7
        D         ORSQ      [D1BW.1++],[D1BR.1++],D0.7,A1.3
        D         ORS       [D1BW.1++],[D1BR.1++],A0.2
        D         ORSQ      [D1BW.1++],[D1BR.1++],[D1BR.1++],A1.3
        D         ORS       [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        DLQ       ORSQ      [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         SUBST8    [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        D         ORS       [D1BW.1++],[D1BR.1+D1BRI.1++],D0Re0
        DLQ       ORSQ      [D1BW.1++],[D1BR.1+D1BRI.1++],RD,A1.3
        DL        ORSQ      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1],A1.3
        D         ORSQ      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        SUBST8    [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        D         ORS       [D1BW.1+D1BWI.1++],D1Re0,A1LbP
        DL        ORS       [D1BW.1+D1BWI.1++],D1Re0,D1.7
        D         ORSQ      [D1BW.1+D1BWI.1++],D1Re0,A1.7,A1.3
        D         ORSQ      [D1BW.1+D1BWI.1++],D1Re0,RD,A1.3
        D         ORS       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        D         ORS       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        D         SUBST8    [D1BW.1+D1BWI.1++],D1.7,D1Re0
        D         ORSQ      [D1BW.1+D1BWI.1++],D1.7,D1Re0,A1.3
        D         ORS       [D1BW.1+D1BWI.1++],D1.7,A1.7
        DLM       SUBSB8    [D1BW.1+D1BWI.1++],D1.7,D1.7
        D         ORS       [D1BW.1+D1BWI.1++],D1.7,D0.7
        D         ORSQ      [D1BW.1+D1BWI.1++],D1.7,A0FrP,A1.3
        DL        ORS       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        DL        ORSQ      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++],A1.3
        DLM       SUBST8    [D1BW.1+D1BWI.1++],D1.16,D1.7
        DMT       SUBST8    D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T3
        DT        SUBST8    D1Re0,D1Re0,D1Re0 T3,[D1BR.1+D1BRI.1++]
        DT        ORSQ      D1Re0,D1Re0,D1Re0,A1.3 T3,[D1BR.1+D1BRI.1++]
        DL        SUBST8    [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        D         ORS       [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        D         ORSQ      [D1BW.1+D1BWI.1++],[D1BR.1],A1.7,A1.3
        D         ORSQ      [D1BW.1+D1BWI.1++],[D1BR.1],RD,A1.3
        D         ORS       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        D         ORS       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         SUBST8    [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        D         ORSQ      [D1BW.1+D1BWI.1++],[D1BR.1++],A1LbP,A1.3
        DL        SUBSB8    [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        D         ORS       [D1BW.1+D1BWI.1++],[D1BR.1++],D0.7
        D         ORSQ      [D1BW.1+D1BWI.1++],[D1BR.1++],A0FrP,A1.3
        DL        ORS       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        DL        ORSQ      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        ORSQ      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         SUBSB8    [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        DLQ       ORSQ      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7,A1.3
        DLQ       ORSQ      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.16,A1.3
        D         ORSQ      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1],A1.3
        DL        ORS       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ORS       D0Re0,D0Re0,#-0x1
        D         ORS       [D0AW.0],[D0AR.0],#0x7fff
        D         ORS       [D0AW.0++],[D0AR.0++],#-0x8000
        D         ORS       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#-0x1
        D         ORS       D1.7,D1.7,#0x7fff
        D         ORS       [D1BW.1],[D1BR.1],#-0x8000
        D         ORS       [D1BW.1++],[D1BR.1++],#-0x1
        DLLS      ORSQLS    D0Re0,D0Re0,D0Re0,A0.2
        D         ORSQNV    D0Re0,D0Re0,D0.7,A0.2
        D         ORSQNV    D0Re0,D0Re0,[D0AR.0],A0.2
        DECS      ORSQLE    D0Re0,D0Re0,A0.2,A0.2
        DHNE      ORSQPL    D0Re0,D0.7,A1.3,A0.2
        DHNE      ORSQPL    D0Re0,D0.7,D1.7,A0.2
        DLLS      ORSQLS    D0Re0,D0.7,[D0AR.0+D0ARI.0++],A0.2
        D         ORSQNV    D0Re0,[D0AR.0],D0Re0,A0.2
        D         ORSQNV    D0Re0,[D0AR.0],RD,A0.2
        D         ORSQNV    D0Re0,[D0AR.0++],A1.3,A0.2
        D         ORSQNV    D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        D         ORSQNV    D0Re0,[D0AR.0+D0ARI.0++],A1.7,A0.2
        D         ORSQNV    D0Re0,[D0AR.0+D0ARI.0++],A0.2,A0.2
        DLNE      ORSQNE    D0.7,D0Re0,D0.7,A0.2
        DLNE      ORSQNE    D0.7,D0Re0,[D0AR.0],A0.2
        DHNE      ORSQPL    D0.7,D0Re0,A0FrP,A0.2
        DHNE      ORSQPL    D0.7,D0.7,A1LbP,A0.2
        DHNE      ORSQPL    D0.7,D0.7,D1Re0,A0.2
        DLNE      ORSQNE    D0.7,D0.7,[D0AR.0++],A0.2
        DHNE      ORSQPL    D0.7,D0.7,A0.7,A0.2
        D         ORSQNV    D0.7,[D0AR.0],D1.7,A0.2
        D         ORSQNV    D0.7,[D0AR.0++],D0Re0,A0.2
        D         ORSQNV    D0.7,[D0AR.0++],RD,A0.2
        D         ORSQNV    D0.7,[D0AR.0+D0ARI.0++],A1.3,A0.2
        DHCS      ORSQVS    D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++],A0.2
        DHNE      ORSQPL    D0.16,D0Re0,A1.3,A0.2
        DHNE      ORSQPL    D0.16,D0Re0,D1.7,A0.2
        DLLS      ORSQLS    D0.16,D0Re0,[D0AR.0+D0ARI.0++],A0.2
        DLLS      ORSQLS    D0.16,D0.7,D0Re0,A0.2
        D         ORSQNV    D0.16,D0.7,D0.7,A0.2
        D         ORSQNV    D0.16,D0.7,[D0AR.0],A0.2
        DECS      ORSQLE    D0.16,D0.7,A0.2,A0.2
        D         ORSQNV    D0.16,[D0AR.0],A1.7,A0.2
        D         ORSQNV    D0.16,[D0AR.0],A0.2,A0.2
        D         ORSQNV    D0.16,[D0AR.0++],D1.7,A0.2
        D         ORSQNV    D0.16,[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         ORSQNV    D0.16,[D0AR.0+D0ARI.0++],RD,A0.2
        DECS      ORSQLE    D1Re0,D1Re0,A1LbP,A1.3
        DECS      ORSQLE    D1Re0,D1Re0,D0Re0,A1.3
        DECS      ORSQLE    D1Re0,D1Re0,A0.2,A1.3
        D         ORSQNV    D1Re0,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DHNE      ORSQPL    D1Re0,D1.7,D1.7,A1.3
        DECS      ORSQLE    D1Re0,D1.7,RD,A1.3
        DHNE      ORSQPL    D1Re0,D1.7,[D1BR.1++],A1.3
        D         ORSQNV    D1Re0,[D1BR.1],A1.3,A1.3
        D         ORSQNV    D1Re0,[D1BR.1],[D1BR.1],A1.3
        D         ORSQNV    D1Re0,[D1BR.1++],A1.7,A1.3
        D         ORSQNV    D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        D         ORSQNV    D1Re0,[D1BR.1+D1BRI.1++],D0.7,A1.3
        DHNE      ORSQPL    D1.7,D1Re0,D1Re0,A1.3
        D         ORSQNV    D1.7,D1Re0,D1.7,A1.3
        DECS      ORSQLE    D1.7,D1Re0,A0FrP,A1.3
        DHNE      ORSQPL    D1.7,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DECS      ORSQLE    D1.7,D1.7,A1.3,A1.3
        DECS      ORSQLE    D1.7,D1.7,D0.7,A1.3
        DHNE      ORSQPL    D1.7,D1.7,[D1BR.1],A1.3
        D         ORSQNV    D1.7,[D1BR.1],D1Re0,A1.3
        D         ORSQNV    D1.7,[D1BR.1],A0FrP,A1.3
        D         ORSQNV    D1.7,[D1BR.1++],A1.3,A1.3
        D         ORSQNV    D1.7,[D1BR.1++],[D1BR.1],A1.3
        D         ORSQNV    D1.7,[D1BR.1+D1BRI.1++],A1.7,A1.3
        D         ORSQNV    D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        DHNE      ORSQPL    D1.16,D1Re0,D1.7,A1.3
        DECS      ORSQLE    D1.16,D1Re0,RD,A1.3
        DHNE      ORSQPL    D1.16,D1Re0,[D1BR.1++],A1.3
        DECS      ORSQLE    D1.16,D1.7,A1LbP,A1.3
        DECS      ORSQLE    D1.16,D1.7,D0Re0,A1.3
        DECS      ORSQLE    D1.16,D1.7,A0.2,A1.3
        D         ORSQNV    D1.16,D1.7,[D1BR.1+D1BRI.1++],A1.3
        D         ORSQNV    D1.16,[D1BR.1],D0.7,A1.3
        D         ORSQNV    D1.16,[D1BR.1++],D1Re0,A1.3
        D         ORSQNV    D1.16,[D1BR.1++],A0FrP,A1.3
        D         ORSQNV    D1.16,[D1BR.1+D1BRI.1++],A1.3,A1.3
        D         ORSQNV    D1.16,[D1BR.1+D1BRI.1++],[D1BR.1],A1.3
        DL        MULB8     D0Re0,D0Re0,D0Re0
        DLM       MULT8     D0Re0,D0Re0,D0Re0
        D         MULT8     D0Re0,D0Re0,D0.7
        DM        MULB8     D0Re0,D0Re0,D0.7
        D         XOR       D0Re0,D0Re0,[D0AR.0]
        DLQ       XORQ      D0Re0,D0Re0,RD,A0.2
        DL        XORQ      D0Re0,D0Re0,[D0AR.0+D0ARI.0++],A0.2
        D         MULB8     D0Re0,D0.7,D0Re0
        DM        MULB8     D0Re0,D0.7,D0Re0
        D         XOR       D0Re0,D0.7,D0.7
        D         XORQ      D0Re0,D0.7,A1.7,A0.2
        D         XORQ      D0Re0,D0.7,D1Re0,A0.2
        DL        XORQ      D0Re0,D0.7,[D0AR.0],A0.2
        D         XOR       D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        D         XOR       D0Re0,D0.7,A0.7
        D         XORQ      D0Re0,[D0AR.0],D0Re0,A0.2
        D         XORQ      D0Re0,[D0AR.0],A1LbP,A0.2
        DL        MULB8     D0Re0,[D0AR.0],D0.7
        DLM       MULB8     D0Re0,D0.16,D0.7
        DL        XOR       D0Re0,[D0AR.0],[D0AR.0]
        DL        XOR       D0Re0,[D0AR.0],[D0AR.0++]
        D         XORQ      D0Re0,[D0AR.0],A0FrP,A0.2
        DL        XOR       D0Re0,[D0AR.0++],D0Re0
        D         XOR       D0Re0,[D0AR.0++],A1.3
        DL        MULT8     D0Re0,[D0AR.0++],D0.7
        D         XORQ      D0Re0,[D0AR.0++],D1.7,A0.2
        DLQ       XORQ      D0Re0,[D0AR.0++],D0.16,A0.2
        D         XORQ      D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        D         XOR       D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        DLQ       XORQ      D0Re0,[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         MULT8     D0Re0,[D0AR.0+D0ARI.0++],D0.7
        D         XOR       D0Re0,[D0AR.0+D0ARI.0++],D1Re0
        D         XORQ      D0Re0,[D0AR.0+D0ARI.0++],RD,A0.2
        DL        XORQ      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++],A0.2
        D         XORQ      D0Re0,[D0AR.0+D0ARI.0++],A0.2,A0.2
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0FrP++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0FrP++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0FrP++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.2++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.2++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.2++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.7++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.7++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.7++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.8++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.8++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.8++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0FrP],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0FrP],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0FrP]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0FrP++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0FrP++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0FrP++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0FrP--],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0FrP--],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0FrP--]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP--]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0FrP--]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.2+A0FrP++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2+A0FrP++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2+A0FrP++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.2++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.2++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.2++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.7++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.7++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.7++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.8++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.8++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.8++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0.2],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0.2],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.2]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0.2++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0.2++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.2++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A0.2--],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A0.2--],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.2--]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2--]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.2--]
        DL        MULT8     D0.7,D0Re0,D0Re0
        D         XOR       D0.7,D0Re0,A1LbP
        DL        XOR       D0.7,D0Re0,D0.7
        DM        MULT8     D0.7,D0Re0,D0.7
        D         XOR       D0.7,D0Re0,RD
        D         XOR       D0.7,D0Re0,[D0AR.0++]
        D         XOR       D0.7,D0Re0,A0FrP
        DT        MULB8     D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T4
        DLT       MULT8     D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T4
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T4
        DLT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.7+A0FrP++]
        DMT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.7+A0FrP++]
        DT        MULB8     D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T4
        DLT       MULT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T4
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T4
        DLT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.2++]
        DMT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.2++]
        D         MULT8     D0.7,D0.7,D0Re0
        DM        MULT8     D0.7,D0.7,D0Re0
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T4
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T4
        DL        MULB8     D0.7,D0.7,D0.7
        DLM       MULB8     D0.7,D0.7,D0.7
        DL        XOR       D0.7,D0.7,[D0AR.0]
        DL        XOR       D0.7,D0.7,[D0AR.0++]
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.7++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.7++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.7++]
        D         XORQ      D0.7,D0.7,A0.7,A0.2
        DLWT      XOR       D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.8++]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.8++]
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.8++]
        DL        XORQ      D0.7,[D0AR.0],D0Re0,A0.2
        D         XOR       D0.7,[D0AR.0],A1.3
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A0.7],T4
        DMT       MULT8     D0Re0,D0Re0,D0Re0 [A0.7],T4
        D         MULB8     D0.7,[D0AR.0],D0.7
        DLQ       XORQ      D0.7,[D0AR.0],D0.7,A0.2
        D         XORQ      D0.7,[D0AR.0],D1.7,A0.2
        DLQ       XORQ      D0.7,[D0AR.0],D0.16,A0.2
        D         XORQ      D0.7,[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        DLT       XOR       D0Re0,D0Re0,D0Re0 T4,[A0.7]
        DMT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.7]
        D         XOR       D0.7,[D0AR.0],A0.2
        DL        MULB8     D0.7,[D0AR.0++],D0Re0
        D         XORQ      D0.7,[D0AR.0++],A1.3,A0.2
        DLWT      XOR       D0Re0,D0Re0,D0Re0 [A0.7++],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A0.7++],T4
        D         MULT8     D0.7,[D0AR.0++],D0.7
        D         XOR       D0.7,[D0AR.0++],D1Re0
        D         XORQ      D0.7,[D0AR.0++],RD,A0.2
        DL        XORQ      D0.7,[D0AR.0++],[D0AR.0++],A0.2
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.7++]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.7++]
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.7++]
        D         MULB8     D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         XOR       D0.7,[D0AR.0+D0ARI.0++],A1LbP
        DL        XOR       D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         XORQ      D0.7,[D0AR.0+D0ARI.0++],D1Re0,A0.2
        DL        XORQ      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0],A0.2
        D         XOR       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         XOR       D0.7,[D0AR.0+D0ARI.0++],A0.7
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A0.7--],T4
        DMT       MULT8     D0Re0,D0Re0,D0Re0 [A0.7--],T4
        DT        MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.7--]
        DLT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.7--]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.7--]
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T4
        DMT       MULT8     D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T4
        DT        MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0FrP++]
        DLT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0FrP++]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0FrP++]
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T4
        DMT       MULT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T4
        DT        MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.2++]
        DLT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.2++]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.2++]
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T4
        DMT       MULT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T4
        DT        MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.7++]
        DLT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.7++]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.7++]
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T4
        DMT       MULT8     D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T4
        DT        MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.8++]
        DLT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.8++]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.8++]
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A0.8],T4
        DMT       MULT8     D0Re0,D0Re0,D0Re0 [A0.8],T4
        DT        MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.8]
        DLT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8]
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A0.8++],T4
        DMT       MULT8     D0Re0,D0Re0,D0Re0 [A0.8++],T4
        DT        MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.8++]
        DLT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8++]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8++]
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A0.8--],T4
        DMT       MULT8     D0Re0,D0Re0,D0Re0 [A0.8--],T4
        DT        MULB8     D0Re0,D0Re0,D0Re0 T4,[A0.8--]
        DLT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8--]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A0.8--]
        D         XORQ      [D0AW.0],D0Re0,D0Re0,A0.2
        D         XORQ      [D0AW.0],D0Re0,A1LbP,A0.2
        DL        MULB8     [D0AW.0],D0Re0,D0.7
        DLM       MULB8     [D0AW.0],D0Re0,D0.7
        DL        XOR       [D0AW.0],D0Re0,[D0AR.0]
        DL        XOR       [D0AW.0],D0Re0,[D0AR.0++]
        D         XORQ      [D0AW.0],D0Re0,A0FrP,A0.2
        DL        XOR       [D0AW.0],D0.7,D0Re0
        DLM       MULB8     [D0AW.0],D0.7,D0Re0
        D         MULB8     [D0AW.0],D0.7,D0.7
        DLQ       XORQ      [D0AW.0],D0.7,D0.7,A0.2
        D         XORQ      [D0AW.0],D0.7,D1.7,A0.2
        DLQ       XORQ      [D0AW.0],D0.7,D0.16,A0.2
        D         XORQ      [D0AW.0],D0.7,[D0AR.0+D0ARI.0++],A0.2
        D         XOR       [D0AW.0],[D0AR.0],D0Re0
        DLQ       XORQ      [D0AW.0],[D0AR.0],D0Re0,A0.2
        D         XORQ      [D0AW.0],[D0AR.0],A1.3,A0.2
        D         XORQ      [D0AW.0],[D0AR.0],D0.7,A0.2
        D         XOR       [D0AW.0],[D0AR.0],D1Re0
        D         XORQ      [D0AW.0],[D0AR.0],RD,A0.2
        DL        XORQ      [D0AW.0],[D0AR.0],[D0AR.0++],A0.2
        D         XORQ      [D0AW.0],[D0AR.0],A0.2,A0.2
        DL        MULT8     [D0AW.0],[D0AR.0++],D0Re0
        D         XOR       [D0AW.0],[D0AR.0++],D0.7
        D         XORQ      [D0AW.0],[D0AR.0++],A1.7,A0.2
        D         XOR       [D0AW.0],[D0AR.0++],RD
        D         XOR       [D0AW.0],[D0AR.0++],[D0AR.0++]
        D         XOR       [D0AW.0],[D0AR.0++],A0FrP
        D         MULT8     [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        D         XORQ      [D0AW.0],[D0AR.0+D0ARI.0++],A1LbP,A0.2
        DL        MULB8     [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        D         XOR       [D0AW.0],[D0AR.0+D0ARI.0++],D1.7
        DL        XORQ      [D0AW.0],[D0AR.0+D0ARI.0++],RD,A0.2
        DL        XOR       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         XORQ      [D0AW.0],[D0AR.0+D0ARI.0++],A0.7,A0.2
        DLT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [D0AW.0],T4
        DT        MULT8     D0Re0,D0Re0,D0Re0 T4,[D0AR.0]
        DMT       MULT8     D0Re0,D0Re0,D0Re0 T4,[D0AR.0]
        DL        MULT8     [D0AW.0++],D0Re0,D0Re0
        D         XOR       [D0AW.0++],D0Re0,A1LbP
        DL        XOR       [D0AW.0++],D0Re0,D0.7
        DM        MULT8     [D0AW.0++],D0Re0,D0.7
        D         XOR       [D0AW.0++],D0Re0,RD
        D         XOR       [D0AW.0++],D0Re0,[D0AR.0++]
        D         XOR       [D0AW.0++],D0Re0,A0FrP
        DT        MULB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T4
        DLT       MULT8     D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T4
        DMT       MULB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T4
        DT        XOR       D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1LbP++]
        DLT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1LbP++]
        DLQT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP+A1LbP++]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1LbP++]
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T4
        DLT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.3++],T4
        DLMT      MULB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T4
        DT        MULT8     D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.3++]
        DT        XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP+A1.3++]
        DMT       MULT8     D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.3++]
        D         MULT8     [D0AW.0++],D0.7,D0Re0
        DM        MULT8     [D0AW.0++],D0.7,D0Re0
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.7++],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T4
        D         XORQ      [D0AW.0++],D0.7,D0.7,A0.2
        D         XOR       [D0AW.0++],D0.7,D1Re0
        D         XORQ      [D0AW.0++],D0.7,RD,A0.2
        DL        XORQ      [D0AW.0++],D0.7,[D0AR.0++],A0.2
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.7++]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.7++]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.7++]
        D         XORQ      [D0AW.0++],D0.7,A0FrP,A0.2
        DWT       MULB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T4
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T4
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T4
        DWT       XOR       D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.8++]
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.8++]
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP+A1.8++]
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.8++]
        DL        XORQ      [D0AW.0++],[D0AR.0],D0Re0,A0.2
        D         XOR       [D0AW.0++],[D0AR.0],A1.3
        DLT       XOR       D0Re0,D0Re0,D0Re0 [A1LbP],T4
        DLT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP],T4
        DLMT      MULB8     D0Re0,D0Re0,D0Re0 [A1LbP],T4
        DL        XOR       [D0AW.0++],[D0AR.0],D0.7
        DM        MULT8     [D0AW.0++],D0.16,D0.7
        D         XOR       [D0AW.0++],[D0AR.0],RD
        DLQ       XORQ      [D0AW.0++],[D0AR.0],RD,A0.2
        DL        XORQ      [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1LbP]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1LbP]
        D         XORQ      [D0AW.0++],[D0AR.0],A0.7,A0.2
        DL        XORQ      [D0AW.0++],[D0AR.0++],D0Re0,A0.2
        DT        MULB8     D0Re0,D0Re0,D0Re0 [A1LbP++],T4
        DLT       MULT8     D0Re0,D0Re0,D0Re0 [A1LbP++],T4
        DMT       MULB8     D0Re0,D0Re0,D0Re0 [A1LbP++],T4
        D         XOR       [D0AW.0++],[D0AR.0++],D0.7
        D         XORQ      [D0AW.0++],[D0AR.0++],A1.7,A0.2
        D         XOR       [D0AW.0++],[D0AR.0++],RD
        D         XOR       [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DT        XOR       D0Re0,D0Re0,D0Re0 T4,[A1LbP++]
        DLT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1LbP++]
        DLQT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP++]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1LbP++]
        D         XOR       [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLQ       XORQ      [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         MULT8     [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        D         XOR       [D0AW.0++],[D0AR.0+D0ARI.0++],D1Re0
        D         XORQ      [D0AW.0++],[D0AR.0+D0ARI.0++],RD,A0.2
        DL        XORQ      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++],A0.2
        D         XORQ      [D0AW.0++],[D0AR.0+D0ARI.0++],A0.2,A0.2
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1LbP--],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1LbP--],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1LbP--],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1LbP--]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1LbP--]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1LbP--]
        DT        MULB8     D0Re0,D0Re0,D0Re0 [D0AW.0++],T4
        DMT       MULB8     D0Re0,D0Re0,D0Re0 [D0AW.0++],T4
        DLT       MULB8     D0Re0,D0Re0,D0Re0 T4,[D0AR.0++]
        DLMT      MULT8     D0Re0,D0Re0,D0Re0 T4,[D0AR.0++]
        DL        XORQ      [D0AW.0+D0AWI.0++],D0Re0,D0Re0,A0.2
        D         XOR       [D0AW.0+D0AWI.0++],D0Re0,A1.3
        DL        MULT8     [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DLM       MULT8     [D0AW.0+D0AWI.0++],D0Re0,D0.7
        D         XORQ      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0],A0.2
        D         XORQ      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++],A0.2
        D         XOR       [D0AW.0+D0AWI.0++],D0Re0,A0.2
        DL        MULB8     [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DLM       MULT8     [D0AW.0+D0AWI.0++],D0.7,D0Re0
        D         MULT8     [D0AW.0+D0AWI.0++],D0.7,D0.7
        DM        MULB8     [D0AW.0+D0AWI.0++],D0.7,D0.7
        D         XOR       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DLQ       XORQ      [D0AW.0+D0AWI.0++],D0.7,RD,A0.2
        DL        XORQ      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++],A0.2
        D         MULB8     [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DM        MULB8     [D0AW.0+D0AWI.0++],D0.16,D0Re0
        D         XOR       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        D         XORQ      [D0AW.0+D0AWI.0++],[D0AR.0],A1.7,A0.2
        D         XORQ      [D0AW.0+D0AWI.0++],[D0AR.0],D1Re0,A0.2
        D         XORQ      [D0AW.0+D0AWI.0++],[D0AR.0],RD,A0.2
        DL        XORQ      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++],A0.2
        D         XORQ      [D0AW.0+D0AWI.0++],[D0AR.0],A0.2,A0.2
        DL        MULT8     [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         XOR       [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        D         XORQ      [D0AW.0+D0AWI.0++],[D0AR.0++],A1.7,A0.2
        D         XOR       [D0AW.0+D0AWI.0++],[D0AR.0++],RD
        D         XOR       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        D         XOR       [D0AW.0+D0AWI.0++],[D0AR.0++],A0FrP
        D         MULT8     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         XORQ      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A1LbP,A0.2
        DL        MULB8     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        D         XOR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D1.7
        DL        XORQ      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],RD,A0.2
        DL        XOR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         XORQ      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A0.7,A0.2
        DLT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [D0AW.0+D0AWI.0++],T4
        DT        MULT8     D0Re0,D0Re0,D0Re0 T4,[D0AR.0+D0ARI.0++]
        DMT       MULT8     D0Re0,D0Re0,D0Re0 T4,[D0AR.0+D0ARI.0++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1LbP++],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3+A1LbP++]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.3+A1LbP++]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3+A1LbP++]
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.3++],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T4
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.3++]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.3+A1.3++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.3++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.7++],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.7++]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.7++]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.7++]
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.8++],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T4
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.8++]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.3+A1.8++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.8++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.3],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.3],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.3]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3]
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1.3++],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1.3++],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3++],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1.3++],T4
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1.3++]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.3++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.3--],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.3--],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.3--],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3--]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.3--]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.3--]
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1LbP++],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T4
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1.7+A1LbP++]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7+A1LbP++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.7+A1LbP++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.3++],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.3++]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.3++]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.3++]
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.7++],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T4
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.7++]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7+A1.7++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.7++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.8++],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.8++]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.8++]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.8++]
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1.7],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1.7],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1.7],T4
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1.7]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.7]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.7++],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7++],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.7++],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.7++]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.7++]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.7++]
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1.7--],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1.7--],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.7--],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1.7--],T4
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1.7--]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7--]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.7--]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1LbP++],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8+A1LbP++]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.8+A1LbP++]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8+A1LbP++]
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.3++],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T4
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.3++]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.8+A1.3++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.3++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.7++],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.7++]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.7++]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.7++]
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.8++],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T4
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.8++]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.8+A1.8++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.8++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.8],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.8],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.8]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8]
        DWT       XOR       D0Re0,D0Re0,D0Re0 [A1.8++],T4
        DLWT      MULB8     D0Re0,D0Re0,D0Re0 [A1.8++],T4
        DLQWT     XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8++],T4
        DLMWT     MULT8     D0Re0,D0Re0,D0Re0 [A1.8++],T4
        DLWT      XOR       D0Re0,D0Re0,D0Re0 T4,[A1.8++]
        DLWT      XORQ      D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.8++]
        DLMWT     MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8++]
        DWT       MULT8     D0Re0,D0Re0,D0Re0 [A1.8--],T4
        DWT       XORQ      D0Re0,D0Re0,D0Re0,A0.2 [A1.8--],T4
        DMWT      MULT8     D0Re0,D0Re0,D0Re0 [A1.8--],T4
        DWT       MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8--]
        DLWT      MULT8     D0Re0,D0Re0,D0Re0 T4,[A1.8--]
        DMWT      MULB8     D0Re0,D0Re0,D0Re0 T4,[A1.8--]
        D         MULB8     D1Re0,D1Re0,D1Re0
        DLM       MULT8     D1Re0,D1Re0,D1Re0
        D         XOR       D1Re0,D1Re0,D1.7
        DM        MULT8     D1Re0,D1Re0,D1.7
        D         XORQ      D1Re0,D1Re0,D0Re0,A1.3
        D         XOR       D1Re0,D1Re0,A0FrP
        D         XOR       D1Re0,D1Re0,[D1BR.1++]
        D         XORQ      D1Re0,D1Re0,A0.7,A1.3
        DM        MULB8     D1Re0,D1.7,D1Re0
        D         XORQ      D1Re0,D1.7,A1LbP,A1.3
        DL        MULB8     D1Re0,D1.7,D1.7
        DL        XORQ      D1Re0,D1.7,D1.7,A1.3
        DL        XORQ      D1Re0,D1.7,RD,A1.3
        DL        XOR       D1Re0,D1.7,[D1BR.1]
        D         XOR       D1Re0,D1.7,A0.7
        DLM       MULT8     D1Re0,D1.16,D1Re0
        DL        XOR       D1Re0,[D1BR.1],D1Re0
        D         XOR       D1Re0,[D1BR.1],A1.3
        DL        MULT8     D1Re0,[D1BR.1],D1.7
        D         XORQ      D1Re0,[D1BR.1],D0.7,A1.3
        D         XOR       D1Re0,[D1BR.1],A0.2
        D         XORQ      D1Re0,[D1BR.1],[D1BR.1++],A1.3
        D         XOR       D1Re0,[D1BR.1++],D1Re0
        DLQ       XORQ      D1Re0,[D1BR.1++],D1Re0,A1.3
        D         MULT8     D1Re0,[D1BR.1++],D1.7
        D         XOR       D1Re0,[D1BR.1++],D0Re0
        DLQ       XORQ      D1Re0,[D1BR.1++],RD,A1.3
        DL        XORQ      D1Re0,[D1BR.1++],[D1BR.1],A1.3
        D         XORQ      D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        MULT8     D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         XOR       D1Re0,[D1BR.1+D1BRI.1++],D1.7
        D         XORQ      D1Re0,[D1BR.1+D1BRI.1++],A1.7,A1.3
        D         XORQ      D1Re0,[D1BR.1+D1BRI.1++],RD,A1.3
        D         XOR       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         XOR       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DT        MULB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T4
        DLT       MULT8     D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T4
        DLMT      MULT8     D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T4
        DT        XOR       D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0FrP++]
        DLT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0FrP++]
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0FrP++]
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP+A0FrP++]
        DLT       XOR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T4
        DMT       MULT8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T4
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.2++],T4
        DT        MULT8     D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.2++]
        DMT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.2++]
        DT        XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP+A0.2++]
        DT        MULB8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T4
        DLT       MULT8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T4
        DLMT      MULT8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T4
        DT        XOR       D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.7++]
        DLT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.7++]
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.7++]
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP+A0.7++]
        DLT       XOR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T4
        DMT       MULT8     D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T4
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.8++],T4
        DT        MULT8     D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.8++]
        DMT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.8++]
        DT        XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP+A0.8++]
        DT        MULB8     D1Re0,D1Re0,D1Re0 [A0FrP],T4
        DLT       MULT8     D1Re0,D1Re0,D1Re0 [A0FrP],T4
        DLMT      MULT8     D1Re0,D1Re0,D1Re0 [A0FrP],T4
        DT        XOR       D1Re0,D1Re0,D1Re0 T4,[A0FrP]
        DLT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP]
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP]
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP]
        DLT       XOR       D1Re0,D1Re0,D1Re0 [A0FrP++],T4
        DMT       MULT8     D1Re0,D1Re0,D1Re0 [A0FrP++],T4
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0FrP++],T4
        DT        MULT8     D1Re0,D1Re0,D1Re0 T4,[A0FrP++]
        DMT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP++]
        DT        XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP++]
        DT        MULB8     D1Re0,D1Re0,D1Re0 [A0FrP--],T4
        DLT       MULT8     D1Re0,D1Re0,D1Re0 [A0FrP--],T4
        DLMT      MULT8     D1Re0,D1Re0,D1Re0 [A0FrP--],T4
        DT        XOR       D1Re0,D1Re0,D1Re0 T4,[A0FrP--]
        DLT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP--]
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0FrP--]
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP--]
        DLT       XOR       D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T4
        DMT       MULT8     D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T4
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0FrP++],T4
        DT        MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.2+A0FrP++]
        DMT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.2+A0FrP++]
        DT        XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.2+A0FrP++]
        DT        MULB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T4
        DLT       MULT8     D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T4
        DLMT      MULT8     D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T4
        DT        XOR       D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.2++]
        DLT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.2++]
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.2++]
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.2+A0.2++]
        DLT       XOR       D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T4
        DMT       MULT8     D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T4
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.7++],T4
        DT        MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.7++]
        DMT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.7++]
        DT        XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.2+A0.7++]
        DT        MULB8     D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T4
        DLT       MULT8     D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T4
        DLMT      MULT8     D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T4
        DT        XOR       D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.8++]
        DLT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.8++]
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.8++]
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.2+A0.8++]
        DLT       XOR       D1Re0,D1Re0,D1Re0 [A0.2],T4
        DMT       MULT8     D1Re0,D1Re0,D1Re0 [A0.2],T4
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2],T4
        DT        MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.2]
        DMT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.2]
        DT        XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.2]
        DT        MULB8     D1Re0,D1Re0,D1Re0 [A0.2++],T4
        DLT       MULT8     D1Re0,D1Re0,D1Re0 [A0.2++],T4
        DLMT      MULT8     D1Re0,D1Re0,D1Re0 [A0.2++],T4
        DT        XOR       D1Re0,D1Re0,D1Re0 T4,[A0.2++]
        DLT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.2++]
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.2++]
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.2++]
        DLT       XOR       D1Re0,D1Re0,D1Re0 [A0.2--],T4
        DMT       MULT8     D1Re0,D1Re0,D1Re0 [A0.2--],T4
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.2--],T4
        DT        MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.2--]
        DMT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.2--]
        DT        XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.2--]
        D         MULT8     D1.7,D1Re0,D1Re0
        D         XORQ      D1.7,D1Re0,D1Re0,A1.3
        D         XOR       D1.7,D1Re0,A1.7
        DLM       MULB8     D1.7,D1Re0,D1.7
        D         XOR       D1.7,D1Re0,D0.7
        D         XORQ      D1.7,D1Re0,A0FrP,A1.3
        DL        XOR       D1.7,D1Re0,[D1BR.1++]
        DL        XORQ      D1.7,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DLWT      XOR       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T4
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T4
        DLWT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0FrP++],T4
        DWT       MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.7+A0FrP++]
        DMWT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.7+A0FrP++]
        DWT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.7+A0FrP++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T4
        DWT       XOR       D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.2++]
        DLWT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.2++]
        DLMWT     MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.2++]
        DLQWT     XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.7+A0.2++]
        DM        MULT8     D1.7,D1.7,D1Re0
        D         XOR       D1.7,D1.7,A1.3
        DLT       XOR       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T4
        DMT       MULT8     D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T4
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.7++],T4
        DL        XOR       D1.7,D1.7,D1.7
        D         XORQ      D1.7,D1.7,A1.7,A1.3
        D         XORQ      D1.7,D1.7,RD,A1.3
        DT        MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.7++]
        DMT       MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.7++]
        DT        XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.7+A0.7++]
        D         XOR       D1.7,D1.7,A0.2
        D         XORQ      D1.7,D1.7,[D1BR.1++],A1.3
        DT        XOR       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T4
        DLT       MULB8     D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T4
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T4
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.8++],T4
        DLT       XOR       D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.8++]
        DMT       MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.8++]
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.7+A0.8++]
        DT        XOR       D1Re0,D1Re0,D1Re0 [A0.7],T4
        DLT       MULB8     D1Re0,D1Re0,D1Re0 [A0.7],T4
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 [A0.7],T4
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7],T4
        DT        MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.7]
        DLT       MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.7]
        DLMT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.7]
        DT        XOR       D1Re0,D1Re0,D1Re0 [A0.7++],T4
        DLT       MULB8     D1Re0,D1Re0,D1Re0 [A0.7++],T4
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 [A0.7++],T4
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7++],T4
        DLT       XOR       D1Re0,D1Re0,D1Re0 T4,[A0.7++]
        DMT       MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.7++]
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.7++]
        DT        MULT8     D1Re0,D1Re0,D1Re0 [A0.7--],T4
        DMT       MULB8     D1Re0,D1Re0,D1Re0 [A0.7--],T4
        DT        XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.7--],T4
        DT        MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.7--]
        DLT       MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.7--]
        DLMT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.7--]
        D         XOR       D1.7,[D1BR.1],D1Re0
        DLQ       XORQ      D1.7,[D1BR.1],D1Re0,A1.3
        D         MULT8     D1.7,[D1BR.1],D1.7
        D         XOR       D1.7,[D1BR.1],D0Re0
        DLQ       XORQ      D1.7,[D1BR.1],RD,A1.3
        DL        XORQ      D1.7,[D1BR.1],[D1BR.1],A1.3
        D         XORQ      D1.7,[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        DL        MULT8     D1.7,[D1BR.1++],D1Re0
        D         XOR       D1.7,[D1BR.1++],D1.7
        D         XORQ      D1.7,[D1BR.1++],A1.7,A1.3
        D         XORQ      D1.7,[D1BR.1++],RD,A1.3
        D         XOR       D1.7,[D1BR.1++],[D1BR.1]
        D         XOR       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         MULT8     D1.7,[D1BR.1+D1BRI.1++],D1Re0
        D         XORQ      D1.7,[D1BR.1+D1BRI.1++],A1LbP,A1.3
        DL        MULB8     D1.7,[D1BR.1+D1BRI.1++],D1.7
        D         XOR       D1.7,[D1BR.1+D1BRI.1++],D0.7
        D         XORQ      D1.7,[D1BR.1+D1BRI.1++],A0FrP,A1.3
        DL        XOR       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        XORQ      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        DLWT      XOR       D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T4
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T4
        DLWT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0FrP++],T4
        DWT       MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.8+A0FrP++]
        DMWT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.8+A0FrP++]
        DWT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.8+A0FrP++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T4
        DWT       XOR       D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.2++]
        DLWT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.2++]
        DLMWT     MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.2++]
        DLQWT     XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.8+A0.2++]
        DLWT      XOR       D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T4
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T4
        DLWT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.7++],T4
        DWT       MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.7++]
        DMWT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.7++]
        DWT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.8+A0.7++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T4
        DWT       XOR       D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.8++]
        DLWT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.8++]
        DLMWT     MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.8++]
        DLQWT     XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.8+A0.8++]
        DLWT      XOR       D1Re0,D1Re0,D1Re0 [A0.8],T4
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 [A0.8],T4
        DLWT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8],T4
        DWT       MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.8]
        DMWT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.8]
        DWT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.8]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A0.8++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A0.8++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A0.8++],T4
        DWT       XOR       D1Re0,D1Re0,D1Re0 T4,[A0.8++]
        DLWT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.8++]
        DLMWT     MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.8++]
        DLQWT     XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.8++]
        DLWT      XOR       D1Re0,D1Re0,D1Re0 [A0.8--],T4
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 [A0.8--],T4
        DLWT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 [A0.8--],T4
        DWT       MULT8     D1Re0,D1Re0,D1Re0 T4,[A0.8--]
        DMWT      MULB8     D1Re0,D1Re0,D1Re0 T4,[A0.8--]
        DWT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.8--]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1LbP++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1LbP++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.3++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.3++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.7++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.7++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.8++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.8++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1LbP],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1LbP],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1LbP],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1LbP]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1LbP]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1LbP++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1LbP++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1LbP++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1LbP++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1LbP++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1LbP--],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1LbP--],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1LbP--],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1LbP--]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1LbP--]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.3+A1LbP++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.3+A1LbP++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.3++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.3++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.7++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.7++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.8++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.8++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.3],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.3],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.3],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.3]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.3]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.3++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.3++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.3++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.3++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.3++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.3--],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.3--],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.3--],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.3--]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.3--]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.7+A1LbP++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.7+A1LbP++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.3++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.3++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.7++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.7++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.8++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.8++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.7],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.7],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.7],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.7]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.7]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.7++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.7++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.7++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.7++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.7++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.7--],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.7--],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.7--],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.7--]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.7--]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.8+A1LbP++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.8+A1LbP++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.3++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.3++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.7++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.7++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.8++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.8++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.8],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.8],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.8],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.8]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.8]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.8++],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.8++],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.8++],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.8++]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.8++]
        DWT       MULB8     D1Re0,D1Re0,D1Re0 [A1.8--],T4
        DLWT      MULT8     D1Re0,D1Re0,D1Re0 [A1.8--],T4
        DLMWT     MULT8     D1Re0,D1Re0,D1Re0 [A1.8--],T4
        DLWT      XOR       D1Re0,D1Re0,D1Re0 T4,[A1.8--]
        DMWT      MULT8     D1Re0,D1Re0,D1Re0 T4,[A1.8--]
        DL        XOR       [D1BW.1],D1Re0,D1Re0
        DL        XORQ      [D1BW.1],D1Re0,D1Re0,A1.3
        D         MULB8     [D1BW.1],D1Re0,D1.7
        DLM       MULT8     [D1BW.1],D1Re0,D1.7
        D         XORQ      [D1BW.1],D1Re0,D0.7,A1.3
        D         XOR       [D1BW.1],D1Re0,A0.2
        D         XORQ      [D1BW.1],D1Re0,[D1BR.1++],A1.3
        D         XOR       [D1BW.1],D1.7,D1Re0
        DLM       MULB8     [D1BW.1],D1.7,D1Re0
        D         XORQ      [D1BW.1],D1.7,A1.3,A1.3
        DM        MULB8     [D1BW.1],D1.7,D1.7
        D         XOR       [D1BW.1],D1.7,D0Re0
        DLQ       XORQ      [D1BW.1],D1.7,RD,A1.3
        DL        XORQ      [D1BW.1],D1.7,[D1BR.1],A1.3
        D         XORQ      [D1BW.1],D1.7,[D1BR.1+D1BRI.1++],A1.3
        DM        MULT8     [D1BW.1],D1.16,D1.7
        DLT       MULT8     D1Re0,D1Re0,D1Re0 [D1BW.1],T4
        DT        XOR       D1Re0,D1Re0,D1Re0 T4,[D1BR.1]
        DLMT      MULB8     D1Re0,D1Re0,D1Re0 T4,[D1BR.1]
        DL        XOR       [D1BW.1],[D1BR.1],D1Re0
        D         XOR       [D1BW.1],[D1BR.1],A1.3
        DL        MULT8     [D1BW.1],[D1BR.1],D1.7
        D         XORQ      [D1BW.1],[D1BR.1],D0.7,A1.3
        D         XOR       [D1BW.1],[D1BR.1],A0.2
        D         XORQ      [D1BW.1],[D1BR.1],[D1BR.1++],A1.3
        D         XOR       [D1BW.1],[D1BR.1++],D1Re0
        DLQ       XORQ      [D1BW.1],[D1BR.1++],D1Re0,A1.3
        D         MULT8     [D1BW.1],[D1BR.1++],D1.7
        D         XOR       [D1BW.1],[D1BR.1++],D0Re0
        DLQ       XORQ      [D1BW.1],[D1BR.1++],RD,A1.3
        DL        XORQ      [D1BW.1],[D1BR.1++],[D1BR.1],A1.3
        D         XORQ      [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        MULT8     [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        D         XOR       [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        D         XORQ      [D1BW.1],[D1BR.1+D1BRI.1++],A1.7,A1.3
        D         XORQ      [D1BW.1],[D1BR.1+D1BRI.1++],RD,A1.3
        D         XOR       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         XOR       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         MULT8     [D1BW.1++],D1Re0,D1Re0
        D         XORQ      [D1BW.1++],D1Re0,D1Re0,A1.3
        D         XOR       [D1BW.1++],D1Re0,A1.7
        DLM       MULB8     [D1BW.1++],D1Re0,D1.7
        D         XOR       [D1BW.1++],D1Re0,D0.7
        D         XORQ      [D1BW.1++],D1Re0,A0FrP,A1.3
        DL        XOR       [D1BW.1++],D1Re0,[D1BR.1++]
        DL        XORQ      [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DM        MULT8     [D1BW.1++],D1.7,D1Re0
        D         XOR       [D1BW.1++],D1.7,A1.3
        DL        MULT8     [D1BW.1++],D1.7,D1.7
        DLQ       XORQ      [D1BW.1++],D1.7,D1.7,A1.3
        DLQ       XORQ      [D1BW.1++],D1.7,D1.16,A1.3
        D         XORQ      [D1BW.1++],D1.7,[D1BR.1],A1.3
        DL        XOR       [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DM        MULB8     [D1BW.1++],D1.16,D1.7
        DLT       MULB8     D1Re0,D1Re0,D1Re0 [D1BW.1++],T4
        DLQT      XORQ      D1Re0,D1Re0,D1Re0,A1.3 [D1BW.1++],T4
        DMT       MULT8     D1Re0,D1Re0,D1Re0 T4,[D1BR.1++]
        D         MULT8     [D1BW.1++],[D1BR.1],D1Re0
        D         XORQ      [D1BW.1++],[D1BR.1],A1LbP,A1.3
        DL        MULB8     [D1BW.1++],[D1BR.1],D1.7
        D         XOR       [D1BW.1++],[D1BR.1],D0.7
        D         XORQ      [D1BW.1++],[D1BR.1],A0FrP,A1.3
        DL        XOR       [D1BW.1++],[D1BR.1],[D1BR.1++]
        DL        XORQ      [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        DL        XORQ      [D1BW.1++],[D1BR.1++],D1Re0,A1.3
        D         MULB8     [D1BW.1++],[D1BR.1++],D1.7
        DLQ       XORQ      [D1BW.1++],[D1BR.1++],D1.7,A1.3
        DLQ       XORQ      [D1BW.1++],[D1BR.1++],D1.16,A1.3
        D         XORQ      [D1BW.1++],[D1BR.1++],[D1BR.1],A1.3
        DL        XOR       [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        MULB8     [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         XORQ      [D1BW.1++],[D1BR.1+D1BRI.1++],A1.3,A1.3
        D         XORQ      [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7,A1.3
        D         XOR       [D1BW.1++],[D1BR.1+D1BRI.1++],RD
        D         XORQ      [D1BW.1++],[D1BR.1+D1BRI.1++],A0.2,A1.3
        DL        XORQ      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++],A1.3
        D         MULB8     [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        DLM       MULT8     [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        D         XOR       [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DM        MULT8     [D1BW.1+D1BWI.1++],D1Re0,D1.7
        D         XORQ      [D1BW.1+D1BWI.1++],D1Re0,D0Re0,A1.3
        D         XOR       [D1BW.1+D1BWI.1++],D1Re0,A0FrP
        D         XOR       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        D         XORQ      [D1BW.1+D1BWI.1++],D1Re0,A0.7,A1.3
        DM        MULB8     [D1BW.1+D1BWI.1++],D1.7,D1Re0
        D         XORQ      [D1BW.1+D1BWI.1++],D1.7,A1LbP,A1.3
        DL        MULB8     [D1BW.1+D1BWI.1++],D1.7,D1.7
        DL        XORQ      [D1BW.1+D1BWI.1++],D1.7,D1.7,A1.3
        DL        XORQ      [D1BW.1+D1BWI.1++],D1.7,RD,A1.3
        DL        XOR       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        D         XOR       [D1BW.1+D1BWI.1++],D1.7,A0.7
        DLM       MULT8     [D1BW.1+D1BWI.1++],D1.16,D1Re0
        DLT       XOR       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T4
        DLT       XORQ      D1Re0,D1Re0,D1Re0,A1.3 [D1BW.1+D1BWI.1++],T4
        DMT       MULB8     D1Re0,D1Re0,D1Re0 T4,[D1BR.1+D1BRI.1++]
        D         MULB8     [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        D         XOR       [D1BW.1+D1BWI.1++],[D1BR.1],A1LbP
        DL        XOR       [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        D         XORQ      [D1BW.1+D1BWI.1++],[D1BR.1],D0Re0,A1.3
        D         XOR       [D1BW.1+D1BWI.1++],[D1BR.1],A0FrP
        D         XOR       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        D         XORQ      [D1BW.1+D1BWI.1++],[D1BR.1],A0.7,A1.3
        D         XORQ      [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0,A1.3
        D         XOR       [D1BW.1+D1BWI.1++],[D1BR.1++],A1.7
        DL        XORQ      [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7,A1.3
        DL        XORQ      [D1BW.1+D1BWI.1++],[D1BR.1++],RD,A1.3
        DL        XOR       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        D         XOR       [D1BW.1+D1BWI.1++],[D1BR.1++],A0.7
        DL        XOR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         XOR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1.3
        DL        MULT8     [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         XORQ      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D0.7,A1.3
        D         XOR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0.2
        D         XORQ      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++],A1.3
        D         XOR       D0Re0,D0Re0,#0x7fff
        D         XOR       D0.7,D0.7,#-0x8000
        D         XOR       [D0AW.0],[D0AR.0],#-0x1
        D         XOR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x7fff
        D         XOR       D1Re0,D1Re0,#-0x8000
        D         XOR       D1.7,D1.7,#-0x1
        D         XOR       [D1BW.1++],[D1BR.1++],#0x7fff
        D         XOR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#-0x8000
        DHNE      XORQPL    D0Re0,D0Re0,A1.3,A0.2
        DHNE      XORQPL    D0Re0,D0Re0,D1.7,A0.2
        DLLS      XORQLS    D0Re0,D0Re0,[D0AR.0+D0ARI.0++],A0.2
        DLLS      XORQLS    D0Re0,D0.7,D0Re0,A0.2
        D         XORQNV    D0Re0,D0.7,D0.7,A0.2
        D         XORQNV    D0Re0,D0.7,[D0AR.0],A0.2
        DECS      XORQLE    D0Re0,D0.7,A0.2,A0.2
        D         XORQNV    D0Re0,[D0AR.0],A1.7,A0.2
        D         XORQNV    D0Re0,[D0AR.0],A0.2,A0.2
        D         XORQNV    D0Re0,[D0AR.0++],D1.7,A0.2
        D         XORQNV    D0Re0,[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         XORQNV    D0Re0,[D0AR.0+D0ARI.0++],RD,A0.2
        DHNE      XORQPL    D0.7,D0Re0,A1LbP,A0.2
        DHNE      XORQPL    D0.7,D0Re0,D1Re0,A0.2
        DLNE      XORQNE    D0.7,D0Re0,[D0AR.0++],A0.2
        DHNE      XORQPL    D0.7,D0Re0,A0.7,A0.2
        DLNE      XORQNE    D0.7,D0.7,D0.7,A0.2
        DLNE      XORQNE    D0.7,D0.7,[D0AR.0],A0.2
        DHNE      XORQPL    D0.7,D0.7,A0FrP,A0.2
        D         XORQNV    D0.7,[D0AR.0],A1.3,A0.2
        DHCS      XORQVS    D0.7,[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        D         XORQNV    D0.7,[D0AR.0++],A1.7,A0.2
        D         XORQNV    D0.7,[D0AR.0++],A0.2,A0.2
        D         XORQNV    D0.7,[D0AR.0+D0ARI.0++],D1.7,A0.2
        DLLS      XORQLS    D0.16,D0Re0,D0Re0,A0.2
        D         XORQNV    D0.16,D0Re0,D0.7,A0.2
        D         XORQNV    D0.16,D0Re0,[D0AR.0],A0.2
        DECS      XORQLE    D0.16,D0Re0,A0.2,A0.2
        DHNE      XORQPL    D0.16,D0.7,A1.3,A0.2
        DHNE      XORQPL    D0.16,D0.7,D1.7,A0.2
        DLLS      XORQLS    D0.16,D0.7,[D0AR.0+D0ARI.0++],A0.2
        D         XORQNV    D0.16,[D0AR.0],D0Re0,A0.2
        D         XORQNV    D0.16,[D0AR.0],RD,A0.2
        D         XORQNV    D0.16,[D0AR.0++],A1.3,A0.2
        D         XORQNV    D0.16,[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        D         XORQNV    D0.16,[D0AR.0+D0ARI.0++],A1.7,A0.2
        D         XORQNV    D0.16,[D0AR.0+D0ARI.0++],A0.2,A0.2
        DHNE      XORQPL    D1Re0,D1Re0,D1.7,A1.3
        DECS      XORQLE    D1Re0,D1Re0,RD,A1.3
        DHNE      XORQPL    D1Re0,D1Re0,[D1BR.1++],A1.3
        DECS      XORQLE    D1Re0,D1.7,A1LbP,A1.3
        DECS      XORQLE    D1Re0,D1.7,D0Re0,A1.3
        DECS      XORQLE    D1Re0,D1.7,A0.2,A1.3
        D         XORQNV    D1Re0,D1.7,[D1BR.1+D1BRI.1++],A1.3
        D         XORQNV    D1Re0,[D1BR.1],D0.7,A1.3
        D         XORQNV    D1Re0,[D1BR.1++],D1Re0,A1.3
        D         XORQNV    D1Re0,[D1BR.1++],A0FrP,A1.3
        D         XORQNV    D1Re0,[D1BR.1+D1BRI.1++],A1.3,A1.3
        D         XORQNV    D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1],A1.3
        DECS      XORQLE    D1.7,D1Re0,A1.3,A1.3
        DECS      XORQLE    D1.7,D1Re0,D0.7,A1.3
        DHNE      XORQPL    D1.7,D1Re0,[D1BR.1],A1.3
        DHNE      XORQPL    D1.7,D1.7,D1Re0,A1.3
        D         XORQNV    D1.7,D1.7,D1.7,A1.3
        DECS      XORQLE    D1.7,D1.7,A0FrP,A1.3
        DHNE      XORQPL    D1.7,D1.7,[D1BR.1+D1BRI.1++],A1.3
        D         XORQNV    D1.7,[D1BR.1],A1.7,A1.3
        D         XORQNV    D1.7,[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        D         XORQNV    D1.7,[D1BR.1++],D0.7,A1.3
        D         XORQNV    D1.7,[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         XORQNV    D1.7,[D1BR.1+D1BRI.1++],A0FrP,A1.3
        DECS      XORQLE    D1.16,D1Re0,A1LbP,A1.3
        DECS      XORQLE    D1.16,D1Re0,D0Re0,A1.3
        DECS      XORQLE    D1.16,D1Re0,A0.2,A1.3
        D         XORQNV    D1.16,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DHNE      XORQPL    D1.16,D1.7,D1.7,A1.3
        DECS      XORQLE    D1.16,D1.7,RD,A1.3
        DHNE      XORQPL    D1.16,D1.7,[D1BR.1++],A1.3
        D         XORQNV    D1.16,[D1BR.1],A1.3,A1.3
        D         XORQNV    D1.16,[D1BR.1],[D1BR.1],A1.3
        D         XORQNV    D1.16,[D1BR.1++],A1.7,A1.3
        D         XORQNV    D1.16,[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        D         XORQNV    D1.16,[D1BR.1+D1BRI.1++],D0.7,A1.3
        D         XORS      D0Re0,D0Re0,D0Re0
        DLQ       XORSQ     D0Re0,D0Re0,D0Re0,A0.2
        D         XORSQ     D0Re0,D0Re0,A1.3,A0.2
        D         XORSQ     D0Re0,D0Re0,D0.7,A0.2
        D         XORS      D0Re0,D0Re0,D1Re0
        D         XORSQ     D0Re0,D0Re0,RD,A0.2
        DL        XORSQ     D0Re0,D0Re0,[D0AR.0++],A0.2
        D         XORSQ     D0Re0,D0Re0,A0.2,A0.2
        DL        MULST8    D0Re0,D0.7,D0Re0
        D         XORS      D0Re0,D0.7,A1LbP
        DL        XORS      D0Re0,D0.7,D0.7
        DM        MULST8    D0Re0,D0.7,D0.7
        D         XORS      D0Re0,D0.7,RD
        D         XORS      D0Re0,D0.7,[D0AR.0++]
        D         XORS      D0Re0,D0.7,A0FrP
        D         MULST8    D0Re0,[D0AR.0],D0Re0
        DM        MULST8    D0Re0,D0.16,D0Re0
        D         XORS      D0Re0,[D0AR.0],A1.7
        DL        XORSQ     D0Re0,[D0AR.0],D0.7,A0.2
        D         XORS      D0Re0,[D0AR.0],D1.7
        DL        XORSQ     D0Re0,[D0AR.0],RD,A0.2
        DL        XORS      D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         XORSQ     D0Re0,[D0AR.0],A0.7,A0.2
        DL        XORSQ     D0Re0,[D0AR.0++],D0Re0,A0.2
        D         MULSB8    D0Re0,[D0AR.0++],D0.7
        DLQ       XORSQ     D0Re0,[D0AR.0++],D0.7,A0.2
        D         XORSQ     D0Re0,[D0AR.0++],[D0AR.0],A0.2
        D         XORSQ     D0Re0,[D0AR.0++],[D0AR.0++],A0.2
        D         XORS      D0Re0,[D0AR.0++],A0.2
        DL        MULSB8    D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         XORSQ     D0Re0,[D0AR.0+D0ARI.0++],A1.3,A0.2
        D         XORSQ     D0Re0,[D0AR.0+D0ARI.0++],D0.7,A0.2
        D         XORS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DLQ       XORSQ     D0Re0,[D0AR.0+D0ARI.0++],RD,A0.2
        DL        XORSQ     D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++],A0.2
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0FrP++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0FrP++]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.2++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.2++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.2++]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.7++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.7++]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.8++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0FrP+A0.8++]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0FrP],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0FrP]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0FrP]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0FrP++],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0FrP++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0FrP++]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0FrP--],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP--],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0FrP--],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0FrP--]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0FrP--]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0FrP++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.2+A0FrP++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.2+A0FrP++]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0.2++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.2++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.2++]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0.7++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.7++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.7++]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0.2+A0.8++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.8++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.2+A0.8++]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0.2],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0.2],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0.2],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.2]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.2]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0.2++],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0.2++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0.2++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.2++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.2++]
        DWT       XORS      D0Re0,D0Re0,D0Re0 [A0.2--],T4
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0.2--],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A0.2--],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.2--]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.2--]
        D         MULSB8    D0.7,D0Re0,D0Re0
        DM        MULSB8    D0.7,D0Re0,D0Re0
        D         XORS      D0.7,D0Re0,D0.7
        D         XORSQ     D0.7,D0Re0,A1.7,A0.2
        D         XORSQ     D0.7,D0Re0,D1Re0,A0.2
        DL        XORSQ     D0.7,D0Re0,[D0AR.0],A0.2
        D         XORS      D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         XORS      D0.7,D0Re0,A0.7
        DLT       XORS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T4
        DMT       MULST8    D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T4
        DT        MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.7+A0FrP++]
        DLT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.7+A0FrP++]
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.7+A0FrP++]
        DLT       XORS      D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T4
        DMT       MULST8    D0Re0,D0Re0,D0Re0 [A0.7+A0.2++],T4
        DT        MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.2++]
        DLT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.2++]
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.2++]
        D         XORSQ     D0.7,D0.7,D0Re0,A0.2
        D         XORSQ     D0.7,D0.7,A1LbP,A0.2
        DWT       MULST8    D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T4
        D         XORS      D0.7,D0.7,A1.7
        DL        XORSQ     D0.7,D0.7,D0.7,A0.2
        D         XORS      D0.7,D0.7,D1.7
        DL        XORSQ     D0.7,D0.7,RD,A0.2
        DL        XORS      D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.7++]
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.7++]
        D         XORSQ     D0.7,D0.7,A0FrP,A0.2
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T4
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T4
        DLWT      XORS      D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.8++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.7+A0.8++]
        DL        XORS      D0.7,[D0AR.0],D0Re0
        DLM       MULSB8    D0.7,D0.16,D0Re0
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A0.7],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A0.7],T4
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 [A0.7],T4
        DL        MULST8    D0.7,[D0AR.0],D0.7
        DLM       MULST8    D0.7,D0.16,D0.7
        D         XORSQ     D0.7,[D0AR.0],[D0AR.0],A0.2
        D         XORSQ     D0.7,[D0AR.0],[D0AR.0++],A0.2
        DT        MULSB8    D0Re0,D0Re0,D0Re0 T4,[A0.7]
        DLT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.7]
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.7]
        D         XORS      D0.7,[D0AR.0++],D0Re0
        DLQ       XORSQ     D0.7,[D0AR.0++],D0Re0,A0.2
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A0.7++],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A0.7++],T4
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 [A0.7++],T4
        D         XORSQ     D0.7,[D0AR.0++],D0.7,A0.2
        D         XORS      D0.7,[D0AR.0++],[D0AR.0]
        DLQ       XORSQ     D0.7,[D0AR.0++],RD,A0.2
        DL        XORSQ     D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        DLWT      XORS      D0Re0,D0Re0,D0Re0 T4,[A0.7++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.7++]
        D         XORSQ     D0.7,[D0AR.0++],A0.2,A0.2
        DL        MULST8    D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         XORS      D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         XORSQ     D0.7,[D0AR.0+D0ARI.0++],A1.7,A0.2
        D         XORS      D0.7,[D0AR.0+D0ARI.0++],RD
        D         XORS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         XORS      D0.7,[D0AR.0+D0ARI.0++],A0FrP
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A0.7--],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A0.7--],T4
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 [A0.7--],T4
        DLT       XORS      D0Re0,D0Re0,D0Re0 T4,[A0.7--]
        DMT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.7--]
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T4
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T4
        DLT       XORS      D0Re0,D0Re0,D0Re0 T4,[A0.8+A0FrP++]
        DMT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.8+A0FrP++]
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T4
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 [A0.8+A0.2++],T4
        DLT       XORS      D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.2++]
        DMT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.2++]
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T4
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T4
        DLT       XORS      D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.7++]
        DMT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.7++]
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T4
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T4
        DLT       XORS      D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.8++]
        DMT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.8+A0.8++]
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A0.8],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A0.8],T4
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 [A0.8],T4
        DLT       XORS      D0Re0,D0Re0,D0Re0 T4,[A0.8]
        DMT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.8]
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A0.8++],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A0.8++],T4
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 [A0.8++],T4
        DLT       XORS      D0Re0,D0Re0,D0Re0 T4,[A0.8++]
        DMT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.8++]
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A0.8--],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A0.8--],T4
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 [A0.8--],T4
        DLT       XORS      D0Re0,D0Re0,D0Re0 T4,[A0.8--]
        DMT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A0.8--]
        D         MULST8    [D0AW.0],D0Re0,D0Re0
        DM        MULST8    [D0AW.0],D0Re0,D0Re0
        D         XORS      [D0AW.0],D0Re0,A1.7
        DL        XORSQ     [D0AW.0],D0Re0,D0.7,A0.2
        D         XORS      [D0AW.0],D0Re0,D1.7
        DL        XORSQ     [D0AW.0],D0Re0,RD,A0.2
        DL        XORS      [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        D         XORSQ     [D0AW.0],D0Re0,A0.7,A0.2
        DL        XORSQ     [D0AW.0],D0.7,D0Re0,A0.2
        D         XORS      [D0AW.0],D0.7,A1.3
        DL        MULST8    [D0AW.0],D0.7,D0.7
        DLM       MULST8    [D0AW.0],D0.7,D0.7
        D         XORSQ     [D0AW.0],D0.7,[D0AR.0],A0.2
        D         XORSQ     [D0AW.0],D0.7,[D0AR.0++],A0.2
        D         XORS      [D0AW.0],D0.7,A0.2
        DL        MULSB8    [D0AW.0],[D0AR.0],D0Re0
        DLM       MULST8    [D0AW.0],D0.16,D0Re0
        D         MULST8    [D0AW.0],[D0AR.0],D0.7
        DM        MULSB8    [D0AW.0],D0.16,D0.7
        D         XORS      [D0AW.0],[D0AR.0],[D0AR.0]
        DLQ       XORSQ     [D0AW.0],[D0AR.0],RD,A0.2
        DL        XORSQ     [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        D         MULSB8    [D0AW.0],[D0AR.0++],D0Re0
        D         XORS      [D0AW.0],[D0AR.0++],A1LbP
        DL        XORS      [D0AW.0],[D0AR.0++],D0.7
        D         XORSQ     [D0AW.0],[D0AR.0++],D1Re0,A0.2
        DL        XORSQ     [D0AW.0],[D0AR.0++],[D0AR.0],A0.2
        D         XORS      [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         XORS      [D0AW.0],[D0AR.0++],A0.7
        D         XORSQ     [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         XORS      [D0AW.0],[D0AR.0+D0ARI.0++],A1.7
        DL        XORSQ     [D0AW.0],[D0AR.0+D0ARI.0++],D0.7,A0.2
        DL        XORS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        XORS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         XORSQ     [D0AW.0],[D0AR.0+D0ARI.0++],A0FrP,A0.2
        DLT       XORS      D0Re0,D0Re0,D0Re0 [D0AW.0],T4
        DLMT      MULSB8    D0Re0,D0Re0,D0Re0 [D0AW.0],T4
        DT        XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[D0AR.0]
        D         MULSB8    [D0AW.0++],D0Re0,D0Re0
        DM        MULSB8    [D0AW.0++],D0Re0,D0Re0
        D         XORS      [D0AW.0++],D0Re0,D0.7
        D         XORSQ     [D0AW.0++],D0Re0,A1.7,A0.2
        D         XORSQ     [D0AW.0++],D0Re0,D1Re0,A0.2
        DL        XORSQ     [D0AW.0++],D0Re0,[D0AR.0],A0.2
        D         XORS      [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        D         XORS      [D0AW.0++],D0Re0,A0.7
        DLT       XORS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T4
        DLT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1LbP++],T4
        DLMT      MULSB8    D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T4
        DT        MULST8    D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1LbP++]
        DT        XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP+A1LbP++]
        DMT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1LbP++]
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T4
        DMT       MULSB8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.3++],T4
        DT        XORS      D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.3++]
        DLT       MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.3++]
        DLQT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP+A1.3++]
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.3++]
        D         XORSQ     [D0AW.0++],D0.7,D0Re0,A0.2
        D         XORSQ     [D0AW.0++],D0.7,A1LbP,A0.2
        DWT       MULST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T4
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.7++],T4
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T4
        D         MULST8    [D0AW.0++],D0.7,D0.7
        DM        MULSB8    [D0AW.0++],D0.7,D0.7
        D         XORS      [D0AW.0++],D0.7,[D0AR.0]
        DLQ       XORSQ     [D0AW.0++],D0.7,RD,A0.2
        DL        XORSQ     [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++],A0.2
        DLWT      XORS      D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.7++]
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP+A1.7++]
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.7++]
        D         XORSQ     [D0AW.0++],D0.7,A0.7,A0.2
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1LbP+A1.8++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.8++]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP+A1.8++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1LbP+A1.8++]
        DL        XORS      [D0AW.0++],[D0AR.0],D0Re0
        DLM       MULSB8    [D0AW.0++],D0.16,D0Re0
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A1LbP],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A1LbP],T4
        DMT       MULSB8    D0Re0,D0Re0,D0Re0 [A1LbP],T4
        D         XORS      [D0AW.0++],[D0AR.0],D0.7
        D         XORSQ     [D0AW.0++],[D0AR.0],A1.7,A0.2
        D         XORSQ     [D0AW.0++],[D0AR.0],D1Re0,A0.2
        DL        XORSQ     [D0AW.0++],[D0AR.0],[D0AR.0],A0.2
        D         XORS      [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DT        MULST8    D0Re0,D0Re0,D0Re0 T4,[A1LbP]
        DT        XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP]
        DMT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1LbP]
        D         XORS      [D0AW.0++],[D0AR.0],A0.2
        DL        MULSB8    [D0AW.0++],[D0AR.0++],D0Re0
        D         XORSQ     [D0AW.0++],[D0AR.0++],A1.3,A0.2
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1LbP++],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1LbP++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1LbP++],T4
        DL        MULSB8    [D0AW.0++],[D0AR.0++],D0.7
        D         XORS      [D0AW.0++],[D0AR.0++],D1.7
        DL        XORSQ     [D0AW.0++],[D0AR.0++],RD,A0.2
        DL        XORS      [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1LbP++]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1LbP++]
        D         XORSQ     [D0AW.0++],[D0AR.0++],A0.2,A0.2
        DL        MULST8    [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         XORS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        D         XORSQ     [D0AW.0++],[D0AR.0+D0ARI.0++],A1.7,A0.2
        D         XORS      [D0AW.0++],[D0AR.0+D0ARI.0++],RD
        D         XORS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         XORS      [D0AW.0++],[D0AR.0+D0ARI.0++],A0FrP
        DT        MULSB8    D0Re0,D0Re0,D0Re0 [A1LbP--],T4
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [A1LbP--],T4
        DMT       MULSB8    D0Re0,D0Re0,D0Re0 [A1LbP--],T4
        DT        XORS      D0Re0,D0Re0,D0Re0 T4,[A1LbP--]
        DLT       MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1LbP--]
        DLQT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1LbP--]
        DLMT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1LbP--]
        DT        XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [D0AW.0++],T4
        DT        MULSB8    D0Re0,D0Re0,D0Re0 T4,[D0AR.0++]
        DMT       MULSB8    D0Re0,D0Re0,D0Re0 T4,[D0AR.0++]
        DL        MULSB8    [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DLM       MULST8    [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        D         MULST8    [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DM        MULSB8    [D0AW.0+D0AWI.0++],D0Re0,D0.7
        D         XORS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DLQ       XORSQ     [D0AW.0+D0AWI.0++],D0Re0,RD,A0.2
        DL        XORSQ     [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++],A0.2
        D         MULSB8    [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DM        MULSB8    [D0AW.0+D0AWI.0++],D0.7,D0Re0
        D         XORS      [D0AW.0+D0AWI.0++],D0.7,D0.7
        D         XORSQ     [D0AW.0+D0AWI.0++],D0.7,A1.7,A0.2
        D         XORSQ     [D0AW.0+D0AWI.0++],D0.7,D1Re0,A0.2
        DL        XORSQ     [D0AW.0+D0AWI.0++],D0.7,[D0AR.0],A0.2
        D         XORS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        D         XORS      [D0AW.0+D0AWI.0++],D0.7,A0.7
        D         XORSQ     [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0,A0.2
        D         XORSQ     [D0AW.0+D0AWI.0++],[D0AR.0],A1LbP,A0.2
        DL        MULSB8    [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DLM       MULSB8    [D0AW.0+D0AWI.0++],D0.16,D0.7
        DL        XORS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DL        XORS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        D         XORSQ     [D0AW.0+D0AWI.0++],[D0AR.0],A0FrP,A0.2
        DL        XORS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         XORS      [D0AW.0+D0AWI.0++],[D0AR.0++],A1.3
        DL        MULST8    [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        D         XORSQ     [D0AW.0+D0AWI.0++],[D0AR.0++],D1.7,A0.2
        DLQ       XORSQ     [D0AW.0+D0AWI.0++],[D0AR.0++],D0.16,A0.2
        D         XORSQ     [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        D         XORS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLQ       XORSQ     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0,A0.2
        D         MULST8    [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        D         XORS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D1Re0
        D         XORSQ     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],RD,A0.2
        DL        XORSQ     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++],A0.2
        D         XORSQ     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A0.2,A0.2
        DLT       MULST8    D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T4
        DT        XORS      D0Re0,D0Re0,D0Re0 T4,[D0AR.0+D0ARI.0++]
        DLQT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[D0AR.0+D0ARI.0++]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.3+A1LbP++],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.3+A1LbP++]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.3+A1LbP++]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.3+A1LbP++]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.3+A1LbP++]
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.3++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1.3+A1.3++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.3++]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.3+A1.3++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.3++]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.3+A1.7++],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.7++]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.7++]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.3+A1.7++]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.7++]
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.3+A1.8++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1.3+A1.8++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.8++]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.3+A1.8++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.3+A1.8++]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.3],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.3],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.3],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.3]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.3]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.3]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.3]
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1.3++],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.3++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1.3++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.3++]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.3++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.3++]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.3--],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.3--],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.3--],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.3--]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.3--]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.3--]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.3--]
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1LbP++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7+A1LbP++]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7+A1LbP++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7+A1LbP++]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.7+A1.3++],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.3++]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.3++]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7+A1.3++]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.3++]
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7+A1.7++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.7++]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7+A1.7++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.7++]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.8++]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.8++]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7+A1.8++]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7+A1.8++]
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1.7],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1.7],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.7++],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.7++],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.7++],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.7++]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.7++]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7++]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7++]
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1.7--],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.7--],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1.7--],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7--]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.7--]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.7--]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.8+A1LbP++]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.8+A1LbP++]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.8+A1LbP++]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.8+A1LbP++]
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.3++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1.8+A1.3++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.3++]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.8+A1.3++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.3++]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.7++]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.7++]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.8+A1.7++]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.7++]
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.8+A1.8++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.8++]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.8+A1.8++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.8+A1.8++]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.8],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.8],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.8],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.8]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.8]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.8]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.8]
        DLWT      XORS      D0Re0,D0Re0,D0Re0 [A1.8++],T4
        DLWT      XORSQ     D0Re0,D0Re0,D0Re0,A0.2 [A1.8++],T4
        DLMWT     MULSB8    D0Re0,D0Re0,D0Re0 [A1.8++],T4
        DWT       MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.8++]
        DWT       XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.8++]
        DMWT      MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.8++]
        DWT       MULSB8    D0Re0,D0Re0,D0Re0 [A1.8--],T4
        DLWT      MULST8    D0Re0,D0Re0,D0Re0 [A1.8--],T4
        DMWT      MULSB8    D0Re0,D0Re0,D0Re0 [A1.8--],T4
        DWT       XORS      D0Re0,D0Re0,D0Re0 T4,[A1.8--]
        DLWT      MULSB8    D0Re0,D0Re0,D0Re0 T4,[A1.8--]
        DLQWT     XORSQ     D0Re0,D0Re0,D0Re0,A0.2 T4,[A1.8--]
        DLMWT     MULST8    D0Re0,D0Re0,D0Re0 T4,[A1.8--]
        DM        MULST8    D1Re0,D1Re0,D1Re0
        D         XORS      D1Re0,D1Re0,A1.3
        DL        MULST8    D1Re0,D1Re0,D1.7
        DLQ       XORSQ     D1Re0,D1Re0,D1.7,A1.3
        DLQ       XORSQ     D1Re0,D1Re0,D1.16,A1.3
        D         XORSQ     D1Re0,D1Re0,[D1BR.1],A1.3
        DL        XORS      D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DL        MULSB8    D1Re0,D1.7,D1Re0
        DLQ       XORSQ     D1Re0,D1.7,D1Re0,A1.3
        D         MULST8    D1Re0,D1.7,D1.7
        D         XORSQ     D1Re0,D1.7,D1.7,A1.3
        D         XORS      D1Re0,D1.7,RD
        D         XORSQ     D1Re0,D1.7,A0.2,A1.3
        DL        XORSQ     D1Re0,D1.7,[D1BR.1++],A1.3
        DM        MULST8    D1Re0,D1.16,D1Re0
        D         MULSB8    D1Re0,[D1BR.1],D1Re0
        D         XORS      D1Re0,[D1BR.1],A1LbP
        DL        XORS      D1Re0,[D1BR.1],D1.7
        D         XORSQ     D1Re0,[D1BR.1],D0Re0,A1.3
        D         XORS      D1Re0,[D1BR.1],A0FrP
        D         XORS      D1Re0,[D1BR.1],[D1BR.1++]
        D         XORSQ     D1Re0,[D1BR.1],A0.7,A1.3
        D         XORSQ     D1Re0,[D1BR.1++],D1Re0,A1.3
        D         XORS      D1Re0,[D1BR.1++],A1.7
        DL        XORSQ     D1Re0,[D1BR.1++],D1.7,A1.3
        DL        XORSQ     D1Re0,[D1BR.1++],RD,A1.3
        DL        XORS      D1Re0,[D1BR.1++],[D1BR.1]
        D         XORS      D1Re0,[D1BR.1++],A0.7
        DL        XORS      D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         XORS      D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DL        MULST8    D1Re0,[D1BR.1+D1BRI.1++],D1.7
        D         XORSQ     D1Re0,[D1BR.1+D1BRI.1++],D0.7,A1.3
        D         XORS      D1Re0,[D1BR.1+D1BRI.1++],A0.2
        D         XORSQ     D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++],A1.3
        DT        XORS      D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T4
        DLT       MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T4
        DLMT      MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T4
        DLQT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0FrP++],T4
        DLT       XORS      D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0FrP++]
        DMT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0FrP++]
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP+A0FrP++]
        DT        MULST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T4
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.2++],T4
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.2++],T4
        DT        MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.2++]
        DLT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.2++]
        DLMT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.2++]
        DT        XORS      D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T4
        DLT       MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T4
        DLMT      MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T4
        DLQT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.7++],T4
        DLT       XORS      D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.7++]
        DMT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.7++]
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP+A0.7++]
        DT        MULST8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T4
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T4
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0FrP+A0.8++],T4
        DT        MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.8++]
        DLT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.8++]
        DLMT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0FrP+A0.8++]
        DT        XORS      D1Re0,D1Re0,D1Re0 [A0FrP],T4
        DLT       MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP],T4
        DLMT      MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP],T4
        DLQT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0FrP],T4
        DLT       XORS      D1Re0,D1Re0,D1Re0 T4,[A0FrP]
        DMT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0FrP]
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP]
        DT        MULST8    D1Re0,D1Re0,D1Re0 [A0FrP++],T4
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP++],T4
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0FrP++],T4
        DT        MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0FrP++]
        DLT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0FrP++]
        DLMT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0FrP++]
        DT        XORS      D1Re0,D1Re0,D1Re0 [A0FrP--],T4
        DLT       MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP--],T4
        DLMT      MULSB8    D1Re0,D1Re0,D1Re0 [A0FrP--],T4
        DLQT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0FrP--],T4
        DLT       XORS      D1Re0,D1Re0,D1Re0 T4,[A0FrP--]
        DMT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0FrP--]
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0FrP--]
        DT        MULST8    D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T4
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 [A0.2+A0FrP++],T4
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0FrP++],T4
        DT        MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.2+A0FrP++]
        DLT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2+A0FrP++]
        DLMT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2+A0FrP++]
        DT        XORS      D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T4
        DLT       MULSB8    D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T4
        DLMT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.2+A0.2++],T4
        DLQT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.2++],T4
        DLT       XORS      D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.2++]
        DMT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.2++]
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.2+A0.2++]
        DT        MULST8    D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T4
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 [A0.2+A0.7++],T4
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.7++],T4
        DT        MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.7++]
        DLT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.7++]
        DLMT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.7++]
        DT        XORS      D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T4
        DLT       MULSB8    D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T4
        DLMT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.2+A0.8++],T4
        DLQT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2+A0.8++],T4
        DLT       XORS      D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.8++]
        DMT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2+A0.8++]
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.2+A0.8++]
        DT        MULST8    D1Re0,D1Re0,D1Re0 [A0.2],T4
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 [A0.2],T4
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2],T4
        DT        MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.2]
        DLT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2]
        DLMT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2]
        DT        XORS      D1Re0,D1Re0,D1Re0 [A0.2++],T4
        DLT       MULSB8    D1Re0,D1Re0,D1Re0 [A0.2++],T4
        DLMT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.2++],T4
        DLQT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2++],T4
        DLT       XORS      D1Re0,D1Re0,D1Re0 T4,[A0.2++]
        DMT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2++]
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.2++]
        DT        MULST8    D1Re0,D1Re0,D1Re0 [A0.2--],T4
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 [A0.2--],T4
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.2--],T4
        DT        MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.2--]
        DLT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2--]
        DLMT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.2--]
        D         XORS      D1.7,D1Re0,D1Re0
        DLM       MULSB8    D1.7,D1Re0,D1Re0
        D         XORSQ     D1.7,D1Re0,A1.3,A1.3
        DM        MULSB8    D1.7,D1Re0,D1.7
        D         XORS      D1.7,D1Re0,D0Re0
        DLQ       XORSQ     D1.7,D1Re0,RD,A1.3
        DL        XORSQ     D1.7,D1Re0,[D1BR.1],A1.3
        D         XORSQ     D1.7,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DWT       MULST8    D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T4
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T4
        DWT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0FrP++],T4
        DWT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.7+A0FrP++]
        DLWT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.7+A0FrP++]
        DLMWT     MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.7+A0FrP++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0.2++],T4
        DLQWT     XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.2++],T4
        DLWT      XORS      D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.2++]
        DMWT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.2++]
        DLWT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.7+A0.2++]
        DL        MULST8    D1.7,D1.7,D1Re0
        D         XORS      D1.7,D1.7,A1LbP
        DT        MULST8    D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T4
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T4
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.7++],T4
        D         MULSB8    D1.7,D1.7,D1.7
        DLM       MULST8    D1.7,D1.7,D1.7
        D         XORSQ     D1.7,D1.7,D0.7,A1.3
        DT        MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.7++]
        DLT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.7++]
        DLMT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.7++]
        D         XORS      D1.7,D1.7,A0FrP
        D         XORS      D1.7,D1.7,[D1BR.1++]
        D         XORSQ     D1.7,D1.7,A0.7,A1.3
        DLT       XORS      D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T4
        DMT       MULST8    D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T4
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7+A0.8++],T4
        DT        MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.8++]
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.7+A0.8++]
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.7+A0.8++]
        DLM       MULSB8    D1.7,D1.16,D1Re0
        DLT       XORS      D1Re0,D1Re0,D1Re0 [A0.7],T4
        DMT       MULST8    D1Re0,D1Re0,D1Re0 [A0.7],T4
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7],T4
        DT        XORS      D1Re0,D1Re0,D1Re0 T4,[A0.7]
        DLT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.7]
        DLMT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.7]
        DLQT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.7]
        DLT       XORS      D1Re0,D1Re0,D1Re0 [A0.7++],T4
        DMT       MULST8    D1Re0,D1Re0,D1Re0 [A0.7++],T4
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.7++],T4
        DT        MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.7++]
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.7++]
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.7++]
        DT        MULSB8    D1Re0,D1Re0,D1Re0 [A0.7--],T4
        DLT       MULST8    D1Re0,D1Re0,D1Re0 [A0.7--],T4
        DLMT      MULST8    D1Re0,D1Re0,D1Re0 [A0.7--],T4
        DT        XORS      D1Re0,D1Re0,D1Re0 T4,[A0.7--]
        DLT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.7--]
        DLMT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.7--]
        DLQT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.7--]
        D         XORSQ     D1.7,[D1BR.1],D1Re0,A1.3
        D         XORS      D1.7,[D1BR.1],A1.7
        DL        XORSQ     D1.7,[D1BR.1],D1.7,A1.3
        DL        XORSQ     D1.7,[D1BR.1],RD,A1.3
        DL        XORS      D1.7,[D1BR.1],[D1BR.1]
        D         XORS      D1.7,[D1BR.1],A0.7
        DL        XORS      D1.7,[D1BR.1++],D1Re0
        D         XORS      D1.7,[D1BR.1++],A1.3
        DL        MULST8    D1.7,[D1BR.1++],D1.7
        D         XORSQ     D1.7,[D1BR.1++],D0.7,A1.3
        D         XORS      D1.7,[D1BR.1++],A0.2
        D         XORSQ     D1.7,[D1BR.1++],[D1BR.1++],A1.3
        D         XORS      D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DLQ       XORSQ     D1.7,[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         MULST8    D1.7,[D1BR.1+D1BRI.1++],D1.7
        D         XORS      D1.7,[D1BR.1+D1BRI.1++],D0Re0
        DLQ       XORSQ     D1.7,[D1BR.1+D1BRI.1++],RD,A1.3
        DL        XORSQ     D1.7,[D1BR.1+D1BRI.1++],[D1BR.1],A1.3
        D         XORSQ     D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        DWT       MULST8    D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T4
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T4
        DWT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0FrP++],T4
        DWT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.8+A0FrP++]
        DLWT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8+A0FrP++]
        DLMWT     MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8+A0FrP++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A0.8+A0.2++],T4
        DLQWT     XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.2++],T4
        DLWT      XORS      D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.2++]
        DMWT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.2++]
        DLWT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.8+A0.2++]
        DWT       MULST8    D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T4
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T4
        DWT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.7++],T4
        DWT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.7++]
        DLWT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.7++]
        DLMWT     MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.7++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T4
        DLQWT     XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8+A0.8++],T4
        DLWT      XORS      D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.8++]
        DMWT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8+A0.8++]
        DLWT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.8+A0.8++]
        DWT       MULST8    D1Re0,D1Re0,D1Re0 [A0.8],T4
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.8],T4
        DWT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8],T4
        DWT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.8]
        DLWT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8]
        DLMWT     MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A0.8++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.8++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A0.8++],T4
        DLQWT     XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8++],T4
        DLWT      XORS      D1Re0,D1Re0,D1Re0 T4,[A0.8++]
        DMWT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8++]
        DLWT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[A0.8++]
        DWT       MULST8    D1Re0,D1Re0,D1Re0 [A0.8--],T4
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 [A0.8--],T4
        DWT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [A0.8--],T4
        DWT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[A0.8--]
        DLWT      MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8--]
        DLMWT     MULST8    D1Re0,D1Re0,D1Re0 T4,[A0.8--]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1LbP++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1LbP++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.3++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.3++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.3++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.7++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.7++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.8++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1LbP+A1.8++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1LbP],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1LbP]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1LbP]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1LbP++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1LbP++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1LbP++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1LbP--],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP--],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1LbP--],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1LbP--]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1LbP--]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1LbP++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.3+A1LbP++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.3+A1LbP++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1.3++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.3++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.3++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1.7++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.7++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.7++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.3+A1.8++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.8++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.3+A1.8++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.3],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.3],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.3],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.3]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.3]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.3++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.3++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.3++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.3++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.3++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.3--],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.3--],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.3--],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.3--]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.3--]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.7+A1LbP++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.7+A1LbP++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1.3++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.3++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.3++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.7++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.7++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.8++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.7+A1.8++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.7],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.7],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.7],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.7]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.7]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.7++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.7++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.7++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.7++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.7++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.7--],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.7--],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.7--],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.7--]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.7--]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.8+A1LbP++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.8+A1LbP++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1.3++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.3++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.3++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.7++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.7++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.8++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.8+A1.8++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.8],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.8],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.8],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.8]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.8]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.8++],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.8++],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.8++],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.8++]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.8++]
        DWT       XORS      D1Re0,D1Re0,D1Re0 [A1.8--],T4
        DLWT      MULSB8    D1Re0,D1Re0,D1Re0 [A1.8--],T4
        DLMWT     MULSB8    D1Re0,D1Re0,D1Re0 [A1.8--],T4
        DWT       MULST8    D1Re0,D1Re0,D1Re0 T4,[A1.8--]
        DMWT      MULSB8    D1Re0,D1Re0,D1Re0 T4,[A1.8--]
        D         MULSB8    [D1BW.1],D1Re0,D1Re0
        DLM       MULST8    [D1BW.1],D1Re0,D1Re0
        D         XORS      [D1BW.1],D1Re0,D1.7
        DM        MULST8    [D1BW.1],D1Re0,D1.7
        D         XORSQ     [D1BW.1],D1Re0,D0Re0,A1.3
        D         XORS      [D1BW.1],D1Re0,A0FrP
        D         XORS      [D1BW.1],D1Re0,[D1BR.1++]
        D         XORSQ     [D1BW.1],D1Re0,A0.7,A1.3
        DM        MULSB8    [D1BW.1],D1.7,D1Re0
        D         XORSQ     [D1BW.1],D1.7,A1LbP,A1.3
        DL        MULSB8    [D1BW.1],D1.7,D1.7
        DL        XORSQ     [D1BW.1],D1.7,D1.7,A1.3
        DL        XORSQ     [D1BW.1],D1.7,RD,A1.3
        DL        XORS      [D1BW.1],D1.7,[D1BR.1]
        D         XORS      [D1BW.1],D1.7,A0.7
        DLM       MULST8    [D1BW.1],D1.16,D1Re0
        DLT       XORS      D1Re0,D1Re0,D1Re0 [D1BW.1],T4
        DLT       XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [D1BW.1],T4
        DMT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[D1BR.1]
        D         MULSB8    [D1BW.1],[D1BR.1],D1Re0
        D         XORS      [D1BW.1],[D1BR.1],A1LbP
        DL        XORS      [D1BW.1],[D1BR.1],D1.7
        D         XORSQ     [D1BW.1],[D1BR.1],D0Re0,A1.3
        D         XORS      [D1BW.1],[D1BR.1],A0FrP
        D         XORS      [D1BW.1],[D1BR.1],[D1BR.1++]
        D         XORSQ     [D1BW.1],[D1BR.1],A0.7,A1.3
        D         XORSQ     [D1BW.1],[D1BR.1++],D1Re0,A1.3
        D         XORS      [D1BW.1],[D1BR.1++],A1.7
        DL        XORSQ     [D1BW.1],[D1BR.1++],D1.7,A1.3
        DL        XORSQ     [D1BW.1],[D1BR.1++],RD,A1.3
        DL        XORS      [D1BW.1],[D1BR.1++],[D1BR.1]
        D         XORS      [D1BW.1],[D1BR.1++],A0.7
        DL        XORS      [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        D         XORS      [D1BW.1],[D1BR.1+D1BRI.1++],A1.3
        DL        MULST8    [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        D         XORSQ     [D1BW.1],[D1BR.1+D1BRI.1++],D0.7,A1.3
        D         XORS      [D1BW.1],[D1BR.1+D1BRI.1++],A0.2
        D         XORSQ     [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++],A1.3
        D         XORS      [D1BW.1++],D1Re0,D1Re0
        DLM       MULSB8    [D1BW.1++],D1Re0,D1Re0
        D         XORSQ     [D1BW.1++],D1Re0,A1.3,A1.3
        DM        MULSB8    [D1BW.1++],D1Re0,D1.7
        D         XORS      [D1BW.1++],D1Re0,D0Re0
        DLQ       XORSQ     [D1BW.1++],D1Re0,RD,A1.3
        DL        XORSQ     [D1BW.1++],D1Re0,[D1BR.1],A1.3
        D         XORSQ     [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DL        MULST8    [D1BW.1++],D1.7,D1Re0
        D         XORS      [D1BW.1++],D1.7,A1LbP
        DL        XORS      [D1BW.1++],D1.7,D1.7
        D         XORSQ     [D1BW.1++],D1.7,A1.7,A1.3
        D         XORSQ     [D1BW.1++],D1.7,RD,A1.3
        D         XORS      [D1BW.1++],D1.7,[D1BR.1]
        D         XORS      [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DLM       MULSB8    [D1BW.1++],D1.16,D1Re0
        DT        MULST8    D1Re0,D1Re0,D1Re0 [D1BW.1++],T4
        DT        XORSQ     D1Re0,D1Re0,D1Re0,A1.3 [D1BW.1++],T4
        DLT       MULST8    D1Re0,D1Re0,D1Re0 T4,[D1BR.1++]
        D         XORS      [D1BW.1++],[D1BR.1],D1Re0
        DLQ       XORSQ     [D1BW.1++],[D1BR.1],D1Re0,A1.3
        D         MULST8    [D1BW.1++],[D1BR.1],D1.7
        D         XORS      [D1BW.1++],[D1BR.1],D0Re0
        DLQ       XORSQ     [D1BW.1++],[D1BR.1],RD,A1.3
        DL        XORSQ     [D1BW.1++],[D1BR.1],[D1BR.1],A1.3
        D         XORSQ     [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        DL        MULST8    [D1BW.1++],[D1BR.1++],D1Re0
        D         XORS      [D1BW.1++],[D1BR.1++],D1.7
        D         XORSQ     [D1BW.1++],[D1BR.1++],A1.7,A1.3
        D         XORSQ     [D1BW.1++],[D1BR.1++],RD,A1.3
        D         XORS      [D1BW.1++],[D1BR.1++],[D1BR.1]
        D         XORS      [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         MULST8    [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         XORSQ     [D1BW.1++],[D1BR.1+D1BRI.1++],A1LbP,A1.3
        DL        MULSB8    [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        D         XORS      [D1BW.1++],[D1BR.1+D1BRI.1++],D0.7
        D         XORSQ     [D1BW.1++],[D1BR.1+D1BRI.1++],A0FrP,A1.3
        DL        XORS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        XORSQ     [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        DM        MULST8    [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        D         XORS      [D1BW.1+D1BWI.1++],D1Re0,A1.3
        DL        MULST8    [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DLQ       XORSQ     [D1BW.1+D1BWI.1++],D1Re0,D1.7,A1.3
        DLQ       XORSQ     [D1BW.1+D1BWI.1++],D1Re0,D1.16,A1.3
        D         XORSQ     [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1],A1.3
        DL        XORS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DL        MULSB8    [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DLQ       XORSQ     [D1BW.1+D1BWI.1++],D1.7,D1Re0,A1.3
        D         MULST8    [D1BW.1+D1BWI.1++],D1.7,D1.7
        D         XORSQ     [D1BW.1+D1BWI.1++],D1.7,D1.7,A1.3
        D         XORS      [D1BW.1+D1BWI.1++],D1.7,RD
        D         XORSQ     [D1BW.1+D1BWI.1++],D1.7,A0.2,A1.3
        DL        XORSQ     [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++],A1.3
        DM        MULST8    [D1BW.1+D1BWI.1++],D1.16,D1Re0
        DT        MULSB8    D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T4
        DLMT      MULST8    D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T4
        DLT       MULSB8    D1Re0,D1Re0,D1Re0 T4,[D1BR.1+D1BRI.1++]
        DLQT      XORSQ     D1Re0,D1Re0,D1Re0,A1.3 T4,[D1BR.1+D1BRI.1++]
        DL        XORSQ     [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0,A1.3
        D         MULSB8    [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        DLQ       XORSQ     [D1BW.1+D1BWI.1++],[D1BR.1],D1.7,A1.3
        DLQ       XORSQ     [D1BW.1+D1BWI.1++],[D1BR.1],D1.16,A1.3
        D         XORSQ     [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1],A1.3
        DL        XORS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        MULSB8    [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        D         XORSQ     [D1BW.1+D1BWI.1++],[D1BR.1++],A1.3,A1.3
        D         XORSQ     [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7,A1.3
        D         XORS      [D1BW.1+D1BWI.1++],[D1BR.1++],RD
        D         XORSQ     [D1BW.1+D1BWI.1++],[D1BR.1++],A0.2,A1.3
        DL        XORSQ     [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++],A1.3
        D         MULSB8    [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         XORS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A1LbP
        DL        XORS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         XORSQ     [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D0Re0,A1.3
        D         XORS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0FrP
        D         XORS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         XORSQ     [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0.7,A1.3
        D         XORS      D0.7,D0.7,#0x7fff
        D         XORS      [D0AW.0],[D0AR.0],#-0x8000
        D         XORS      [D0AW.0++],[D0AR.0++],#-0x1
        D         XORS      D1Re0,D1Re0,#0x7fff
        D         XORS      D1.7,D1.7,#-0x8000
        D         XORS      [D1BW.1],[D1BR.1],#-0x1
        D         XORS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x7fff
        DHNE      XORSQPL   D0Re0,D0Re0,A1LbP,A0.2
        DECS      XORSQLE   D0Re0,D0Re0,D1Re0,A0.2
        DLNE      XORSQNE   D0Re0,D0Re0,[D0AR.0++],A0.2
        DHNE      XORSQPL   D0Re0,D0Re0,A0.7,A0.2
        DLNE      XORSQNE   D0Re0,D0.7,D0.7,A0.2
        DLLS      XORSQLS   D0Re0,D0.7,[D0AR.0],A0.2
        DHNE      XORSQPL   D0Re0,D0.7,A0FrP,A0.2
        D         XORSQNV   D0Re0,[D0AR.0],A1.3,A0.2
        D         XORSQNV   D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++],A0.2
        D         XORSQNV   D0Re0,[D0AR.0++],A1.7,A0.2
        D         XORSQNV   D0Re0,[D0AR.0++],A0.2,A0.2
        D         XORSQNV   D0Re0,[D0AR.0+D0ARI.0++],D1.7,A0.2
        DLNE      XORSQNE   D0.7,D0Re0,D0Re0,A0.2
        D         XORSQNV   D0.7,D0Re0,D0.7,A0.2
        D         XORSQNV   D0.7,D0Re0,[D0AR.0],A0.2
        DHNE      XORSQPL   D0.7,D0Re0,A0.2,A0.2
        DHNE      XORSQPL   D0.7,D0.7,A1.3,A0.2
        DHNE      XORSQPL   D0.7,D0.7,D1.7,A0.2
        DLNE      XORSQNE   D0.7,D0.7,[D0AR.0+D0ARI.0++],A0.2
        D         XORSQNV   D0.7,[D0AR.0],D0Re0,A0.2
        D         XORSQNV   D0.7,[D0AR.0],RD,A0.2
        D         XORSQNV   D0.7,[D0AR.0++],A1.3,A0.2
        DHCS      XORSQVS   D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++],A0.2
        D         XORSQNV   D0.7,[D0AR.0+D0ARI.0++],A1.7,A0.2
        D         XORSQNV   D0.7,[D0AR.0+D0ARI.0++],A0.2,A0.2
        DLNE      XORSQNE   D0.16,D0Re0,D0.7,A0.2
        DLLS      XORSQLS   D0.16,D0Re0,[D0AR.0],A0.2
        DHNE      XORSQPL   D0.16,D0Re0,A0FrP,A0.2
        DHNE      XORSQPL   D0.16,D0.7,A1LbP,A0.2
        DECS      XORSQLE   D0.16,D0.7,D1Re0,A0.2
        DLNE      XORSQNE   D0.16,D0.7,[D0AR.0++],A0.2
        DHNE      XORSQPL   D0.16,D0.7,A0.7,A0.2
        D         XORSQNV   D0.16,[D0AR.0],D1.7,A0.2
        D         XORSQNV   D0.16,[D0AR.0++],D0Re0,A0.2
        D         XORSQNV   D0.16,[D0AR.0++],RD,A0.2
        D         XORSQNV   D0.16,[D0AR.0+D0ARI.0++],A1.3,A0.2
        D         XORSQNV   D0.16,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++],A0.2
        DECS      XORSQLE   D1Re0,D1Re0,A1.3,A1.3
        DECS      XORSQLE   D1Re0,D1Re0,D0.7,A1.3
        DECS      XORSQLE   D1Re0,D1Re0,[D1BR.1],A1.3
        DECS      XORSQLE   D1Re0,D1.7,D1Re0,A1.3
        D         XORSQNV   D1Re0,D1.7,D1.7,A1.3
        DECS      XORSQLE   D1Re0,D1.7,A0FrP,A1.3
        DHNE      XORSQPL   D1Re0,D1.7,[D1BR.1+D1BRI.1++],A1.3
        D         XORSQNV   D1Re0,[D1BR.1],A1.7,A1.3
        D         XORSQNV   D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++],A1.3
        D         XORSQNV   D1Re0,[D1BR.1++],D0.7,A1.3
        D         XORSQNV   D1Re0,[D1BR.1+D1BRI.1++],D1Re0,A1.3
        D         XORSQNV   D1Re0,[D1BR.1+D1BRI.1++],A0FrP,A1.3
        DECS      XORSQLE   D1.7,D1Re0,A1LbP,A1.3
        DECS      XORSQLE   D1.7,D1Re0,D0Re0,A1.3
        DECS      XORSQLE   D1.7,D1Re0,A0.2,A1.3
        D         XORSQNV   D1.7,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DHNE      XORSQPL   D1.7,D1.7,D1.7,A1.3
        DECS      XORSQLE   D1.7,D1.7,RD,A1.3
        DHNE      XORSQPL   D1.7,D1.7,[D1BR.1++],A1.3
        D         XORSQNV   D1.7,[D1BR.1],A1.3,A1.3
        D         XORSQNV   D1.7,[D1BR.1],[D1BR.1],A1.3
        D         XORSQNV   D1.7,[D1BR.1++],A1.7,A1.3
        D         XORSQNV   D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++],A1.3
        D         XORSQNV   D1.7,[D1BR.1+D1BRI.1++],D0.7,A1.3
        DECS      XORSQLE   D1.16,D1Re0,D1Re0,A1.3
        D         XORSQNV   D1.16,D1Re0,D1.7,A1.3
        DECS      XORSQLE   D1.16,D1Re0,A0FrP,A1.3
        DHNE      XORSQPL   D1.16,D1Re0,[D1BR.1+D1BRI.1++],A1.3
        DECS      XORSQLE   D1.16,D1.7,A1.3,A1.3
        DECS      XORSQLE   D1.16,D1.7,D0.7,A1.3
        DECS      XORSQLE   D1.16,D1.7,[D1BR.1],A1.3
        D         XORSQNV   D1.16,[D1BR.1],D1Re0,A1.3
        D         XORSQNV   D1.16,[D1BR.1],A0FrP,A1.3
        D         XORSQNV   D1.16,[D1BR.1++],A1.3,A1.3
        D         XORSQNV   D1.16,[D1BR.1++],[D1BR.1],A1.3
        D         XORSQNV   D1.16,[D1BR.1+D1BRI.1++],A1.7,A1.3
        D         XORSQNV   D1.16,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++],A1.3
        DLG       LSL       D0Re0,D0Re0,D0Re0
        DLG       LSR       D0Re0,D0Re0,D0Re0
        DLG       ASL       D0Re0,D0Re0,D0Re0
        DLG       ASR       D0Re0,D0Re0,D0Re0
        DLG       LSL       D0Re0,D0Re0,D0.7
        DLG       LSR       D0Re0,D0Re0,D0.7
        DLG       ASL       D0Re0,D0Re0,D0.7
        DLG       ASR       D0Re0,D0Re0,D0.7
        DLG       LSL       D0Re0,D0Re0,[D0AR.0]
        DLG       LSR       D0Re0,D0Re0,[D0AR.0]
        DLG       ASL       D0Re0,D0Re0,[D0AR.0]
        DLG       ASR       D0Re0,D0Re0,[D0AR.0]
        DLG       LSL       D0Re0,D0Re0,[D0AR.0++]
        DLG       LSR       D0Re0,D0Re0,[D0AR.0++]
        DLG       ASL       D0Re0,D0Re0,[D0AR.0++]
        DLG       ASR       D0Re0,D0Re0,[D0AR.0++]
        DLG       LSL       D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DLG       LSR       D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DLG       ASL       D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DLG       ASR       D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DLG       LSL       D0Re0,D0.7,D0Re0
        DLG       LSR       D0Re0,D0.7,D0Re0
        DLG       ASL       D0Re0,D0.7,D0Re0
        DLG       ASR       D0Re0,D0.7,D0Re0
        DLG       LSL       D0Re0,D0.7,D0.7
        DLG       LSR       D0Re0,D0.7,D0.7
        DLG       ASL       D0Re0,D0.7,D0.7
        DLG       ASR       D0Re0,D0.7,D0.7
        DLG       LSL       D0Re0,D0.7,[D0AR.0]
        DLG       LSR       D0Re0,D0.7,[D0AR.0]
        DLG       ASL       D0Re0,D0.7,[D0AR.0]
        DLG       ASR       D0Re0,D0.7,[D0AR.0]
        DLG       LSL       D0Re0,D0.7,[D0AR.0++]
        DLG       LSR       D0Re0,D0.7,[D0AR.0++]
        DLG       ASL       D0Re0,D0.7,[D0AR.0++]
        DLG       ASR       D0Re0,D0.7,[D0AR.0++]
        DLG       LSL       D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DLG       LSR       D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DLG       ASL       D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DLG       ASR       D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DB        LSL       D0Re0,[D0AR.0],D0Re0
        DLB       LSL       D0Re0,AC0.0,D0Re0
        DB        LSR       D0Re0,AC0.0,D0Re0
        DLB       LSR       D0Re0,AC0.0,D0Re0
        DB        ASL       D0Re0,AC0.0,D0Re0
        DLB       ASL       D0Re0,AC0.0,D0Re0
        DB        ASR       D0Re0,AC0.0,D0Re0
        DLB       ASR       D0Re0,AC0.0,D0Re0
        DB        LSL       D0Re0,AC0.0,D0.7
        D         LSR       D0Re0,[D0AR.0],D0.7
        DL        LSR       D0Re0,[D0AR.0],D0.7
        D         ASL       D0Re0,[D0AR.0],D0.7
        DL        ASL       D0Re0,[D0AR.0],D0.7
        D         ASR       D0Re0,[D0AR.0],D0.7
        DL        ASR       D0Re0,[D0AR.0],D0.7
        D         LSL       D0Re0,[D0AR.0],[D0AR.0]
        DL        LSL       D0Re0,[D0AR.0],[D0AR.0]
        D         LSR       D0Re0,AC0.0,[D0AR.0]
        DL        LSR       D0Re0,AC0.0,[D0AR.0]
        D         ASL       D0Re0,AC0.0,[D0AR.0]
        DL        ASL       D0Re0,AC0.0,[D0AR.0]
        D         ASR       D0Re0,AC0.0,[D0AR.0]
        DL        ASR       D0Re0,AC0.0,[D0AR.0]
        D         LSL       D0Re0,AC0.0,[D0AR.0++]
        DLR       LSL       D0Re0,[D0AR.0],[D0AR.0++]
        DR        LSR       D0Re0,[D0AR.0],[D0AR.0++]
        DLR       LSR       D0Re0,[D0AR.0],[D0AR.0++]
        DR        ASL       D0Re0,[D0AR.0],[D0AR.0++]
        DLR       ASL       D0Re0,[D0AR.0],[D0AR.0++]
        DR        ASR       D0Re0,[D0AR.0],[D0AR.0++]
        DLR       ASR       D0Re0,[D0AR.0],[D0AR.0++]
        DR        LSL       D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        DLR       LSL       D0Re0,AC0.0,[D0AR.0+D0ARI.0++]
        DR        LSR       D0Re0,AC0.0,[D0AR.0+D0ARI.0++]
        DLR       LSR       D0Re0,AC0.0,[D0AR.0+D0ARI.0++]
        DR        ASL       D0Re0,AC0.0,[D0AR.0+D0ARI.0++]
        DLR       ASL       D0Re0,AC0.0,[D0AR.0+D0ARI.0++]
        DR        ASR       D0Re0,AC0.0,[D0AR.0+D0ARI.0++]
        DLR       ASR       D0Re0,AC0.0,[D0AR.0+D0ARI.0++]
        DB        LSL       D0Re0,[D0AR.0++],D0Re0
        DB        LSR       D0Re0,[D0AR.0++],D0Re0
        DB        ASL       D0Re0,[D0AR.0++],D0Re0
        DB        ASR       D0Re0,[D0AR.0++],D0Re0
        DB        LSL       D0Re0,[D0AR.0++],D0.7
        DB        LSR       D0Re0,[D0AR.0++],D0.7
        DB        ASL       D0Re0,[D0AR.0++],D0.7
        DB        ASR       D0Re0,[D0AR.0++],D0.7
        DB        LSL       D0Re0,[D0AR.0++],[D0AR.0]
        DB        LSR       D0Re0,[D0AR.0++],[D0AR.0]
        DB        ASL       D0Re0,[D0AR.0++],[D0AR.0]
        DB        ASR       D0Re0,[D0AR.0++],[D0AR.0]
        DB        LSL       D0Re0,[D0AR.0++],[D0AR.0++]
        DB        LSR       D0Re0,[D0AR.0++],[D0AR.0++]
        DB        ASL       D0Re0,[D0AR.0++],[D0AR.0++]
        DB        ASR       D0Re0,[D0AR.0++],[D0AR.0++]
        DB        LSL       D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DB        LSR       D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DB        ASL       D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DB        ASR       D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DB        LSL       D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        DB        LSR       D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        DB        ASL       D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        DB        ASR       D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        DB        LSL       D0Re0,[D0AR.0+D0ARI.0++],D0.7
        DB        LSR       D0Re0,[D0AR.0+D0ARI.0++],D0.7
        DB        ASL       D0Re0,[D0AR.0+D0ARI.0++],D0.7
        DB        ASR       D0Re0,[D0AR.0+D0ARI.0++],D0.7
        DB        LSL       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DB        LSR       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DB        ASL       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DB        ASR       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DB        LSL       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DB        LSR       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DB        ASL       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DB        ASR       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DB        LSL       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DB        LSR       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DB        ASL       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DB        ASR       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DRT       LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DLRT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DRWT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DRT       LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DLRT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DRWT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DRT       ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DLRT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DRWT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DRT       ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DLRT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DRWT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0FrP++],T5
        DRT       LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DLRT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DRWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DRT       LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DLRT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DRWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DRT       ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DLRT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DRWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DRT       ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DLRT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DRWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0FrP++]
        DRT       LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DLRT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DRWT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DRT       LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DLRT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DRWT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DRT       ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DLRT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DRWT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DRT       ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DLRT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DRWT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.7++],T5
        DRT       LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DLRT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DRWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DRT       LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DLRT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DRWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DRT       ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DLRT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DRWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DRT       ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DLRT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DRWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.7++]
        DRT       LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DLRT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DRWT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DRT       LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DLRT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DRWT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DRT       ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DLRT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DRWT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DRT       ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DLRT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DRWT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 [A0FrP+A0.8++],T5
        DRT       LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DLRT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DRWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DRT       LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DLRT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DRWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DRT       ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DLRT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DRWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DRT       ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DLRT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DRWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP+A0.8++]
        DRT       LSL       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DLRT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DRWT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DRT       LSR       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DLRT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DRWT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DRT       ASL       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DLRT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DRWT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DRT       ASR       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DLRT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DRWT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 [A0FrP],T5
        DRT       LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DLRT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DRWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DRT       LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DLRT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DRWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DRT       ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DLRT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DRWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DRT       ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DLRT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DRWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP]
        DRT       LSL       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DLRT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DRWT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DRT       LSR       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DLRT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DRWT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DRT       ASL       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DLRT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DRWT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DRT       ASR       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DLRT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DRWT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 [A0FrP++],T5
        DRT       LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DLRT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DRWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DRT       LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DLRT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DRWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DRT       ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DLRT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DRWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DRT       ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DLRT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DRWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP++]
        DRT       LSL       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DLRT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DRWT      LSL       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DRT       LSR       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DLRT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DRWT      LSR       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DRT       ASL       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DLRT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DRWT      ASL       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DRT       ASR       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DLRT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DRWT      ASR       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 [A0FrP--],T5
        DRT       LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DLRT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DRWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DRT       LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DLRT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DRWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DRT       ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DLRT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DRWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DRT       ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DLRT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DRWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A0FrP--]
        DB        LSL       D0.7,D0Re0,D0Re0
        DB        LSR       D0.7,D0Re0,D0Re0
        DB        ASL       D0.7,D0Re0,D0Re0
        DB        ASR       D0.7,D0Re0,D0Re0
        DB        LSL       D0.7,D0Re0,D0.7
        DB        LSR       D0.7,D0Re0,D0.7
        DB        ASL       D0.7,D0Re0,D0.7
        DB        ASR       D0.7,D0Re0,D0.7
        DB        LSL       D0.7,D0Re0,[D0AR.0]
        DB        LSR       D0.7,D0Re0,[D0AR.0]
        DB        ASL       D0.7,D0Re0,[D0AR.0]
        DB        ASR       D0.7,D0Re0,[D0AR.0]
        DB        LSL       D0.7,D0Re0,[D0AR.0++]
        DB        LSR       D0.7,D0Re0,[D0AR.0++]
        DB        ASL       D0.7,D0Re0,[D0AR.0++]
        DB        ASR       D0.7,D0Re0,[D0AR.0++]
        DB        LSL       D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DB        LSR       D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DB        ASL       D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DB        ASR       D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DRT       LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DLRT      LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DRWT      LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DRT       LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DLRT      LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DRWT      LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DRT       ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DLRT      ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DRWT      ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DRT       ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DLRT      ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DRWT      ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0FrP++],T5
        DRT       LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DLRT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DRWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DRT       LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DLRT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DRWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DRT       ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DLRT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DRWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DRT       ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DLRT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DRWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0FrP++]
        DB        LSL       D0.7,D0.7,D0Re0
        DB        LSR       D0.7,D0.7,D0Re0
        DB        ASL       D0.7,D0.7,D0Re0
        DB        ASR       D0.7,D0.7,D0Re0
        DRT       LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DLRT      LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DRWT      LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DRT       LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DLRT      LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DRWT      LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DRT       ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DLRT      ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DRWT      ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DRT       ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DLRT      ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DRWT      ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0.7++],T5
        DB        LSL       D0.7,D0.7,D0.7
        DB        LSR       D0.7,D0.7,D0.7
        DB        ASL       D0.7,D0.7,D0.7
        DB        ASR       D0.7,D0.7,D0.7
        DB        LSL       D0.7,D0.7,[D0AR.0]
        DB        LSR       D0.7,D0.7,[D0AR.0]
        DB        ASL       D0.7,D0.7,[D0AR.0]
        DB        ASR       D0.7,D0.7,[D0AR.0]
        DB        LSL       D0.7,D0.7,[D0AR.0++]
        DB        LSR       D0.7,D0.7,[D0AR.0++]
        DB        ASL       D0.7,D0.7,[D0AR.0++]
        DB        ASR       D0.7,D0.7,[D0AR.0++]
        DB        LSL       D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DB        LSR       D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DB        ASL       D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DB        ASR       D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DRT       LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DLRT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DRWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DRT       LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DLRT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DRWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DRT       ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DLRT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DRWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DRT       ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DLRT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DRWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.7++]
        DRT       LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DLRT      LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DRWT      LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DRT       LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DLRT      LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DRWT      LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DRT       ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DLRT      ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DRWT      ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DRT       ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DLRT      ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DRWT      ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 [A0.7+A0.8++],T5
        DRT       LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DLRT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DRWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DLRWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DRT       LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DLRT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DRWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DLRWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DRT       ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DLRT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DRWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DLRWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DRT       ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DLRT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DRWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DLRWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A0.7+A0.8++]
        DR        LSL       D0.7,AC0.0,D0Re0
        DLG       LSL       D0.7,[D0AR.0],D0Re0
        DG        LSR       D0.7,[D0AR.0],D0Re0
        DLG       LSR       D0.7,[D0AR.0],D0Re0
        DG        ASL       D0.7,[D0AR.0],D0Re0
        DLG       ASL       D0.7,[D0AR.0],D0Re0
        DG        ASR       D0.7,[D0AR.0],D0Re0
        DLG       ASR       D0.7,[D0AR.0],D0Re0
        DGT       LSL       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DLGT      LSL       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DGWT      LSL       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DLGWT     LSL       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DGT       LSR       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DLGT      LSR       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DGWT      LSR       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DLGWT     LSR       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DGT       ASL       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DLGT      ASL       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DGWT      ASL       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DLGWT     ASL       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DGT       ASR       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DLGT      ASR       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DGWT      ASR       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DLGWT     ASR       D0Re0,D0Re0,D0Re0 [A0.7],T5
        DG        LSL       D0.7,[D0AR.0],D0.7
        DLG       LSL       D0.7,AC0.0,D0.7
        DG        LSR       D0.7,AC0.0,D0.7
        DLG       LSR       D0.7,AC0.0,D0.7
        DG        ASL       D0.7,AC0.0,D0.7
        DLG       ASL       D0.7,AC0.0,D0.7
        DG        ASR       D0.7,AC0.0,D0.7
        DLG       ASR       D0.7,AC0.0,D0.7
        DG        LSL       D0.7,AC0.0,[D0AR.0]
        DLB       LSL       D0.7,[D0AR.0],[D0AR.0]
        DB        LSR       D0.7,[D0AR.0],[D0AR.0]
        DLB       LSR       D0.7,[D0AR.0],[D0AR.0]
        DB        ASL       D0.7,[D0AR.0],[D0AR.0]
        DLB       ASL       D0.7,[D0AR.0],[D0AR.0]
        DB        ASR       D0.7,[D0AR.0],[D0AR.0]
        DLB       ASR       D0.7,[D0AR.0],[D0AR.0]
        DB        LSL       D0.7,[D0AR.0],[D0AR.0++]
        DLB       LSL       D0.7,AC0.0,[D0AR.0++]
        DB        LSR       D0.7,AC0.0,[D0AR.0++]
        DLB       LSR       D0.7,AC0.0,[D0AR.0++]
        DB        ASL       D0.7,AC0.0,[D0AR.0++]
        DLB       ASL       D0.7,AC0.0,[D0AR.0++]
        DB        ASR       D0.7,AC0.0,[D0AR.0++]
        DLB       ASR       D0.7,AC0.0,[D0AR.0++]
        DB        LSL       D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSR       D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DL        LSR       D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASL       D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DL        ASL       D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASR       D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DL        ASR       D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DT        LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DLT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DWT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DLWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DT        LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DLT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DWT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DLWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DT        ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DLT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DWT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DLWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DT        ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DLT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DWT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        DLWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7]
        D         LSL       D0.7,[D0AR.0++],D0Re0
        D         LSR       D0.7,[D0AR.0++],D0Re0
        D         ASL       D0.7,[D0AR.0++],D0Re0
        D         ASR       D0.7,[D0AR.0++],D0Re0
        DT        LSL       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DLT       LSL       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DWT       LSL       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DLWT      LSL       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DT        LSR       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DLT       LSR       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DWT       LSR       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DLWT      LSR       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DT        ASL       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DLT       ASL       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DWT       ASL       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DLWT      ASL       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DT        ASR       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DLT       ASR       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DWT       ASR       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        DLWT      ASR       D0Re0,D0Re0,D0Re0 [A0.7++],T5
        D         LSL       D0.7,[D0AR.0++],D0.7
        D         LSR       D0.7,[D0AR.0++],D0.7
        D         ASL       D0.7,[D0AR.0++],D0.7
        D         ASR       D0.7,[D0AR.0++],D0.7
        D         LSL       D0.7,[D0AR.0++],[D0AR.0]
        D         LSR       D0.7,[D0AR.0++],[D0AR.0]
        D         ASL       D0.7,[D0AR.0++],[D0AR.0]
        D         ASR       D0.7,[D0AR.0++],[D0AR.0]
        D         LSL       D0.7,[D0AR.0++],[D0AR.0++]
        D         LSR       D0.7,[D0AR.0++],[D0AR.0++]
        D         ASL       D0.7,[D0AR.0++],[D0AR.0++]
        D         ASR       D0.7,[D0AR.0++],[D0AR.0++]
        D         LSL       D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSR       D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASL       D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASR       D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DT        LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DLT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DWT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DLWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DT        LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DLT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DWT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DLWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DT        ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DLT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DWT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DLWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DT        ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DLT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DWT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        DLWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7++]
        D         LSL       D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         LSR       D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         ASL       D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         ASR       D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         LSL       D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         LSR       D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         ASL       D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         ASR       D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         LSL       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSR       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASL       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASR       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSL       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSR       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASL       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASR       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSL       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSR       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASL       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASR       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DT        LSL       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DLT       LSL       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DWT       LSL       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DLWT      LSL       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DT        LSR       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DLT       LSR       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DWT       LSR       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DLWT      LSR       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DT        ASL       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DLT       ASL       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DWT       ASL       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DLWT      ASL       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DT        ASR       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DLT       ASR       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DWT       ASR       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DLWT      ASR       D0Re0,D0Re0,D0Re0 [A0.7--],T5
        DT        LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DLT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DWT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DLWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DT        LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DLT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DWT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DLWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DT        ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DLT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DWT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DLWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DT        ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DLT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DWT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DLWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A0.7--]
        DT        LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DLT       LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DWT       LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DLWT      LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DT        LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DLT       LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DWT       LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DLWT      LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DT        ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DLT       ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DWT       ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DLWT      ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DT        ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DLT       ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DWT       ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DLWT      ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T5
        DT        LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DLT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DWT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DLWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DT        LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DLT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DWT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DLWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DT        ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DLT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DWT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DLWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DT        ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DLT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DWT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DLWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0FrP++]
        DT        LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DLT       LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DWT       LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DLWT      LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DT        LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DLT       LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DWT       LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DLWT      LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DT        ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DLT       ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DWT       ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DLWT      ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DT        ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DLT       ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DWT       ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DLWT      ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T5
        DT        LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DLT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DWT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DLWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DT        LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DLT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DWT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DLWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DT        ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DLT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DWT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DLWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DT        ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DLT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DWT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DLWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.7++]
        DT        LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DLT       LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DWT       LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DLWT      LSL       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DT        LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DLT       LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DWT       LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DLWT      LSR       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DT        ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DLT       ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DWT       ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DLWT      ASL       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DT        ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DLT       ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DWT       ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DLWT      ASR       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T5
        DT        LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DLT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DWT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DLWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DT        LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DLT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DWT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DLWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DT        ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DLT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DWT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DLWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DT        ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DLT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DWT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DLWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8+A0.8++]
        DT        LSL       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DLT       LSL       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DWT       LSL       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DLWT      LSL       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DT        LSR       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DLT       LSR       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DWT       LSR       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DLWT      LSR       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DT        ASL       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DLT       ASL       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DWT       ASL       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DLWT      ASL       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DT        ASR       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DLT       ASR       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DWT       ASR       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DLWT      ASR       D0Re0,D0Re0,D0Re0 [A0.8],T5
        DT        LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DLT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DWT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DLWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DT        LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DLT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DWT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DLWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DT        ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DLT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DWT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DLWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DT        ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DLT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DWT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DLWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8]
        DT        LSL       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DLT       LSL       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DWT       LSL       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DLWT      LSL       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DT        LSR       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DLT       LSR       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DWT       LSR       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DLWT      LSR       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DT        ASL       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DLT       ASL       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DWT       ASL       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DLWT      ASL       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DT        ASR       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DLT       ASR       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DWT       ASR       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DLWT      ASR       D0Re0,D0Re0,D0Re0 [A0.8++],T5
        DT        LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DLT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DWT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DLWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DT        LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DLT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DWT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DLWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DT        ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DLT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DWT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DLWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DT        ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DLT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DWT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DLWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8++]
        DT        LSL       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DLT       LSL       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DWT       LSL       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DLWT      LSL       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DT        LSR       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DLT       LSR       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DWT       LSR       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DLWT      LSR       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DT        ASL       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DLT       ASL       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DWT       ASL       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DLWT      ASL       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DT        ASR       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DLT       ASR       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DWT       ASR       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DLWT      ASR       D0Re0,D0Re0,D0Re0 [A0.8--],T5
        DT        LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DLT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DWT       LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DLWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DT        LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DLT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DWT       LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DLWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DT        ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DLT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DWT       ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DLWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DT        ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DLT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DWT       ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        DLWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A0.8--]
        D         LSL       [D0AW.0],D0Re0,D0Re0
        D         LSR       [D0AW.0],D0Re0,D0Re0
        D         ASL       [D0AW.0],D0Re0,D0Re0
        D         ASR       [D0AW.0],D0Re0,D0Re0
        D         LSL       [D0AW.0],D0Re0,D0.7
        D         LSR       [D0AW.0],D0Re0,D0.7
        D         ASL       [D0AW.0],D0Re0,D0.7
        D         ASR       [D0AW.0],D0Re0,D0.7
        D         LSL       [D0AW.0],D0Re0,[D0AR.0]
        D         LSR       [D0AW.0],D0Re0,[D0AR.0]
        D         ASL       [D0AW.0],D0Re0,[D0AR.0]
        D         ASR       [D0AW.0],D0Re0,[D0AR.0]
        D         LSL       [D0AW.0],D0Re0,[D0AR.0++]
        D         LSR       [D0AW.0],D0Re0,[D0AR.0++]
        D         ASL       [D0AW.0],D0Re0,[D0AR.0++]
        D         ASR       [D0AW.0],D0Re0,[D0AR.0++]
        D         LSL       [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        D         LSR       [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        D         ASL       [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        D         ASR       [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        D         LSL       [D0AW.0],D0.7,D0Re0
        D         LSR       [D0AW.0],D0.7,D0Re0
        D         ASL       [D0AW.0],D0.7,D0Re0
        D         ASR       [D0AW.0],D0.7,D0Re0
        D         LSL       [D0AW.0],D0.7,D0.7
        D         LSR       [D0AW.0],D0.7,D0.7
        D         ASL       [D0AW.0],D0.7,D0.7
        D         ASR       [D0AW.0],D0.7,D0.7
        D         LSL       [D0AW.0],D0.7,[D0AR.0]
        D         LSR       [D0AW.0],D0.7,[D0AR.0]
        D         ASL       [D0AW.0],D0.7,[D0AR.0]
        D         ASR       [D0AW.0],D0.7,[D0AR.0]
        D         LSL       [D0AW.0],D0.7,[D0AR.0++]
        D         LSR       [D0AW.0],D0.7,[D0AR.0++]
        D         ASL       [D0AW.0],D0.7,[D0AR.0++]
        D         ASR       [D0AW.0],D0.7,[D0AR.0++]
        D         LSL       [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        D         LSR       [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        D         ASL       [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        D         ASR       [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        D         LSL       [D0AW.0],[D0AR.0],D0Re0
        DL        LSL       [D0AW.0],[D0AR.0],D0Re0
        D         LSR       [D0AW.0],AC0.0,D0Re0
        DL        LSR       [D0AW.0],AC0.0,D0Re0
        D         ASL       [D0AW.0],AC0.0,D0Re0
        DL        ASL       [D0AW.0],AC0.0,D0Re0
        D         ASR       [D0AW.0],AC0.0,D0Re0
        DL        ASR       [D0AW.0],AC0.0,D0Re0
        D         LSL       [D0AW.0],AC0.0,D0.7
        DLR       LSL       [D0AW.0],[D0AR.0],D0.7
        DR        LSR       [D0AW.0],[D0AR.0],D0.7
        DLR       LSR       [D0AW.0],[D0AR.0],D0.7
        DR        ASL       [D0AW.0],[D0AR.0],D0.7
        DLR       ASL       [D0AW.0],[D0AR.0],D0.7
        DR        ASR       [D0AW.0],[D0AR.0],D0.7
        DLR       ASR       [D0AW.0],[D0AR.0],D0.7
        DR        LSL       [D0AW.0],[D0AR.0],[D0AR.0]
        DLR       LSL       [D0AW.0],AC0.0,[D0AR.0]
        DR        LSR       [D0AW.0],AC0.0,[D0AR.0]
        DLR       LSR       [D0AW.0],AC0.0,[D0AR.0]
        DR        ASL       [D0AW.0],AC0.0,[D0AR.0]
        DLR       ASL       [D0AW.0],AC0.0,[D0AR.0]
        DR        ASR       [D0AW.0],AC0.0,[D0AR.0]
        DLR       ASR       [D0AW.0],AC0.0,[D0AR.0]
        DR        LSL       [D0AW.0],AC0.0,[D0AR.0++]
        DLG       LSL       [D0AW.0],[D0AR.0],[D0AR.0++]
        DG        LSR       [D0AW.0],[D0AR.0],[D0AR.0++]
        DLG       LSR       [D0AW.0],[D0AR.0],[D0AR.0++]
        DG        ASL       [D0AW.0],[D0AR.0],[D0AR.0++]
        DLG       ASL       [D0AW.0],[D0AR.0],[D0AR.0++]
        DG        ASR       [D0AW.0],[D0AR.0],[D0AR.0++]
        DLG       ASR       [D0AW.0],[D0AR.0],[D0AR.0++]
        DG        LSL       [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLG       LSL       [D0AW.0],AC0.0,[D0AR.0+D0ARI.0++]
        DG        LSR       [D0AW.0],AC0.0,[D0AR.0+D0ARI.0++]
        DLG       LSR       [D0AW.0],AC0.0,[D0AR.0+D0ARI.0++]
        DG        ASL       [D0AW.0],AC0.0,[D0AR.0+D0ARI.0++]
        DLG       ASL       [D0AW.0],AC0.0,[D0AR.0+D0ARI.0++]
        DG        ASR       [D0AW.0],AC0.0,[D0AR.0+D0ARI.0++]
        DLG       ASR       [D0AW.0],AC0.0,[D0AR.0+D0ARI.0++]
        DLR       LSL       [D0AW.0],[D0AR.0++],D0Re0
        DLR       LSR       [D0AW.0],[D0AR.0++],D0Re0
        DLR       ASL       [D0AW.0],[D0AR.0++],D0Re0
        DLR       ASR       [D0AW.0],[D0AR.0++],D0Re0
        DLR       LSL       [D0AW.0],[D0AR.0++],D0.7
        DLR       LSR       [D0AW.0],[D0AR.0++],D0.7
        DLR       ASL       [D0AW.0],[D0AR.0++],D0.7
        DLR       ASR       [D0AW.0],[D0AR.0++],D0.7
        DLR       LSL       [D0AW.0],[D0AR.0++],[D0AR.0]
        DLR       LSR       [D0AW.0],[D0AR.0++],[D0AR.0]
        DLR       ASL       [D0AW.0],[D0AR.0++],[D0AR.0]
        DLR       ASR       [D0AW.0],[D0AR.0++],[D0AR.0]
        DLR       LSL       [D0AW.0],[D0AR.0++],[D0AR.0++]
        DLR       LSR       [D0AW.0],[D0AR.0++],[D0AR.0++]
        DLR       ASL       [D0AW.0],[D0AR.0++],[D0AR.0++]
        DLR       ASR       [D0AW.0],[D0AR.0++],[D0AR.0++]
        DLR       LSL       [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLR       LSR       [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLR       ASL       [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLR       ASR       [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLR       LSL       [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DLR       LSR       [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DLR       ASL       [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DLR       ASR       [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DLR       LSL       [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DLR       LSR       [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DLR       ASL       [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DLR       ASR       [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DLR       LSL       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLR       LSR       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLR       ASL       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLR       ASR       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLR       LSL       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLR       LSR       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLR       ASL       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLR       ASR       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLR       LSL       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLR       LSR       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLR       ASL       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLR       ASR       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DGT       LSL       D0Re0,AC0.0,D0Re0 [D0AW.0],T5
        DLGT      LSL       D0Re0,AC0.0,D0Re0 [D0AW.0],T5
        DGT       LSR       D0Re0,AC0.0,D0Re0 [D0AW.0],T5
        DLGT      LSR       D0Re0,AC0.0,D0Re0 [D0AW.0],T5
        DGT       ASL       D0Re0,AC0.0,D0Re0 [D0AW.0],T5
        DLGT      ASL       D0Re0,AC0.0,D0Re0 [D0AW.0],T5
        DGT       ASR       D0Re0,AC0.0,D0Re0 [D0AW.0],T5
        DLGT      ASR       D0Re0,AC0.0,D0Re0 [D0AW.0],T5
        DGT       LSL       D0Re0,AC0.0,D0Re0 T5,[D0AR.0]
        DLGT      LSL       D0Re0,AC0.0,D0Re0 T5,[D0AR.0]
        DGT       LSR       D0Re0,AC0.0,D0Re0 T5,[D0AR.0]
        DLGT      LSR       D0Re0,AC0.0,D0Re0 T5,[D0AR.0]
        DGT       ASL       D0Re0,AC0.0,D0Re0 T5,[D0AR.0]
        DLGT      ASL       D0Re0,AC0.0,D0Re0 T5,[D0AR.0]
        DGT       ASR       D0Re0,AC0.0,D0Re0 T5,[D0AR.0]
        DLGT      ASR       D0Re0,AC0.0,D0Re0 T5,[D0AR.0]
        DLR       LSL       [D0AW.0++],D0Re0,D0Re0
        DLR       LSR       [D0AW.0++],D0Re0,D0Re0
        DLR       ASL       [D0AW.0++],D0Re0,D0Re0
        DLR       ASR       [D0AW.0++],D0Re0,D0Re0
        DLR       LSL       [D0AW.0++],D0Re0,D0.7
        DLR       LSR       [D0AW.0++],D0Re0,D0.7
        DLR       ASL       [D0AW.0++],D0Re0,D0.7
        DLR       ASR       [D0AW.0++],D0Re0,D0.7
        DLR       LSL       [D0AW.0++],D0Re0,[D0AR.0]
        DLR       LSR       [D0AW.0++],D0Re0,[D0AR.0]
        DLR       ASL       [D0AW.0++],D0Re0,[D0AR.0]
        DLR       ASR       [D0AW.0++],D0Re0,[D0AR.0]
        DLR       LSL       [D0AW.0++],D0Re0,[D0AR.0++]
        DLR       LSR       [D0AW.0++],D0Re0,[D0AR.0++]
        DLR       ASL       [D0AW.0++],D0Re0,[D0AR.0++]
        DLR       ASR       [D0AW.0++],D0Re0,[D0AR.0++]
        DLR       LSL       [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DLR       LSR       [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DLR       ASL       [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DLR       ASR       [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DGT       LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DLGT      LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DGWT      LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DLGWT     LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DGT       LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DLGT      LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DGWT      LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DLGWT     LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DGT       ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DLGT      ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DGWT      ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DLGWT     ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DGT       ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DLGT      ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DGWT      ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DLGWT     ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1LbP++],T5
        DGT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DLGT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DGWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DLGWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DGT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DLGT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DGWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DLGWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DGT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DLGT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DGWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DLGWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DGT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DLGT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DGWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DLGWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1LbP++]
        DLR       LSL       [D0AW.0++],D0.7,D0Re0
        DLR       LSR       [D0AW.0++],D0.7,D0Re0
        DLR       ASL       [D0AW.0++],D0.7,D0Re0
        DLR       ASR       [D0AW.0++],D0.7,D0Re0
        DGT       LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DLGT      LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DGWT      LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DLGWT     LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DGT       LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DLGT      LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DGWT      LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DLGWT     LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DGT       ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DLGT      ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DGWT      ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DLGWT     ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DGT       ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DLGT      ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DGWT      ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DLGWT     ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.7++],T5
        DLR       LSL       [D0AW.0++],D0.7,D0.7
        DLR       LSR       [D0AW.0++],D0.7,D0.7
        DLR       ASL       [D0AW.0++],D0.7,D0.7
        DLR       ASR       [D0AW.0++],D0.7,D0.7
        DLR       LSL       [D0AW.0++],D0.7,[D0AR.0]
        DLR       LSR       [D0AW.0++],D0.7,[D0AR.0]
        DLR       ASL       [D0AW.0++],D0.7,[D0AR.0]
        DLR       ASR       [D0AW.0++],D0.7,[D0AR.0]
        DLR       LSL       [D0AW.0++],D0.7,[D0AR.0++]
        DLR       LSR       [D0AW.0++],D0.7,[D0AR.0++]
        DLR       ASL       [D0AW.0++],D0.7,[D0AR.0++]
        DLR       ASR       [D0AW.0++],D0.7,[D0AR.0++]
        DLR       LSL       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DLR       LSR       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DLR       ASL       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DLR       ASR       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DGT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DLGT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DGWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DLGWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DGT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DLGT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DGWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DLGWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DGT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DLGT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DGWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DLGWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DGT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DLGT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DGWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DLGWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.7++]
        DGT       LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DLGT      LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DGWT      LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DLGWT     LSL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DGT       LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DLGT      LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DGWT      LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DLGWT     LSR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DGT       ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DLGT      ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DGWT      ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DLGWT     ASL       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DGT       ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DLGT      ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DGWT      ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DLGWT     ASR       D0Re0,AC0.0,D0Re0 [A1LbP+A1.8++],T5
        DGT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DLGT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DGWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DLGWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DGT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DLGT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DGWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DLGWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DGT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DLGT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DGWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DLGWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DGT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DLGT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DGWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DLGWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1LbP+A1.8++]
        DG        LSL       [D0AW.0++],AC0.0,D0Re0
        DLB       LSL       [D0AW.0++],[D0AR.0],D0Re0
        DB        LSR       [D0AW.0++],[D0AR.0],D0Re0
        DLB       LSR       [D0AW.0++],[D0AR.0],D0Re0
        DB        ASL       [D0AW.0++],[D0AR.0],D0Re0
        DLB       ASL       [D0AW.0++],[D0AR.0],D0Re0
        DB        ASR       [D0AW.0++],[D0AR.0],D0Re0
        DLB       ASR       [D0AW.0++],[D0AR.0],D0Re0
        DBT       LSL       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DLBT      LSL       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DBWT      LSL       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DLBWT     LSL       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DBT       LSR       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DLBT      LSR       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DBWT      LSR       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DLBWT     LSR       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DBT       ASL       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DLBT      ASL       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DBWT      ASL       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DLBWT     ASL       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DBT       ASR       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DLBT      ASR       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DBWT      ASR       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DLBWT     ASR       D0Re0,D0Re0,D0Re0 [A1LbP],T5
        DB        LSL       [D0AW.0++],[D0AR.0],D0.7
        DLB       LSL       [D0AW.0++],AC0.0,D0.7
        DB        LSR       [D0AW.0++],AC0.0,D0.7
        DLB       LSR       [D0AW.0++],AC0.0,D0.7
        DB        ASL       [D0AW.0++],AC0.0,D0.7
        DLB       ASL       [D0AW.0++],AC0.0,D0.7
        DB        ASR       [D0AW.0++],AC0.0,D0.7
        DLB       ASR       [D0AW.0++],AC0.0,D0.7
        DB        LSL       [D0AW.0++],AC0.0,[D0AR.0]
        D         LSR       [D0AW.0++],[D0AR.0],[D0AR.0]
        DL        LSR       [D0AW.0++],[D0AR.0],[D0AR.0]
        D         ASL       [D0AW.0++],[D0AR.0],[D0AR.0]
        DL        ASL       [D0AW.0++],[D0AR.0],[D0AR.0]
        D         ASR       [D0AW.0++],[D0AR.0],[D0AR.0]
        DL        ASR       [D0AW.0++],[D0AR.0],[D0AR.0]
        D         LSL       [D0AW.0++],[D0AR.0],[D0AR.0++]
        DL        LSL       [D0AW.0++],[D0AR.0],[D0AR.0++]
        D         LSR       [D0AW.0++],AC0.0,[D0AR.0++]
        DL        LSR       [D0AW.0++],AC0.0,[D0AR.0++]
        D         ASL       [D0AW.0++],AC0.0,[D0AR.0++]
        DL        ASL       [D0AW.0++],AC0.0,[D0AR.0++]
        D         ASR       [D0AW.0++],AC0.0,[D0AR.0++]
        DL        ASR       [D0AW.0++],AC0.0,[D0AR.0++]
        D         LSL       [D0AW.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DLR       LSL       [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DR        LSR       [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLR       LSR       [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DR        ASL       [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLR       ASL       [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DR        ASR       [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLR       ASR       [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DRT       LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DLRT      LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DRWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DLRWT     LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DRT       LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DLRT      LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DRWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DLRWT     LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DRT       ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DLRT      ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DRWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DLRWT     ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DRT       ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DLRT      ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DRWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DLRWT     ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP]
        DG        LSL       [D0AW.0++],[D0AR.0++],D0Re0
        DG        LSR       [D0AW.0++],[D0AR.0++],D0Re0
        DG        ASL       [D0AW.0++],[D0AR.0++],D0Re0
        DG        ASR       [D0AW.0++],[D0AR.0++],D0Re0
        DRT       LSL       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DLRT      LSL       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DRWT      LSL       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DLRWT     LSL       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DRT       LSR       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DLRT      LSR       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DRWT      LSR       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DLRWT     LSR       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DRT       ASL       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DLRT      ASL       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DRWT      ASL       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DLRWT     ASL       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DRT       ASR       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DLRT      ASR       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DRWT      ASR       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DLRWT     ASR       D0Re0,D0Re0,D0Re0 [A1LbP++],T5
        DG        LSL       [D0AW.0++],[D0AR.0++],D0.7
        DG        LSR       [D0AW.0++],[D0AR.0++],D0.7
        DG        ASL       [D0AW.0++],[D0AR.0++],D0.7
        DG        ASR       [D0AW.0++],[D0AR.0++],D0.7
        DG        LSL       [D0AW.0++],[D0AR.0++],[D0AR.0]
        DG        LSR       [D0AW.0++],[D0AR.0++],[D0AR.0]
        DG        ASL       [D0AW.0++],[D0AR.0++],[D0AR.0]
        DG        ASR       [D0AW.0++],[D0AR.0++],[D0AR.0]
        DG        LSL       [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DG        LSR       [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DG        ASL       [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DG        ASR       [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DG        LSL       [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DG        LSR       [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DG        ASL       [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DG        ASR       [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DRT       LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DLRT      LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DRWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DLRWT     LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DRT       LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DLRT      LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DRWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DLRWT     LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DRT       ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DLRT      ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DRWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DLRWT     ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DRT       ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DLRT      ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DRWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DLRWT     ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP++]
        DG        LSL       [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DG        LSR       [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DG        ASL       [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DG        ASR       [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DG        LSL       [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DG        LSR       [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DG        ASL       [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DG        ASR       [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DG        LSL       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DG        LSR       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DG        ASL       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DG        ASR       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DG        LSL       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DG        LSR       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DG        ASL       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DG        ASR       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DG        LSL       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DG        LSR       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DG        ASL       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DG        ASR       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DRT       LSL       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DLRT      LSL       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DRWT      LSL       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DLRWT     LSL       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DRT       LSR       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DLRT      LSR       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DRWT      LSR       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DLRWT     LSR       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DRT       ASL       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DLRT      ASL       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DRWT      ASL       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DLRWT     ASL       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DRT       ASR       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DLRT      ASR       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DRWT      ASR       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DLRWT     ASR       D0Re0,D0Re0,D0Re0 [A1LbP--],T5
        DRT       LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DLRT      LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DRWT      LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DLRWT     LSL       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DRT       LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DLRT      LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DRWT      LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DLRWT     LSR       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DRT       ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DLRT      ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DRWT      ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DLRWT     ASL       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DRT       ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DLRT      ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DRWT      ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DLRWT     ASR       D0Re0,D0Re0,D0Re0 T5,[A1LbP--]
        DRT       LSL       D0Re0,D0Re0,D0Re0 [D0AW.0++],T5
        DLRT      LSL       D0Re0,D0Re0,D0Re0 [D0AW.0++],T5
        DRT       LSR       D0Re0,D0Re0,D0Re0 [D0AW.0++],T5
        DLRT      LSR       D0Re0,D0Re0,D0Re0 [D0AW.0++],T5
        DRT       ASL       D0Re0,D0Re0,D0Re0 [D0AW.0++],T5
        DLRT      ASL       D0Re0,D0Re0,D0Re0 [D0AW.0++],T5
        DRT       ASR       D0Re0,D0Re0,D0Re0 [D0AW.0++],T5
        DLRT      ASR       D0Re0,D0Re0,D0Re0 [D0AW.0++],T5
        DRT       LSL       D0Re0,D0Re0,D0Re0 T5,[D0AR.0++]
        DLRT      LSL       D0Re0,D0Re0,D0Re0 T5,[D0AR.0++]
        DRT       LSR       D0Re0,D0Re0,D0Re0 T5,[D0AR.0++]
        DLRT      LSR       D0Re0,D0Re0,D0Re0 T5,[D0AR.0++]
        DRT       ASL       D0Re0,D0Re0,D0Re0 T5,[D0AR.0++]
        DLRT      ASL       D0Re0,D0Re0,D0Re0 T5,[D0AR.0++]
        DRT       ASR       D0Re0,D0Re0,D0Re0 T5,[D0AR.0++]
        DLRT      ASR       D0Re0,D0Re0,D0Re0 T5,[D0AR.0++]
        DG        LSL       [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DG        LSR       [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DG        ASL       [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DG        ASR       [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DG        LSL       [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DG        LSR       [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DG        ASL       [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DG        ASR       [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DG        LSL       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DG        LSR       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DG        ASL       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DG        ASR       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DG        LSL       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DG        LSR       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DG        ASL       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DG        ASR       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DG        LSL       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DG        LSR       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DG        ASL       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DG        ASR       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DG        LSL       [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DG        LSR       [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DG        ASL       [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DG        ASR       [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DG        LSL       [D0AW.0+D0AWI.0++],D0.7,D0.7
        DG        LSR       [D0AW.0+D0AWI.0++],D0.7,D0.7
        DG        ASL       [D0AW.0+D0AWI.0++],D0.7,D0.7
        DG        ASR       [D0AW.0+D0AWI.0++],D0.7,D0.7
        DG        LSL       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DG        LSR       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DG        ASL       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DG        ASR       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DG        LSL       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DG        LSR       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DG        ASL       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DG        ASR       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DG        LSL       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DG        LSR       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DG        ASL       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DG        ASR       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DR        LSL       [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DLR       LSL       [D0AW.0+D0AWI.0++],AC0.0,D0Re0
        DR        LSR       [D0AW.0+D0AWI.0++],AC0.0,D0Re0
        DLR       LSR       [D0AW.0+D0AWI.0++],AC0.0,D0Re0
        DR        ASL       [D0AW.0+D0AWI.0++],AC0.0,D0Re0
        DLR       ASL       [D0AW.0+D0AWI.0++],AC0.0,D0Re0
        DR        ASR       [D0AW.0+D0AWI.0++],AC0.0,D0Re0
        DLR       ASR       [D0AW.0+D0AWI.0++],AC0.0,D0Re0
        DR        LSL       [D0AW.0+D0AWI.0++],AC0.0,D0.7
        DLG       LSL       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DG        LSR       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DLG       LSR       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DG        ASL       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DLG       ASL       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DG        ASR       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DLG       ASR       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DG        LSL       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DLG       LSL       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0]
        DG        LSR       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0]
        DLG       LSR       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0]
        DG        ASL       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0]
        DLG       ASL       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0]
        DG        ASR       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0]
        DLG       ASR       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0]
        DG        LSL       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0++]
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        DB        LSR       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        DB        ASL       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        DB        ASR       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        DB        LSL       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLB       LSL       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DB        LSR       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DLB       LSR       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DB        ASL       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DLB       ASL       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DB        ASR       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DLB       ASR       [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1LbP++],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1LbP++]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1.7++],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.7++]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.7+A1.8++],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7+A1.8++]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.7],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.7++],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7++]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.7--],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.7--]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1LbP++],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1LbP++]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1.7++],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.7++]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.8+A1.8++],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8+A1.8++]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.8],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.8++],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8++]
        DBT       LSL       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DLBT      LSL       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DBWT      LSL       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DBT       LSR       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DLBT      LSR       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DBWT      LSR       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DBT       ASL       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DLBT      ASL       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DBWT      ASL       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DBT       ASR       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DLBT      ASR       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DBWT      ASR       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 [A1.8--],T5
        DBT       LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DLBT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DBWT      LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DLBWT     LSL       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DBT       LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DLBT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DBWT      LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DLBWT     LSR       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DBT       ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DLBT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DBWT      ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DLBWT     ASL       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DBT       ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DLBT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DBWT      ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DLBWT     ASR       D0Re0,AC0.0,D0Re0 T5,[A1.8--]
        DLB       LSL       D1Re0,D1Re0,D1Re0
        DLB       LSR       D1Re0,D1Re0,D1Re0
        DLB       ASL       D1Re0,D1Re0,D1Re0
        DLB       ASR       D1Re0,D1Re0,D1Re0
        DLB       LSL       D1Re0,D1Re0,D1.7
        DLB       LSR       D1Re0,D1Re0,D1.7
        DLB       ASL       D1Re0,D1Re0,D1.7
        DLB       ASR       D1Re0,D1Re0,D1.7
        DLB       LSL       D1Re0,D1Re0,[D1BR.1]
        DLB       LSR       D1Re0,D1Re0,[D1BR.1]
        DLB       ASL       D1Re0,D1Re0,[D1BR.1]
        DLB       ASR       D1Re0,D1Re0,[D1BR.1]
        DLB       LSL       D1Re0,D1Re0,[D1BR.1++]
        DLB       LSR       D1Re0,D1Re0,[D1BR.1++]
        DLB       ASL       D1Re0,D1Re0,[D1BR.1++]
        DLB       ASR       D1Re0,D1Re0,[D1BR.1++]
        DLB       LSL       D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DLB       LSR       D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DLB       ASL       D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DLB       ASR       D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DLB       LSL       D1Re0,D1.7,D1Re0
        DLB       LSR       D1Re0,D1.7,D1Re0
        DLB       ASL       D1Re0,D1.7,D1Re0
        DLB       ASR       D1Re0,D1.7,D1Re0
        DLB       LSL       D1Re0,D1.7,D1.7
        DLB       LSR       D1Re0,D1.7,D1.7
        DLB       ASL       D1Re0,D1.7,D1.7
        DLB       ASR       D1Re0,D1.7,D1.7
        DLB       LSL       D1Re0,D1.7,[D1BR.1]
        DLB       LSR       D1Re0,D1.7,[D1BR.1]
        DLB       ASL       D1Re0,D1.7,[D1BR.1]
        DLB       ASR       D1Re0,D1.7,[D1BR.1]
        DLB       LSL       D1Re0,D1.7,[D1BR.1++]
        DLB       LSR       D1Re0,D1.7,[D1BR.1++]
        DLB       ASL       D1Re0,D1.7,[D1BR.1++]
        DLB       ASR       D1Re0,D1.7,[D1BR.1++]
        DLB       LSL       D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DLB       LSR       D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DLB       ASL       D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DLB       ASR       D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        D         LSR       D1Re0,AC1.0,D1Re0
        D         ASL       D1Re0,AC1.0,D1Re0
        D         ASR       D1Re0,AC1.0,D1Re0
        D         LSL       D1Re0,AC1.0,D1.7
        DR        LSR       D1Re0,AC1.0,D1.7
        DR        ASL       D1Re0,AC1.0,D1.7
        DR        ASR       D1Re0,AC1.0,D1.7
        DR        LSL       D1Re0,AC1.0,[D1BR.1]
        DG        LSR       D1Re0,AC1.0,[D1BR.1]
        DG        ASL       D1Re0,AC1.0,[D1BR.1]
        DG        ASR       D1Re0,AC1.0,[D1BR.1]
        DG        LSL       D1Re0,AC1.0,[D1BR.1++]
        DB        LSR       D1Re0,AC1.0,[D1BR.1++]
        DB        ASL       D1Re0,AC1.0,[D1BR.1++]
        DB        ASR       D1Re0,AC1.0,[D1BR.1++]
        DB        LSL       D1Re0,AC1.0,[D1BR.1+D1BRI.1++]
        DL        LSR       D1Re0,AC1.0,[D1BR.1+D1BRI.1++]
        DL        ASL       D1Re0,AC1.0,[D1BR.1+D1BRI.1++]
        DL        ASR       D1Re0,AC1.0,[D1BR.1+D1BRI.1++]
        DL        LSL       D1Re0,[D1BR.1],D1Re0
        DL        LSR       D1Re0,[D1BR.1],D1Re0
        DL        ASL       D1Re0,[D1BR.1],D1Re0
        DL        ASR       D1Re0,[D1BR.1],D1Re0
        DL        LSL       D1Re0,[D1BR.1],D1.7
        DL        LSR       D1Re0,[D1BR.1],D1.7
        DL        ASL       D1Re0,[D1BR.1],D1.7
        DL        ASR       D1Re0,[D1BR.1],D1.7
        DL        LSL       D1Re0,[D1BR.1],[D1BR.1]
        DL        LSR       D1Re0,[D1BR.1],[D1BR.1]
        DL        ASL       D1Re0,[D1BR.1],[D1BR.1]
        DL        ASR       D1Re0,[D1BR.1],[D1BR.1]
        DL        LSL       D1Re0,[D1BR.1],[D1BR.1++]
        DL        LSR       D1Re0,[D1BR.1],[D1BR.1++]
        DL        ASL       D1Re0,[D1BR.1],[D1BR.1++]
        DL        ASR       D1Re0,[D1BR.1],[D1BR.1++]
        DL        LSL       D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        LSR       D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        ASL       D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        ASR       D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        LSL       D1Re0,[D1BR.1++],D1Re0
        DL        LSR       D1Re0,[D1BR.1++],D1Re0
        DL        ASL       D1Re0,[D1BR.1++],D1Re0
        DL        ASR       D1Re0,[D1BR.1++],D1Re0
        DL        LSL       D1Re0,[D1BR.1++],D1.7
        DL        LSR       D1Re0,[D1BR.1++],D1.7
        DL        ASL       D1Re0,[D1BR.1++],D1.7
        DL        ASR       D1Re0,[D1BR.1++],D1.7
        DL        LSL       D1Re0,[D1BR.1++],[D1BR.1]
        DL        LSR       D1Re0,[D1BR.1++],[D1BR.1]
        DL        ASL       D1Re0,[D1BR.1++],[D1BR.1]
        DL        ASR       D1Re0,[D1BR.1++],[D1BR.1]
        DL        LSL       D1Re0,[D1BR.1++],[D1BR.1++]
        DL        LSR       D1Re0,[D1BR.1++],[D1BR.1++]
        DL        ASL       D1Re0,[D1BR.1++],[D1BR.1++]
        DL        ASR       D1Re0,[D1BR.1++],[D1BR.1++]
        DL        LSL       D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        LSR       D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        ASL       D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        ASR       D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        LSL       D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DL        LSR       D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DL        ASL       D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DL        ASR       D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DL        LSL       D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DL        LSR       D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DL        ASL       D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DL        ASR       D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DL        LSL       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DL        LSR       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DL        ASL       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DL        ASR       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DL        LSL       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        LSR       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        ASL       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        ASR       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        LSL       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DL        LSR       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DL        ASL       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DL        ASR       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DGT       LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DLGT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DGWT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DGT       LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DLGT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DGWT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DGT       ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DLGT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DGWT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DGT       ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DLGT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DGWT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T5
        DGT       LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DLGT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DGWT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DGT       LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DLGT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DGWT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DGT       ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DLGT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DGWT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DGT       ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DLGT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DGWT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0FrP++]
        DGT       LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DLGT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DGWT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DGT       LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DLGT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DGWT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DGT       ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DLGT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DGWT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DGT       ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DLGT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DGWT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T5
        DGT       LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DLGT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DGWT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DGT       LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DLGT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DGWT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DGT       ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DLGT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DGWT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DGT       ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DLGT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DGWT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.7++]
        DGT       LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DLGT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DGWT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DGT       LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DLGT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DGWT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DGT       ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DLGT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DGWT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DGT       ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DLGT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DGWT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T5
        DGT       LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DLGT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DGWT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DGT       LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DLGT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DGWT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DGT       ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DLGT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DGWT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DGT       ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DLGT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DGWT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP+A0.8++]
        DGT       LSL       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DLGT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DGWT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DGT       LSR       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DLGT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DGWT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DGT       ASL       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DLGT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DGWT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DGT       ASR       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DLGT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DGWT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 [A0FrP],T5
        DGT       LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DLGT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DGWT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DGT       LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DLGT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DGWT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DGT       ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DLGT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DGWT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DGT       ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DLGT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DGWT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP]
        DGT       LSL       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DLGT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DGWT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DGT       LSR       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DLGT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DGWT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DGT       ASL       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DLGT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DGWT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DGT       ASR       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DLGT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DGWT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 [A0FrP++],T5
        DGT       LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DLGT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DGWT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DGT       LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DLGT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DGWT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DGT       ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DLGT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DGWT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DGT       ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DLGT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DGWT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP++]
        DGT       LSL       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DLGT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DGWT      LSL       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DGT       LSR       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DLGT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DGWT      LSR       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DGT       ASL       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DLGT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DGWT      ASL       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DGT       ASR       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DLGT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DGWT      ASR       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 [A0FrP--],T5
        DGT       LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DLGT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DGWT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DGT       LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DLGT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DGWT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DGT       ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DLGT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DGWT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DGT       ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DLGT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DGWT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 T5,[A0FrP--]
        DL        LSL       D1.7,D1Re0,D1Re0
        DL        LSR       D1.7,D1Re0,D1Re0
        DL        ASL       D1.7,D1Re0,D1Re0
        DL        ASR       D1.7,D1Re0,D1Re0
        DL        LSL       D1.7,D1Re0,D1.7
        DL        LSR       D1.7,D1Re0,D1.7
        DL        ASL       D1.7,D1Re0,D1.7
        DL        ASR       D1.7,D1Re0,D1.7
        DL        LSL       D1.7,D1Re0,[D1BR.1]
        DL        LSR       D1.7,D1Re0,[D1BR.1]
        DL        ASL       D1.7,D1Re0,[D1BR.1]
        DL        ASR       D1.7,D1Re0,[D1BR.1]
        DL        LSL       D1.7,D1Re0,[D1BR.1++]
        DL        LSR       D1.7,D1Re0,[D1BR.1++]
        DL        ASL       D1.7,D1Re0,[D1BR.1++]
        DL        ASR       D1.7,D1Re0,[D1BR.1++]
        DL        LSL       D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DL        LSR       D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DL        ASL       D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DL        ASR       D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DGT       LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DLGT      LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DGWT      LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DGT       LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DLGT      LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DGWT      LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DGT       ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DLGT      ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DGWT      ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DGT       ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DLGT      ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DGWT      ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T5
        DGT       LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DLGT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DGWT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DGT       LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DLGT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DGWT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DGT       ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DLGT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DGWT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DGT       ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DLGT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DGWT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0FrP++]
        DL        LSL       D1.7,D1.7,D1Re0
        DL        LSR       D1.7,D1.7,D1Re0
        DL        ASL       D1.7,D1.7,D1Re0
        DL        ASR       D1.7,D1.7,D1Re0
        DGT       LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DLGT      LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DGWT      LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DGT       LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DLGT      LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DGWT      LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DGT       ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DLGT      ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DGWT      ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DGT       ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DLGT      ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DGWT      ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T5
        DL        LSL       D1.7,D1.7,D1.7
        DL        LSR       D1.7,D1.7,D1.7
        DL        ASL       D1.7,D1.7,D1.7
        DL        ASR       D1.7,D1.7,D1.7
        DGT       LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DLGT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DGWT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DGT       LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DLGT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DGWT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DGT       ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DLGT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DGWT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DGT       ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DLGT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DGWT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.7++]
        DL        LSL       D1.7,D1.7,[D1BR.1]
        DL        LSR       D1.7,D1.7,[D1BR.1]
        DL        ASL       D1.7,D1.7,[D1BR.1]
        DL        ASR       D1.7,D1.7,[D1BR.1]
        DL        LSL       D1.7,D1.7,[D1BR.1++]
        DL        LSR       D1.7,D1.7,[D1BR.1++]
        DL        ASL       D1.7,D1.7,[D1BR.1++]
        DL        ASR       D1.7,D1.7,[D1BR.1++]
        DL        LSL       D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DL        LSR       D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DL        ASL       D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DL        ASR       D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DGT       LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DLGT      LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DGWT      LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DGT       LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DLGT      LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DGWT      LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DGT       ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DLGT      ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DGWT      ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DGT       ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DLGT      ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DGWT      ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T5
        DGT       LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DLGT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DGWT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DLGWT     LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DGT       LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DLGT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DGWT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DLGWT     LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DGT       ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DLGT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DGWT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DLGWT     ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DGT       ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DLGT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DGWT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DLGWT     ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7+A0.8++]
        DLR       LSL       D1.7,AC1.0,D1Re0
        DLR       LSR       D1.7,AC1.0,D1Re0
        DLR       ASL       D1.7,AC1.0,D1Re0
        DLR       ASR       D1.7,AC1.0,D1Re0
        DGT       LSL       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DLGT      LSL       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DGWT      LSL       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DLGWT     LSL       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DGT       LSR       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DLGT      LSR       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DGWT      LSR       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DLGWT     LSR       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DGT       ASL       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DLGT      ASL       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DGWT      ASL       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DLGWT     ASL       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DGT       ASR       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DLGT      ASR       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DGWT      ASR       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DLGWT     ASR       D1Re0,AC1.0,D1Re0 [A0.7],T5
        DLG       LSL       D1.7,AC1.0,D1.7
        DLG       LSR       D1.7,AC1.0,D1.7
        DLG       ASL       D1.7,AC1.0,D1.7
        DLG       ASR       D1.7,AC1.0,D1.7
        DBT       LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DLBT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DBWT      LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DLBWT     LSL       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DBT       LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DLBT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DBWT      LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DLBWT     LSR       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DBT       ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DLBT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DBWT      ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DLBWT     ASL       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DBT       ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DLBT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DBWT      ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DLBWT     ASR       D1Re0,D1Re0,D1Re0 T5,[A0.7]
        DLB       LSL       D1.7,AC1.0,[D1BR.1]
        DLB       LSR       D1.7,AC1.0,[D1BR.1]
        DLB       ASL       D1.7,AC1.0,[D1BR.1]
        DLB       ASR       D1.7,AC1.0,[D1BR.1]
        D         LSR       D1.7,AC1.0,[D1BR.1++]
        D         ASL       D1.7,AC1.0,[D1BR.1++]
        D         ASR       D1.7,AC1.0,[D1BR.1++]
        D         LSL       D1.7,AC1.0,[D1BR.1+D1BRI.1++]
        DR        LSR       D1.7,AC1.0,[D1BR.1+D1BRI.1++]
        DR        ASL       D1.7,AC1.0,[D1BR.1+D1BRI.1++]
        DR        ASR       D1.7,AC1.0,[D1BR.1+D1BRI.1++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A0.7++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A0.7++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A0.7--],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A0.7--]
        DR        LSL       D1.7,[D1BR.1],D1Re0
        DR        LSR       D1.7,[D1BR.1],D1Re0
        DR        ASL       D1.7,[D1BR.1],D1Re0
        DR        ASR       D1.7,[D1BR.1],D1Re0
        DR        LSL       D1.7,[D1BR.1],D1.7
        DR        LSR       D1.7,[D1BR.1],D1.7
        DR        ASL       D1.7,[D1BR.1],D1.7
        DR        ASR       D1.7,[D1BR.1],D1.7
        DR        LSL       D1.7,[D1BR.1],[D1BR.1]
        DR        LSR       D1.7,[D1BR.1],[D1BR.1]
        DR        ASL       D1.7,[D1BR.1],[D1BR.1]
        DR        ASR       D1.7,[D1BR.1],[D1BR.1]
        DR        LSL       D1.7,[D1BR.1],[D1BR.1++]
        DR        LSR       D1.7,[D1BR.1],[D1BR.1++]
        DR        ASL       D1.7,[D1BR.1],[D1BR.1++]
        DR        ASR       D1.7,[D1BR.1],[D1BR.1++]
        DR        LSL       D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DR        LSR       D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DR        ASL       D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DR        ASR       D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DR        LSL       D1.7,[D1BR.1++],D1Re0
        DR        LSR       D1.7,[D1BR.1++],D1Re0
        DR        ASL       D1.7,[D1BR.1++],D1Re0
        DR        ASR       D1.7,[D1BR.1++],D1Re0
        DR        LSL       D1.7,[D1BR.1++],D1.7
        DR        LSR       D1.7,[D1BR.1++],D1.7
        DR        ASL       D1.7,[D1BR.1++],D1.7
        DR        ASR       D1.7,[D1BR.1++],D1.7
        DR        LSL       D1.7,[D1BR.1++],[D1BR.1]
        DR        LSR       D1.7,[D1BR.1++],[D1BR.1]
        DR        ASL       D1.7,[D1BR.1++],[D1BR.1]
        DR        ASR       D1.7,[D1BR.1++],[D1BR.1]
        DR        LSL       D1.7,[D1BR.1++],[D1BR.1++]
        DR        LSR       D1.7,[D1BR.1++],[D1BR.1++]
        DR        ASL       D1.7,[D1BR.1++],[D1BR.1++]
        DR        ASR       D1.7,[D1BR.1++],[D1BR.1++]
        DR        LSL       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DR        LSR       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DR        ASL       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DR        ASR       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DR        LSL       D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DR        LSR       D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DR        ASL       D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DR        ASR       D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DR        LSL       D1.7,[D1BR.1+D1BRI.1++],D1.7
        DR        LSR       D1.7,[D1BR.1+D1BRI.1++],D1.7
        DR        ASL       D1.7,[D1BR.1+D1BRI.1++],D1.7
        DR        ASR       D1.7,[D1BR.1+D1BRI.1++],D1.7
        DR        LSL       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DR        LSR       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DR        ASL       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DR        ASR       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DR        LSL       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DR        LSR       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DR        ASL       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DR        ASR       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DR        LSL       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DR        LSR       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DR        ASL       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DR        ASR       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0FrP++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0FrP++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0.7++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.7++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A0.8+A0.8++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8+A0.8++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A0.8],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A0.8++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A0.8--],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A0.8--]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1LbP++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1LbP++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.7++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.7++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1LbP+A1.8++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP+A1.8++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1LbP],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1LbP++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1LbP--],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1LbP--]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1LbP++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1LbP++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1.7++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.7++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.7+A1.8++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7+A1.8++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.7],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.7++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.7--],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.7--]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1LbP++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1LbP++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1.7++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.7++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.8+A1.8++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8+A1.8++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.8],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.8++],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8++]
        DT        LSL       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DLT       LSL       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DWT       LSL       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DLWT      LSL       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DT        LSR       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DLT       LSR       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DWT       LSR       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DLWT      LSR       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DT        ASL       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DLT       ASL       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DWT       ASL       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DLWT      ASL       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DT        ASR       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DLT       ASR       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DWT       ASR       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DLWT      ASR       D1Re0,AC1.0,D1Re0 [A1.8--],T5
        DT        LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DLT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DWT       LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DLWT      LSL       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DT        LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DLT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DWT       LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DLWT      LSR       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DT        ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DLT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DWT       ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DLWT      ASL       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DT        ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DLT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DWT       ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DLWT      ASR       D1Re0,AC1.0,D1Re0 T5,[A1.8--]
        DR        LSL       [D1BW.1],D1Re0,D1Re0
        DR        LSR       [D1BW.1],D1Re0,D1Re0
        DR        ASL       [D1BW.1],D1Re0,D1Re0
        DR        ASR       [D1BW.1],D1Re0,D1Re0
        DR        LSL       [D1BW.1],D1Re0,D1.7
        DR        LSR       [D1BW.1],D1Re0,D1.7
        DR        ASL       [D1BW.1],D1Re0,D1.7
        DR        ASR       [D1BW.1],D1Re0,D1.7
        DR        LSL       [D1BW.1],D1Re0,[D1BR.1]
        DR        LSR       [D1BW.1],D1Re0,[D1BR.1]
        DR        ASL       [D1BW.1],D1Re0,[D1BR.1]
        DR        ASR       [D1BW.1],D1Re0,[D1BR.1]
        DR        LSL       [D1BW.1],D1Re0,[D1BR.1++]
        DR        LSR       [D1BW.1],D1Re0,[D1BR.1++]
        DR        ASL       [D1BW.1],D1Re0,[D1BR.1++]
        DR        ASR       [D1BW.1],D1Re0,[D1BR.1++]
        DR        LSL       [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DR        LSR       [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DR        ASL       [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DR        ASR       [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DR        LSL       [D1BW.1],D1.7,D1Re0
        DR        LSR       [D1BW.1],D1.7,D1Re0
        DR        ASL       [D1BW.1],D1.7,D1Re0
        DR        ASR       [D1BW.1],D1.7,D1Re0
        DR        LSL       [D1BW.1],D1.7,D1.7
        DR        LSR       [D1BW.1],D1.7,D1.7
        DR        ASL       [D1BW.1],D1.7,D1.7
        DR        ASR       [D1BW.1],D1.7,D1.7
        DR        LSL       [D1BW.1],D1.7,[D1BR.1]
        DR        LSR       [D1BW.1],D1.7,[D1BR.1]
        DR        ASL       [D1BW.1],D1.7,[D1BR.1]
        DR        ASR       [D1BW.1],D1.7,[D1BR.1]
        DR        LSL       [D1BW.1],D1.7,[D1BR.1++]
        DR        LSR       [D1BW.1],D1.7,[D1BR.1++]
        DR        ASL       [D1BW.1],D1.7,[D1BR.1++]
        DR        ASR       [D1BW.1],D1.7,[D1BR.1++]
        DR        LSL       [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DR        LSR       [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DR        ASL       [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DR        ASR       [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DR        LSL       [D1BW.1],AC1.0,D1Re0
        DG        LSR       [D1BW.1],AC1.0,D1Re0
        DG        ASL       [D1BW.1],AC1.0,D1Re0
        DG        ASR       [D1BW.1],AC1.0,D1Re0
        DG        LSL       [D1BW.1],AC1.0,D1.7
        DB        LSR       [D1BW.1],AC1.0,D1.7
        DB        ASL       [D1BW.1],AC1.0,D1.7
        DB        ASR       [D1BW.1],AC1.0,D1.7
        DB        LSL       [D1BW.1],AC1.0,[D1BR.1]
        DL        LSR       [D1BW.1],AC1.0,[D1BR.1]
        DL        ASL       [D1BW.1],AC1.0,[D1BR.1]
        DL        ASR       [D1BW.1],AC1.0,[D1BR.1]
        DLR       LSL       [D1BW.1],AC1.0,[D1BR.1++]
        DLR       LSR       [D1BW.1],AC1.0,[D1BR.1++]
        DLR       ASL       [D1BW.1],AC1.0,[D1BR.1++]
        DLR       ASR       [D1BW.1],AC1.0,[D1BR.1++]
        DLG       LSL       [D1BW.1],AC1.0,[D1BR.1+D1BRI.1++]
        DLG       LSR       [D1BW.1],AC1.0,[D1BR.1+D1BRI.1++]
        DLG       ASL       [D1BW.1],AC1.0,[D1BR.1+D1BRI.1++]
        DLG       ASR       [D1BW.1],AC1.0,[D1BR.1+D1BRI.1++]
        DBT       LSL       D1Re0,D1Re0,D1Re0 [D1BW.1],T5
        DLBT      LSL       D1Re0,D1Re0,D1Re0 [D1BW.1],T5
        DBT       LSR       D1Re0,D1Re0,D1Re0 [D1BW.1],T5
        DLBT      LSR       D1Re0,D1Re0,D1Re0 [D1BW.1],T5
        DBT       ASL       D1Re0,D1Re0,D1Re0 [D1BW.1],T5
        DLBT      ASL       D1Re0,D1Re0,D1Re0 [D1BW.1],T5
        DBT       ASR       D1Re0,D1Re0,D1Re0 [D1BW.1],T5
        DLBT      ASR       D1Re0,D1Re0,D1Re0 [D1BW.1],T5
        DBT       LSL       D1Re0,D1Re0,D1Re0 T5,[D1BR.1]
        DLBT      LSL       D1Re0,D1Re0,D1Re0 T5,[D1BR.1]
        DBT       LSR       D1Re0,D1Re0,D1Re0 T5,[D1BR.1]
        DLBT      LSR       D1Re0,D1Re0,D1Re0 T5,[D1BR.1]
        DBT       ASL       D1Re0,D1Re0,D1Re0 T5,[D1BR.1]
        DLBT      ASL       D1Re0,D1Re0,D1Re0 T5,[D1BR.1]
        DBT       ASR       D1Re0,D1Re0,D1Re0 T5,[D1BR.1]
        DLBT      ASR       D1Re0,D1Re0,D1Re0 T5,[D1BR.1]
        DLG       LSL       [D1BW.1],[D1BR.1],D1Re0
        DLG       LSR       [D1BW.1],[D1BR.1],D1Re0
        DLG       ASL       [D1BW.1],[D1BR.1],D1Re0
        DLG       ASR       [D1BW.1],[D1BR.1],D1Re0
        DLG       LSL       [D1BW.1],[D1BR.1],D1.7
        DLG       LSR       [D1BW.1],[D1BR.1],D1.7
        DLG       ASL       [D1BW.1],[D1BR.1],D1.7
        DLG       ASR       [D1BW.1],[D1BR.1],D1.7
        DLG       LSL       [D1BW.1],[D1BR.1],[D1BR.1]
        DLG       LSR       [D1BW.1],[D1BR.1],[D1BR.1]
        DLG       ASL       [D1BW.1],[D1BR.1],[D1BR.1]
        DLG       ASR       [D1BW.1],[D1BR.1],[D1BR.1]
        DLG       LSL       [D1BW.1],[D1BR.1],[D1BR.1++]
        DLG       LSR       [D1BW.1],[D1BR.1],[D1BR.1++]
        DLG       ASL       [D1BW.1],[D1BR.1],[D1BR.1++]
        DLG       ASR       [D1BW.1],[D1BR.1],[D1BR.1++]
        DLG       LSL       [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLG       LSR       [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLG       ASL       [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLG       ASR       [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLG       LSL       [D1BW.1],[D1BR.1++],D1Re0
        DLG       LSR       [D1BW.1],[D1BR.1++],D1Re0
        DLG       ASL       [D1BW.1],[D1BR.1++],D1Re0
        DLG       ASR       [D1BW.1],[D1BR.1++],D1Re0
        DLG       LSL       [D1BW.1],[D1BR.1++],D1.7
        DLG       LSR       [D1BW.1],[D1BR.1++],D1.7
        DLG       ASL       [D1BW.1],[D1BR.1++],D1.7
        DLG       ASR       [D1BW.1],[D1BR.1++],D1.7
        DLG       LSL       [D1BW.1],[D1BR.1++],[D1BR.1]
        DLG       LSR       [D1BW.1],[D1BR.1++],[D1BR.1]
        DLG       ASL       [D1BW.1],[D1BR.1++],[D1BR.1]
        DLG       ASR       [D1BW.1],[D1BR.1++],[D1BR.1]
        DLG       LSL       [D1BW.1],[D1BR.1++],[D1BR.1++]
        DLG       LSR       [D1BW.1],[D1BR.1++],[D1BR.1++]
        DLG       ASL       [D1BW.1],[D1BR.1++],[D1BR.1++]
        DLG       ASR       [D1BW.1],[D1BR.1++],[D1BR.1++]
        DLG       LSL       [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLG       LSR       [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLG       ASL       [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLG       ASR       [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLG       LSL       [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DLG       LSR       [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DLG       ASL       [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DLG       ASR       [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DLG       LSL       [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DLG       LSR       [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DLG       ASL       [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DLG       ASR       [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DLG       LSL       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLG       LSR       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLG       ASL       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLG       ASR       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLG       LSL       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLG       LSR       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLG       ASL       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLG       ASR       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLG       LSL       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLG       LSR       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLG       ASL       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLG       ASR       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLG       LSL       [D1BW.1++],D1Re0,D1Re0
        DLG       LSR       [D1BW.1++],D1Re0,D1Re0
        DLG       ASL       [D1BW.1++],D1Re0,D1Re0
        DLG       ASR       [D1BW.1++],D1Re0,D1Re0
        DLG       LSL       [D1BW.1++],D1Re0,D1.7
        DLG       LSR       [D1BW.1++],D1Re0,D1.7
        DLG       ASL       [D1BW.1++],D1Re0,D1.7
        DLG       ASR       [D1BW.1++],D1Re0,D1.7
        DLG       LSL       [D1BW.1++],D1Re0,[D1BR.1]
        DLG       LSR       [D1BW.1++],D1Re0,[D1BR.1]
        DLG       ASL       [D1BW.1++],D1Re0,[D1BR.1]
        DLG       ASR       [D1BW.1++],D1Re0,[D1BR.1]
        DLG       LSL       [D1BW.1++],D1Re0,[D1BR.1++]
        DLG       LSR       [D1BW.1++],D1Re0,[D1BR.1++]
        DLG       ASL       [D1BW.1++],D1Re0,[D1BR.1++]
        DLG       ASR       [D1BW.1++],D1Re0,[D1BR.1++]
        DLG       LSL       [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DLG       LSR       [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DLG       ASL       [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DLG       ASR       [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DLG       LSL       [D1BW.1++],D1.7,D1Re0
        DLG       LSR       [D1BW.1++],D1.7,D1Re0
        DLG       ASL       [D1BW.1++],D1.7,D1Re0
        DLG       ASR       [D1BW.1++],D1.7,D1Re0
        DLG       LSL       [D1BW.1++],D1.7,D1.7
        DLG       LSR       [D1BW.1++],D1.7,D1.7
        DLG       ASL       [D1BW.1++],D1.7,D1.7
        DLG       ASR       [D1BW.1++],D1.7,D1.7
        DLG       LSL       [D1BW.1++],D1.7,[D1BR.1]
        DLG       LSR       [D1BW.1++],D1.7,[D1BR.1]
        DLG       ASL       [D1BW.1++],D1.7,[D1BR.1]
        DLG       ASR       [D1BW.1++],D1.7,[D1BR.1]
        DLG       LSL       [D1BW.1++],D1.7,[D1BR.1++]
        DLG       LSR       [D1BW.1++],D1.7,[D1BR.1++]
        DLG       ASL       [D1BW.1++],D1.7,[D1BR.1++]
        DLG       ASR       [D1BW.1++],D1.7,[D1BR.1++]
        DLG       LSL       [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DLG       LSR       [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DLG       ASL       [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DLG       ASR       [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DLB       LSL       [D1BW.1++],AC1.0,D1Re0
        DLB       LSR       [D1BW.1++],AC1.0,D1Re0
        DLB       ASL       [D1BW.1++],AC1.0,D1Re0
        DLB       ASR       [D1BW.1++],AC1.0,D1Re0
        D         LSR       [D1BW.1++],AC1.0,D1.7
        D         ASL       [D1BW.1++],AC1.0,D1.7
        D         ASR       [D1BW.1++],AC1.0,D1.7
        D         LSL       [D1BW.1++],AC1.0,[D1BR.1]
        DR        LSR       [D1BW.1++],AC1.0,[D1BR.1]
        DR        ASL       [D1BW.1++],AC1.0,[D1BR.1]
        DR        ASR       [D1BW.1++],AC1.0,[D1BR.1]
        DR        LSL       [D1BW.1++],AC1.0,[D1BR.1++]
        DG        LSR       [D1BW.1++],AC1.0,[D1BR.1++]
        DG        ASL       [D1BW.1++],AC1.0,[D1BR.1++]
        DG        ASR       [D1BW.1++],AC1.0,[D1BR.1++]
        DG        LSL       [D1BW.1++],AC1.0,[D1BR.1+D1BRI.1++]
        DB        LSR       [D1BW.1++],AC1.0,[D1BR.1+D1BRI.1++]
        DB        ASL       [D1BW.1++],AC1.0,[D1BR.1+D1BRI.1++]
        DB        ASR       [D1BW.1++],AC1.0,[D1BR.1+D1BRI.1++]
        DRT       LSL       D1Re0,AC1.0,D1Re0 [D1BW.1++],T5
        DLRT      LSL       D1Re0,AC1.0,D1Re0 [D1BW.1++],T5
        DRT       LSR       D1Re0,AC1.0,D1Re0 [D1BW.1++],T5
        DLRT      LSR       D1Re0,AC1.0,D1Re0 [D1BW.1++],T5
        DRT       ASL       D1Re0,AC1.0,D1Re0 [D1BW.1++],T5
        DLRT      ASL       D1Re0,AC1.0,D1Re0 [D1BW.1++],T5
        DRT       ASR       D1Re0,AC1.0,D1Re0 [D1BW.1++],T5
        DLRT      ASR       D1Re0,AC1.0,D1Re0 [D1BW.1++],T5
        DRT       LSL       D1Re0,AC1.0,D1Re0 T5,[D1BR.1++]
        DLRT      LSL       D1Re0,AC1.0,D1Re0 T5,[D1BR.1++]
        DRT       LSR       D1Re0,AC1.0,D1Re0 T5,[D1BR.1++]
        DLRT      LSR       D1Re0,AC1.0,D1Re0 T5,[D1BR.1++]
        DRT       ASL       D1Re0,AC1.0,D1Re0 T5,[D1BR.1++]
        DLRT      ASL       D1Re0,AC1.0,D1Re0 T5,[D1BR.1++]
        DRT       ASR       D1Re0,AC1.0,D1Re0 T5,[D1BR.1++]
        DLRT      ASR       D1Re0,AC1.0,D1Re0 T5,[D1BR.1++]
        DB        LSL       [D1BW.1++],[D1BR.1],D1Re0
        DB        LSR       [D1BW.1++],[D1BR.1],D1Re0
        DB        ASL       [D1BW.1++],[D1BR.1],D1Re0
        DB        ASR       [D1BW.1++],[D1BR.1],D1Re0
        DB        LSL       [D1BW.1++],[D1BR.1],D1.7
        DB        LSR       [D1BW.1++],[D1BR.1],D1.7
        DB        ASL       [D1BW.1++],[D1BR.1],D1.7
        DB        ASR       [D1BW.1++],[D1BR.1],D1.7
        DB        LSL       [D1BW.1++],[D1BR.1],[D1BR.1]
        DB        LSR       [D1BW.1++],[D1BR.1],[D1BR.1]
        DB        ASL       [D1BW.1++],[D1BR.1],[D1BR.1]
        DB        ASR       [D1BW.1++],[D1BR.1],[D1BR.1]
        DB        LSL       [D1BW.1++],[D1BR.1],[D1BR.1++]
        DB        LSR       [D1BW.1++],[D1BR.1],[D1BR.1++]
        DB        ASL       [D1BW.1++],[D1BR.1],[D1BR.1++]
        DB        ASR       [D1BW.1++],[D1BR.1],[D1BR.1++]
        DB        LSL       [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DB        LSR       [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DB        ASL       [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DB        ASR       [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DB        LSL       [D1BW.1++],[D1BR.1++],D1Re0
        DB        LSR       [D1BW.1++],[D1BR.1++],D1Re0
        DB        ASL       [D1BW.1++],[D1BR.1++],D1Re0
        DB        ASR       [D1BW.1++],[D1BR.1++],D1Re0
        DB        LSL       [D1BW.1++],[D1BR.1++],D1.7
        DB        LSR       [D1BW.1++],[D1BR.1++],D1.7
        DB        ASL       [D1BW.1++],[D1BR.1++],D1.7
        DB        ASR       [D1BW.1++],[D1BR.1++],D1.7
        DB        LSL       [D1BW.1++],[D1BR.1++],[D1BR.1]
        DB        LSR       [D1BW.1++],[D1BR.1++],[D1BR.1]
        DB        ASL       [D1BW.1++],[D1BR.1++],[D1BR.1]
        DB        ASR       [D1BW.1++],[D1BR.1++],[D1BR.1]
        DB        LSL       [D1BW.1++],[D1BR.1++],[D1BR.1++]
        DB        LSR       [D1BW.1++],[D1BR.1++],[D1BR.1++]
        DB        ASL       [D1BW.1++],[D1BR.1++],[D1BR.1++]
        DB        ASR       [D1BW.1++],[D1BR.1++],[D1BR.1++]
        DB        LSL       [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DB        LSR       [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DB        ASL       [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DB        ASR       [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DB        LSL       [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        DB        LSR       [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        DB        ASL       [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        DB        ASR       [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        DB        LSL       [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        DB        LSR       [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        DB        ASL       [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        DB        ASR       [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        DB        LSL       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DB        LSR       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DB        ASL       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DB        ASR       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DB        LSL       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DB        LSR       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DB        ASL       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DB        ASR       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DB        LSL       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DB        LSR       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DB        ASL       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DB        ASR       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DB        LSL       [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        DB        LSR       [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        DB        ASL       [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        DB        ASR       [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        DB        LSL       [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DB        LSR       [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DB        ASL       [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DB        ASR       [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DB        LSL       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        DB        LSR       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        DB        ASL       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        DB        ASR       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        DB        LSL       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        DB        LSR       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        DB        ASL       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        DB        ASR       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        DB        LSL       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DB        LSR       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DB        ASL       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DB        ASR       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DB        LSL       [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DB        LSR       [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DB        ASL       [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DB        ASR       [D1BW.1+D1BWI.1++],D1.7,D1Re0
        DB        LSL       [D1BW.1+D1BWI.1++],D1.7,D1.7
        DB        LSR       [D1BW.1+D1BWI.1++],D1.7,D1.7
        DB        ASL       [D1BW.1+D1BWI.1++],D1.7,D1.7
        DB        ASR       [D1BW.1+D1BWI.1++],D1.7,D1.7
        DB        LSL       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        DB        LSR       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        DB        ASL       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        DB        ASR       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        DB        LSL       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        DB        LSR       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        DB        ASL       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        DB        ASR       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        DB        LSL       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DB        LSR       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DB        ASL       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DB        ASR       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DB        LSL       [D1BW.1+D1BWI.1++],AC1.0,D1Re0
        DL        LSR       [D1BW.1+D1BWI.1++],AC1.0,D1Re0
        DL        ASL       [D1BW.1+D1BWI.1++],AC1.0,D1Re0
        DL        ASR       [D1BW.1+D1BWI.1++],AC1.0,D1Re0
        DLR       LSL       [D1BW.1+D1BWI.1++],AC1.0,D1.7
        DLR       LSR       [D1BW.1+D1BWI.1++],AC1.0,D1.7
        DLR       ASL       [D1BW.1+D1BWI.1++],AC1.0,D1.7
        DLR       ASR       [D1BW.1+D1BWI.1++],AC1.0,D1.7
        DLG       LSL       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1]
        DLG       LSR       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1]
        DLG       ASL       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1]
        DLG       ASR       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1]
        DLB       LSL       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1++]
        DLB       LSR       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1++]
        DLB       ASL       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1++]
        DLB       ASR       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1++]
        D         LSR       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1+D1BRI.1++]
        D         ASL       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1+D1BRI.1++]
        D         ASR       [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1+D1BRI.1++]
        DT        LSL       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DLT       LSL       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DT        LSR       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DLT       LSR       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DT        ASL       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DLT       ASL       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DT        ASR       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DLT       ASR       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DT        LSL       D1Re0,D1Re0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DLT       LSL       D1Re0,D1Re0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DT        LSR       D1Re0,D1Re0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DLT       LSR       D1Re0,D1Re0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DT        ASL       D1Re0,D1Re0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DLT       ASL       D1Re0,D1Re0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DT        ASR       D1Re0,D1Re0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DLT       ASR       D1Re0,D1Re0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSL       D0Re0,D0Re0,#0xf
        D         LSR       D0Re0,D0Re0,#0xf
        D         ASL       D0Re0,D0Re0,#0xf
        D         ASR       D0Re0,D0Re0,#0xf
        D         LSL       D0Re0,D0Re0,#0x10
        D         LSR       D0Re0,D0Re0,#0x10
        D         ASL       D0Re0,D0Re0,#0x10
        D         ASR       D0Re0,D0Re0,#0x10
        D         LSL       D0Re0,D0Re0,#0x1f
        D         LSR       D0Re0,D0Re0,#0x1f
        D         ASL       D0Re0,D0Re0,#0x1f
        D         ASR       D0Re0,D0Re0,#0x1f
        D         LSL       D0Re0,D0.7,#0xf
        D         LSR       D0Re0,D0.7,#0xf
        D         ASL       D0Re0,D0.7,#0xf
        D         ASR       D0Re0,D0.7,#0xf
        D         LSL       D0Re0,D0.7,#0x10
        D         LSR       D0Re0,D0.7,#0x10
        D         ASL       D0Re0,D0.7,#0x10
        D         ASR       D0Re0,D0.7,#0x10
        D         LSL       D0Re0,D0.7,#0x1f
        D         LSR       D0Re0,D0.7,#0x1f
        D         ASL       D0Re0,D0.7,#0x1f
        D         ASR       D0Re0,D0.7,#0x1f
        D         LSL       D0Re0,[D0AR.0],#0xf
        DL        LSL       D0Re0,[D0AR.0],#0xf
        D         LSR       D0Re0,AC0.0,#0xf
        DL        LSR       D0Re0,AC0.0,#0xf
        D         ASL       D0Re0,AC0.0,#0xf
        DL        ASL       D0Re0,AC0.0,#0xf
        D         ASR       D0Re0,AC0.0,#0xf
        DL        ASR       D0Re0,AC0.0,#0xf
        D         LSL       D0Re0,AC0.0,#0x10
        DLR       LSL       D0Re0,[D0AR.0],#0x10
        DR        LSR       D0Re0,[D0AR.0],#0x10
        DLR       LSR       D0Re0,[D0AR.0],#0x10
        DR        ASL       D0Re0,[D0AR.0],#0x10
        DLR       ASL       D0Re0,[D0AR.0],#0x10
        DR        ASR       D0Re0,[D0AR.0],#0x10
        DLR       ASR       D0Re0,[D0AR.0],#0x10
        DR        LSL       D0Re0,[D0AR.0],#0x1f
        DLR       LSL       D0Re0,AC0.0,#0x1f
        DR        LSR       D0Re0,AC0.0,#0x1f
        DLR       LSR       D0Re0,AC0.0,#0x1f
        DR        ASL       D0Re0,AC0.0,#0x1f
        DLR       ASL       D0Re0,AC0.0,#0x1f
        DR        ASR       D0Re0,AC0.0,#0x1f
        DLR       ASR       D0Re0,AC0.0,#0x1f
        DB        LSL       D0Re0,[D0AR.0++],#0xf
        DB        LSR       D0Re0,[D0AR.0++],#0xf
        DB        ASL       D0Re0,[D0AR.0++],#0xf
        DB        ASR       D0Re0,[D0AR.0++],#0xf
        DB        LSL       D0Re0,[D0AR.0++],#0x10
        DB        LSR       D0Re0,[D0AR.0++],#0x10
        DB        ASL       D0Re0,[D0AR.0++],#0x10
        DB        ASR       D0Re0,[D0AR.0++],#0x10
        DB        LSL       D0Re0,[D0AR.0++],#0x1f
        DB        LSR       D0Re0,[D0AR.0++],#0x1f
        DB        ASL       D0Re0,[D0AR.0++],#0x1f
        DB        ASR       D0Re0,[D0AR.0++],#0x1f
        DB        LSL       D0Re0,[D0AR.0+D0ARI.0++],#0xf
        DB        LSR       D0Re0,[D0AR.0+D0ARI.0++],#0xf
        DB        ASL       D0Re0,[D0AR.0+D0ARI.0++],#0xf
        DB        ASR       D0Re0,[D0AR.0+D0ARI.0++],#0xf
        DB        LSL       D0Re0,[D0AR.0+D0ARI.0++],#0x10
        DB        LSR       D0Re0,[D0AR.0+D0ARI.0++],#0x10
        DB        ASL       D0Re0,[D0AR.0+D0ARI.0++],#0x10
        DB        ASR       D0Re0,[D0AR.0+D0ARI.0++],#0x10
        DB        LSL       D0Re0,[D0AR.0+D0ARI.0++],#0x1f
        DB        LSR       D0Re0,[D0AR.0+D0ARI.0++],#0x1f
        DB        ASL       D0Re0,[D0AR.0+D0ARI.0++],#0x1f
        DB        ASR       D0Re0,[D0AR.0+D0ARI.0++],#0x1f
        DB        LSL       D0.7,D0Re0,#0xf
        DB        LSR       D0.7,D0Re0,#0xf
        DB        ASL       D0.7,D0Re0,#0xf
        DB        ASR       D0.7,D0Re0,#0xf
        DB        LSL       D0.7,D0Re0,#0x10
        DB        LSR       D0.7,D0Re0,#0x10
        DB        ASL       D0.7,D0Re0,#0x10
        DB        ASR       D0.7,D0Re0,#0x10
        DB        LSL       D0.7,D0Re0,#0x1f
        DB        LSR       D0.7,D0Re0,#0x1f
        DB        ASL       D0.7,D0Re0,#0x1f
        DB        ASR       D0.7,D0Re0,#0x1f
        DB        LSL       D0.7,D0.7,#0xf
        DB        LSR       D0.7,D0.7,#0xf
        DB        ASL       D0.7,D0.7,#0xf
        DB        ASR       D0.7,D0.7,#0xf
        DB        LSL       D0.7,D0.7,#0x10
        DB        LSR       D0.7,D0.7,#0x10
        DB        ASL       D0.7,D0.7,#0x10
        DB        ASR       D0.7,D0.7,#0x10
        DB        LSL       D0.7,D0.7,#0x1f
        DB        LSR       D0.7,D0.7,#0x1f
        DB        ASL       D0.7,D0.7,#0x1f
        DB        ASR       D0.7,D0.7,#0x1f
        DR        LSL       D0.7,AC0.0,#0xf
        DLG       LSL       D0.7,[D0AR.0],#0xf
        DG        LSR       D0.7,[D0AR.0],#0xf
        DLG       LSR       D0.7,[D0AR.0],#0xf
        DG        ASL       D0.7,[D0AR.0],#0xf
        DLG       ASL       D0.7,[D0AR.0],#0xf
        DG        ASR       D0.7,[D0AR.0],#0xf
        DLG       ASR       D0.7,[D0AR.0],#0xf
        DG        LSL       D0.7,[D0AR.0],#0x10
        DLG       LSL       D0.7,AC0.0,#0x10
        DG        LSR       D0.7,AC0.0,#0x10
        DLG       LSR       D0.7,AC0.0,#0x10
        DG        ASL       D0.7,AC0.0,#0x10
        DLG       ASL       D0.7,AC0.0,#0x10
        DG        ASR       D0.7,AC0.0,#0x10
        DLG       ASR       D0.7,AC0.0,#0x10
        DG        LSL       D0.7,AC0.0,#0x1f
        DLB       LSL       D0.7,[D0AR.0],#0x1f
        DB        LSR       D0.7,[D0AR.0],#0x1f
        DLB       LSR       D0.7,[D0AR.0],#0x1f
        DB        ASL       D0.7,[D0AR.0],#0x1f
        DLB       ASL       D0.7,[D0AR.0],#0x1f
        DB        ASR       D0.7,[D0AR.0],#0x1f
        DLB       ASR       D0.7,[D0AR.0],#0x1f
        DLG       LSL       D0.7,[D0AR.0++],#0xf
        DLG       LSR       D0.7,[D0AR.0++],#0xf
        DLG       ASL       D0.7,[D0AR.0++],#0xf
        DLG       ASR       D0.7,[D0AR.0++],#0xf
        DLG       LSL       D0.7,[D0AR.0++],#0x10
        DLG       LSR       D0.7,[D0AR.0++],#0x10
        DLG       ASL       D0.7,[D0AR.0++],#0x10
        DLG       ASR       D0.7,[D0AR.0++],#0x10
        DLG       LSL       D0.7,[D0AR.0++],#0x1f
        DLG       LSR       D0.7,[D0AR.0++],#0x1f
        DLG       ASL       D0.7,[D0AR.0++],#0x1f
        DLG       ASR       D0.7,[D0AR.0++],#0x1f
        DLG       LSL       D0.7,[D0AR.0+D0ARI.0++],#0xf
        DLG       LSR       D0.7,[D0AR.0+D0ARI.0++],#0xf
        DLG       ASL       D0.7,[D0AR.0+D0ARI.0++],#0xf
        DLG       ASR       D0.7,[D0AR.0+D0ARI.0++],#0xf
        DLG       LSL       D0.7,[D0AR.0+D0ARI.0++],#0x10
        DLG       LSR       D0.7,[D0AR.0+D0ARI.0++],#0x10
        DLG       ASL       D0.7,[D0AR.0+D0ARI.0++],#0x10
        DLG       ASR       D0.7,[D0AR.0+D0ARI.0++],#0x10
        DLG       LSL       D0.7,[D0AR.0+D0ARI.0++],#0x1f
        DLG       LSR       D0.7,[D0AR.0+D0ARI.0++],#0x1f
        DLG       ASL       D0.7,[D0AR.0+D0ARI.0++],#0x1f
        DLG       ASR       D0.7,[D0AR.0+D0ARI.0++],#0x1f
        DLG       LSL       [D0AW.0],D0Re0,#0xf
        DLG       LSR       [D0AW.0],D0Re0,#0xf
        DLG       ASL       [D0AW.0],D0Re0,#0xf
        DLG       ASR       [D0AW.0],D0Re0,#0xf
        DLG       LSL       [D0AW.0],D0Re0,#0x10
        DLG       LSR       [D0AW.0],D0Re0,#0x10
        DLG       ASL       [D0AW.0],D0Re0,#0x10
        DLG       ASR       [D0AW.0],D0Re0,#0x10
        DLG       LSL       [D0AW.0],D0Re0,#0x1f
        DLG       LSR       [D0AW.0],D0Re0,#0x1f
        DLG       ASL       [D0AW.0],D0Re0,#0x1f
        DLG       ASR       [D0AW.0],D0Re0,#0x1f
        DLG       LSL       [D0AW.0],D0.7,#0xf
        DLG       LSR       [D0AW.0],D0.7,#0xf
        DLG       ASL       [D0AW.0],D0.7,#0xf
        DLG       ASR       [D0AW.0],D0.7,#0xf
        DLG       LSL       [D0AW.0],D0.7,#0x10
        DLG       LSR       [D0AW.0],D0.7,#0x10
        DLG       ASL       [D0AW.0],D0.7,#0x10
        DLG       ASR       [D0AW.0],D0.7,#0x10
        DLG       LSL       [D0AW.0],D0.7,#0x1f
        DLG       LSR       [D0AW.0],D0.7,#0x1f
        DLG       ASL       [D0AW.0],D0.7,#0x1f
        DLG       ASR       [D0AW.0],D0.7,#0x1f
        DB        LSL       [D0AW.0],[D0AR.0],#0xf
        DLB       LSL       [D0AW.0],AC0.0,#0xf
        DB        LSR       [D0AW.0],AC0.0,#0xf
        DLB       LSR       [D0AW.0],AC0.0,#0xf
        DB        ASL       [D0AW.0],AC0.0,#0xf
        DLB       ASL       [D0AW.0],AC0.0,#0xf
        DB        ASR       [D0AW.0],AC0.0,#0xf
        DLB       ASR       [D0AW.0],AC0.0,#0xf
        DB        LSL       [D0AW.0],AC0.0,#0x10
        D         LSR       [D0AW.0],[D0AR.0],#0x10
        DL        LSR       [D0AW.0],[D0AR.0],#0x10
        D         ASL       [D0AW.0],[D0AR.0],#0x10
        DL        ASL       [D0AW.0],[D0AR.0],#0x10
        D         ASR       [D0AW.0],[D0AR.0],#0x10
        DL        ASR       [D0AW.0],[D0AR.0],#0x10
        D         LSL       [D0AW.0],[D0AR.0],#0x1f
        DL        LSL       [D0AW.0],[D0AR.0],#0x1f
        D         LSR       [D0AW.0],AC0.0,#0x1f
        DL        LSR       [D0AW.0],AC0.0,#0x1f
        D         ASL       [D0AW.0],AC0.0,#0x1f
        DL        ASL       [D0AW.0],AC0.0,#0x1f
        D         ASR       [D0AW.0],AC0.0,#0x1f
        DL        ASR       [D0AW.0],AC0.0,#0x1f
        DR        LSL       [D0AW.0],[D0AR.0++],#0xf
        DR        LSR       [D0AW.0],[D0AR.0++],#0xf
        DR        ASL       [D0AW.0],[D0AR.0++],#0xf
        DR        ASR       [D0AW.0],[D0AR.0++],#0xf
        DR        LSL       [D0AW.0],[D0AR.0++],#0x10
        DR        LSR       [D0AW.0],[D0AR.0++],#0x10
        DR        ASL       [D0AW.0],[D0AR.0++],#0x10
        DR        ASR       [D0AW.0],[D0AR.0++],#0x10
        DR        LSL       [D0AW.0],[D0AR.0++],#0x1f
        DR        LSR       [D0AW.0],[D0AR.0++],#0x1f
        DR        ASL       [D0AW.0],[D0AR.0++],#0x1f
        DR        ASR       [D0AW.0],[D0AR.0++],#0x1f
        DR        LSL       [D0AW.0],[D0AR.0+D0ARI.0++],#0xf
        DR        LSR       [D0AW.0],[D0AR.0+D0ARI.0++],#0xf
        DR        ASL       [D0AW.0],[D0AR.0+D0ARI.0++],#0xf
        DR        ASR       [D0AW.0],[D0AR.0+D0ARI.0++],#0xf
        DR        LSL       [D0AW.0],[D0AR.0+D0ARI.0++],#0x10
        DR        LSR       [D0AW.0],[D0AR.0+D0ARI.0++],#0x10
        DR        ASL       [D0AW.0],[D0AR.0+D0ARI.0++],#0x10
        DR        ASR       [D0AW.0],[D0AR.0+D0ARI.0++],#0x10
        DR        LSL       [D0AW.0],[D0AR.0+D0ARI.0++],#0x1f
        DR        LSR       [D0AW.0],[D0AR.0+D0ARI.0++],#0x1f
        DR        ASL       [D0AW.0],[D0AR.0+D0ARI.0++],#0x1f
        DR        ASR       [D0AW.0],[D0AR.0+D0ARI.0++],#0x1f
        DR        LSL       [D0AW.0++],D0Re0,#0xf
        DR        LSR       [D0AW.0++],D0Re0,#0xf
        DR        ASL       [D0AW.0++],D0Re0,#0xf
        DR        ASR       [D0AW.0++],D0Re0,#0xf
        DR        LSL       [D0AW.0++],D0Re0,#0x10
        DR        LSR       [D0AW.0++],D0Re0,#0x10
        DR        ASL       [D0AW.0++],D0Re0,#0x10
        DR        ASR       [D0AW.0++],D0Re0,#0x10
        DR        LSL       [D0AW.0++],D0Re0,#0x1f
        DR        LSR       [D0AW.0++],D0Re0,#0x1f
        DR        ASL       [D0AW.0++],D0Re0,#0x1f
        DR        ASR       [D0AW.0++],D0Re0,#0x1f
        DR        LSL       [D0AW.0++],D0.7,#0xf
        DR        LSR       [D0AW.0++],D0.7,#0xf
        DR        ASL       [D0AW.0++],D0.7,#0xf
        DR        ASR       [D0AW.0++],D0.7,#0xf
        DR        LSL       [D0AW.0++],D0.7,#0x10
        DR        LSR       [D0AW.0++],D0.7,#0x10
        DR        ASL       [D0AW.0++],D0.7,#0x10
        DR        ASR       [D0AW.0++],D0.7,#0x10
        DR        LSL       [D0AW.0++],D0.7,#0x1f
        DR        LSR       [D0AW.0++],D0.7,#0x1f
        DR        ASL       [D0AW.0++],D0.7,#0x1f
        DR        ASR       [D0AW.0++],D0.7,#0x1f
        D         LSL       [D0AW.0++],AC0.0,#0xf
        DLR       LSL       [D0AW.0++],[D0AR.0],#0xf
        DR        LSR       [D0AW.0++],[D0AR.0],#0xf
        DLR       LSR       [D0AW.0++],[D0AR.0],#0xf
        DR        ASL       [D0AW.0++],[D0AR.0],#0xf
        DLR       ASL       [D0AW.0++],[D0AR.0],#0xf
        DR        ASR       [D0AW.0++],[D0AR.0],#0xf
        DLR       ASR       [D0AW.0++],[D0AR.0],#0xf
        DR        LSL       [D0AW.0++],[D0AR.0],#0x10
        DLR       LSL       [D0AW.0++],AC0.0,#0x10
        DR        LSR       [D0AW.0++],AC0.0,#0x10
        DLR       LSR       [D0AW.0++],AC0.0,#0x10
        DR        ASL       [D0AW.0++],AC0.0,#0x10
        DLR       ASL       [D0AW.0++],AC0.0,#0x10
        DR        ASR       [D0AW.0++],AC0.0,#0x10
        DLR       ASR       [D0AW.0++],AC0.0,#0x10
        DR        LSL       [D0AW.0++],AC0.0,#0x1f
        DLG       LSL       [D0AW.0++],[D0AR.0],#0x1f
        DG        LSR       [D0AW.0++],[D0AR.0],#0x1f
        DLG       LSR       [D0AW.0++],[D0AR.0],#0x1f
        DG        ASL       [D0AW.0++],[D0AR.0],#0x1f
        DLG       ASL       [D0AW.0++],[D0AR.0],#0x1f
        DG        ASR       [D0AW.0++],[D0AR.0],#0x1f
        DLG       ASR       [D0AW.0++],[D0AR.0],#0x1f
        DL        LSL       [D0AW.0++],[D0AR.0++],#0xf
        DL        LSR       [D0AW.0++],[D0AR.0++],#0xf
        DL        ASL       [D0AW.0++],[D0AR.0++],#0xf
        DL        ASR       [D0AW.0++],[D0AR.0++],#0xf
        DL        LSL       [D0AW.0++],[D0AR.0++],#0x10
        DL        LSR       [D0AW.0++],[D0AR.0++],#0x10
        DL        ASL       [D0AW.0++],[D0AR.0++],#0x10
        DL        ASR       [D0AW.0++],[D0AR.0++],#0x10
        DL        LSL       [D0AW.0++],[D0AR.0++],#0x1f
        DL        LSR       [D0AW.0++],[D0AR.0++],#0x1f
        DL        ASL       [D0AW.0++],[D0AR.0++],#0x1f
        DL        ASR       [D0AW.0++],[D0AR.0++],#0x1f
        DL        LSL       [D0AW.0++],[D0AR.0+D0ARI.0++],#0xf
        DL        LSR       [D0AW.0++],[D0AR.0+D0ARI.0++],#0xf
        DL        ASL       [D0AW.0++],[D0AR.0+D0ARI.0++],#0xf
        DL        ASR       [D0AW.0++],[D0AR.0+D0ARI.0++],#0xf
        DL        LSL       [D0AW.0++],[D0AR.0+D0ARI.0++],#0x10
        DL        LSR       [D0AW.0++],[D0AR.0+D0ARI.0++],#0x10
        DL        ASL       [D0AW.0++],[D0AR.0+D0ARI.0++],#0x10
        DL        ASR       [D0AW.0++],[D0AR.0+D0ARI.0++],#0x10
        DL        LSL       [D0AW.0++],[D0AR.0+D0ARI.0++],#0x1f
        DL        LSR       [D0AW.0++],[D0AR.0+D0ARI.0++],#0x1f
        DL        ASL       [D0AW.0++],[D0AR.0+D0ARI.0++],#0x1f
        DL        ASR       [D0AW.0++],[D0AR.0+D0ARI.0++],#0x1f
        DL        LSL       [D0AW.0+D0AWI.0++],D0Re0,#0xf
        DL        LSR       [D0AW.0+D0AWI.0++],D0Re0,#0xf
        DL        ASL       [D0AW.0+D0AWI.0++],D0Re0,#0xf
        DL        ASR       [D0AW.0+D0AWI.0++],D0Re0,#0xf
        DL        LSL       [D0AW.0+D0AWI.0++],D0Re0,#0x10
        DL        LSR       [D0AW.0+D0AWI.0++],D0Re0,#0x10
        DL        ASL       [D0AW.0+D0AWI.0++],D0Re0,#0x10
        DL        ASR       [D0AW.0+D0AWI.0++],D0Re0,#0x10
        DL        LSL       [D0AW.0+D0AWI.0++],D0Re0,#0x1f
        DL        LSR       [D0AW.0+D0AWI.0++],D0Re0,#0x1f
        DL        ASL       [D0AW.0+D0AWI.0++],D0Re0,#0x1f
        DL        ASR       [D0AW.0+D0AWI.0++],D0Re0,#0x1f
        DL        LSL       [D0AW.0+D0AWI.0++],D0.7,#0xf
        DL        LSR       [D0AW.0+D0AWI.0++],D0.7,#0xf
        DL        ASL       [D0AW.0+D0AWI.0++],D0.7,#0xf
        DL        ASR       [D0AW.0+D0AWI.0++],D0.7,#0xf
        DL        LSL       [D0AW.0+D0AWI.0++],D0.7,#0x10
        DL        LSR       [D0AW.0+D0AWI.0++],D0.7,#0x10
        DL        ASL       [D0AW.0+D0AWI.0++],D0.7,#0x10
        DL        ASR       [D0AW.0+D0AWI.0++],D0.7,#0x10
        DL        LSL       [D0AW.0+D0AWI.0++],D0.7,#0x1f
        DL        LSR       [D0AW.0+D0AWI.0++],D0.7,#0x1f
        DL        ASL       [D0AW.0+D0AWI.0++],D0.7,#0x1f
        DL        ASR       [D0AW.0+D0AWI.0++],D0.7,#0x1f
        DG        LSL       [D0AW.0+D0AWI.0++],[D0AR.0],#0xf
        DLG       LSL       [D0AW.0+D0AWI.0++],AC0.0,#0xf
        DG        LSR       [D0AW.0+D0AWI.0++],AC0.0,#0xf
        DLG       LSR       [D0AW.0+D0AWI.0++],AC0.0,#0xf
        DG        ASL       [D0AW.0+D0AWI.0++],AC0.0,#0xf
        DLG       ASL       [D0AW.0+D0AWI.0++],AC0.0,#0xf
        DG        ASR       [D0AW.0+D0AWI.0++],AC0.0,#0xf
        DLG       ASR       [D0AW.0+D0AWI.0++],AC0.0,#0xf
        DG        LSL       [D0AW.0+D0AWI.0++],AC0.0,#0x10
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0],#0x10
        DB        LSR       [D0AW.0+D0AWI.0++],[D0AR.0],#0x10
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0],#0x10
        DB        ASL       [D0AW.0+D0AWI.0++],[D0AR.0],#0x10
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0],#0x10
        DB        ASR       [D0AW.0+D0AWI.0++],[D0AR.0],#0x10
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0],#0x10
        DB        LSL       [D0AW.0+D0AWI.0++],[D0AR.0],#0x1f
        DLB       LSL       [D0AW.0+D0AWI.0++],AC0.0,#0x1f
        DB        LSR       [D0AW.0+D0AWI.0++],AC0.0,#0x1f
        DLB       LSR       [D0AW.0+D0AWI.0++],AC0.0,#0x1f
        DB        ASL       [D0AW.0+D0AWI.0++],AC0.0,#0x1f
        DLB       ASL       [D0AW.0+D0AWI.0++],AC0.0,#0x1f
        DB        ASR       [D0AW.0+D0AWI.0++],AC0.0,#0x1f
        DLB       ASR       [D0AW.0+D0AWI.0++],AC0.0,#0x1f
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0++],#0xf
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0++],#0xf
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0++],#0xf
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0++],#0xf
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0++],#0x10
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0++],#0x10
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0++],#0x10
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0++],#0x10
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0++],#0x1f
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0++],#0x1f
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0++],#0x1f
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0++],#0x1f
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0xf
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0xf
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0xf
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0xf
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x10
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x10
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x10
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x10
        DLB       LSL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x1f
        DLB       LSR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x1f
        DLB       ASL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x1f
        DLB       ASR       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x1f
        DLB       LSL       D1Re0,D1Re0,#0xf
        DLB       LSR       D1Re0,D1Re0,#0xf
        DLB       ASL       D1Re0,D1Re0,#0xf
        DLB       ASR       D1Re0,D1Re0,#0xf
        DLB       LSL       D1Re0,D1Re0,#0x10
        DLB       LSR       D1Re0,D1Re0,#0x10
        DLB       ASL       D1Re0,D1Re0,#0x10
        DLB       ASR       D1Re0,D1Re0,#0x10
        DLB       LSL       D1Re0,D1Re0,#0x1f
        DLB       LSR       D1Re0,D1Re0,#0x1f
        DLB       ASL       D1Re0,D1Re0,#0x1f
        DLB       ASR       D1Re0,D1Re0,#0x1f
        DLB       LSL       D1Re0,D1.7,#0xf
        DLB       LSR       D1Re0,D1.7,#0xf
        DLB       ASL       D1Re0,D1.7,#0xf
        DLB       ASR       D1Re0,D1.7,#0xf
        DLB       LSL       D1Re0,D1.7,#0x10
        DLB       LSR       D1Re0,D1.7,#0x10
        DLB       ASL       D1Re0,D1.7,#0x10
        DLB       ASR       D1Re0,D1.7,#0x10
        DLB       LSL       D1Re0,D1.7,#0x1f
        DLB       LSR       D1Re0,D1.7,#0x1f
        DLB       ASL       D1Re0,D1.7,#0x1f
        DLB       ASR       D1Re0,D1.7,#0x1f
        D         LSR       D1Re0,AC1.0,#0xf
        D         ASL       D1Re0,AC1.0,#0xf
        D         ASR       D1Re0,AC1.0,#0xf
        D         LSL       D1Re0,AC1.0,#0x10
        DR        LSR       D1Re0,AC1.0,#0x10
        DR        ASL       D1Re0,AC1.0,#0x10
        DR        ASR       D1Re0,AC1.0,#0x10
        DR        LSL       D1Re0,AC1.0,#0x1f
        DG        LSR       D1Re0,AC1.0,#0x1f
        DG        ASL       D1Re0,AC1.0,#0x1f
        DG        ASR       D1Re0,AC1.0,#0x1f
        DG        LSL       D1Re0,[D1BR.1],#0xf
        DG        LSR       D1Re0,[D1BR.1],#0xf
        DG        ASL       D1Re0,[D1BR.1],#0xf
        DG        ASR       D1Re0,[D1BR.1],#0xf
        DG        LSL       D1Re0,[D1BR.1],#0x10
        DG        LSR       D1Re0,[D1BR.1],#0x10
        DG        ASL       D1Re0,[D1BR.1],#0x10
        DG        ASR       D1Re0,[D1BR.1],#0x10
        DG        LSL       D1Re0,[D1BR.1],#0x1f
        DG        LSR       D1Re0,[D1BR.1],#0x1f
        DG        ASL       D1Re0,[D1BR.1],#0x1f
        DG        ASR       D1Re0,[D1BR.1],#0x1f
        DG        LSL       D1Re0,[D1BR.1++],#0xf
        DG        LSR       D1Re0,[D1BR.1++],#0xf
        DG        ASL       D1Re0,[D1BR.1++],#0xf
        DG        ASR       D1Re0,[D1BR.1++],#0xf
        DG        LSL       D1Re0,[D1BR.1++],#0x10
        DG        LSR       D1Re0,[D1BR.1++],#0x10
        DG        ASL       D1Re0,[D1BR.1++],#0x10
        DG        ASR       D1Re0,[D1BR.1++],#0x10
        DG        LSL       D1Re0,[D1BR.1++],#0x1f
        DG        LSR       D1Re0,[D1BR.1++],#0x1f
        DG        ASL       D1Re0,[D1BR.1++],#0x1f
        DG        ASR       D1Re0,[D1BR.1++],#0x1f
        DG        LSL       D1Re0,[D1BR.1+D1BRI.1++],#0xf
        DG        LSR       D1Re0,[D1BR.1+D1BRI.1++],#0xf
        DG        ASL       D1Re0,[D1BR.1+D1BRI.1++],#0xf
        DG        ASR       D1Re0,[D1BR.1+D1BRI.1++],#0xf
        DG        LSL       D1Re0,[D1BR.1+D1BRI.1++],#0x10
        DG        LSR       D1Re0,[D1BR.1+D1BRI.1++],#0x10
        DG        ASL       D1Re0,[D1BR.1+D1BRI.1++],#0x10
        DG        ASR       D1Re0,[D1BR.1+D1BRI.1++],#0x10
        DG        LSL       D1Re0,[D1BR.1+D1BRI.1++],#0x1f
        DG        LSR       D1Re0,[D1BR.1+D1BRI.1++],#0x1f
        DG        ASL       D1Re0,[D1BR.1+D1BRI.1++],#0x1f
        DG        ASR       D1Re0,[D1BR.1+D1BRI.1++],#0x1f
        DG        LSL       D1.7,D1Re0,#0xf
        DG        LSR       D1.7,D1Re0,#0xf
        DG        ASL       D1.7,D1Re0,#0xf
        DG        ASR       D1.7,D1Re0,#0xf
        DG        LSL       D1.7,D1Re0,#0x10
        DG        LSR       D1.7,D1Re0,#0x10
        DG        ASL       D1.7,D1Re0,#0x10
        DG        ASR       D1.7,D1Re0,#0x10
        DG        LSL       D1.7,D1Re0,#0x1f
        DG        LSR       D1.7,D1Re0,#0x1f
        DG        ASL       D1.7,D1Re0,#0x1f
        DG        ASR       D1.7,D1Re0,#0x1f
        DG        LSL       D1.7,D1.7,#0xf
        DG        LSR       D1.7,D1.7,#0xf
        DG        ASL       D1.7,D1.7,#0xf
        DG        ASR       D1.7,D1.7,#0xf
        DG        LSL       D1.7,D1.7,#0x10
        DG        LSR       D1.7,D1.7,#0x10
        DG        ASL       D1.7,D1.7,#0x10
        DG        ASR       D1.7,D1.7,#0x10
        DG        LSL       D1.7,D1.7,#0x1f
        DG        LSR       D1.7,D1.7,#0x1f
        DG        ASL       D1.7,D1.7,#0x1f
        DG        ASR       D1.7,D1.7,#0x1f
        DG        LSL       D1.7,AC1.0,#0xf
        DB        LSR       D1.7,AC1.0,#0xf
        DB        ASL       D1.7,AC1.0,#0xf
        DB        ASR       D1.7,AC1.0,#0xf
        DB        LSL       D1.7,AC1.0,#0x10
        DL        LSR       D1.7,AC1.0,#0x10
        DL        ASL       D1.7,AC1.0,#0x10
        DL        ASR       D1.7,AC1.0,#0x10
        DLR       LSL       D1.7,AC1.0,#0x1f
        DLR       LSR       D1.7,AC1.0,#0x1f
        DLR       ASL       D1.7,AC1.0,#0x1f
        DLR       ASR       D1.7,AC1.0,#0x1f
        DLR       LSL       D1.7,[D1BR.1],#0xf
        DLR       LSR       D1.7,[D1BR.1],#0xf
        DLR       ASL       D1.7,[D1BR.1],#0xf
        DLR       ASR       D1.7,[D1BR.1],#0xf
        DLR       LSL       D1.7,[D1BR.1],#0x10
        DLR       LSR       D1.7,[D1BR.1],#0x10
        DLR       ASL       D1.7,[D1BR.1],#0x10
        DLR       ASR       D1.7,[D1BR.1],#0x10
        DLR       LSL       D1.7,[D1BR.1],#0x1f
        DLR       LSR       D1.7,[D1BR.1],#0x1f
        DLR       ASL       D1.7,[D1BR.1],#0x1f
        DLR       ASR       D1.7,[D1BR.1],#0x1f
        DLR       LSL       D1.7,[D1BR.1++],#0xf
        DLR       LSR       D1.7,[D1BR.1++],#0xf
        DLR       ASL       D1.7,[D1BR.1++],#0xf
        DLR       ASR       D1.7,[D1BR.1++],#0xf
        DLR       LSL       D1.7,[D1BR.1++],#0x10
        DLR       LSR       D1.7,[D1BR.1++],#0x10
        DLR       ASL       D1.7,[D1BR.1++],#0x10
        DLR       ASR       D1.7,[D1BR.1++],#0x10
        DLR       LSL       D1.7,[D1BR.1++],#0x1f
        DLR       LSR       D1.7,[D1BR.1++],#0x1f
        DLR       ASL       D1.7,[D1BR.1++],#0x1f
        DLR       ASR       D1.7,[D1BR.1++],#0x1f
        DLR       LSL       D1.7,[D1BR.1+D1BRI.1++],#0xf
        DLR       LSR       D1.7,[D1BR.1+D1BRI.1++],#0xf
        DLR       ASL       D1.7,[D1BR.1+D1BRI.1++],#0xf
        DLR       ASR       D1.7,[D1BR.1+D1BRI.1++],#0xf
        DLR       LSL       D1.7,[D1BR.1+D1BRI.1++],#0x10
        DLR       LSR       D1.7,[D1BR.1+D1BRI.1++],#0x10
        DLR       ASL       D1.7,[D1BR.1+D1BRI.1++],#0x10
        DLR       ASR       D1.7,[D1BR.1+D1BRI.1++],#0x10
        DLR       LSL       D1.7,[D1BR.1+D1BRI.1++],#0x1f
        DLR       LSR       D1.7,[D1BR.1+D1BRI.1++],#0x1f
        DLR       ASL       D1.7,[D1BR.1+D1BRI.1++],#0x1f
        DLR       ASR       D1.7,[D1BR.1+D1BRI.1++],#0x1f
        DLR       LSL       [D1BW.1],D1Re0,#0xf
        DLR       LSR       [D1BW.1],D1Re0,#0xf
        DLR       ASL       [D1BW.1],D1Re0,#0xf
        DLR       ASR       [D1BW.1],D1Re0,#0xf
        DLR       LSL       [D1BW.1],D1Re0,#0x10
        DLR       LSR       [D1BW.1],D1Re0,#0x10
        DLR       ASL       [D1BW.1],D1Re0,#0x10
        DLR       ASR       [D1BW.1],D1Re0,#0x10
        DLR       LSL       [D1BW.1],D1Re0,#0x1f
        DLR       LSR       [D1BW.1],D1Re0,#0x1f
        DLR       ASL       [D1BW.1],D1Re0,#0x1f
        DLR       ASR       [D1BW.1],D1Re0,#0x1f
        DLR       LSL       [D1BW.1],D1.7,#0xf
        DLR       LSR       [D1BW.1],D1.7,#0xf
        DLR       ASL       [D1BW.1],D1.7,#0xf
        DLR       ASR       [D1BW.1],D1.7,#0xf
        DLR       LSL       [D1BW.1],D1.7,#0x10
        DLR       LSR       [D1BW.1],D1.7,#0x10
        DLR       ASL       [D1BW.1],D1.7,#0x10
        DLR       ASR       [D1BW.1],D1.7,#0x10
        DLR       LSL       [D1BW.1],D1.7,#0x1f
        DLR       LSR       [D1BW.1],D1.7,#0x1f
        DLR       ASL       [D1BW.1],D1.7,#0x1f
        DLR       ASR       [D1BW.1],D1.7,#0x1f
        DLG       LSL       [D1BW.1],AC1.0,#0xf
        DLG       LSR       [D1BW.1],AC1.0,#0xf
        DLG       ASL       [D1BW.1],AC1.0,#0xf
        DLG       ASR       [D1BW.1],AC1.0,#0xf
        DLB       LSL       [D1BW.1],AC1.0,#0x10
        DLB       LSR       [D1BW.1],AC1.0,#0x10
        DLB       ASL       [D1BW.1],AC1.0,#0x10
        DLB       ASR       [D1BW.1],AC1.0,#0x10
        D         LSR       [D1BW.1],AC1.0,#0x1f
        D         ASL       [D1BW.1],AC1.0,#0x1f
        D         ASR       [D1BW.1],AC1.0,#0x1f
        D         LSL       [D1BW.1],[D1BR.1],#0xf
        D         LSR       [D1BW.1],[D1BR.1],#0xf
        D         ASL       [D1BW.1],[D1BR.1],#0xf
        D         ASR       [D1BW.1],[D1BR.1],#0xf
        D         LSL       [D1BW.1],[D1BR.1],#0x10
        D         LSR       [D1BW.1],[D1BR.1],#0x10
        D         ASL       [D1BW.1],[D1BR.1],#0x10
        D         ASR       [D1BW.1],[D1BR.1],#0x10
        D         LSL       [D1BW.1],[D1BR.1],#0x1f
        D         LSR       [D1BW.1],[D1BR.1],#0x1f
        D         ASL       [D1BW.1],[D1BR.1],#0x1f
        D         ASR       [D1BW.1],[D1BR.1],#0x1f
        D         LSL       [D1BW.1],[D1BR.1++],#0xf
        D         LSR       [D1BW.1],[D1BR.1++],#0xf
        D         ASL       [D1BW.1],[D1BR.1++],#0xf
        D         ASR       [D1BW.1],[D1BR.1++],#0xf
        D         LSL       [D1BW.1],[D1BR.1++],#0x10
        D         LSR       [D1BW.1],[D1BR.1++],#0x10
        D         ASL       [D1BW.1],[D1BR.1++],#0x10
        D         ASR       [D1BW.1],[D1BR.1++],#0x10
        D         LSL       [D1BW.1],[D1BR.1++],#0x1f
        D         LSR       [D1BW.1],[D1BR.1++],#0x1f
        D         ASL       [D1BW.1],[D1BR.1++],#0x1f
        D         ASR       [D1BW.1],[D1BR.1++],#0x1f
        D         LSL       [D1BW.1],[D1BR.1+D1BRI.1++],#0xf
        D         LSR       [D1BW.1],[D1BR.1+D1BRI.1++],#0xf
        D         ASL       [D1BW.1],[D1BR.1+D1BRI.1++],#0xf
        D         ASR       [D1BW.1],[D1BR.1+D1BRI.1++],#0xf
        D         LSL       [D1BW.1],[D1BR.1+D1BRI.1++],#0x10
        D         LSR       [D1BW.1],[D1BR.1+D1BRI.1++],#0x10
        D         ASL       [D1BW.1],[D1BR.1+D1BRI.1++],#0x10
        D         ASR       [D1BW.1],[D1BR.1+D1BRI.1++],#0x10
        D         LSL       [D1BW.1],[D1BR.1+D1BRI.1++],#0x1f
        D         LSR       [D1BW.1],[D1BR.1+D1BRI.1++],#0x1f
        D         ASL       [D1BW.1],[D1BR.1+D1BRI.1++],#0x1f
        D         ASR       [D1BW.1],[D1BR.1+D1BRI.1++],#0x1f
        D         LSL       [D1BW.1++],D1Re0,#0xf
        D         LSR       [D1BW.1++],D1Re0,#0xf
        D         ASL       [D1BW.1++],D1Re0,#0xf
        D         ASR       [D1BW.1++],D1Re0,#0xf
        D         LSL       [D1BW.1++],D1Re0,#0x10
        D         LSR       [D1BW.1++],D1Re0,#0x10
        D         ASL       [D1BW.1++],D1Re0,#0x10
        D         ASR       [D1BW.1++],D1Re0,#0x10
        D         LSL       [D1BW.1++],D1Re0,#0x1f
        D         LSR       [D1BW.1++],D1Re0,#0x1f
        D         ASL       [D1BW.1++],D1Re0,#0x1f
        D         ASR       [D1BW.1++],D1Re0,#0x1f
        D         LSL       [D1BW.1++],D1.7,#0xf
        D         LSR       [D1BW.1++],D1.7,#0xf
        D         ASL       [D1BW.1++],D1.7,#0xf
        D         ASR       [D1BW.1++],D1.7,#0xf
        D         LSL       [D1BW.1++],D1.7,#0x10
        D         LSR       [D1BW.1++],D1.7,#0x10
        D         ASL       [D1BW.1++],D1.7,#0x10
        D         ASR       [D1BW.1++],D1.7,#0x10
        D         LSL       [D1BW.1++],D1.7,#0x1f
        D         LSR       [D1BW.1++],D1.7,#0x1f
        D         ASL       [D1BW.1++],D1.7,#0x1f
        D         ASR       [D1BW.1++],D1.7,#0x1f
        D         LSL       [D1BW.1++],AC1.0,#0xf
        DR        LSR       [D1BW.1++],AC1.0,#0xf
        DR        ASL       [D1BW.1++],AC1.0,#0xf
        DR        ASR       [D1BW.1++],AC1.0,#0xf
        DR        LSL       [D1BW.1++],AC1.0,#0x10
        DG        LSR       [D1BW.1++],AC1.0,#0x10
        DG        ASL       [D1BW.1++],AC1.0,#0x10
        DG        ASR       [D1BW.1++],AC1.0,#0x10
        DG        LSL       [D1BW.1++],AC1.0,#0x1f
        DB        LSR       [D1BW.1++],AC1.0,#0x1f
        DB        ASL       [D1BW.1++],AC1.0,#0x1f
        DB        ASR       [D1BW.1++],AC1.0,#0x1f
        DB        LSL       [D1BW.1++],[D1BR.1],#0xf
        DB        LSR       [D1BW.1++],[D1BR.1],#0xf
        DB        ASL       [D1BW.1++],[D1BR.1],#0xf
        DB        ASR       [D1BW.1++],[D1BR.1],#0xf
        DB        LSL       [D1BW.1++],[D1BR.1],#0x10
        DB        LSR       [D1BW.1++],[D1BR.1],#0x10
        DB        ASL       [D1BW.1++],[D1BR.1],#0x10
        DB        ASR       [D1BW.1++],[D1BR.1],#0x10
        DB        LSL       [D1BW.1++],[D1BR.1],#0x1f
        DB        LSR       [D1BW.1++],[D1BR.1],#0x1f
        DB        ASL       [D1BW.1++],[D1BR.1],#0x1f
        DB        ASR       [D1BW.1++],[D1BR.1],#0x1f
        DB        LSL       [D1BW.1++],[D1BR.1++],#0xf
        DB        LSR       [D1BW.1++],[D1BR.1++],#0xf
        DB        ASL       [D1BW.1++],[D1BR.1++],#0xf
        DB        ASR       [D1BW.1++],[D1BR.1++],#0xf
        DB        LSL       [D1BW.1++],[D1BR.1++],#0x10
        DB        LSR       [D1BW.1++],[D1BR.1++],#0x10
        DB        ASL       [D1BW.1++],[D1BR.1++],#0x10
        DB        ASR       [D1BW.1++],[D1BR.1++],#0x10
        DB        LSL       [D1BW.1++],[D1BR.1++],#0x1f
        DB        LSR       [D1BW.1++],[D1BR.1++],#0x1f
        DB        ASL       [D1BW.1++],[D1BR.1++],#0x1f
        DB        ASR       [D1BW.1++],[D1BR.1++],#0x1f
        DB        LSL       [D1BW.1++],[D1BR.1+D1BRI.1++],#0xf
        DB        LSR       [D1BW.1++],[D1BR.1+D1BRI.1++],#0xf
        DB        ASL       [D1BW.1++],[D1BR.1+D1BRI.1++],#0xf
        DB        ASR       [D1BW.1++],[D1BR.1+D1BRI.1++],#0xf
        DB        LSL       [D1BW.1++],[D1BR.1+D1BRI.1++],#0x10
        DB        LSR       [D1BW.1++],[D1BR.1+D1BRI.1++],#0x10
        DB        ASL       [D1BW.1++],[D1BR.1+D1BRI.1++],#0x10
        DB        ASR       [D1BW.1++],[D1BR.1+D1BRI.1++],#0x10
        DB        LSL       [D1BW.1++],[D1BR.1+D1BRI.1++],#0x1f
        DB        LSR       [D1BW.1++],[D1BR.1+D1BRI.1++],#0x1f
        DB        ASL       [D1BW.1++],[D1BR.1+D1BRI.1++],#0x1f
        DB        ASR       [D1BW.1++],[D1BR.1+D1BRI.1++],#0x1f
        DB        LSL       [D1BW.1+D1BWI.1++],D1Re0,#0xf
        DB        LSR       [D1BW.1+D1BWI.1++],D1Re0,#0xf
        DB        ASL       [D1BW.1+D1BWI.1++],D1Re0,#0xf
        DB        ASR       [D1BW.1+D1BWI.1++],D1Re0,#0xf
        DB        LSL       [D1BW.1+D1BWI.1++],D1Re0,#0x10
        DB        LSR       [D1BW.1+D1BWI.1++],D1Re0,#0x10
        DB        ASL       [D1BW.1+D1BWI.1++],D1Re0,#0x10
        DB        ASR       [D1BW.1+D1BWI.1++],D1Re0,#0x10
        DB        LSL       [D1BW.1+D1BWI.1++],D1Re0,#0x1f
        DB        LSR       [D1BW.1+D1BWI.1++],D1Re0,#0x1f
        DB        ASL       [D1BW.1+D1BWI.1++],D1Re0,#0x1f
        DB        ASR       [D1BW.1+D1BWI.1++],D1Re0,#0x1f
        DB        LSL       [D1BW.1+D1BWI.1++],D1.7,#0xf
        DB        LSR       [D1BW.1+D1BWI.1++],D1.7,#0xf
        DB        ASL       [D1BW.1+D1BWI.1++],D1.7,#0xf
        DB        ASR       [D1BW.1+D1BWI.1++],D1.7,#0xf
        DB        LSL       [D1BW.1+D1BWI.1++],D1.7,#0x10
        DB        LSR       [D1BW.1+D1BWI.1++],D1.7,#0x10
        DB        ASL       [D1BW.1+D1BWI.1++],D1.7,#0x10
        DB        ASR       [D1BW.1+D1BWI.1++],D1.7,#0x10
        DB        LSL       [D1BW.1+D1BWI.1++],D1.7,#0x1f
        DB        LSR       [D1BW.1+D1BWI.1++],D1.7,#0x1f
        DB        ASL       [D1BW.1+D1BWI.1++],D1.7,#0x1f
        DB        ASR       [D1BW.1+D1BWI.1++],D1.7,#0x1f
        DB        LSL       [D1BW.1+D1BWI.1++],AC1.0,#0xf
        DL        LSR       [D1BW.1+D1BWI.1++],AC1.0,#0xf
        DL        ASL       [D1BW.1+D1BWI.1++],AC1.0,#0xf
        DL        ASR       [D1BW.1+D1BWI.1++],AC1.0,#0xf
        DLR       LSL       [D1BW.1+D1BWI.1++],AC1.0,#0x10
        DLR       LSR       [D1BW.1+D1BWI.1++],AC1.0,#0x10
        DLR       ASL       [D1BW.1+D1BWI.1++],AC1.0,#0x10
        DLR       ASR       [D1BW.1+D1BWI.1++],AC1.0,#0x10
        DLG       LSL       [D1BW.1+D1BWI.1++],AC1.0,#0x1f
        DLG       LSR       [D1BW.1+D1BWI.1++],AC1.0,#0x1f
        DLG       ASL       [D1BW.1+D1BWI.1++],AC1.0,#0x1f
        DLG       ASR       [D1BW.1+D1BWI.1++],AC1.0,#0x1f
        DLG       LSL       [D1BW.1+D1BWI.1++],[D1BR.1],#0xf
        DLG       LSR       [D1BW.1+D1BWI.1++],[D1BR.1],#0xf
        DLG       ASL       [D1BW.1+D1BWI.1++],[D1BR.1],#0xf
        DLG       ASR       [D1BW.1+D1BWI.1++],[D1BR.1],#0xf
        DLG       LSL       [D1BW.1+D1BWI.1++],[D1BR.1],#0x10
        DLG       LSR       [D1BW.1+D1BWI.1++],[D1BR.1],#0x10
        DLG       ASL       [D1BW.1+D1BWI.1++],[D1BR.1],#0x10
        DLG       ASR       [D1BW.1+D1BWI.1++],[D1BR.1],#0x10
        DLG       LSL       [D1BW.1+D1BWI.1++],[D1BR.1],#0x1f
        DLG       LSR       [D1BW.1+D1BWI.1++],[D1BR.1],#0x1f
        DLG       ASL       [D1BW.1+D1BWI.1++],[D1BR.1],#0x1f
        DLG       ASR       [D1BW.1+D1BWI.1++],[D1BR.1],#0x1f
        DLG       LSL       [D1BW.1+D1BWI.1++],[D1BR.1++],#0xf
        DLG       LSR       [D1BW.1+D1BWI.1++],[D1BR.1++],#0xf
        DLG       ASL       [D1BW.1+D1BWI.1++],[D1BR.1++],#0xf
        DLG       ASR       [D1BW.1+D1BWI.1++],[D1BR.1++],#0xf
        DLG       LSL       [D1BW.1+D1BWI.1++],[D1BR.1++],#0x10
        DLG       LSR       [D1BW.1+D1BWI.1++],[D1BR.1++],#0x10
        DLG       ASL       [D1BW.1+D1BWI.1++],[D1BR.1++],#0x10
        DLG       ASR       [D1BW.1+D1BWI.1++],[D1BR.1++],#0x10
        DLG       LSL       [D1BW.1+D1BWI.1++],[D1BR.1++],#0x1f
        DLG       LSR       [D1BW.1+D1BWI.1++],[D1BR.1++],#0x1f
        DLG       ASL       [D1BW.1+D1BWI.1++],[D1BR.1++],#0x1f
        DLG       ASR       [D1BW.1+D1BWI.1++],[D1BR.1++],#0x1f
        DLG       LSL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0xf
        DLG       LSR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0xf
        DLG       ASL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0xf
        DLG       ASR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0xf
        DLG       LSL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x10
        DLG       LSR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x10
        DLG       ASL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x10
        DLG       ASR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x10
        DLG       LSL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x1f
        DLG       LSR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x1f
        DLG       ASL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x1f
        DLG       ASR       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x1f
        D         LSL       TTEXEC,D0Re0,D0Re0
        DLNE      ASLNE     D0Re0,D0Re0,D0Re0
        D         ASRNV     D0Re0,D0Re0,D0Re0
        D         LSL       TXENABLE,D0Re0,D0.7
        D         LSR       D1Re0,D0Re0,D0.7
        D         ASL       PC,D0Re0,D0.7
        D         ASR       TXSTAT,D0Re0,D0.7
        D         LSL       TTEXEC,D0Re0,[D0AR.0]
        DLNE      ASLNE     D0Re0,D0Re0,[D0AR.0]
        D         ASRNV     D0Re0,D0Re0,[D0AR.0]
        D         LSL       TXENABLE,D0Re0,[D0AR.0++]
        D         LSR       D1Re0,D0Re0,[D0AR.0++]
        D         ASL       PC,D0Re0,[D0AR.0++]
        D         ASR       TXSTAT,D0Re0,[D0AR.0++]
        D         LSL       TTEXEC,D0Re0,[D0AR.0+D0ARI.0++]
        DLNE      ASLNE     D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASRNV     D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSL       TXENABLE,D0.7,D0Re0
        D         LSR       D1Re0,D0.7,D0Re0
        D         ASL       PC,D0.7,D0Re0
        D         ASR       TXSTAT,D0.7,D0Re0
        D         LSL       TTEXEC,D0.7,D0.7
        DLNE      ASLNE     D0Re0,D0.7,D0.7
        D         ASRNV     D0Re0,D0.7,D0.7
        D         LSL       TXENABLE,D0.7,[D0AR.0]
        D         LSR       D1Re0,D0.7,[D0AR.0]
        D         ASL       PC,D0.7,[D0AR.0]
        D         ASR       TXSTAT,D0.7,[D0AR.0]
        DECS      LSRLE     D0Re0,D0.7,[D0AR.0++]
        D         ASL       TXENABLE,D0.7,[D0AR.0++]
        D         ASR       PC,D0.7,[D0AR.0++]
        DECS      LSRLE     D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        D         ASL       TXENABLE,D0.7,[D0AR.0+D0ARI.0++]
        D         ASR       PC,D0.7,[D0AR.0+D0ARI.0++]
        D         LSL       D1Re0,[D0AR.0],D0Re0
        D         LSRNV     D0Re0,[D0AR.0],D0Re0
        D         LSR       TXSTAT,[D0AR.0],D0Re0
        D         ASL       D1Re0,[D0AR.0],D0Re0
        D         ASRNV     D0Re0,[D0AR.0],D0Re0
        D         ASR       TXSTAT,[D0AR.0],D0Re0
        D         LSL       D1Re0,[D0AR.0],D0.7
        D         LSRNV     D0Re0,[D0AR.0],D0.7
        D         LSR       TXSTAT,[D0AR.0],D0.7
        D         ASL       D1Re0,[D0AR.0],D0.7
        D         ASRNV     D0Re0,[D0AR.0],D0.7
        D         ASR       TXSTAT,[D0AR.0],D0.7
        D         LSL       D1Re0,[D0AR.0],[D0AR.0]
        D         LSRNV     D0Re0,[D0AR.0],[D0AR.0]
        D         LSR       TXSTAT,[D0AR.0],[D0AR.0]
        D         ASL       D1Re0,[D0AR.0],[D0AR.0]
        D         ASRNV     D0Re0,[D0AR.0],[D0AR.0]
        D         ASR       TXSTAT,[D0AR.0],[D0AR.0]
        D         LSL       D1Re0,[D0AR.0],[D0AR.0++]
        D         LSRNV     D0Re0,[D0AR.0],[D0AR.0++]
        D         LSR       TXSTAT,[D0AR.0],[D0AR.0++]
        D         ASL       D1Re0,[D0AR.0],[D0AR.0++]
        D         ASRNV     D0Re0,[D0AR.0],[D0AR.0++]
        D         ASR       TXSTAT,[D0AR.0],[D0AR.0++]
        D         LSL       D1Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSRNV     D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSR       TXSTAT,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASL       D1Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASRNV     D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASR       TXSTAT,[D0AR.0],[D0AR.0+D0ARI.0++]
        DECS      LSRLE     D0Re0,[D0AR.0++],D0Re0
        D         ASL       TXENABLE,[D0AR.0++],D0Re0
        D         ASR       PC,[D0AR.0++],D0Re0
        DECS      LSRLE     D0Re0,[D0AR.0++],D0.7
        D         ASL       TXENABLE,[D0AR.0++],D0.7
        D         ASR       PC,[D0AR.0++],D0.7
        DECS      LSRLE     D0Re0,[D0AR.0++],[D0AR.0]
        D         ASL       TXENABLE,[D0AR.0++],[D0AR.0]
        D         ASR       PC,[D0AR.0++],[D0AR.0]
        DECS      LSRLE     D0Re0,[D0AR.0++],[D0AR.0++]
        D         ASL       TXENABLE,[D0AR.0++],[D0AR.0++]
        D         ASR       PC,[D0AR.0++],[D0AR.0++]
        DECS      LSRLE     D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASL       TXENABLE,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASR       PC,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DECS      LSRLE     D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         ASL       TXENABLE,[D0AR.0+D0ARI.0++],D0Re0
        D         ASR       PC,[D0AR.0+D0ARI.0++],D0Re0
        DECS      LSRLE     D0Re0,[D0AR.0+D0ARI.0++],D0.7
        D         ASL       TXENABLE,[D0AR.0+D0ARI.0++],D0.7
        D         ASR       PC,[D0AR.0+D0ARI.0++],D0.7
        DECS      LSRLE     D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASL       TXENABLE,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASR       PC,[D0AR.0+D0ARI.0++],[D0AR.0]
        DECS      LSRLE     D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASL       TXENABLE,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASR       PC,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DECS      LSRLE     D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASL       TXENABLE,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASR       PC,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASL       A0FrP,D0Re0,D0Re0
        D         LSL       PCX,D0Re0,D0.7
        D         ASR       A1LbP,D0Re0,D0.7
        D         ASL       A0FrP,D0Re0,[D0AR.0]
        D         LSL       PCX,D0Re0,[D0AR.0++]
        D         ASR       A1LbP,D0Re0,[D0AR.0++]
        D         ASL       A0FrP,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSL       PCX,D0.7,D0Re0
        D         ASR       A1LbP,D0.7,D0Re0
        D         ASL       A0FrP,D0.7,D0.7
        D         LSL       PCX,D0.7,[D0AR.0]
        D         ASR       A1LbP,D0.7,[D0AR.0]
        D         ASL       A0FrP,D0.7,[D0AR.0++]
        D         LSL       PCX,D0.7,[D0AR.0+D0ARI.0++]
        D         ASR       A1LbP,D0.7,[D0AR.0+D0ARI.0++]
        D         LSR       A0FrP,[D0AR.0],D0Re0
        D         ASL       A1LbP,[D0AR.0],D0Re0
        D         ASR       PCX,[D0AR.0],D0Re0
        D         LSR       A0FrP,[D0AR.0],D0.7
        D         ASL       A1LbP,[D0AR.0],D0.7
        D         ASR       PCX,[D0AR.0],D0.7
        D         LSR       A0FrP,[D0AR.0],[D0AR.0]
        D         ASL       A1LbP,[D0AR.0],[D0AR.0]
        D         ASR       PCX,[D0AR.0],[D0AR.0]
        D         LSR       A0FrP,[D0AR.0],[D0AR.0++]
        D         ASL       A1LbP,[D0AR.0],[D0AR.0++]
        D         ASR       PCX,[D0AR.0],[D0AR.0++]
        D         LSR       A0FrP,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASL       A1LbP,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASR       PCX,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASL       A0FrP,[D0AR.0++],D0Re0
        D         LSL       PCX,[D0AR.0++],D0.7
        D         ASR       A1LbP,[D0AR.0++],D0.7
        D         ASL       A0FrP,[D0AR.0++],[D0AR.0]
        D         LSL       PCX,[D0AR.0++],[D0AR.0++]
        D         ASR       A1LbP,[D0AR.0++],[D0AR.0++]
        D         ASL       A0FrP,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSL       PCX,[D0AR.0+D0ARI.0++],D0Re0
        D         ASR       A1LbP,[D0AR.0+D0ARI.0++],D0Re0
        D         ASL       A0FrP,[D0AR.0+D0ARI.0++],D0.7
        D         LSL       PCX,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASR       A1LbP,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASL       A0FrP,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSL       PCX,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASR       A1LbP,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASR       TXMASKI,D0Re0,D0.7
        D         ASR       TXMASKI,D0Re0,[D0AR.0++]
        D         ASR       TXMASKI,D0.7,D0Re0
        D         ASR       TXMASKI,D0.7,[D0AR.0]
        D         ASR       TXMASKI,D0.7,[D0AR.0+D0ARI.0++]
        D         ASR       TXMASKI,AC0.0,D0Re0
        D         ASR       TXMASKI,AC0.0,D0.7
        D         ASR       TXMASKI,AC0.0,[D0AR.0]
        D         ASR       TXMASKI,AC0.0,[D0AR.0++]
        D         ASR       TXMASKI,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASR       TXMASKI,[D0AR.0++],D0.7
        D         ASR       TXMASKI,[D0AR.0++],[D0AR.0++]
        D         ASR       TXMASKI,[D0AR.0+D0ARI.0++],D0Re0
        D         ASR       TXMASKI,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASR       TXMASKI,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSRNV     D0.7,D0Re0,D0Re0
        D         ASL       A0.7,D0Re0,D0Re0
        D         ASR       TXGPIOO,D0Re0,D0Re0
        D         LSRNV     D0.7,D0Re0,D0.7
        D         ASL       A0.7,D0Re0,D0.7
        D         ASR       TXGPIOO,D0Re0,D0.7
        D         LSRNV     D0.7,D0Re0,[D0AR.0]
        D         ASL       A0.7,D0Re0,[D0AR.0]
        D         ASR       TXGPIOO,D0Re0,[D0AR.0]
        D         LSRNV     D0.7,D0Re0,[D0AR.0++]
        D         ASL       A0.7,D0Re0,[D0AR.0++]
        D         ASR       TXGPIOO,D0Re0,[D0AR.0++]
        D         LSRNV     D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASL       A0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASR       TXGPIOO,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSRNV     D0.7,D0.7,D0Re0
        D         ASL       A0.7,D0.7,D0Re0
        D         ASR       TXGPIOO,D0.7,D0Re0
        D         LSRNV     D0.7,D0.7,D0.7
        D         ASL       A0.7,D0.7,D0.7
        D         ASR       TXGPIOO,D0.7,D0.7
        D         LSRNV     D0.7,D0.7,[D0AR.0]
        D         ASL       A0.7,D0.7,[D0AR.0]
        D         ASR       TXGPIOO,D0.7,[D0AR.0]
        D         LSRNV     D0.7,D0.7,[D0AR.0++]
        D         ASL       A0.7,D0.7,[D0AR.0++]
        D         ASR       TXGPIOO,D0.7,[D0AR.0++]
        D         LSRNV     D0.7,D0.7,[D0AR.0+D0ARI.0++]
        D         ASL       A0.7,D0.7,[D0AR.0+D0ARI.0++]
        D         ASR       TXGPIOO,D0.7,[D0AR.0+D0ARI.0++]
        D         LSL       A0.7,AC0.0,D0Re0
        D         LSRNV     D0.7,AC0.0,D0Re0
        D         LSR       TXGPIOO,AC0.0,D0Re0
        D         ASL       A0.7,AC0.0,D0Re0
        D         ASRNV     D0.7,AC0.0,D0Re0
        D         ASR       TXGPIOO,AC0.0,D0Re0
        D         LSL       A0.7,AC0.0,D0.7
        D         LSRNV     D0.7,AC0.0,D0.7
        D         LSR       TXGPIOO,AC0.0,D0.7
        D         ASL       A0.7,AC0.0,D0.7
        D         ASRNV     D0.7,AC0.0,D0.7
        D         ASR       TXGPIOO,AC0.0,D0.7
        D         LSL       A0.7,AC0.0,[D0AR.0]
        D         LSRNV     D0.7,AC0.0,[D0AR.0]
        D         LSR       TXGPIOO,AC0.0,[D0AR.0]
        D         ASL       A0.7,AC0.0,[D0AR.0]
        D         ASRNV     D0.7,AC0.0,[D0AR.0]
        D         ASR       TXGPIOO,AC0.0,[D0AR.0]
        D         LSL       A0.7,AC0.0,[D0AR.0++]
        D         LSRNV     D0.7,AC0.0,[D0AR.0++]
        D         LSR       TXGPIOO,AC0.0,[D0AR.0++]
        D         ASL       A0.7,AC0.0,[D0AR.0++]
        D         ASRNV     D0.7,AC0.0,[D0AR.0++]
        D         ASR       TXGPIOO,AC0.0,[D0AR.0++]
        D         LSL       A0.7,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSRNV     D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSR       TXGPIOO,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASL       A0.7,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASRNV     D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASR       TXGPIOO,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSRNV     D0.7,[D0AR.0++],D0Re0
        D         ASL       A0.7,[D0AR.0++],D0Re0
        D         ASR       TXGPIOO,[D0AR.0++],D0Re0
        D         LSRNV     D0.7,[D0AR.0++],D0.7
        D         ASL       A0.7,[D0AR.0++],D0.7
        D         ASR       TXGPIOO,[D0AR.0++],D0.7
        D         LSRNV     D0.7,[D0AR.0++],[D0AR.0]
        D         ASL       A0.7,[D0AR.0++],[D0AR.0]
        D         ASR       TXGPIOO,[D0AR.0++],[D0AR.0]
        D         LSRNV     D0.7,[D0AR.0++],[D0AR.0++]
        D         ASL       A0.7,[D0AR.0++],[D0AR.0++]
        D         ASR       TXGPIOO,[D0AR.0++],[D0AR.0++]
        D         LSRNV     D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASL       A0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASR       TXGPIOO,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSRNV     D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         ASL       A0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         ASR       TXGPIOO,[D0AR.0+D0ARI.0++],D0Re0
        D         LSRNV     D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         ASL       A0.7,[D0AR.0+D0ARI.0++],D0.7
        D         ASR       TXGPIOO,[D0AR.0+D0ARI.0++],D0.7
        D         LSRNV     D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASL       A0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASR       TXGPIOO,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSRNV     D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASL       A0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASR       TXGPIOO,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSRNV     D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASL       A0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASR       TXGPIOO,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASR       A1.8,D0Re0,D0Re0
        D         ASR       A1.8,D0Re0,D0.7
        D         ASR       A1.8,D0Re0,[D0AR.0]
        D         ASR       A1.8,D0Re0,[D0AR.0++]
        D         ASR       A1.8,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASR       A1.8,D0.7,D0Re0
        D         ASR       A1.8,D0.7,D0.7
        D         ASR       A1.8,D0.7,[D0AR.0]
        D         ASR       A1.8,D0.7,[D0AR.0++]
        D         ASR       A1.8,D0.7,[D0AR.0+D0ARI.0++]
        D         LSR       A1.8,AC0.0,D0Re0
        D         ASR       A1.8,AC0.0,D0Re0
        D         LSR       A1.8,AC0.0,D0.7
        D         ASR       A1.8,AC0.0,D0.7
        D         LSR       A1.8,AC0.0,[D0AR.0]
        D         ASR       A1.8,AC0.0,[D0AR.0]
        D         LSR       A1.8,AC0.0,[D0AR.0++]
        D         ASR       A1.8,AC0.0,[D0AR.0++]
        D         LSR       A1.8,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASR       A1.8,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASR       A1.8,[D0AR.0++],D0Re0
        D         ASR       A1.8,[D0AR.0++],D0.7
        D         ASR       A1.8,[D0AR.0++],[D0AR.0]
        D         ASR       A1.8,[D0AR.0++],[D0AR.0++]
        D         ASR       A1.8,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASR       A1.8,[D0AR.0+D0ARI.0++],D0Re0
        D         ASR       A1.8,[D0AR.0+D0ARI.0++],D0.7
        D         ASR       A1.8,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASR       A1.8,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASR       A1.8,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSL       RA,D0Re0,D0.7
        D         LSR       RA,D0Re0,[D0AR.0]
        DLLS      ASLLS     D0.16,D0Re0,[D0AR.0++]
        D         ASLNV     D0.16,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASL       D1.16,D0.7,D0Re0
        D         ASL       RA,D0.7,D0.7
        D         ASR       RA,D0.7,[D0AR.0]
        D         LSL       RA,D0.7,[D0AR.0+D0ARI.0++]
        D         LSLNV     D0.16,AC0.0,D0Re0
        D         LSR       RA,AC0.0,D0Re0
        D         ASL       RA,AC0.0,D0Re0
        D         LSL       D1.16,AC0.0,D0.7
        DLLS      ASLLS     D0.16,[D0AR.0],D0.7
        DECS      ASRLE     D0.16,AC0.0,D0.7
        D         LSL       RA,[D0AR.0],[D0AR.0]
        DHNE      ASLPL     D0.16,AC0.0,[D0AR.0]
        D         ASRNV     D0.16,AC0.0,[D0AR.0]
        D         LSL       RA,AC0.0,[D0AR.0++]
        D         ASLNV     D0.16,[D0AR.0],[D0AR.0++]
        D         ASR       D1.16,AC0.0,[D0AR.0++]
        DHNE      LSRPL     D0.16,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASLNV     D0.16,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASR       RA,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASR       RA,[D0AR.0++],D0Re0
        D         LSL       RA,[D0AR.0++],[D0AR.0]
        D         LSR       RA,[D0AR.0++],[D0AR.0++]
        DLNE      ASLNE     D0.16,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASLNV     D0.16,[D0AR.0+D0ARI.0++],D0Re0
        D         ASL       D1.16,[D0AR.0+D0ARI.0++],D0.7
        D         ASL       RA,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASR       RA,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSL       RAM8X32,D0Re0,D0Re0
        D         LSL       RAM8X32,D0Re0,[D0AR.0]
        D         LSL       RAM8X32,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSL       RAM8X32,D0.7,D0.7
        D         LSL       RAM8X32,D0.7,[D0AR.0++]
        D         LSL       RAM8X32,[D0AR.0],D0Re0
        D         LSL       RAM8X32,[D0AR.0],D0.7
        D         LSL       RAM8X32,[D0AR.0],[D0AR.0]
        D         LSL       RAM8X32,[D0AR.0],[D0AR.0++]
        D         LSL       RAM8X32,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSL       RAM8X32,[D0AR.0++],D0Re0
        D         LSL       RAM8X32,[D0AR.0++],[D0AR.0]
        D         LSL       RAM8X32,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSL       RAM8X32,[D0AR.0+D0ARI.0++],D0.7
        D         LSL       RAM8X32,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSL       RAM8X,D0Re0,D0Re0
        D         LSL       RAM8X,D0Re0,[D0AR.0]
        D         LSL       RAM8X,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSL       RAM8X,D0.7,D0.7
        D         LSL       RAM8X,D0.7,[D0AR.0++]
        D         LSL       RAM8X,[D0AR.0],D0Re0
        D         LSL       RAM8X,[D0AR.0],D0.7
        D         LSL       RAM8X,[D0AR.0],[D0AR.0]
        D         LSL       RAM8X,[D0AR.0],[D0AR.0++]
        D         LSL       RAM8X,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSL       RAM8X,[D0AR.0++],D0Re0
        D         LSL       RAM8X,[D0AR.0++],[D0AR.0]
        D         LSL       RAM8X,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSL       RAM8X,[D0AR.0+D0ARI.0++],D0.7
        D         LSL       RAM8X,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSL       RABZ,D0Re0,D0Re0
        D         LSL       RABZ,D0Re0,[D0AR.0]
        D         LSL       RABZ,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSL       RABZ,D0.7,D0.7
        D         LSL       RABZ,D0.7,[D0AR.0++]
        D         LSL       RABZ,[D0AR.0],D0Re0
        D         LSL       RABZ,[D0AR.0],D0.7
        D         LSL       RABZ,[D0AR.0],[D0AR.0]
        D         LSL       RABZ,[D0AR.0],[D0AR.0++]
        D         LSL       RABZ,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSL       RABZ,[D0AR.0++],D0Re0
        D         LSL       RABZ,[D0AR.0++],[D0AR.0]
        D         LSL       RABZ,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSL       RABZ,[D0AR.0+D0ARI.0++],D0.7
        D         LSL       RABZ,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSL       RAWX,D0Re0,D0Re0
        D         LSL       RAWX,D0Re0,[D0AR.0]
        D         LSL       RAWX,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSL       RAWX,D0.7,D0.7
        D         LSL       RAWX,D0.7,[D0AR.0++]
        D         LSL       RAWX,[D0AR.0],D0Re0
        D         LSL       RAWX,[D0AR.0],D0.7
        D         LSL       RAWX,[D0AR.0],[D0AR.0]
        D         LSL       RAWX,[D0AR.0],[D0AR.0++]
        D         LSL       RAWX,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSL       RAWX,[D0AR.0++],D0Re0
        D         LSL       RAWX,[D0AR.0++],[D0AR.0]
        D         LSL       RAWX,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSL       RAWX,[D0AR.0+D0ARI.0++],D0.7
        D         LSL       RAWX,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSL       TXIDLECYC,D0Re0,D0Re0
        D         LSL       TXIDLECYC,D0Re0,D0.7
        D         LSL       TXIDLECYC,D0Re0,[D0AR.0]
        D         LSL       TXIDLECYC,D0Re0,[D0AR.0++]
        D         LSL       TXIDLECYC,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSL       TXIDLECYC,D0.7,D0Re0
        D         LSL       TXIDLECYC,D0.7,D0.7
        D         LSL       TXIDLECYC,D0.7,[D0AR.0]
        D         LSL       TXIDLECYC,D0.7,[D0AR.0++]
        D         LSL       TXIDLECYC,D0.7,[D0AR.0+D0ARI.0++]
        D         LSL       TXIDLECYC,[D0AR.0],D0Re0
        D         ASL       TXIDLECYC,[D0AR.0],D0Re0
        D         LSL       TXIDLECYC,[D0AR.0],D0.7
        D         ASL       TXIDLECYC,[D0AR.0],D0.7
        D         LSL       TXIDLECYC,[D0AR.0],[D0AR.0]
        D         ASL       TXIDLECYC,[D0AR.0],[D0AR.0]
        D         LSL       TXIDLECYC,[D0AR.0],[D0AR.0++]
        D         ASL       TXIDLECYC,[D0AR.0],[D0AR.0++]
        D         LSL       TXIDLECYC,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASL       TXIDLECYC,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSL       TXIDLECYC,[D0AR.0++],D0Re0
        D         LSL       TXIDLECYC,[D0AR.0++],D0.7
        D         LSL       TXIDLECYC,[D0AR.0++],[D0AR.0]
        D         LSL       TXIDLECYC,[D0AR.0++],[D0AR.0++]
        D         LSL       TXIDLECYC,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSL       TXIDLECYC,[D0AR.0+D0ARI.0++],D0Re0
        D         LSL       TXIDLECYC,[D0AR.0+D0ARI.0++],D0.7
        D         LSL       TXIDLECYC,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSL       TXIDLECYC,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSL       TXIDLECYC,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLLS      LSLLS     D1Re0,D1Re0,D1Re0
        D         LSR       TXENABLE,D1Re0,D1Re0
        D         ASL       PC,D1Re0,D1Re0
        DLLS      LSLLS     D1Re0,D1Re0,D1.7
        D         LSR       TXENABLE,D1Re0,D1.7
        D         ASL       PC,D1Re0,D1.7
        DLLS      LSLLS     D1Re0,D1Re0,[D1BR.1]
        D         LSR       TXENABLE,D1Re0,[D1BR.1]
        D         ASL       PC,D1Re0,[D1BR.1]
        DLLS      LSLLS     D1Re0,D1Re0,[D1BR.1++]
        D         LSR       TXENABLE,D1Re0,[D1BR.1++]
        D         ASL       PC,D1Re0,[D1BR.1++]
        DLLS      LSLLS     D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSR       TXENABLE,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASL       PC,D1Re0,[D1BR.1+D1BRI.1++]
        DLLS      LSLLS     D1Re0,D1.7,D1Re0
        D         LSR       TXENABLE,D1.7,D1Re0
        D         ASL       PC,D1.7,D1Re0
        DLLS      LSLLS     D1Re0,D1.7,D1.7
        D         LSR       TXENABLE,D1.7,D1.7
        D         ASL       PC,D1.7,D1.7
        DLLS      LSLLS     D1Re0,D1.7,[D1BR.1]
        D         LSR       TXENABLE,D1.7,[D1BR.1]
        D         ASL       PC,D1.7,[D1BR.1]
        DLLS      LSLLS     D1Re0,D1.7,[D1BR.1++]
        D         LSR       TXENABLE,D1.7,[D1BR.1++]
        D         ASL       PC,D1.7,[D1BR.1++]
        DLLS      LSLLS     D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        D         LSR       TXENABLE,D1.7,[D1BR.1+D1BRI.1++]
        D         ASL       PC,D1.7,[D1BR.1+D1BRI.1++]
        DECS      LSLLE     D1Re0,AC1.0,D1Re0
        D         LSR       TXENABLE,AC1.0,D1Re0
        D         ASL       PC,AC1.0,D1Re0
        DECS      LSLLE     D1Re0,AC1.0,D1.7
        D         LSR       TXENABLE,AC1.0,D1.7
        D         ASL       PC,AC1.0,D1.7
        DECS      LSLLE     D1Re0,AC1.0,[D1BR.1]
        D         LSR       TXENABLE,AC1.0,[D1BR.1]
        D         ASL       PC,AC1.0,[D1BR.1]
        DECS      LSLLE     D1Re0,AC1.0,[D1BR.1++]
        D         LSR       TXENABLE,AC1.0,[D1BR.1++]
        D         ASL       PC,AC1.0,[D1BR.1++]
        DECS      LSLLE     D1Re0,AC1.0,[D1BR.1+D1BRI.1++]
        D         LSR       TXENABLE,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASL       PC,AC1.0,[D1BR.1+D1BRI.1++]
        DLLS      LSLLS     D1Re0,[D1BR.1],D1Re0
        D         LSR       TXENABLE,[D1BR.1],D1Re0
        D         ASL       PC,[D1BR.1],D1Re0
        DLLS      LSLLS     D1Re0,[D1BR.1],D1.7
        D         LSR       TXENABLE,[D1BR.1],D1.7
        D         ASL       PC,[D1BR.1],D1.7
        DLLS      LSLLS     D1Re0,[D1BR.1],[D1BR.1]
        D         LSR       TXENABLE,[D1BR.1],[D1BR.1]
        D         ASL       PC,[D1BR.1],[D1BR.1]
        DLLS      LSLLS     D1Re0,[D1BR.1],[D1BR.1++]
        D         LSR       TXENABLE,[D1BR.1],[D1BR.1++]
        D         ASL       PC,[D1BR.1],[D1BR.1++]
        DLLS      LSLLS     D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSR       TXENABLE,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASL       PC,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLLS      LSLLS     D1Re0,[D1BR.1++],D1Re0
        D         LSR       TXENABLE,[D1BR.1++],D1Re0
        D         ASL       PC,[D1BR.1++],D1Re0
        DLLS      LSLLS     D1Re0,[D1BR.1++],D1.7
        D         LSR       TXENABLE,[D1BR.1++],D1.7
        D         ASL       PC,[D1BR.1++],D1.7
        DLLS      LSLLS     D1Re0,[D1BR.1++],[D1BR.1]
        D         LSR       TXENABLE,[D1BR.1++],[D1BR.1]
        D         ASL       PC,[D1BR.1++],[D1BR.1]
        DLLS      LSLLS     D1Re0,[D1BR.1++],[D1BR.1++]
        D         LSR       TXENABLE,[D1BR.1++],[D1BR.1++]
        D         ASL       PC,[D1BR.1++],[D1BR.1++]
        DLLS      LSLLS     D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSR       TXENABLE,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASL       PC,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLLS      LSLLS     D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         LSR       TXENABLE,[D1BR.1+D1BRI.1++],D1Re0
        D         ASL       PC,[D1BR.1+D1BRI.1++],D1Re0
        DLLS      LSLLS     D1Re0,[D1BR.1+D1BRI.1++],D1.7
        D         LSR       TXENABLE,[D1BR.1+D1BRI.1++],D1.7
        D         ASL       PC,[D1BR.1+D1BRI.1++],D1.7
        DLLS      LSLLS     D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSR       TXENABLE,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASL       PC,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLLS      LSLLS     D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSR       TXENABLE,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASL       PC,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLLS      LSLLS     D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSR       TXENABLE,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASL       PC,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSL       A0FrP,D1Re0,D1Re0
        D         ASL       PCX,D1Re0,D1Re0
        D         LSR       A1LbP,D1Re0,D1.7
        D         LSL       A0FrP,D1Re0,[D1BR.1]
        D         ASL       PCX,D1Re0,[D1BR.1]
        D         LSR       A1LbP,D1Re0,[D1BR.1++]
        D         LSL       A0FrP,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASL       PCX,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSR       A1LbP,D1.7,D1Re0
        D         LSL       A0FrP,D1.7,D1.7
        D         ASL       PCX,D1.7,D1.7
        D         LSR       A1LbP,D1.7,[D1BR.1]
        D         LSL       A0FrP,D1.7,[D1BR.1++]
        D         ASL       PCX,D1.7,[D1BR.1++]
        D         LSR       A1LbP,D1.7,[D1BR.1+D1BRI.1++]
        D         LSL       A0FrP,AC1.0,D1Re0
        D         ASL       PCX,AC1.0,D1Re0
        D         LSR       A1LbP,AC1.0,D1.7
        D         LSL       A0FrP,AC1.0,[D1BR.1]
        D         ASL       PCX,AC1.0,[D1BR.1]
        D         LSR       A1LbP,AC1.0,[D1BR.1++]
        D         LSL       A0FrP,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASL       PCX,AC1.0,[D1BR.1+D1BRI.1++]
        D         LSR       A1LbP,[D1BR.1],D1Re0
        D         LSL       A0FrP,[D1BR.1],D1.7
        D         ASL       PCX,[D1BR.1],D1.7
        D         LSR       A1LbP,[D1BR.1],[D1BR.1]
        D         LSL       A0FrP,[D1BR.1],[D1BR.1++]
        D         ASL       PCX,[D1BR.1],[D1BR.1++]
        D         LSR       A1LbP,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSL       A0FrP,[D1BR.1++],D1Re0
        D         ASL       PCX,[D1BR.1++],D1Re0
        D         LSR       A1LbP,[D1BR.1++],D1.7
        D         LSL       A0FrP,[D1BR.1++],[D1BR.1]
        D         ASL       PCX,[D1BR.1++],[D1BR.1]
        D         LSR       A1LbP,[D1BR.1++],[D1BR.1++]
        D         LSL       A0FrP,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASL       PCX,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSR       A1LbP,[D1BR.1+D1BRI.1++],D1Re0
        D         LSL       A0FrP,[D1BR.1+D1BRI.1++],D1.7
        D         ASL       PCX,[D1BR.1+D1BRI.1++],D1.7
        D         LSR       A1LbP,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSL       A0FrP,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASL       PCX,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSR       A1LbP,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSR       TXMASKI,D1Re0,D1Re0
        D         LSR       TXMASKI,D1Re0,[D1BR.1]
        D         LSR       TXMASKI,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSR       TXMASKI,D1.7,D1.7
        D         LSR       TXMASKI,D1.7,[D1BR.1++]
        D         LSR       TXMASKI,AC1.0,D1Re0
        D         LSR       TXMASKI,AC1.0,[D1BR.1]
        D         LSR       TXMASKI,AC1.0,[D1BR.1+D1BRI.1++]
        D         LSR       TXMASKI,[D1BR.1],D1.7
        D         LSR       TXMASKI,[D1BR.1],[D1BR.1++]
        D         LSR       TXMASKI,[D1BR.1++],D1Re0
        D         LSR       TXMASKI,[D1BR.1++],[D1BR.1]
        D         LSR       TXMASKI,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSR       TXMASKI,[D1BR.1+D1BRI.1++],D1.7
        D         LSR       TXMASKI,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSLNV     D1.7,D1Re0,D1Re0
        D         LSR       A0.7,D1Re0,D1Re0
        D         ASL       TXGPIOO,D1Re0,D1Re0
        D         LSLNV     D1.7,D1Re0,D1.7
        D         LSR       A0.7,D1Re0,D1.7
        D         ASL       TXGPIOO,D1Re0,D1.7
        D         LSLNV     D1.7,D1Re0,[D1BR.1]
        D         LSR       A0.7,D1Re0,[D1BR.1]
        D         ASL       TXGPIOO,D1Re0,[D1BR.1]
        D         LSLNV     D1.7,D1Re0,[D1BR.1++]
        D         LSR       A0.7,D1Re0,[D1BR.1++]
        D         ASL       TXGPIOO,D1Re0,[D1BR.1++]
        D         LSLNV     D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSR       A0.7,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASL       TXGPIOO,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSLNV     D1.7,D1.7,D1Re0
        D         LSR       A0.7,D1.7,D1Re0
        D         ASL       TXGPIOO,D1.7,D1Re0
        D         LSLNV     D1.7,D1.7,D1.7
        D         LSR       A0.7,D1.7,D1.7
        D         ASL       TXGPIOO,D1.7,D1.7
        D         LSLNV     D1.7,D1.7,[D1BR.1]
        D         LSR       A0.7,D1.7,[D1BR.1]
        D         ASL       TXGPIOO,D1.7,[D1BR.1]
        D         LSLNV     D1.7,D1.7,[D1BR.1++]
        D         LSR       A0.7,D1.7,[D1BR.1++]
        D         ASL       TXGPIOO,D1.7,[D1BR.1++]
        D         LSLNV     D1.7,D1.7,[D1BR.1+D1BRI.1++]
        D         LSR       A0.7,D1.7,[D1BR.1+D1BRI.1++]
        D         ASL       TXGPIOO,D1.7,[D1BR.1+D1BRI.1++]
        D         LSLNV     D1.7,AC1.0,D1Re0
        D         LSR       A0.7,AC1.0,D1Re0
        D         ASL       TXGPIOO,AC1.0,D1Re0
        D         LSLNV     D1.7,AC1.0,D1.7
        D         LSR       A0.7,AC1.0,D1.7
        D         ASL       TXGPIOO,AC1.0,D1.7
        D         LSLNV     D1.7,AC1.0,[D1BR.1]
        D         LSR       A0.7,AC1.0,[D1BR.1]
        D         ASL       TXGPIOO,AC1.0,[D1BR.1]
        D         LSLNV     D1.7,AC1.0,[D1BR.1++]
        D         LSR       A0.7,AC1.0,[D1BR.1++]
        D         ASL       TXGPIOO,AC1.0,[D1BR.1++]
        D         LSLNV     D1.7,AC1.0,[D1BR.1+D1BRI.1++]
        D         LSR       A0.7,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASL       TXGPIOO,AC1.0,[D1BR.1+D1BRI.1++]
        D         LSLNV     D1.7,[D1BR.1],D1Re0
        D         LSR       A0.7,[D1BR.1],D1Re0
        D         ASL       TXGPIOO,[D1BR.1],D1Re0
        D         LSLNV     D1.7,[D1BR.1],D1.7
        D         LSR       A0.7,[D1BR.1],D1.7
        D         ASL       TXGPIOO,[D1BR.1],D1.7
        D         LSLNV     D1.7,[D1BR.1],[D1BR.1]
        D         LSR       A0.7,[D1BR.1],[D1BR.1]
        D         ASL       TXGPIOO,[D1BR.1],[D1BR.1]
        D         LSLNV     D1.7,[D1BR.1],[D1BR.1++]
        D         LSR       A0.7,[D1BR.1],[D1BR.1++]
        D         ASL       TXGPIOO,[D1BR.1],[D1BR.1++]
        D         LSLNV     D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSR       A0.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASL       TXGPIOO,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSLNV     D1.7,[D1BR.1++],D1Re0
        D         LSR       A0.7,[D1BR.1++],D1Re0
        D         ASL       TXGPIOO,[D1BR.1++],D1Re0
        D         LSLNV     D1.7,[D1BR.1++],D1.7
        D         LSR       A0.7,[D1BR.1++],D1.7
        D         ASL       TXGPIOO,[D1BR.1++],D1.7
        D         LSLNV     D1.7,[D1BR.1++],[D1BR.1]
        D         LSR       A0.7,[D1BR.1++],[D1BR.1]
        D         ASL       TXGPIOO,[D1BR.1++],[D1BR.1]
        D         LSLNV     D1.7,[D1BR.1++],[D1BR.1++]
        D         LSR       A0.7,[D1BR.1++],[D1BR.1++]
        D         ASL       TXGPIOO,[D1BR.1++],[D1BR.1++]
        D         LSLNV     D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSR       A0.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASL       TXGPIOO,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSLNV     D1.7,[D1BR.1+D1BRI.1++],D1Re0
        D         LSR       A0.7,[D1BR.1+D1BRI.1++],D1Re0
        D         ASL       TXGPIOO,[D1BR.1+D1BRI.1++],D1Re0
        D         LSLNV     D1.7,[D1BR.1+D1BRI.1++],D1.7
        D         LSR       A0.7,[D1BR.1+D1BRI.1++],D1.7
        D         ASL       TXGPIOO,[D1BR.1+D1BRI.1++],D1.7
        D         LSLNV     D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSR       A0.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASL       TXGPIOO,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSLNV     D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSR       A0.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASL       TXGPIOO,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSLNV     D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSR       A0.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASL       TXGPIOO,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSL       A1.8,D1Re0,D1Re0
        D         LSL       A1.8,D1Re0,D1.7
        D         LSL       A1.8,D1Re0,[D1BR.1]
        D         LSL       A1.8,D1Re0,[D1BR.1++]
        D         LSL       A1.8,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSL       A1.8,D1.7,D1Re0
        D         LSL       A1.8,D1.7,D1.7
        D         LSL       A1.8,D1.7,[D1BR.1]
        D         LSL       A1.8,D1.7,[D1BR.1++]
        D         LSL       A1.8,D1.7,[D1BR.1+D1BRI.1++]
        D         LSL       A1.8,AC1.0,D1Re0
        D         LSL       A1.8,AC1.0,D1.7
        D         LSL       A1.8,AC1.0,[D1BR.1]
        D         LSL       A1.8,AC1.0,[D1BR.1++]
        D         LSL       A1.8,AC1.0,[D1BR.1+D1BRI.1++]
        D         LSL       A1.8,[D1BR.1],D1Re0
        D         LSL       A1.8,[D1BR.1],D1.7
        D         LSL       A1.8,[D1BR.1],[D1BR.1]
        D         LSL       A1.8,[D1BR.1],[D1BR.1++]
        D         LSL       A1.8,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSL       A1.8,[D1BR.1++],D1Re0
        D         LSL       A1.8,[D1BR.1++],D1.7
        D         LSL       A1.8,[D1BR.1++],[D1BR.1]
        D         LSL       A1.8,[D1BR.1++],[D1BR.1++]
        D         LSL       A1.8,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSL       A1.8,[D1BR.1+D1BRI.1++],D1Re0
        D         LSL       A1.8,[D1BR.1+D1BRI.1++],D1.7
        D         LSL       A1.8,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSL       A1.8,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSL       A1.8,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSR       RA,D1Re0,D1Re0
        DLLS      ASLLS     D1.16,D1Re0,D1.7
        D         ASLNV     D1.16,D1Re0,[D1BR.1]
        D         ASL       D0.16,D1Re0,[D1BR.1++]
        D         ASL       RA,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASR       RA,D1.7,D1Re0
        D         LSL       RA,D1.7,[D1BR.1]
        D         LSR       RA,D1.7,[D1BR.1++]
        DLLS      ASLLS     D1.16,D1.7,[D1BR.1+D1BRI.1++]
        D         LSL       RA,AC1.0,D1Re0
        D         ASL       RA,AC1.0,D1Re0
        D         LSL       RA,AC1.0,D1.7
        D         ASL       RA,AC1.0,D1.7
        D         LSL       RA,AC1.0,[D1BR.1]
        D         ASL       RA,AC1.0,[D1BR.1]
        D         LSL       RA,AC1.0,[D1BR.1++]
        D         ASL       RA,AC1.0,[D1BR.1++]
        D         LSL       RA,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASL       RA,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASLNV     D1.16,[D1BR.1],D1Re0
        D         ASL       D0.16,[D1BR.1],D1.7
        D         ASL       RA,[D1BR.1],[D1BR.1]
        D         ASR       RA,[D1BR.1],[D1BR.1++]
        D         LSL       RA,[D1BR.1++],D1Re0
        D         LSR       RA,[D1BR.1++],D1.7
        DLLS      ASLLS     D1.16,[D1BR.1++],[D1BR.1]
        D         ASLNV     D1.16,[D1BR.1++],[D1BR.1++]
        D         ASL       D0.16,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASL       RA,[D1BR.1+D1BRI.1++],D1Re0
        D         ASR       RA,[D1BR.1+D1BRI.1++],D1.7
        D         LSL       RA,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSR       RA,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASL       RAM8X32,D1Re0,D1Re0
        D         ASL       RAM8X32,D1Re0,[D1BR.1]
        D         ASL       RAM8X32,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASL       RAM8X32,D1.7,D1.7
        D         ASL       RAM8X32,D1.7,[D1BR.1++]
        D         ASL       RAM8X32,AC1.0,D1Re0
        D         ASL       RAM8X32,AC1.0,[D1BR.1]
        D         ASL       RAM8X32,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASL       RAM8X32,[D1BR.1],D1.7
        D         ASL       RAM8X32,[D1BR.1],[D1BR.1++]
        D         ASL       RAM8X32,[D1BR.1++],D1Re0
        D         ASL       RAM8X32,[D1BR.1++],[D1BR.1]
        D         ASL       RAM8X32,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASL       RAM8X32,[D1BR.1+D1BRI.1++],D1.7
        D         ASL       RAM8X32,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASL       RAM8X,D1Re0,D1Re0
        D         ASL       RAM8X,D1Re0,[D1BR.1]
        D         ASL       RAM8X,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASL       RAM8X,D1.7,D1.7
        D         ASL       RAM8X,D1.7,[D1BR.1++]
        D         ASL       RAM8X,AC1.0,D1Re0
        D         ASL       RAM8X,AC1.0,[D1BR.1]
        D         ASL       RAM8X,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASL       RAM8X,[D1BR.1],D1.7
        D         ASL       RAM8X,[D1BR.1],[D1BR.1++]
        D         ASL       RAM8X,[D1BR.1++],D1Re0
        D         ASL       RAM8X,[D1BR.1++],[D1BR.1]
        D         ASL       RAM8X,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASL       RAM8X,[D1BR.1+D1BRI.1++],D1.7
        D         ASL       RAM8X,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASL       RABZ,D1Re0,D1Re0
        D         ASL       RABZ,D1Re0,[D1BR.1]
        D         ASL       RABZ,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASL       RABZ,D1.7,D1.7
        D         ASL       RABZ,D1.7,[D1BR.1++]
        D         ASL       RABZ,AC1.0,D1Re0
        D         ASL       RABZ,AC1.0,[D1BR.1]
        D         ASL       RABZ,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASL       RABZ,[D1BR.1],D1.7
        D         ASL       RABZ,[D1BR.1],[D1BR.1++]
        D         ASL       RABZ,[D1BR.1++],D1Re0
        D         ASL       RABZ,[D1BR.1++],[D1BR.1]
        D         ASL       RABZ,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASL       RABZ,[D1BR.1+D1BRI.1++],D1.7
        D         ASL       RABZ,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASL       RAWX,D1Re0,D1Re0
        D         ASL       RAWX,D1Re0,[D1BR.1]
        D         ASL       RAWX,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASL       RAWX,D1.7,D1.7
        D         ASL       RAWX,D1.7,[D1BR.1++]
        D         ASL       RAWX,AC1.0,D1Re0
        D         ASL       RAWX,AC1.0,[D1BR.1]
        D         ASL       RAWX,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASL       RAWX,[D1BR.1],D1.7
        D         ASL       RAWX,[D1BR.1],[D1BR.1++]
        D         ASL       RAWX,[D1BR.1++],D1Re0
        D         ASL       RAWX,[D1BR.1++],[D1BR.1]
        D         ASL       RAWX,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASL       RAWX,[D1BR.1+D1BRI.1++],D1.7
        D         ASL       RAWX,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSR       TXIDLECYC,D1Re0,D1Re0
        D         LSR       TXIDLECYC,D1Re0,D1.7
        D         LSR       TXIDLECYC,D1Re0,[D1BR.1]
        D         LSR       TXIDLECYC,D1Re0,[D1BR.1++]
        D         LSR       TXIDLECYC,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSR       TXIDLECYC,D1.7,D1Re0
        D         LSR       TXIDLECYC,D1.7,D1.7
        D         LSR       TXIDLECYC,D1.7,[D1BR.1]
        D         LSR       TXIDLECYC,D1.7,[D1BR.1++]
        D         LSR       TXIDLECYC,D1.7,[D1BR.1+D1BRI.1++]
        D         LSR       TXIDLECYC,AC1.0,D1Re0
        D         LSR       TXIDLECYC,AC1.0,D1.7
        D         LSR       TXIDLECYC,AC1.0,[D1BR.1]
        D         LSR       TXIDLECYC,AC1.0,[D1BR.1++]
        D         LSR       TXIDLECYC,AC1.0,[D1BR.1+D1BRI.1++]
        D         LSR       TXIDLECYC,[D1BR.1],D1Re0
        D         LSR       TXIDLECYC,[D1BR.1],D1.7
        D         LSR       TXIDLECYC,[D1BR.1],[D1BR.1]
        D         LSR       TXIDLECYC,[D1BR.1],[D1BR.1++]
        D         LSR       TXIDLECYC,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSR       TXIDLECYC,[D1BR.1++],D1Re0
        D         LSR       TXIDLECYC,[D1BR.1++],D1.7
        D         LSR       TXIDLECYC,[D1BR.1++],[D1BR.1]
        D         LSR       TXIDLECYC,[D1BR.1++],[D1BR.1++]
        D         LSR       TXIDLECYC,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSR       TXIDLECYC,[D1BR.1+D1BRI.1++],D1Re0
        D         LSR       TXIDLECYC,[D1BR.1+D1BRI.1++],D1.7
        D         LSR       TXIDLECYC,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSR       TXIDLECYC,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSR       TXIDLECYC,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSL       TXENABLE,D0Re0,#0xf
        D         LSR       PC,D0Re0,#0xf
        DHNE      ASRPL     D0Re0,D0Re0,#0xf
        D         LSL       TXENABLE,D0Re0,#0x10
        D         LSR       PC,D0Re0,#0x10
        DHNE      ASRPL     D0Re0,D0Re0,#0x10
        D         LSL       TXENABLE,D0Re0,#0x1f
        D         LSR       PC,D0Re0,#0x1f
        DHNE      ASRPL     D0Re0,D0Re0,#0x1f
        D         LSL       TXENABLE,D0.7,#0xf
        D         LSR       PC,D0.7,#0xf
        DHNE      ASRPL     D0Re0,D0.7,#0xf
        D         LSL       TXENABLE,D0.7,#0x10
        D         LSR       PC,D0.7,#0x10
        DHNE      ASRPL     D0Re0,D0.7,#0x10
        D         LSL       TXENABLE,D0.7,#0x1f
        D         LSR       PC,D0.7,#0x1f
        DHNE      ASRPL     D0Re0,D0.7,#0x1f
        D         LSLNV     D0Re0,[D0AR.0],#0xf
        D         LSL       TXSTAT,[D0AR.0],#0xf
        D         LSR       D1Re0,[D0AR.0],#0xf
        D         ASLNV     D0Re0,[D0AR.0],#0xf
        D         ASL       TXSTAT,[D0AR.0],#0xf
        D         ASR       D1Re0,[D0AR.0],#0xf
        D         LSLNV     D0Re0,[D0AR.0],#0x10
        D         LSL       TXSTAT,[D0AR.0],#0x10
        D         LSR       D1Re0,[D0AR.0],#0x10
        D         ASLNV     D0Re0,[D0AR.0],#0x10
        D         ASL       TXSTAT,[D0AR.0],#0x10
        D         ASR       D1Re0,[D0AR.0],#0x10
        D         LSLNV     D0Re0,[D0AR.0],#0x1f
        D         LSL       TXSTAT,[D0AR.0],#0x1f
        D         LSR       D1Re0,[D0AR.0],#0x1f
        D         ASLNV     D0Re0,[D0AR.0],#0x1f
        D         ASL       TXSTAT,[D0AR.0],#0x1f
        D         ASR       D1Re0,[D0AR.0],#0x1f
        D         LSL       TXENABLE,[D0AR.0++],#0xf
        D         LSR       PC,[D0AR.0++],#0xf
        DHNE      ASRPL     D0Re0,[D0AR.0++],#0xf
        D         LSL       TXENABLE,[D0AR.0++],#0x10
        D         LSR       PC,[D0AR.0++],#0x10
        DHNE      ASRPL     D0Re0,[D0AR.0++],#0x10
        D         LSL       TXENABLE,[D0AR.0++],#0x1f
        D         LSR       PC,[D0AR.0++],#0x1f
        DHNE      ASRPL     D0Re0,[D0AR.0++],#0x1f
        D         LSL       TXENABLE,[D0AR.0+D0ARI.0++],#0xf
        D         LSR       PC,[D0AR.0+D0ARI.0++],#0xf
        DHNE      ASRPL     D0Re0,[D0AR.0+D0ARI.0++],#0xf
        D         LSL       TXENABLE,[D0AR.0+D0ARI.0++],#0x10
        D         LSR       PC,[D0AR.0+D0ARI.0++],#0x10
        DHNE      ASRPL     D0Re0,[D0AR.0+D0ARI.0++],#0x10
        D         LSL       TXENABLE,[D0AR.0+D0ARI.0++],#0x1f
        D         LSR       PC,[D0AR.0+D0ARI.0++],#0x1f
        DHNE      ASRPL     D0Re0,[D0AR.0+D0ARI.0++],#0x1f
        D         LSL       PCX,D0Re0,#0xf
        D         ASR       A1LbP,D0Re0,#0xf
        D         ASL       A0FrP,D0Re0,#0x10
        D         LSL       PCX,D0Re0,#0x1f
        D         ASR       A1LbP,D0Re0,#0x1f
        D         ASL       A0FrP,D0.7,#0xf
        D         LSL       PCX,D0.7,#0x10
        D         ASR       A1LbP,D0.7,#0x10
        D         ASL       A0FrP,D0.7,#0x1f
        D         LSL       A1LbP,[D0AR.0],#0xf
        D         LSR       PCX,[D0AR.0],#0xf
        D         ASR       A0FrP,[D0AR.0],#0xf
        D         LSL       A1LbP,[D0AR.0],#0x10
        D         LSR       PCX,[D0AR.0],#0x10
        D         ASR       A0FrP,[D0AR.0],#0x10
        D         LSL       A1LbP,[D0AR.0],#0x1f
        D         LSR       PCX,[D0AR.0],#0x1f
        D         ASR       A0FrP,[D0AR.0],#0x1f
        D         LSL       PCX,[D0AR.0++],#0xf
        D         ASR       A1LbP,[D0AR.0++],#0xf
        D         ASL       A0FrP,[D0AR.0++],#0x10
        D         LSL       PCX,[D0AR.0++],#0x1f
        D         ASR       A1LbP,[D0AR.0++],#0x1f
        D         ASL       A0FrP,[D0AR.0+D0ARI.0++],#0xf
        D         LSL       PCX,[D0AR.0+D0ARI.0++],#0x10
        D         ASR       A1LbP,[D0AR.0+D0ARI.0++],#0x10
        D         ASL       A0FrP,[D0AR.0+D0ARI.0++],#0x1f
        D         ASR       TXMASKI,D0Re0,#0xf
        D         ASR       TXMASKI,D0Re0,#0x1f
        D         ASR       TXMASKI,D0.7,#0x10
        D         LSR       TXMASKI,AC0.0,#0xf
        D         LSR       TXMASKI,AC0.0,#0x10
        D         LSR       TXMASKI,AC0.0,#0x1f
        D         ASR       TXMASKI,[D0AR.0++],#0xf
        D         ASR       TXMASKI,[D0AR.0++],#0x1f
        D         ASR       TXMASKI,[D0AR.0+D0ARI.0++],#0x10
        D         LSL       A0.7,D0Re0,#0xf
        D         LSR       TXGPIOO,D0Re0,#0xf
        D         ASRNV     D0.7,D0Re0,#0xf
        D         LSL       A0.7,D0Re0,#0x10
        D         LSR       TXGPIOO,D0Re0,#0x10
        D         ASRNV     D0.7,D0Re0,#0x10
        D         LSL       A0.7,D0Re0,#0x1f
        D         LSR       TXGPIOO,D0Re0,#0x1f
        D         ASRNV     D0.7,D0Re0,#0x1f
        D         LSL       A0.7,D0.7,#0xf
        D         LSR       TXGPIOO,D0.7,#0xf
        D         ASRNV     D0.7,D0.7,#0xf
        D         LSL       A0.7,D0.7,#0x10
        D         LSR       TXGPIOO,D0.7,#0x10
        D         ASRNV     D0.7,D0.7,#0x10
        D         LSL       A0.7,D0.7,#0x1f
        D         LSR       TXGPIOO,D0.7,#0x1f
        D         ASRNV     D0.7,D0.7,#0x1f
        D         LSLNV     D0.7,AC0.0,#0xf
        D         LSL       TXGPIOO,AC0.0,#0xf
        D         LSR       A0.7,AC0.0,#0xf
        D         ASLNV     D0.7,AC0.0,#0xf
        D         ASL       TXGPIOO,AC0.0,#0xf
        D         ASR       A0.7,AC0.0,#0xf
        D         LSLNV     D0.7,AC0.0,#0x10
        D         LSL       TXGPIOO,AC0.0,#0x10
        D         LSR       A0.7,AC0.0,#0x10
        D         ASLNV     D0.7,AC0.0,#0x10
        D         ASL       TXGPIOO,AC0.0,#0x10
        D         ASR       A0.7,AC0.0,#0x10
        D         LSLNV     D0.7,AC0.0,#0x1f
        D         LSL       TXGPIOO,AC0.0,#0x1f
        D         LSR       A0.7,AC0.0,#0x1f
        D         ASLNV     D0.7,AC0.0,#0x1f
        D         ASL       TXGPIOO,AC0.0,#0x1f
        D         ASR       A0.7,AC0.0,#0x1f
        D         LSL       A0.7,[D0AR.0++],#0xf
        D         LSR       TXGPIOO,[D0AR.0++],#0xf
        D         ASRNV     D0.7,[D0AR.0++],#0xf
        D         LSL       A0.7,[D0AR.0++],#0x10
        D         LSR       TXGPIOO,[D0AR.0++],#0x10
        D         ASRNV     D0.7,[D0AR.0++],#0x10
        D         LSL       A0.7,[D0AR.0++],#0x1f
        D         LSR       TXGPIOO,[D0AR.0++],#0x1f
        D         ASRNV     D0.7,[D0AR.0++],#0x1f
        D         LSL       A0.7,[D0AR.0+D0ARI.0++],#0xf
        D         LSR       TXGPIOO,[D0AR.0+D0ARI.0++],#0xf
        D         ASRNV     D0.7,[D0AR.0+D0ARI.0++],#0xf
        D         LSL       A0.7,[D0AR.0+D0ARI.0++],#0x10
        D         LSR       TXGPIOO,[D0AR.0+D0ARI.0++],#0x10
        D         ASRNV     D0.7,[D0AR.0+D0ARI.0++],#0x10
        D         LSL       A0.7,[D0AR.0+D0ARI.0++],#0x1f
        D         LSR       TXGPIOO,[D0AR.0+D0ARI.0++],#0x1f
        D         ASRNV     D0.7,[D0AR.0+D0ARI.0++],#0x1f
        D         LSR       A1.8,D0Re0,#0xf
        D         LSR       A1.8,D0Re0,#0x10
        D         LSR       A1.8,D0Re0,#0x1f
        D         LSR       A1.8,D0.7,#0xf
        D         LSR       A1.8,D0.7,#0x10
        D         LSR       A1.8,D0.7,#0x1f
        D         LSL       A1.8,AC0.0,#0xf
        D         ASL       A1.8,AC0.0,#0xf
        D         LSL       A1.8,AC0.0,#0x10
        D         ASL       A1.8,AC0.0,#0x10
        D         LSL       A1.8,AC0.0,#0x1f
        D         ASL       A1.8,AC0.0,#0x1f
        D         LSR       A1.8,[D0AR.0++],#0xf
        D         LSR       A1.8,[D0AR.0++],#0x10
        D         LSR       A1.8,[D0AR.0++],#0x1f
        D         LSR       A1.8,[D0AR.0+D0ARI.0++],#0xf
        D         LSR       A1.8,[D0AR.0+D0ARI.0++],#0x10
        D         LSR       A1.8,[D0AR.0+D0ARI.0++],#0x1f
        D         LSL       RA,D0Re0,#0xf
        D         ASL       RA,D0Re0,#0xf
        D         LSL       RA,D0Re0,#0x10
        D         ASL       RA,D0Re0,#0x10
        D         LSL       RA,D0Re0,#0x1f
        D         ASL       RA,D0Re0,#0x1f
        D         LSL       RA,D0.7,#0xf
        D         ASL       RA,D0.7,#0xf
        D         LSL       RA,D0.7,#0x10
        D         ASL       RA,D0.7,#0x10
        D         LSL       RA,D0.7,#0x1f
        D         ASL       RA,D0.7,#0x1f
        D         LSLNV     D0.16,AC0.0,#0xf
        D         LSRNV     D0.16,AC0.0,#0xf
        D         ASLNV     D0.16,AC0.0,#0xf
        D         ASRNV     D0.16,AC0.0,#0xf
        D         LSLNV     D0.16,AC0.0,#0x10
        D         LSRNV     D0.16,AC0.0,#0x10
        D         ASLNV     D0.16,AC0.0,#0x10
        D         ASRNV     D0.16,AC0.0,#0x10
        D         LSLNV     D0.16,AC0.0,#0x1f
        D         LSRNV     D0.16,AC0.0,#0x1f
        D         ASLNV     D0.16,AC0.0,#0x1f
        D         ASRNV     D0.16,AC0.0,#0x1f
        D         LSL       RA,[D0AR.0++],#0xf
        D         ASL       RA,[D0AR.0++],#0xf
        D         LSL       RA,[D0AR.0++],#0x10
        D         ASL       RA,[D0AR.0++],#0x10
        D         LSL       RA,[D0AR.0++],#0x1f
        D         ASL       RA,[D0AR.0++],#0x1f
        D         LSL       RA,[D0AR.0+D0ARI.0++],#0xf
        D         ASL       RA,[D0AR.0+D0ARI.0++],#0xf
        D         LSL       RA,[D0AR.0+D0ARI.0++],#0x10
        D         ASL       RA,[D0AR.0+D0ARI.0++],#0x10
        D         LSL       RA,[D0AR.0+D0ARI.0++],#0x1f
        D         ASL       RA,[D0AR.0+D0ARI.0++],#0x1f
        D         ASR       RAM8X32,D0Re0,#0xf
        D         ASR       RAM8X32,D0Re0,#0x1f
        D         ASR       RAM8X32,D0.7,#0x10
        D         LSR       RAM8X32,AC0.0,#0xf
        D         LSR       RAM8X32,AC0.0,#0x10
        D         LSR       RAM8X32,AC0.0,#0x1f
        D         ASR       RAM8X32,[D0AR.0++],#0xf
        D         ASR       RAM8X32,[D0AR.0++],#0x1f
        D         ASR       RAM8X32,[D0AR.0+D0ARI.0++],#0x10
        D         ASR       RAM8X,D0Re0,#0xf
        D         ASR       RAM8X,D0Re0,#0x1f
        D         ASR       RAM8X,D0.7,#0x10
        D         LSR       RAM8X,AC0.0,#0xf
        D         LSR       RAM8X,AC0.0,#0x10
        D         LSR       RAM8X,AC0.0,#0x1f
        D         ASR       RAM8X,[D0AR.0++],#0xf
        D         ASR       RAM8X,[D0AR.0++],#0x1f
        D         ASR       RAM8X,[D0AR.0+D0ARI.0++],#0x10
        D         ASR       RABZ,D0Re0,#0xf
        D         ASR       RABZ,D0Re0,#0x1f
        D         ASR       RABZ,D0.7,#0x10
        D         LSR       RABZ,AC0.0,#0xf
        D         LSR       RABZ,AC0.0,#0x10
        D         LSR       RABZ,AC0.0,#0x1f
        D         ASR       RABZ,[D0AR.0++],#0xf
        D         ASR       RABZ,[D0AR.0++],#0x1f
        D         ASR       RABZ,[D0AR.0+D0ARI.0++],#0x10
        D         ASR       RAWX,D0Re0,#0xf
        D         ASR       RAWX,D0Re0,#0x1f
        D         ASR       RAWX,D0.7,#0x10
        D         LSR       RAWX,AC0.0,#0xf
        D         LSR       RAWX,AC0.0,#0x10
        D         LSR       RAWX,AC0.0,#0x1f
        D         ASR       RAWX,[D0AR.0++],#0xf
        D         ASR       RAWX,[D0AR.0++],#0x1f
        D         ASR       RAWX,[D0AR.0+D0ARI.0++],#0x10
        D         LSR       RAMX,D0Re0,#0xf
        D         LSR       RAMX,D0Re0,#0x10
        D         LSR       RAMX,D0Re0,#0x1f
        D         LSR       RAMX,D0.7,#0xf
        D         LSR       RAMX,D0.7,#0x10
        D         LSR       RAMX,D0.7,#0x1f
        D         LSL       RAMX,AC0.0,#0xf
        D         ASL       RAMX,AC0.0,#0xf
        D         LSL       RAMX,AC0.0,#0x10
        D         ASL       RAMX,AC0.0,#0x10
        D         LSL       RAMX,AC0.0,#0x1f
        D         ASL       RAMX,AC0.0,#0x1f
        D         LSR       RAMX,[D0AR.0++],#0xf
        D         LSR       RAMX,[D0AR.0++],#0x10
        D         LSR       RAMX,[D0AR.0++],#0x1f
        D         LSR       RAMX,[D0AR.0+D0ARI.0++],#0xf
        D         LSR       RAMX,[D0AR.0+D0ARI.0++],#0x10
        D         LSR       RAMX,[D0AR.0+D0ARI.0++],#0x1f
        D         LSL       D0Re0,D1Re0,#0xf
        D         LSR       TXSTAT,D1Re0,#0xf
        D         ASRNV     D1Re0,D1Re0,#0xf
        D         LSL       D0Re0,D1Re0,#0x10
        D         LSR       TXSTAT,D1Re0,#0x10
        D         ASRNV     D1Re0,D1Re0,#0x10
        D         LSL       D0Re0,D1Re0,#0x1f
        D         LSR       TXSTAT,D1Re0,#0x1f
        D         ASRNV     D1Re0,D1Re0,#0x1f
        D         LSL       D0Re0,D1.7,#0xf
        D         LSR       TXSTAT,D1.7,#0xf
        D         ASRNV     D1Re0,D1.7,#0xf
        D         LSL       D0Re0,D1.7,#0x10
        D         LSR       TXSTAT,D1.7,#0x10
        D         ASRNV     D1Re0,D1.7,#0x10
        D         LSL       D0Re0,D1.7,#0x1f
        D         LSR       TXSTAT,D1.7,#0x1f
        D         ASRNV     D1Re0,D1.7,#0x1f
        D         LSL       D0Re0,AC1.0,#0xf
        D         LSR       TXSTAT,AC1.0,#0xf
        D         ASRNV     D1Re0,AC1.0,#0xf
        D         LSL       D0Re0,AC1.0,#0x10
        D         LSR       TXSTAT,AC1.0,#0x10
        D         ASRNV     D1Re0,AC1.0,#0x10
        D         LSL       D0Re0,AC1.0,#0x1f
        D         LSR       TXSTAT,AC1.0,#0x1f
        D         ASRNV     D1Re0,AC1.0,#0x1f
        D         LSL       D0Re0,[D1BR.1],#0xf
        D         LSR       TXSTAT,[D1BR.1],#0xf
        D         ASRNV     D1Re0,[D1BR.1],#0xf
        D         LSL       D0Re0,[D1BR.1],#0x10
        D         LSR       TXSTAT,[D1BR.1],#0x10
        D         ASRNV     D1Re0,[D1BR.1],#0x10
        D         LSL       D0Re0,[D1BR.1],#0x1f
        D         LSR       TXSTAT,[D1BR.1],#0x1f
        D         ASRNV     D1Re0,[D1BR.1],#0x1f
        D         LSL       D0Re0,[D1BR.1++],#0xf
        D         LSR       TXSTAT,[D1BR.1++],#0xf
        D         ASRNV     D1Re0,[D1BR.1++],#0xf
        D         LSL       D0Re0,[D1BR.1++],#0x10
        D         LSR       TXSTAT,[D1BR.1++],#0x10
        D         ASRNV     D1Re0,[D1BR.1++],#0x10
        D         LSL       D0Re0,[D1BR.1++],#0x1f
        D         LSR       TXSTAT,[D1BR.1++],#0x1f
        D         ASRNV     D1Re0,[D1BR.1++],#0x1f
        D         LSL       D0Re0,[D1BR.1+D1BRI.1++],#0xf
        D         LSR       TXSTAT,[D1BR.1+D1BRI.1++],#0xf
        D         ASRNV     D1Re0,[D1BR.1+D1BRI.1++],#0xf
        D         LSL       D0Re0,[D1BR.1+D1BRI.1++],#0x10
        D         LSR       TXSTAT,[D1BR.1+D1BRI.1++],#0x10
        D         ASRNV     D1Re0,[D1BR.1+D1BRI.1++],#0x10
        D         LSL       D0Re0,[D1BR.1+D1BRI.1++],#0x1f
        D         LSR       TXSTAT,[D1BR.1+D1BRI.1++],#0x1f
        D         ASRNV     D1Re0,[D1BR.1+D1BRI.1++],#0x1f
        D         LSR       A0FrP,D1Re0,#0xf
        D         ASR       PCX,D1Re0,#0xf
        D         ASL       A1LbP,D1Re0,#0x10
        D         LSR       A0FrP,D1Re0,#0x1f
        D         ASR       PCX,D1Re0,#0x1f
        D         ASL       A1LbP,D1.7,#0xf
        D         LSR       A0FrP,D1.7,#0x10
        D         ASR       PCX,D1.7,#0x10
        D         ASL       A1LbP,D1.7,#0x1f
        D         LSR       A0FrP,AC1.0,#0xf
        D         ASR       PCX,AC1.0,#0xf
        D         ASL       A1LbP,AC1.0,#0x10
        D         LSR       A0FrP,AC1.0,#0x1f
        D         ASR       PCX,AC1.0,#0x1f
        D         ASL       A1LbP,[D1BR.1],#0xf
        D         LSR       A0FrP,[D1BR.1],#0x10
        D         ASR       PCX,[D1BR.1],#0x10
        D         ASL       A1LbP,[D1BR.1],#0x1f
        D         LSR       A0FrP,[D1BR.1++],#0xf
        D         ASR       PCX,[D1BR.1++],#0xf
        D         ASL       A1LbP,[D1BR.1++],#0x10
        D         LSR       A0FrP,[D1BR.1++],#0x1f
        D         ASR       PCX,[D1BR.1++],#0x1f
        D         ASL       A1LbP,[D1BR.1+D1BRI.1++],#0xf
        D         LSR       A0FrP,[D1BR.1+D1BRI.1++],#0x10
        D         ASR       PCX,[D1BR.1+D1BRI.1++],#0x10
        D         ASL       A1LbP,[D1BR.1+D1BRI.1++],#0x1f
        D         ASR       TXMASKI,D1Re0,#0xf
        D         ASR       TXMASKI,D1Re0,#0x1f
        D         ASR       TXMASKI,D1.7,#0x10
        D         ASR       TXMASKI,AC1.0,#0xf
        D         ASR       TXMASKI,AC1.0,#0x1f
        D         ASR       TXMASKI,[D1BR.1],#0x10
        D         ASR       TXMASKI,[D1BR.1++],#0xf
        D         ASR       TXMASKI,[D1BR.1++],#0x1f
        D         ASR       TXMASKI,[D1BR.1+D1BRI.1++],#0x10
        D         LSL       A0.7,D1Re0,#0xf
        D         LSR       TXGPIOO,D1Re0,#0xf
        D         ASRNV     D1.7,D1Re0,#0xf
        D         LSL       A0.7,D1Re0,#0x10
        D         LSR       TXGPIOO,D1Re0,#0x10
        D         ASRNV     D1.7,D1Re0,#0x10
        D         LSL       A0.7,D1Re0,#0x1f
        D         LSR       TXGPIOO,D1Re0,#0x1f
        D         ASRNV     D1.7,D1Re0,#0x1f
        D         LSL       A0.7,D1.7,#0xf
        D         LSR       TXGPIOO,D1.7,#0xf
        D         ASRNV     D1.7,D1.7,#0xf
        D         LSL       A0.7,D1.7,#0x10
        D         LSR       TXGPIOO,D1.7,#0x10
        D         ASRNV     D1.7,D1.7,#0x10
        D         LSL       A0.7,D1.7,#0x1f
        D         LSR       TXGPIOO,D1.7,#0x1f
        D         ASRNV     D1.7,D1.7,#0x1f
        D         LSL       A0.7,AC1.0,#0xf
        D         LSR       TXGPIOO,AC1.0,#0xf
        D         ASRNV     D1.7,AC1.0,#0xf
        D         LSL       A0.7,AC1.0,#0x10
        D         LSR       TXGPIOO,AC1.0,#0x10
        D         ASRNV     D1.7,AC1.0,#0x10
        D         LSL       A0.7,AC1.0,#0x1f
        D         LSR       TXGPIOO,AC1.0,#0x1f
        D         ASRNV     D1.7,AC1.0,#0x1f
        D         LSL       A0.7,[D1BR.1],#0xf
        D         LSR       TXGPIOO,[D1BR.1],#0xf
        D         ASRNV     D1.7,[D1BR.1],#0xf
        D         LSL       A0.7,[D1BR.1],#0x10
        D         LSR       TXGPIOO,[D1BR.1],#0x10
        D         ASRNV     D1.7,[D1BR.1],#0x10
        D         LSL       A0.7,[D1BR.1],#0x1f
        D         LSR       TXGPIOO,[D1BR.1],#0x1f
        D         ASRNV     D1.7,[D1BR.1],#0x1f
        D         LSL       A0.7,[D1BR.1++],#0xf
        D         LSR       TXGPIOO,[D1BR.1++],#0xf
        D         ASRNV     D1.7,[D1BR.1++],#0xf
        D         LSL       A0.7,[D1BR.1++],#0x10
        D         LSR       TXGPIOO,[D1BR.1++],#0x10
        D         ASRNV     D1.7,[D1BR.1++],#0x10
        D         LSL       A0.7,[D1BR.1++],#0x1f
        D         LSR       TXGPIOO,[D1BR.1++],#0x1f
        D         ASRNV     D1.7,[D1BR.1++],#0x1f
        D         LSL       A0.7,[D1BR.1+D1BRI.1++],#0xf
        D         LSR       TXGPIOO,[D1BR.1+D1BRI.1++],#0xf
        D         ASRNV     D1.7,[D1BR.1+D1BRI.1++],#0xf
        D         LSL       A0.7,[D1BR.1+D1BRI.1++],#0x10
        D         LSR       TXGPIOO,[D1BR.1+D1BRI.1++],#0x10
        D         ASRNV     D1.7,[D1BR.1+D1BRI.1++],#0x10
        D         LSL       A0.7,[D1BR.1+D1BRI.1++],#0x1f
        D         LSR       TXGPIOO,[D1BR.1+D1BRI.1++],#0x1f
        D         ASRNV     D1.7,[D1BR.1+D1BRI.1++],#0x1f
        D         LSR       A1.8,D1Re0,#0xf
        D         LSR       A1.8,D1Re0,#0x10
        D         LSR       A1.8,D1Re0,#0x1f
        D         LSR       A1.8,D1.7,#0xf
        D         LSR       A1.8,D1.7,#0x10
        D         LSR       A1.8,D1.7,#0x1f
        D         LSR       A1.8,AC1.0,#0xf
        D         LSR       A1.8,AC1.0,#0x10
        D         LSR       A1.8,AC1.0,#0x1f
        D         LSR       A1.8,[D1BR.1],#0xf
        D         LSR       A1.8,[D1BR.1],#0x10
        D         LSR       A1.8,[D1BR.1],#0x1f
        D         LSR       A1.8,[D1BR.1++],#0xf
        D         LSR       A1.8,[D1BR.1++],#0x10
        D         LSR       A1.8,[D1BR.1++],#0x1f
        D         LSR       A1.8,[D1BR.1+D1BRI.1++],#0xf
        D         LSR       A1.8,[D1BR.1+D1BRI.1++],#0x10
        D         LSR       A1.8,[D1BR.1+D1BRI.1++],#0x1f
        D         LSL       RA,D1Re0,#0xf
        D         ASL       RA,D1Re0,#0xf
        D         LSL       RA,D1Re0,#0x10
        D         ASL       RA,D1Re0,#0x10
        D         LSL       RA,D1Re0,#0x1f
        D         ASL       RA,D1Re0,#0x1f
        D         LSL       RA,D1.7,#0xf
        D         ASL       RA,D1.7,#0xf
        D         LSL       RA,D1.7,#0x10
        D         ASL       RA,D1.7,#0x10
        D         LSL       RA,D1.7,#0x1f
        D         ASL       RA,D1.7,#0x1f
        D         LSL       RA,AC1.0,#0xf
        D         ASL       RA,AC1.0,#0xf
        D         LSL       RA,AC1.0,#0x10
        D         ASL       RA,AC1.0,#0x10
        D         LSL       RA,AC1.0,#0x1f
        D         ASL       RA,AC1.0,#0x1f
        D         LSL       RA,[D1BR.1],#0xf
        D         ASL       RA,[D1BR.1],#0xf
        D         LSL       RA,[D1BR.1],#0x10
        D         ASL       RA,[D1BR.1],#0x10
        D         LSL       RA,[D1BR.1],#0x1f
        D         ASL       RA,[D1BR.1],#0x1f
        D         LSL       RA,[D1BR.1++],#0xf
        D         ASL       RA,[D1BR.1++],#0xf
        D         LSL       RA,[D1BR.1++],#0x10
        D         ASL       RA,[D1BR.1++],#0x10
        D         LSL       RA,[D1BR.1++],#0x1f
        D         ASL       RA,[D1BR.1++],#0x1f
        D         LSL       RA,[D1BR.1+D1BRI.1++],#0xf
        D         ASL       RA,[D1BR.1+D1BRI.1++],#0xf
        D         LSL       RA,[D1BR.1+D1BRI.1++],#0x10
        D         ASL       RA,[D1BR.1+D1BRI.1++],#0x10
        D         LSL       RA,[D1BR.1+D1BRI.1++],#0x1f
        D         ASL       RA,[D1BR.1+D1BRI.1++],#0x1f
        D         ASR       RAM8X32,D1Re0,#0xf
        D         ASR       RAM8X32,D1Re0,#0x1f
        D         ASR       RAM8X32,D1.7,#0x10
        D         ASR       RAM8X32,AC1.0,#0xf
        D         ASR       RAM8X32,AC1.0,#0x1f
        D         ASR       RAM8X32,[D1BR.1],#0x10
        D         ASR       RAM8X32,[D1BR.1++],#0xf
        D         ASR       RAM8X32,[D1BR.1++],#0x1f
        D         ASR       RAM8X32,[D1BR.1+D1BRI.1++],#0x10
        D         ASR       RAM8X,D1Re0,#0xf
        D         ASR       RAM8X,D1Re0,#0x1f
        D         ASR       RAM8X,D1.7,#0x10
        D         ASR       RAM8X,AC1.0,#0xf
        D         ASR       RAM8X,AC1.0,#0x1f
        D         ASR       RAM8X,[D1BR.1],#0x10
        D         ASR       RAM8X,[D1BR.1++],#0xf
        D         ASR       RAM8X,[D1BR.1++],#0x1f
        D         ASR       RAM8X,[D1BR.1+D1BRI.1++],#0x10
        D         ASR       RABZ,D1Re0,#0xf
        D         ASR       RABZ,D1Re0,#0x1f
        D         ASR       RABZ,D1.7,#0x10
        D         ASR       RABZ,AC1.0,#0xf
        D         ASR       RABZ,AC1.0,#0x1f
        D         ASR       RABZ,[D1BR.1],#0x10
        D         ASR       RABZ,[D1BR.1++],#0xf
        D         ASR       RABZ,[D1BR.1++],#0x1f
        D         ASR       RABZ,[D1BR.1+D1BRI.1++],#0x10
        D         ASR       RAWX,D1Re0,#0xf
        D         ASR       RAWX,D1Re0,#0x1f
        D         ASR       RAWX,D1.7,#0x10
        D         ASR       RAWX,AC1.0,#0xf
        D         ASR       RAWX,AC1.0,#0x1f
        D         ASR       RAWX,[D1BR.1],#0x10
        D         ASR       RAWX,[D1BR.1++],#0xf
        D         ASR       RAWX,[D1BR.1++],#0x1f
        D         ASR       RAWX,[D1BR.1+D1BRI.1++],#0x10
        D         LSR       RAMX,D1Re0,#0xf
        D         LSR       RAMX,D1Re0,#0x10
        D         LSR       RAMX,D1Re0,#0x1f
        D         LSR       RAMX,D1.7,#0xf
        D         LSR       RAMX,D1.7,#0x10
        D         LSR       RAMX,D1.7,#0x1f
        D         LSR       RAMX,AC1.0,#0xf
        D         LSR       RAMX,AC1.0,#0x10
        D         LSR       RAMX,AC1.0,#0x1f
        D         LSR       RAMX,[D1BR.1],#0xf
        D         LSR       RAMX,[D1BR.1],#0x10
        D         LSR       RAMX,[D1BR.1],#0x1f
        D         LSR       RAMX,[D1BR.1++],#0xf
        D         LSR       RAMX,[D1BR.1++],#0x10
        D         LSR       RAMX,[D1BR.1++],#0x1f
        D         LSR       RAMX,[D1BR.1+D1BRI.1++],#0xf
        D         LSR       RAMX,[D1BR.1+D1BRI.1++],#0x10
        D         LSR       RAMX,[D1BR.1+D1BRI.1++],#0x1f
        DB        LSLS      D0Re0,D0Re0,D0Re0
        DB        LSRS      D0Re0,D0Re0,D0Re0
        DB        ASLS      D0Re0,D0Re0,D0Re0
        DB        ASRS      D0Re0,D0Re0,D0Re0
        DB        LSLS      D0Re0,D0Re0,D0.7
        DB        LSRS      D0Re0,D0Re0,D0.7
        DB        ASLS      D0Re0,D0Re0,D0.7
        DB        ASRS      D0Re0,D0Re0,D0.7
        DB        LSLS      D0Re0,D0Re0,[D0AR.0]
        DB        LSRS      D0Re0,D0Re0,[D0AR.0]
        DB        ASLS      D0Re0,D0Re0,[D0AR.0]
        DB        ASRS      D0Re0,D0Re0,[D0AR.0]
        DB        LSLS      D0Re0,D0Re0,[D0AR.0++]
        DB        LSRS      D0Re0,D0Re0,[D0AR.0++]
        DB        ASLS      D0Re0,D0Re0,[D0AR.0++]
        DB        ASRS      D0Re0,D0Re0,[D0AR.0++]
        DB        LSLS      D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DB        LSRS      D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DB        ASLS      D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DB        ASRS      D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DB        LSLS      D0Re0,D0.7,D0Re0
        DB        LSRS      D0Re0,D0.7,D0Re0
        DB        ASLS      D0Re0,D0.7,D0Re0
        DB        ASRS      D0Re0,D0.7,D0Re0
        DB        LSLS      D0Re0,D0.7,D0.7
        DB        LSRS      D0Re0,D0.7,D0.7
        DB        ASLS      D0Re0,D0.7,D0.7
        DB        ASRS      D0Re0,D0.7,D0.7
        DB        LSLS      D0Re0,D0.7,[D0AR.0]
        DB        LSRS      D0Re0,D0.7,[D0AR.0]
        DB        ASLS      D0Re0,D0.7,[D0AR.0]
        DB        ASRS      D0Re0,D0.7,[D0AR.0]
        DB        LSLS      D0Re0,D0.7,[D0AR.0++]
        DB        LSRS      D0Re0,D0.7,[D0AR.0++]
        DB        ASLS      D0Re0,D0.7,[D0AR.0++]
        DB        ASRS      D0Re0,D0.7,[D0AR.0++]
        DB        LSLS      D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DB        LSRS      D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DB        ASLS      D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DB        ASRS      D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DR        LSLS      D0Re0,AC0.0,D0Re0
        DLG       LSLS      D0Re0,[D0AR.0],D0Re0
        DG        LSRS      D0Re0,[D0AR.0],D0Re0
        DLG       LSRS      D0Re0,[D0AR.0],D0Re0
        DG        ASLS      D0Re0,[D0AR.0],D0Re0
        DLG       ASLS      D0Re0,[D0AR.0],D0Re0
        DG        ASRS      D0Re0,[D0AR.0],D0Re0
        DLG       ASRS      D0Re0,[D0AR.0],D0Re0
        DG        LSLS      D0Re0,[D0AR.0],D0.7
        DLG       LSLS      D0Re0,AC0.0,D0.7
        DG        LSRS      D0Re0,AC0.0,D0.7
        DLG       LSRS      D0Re0,AC0.0,D0.7
        DG        ASLS      D0Re0,AC0.0,D0.7
        DLG       ASLS      D0Re0,AC0.0,D0.7
        DG        ASRS      D0Re0,AC0.0,D0.7
        DLG       ASRS      D0Re0,AC0.0,D0.7
        DG        LSLS      D0Re0,AC0.0,[D0AR.0]
        DLB       LSLS      D0Re0,[D0AR.0],[D0AR.0]
        DB        LSRS      D0Re0,[D0AR.0],[D0AR.0]
        DLB       LSRS      D0Re0,[D0AR.0],[D0AR.0]
        DB        ASLS      D0Re0,[D0AR.0],[D0AR.0]
        DLB       ASLS      D0Re0,[D0AR.0],[D0AR.0]
        DB        ASRS      D0Re0,[D0AR.0],[D0AR.0]
        DLB       ASRS      D0Re0,[D0AR.0],[D0AR.0]
        DB        LSLS      D0Re0,[D0AR.0],[D0AR.0++]
        DLB       LSLS      D0Re0,AC0.0,[D0AR.0++]
        DB        LSRS      D0Re0,AC0.0,[D0AR.0++]
        DLB       LSRS      D0Re0,AC0.0,[D0AR.0++]
        DB        ASLS      D0Re0,AC0.0,[D0AR.0++]
        DLB       ASLS      D0Re0,AC0.0,[D0AR.0++]
        DB        ASRS      D0Re0,AC0.0,[D0AR.0++]
        DLB       ASRS      D0Re0,AC0.0,[D0AR.0++]
        DB        LSLS      D0Re0,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSRS      D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        DL        LSRS      D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASLS      D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        DL        ASLS      D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASRS      D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        DL        ASRS      D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSLS      D0Re0,[D0AR.0++],D0Re0
        D         LSRS      D0Re0,[D0AR.0++],D0Re0
        D         ASLS      D0Re0,[D0AR.0++],D0Re0
        D         ASRS      D0Re0,[D0AR.0++],D0Re0
        D         LSLS      D0Re0,[D0AR.0++],D0.7
        D         LSRS      D0Re0,[D0AR.0++],D0.7
        D         ASLS      D0Re0,[D0AR.0++],D0.7
        D         ASRS      D0Re0,[D0AR.0++],D0.7
        D         LSLS      D0Re0,[D0AR.0++],[D0AR.0]
        D         LSRS      D0Re0,[D0AR.0++],[D0AR.0]
        D         ASLS      D0Re0,[D0AR.0++],[D0AR.0]
        D         ASRS      D0Re0,[D0AR.0++],[D0AR.0]
        D         LSLS      D0Re0,[D0AR.0++],[D0AR.0++]
        D         LSRS      D0Re0,[D0AR.0++],[D0AR.0++]
        D         ASLS      D0Re0,[D0AR.0++],[D0AR.0++]
        D         ASRS      D0Re0,[D0AR.0++],[D0AR.0++]
        D         LSLS      D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSRS      D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASLS      D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASRS      D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSLS      D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         LSRS      D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         ASLS      D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         ASRS      D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         LSLS      D0Re0,[D0AR.0+D0ARI.0++],D0.7
        D         LSRS      D0Re0,[D0AR.0+D0ARI.0++],D0.7
        D         ASLS      D0Re0,[D0AR.0+D0ARI.0++],D0.7
        D         ASRS      D0Re0,[D0AR.0+D0ARI.0++],D0.7
        D         LSLS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSRS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASLS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASRS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSLS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSRS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASLS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASRS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSLS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSRS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASLS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASRS      D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DT        LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DLT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DWT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DT        LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DLT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DWT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DT        ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DLT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DWT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DT        ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DLT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DWT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T5
        DT        LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DLT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DWT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DT        LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DLT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DWT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DT        ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DLT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DWT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DT        ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DLT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DWT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0FrP++]
        DT        LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DLT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DWT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DT        LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DLT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DWT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DT        ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DLT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DWT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DT        ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DLT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DWT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T5
        DT        LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DLT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DWT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DT        LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DLT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DWT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DT        ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DLT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DWT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DT        ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DLT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DWT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.7++]
        DT        LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DLT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DWT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DT        LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DLT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DWT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DT        ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DLT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DWT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DT        ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DLT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DWT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T5
        DT        LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DLT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DWT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DT        LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DLT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DWT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DT        ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DLT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DWT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DT        ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DLT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DWT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP+A0.8++]
        DT        LSLS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DLT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DWT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DT        LSRS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DLT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DWT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DT        ASLS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DLT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DWT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DT        ASRS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DLT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DWT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 [A0FrP],T5
        DT        LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DLT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DWT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DT        LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DLT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DWT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DT        ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DLT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DWT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DT        ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DLT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DWT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP]
        DT        LSLS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DLT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DWT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DT        LSRS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DLT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DWT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DT        ASLS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DLT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DWT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DT        ASRS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DLT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DWT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 [A0FrP++],T5
        DT        LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DLT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DWT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DT        LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DLT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DWT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DT        ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DLT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DWT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DT        ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DLT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DWT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP++]
        DT        LSLS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DLT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DWT       LSLS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DT        LSRS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DLT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DWT       LSRS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DT        ASLS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DLT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DWT       ASLS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DT        ASRS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DLT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DWT       ASRS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 [A0FrP--],T5
        DT        LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DLT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DWT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DT        LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DLT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DWT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DT        ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DLT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DWT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DT        ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DLT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DWT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A0FrP--]
        D         LSLS      D0.7,D0Re0,D0Re0
        D         LSRS      D0.7,D0Re0,D0Re0
        D         ASLS      D0.7,D0Re0,D0Re0
        D         ASRS      D0.7,D0Re0,D0Re0
        D         LSLS      D0.7,D0Re0,D0.7
        D         LSRS      D0.7,D0Re0,D0.7
        D         ASLS      D0.7,D0Re0,D0.7
        D         ASRS      D0.7,D0Re0,D0.7
        D         LSLS      D0.7,D0Re0,[D0AR.0]
        D         LSRS      D0.7,D0Re0,[D0AR.0]
        D         ASLS      D0.7,D0Re0,[D0AR.0]
        D         ASRS      D0.7,D0Re0,[D0AR.0]
        D         LSLS      D0.7,D0Re0,[D0AR.0++]
        D         LSRS      D0.7,D0Re0,[D0AR.0++]
        D         ASLS      D0.7,D0Re0,[D0AR.0++]
        D         ASRS      D0.7,D0Re0,[D0AR.0++]
        D         LSLS      D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSRS      D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASLS      D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASRS      D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DT        LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DLT       LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DWT       LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DT        LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DLT       LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DWT       LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DT        ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DLT       ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DWT       ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DT        ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DLT       ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DWT       ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T5
        DT        LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DLT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DWT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DT        LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DLT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DWT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DT        ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DLT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DWT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DT        ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DLT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DWT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0FrP++]
        D         LSLS      D0.7,D0.7,D0Re0
        D         LSRS      D0.7,D0.7,D0Re0
        D         ASLS      D0.7,D0.7,D0Re0
        D         ASRS      D0.7,D0.7,D0Re0
        DT        LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DLT       LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DWT       LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DT        LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DLT       LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DWT       LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DT        ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DLT       ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DWT       ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DT        ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DLT       ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DWT       ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T5
        D         LSLS      D0.7,D0.7,D0.7
        D         LSRS      D0.7,D0.7,D0.7
        D         ASLS      D0.7,D0.7,D0.7
        D         ASRS      D0.7,D0.7,D0.7
        D         LSLS      D0.7,D0.7,[D0AR.0]
        D         LSRS      D0.7,D0.7,[D0AR.0]
        D         ASLS      D0.7,D0.7,[D0AR.0]
        D         ASRS      D0.7,D0.7,[D0AR.0]
        D         LSLS      D0.7,D0.7,[D0AR.0++]
        D         LSRS      D0.7,D0.7,[D0AR.0++]
        D         ASLS      D0.7,D0.7,[D0AR.0++]
        D         ASRS      D0.7,D0.7,[D0AR.0++]
        D         LSLS      D0.7,D0.7,[D0AR.0+D0ARI.0++]
        D         LSRS      D0.7,D0.7,[D0AR.0+D0ARI.0++]
        D         ASLS      D0.7,D0.7,[D0AR.0+D0ARI.0++]
        D         ASRS      D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DT        LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DLT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DWT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DT        LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DLT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DWT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DT        ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DLT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DWT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DT        ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DLT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DWT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.7++]
        DT        LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DLT       LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DWT       LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DT        LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DLT       LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DWT       LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DT        ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DLT       ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DWT       ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DT        ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DLT       ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DWT       ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T5
        DT        LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DLT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DWT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DLWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DT        LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DLT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DWT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DLWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DT        ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DLT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DWT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DLWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DT        ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DLT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DWT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        DLWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A0.7+A0.8++]
        D         LSLS      D0.7,[D0AR.0],D0Re0
        DL        LSLS      D0.7,[D0AR.0],D0Re0
        D         LSRS      D0.7,AC0.0,D0Re0
        DL        LSRS      D0.7,AC0.0,D0Re0
        D         ASLS      D0.7,AC0.0,D0Re0
        DL        ASLS      D0.7,AC0.0,D0Re0
        D         ASRS      D0.7,AC0.0,D0Re0
        DL        ASRS      D0.7,AC0.0,D0Re0
        DT        LSLS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DLT       LSLS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DWT       LSLS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DLWT      LSLS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DT        LSRS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DLT       LSRS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DWT       LSRS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DLWT      LSRS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DT        ASLS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DLT       ASLS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DWT       ASLS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DLWT      ASLS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DT        ASRS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DLT       ASRS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DWT       ASRS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        DLWT      ASRS      D0Re0,AC0.0,D0Re0 [A0.7],T5
        D         LSLS      D0.7,AC0.0,D0.7
        DLR       LSLS      D0.7,[D0AR.0],D0.7
        DR        LSRS      D0.7,[D0AR.0],D0.7
        DLR       LSRS      D0.7,[D0AR.0],D0.7
        DR        ASLS      D0.7,[D0AR.0],D0.7
        DLR       ASLS      D0.7,[D0AR.0],D0.7
        DR        ASRS      D0.7,[D0AR.0],D0.7
        DLR       ASRS      D0.7,[D0AR.0],D0.7
        DR        LSLS      D0.7,[D0AR.0],[D0AR.0]
        DLR       LSLS      D0.7,AC0.0,[D0AR.0]
        DR        LSRS      D0.7,AC0.0,[D0AR.0]
        DLR       LSRS      D0.7,AC0.0,[D0AR.0]
        DR        ASLS      D0.7,AC0.0,[D0AR.0]
        DLR       ASLS      D0.7,AC0.0,[D0AR.0]
        DR        ASRS      D0.7,AC0.0,[D0AR.0]
        DLR       ASRS      D0.7,AC0.0,[D0AR.0]
        DR        LSLS      D0.7,AC0.0,[D0AR.0++]
        DLG       LSLS      D0.7,[D0AR.0],[D0AR.0++]
        DG        LSRS      D0.7,[D0AR.0],[D0AR.0++]
        DLG       LSRS      D0.7,[D0AR.0],[D0AR.0++]
        DG        ASLS      D0.7,[D0AR.0],[D0AR.0++]
        DLG       ASLS      D0.7,[D0AR.0],[D0AR.0++]
        DG        ASRS      D0.7,[D0AR.0],[D0AR.0++]
        DLG       ASRS      D0.7,[D0AR.0],[D0AR.0++]
        DG        LSLS      D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DLG       LSLS      D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        DG        LSRS      D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        DLG       LSRS      D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        DG        ASLS      D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        DLG       ASLS      D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        DG        ASRS      D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        DLG       ASRS      D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        DGT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DGT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DGT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DGT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7]
        DLR       LSLS      D0.7,[D0AR.0++],D0Re0
        DLR       LSRS      D0.7,[D0AR.0++],D0Re0
        DLR       ASLS      D0.7,[D0AR.0++],D0Re0
        DLR       ASRS      D0.7,[D0AR.0++],D0Re0
        DGT       LSLS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DGT       LSRS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DGT       ASLS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DGT       ASRS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 [A0.7++],T5
        DLR       LSLS      D0.7,[D0AR.0++],D0.7
        DLR       LSRS      D0.7,[D0AR.0++],D0.7
        DLR       ASLS      D0.7,[D0AR.0++],D0.7
        DLR       ASRS      D0.7,[D0AR.0++],D0.7
        DLR       LSLS      D0.7,[D0AR.0++],[D0AR.0]
        DLR       LSRS      D0.7,[D0AR.0++],[D0AR.0]
        DLR       ASLS      D0.7,[D0AR.0++],[D0AR.0]
        DLR       ASRS      D0.7,[D0AR.0++],[D0AR.0]
        DLR       LSLS      D0.7,[D0AR.0++],[D0AR.0++]
        DLR       LSRS      D0.7,[D0AR.0++],[D0AR.0++]
        DLR       ASLS      D0.7,[D0AR.0++],[D0AR.0++]
        DLR       ASRS      D0.7,[D0AR.0++],[D0AR.0++]
        DLR       LSLS      D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLR       LSRS      D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLR       ASLS      D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLR       ASRS      D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DGT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DGT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DGT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DGT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7++]
        DLR       LSLS      D0.7,[D0AR.0+D0ARI.0++],D0Re0
        DLR       LSRS      D0.7,[D0AR.0+D0ARI.0++],D0Re0
        DLR       ASLS      D0.7,[D0AR.0+D0ARI.0++],D0Re0
        DLR       ASRS      D0.7,[D0AR.0+D0ARI.0++],D0Re0
        DLR       LSLS      D0.7,[D0AR.0+D0ARI.0++],D0.7
        DLR       LSRS      D0.7,[D0AR.0+D0ARI.0++],D0.7
        DLR       ASLS      D0.7,[D0AR.0+D0ARI.0++],D0.7
        DLR       ASRS      D0.7,[D0AR.0+D0ARI.0++],D0.7
        DLR       LSLS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        DLR       LSRS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        DLR       ASLS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        DLR       ASRS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        DLR       LSLS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLR       LSRS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLR       ASLS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLR       ASRS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLR       LSLS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLR       LSRS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLR       ASLS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLR       ASRS      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DGT       LSLS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DGT       LSRS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DGT       ASLS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DGT       ASRS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 [A0.7--],T5
        DGT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DGT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DGT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DGT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.7--]
        DGT       LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DGT       LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DGT       ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DGT       ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0FrP++],T5
        DGT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DGT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DGT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DGT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0FrP++]
        DGT       LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DGT       LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DGT       ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DGT       ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.7++],T5
        DGT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DGT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DGT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DGT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.7++]
        DGT       LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DGT       LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DGT       ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DGT       ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 [A0.8+A0.8++],T5
        DGT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DGT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DGT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DGT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8+A0.8++]
        DGT       LSLS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DGT       LSRS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DGT       ASLS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DGT       ASRS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 [A0.8],T5
        DGT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DGT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DGT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DGT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8]
        DGT       LSLS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DGT       LSRS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DGT       ASLS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DGT       ASRS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 [A0.8++],T5
        DGT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DGT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DGT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DGT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8++]
        DGT       LSLS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DGT       LSRS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DGT       ASLS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DGT       ASRS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 [A0.8--],T5
        DGT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DLGT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DGWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DLGWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DGT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DLGT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DGWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DLGWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DGT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DLGT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DGWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DLGWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DGT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DLGT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DGWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DLGWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A0.8--]
        DLR       LSLS      [D0AW.0],D0Re0,D0Re0
        DLR       LSRS      [D0AW.0],D0Re0,D0Re0
        DLR       ASLS      [D0AW.0],D0Re0,D0Re0
        DLR       ASRS      [D0AW.0],D0Re0,D0Re0
        DLR       LSLS      [D0AW.0],D0Re0,D0.7
        DLR       LSRS      [D0AW.0],D0Re0,D0.7
        DLR       ASLS      [D0AW.0],D0Re0,D0.7
        DLR       ASRS      [D0AW.0],D0Re0,D0.7
        DLR       LSLS      [D0AW.0],D0Re0,[D0AR.0]
        DLR       LSRS      [D0AW.0],D0Re0,[D0AR.0]
        DLR       ASLS      [D0AW.0],D0Re0,[D0AR.0]
        DLR       ASRS      [D0AW.0],D0Re0,[D0AR.0]
        DLR       LSLS      [D0AW.0],D0Re0,[D0AR.0++]
        DLR       LSRS      [D0AW.0],D0Re0,[D0AR.0++]
        DLR       ASLS      [D0AW.0],D0Re0,[D0AR.0++]
        DLR       ASRS      [D0AW.0],D0Re0,[D0AR.0++]
        DLR       LSLS      [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        DLR       LSRS      [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        DLR       ASLS      [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        DLR       ASRS      [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        DLR       LSLS      [D0AW.0],D0.7,D0Re0
        DLR       LSRS      [D0AW.0],D0.7,D0Re0
        DLR       ASLS      [D0AW.0],D0.7,D0Re0
        DLR       ASRS      [D0AW.0],D0.7,D0Re0
        DLR       LSLS      [D0AW.0],D0.7,D0.7
        DLR       LSRS      [D0AW.0],D0.7,D0.7
        DLR       ASLS      [D0AW.0],D0.7,D0.7
        DLR       ASRS      [D0AW.0],D0.7,D0.7
        DLR       LSLS      [D0AW.0],D0.7,[D0AR.0]
        DLR       LSRS      [D0AW.0],D0.7,[D0AR.0]
        DLR       ASLS      [D0AW.0],D0.7,[D0AR.0]
        DLR       ASRS      [D0AW.0],D0.7,[D0AR.0]
        DLR       LSLS      [D0AW.0],D0.7,[D0AR.0++]
        DLR       LSRS      [D0AW.0],D0.7,[D0AR.0++]
        DLR       ASLS      [D0AW.0],D0.7,[D0AR.0++]
        DLR       ASRS      [D0AW.0],D0.7,[D0AR.0++]
        DLR       LSLS      [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        DLR       LSRS      [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        DLR       ASLS      [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        DLR       ASRS      [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        DG        LSLS      [D0AW.0],AC0.0,D0Re0
        DLB       LSLS      [D0AW.0],[D0AR.0],D0Re0
        DB        LSRS      [D0AW.0],[D0AR.0],D0Re0
        DLB       LSRS      [D0AW.0],[D0AR.0],D0Re0
        DB        ASLS      [D0AW.0],[D0AR.0],D0Re0
        DLB       ASLS      [D0AW.0],[D0AR.0],D0Re0
        DB        ASRS      [D0AW.0],[D0AR.0],D0Re0
        DLB       ASRS      [D0AW.0],[D0AR.0],D0Re0
        DB        LSLS      [D0AW.0],[D0AR.0],D0.7
        DLB       LSLS      [D0AW.0],AC0.0,D0.7
        DB        LSRS      [D0AW.0],AC0.0,D0.7
        DLB       LSRS      [D0AW.0],AC0.0,D0.7
        DB        ASLS      [D0AW.0],AC0.0,D0.7
        DLB       ASLS      [D0AW.0],AC0.0,D0.7
        DB        ASRS      [D0AW.0],AC0.0,D0.7
        DLB       ASRS      [D0AW.0],AC0.0,D0.7
        DB        LSLS      [D0AW.0],AC0.0,[D0AR.0]
        D         LSRS      [D0AW.0],[D0AR.0],[D0AR.0]
        DL        LSRS      [D0AW.0],[D0AR.0],[D0AR.0]
        D         ASLS      [D0AW.0],[D0AR.0],[D0AR.0]
        DL        ASLS      [D0AW.0],[D0AR.0],[D0AR.0]
        D         ASRS      [D0AW.0],[D0AR.0],[D0AR.0]
        DL        ASRS      [D0AW.0],[D0AR.0],[D0AR.0]
        D         LSLS      [D0AW.0],[D0AR.0],[D0AR.0++]
        DL        LSLS      [D0AW.0],[D0AR.0],[D0AR.0++]
        D         LSRS      [D0AW.0],AC0.0,[D0AR.0++]
        DL        LSRS      [D0AW.0],AC0.0,[D0AR.0++]
        D         ASLS      [D0AW.0],AC0.0,[D0AR.0++]
        DL        ASLS      [D0AW.0],AC0.0,[D0AR.0++]
        D         ASRS      [D0AW.0],AC0.0,[D0AR.0++]
        DL        ASRS      [D0AW.0],AC0.0,[D0AR.0++]
        D         LSLS      [D0AW.0],AC0.0,[D0AR.0+D0ARI.0++]
        DLR       LSLS      [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DR        LSRS      [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLR       LSRS      [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DR        ASLS      [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLR       ASLS      [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DR        ASRS      [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLR       ASRS      [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DG        LSLS      [D0AW.0],[D0AR.0++],D0Re0
        DG        LSRS      [D0AW.0],[D0AR.0++],D0Re0
        DG        ASLS      [D0AW.0],[D0AR.0++],D0Re0
        DG        ASRS      [D0AW.0],[D0AR.0++],D0Re0
        DG        LSLS      [D0AW.0],[D0AR.0++],D0.7
        DG        LSRS      [D0AW.0],[D0AR.0++],D0.7
        DG        ASLS      [D0AW.0],[D0AR.0++],D0.7
        DG        ASRS      [D0AW.0],[D0AR.0++],D0.7
        DG        LSLS      [D0AW.0],[D0AR.0++],[D0AR.0]
        DG        LSRS      [D0AW.0],[D0AR.0++],[D0AR.0]
        DG        ASLS      [D0AW.0],[D0AR.0++],[D0AR.0]
        DG        ASRS      [D0AW.0],[D0AR.0++],[D0AR.0]
        DG        LSLS      [D0AW.0],[D0AR.0++],[D0AR.0++]
        DG        LSRS      [D0AW.0],[D0AR.0++],[D0AR.0++]
        DG        ASLS      [D0AW.0],[D0AR.0++],[D0AR.0++]
        DG        ASRS      [D0AW.0],[D0AR.0++],[D0AR.0++]
        DG        LSLS      [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DG        LSRS      [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DG        ASLS      [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DG        ASRS      [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DG        LSLS      [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DG        LSRS      [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DG        ASLS      [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DG        ASRS      [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DG        LSLS      [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DG        LSRS      [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DG        ASLS      [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DG        ASRS      [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DG        LSLS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DG        LSRS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DG        ASLS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DG        ASRS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DG        LSLS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DG        LSRS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DG        ASLS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DG        ASRS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DG        LSLS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DG        LSRS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DG        ASLS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DG        ASRS      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DRT       LSLS      D0Re0,D0Re0,D0Re0 [D0AW.0],T5
        DLRT      LSLS      D0Re0,D0Re0,D0Re0 [D0AW.0],T5
        DRT       LSRS      D0Re0,D0Re0,D0Re0 [D0AW.0],T5
        DLRT      LSRS      D0Re0,D0Re0,D0Re0 [D0AW.0],T5
        DRT       ASLS      D0Re0,D0Re0,D0Re0 [D0AW.0],T5
        DLRT      ASLS      D0Re0,D0Re0,D0Re0 [D0AW.0],T5
        DRT       ASRS      D0Re0,D0Re0,D0Re0 [D0AW.0],T5
        DLRT      ASRS      D0Re0,D0Re0,D0Re0 [D0AW.0],T5
        DRT       LSLS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0]
        DLRT      LSLS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0]
        DRT       LSRS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0]
        DLRT      LSRS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0]
        DRT       ASLS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0]
        DLRT      ASLS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0]
        DRT       ASRS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0]
        DLRT      ASRS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0]
        DG        LSLS      [D0AW.0++],D0Re0,D0Re0
        DG        LSRS      [D0AW.0++],D0Re0,D0Re0
        DG        ASLS      [D0AW.0++],D0Re0,D0Re0
        DG        ASRS      [D0AW.0++],D0Re0,D0Re0
        DG        LSLS      [D0AW.0++],D0Re0,D0.7
        DG        LSRS      [D0AW.0++],D0Re0,D0.7
        DG        ASLS      [D0AW.0++],D0Re0,D0.7
        DG        ASRS      [D0AW.0++],D0Re0,D0.7
        DG        LSLS      [D0AW.0++],D0Re0,[D0AR.0]
        DG        LSRS      [D0AW.0++],D0Re0,[D0AR.0]
        DG        ASLS      [D0AW.0++],D0Re0,[D0AR.0]
        DG        ASRS      [D0AW.0++],D0Re0,[D0AR.0]
        DG        LSLS      [D0AW.0++],D0Re0,[D0AR.0++]
        DG        LSRS      [D0AW.0++],D0Re0,[D0AR.0++]
        DG        ASLS      [D0AW.0++],D0Re0,[D0AR.0++]
        DG        ASRS      [D0AW.0++],D0Re0,[D0AR.0++]
        DG        LSLS      [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DG        LSRS      [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DG        ASLS      [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DG        ASRS      [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DRT       LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DLRT      LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DRWT      LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DLRWT     LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DRT       LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DLRT      LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DRWT      LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DLRWT     LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DRT       ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DLRT      ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DRWT      ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DLRWT     ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DRT       ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DLRT      ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DRWT      ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DLRWT     ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T5
        DRT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DLRT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DRWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DLRWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DRT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DLRT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DRWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DLRWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DRT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DLRT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DRWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DLRWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DRT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DLRT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DRWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DLRWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1LbP++]
        DG        LSLS      [D0AW.0++],D0.7,D0Re0
        DG        LSRS      [D0AW.0++],D0.7,D0Re0
        DG        ASLS      [D0AW.0++],D0.7,D0Re0
        DG        ASRS      [D0AW.0++],D0.7,D0Re0
        DRT       LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DLRT      LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DRWT      LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DLRWT     LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DRT       LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DLRT      LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DRWT      LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DLRWT     LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DRT       ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DLRT      ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DRWT      ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DLRWT     ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DRT       ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DLRT      ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DRWT      ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DLRWT     ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T5
        DG        LSLS      [D0AW.0++],D0.7,D0.7
        DG        LSRS      [D0AW.0++],D0.7,D0.7
        DG        ASLS      [D0AW.0++],D0.7,D0.7
        DG        ASRS      [D0AW.0++],D0.7,D0.7
        DG        LSLS      [D0AW.0++],D0.7,[D0AR.0]
        DG        LSRS      [D0AW.0++],D0.7,[D0AR.0]
        DG        ASLS      [D0AW.0++],D0.7,[D0AR.0]
        DG        ASRS      [D0AW.0++],D0.7,[D0AR.0]
        DG        LSLS      [D0AW.0++],D0.7,[D0AR.0++]
        DG        LSRS      [D0AW.0++],D0.7,[D0AR.0++]
        DG        ASLS      [D0AW.0++],D0.7,[D0AR.0++]
        DG        ASRS      [D0AW.0++],D0.7,[D0AR.0++]
        DG        LSLS      [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DG        LSRS      [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DG        ASLS      [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DG        ASRS      [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DRT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DLRT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DRWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DLRWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DRT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DLRT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DRWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DLRWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DRT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DLRT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DRWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DLRWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DRT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DLRT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DRWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DLRWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.7++]
        DRT       LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DLRT      LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DRWT      LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DLRWT     LSLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DRT       LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DLRT      LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DRWT      LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DLRWT     LSRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DRT       ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DLRT      ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DRWT      ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DLRWT     ASLS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DRT       ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DLRT      ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DRWT      ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DLRWT     ASRS      D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T5
        DRT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DLRT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DRWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DLRWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DRT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DLRT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DRWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DLRWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DRT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DLRT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DRWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DLRWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DRT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DLRT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DRWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DLRWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1LbP+A1.8++]
        DR        LSLS      [D0AW.0++],[D0AR.0],D0Re0
        DLR       LSLS      [D0AW.0++],AC0.0,D0Re0
        DR        LSRS      [D0AW.0++],AC0.0,D0Re0
        DLR       LSRS      [D0AW.0++],AC0.0,D0Re0
        DR        ASLS      [D0AW.0++],AC0.0,D0Re0
        DLR       ASLS      [D0AW.0++],AC0.0,D0Re0
        DR        ASRS      [D0AW.0++],AC0.0,D0Re0
        DLR       ASRS      [D0AW.0++],AC0.0,D0Re0
        DRT       LSLS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DLRT      LSLS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DRWT      LSLS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DLRWT     LSLS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DRT       LSRS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DLRT      LSRS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DRWT      LSRS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DLRWT     LSRS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DRT       ASLS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DLRT      ASLS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DRWT      ASLS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DLRWT     ASLS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DRT       ASRS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DLRT      ASRS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DRWT      ASRS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DLRWT     ASRS      D0Re0,AC0.0,D0Re0 [A1LbP],T5
        DR        LSLS      [D0AW.0++],AC0.0,D0.7
        DLG       LSLS      [D0AW.0++],[D0AR.0],D0.7
        DG        LSRS      [D0AW.0++],[D0AR.0],D0.7
        DLG       LSRS      [D0AW.0++],[D0AR.0],D0.7
        DG        ASLS      [D0AW.0++],[D0AR.0],D0.7
        DLG       ASLS      [D0AW.0++],[D0AR.0],D0.7
        DG        ASRS      [D0AW.0++],[D0AR.0],D0.7
        DLG       ASRS      [D0AW.0++],[D0AR.0],D0.7
        DG        LSLS      [D0AW.0++],[D0AR.0],[D0AR.0]
        DLG       LSLS      [D0AW.0++],AC0.0,[D0AR.0]
        DG        LSRS      [D0AW.0++],AC0.0,[D0AR.0]
        DLG       LSRS      [D0AW.0++],AC0.0,[D0AR.0]
        DG        ASLS      [D0AW.0++],AC0.0,[D0AR.0]
        DLG       ASLS      [D0AW.0++],AC0.0,[D0AR.0]
        DG        ASRS      [D0AW.0++],AC0.0,[D0AR.0]
        DLG       ASRS      [D0AW.0++],AC0.0,[D0AR.0]
        DG        LSLS      [D0AW.0++],AC0.0,[D0AR.0++]
        DLB       LSLS      [D0AW.0++],[D0AR.0],[D0AR.0++]
        DB        LSRS      [D0AW.0++],[D0AR.0],[D0AR.0++]
        DLB       LSRS      [D0AW.0++],[D0AR.0],[D0AR.0++]
        DB        ASLS      [D0AW.0++],[D0AR.0],[D0AR.0++]
        DLB       ASLS      [D0AW.0++],[D0AR.0],[D0AR.0++]
        DB        ASRS      [D0AW.0++],[D0AR.0],[D0AR.0++]
        DLB       ASRS      [D0AW.0++],[D0AR.0],[D0AR.0++]
        DB        LSLS      [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLB       LSLS      [D0AW.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DB        LSRS      [D0AW.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DLB       LSRS      [D0AW.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DB        ASLS      [D0AW.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DLB       ASLS      [D0AW.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DB        ASRS      [D0AW.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DLB       ASRS      [D0AW.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DBT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DLBT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DBWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DLBWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DBT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DLBT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DBWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DLBWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DBT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DLBT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DBWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DLBWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DBT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DLBT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DBWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DLBWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP]
        DLB       LSLS      [D0AW.0++],[D0AR.0++],D0Re0
        DLB       LSRS      [D0AW.0++],[D0AR.0++],D0Re0
        DLB       ASLS      [D0AW.0++],[D0AR.0++],D0Re0
        DLB       ASRS      [D0AW.0++],[D0AR.0++],D0Re0
        DBT       LSLS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DLBT      LSLS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DBWT      LSLS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DLBWT     LSLS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DBT       LSRS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DLBT      LSRS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DBWT      LSRS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DLBWT     LSRS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DBT       ASLS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DLBT      ASLS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DBWT      ASLS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DLBWT     ASLS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DBT       ASRS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DLBT      ASRS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DBWT      ASRS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DLBWT     ASRS      D0Re0,AC0.0,D0Re0 [A1LbP++],T5
        DLB       LSLS      [D0AW.0++],[D0AR.0++],D0.7
        DLB       LSRS      [D0AW.0++],[D0AR.0++],D0.7
        DLB       ASLS      [D0AW.0++],[D0AR.0++],D0.7
        DLB       ASRS      [D0AW.0++],[D0AR.0++],D0.7
        DLB       LSLS      [D0AW.0++],[D0AR.0++],[D0AR.0]
        DLB       LSRS      [D0AW.0++],[D0AR.0++],[D0AR.0]
        DLB       ASLS      [D0AW.0++],[D0AR.0++],[D0AR.0]
        DLB       ASRS      [D0AW.0++],[D0AR.0++],[D0AR.0]
        DLB       LSLS      [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DLB       LSRS      [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DLB       ASLS      [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DLB       ASRS      [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DLB       LSLS      [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLB       LSRS      [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLB       ASLS      [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLB       ASRS      [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DBT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DLBT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DBWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DLBWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DBT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DLBT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DBWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DLBWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DBT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DLBT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DBWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DLBWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DBT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DLBT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DBWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DLBWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP++]
        DLB       LSLS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLB       LSRS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLB       ASLS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLB       ASRS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DLB       LSLS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DLB       LSRS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DLB       ASLS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DLB       ASRS      [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DLB       LSLS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLB       LSRS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLB       ASLS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLB       ASRS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLB       LSLS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLB       LSRS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLB       ASLS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLB       ASRS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLB       LSLS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLB       LSRS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLB       ASLS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLB       ASRS      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DBT       LSLS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DLBT      LSLS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DBWT      LSLS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DLBWT     LSLS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DBT       LSRS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DLBT      LSRS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DBWT      LSRS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DLBWT     LSRS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DBT       ASLS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DLBT      ASLS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DBWT      ASLS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DLBWT     ASLS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DBT       ASRS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DLBT      ASRS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DBWT      ASRS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DLBWT     ASRS      D0Re0,AC0.0,D0Re0 [A1LbP--],T5
        DBT       LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DLBT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DBWT      LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DLBWT     LSLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DBT       LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DLBT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DBWT      LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DLBWT     LSRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DBT       ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DLBT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DBWT      ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DLBWT     ASLS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DBT       ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DLBT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DBWT      ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DLBWT     ASRS      D0Re0,AC0.0,D0Re0 T5,[A1LbP--]
        DBT       LSLS      D0Re0,AC0.0,D0Re0 [D0AW.0++],T5
        DLBT      LSLS      D0Re0,AC0.0,D0Re0 [D0AW.0++],T5
        DBT       LSRS      D0Re0,AC0.0,D0Re0 [D0AW.0++],T5
        DLBT      LSRS      D0Re0,AC0.0,D0Re0 [D0AW.0++],T5
        DBT       ASLS      D0Re0,AC0.0,D0Re0 [D0AW.0++],T5
        DLBT      ASLS      D0Re0,AC0.0,D0Re0 [D0AW.0++],T5
        DBT       ASRS      D0Re0,AC0.0,D0Re0 [D0AW.0++],T5
        DLBT      ASRS      D0Re0,AC0.0,D0Re0 [D0AW.0++],T5
        DBT       LSLS      D0Re0,AC0.0,D0Re0 T5,[D0AR.0++]
        DLBT      LSLS      D0Re0,AC0.0,D0Re0 T5,[D0AR.0++]
        DBT       LSRS      D0Re0,AC0.0,D0Re0 T5,[D0AR.0++]
        DLBT      LSRS      D0Re0,AC0.0,D0Re0 T5,[D0AR.0++]
        DBT       ASLS      D0Re0,AC0.0,D0Re0 T5,[D0AR.0++]
        DLBT      ASLS      D0Re0,AC0.0,D0Re0 T5,[D0AR.0++]
        DBT       ASRS      D0Re0,AC0.0,D0Re0 T5,[D0AR.0++]
        DLBT      ASRS      D0Re0,AC0.0,D0Re0 T5,[D0AR.0++]
        DLB       LSLS      [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DLB       LSRS      [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DLB       ASLS      [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DLB       ASRS      [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DLB       LSLS      [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DLB       LSRS      [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DLB       ASLS      [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DLB       ASRS      [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DLB       LSLS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DLB       LSRS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DLB       ASLS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DLB       ASRS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        DLB       LSLS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DLB       LSRS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DLB       ASLS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DLB       ASRS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DLB       LSLS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DLB       LSRS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DLB       ASLS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DLB       ASRS      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DLB       LSLS      [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DLB       LSRS      [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DLB       ASLS      [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DLB       ASRS      [D0AW.0+D0AWI.0++],D0.7,D0Re0
        DLB       LSLS      [D0AW.0+D0AWI.0++],D0.7,D0.7
        DLB       LSRS      [D0AW.0+D0AWI.0++],D0.7,D0.7
        DLB       ASLS      [D0AW.0+D0AWI.0++],D0.7,D0.7
        DLB       ASRS      [D0AW.0+D0AWI.0++],D0.7,D0.7
        DLB       LSLS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DLB       LSRS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DLB       ASLS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DLB       ASRS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DLB       LSLS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DLB       LSRS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DLB       ASLS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DLB       ASRS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DLB       LSLS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DLB       LSRS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DLB       ASLS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DLB       ASRS      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DB        LSLS      [D0AW.0+D0AWI.0++],AC0.0,D0Re0
        D         LSRS      [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        D         ASLS      [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        D         ASRS      [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        D         LSLS      [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        D         LSRS      [D0AW.0+D0AWI.0++],AC0.0,D0.7
        DL        LSRS      [D0AW.0+D0AWI.0++],AC0.0,D0.7
        D         ASLS      [D0AW.0+D0AWI.0++],AC0.0,D0.7
        DL        ASLS      [D0AW.0+D0AWI.0++],AC0.0,D0.7
        D         ASRS      [D0AW.0+D0AWI.0++],AC0.0,D0.7
        DL        ASRS      [D0AW.0+D0AWI.0++],AC0.0,D0.7
        D         LSLS      [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0]
        DLR       LSLS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DR        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DLR       LSRS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DR        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DLR       ASLS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DR        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DLR       ASRS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        DR        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        DLR       LSLS      [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0++]
        DR        LSRS      [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0++]
        DLR       LSRS      [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0++]
        DR        ASLS      [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0++]
        DLR       ASLS      [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0++]
        DR        ASRS      [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0++]
        DLR       ASRS      [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0++]
        DR        LSLS      [D0AW.0+D0AWI.0++],AC0.0,[D0AR.0+D0ARI.0++]
        DLG       LSLS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DG        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLG       LSRS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DG        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLG       ASLS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DG        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLG       ASRS      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0++],D0.7
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[D0AR.0+D0ARI.0++]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1LbP++]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.7++]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7+A1.8++]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.7],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.7++],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7++]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.7--],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.7--]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1LbP++]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.7++]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8+A1.8++]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.8],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.8++],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8++]
        DGT       LSLS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DGT       LSRS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DGT       ASLS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DGT       ASRS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 [A1.8--],T5
        DGT       LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DLGT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DGWT      LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DLGWT     LSLS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DGT       LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DLGT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DGWT      LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DLGWT     LSRS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DGT       ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DLGT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DGWT      ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DLGWT     ASLS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DGT       ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DLGT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DGWT      ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DLGWT     ASRS      D0Re0,D0Re0,D0Re0 T5,[A1.8--]
        DL        LSLS      D1Re0,D1Re0,D1Re0
        DL        LSRS      D1Re0,D1Re0,D1Re0
        DL        ASLS      D1Re0,D1Re0,D1Re0
        DL        ASRS      D1Re0,D1Re0,D1Re0
        DL        LSLS      D1Re0,D1Re0,D1.7
        DL        LSRS      D1Re0,D1Re0,D1.7
        DL        ASLS      D1Re0,D1Re0,D1.7
        DL        ASRS      D1Re0,D1Re0,D1.7
        DL        LSLS      D1Re0,D1Re0,[D1BR.1]
        DL        LSRS      D1Re0,D1Re0,[D1BR.1]
        DL        ASLS      D1Re0,D1Re0,[D1BR.1]
        DL        ASRS      D1Re0,D1Re0,[D1BR.1]
        DL        LSLS      D1Re0,D1Re0,[D1BR.1++]
        DL        LSRS      D1Re0,D1Re0,[D1BR.1++]
        DL        ASLS      D1Re0,D1Re0,[D1BR.1++]
        DL        ASRS      D1Re0,D1Re0,[D1BR.1++]
        DL        LSLS      D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DL        LSRS      D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DL        ASLS      D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DL        ASRS      D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DL        LSLS      D1Re0,D1.7,D1Re0
        DL        LSRS      D1Re0,D1.7,D1Re0
        DL        ASLS      D1Re0,D1.7,D1Re0
        DL        ASRS      D1Re0,D1.7,D1Re0
        DL        LSLS      D1Re0,D1.7,D1.7
        DL        LSRS      D1Re0,D1.7,D1.7
        DL        ASLS      D1Re0,D1.7,D1.7
        DL        ASRS      D1Re0,D1.7,D1.7
        DL        LSLS      D1Re0,D1.7,[D1BR.1]
        DL        LSRS      D1Re0,D1.7,[D1BR.1]
        DL        ASLS      D1Re0,D1.7,[D1BR.1]
        DL        ASRS      D1Re0,D1.7,[D1BR.1]
        DL        LSLS      D1Re0,D1.7,[D1BR.1++]
        DL        LSRS      D1Re0,D1.7,[D1BR.1++]
        DL        ASLS      D1Re0,D1.7,[D1BR.1++]
        DL        ASRS      D1Re0,D1.7,[D1BR.1++]
        DL        LSLS      D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DL        LSRS      D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DL        ASLS      D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DL        ASRS      D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DLR       LSLS      D1Re0,AC1.0,D1Re0
        DLR       LSRS      D1Re0,AC1.0,D1Re0
        DLR       ASLS      D1Re0,AC1.0,D1Re0
        DLR       ASRS      D1Re0,AC1.0,D1Re0
        DLG       LSLS      D1Re0,AC1.0,D1.7
        DLG       LSRS      D1Re0,AC1.0,D1.7
        DLG       ASLS      D1Re0,AC1.0,D1.7
        DLG       ASRS      D1Re0,AC1.0,D1.7
        DLB       LSLS      D1Re0,AC1.0,[D1BR.1]
        DLB       LSRS      D1Re0,AC1.0,[D1BR.1]
        DLB       ASLS      D1Re0,AC1.0,[D1BR.1]
        DLB       ASRS      D1Re0,AC1.0,[D1BR.1]
        D         LSRS      D1Re0,AC1.0,[D1BR.1++]
        D         ASLS      D1Re0,AC1.0,[D1BR.1++]
        D         ASRS      D1Re0,AC1.0,[D1BR.1++]
        D         LSLS      D1Re0,AC1.0,[D1BR.1+D1BRI.1++]
        DR        LSRS      D1Re0,AC1.0,[D1BR.1+D1BRI.1++]
        DR        ASLS      D1Re0,AC1.0,[D1BR.1+D1BRI.1++]
        DR        ASRS      D1Re0,AC1.0,[D1BR.1+D1BRI.1++]
        DR        LSLS      D1Re0,[D1BR.1],D1Re0
        DR        LSRS      D1Re0,[D1BR.1],D1Re0
        DR        ASLS      D1Re0,[D1BR.1],D1Re0
        DR        ASRS      D1Re0,[D1BR.1],D1Re0
        DR        LSLS      D1Re0,[D1BR.1],D1.7
        DR        LSRS      D1Re0,[D1BR.1],D1.7
        DR        ASLS      D1Re0,[D1BR.1],D1.7
        DR        ASRS      D1Re0,[D1BR.1],D1.7
        DR        LSLS      D1Re0,[D1BR.1],[D1BR.1]
        DR        LSRS      D1Re0,[D1BR.1],[D1BR.1]
        DR        ASLS      D1Re0,[D1BR.1],[D1BR.1]
        DR        ASRS      D1Re0,[D1BR.1],[D1BR.1]
        DR        LSLS      D1Re0,[D1BR.1],[D1BR.1++]
        DR        LSRS      D1Re0,[D1BR.1],[D1BR.1++]
        DR        ASLS      D1Re0,[D1BR.1],[D1BR.1++]
        DR        ASRS      D1Re0,[D1BR.1],[D1BR.1++]
        DR        LSLS      D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DR        LSRS      D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DR        ASLS      D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DR        ASRS      D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DR        LSLS      D1Re0,[D1BR.1++],D1Re0
        DR        LSRS      D1Re0,[D1BR.1++],D1Re0
        DR        ASLS      D1Re0,[D1BR.1++],D1Re0
        DR        ASRS      D1Re0,[D1BR.1++],D1Re0
        DR        LSLS      D1Re0,[D1BR.1++],D1.7
        DR        LSRS      D1Re0,[D1BR.1++],D1.7
        DR        ASLS      D1Re0,[D1BR.1++],D1.7
        DR        ASRS      D1Re0,[D1BR.1++],D1.7
        DR        LSLS      D1Re0,[D1BR.1++],[D1BR.1]
        DR        LSRS      D1Re0,[D1BR.1++],[D1BR.1]
        DR        ASLS      D1Re0,[D1BR.1++],[D1BR.1]
        DR        ASRS      D1Re0,[D1BR.1++],[D1BR.1]
        DR        LSLS      D1Re0,[D1BR.1++],[D1BR.1++]
        DR        LSRS      D1Re0,[D1BR.1++],[D1BR.1++]
        DR        ASLS      D1Re0,[D1BR.1++],[D1BR.1++]
        DR        ASRS      D1Re0,[D1BR.1++],[D1BR.1++]
        DR        LSLS      D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DR        LSRS      D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DR        ASLS      D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DR        ASRS      D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DR        LSLS      D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DR        LSRS      D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DR        ASLS      D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DR        ASRS      D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        DR        LSLS      D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DR        LSRS      D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DR        ASLS      D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DR        ASRS      D1Re0,[D1BR.1+D1BRI.1++],D1.7
        DR        LSLS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DR        LSRS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DR        ASLS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DR        ASRS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DR        LSLS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DR        LSRS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DR        ASLS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DR        ASRS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DR        LSLS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DR        LSRS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DR        ASLS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DR        ASRS      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DT        LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DLT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DWT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DT        LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DLT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DWT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DT        ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DLT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DWT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DT        ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DLT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DWT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0FrP++],T5
        DT        LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DLT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DWT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DT        LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DLT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DWT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DT        ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DLT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DWT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DT        ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DLT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DWT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0FrP++]
        DT        LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DLT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DWT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DT        LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DLT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DWT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DT        ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DLT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DWT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DT        ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DLT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DWT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.7++],T5
        DT        LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DLT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DWT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DT        LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DLT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DWT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DT        ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DLT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DWT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DT        ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DLT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DWT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.7++]
        DT        LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DLT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DWT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DT        LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DLT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DWT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DT        ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DLT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DWT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DT        ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DLT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DWT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 [A0FrP+A0.8++],T5
        DT        LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DLT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DWT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DT        LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DLT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DWT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DT        ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DLT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DWT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DT        ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DLT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DWT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP+A0.8++]
        DT        LSLS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DLT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DWT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DT        LSRS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DLT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DWT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DT        ASLS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DLT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DWT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DT        ASRS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DLT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DWT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 [A0FrP],T5
        DT        LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DLT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DWT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DT        LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DLT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DWT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DT        ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DLT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DWT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DT        ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DLT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DWT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP]
        DT        LSLS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DLT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DWT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DT        LSRS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DLT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DWT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DT        ASLS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DLT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DWT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DT        ASRS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DLT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DWT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 [A0FrP++],T5
        DT        LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DLT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DWT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DT        LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DLT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DWT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DT        ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DLT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DWT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DT        ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DLT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DWT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP++]
        DT        LSLS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DLT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DWT       LSLS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DT        LSRS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DLT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DWT       LSRS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DT        ASLS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DLT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DWT       ASLS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DT        ASRS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DLT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DWT       ASRS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 [A0FrP--],T5
        DT        LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DLT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DWT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DT        LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DLT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DWT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DT        ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DLT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DWT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DT        ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DLT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DWT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0FrP--]
        DR        LSLS      D1.7,D1Re0,D1Re0
        DR        LSRS      D1.7,D1Re0,D1Re0
        DR        ASLS      D1.7,D1Re0,D1Re0
        DR        ASRS      D1.7,D1Re0,D1Re0
        DR        LSLS      D1.7,D1Re0,D1.7
        DR        LSRS      D1.7,D1Re0,D1.7
        DR        ASLS      D1.7,D1Re0,D1.7
        DR        ASRS      D1.7,D1Re0,D1.7
        DR        LSLS      D1.7,D1Re0,[D1BR.1]
        DR        LSRS      D1.7,D1Re0,[D1BR.1]
        DR        ASLS      D1.7,D1Re0,[D1BR.1]
        DR        ASRS      D1.7,D1Re0,[D1BR.1]
        DR        LSLS      D1.7,D1Re0,[D1BR.1++]
        DR        LSRS      D1.7,D1Re0,[D1BR.1++]
        DR        ASLS      D1.7,D1Re0,[D1BR.1++]
        DR        ASRS      D1.7,D1Re0,[D1BR.1++]
        DR        LSLS      D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DR        LSRS      D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DR        ASLS      D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DR        ASRS      D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DT        LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DLT       LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DWT       LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DT        LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DLT       LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DWT       LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DT        ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DLT       ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DWT       ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DT        ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DLT       ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DWT       ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0FrP++],T5
        DT        LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DLT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DWT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DT        LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DLT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DWT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DT        ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DLT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DWT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DT        ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DLT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DWT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0FrP++]
        DR        LSLS      D1.7,D1.7,D1Re0
        DR        LSRS      D1.7,D1.7,D1Re0
        DR        ASLS      D1.7,D1.7,D1Re0
        DR        ASRS      D1.7,D1.7,D1Re0
        DT        LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DLT       LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DWT       LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DT        LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DLT       LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DWT       LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DT        ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DLT       ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DWT       ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DT        ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DLT       ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DWT       ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.7++],T5
        DR        LSLS      D1.7,D1.7,D1.7
        DR        LSRS      D1.7,D1.7,D1.7
        DR        ASLS      D1.7,D1.7,D1.7
        DR        ASRS      D1.7,D1.7,D1.7
        DT        LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DLT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DWT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DT        LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DLT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DWT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DT        ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DLT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DWT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DT        ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DLT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DWT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.7++]
        DR        LSLS      D1.7,D1.7,[D1BR.1]
        DR        LSRS      D1.7,D1.7,[D1BR.1]
        DR        ASLS      D1.7,D1.7,[D1BR.1]
        DR        ASRS      D1.7,D1.7,[D1BR.1]
        DR        LSLS      D1.7,D1.7,[D1BR.1++]
        DR        LSRS      D1.7,D1.7,[D1BR.1++]
        DR        ASLS      D1.7,D1.7,[D1BR.1++]
        DR        ASRS      D1.7,D1.7,[D1BR.1++]
        DR        LSLS      D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DR        LSRS      D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DR        ASLS      D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DR        ASRS      D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DT        LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DLT       LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DWT       LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DT        LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DLT       LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DWT       LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DT        ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DLT       ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DWT       ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DT        ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DLT       ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DWT       ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 [A0.7+A0.8++],T5
        DT        LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DLT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DWT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DLWT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DT        LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DLT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DWT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DLWT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DT        ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DLT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DWT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DLWT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DT        ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DLT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DWT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DLWT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7+A0.8++]
        DR        LSLS      D1.7,AC1.0,D1Re0
        DG        LSRS      D1.7,AC1.0,D1Re0
        DG        ASLS      D1.7,AC1.0,D1Re0
        DG        ASRS      D1.7,AC1.0,D1Re0
        DRT       LSLS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DLRT      LSLS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DRWT      LSLS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DLRWT     LSLS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DRT       LSRS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DLRT      LSRS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DRWT      LSRS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DLRWT     LSRS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DRT       ASLS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DLRT      ASLS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DRWT      ASLS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DLRWT     ASLS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DRT       ASRS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DLRT      ASRS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DRWT      ASRS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DLRWT     ASRS      D1Re0,D1Re0,D1Re0 [A0.7],T5
        DG        LSLS      D1.7,AC1.0,D1.7
        DB        LSRS      D1.7,AC1.0,D1.7
        DB        ASLS      D1.7,AC1.0,D1.7
        DB        ASRS      D1.7,AC1.0,D1.7
        DRT       LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DLRT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DRWT      LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DLRWT     LSLS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DRT       LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DLRT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DRWT      LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DLRWT     LSRS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DRT       ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DLRT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DRWT      ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DLRWT     ASLS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DRT       ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DLRT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DRWT      ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DLRWT     ASRS      D1Re0,AC1.0,D1Re0 T5,[A0.7]
        DB        LSLS      D1.7,AC1.0,[D1BR.1]
        DL        LSRS      D1.7,AC1.0,[D1BR.1]
        DL        ASLS      D1.7,AC1.0,[D1BR.1]
        DL        ASRS      D1.7,AC1.0,[D1BR.1]
        DLR       LSLS      D1.7,AC1.0,[D1BR.1++]
        DLR       LSRS      D1.7,AC1.0,[D1BR.1++]
        DLR       ASLS      D1.7,AC1.0,[D1BR.1++]
        DLR       ASRS      D1.7,AC1.0,[D1BR.1++]
        DLG       LSLS      D1.7,AC1.0,[D1BR.1+D1BRI.1++]
        DLG       LSRS      D1.7,AC1.0,[D1BR.1+D1BRI.1++]
        DLG       ASLS      D1.7,AC1.0,[D1BR.1+D1BRI.1++]
        DLG       ASRS      D1.7,AC1.0,[D1BR.1+D1BRI.1++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A0.7++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.7++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A0.7--],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.7--]
        DLG       LSLS      D1.7,[D1BR.1],D1Re0
        DLG       LSRS      D1.7,[D1BR.1],D1Re0
        DLG       ASLS      D1.7,[D1BR.1],D1Re0
        DLG       ASRS      D1.7,[D1BR.1],D1Re0
        DLG       LSLS      D1.7,[D1BR.1],D1.7
        DLG       LSRS      D1.7,[D1BR.1],D1.7
        DLG       ASLS      D1.7,[D1BR.1],D1.7
        DLG       ASRS      D1.7,[D1BR.1],D1.7
        DLG       LSLS      D1.7,[D1BR.1],[D1BR.1]
        DLG       LSRS      D1.7,[D1BR.1],[D1BR.1]
        DLG       ASLS      D1.7,[D1BR.1],[D1BR.1]
        DLG       ASRS      D1.7,[D1BR.1],[D1BR.1]
        DLG       LSLS      D1.7,[D1BR.1],[D1BR.1++]
        DLG       LSRS      D1.7,[D1BR.1],[D1BR.1++]
        DLG       ASLS      D1.7,[D1BR.1],[D1BR.1++]
        DLG       ASRS      D1.7,[D1BR.1],[D1BR.1++]
        DLG       LSLS      D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLG       LSRS      D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLG       ASLS      D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLG       ASRS      D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLG       LSLS      D1.7,[D1BR.1++],D1Re0
        DLG       LSRS      D1.7,[D1BR.1++],D1Re0
        DLG       ASLS      D1.7,[D1BR.1++],D1Re0
        DLG       ASRS      D1.7,[D1BR.1++],D1Re0
        DLG       LSLS      D1.7,[D1BR.1++],D1.7
        DLG       LSRS      D1.7,[D1BR.1++],D1.7
        DLG       ASLS      D1.7,[D1BR.1++],D1.7
        DLG       ASRS      D1.7,[D1BR.1++],D1.7
        DLG       LSLS      D1.7,[D1BR.1++],[D1BR.1]
        DLG       LSRS      D1.7,[D1BR.1++],[D1BR.1]
        DLG       ASLS      D1.7,[D1BR.1++],[D1BR.1]
        DLG       ASRS      D1.7,[D1BR.1++],[D1BR.1]
        DLG       LSLS      D1.7,[D1BR.1++],[D1BR.1++]
        DLG       LSRS      D1.7,[D1BR.1++],[D1BR.1++]
        DLG       ASLS      D1.7,[D1BR.1++],[D1BR.1++]
        DLG       ASRS      D1.7,[D1BR.1++],[D1BR.1++]
        DLG       LSLS      D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLG       LSRS      D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLG       ASLS      D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLG       ASRS      D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLG       LSLS      D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DLG       LSRS      D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DLG       ASLS      D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DLG       ASRS      D1.7,[D1BR.1+D1BRI.1++],D1Re0
        DLG       LSLS      D1.7,[D1BR.1+D1BRI.1++],D1.7
        DLG       LSRS      D1.7,[D1BR.1+D1BRI.1++],D1.7
        DLG       ASLS      D1.7,[D1BR.1+D1BRI.1++],D1.7
        DLG       ASRS      D1.7,[D1BR.1+D1BRI.1++],D1.7
        DLG       LSLS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLG       LSRS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLG       ASLS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLG       ASRS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLG       LSLS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLG       LSRS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLG       ASLS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLG       ASRS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLG       LSLS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLG       LSRS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLG       ASLS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLG       ASRS      D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0FrP++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.7++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8+A0.8++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A0.8],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A0.8++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A0.8--],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A0.8--]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1LbP++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.7++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP+A1.8++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1LbP],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1LbP++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1LbP--],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1LbP--]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1LbP++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.7++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7+A1.8++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.7],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.7++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.7--],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.7--]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1LbP++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.7++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8+A1.8++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.8],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.8++],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8++]
        DBT       LSLS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DBT       LSRS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DBT       ASLS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DBT       ASRS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 [A1.8--],T5
        DBT       LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DLBT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DBWT      LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DLBWT     LSLS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DBT       LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DLBT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DBWT      LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DLBWT     LSRS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DBT       ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DLBT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DBWT      ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DLBWT     ASLS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DBT       ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DLBT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DBWT      ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DLBWT     ASRS      D1Re0,D1Re0,D1Re0 T5,[A1.8--]
        DLG       LSLS      [D1BW.1],D1Re0,D1Re0
        DLG       LSRS      [D1BW.1],D1Re0,D1Re0
        DLG       ASLS      [D1BW.1],D1Re0,D1Re0
        DLG       ASRS      [D1BW.1],D1Re0,D1Re0
        DLG       LSLS      [D1BW.1],D1Re0,D1.7
        DLG       LSRS      [D1BW.1],D1Re0,D1.7
        DLG       ASLS      [D1BW.1],D1Re0,D1.7
        DLG       ASRS      [D1BW.1],D1Re0,D1.7
        DLG       LSLS      [D1BW.1],D1Re0,[D1BR.1]
        DLG       LSRS      [D1BW.1],D1Re0,[D1BR.1]
        DLG       ASLS      [D1BW.1],D1Re0,[D1BR.1]
        DLG       ASRS      [D1BW.1],D1Re0,[D1BR.1]
        DLG       LSLS      [D1BW.1],D1Re0,[D1BR.1++]
        DLG       LSRS      [D1BW.1],D1Re0,[D1BR.1++]
        DLG       ASLS      [D1BW.1],D1Re0,[D1BR.1++]
        DLG       ASRS      [D1BW.1],D1Re0,[D1BR.1++]
        DLG       LSLS      [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DLG       LSRS      [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DLG       ASLS      [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DLG       ASRS      [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        DLG       LSLS      [D1BW.1],D1.7,D1Re0
        DLG       LSRS      [D1BW.1],D1.7,D1Re0
        DLG       ASLS      [D1BW.1],D1.7,D1Re0
        DLG       ASRS      [D1BW.1],D1.7,D1Re0
        DLG       LSLS      [D1BW.1],D1.7,D1.7
        DLG       LSRS      [D1BW.1],D1.7,D1.7
        DLG       ASLS      [D1BW.1],D1.7,D1.7
        DLG       ASRS      [D1BW.1],D1.7,D1.7
        DLG       LSLS      [D1BW.1],D1.7,[D1BR.1]
        DLG       LSRS      [D1BW.1],D1.7,[D1BR.1]
        DLG       ASLS      [D1BW.1],D1.7,[D1BR.1]
        DLG       ASRS      [D1BW.1],D1.7,[D1BR.1]
        DLG       LSLS      [D1BW.1],D1.7,[D1BR.1++]
        DLG       LSRS      [D1BW.1],D1.7,[D1BR.1++]
        DLG       ASLS      [D1BW.1],D1.7,[D1BR.1++]
        DLG       ASRS      [D1BW.1],D1.7,[D1BR.1++]
        DLG       LSLS      [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DLG       LSRS      [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DLG       ASLS      [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DLG       ASRS      [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DLB       LSLS      [D1BW.1],AC1.0,D1Re0
        DLB       LSRS      [D1BW.1],AC1.0,D1Re0
        DLB       ASLS      [D1BW.1],AC1.0,D1Re0
        DLB       ASRS      [D1BW.1],AC1.0,D1Re0
        D         LSRS      [D1BW.1],AC1.0,D1.7
        D         ASLS      [D1BW.1],AC1.0,D1.7
        D         ASRS      [D1BW.1],AC1.0,D1.7
        D         LSLS      [D1BW.1],AC1.0,[D1BR.1]
        DR        LSRS      [D1BW.1],AC1.0,[D1BR.1]
        DR        ASLS      [D1BW.1],AC1.0,[D1BR.1]
        DR        ASRS      [D1BW.1],AC1.0,[D1BR.1]
        DR        LSLS      [D1BW.1],AC1.0,[D1BR.1++]
        DG        LSRS      [D1BW.1],AC1.0,[D1BR.1++]
        DG        ASLS      [D1BW.1],AC1.0,[D1BR.1++]
        DG        ASRS      [D1BW.1],AC1.0,[D1BR.1++]
        DG        LSLS      [D1BW.1],AC1.0,[D1BR.1+D1BRI.1++]
        DB        LSRS      [D1BW.1],AC1.0,[D1BR.1+D1BRI.1++]
        DB        ASLS      [D1BW.1],AC1.0,[D1BR.1+D1BRI.1++]
        DB        ASRS      [D1BW.1],AC1.0,[D1BR.1+D1BRI.1++]
        DRT       LSLS      D1Re0,AC1.0,D1Re0 [D1BW.1],T5
        DLRT      LSLS      D1Re0,AC1.0,D1Re0 [D1BW.1],T5
        DRT       LSRS      D1Re0,AC1.0,D1Re0 [D1BW.1],T5
        DLRT      LSRS      D1Re0,AC1.0,D1Re0 [D1BW.1],T5
        DRT       ASLS      D1Re0,AC1.0,D1Re0 [D1BW.1],T5
        DLRT      ASLS      D1Re0,AC1.0,D1Re0 [D1BW.1],T5
        DRT       ASRS      D1Re0,AC1.0,D1Re0 [D1BW.1],T5
        DLRT      ASRS      D1Re0,AC1.0,D1Re0 [D1BW.1],T5
        DRT       LSLS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1]
        DLRT      LSLS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1]
        DRT       LSRS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1]
        DLRT      LSRS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1]
        DRT       ASLS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1]
        DLRT      ASLS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1]
        DRT       ASRS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1]
        DLRT      ASRS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1]
        DB        LSLS      [D1BW.1],[D1BR.1],D1Re0
        DB        LSRS      [D1BW.1],[D1BR.1],D1Re0
        DB        ASLS      [D1BW.1],[D1BR.1],D1Re0
        DB        ASRS      [D1BW.1],[D1BR.1],D1Re0
        DB        LSLS      [D1BW.1],[D1BR.1],D1.7
        DB        LSRS      [D1BW.1],[D1BR.1],D1.7
        DB        ASLS      [D1BW.1],[D1BR.1],D1.7
        DB        ASRS      [D1BW.1],[D1BR.1],D1.7
        DB        LSLS      [D1BW.1],[D1BR.1],[D1BR.1]
        DB        LSRS      [D1BW.1],[D1BR.1],[D1BR.1]
        DB        ASLS      [D1BW.1],[D1BR.1],[D1BR.1]
        DB        ASRS      [D1BW.1],[D1BR.1],[D1BR.1]
        DB        LSLS      [D1BW.1],[D1BR.1],[D1BR.1++]
        DB        LSRS      [D1BW.1],[D1BR.1],[D1BR.1++]
        DB        ASLS      [D1BW.1],[D1BR.1],[D1BR.1++]
        DB        ASRS      [D1BW.1],[D1BR.1],[D1BR.1++]
        DB        LSLS      [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DB        LSRS      [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DB        ASLS      [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DB        ASRS      [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DB        LSLS      [D1BW.1],[D1BR.1++],D1Re0
        DB        LSRS      [D1BW.1],[D1BR.1++],D1Re0
        DB        ASLS      [D1BW.1],[D1BR.1++],D1Re0
        DB        ASRS      [D1BW.1],[D1BR.1++],D1Re0
        DB        LSLS      [D1BW.1],[D1BR.1++],D1.7
        DB        LSRS      [D1BW.1],[D1BR.1++],D1.7
        DB        ASLS      [D1BW.1],[D1BR.1++],D1.7
        DB        ASRS      [D1BW.1],[D1BR.1++],D1.7
        DB        LSLS      [D1BW.1],[D1BR.1++],[D1BR.1]
        DB        LSRS      [D1BW.1],[D1BR.1++],[D1BR.1]
        DB        ASLS      [D1BW.1],[D1BR.1++],[D1BR.1]
        DB        ASRS      [D1BW.1],[D1BR.1++],[D1BR.1]
        DB        LSLS      [D1BW.1],[D1BR.1++],[D1BR.1++]
        DB        LSRS      [D1BW.1],[D1BR.1++],[D1BR.1++]
        DB        ASLS      [D1BW.1],[D1BR.1++],[D1BR.1++]
        DB        ASRS      [D1BW.1],[D1BR.1++],[D1BR.1++]
        DB        LSLS      [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DB        LSRS      [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DB        ASLS      [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DB        ASRS      [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DB        LSLS      [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DB        LSRS      [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DB        ASLS      [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DB        ASRS      [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        DB        LSLS      [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DB        LSRS      [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DB        ASLS      [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DB        ASRS      [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DB        LSLS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DB        LSRS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DB        ASLS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DB        ASRS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DB        LSLS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DB        LSRS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DB        ASLS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DB        ASRS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DB        LSLS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DB        LSRS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DB        ASLS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DB        ASRS      [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DB        LSLS      [D1BW.1++],D1Re0,D1Re0
        DB        LSRS      [D1BW.1++],D1Re0,D1Re0
        DB        ASLS      [D1BW.1++],D1Re0,D1Re0
        DB        ASRS      [D1BW.1++],D1Re0,D1Re0
        DB        LSLS      [D1BW.1++],D1Re0,D1.7
        DB        LSRS      [D1BW.1++],D1Re0,D1.7
        DB        ASLS      [D1BW.1++],D1Re0,D1.7
        DB        ASRS      [D1BW.1++],D1Re0,D1.7
        DB        LSLS      [D1BW.1++],D1Re0,[D1BR.1]
        DB        LSRS      [D1BW.1++],D1Re0,[D1BR.1]
        DB        ASLS      [D1BW.1++],D1Re0,[D1BR.1]
        DB        ASRS      [D1BW.1++],D1Re0,[D1BR.1]
        DB        LSLS      [D1BW.1++],D1Re0,[D1BR.1++]
        DB        LSRS      [D1BW.1++],D1Re0,[D1BR.1++]
        DB        ASLS      [D1BW.1++],D1Re0,[D1BR.1++]
        DB        ASRS      [D1BW.1++],D1Re0,[D1BR.1++]
        DB        LSLS      [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DB        LSRS      [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DB        ASLS      [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DB        ASRS      [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DB        LSLS      [D1BW.1++],D1.7,D1Re0
        DB        LSRS      [D1BW.1++],D1.7,D1Re0
        DB        ASLS      [D1BW.1++],D1.7,D1Re0
        DB        ASRS      [D1BW.1++],D1.7,D1Re0
        DB        LSLS      [D1BW.1++],D1.7,D1.7
        DB        LSRS      [D1BW.1++],D1.7,D1.7
        DB        ASLS      [D1BW.1++],D1.7,D1.7
        DB        ASRS      [D1BW.1++],D1.7,D1.7
        DB        LSLS      [D1BW.1++],D1.7,[D1BR.1]
        DB        LSRS      [D1BW.1++],D1.7,[D1BR.1]
        DB        ASLS      [D1BW.1++],D1.7,[D1BR.1]
        DB        ASRS      [D1BW.1++],D1.7,[D1BR.1]
        DB        LSLS      [D1BW.1++],D1.7,[D1BR.1++]
        DB        LSRS      [D1BW.1++],D1.7,[D1BR.1++]
        DB        ASLS      [D1BW.1++],D1.7,[D1BR.1++]
        DB        ASRS      [D1BW.1++],D1.7,[D1BR.1++]
        DB        LSLS      [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DB        LSRS      [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DB        ASLS      [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DB        ASRS      [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DB        LSLS      [D1BW.1++],AC1.0,D1Re0
        DL        LSRS      [D1BW.1++],AC1.0,D1Re0
        DL        ASLS      [D1BW.1++],AC1.0,D1Re0
        DL        ASRS      [D1BW.1++],AC1.0,D1Re0
        DLR       LSLS      [D1BW.1++],AC1.0,D1.7
        DLR       LSRS      [D1BW.1++],AC1.0,D1.7
        DLR       ASLS      [D1BW.1++],AC1.0,D1.7
        DLR       ASRS      [D1BW.1++],AC1.0,D1.7
        DLG       LSLS      [D1BW.1++],AC1.0,[D1BR.1]
        DLG       LSRS      [D1BW.1++],AC1.0,[D1BR.1]
        DLG       ASLS      [D1BW.1++],AC1.0,[D1BR.1]
        DLG       ASRS      [D1BW.1++],AC1.0,[D1BR.1]
        DLB       LSLS      [D1BW.1++],AC1.0,[D1BR.1++]
        DLB       LSRS      [D1BW.1++],AC1.0,[D1BR.1++]
        DLB       ASLS      [D1BW.1++],AC1.0,[D1BR.1++]
        DLB       ASRS      [D1BW.1++],AC1.0,[D1BR.1++]
        D         LSRS      [D1BW.1++],AC1.0,[D1BR.1+D1BRI.1++]
        D         ASLS      [D1BW.1++],AC1.0,[D1BR.1+D1BRI.1++]
        D         ASRS      [D1BW.1++],AC1.0,[D1BR.1+D1BRI.1++]
        DT        LSLS      D1Re0,D1Re0,D1Re0 [D1BW.1++],T5
        DLT       LSLS      D1Re0,D1Re0,D1Re0 [D1BW.1++],T5
        DT        LSRS      D1Re0,D1Re0,D1Re0 [D1BW.1++],T5
        DLT       LSRS      D1Re0,D1Re0,D1Re0 [D1BW.1++],T5
        DT        ASLS      D1Re0,D1Re0,D1Re0 [D1BW.1++],T5
        DLT       ASLS      D1Re0,D1Re0,D1Re0 [D1BW.1++],T5
        DT        ASRS      D1Re0,D1Re0,D1Re0 [D1BW.1++],T5
        DLT       ASRS      D1Re0,D1Re0,D1Re0 [D1BW.1++],T5
        DT        LSLS      D1Re0,D1Re0,D1Re0 T5,[D1BR.1++]
        DLT       LSLS      D1Re0,D1Re0,D1Re0 T5,[D1BR.1++]
        DT        LSRS      D1Re0,D1Re0,D1Re0 T5,[D1BR.1++]
        DLT       LSRS      D1Re0,D1Re0,D1Re0 T5,[D1BR.1++]
        DT        ASLS      D1Re0,D1Re0,D1Re0 T5,[D1BR.1++]
        DLT       ASLS      D1Re0,D1Re0,D1Re0 T5,[D1BR.1++]
        DT        ASRS      D1Re0,D1Re0,D1Re0 T5,[D1BR.1++]
        DLT       ASRS      D1Re0,D1Re0,D1Re0 T5,[D1BR.1++]
        D         LSLS      [D1BW.1++],[D1BR.1],D1Re0
        D         LSRS      [D1BW.1++],[D1BR.1],D1Re0
        D         ASLS      [D1BW.1++],[D1BR.1],D1Re0
        D         ASRS      [D1BW.1++],[D1BR.1],D1Re0
        D         LSLS      [D1BW.1++],[D1BR.1],D1.7
        D         LSRS      [D1BW.1++],[D1BR.1],D1.7
        D         ASLS      [D1BW.1++],[D1BR.1],D1.7
        D         ASRS      [D1BW.1++],[D1BR.1],D1.7
        D         LSLS      [D1BW.1++],[D1BR.1],[D1BR.1]
        D         LSRS      [D1BW.1++],[D1BR.1],[D1BR.1]
        D         ASLS      [D1BW.1++],[D1BR.1],[D1BR.1]
        D         ASRS      [D1BW.1++],[D1BR.1],[D1BR.1]
        D         LSLS      [D1BW.1++],[D1BR.1],[D1BR.1++]
        D         LSRS      [D1BW.1++],[D1BR.1],[D1BR.1++]
        D         ASLS      [D1BW.1++],[D1BR.1],[D1BR.1++]
        D         ASRS      [D1BW.1++],[D1BR.1],[D1BR.1++]
        D         LSLS      [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSRS      [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASLS      [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASRS      [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSLS      [D1BW.1++],[D1BR.1++],D1Re0
        D         LSRS      [D1BW.1++],[D1BR.1++],D1Re0
        D         ASLS      [D1BW.1++],[D1BR.1++],D1Re0
        D         ASRS      [D1BW.1++],[D1BR.1++],D1Re0
        D         LSLS      [D1BW.1++],[D1BR.1++],D1.7
        D         LSRS      [D1BW.1++],[D1BR.1++],D1.7
        D         ASLS      [D1BW.1++],[D1BR.1++],D1.7
        D         ASRS      [D1BW.1++],[D1BR.1++],D1.7
        D         LSLS      [D1BW.1++],[D1BR.1++],[D1BR.1]
        D         LSRS      [D1BW.1++],[D1BR.1++],[D1BR.1]
        D         ASLS      [D1BW.1++],[D1BR.1++],[D1BR.1]
        D         ASRS      [D1BW.1++],[D1BR.1++],[D1BR.1]
        D         LSLS      [D1BW.1++],[D1BR.1++],[D1BR.1++]
        D         LSRS      [D1BW.1++],[D1BR.1++],[D1BR.1++]
        D         ASLS      [D1BW.1++],[D1BR.1++],[D1BR.1++]
        D         ASRS      [D1BW.1++],[D1BR.1++],[D1BR.1++]
        D         LSLS      [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSRS      [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASLS      [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASRS      [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSLS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         LSRS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         ASLS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         ASRS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         LSLS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        D         LSRS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        D         ASLS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        D         ASRS      [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        D         LSLS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSRS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASLS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASRS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSLS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSRS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASLS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASRS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSLS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSRS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASLS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASRS      [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSLS      [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        D         LSRS      [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        D         ASLS      [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        D         ASRS      [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        D         LSLS      [D1BW.1+D1BWI.1++],D1Re0,D1.7
        D         LSRS      [D1BW.1+D1BWI.1++],D1Re0,D1.7
        D         ASLS      [D1BW.1+D1BWI.1++],D1Re0,D1.7
        D         ASRS      [D1BW.1+D1BWI.1++],D1Re0,D1.7
        D         LSLS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        D         LSRS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        D         ASLS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        D         ASRS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        D         LSLS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        D         LSRS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        D         ASLS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        D         ASRS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        D         LSLS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        D         LSRS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        D         ASLS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        D         ASRS      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        D         LSLS      [D1BW.1+D1BWI.1++],D1.7,D1Re0
        D         LSRS      [D1BW.1+D1BWI.1++],D1.7,D1Re0
        D         ASLS      [D1BW.1+D1BWI.1++],D1.7,D1Re0
        D         ASRS      [D1BW.1+D1BWI.1++],D1.7,D1Re0
        D         LSLS      [D1BW.1+D1BWI.1++],D1.7,D1.7
        D         LSRS      [D1BW.1+D1BWI.1++],D1.7,D1.7
        D         ASLS      [D1BW.1+D1BWI.1++],D1.7,D1.7
        D         ASRS      [D1BW.1+D1BWI.1++],D1.7,D1.7
        D         LSLS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        D         LSRS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        D         ASLS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        D         ASRS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        D         LSLS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        D         LSRS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        D         ASLS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        D         ASRS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        D         LSLS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        D         LSRS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        D         ASLS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        D         ASRS      [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        D         LSLS      [D1BW.1+D1BWI.1++],AC1.0,D1Re0
        DR        LSRS      [D1BW.1+D1BWI.1++],AC1.0,D1Re0
        DR        ASLS      [D1BW.1+D1BWI.1++],AC1.0,D1Re0
        DR        ASRS      [D1BW.1+D1BWI.1++],AC1.0,D1Re0
        DR        LSLS      [D1BW.1+D1BWI.1++],AC1.0,D1.7
        DG        LSRS      [D1BW.1+D1BWI.1++],AC1.0,D1.7
        DG        ASLS      [D1BW.1+D1BWI.1++],AC1.0,D1.7
        DG        ASRS      [D1BW.1+D1BWI.1++],AC1.0,D1.7
        DG        LSLS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1]
        DB        LSRS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1]
        DB        ASLS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1]
        DB        ASRS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1]
        DB        LSLS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1++]
        DL        LSRS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1++]
        DL        ASLS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1++]
        DL        ASRS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1++]
        DLR       LSLS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1+D1BRI.1++]
        DLR       LSRS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1+D1BRI.1++]
        DLR       ASLS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1+D1BRI.1++]
        DLR       ASRS      [D1BW.1+D1BWI.1++],AC1.0,[D1BR.1+D1BRI.1++]
        DGT       LSLS      D1Re0,AC1.0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DLGT      LSLS      D1Re0,AC1.0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DGT       LSRS      D1Re0,AC1.0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DLGT      LSRS      D1Re0,AC1.0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DGT       ASLS      D1Re0,AC1.0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DLGT      ASLS      D1Re0,AC1.0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DGT       ASRS      D1Re0,AC1.0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DLGT      ASRS      D1Re0,AC1.0,D1Re0 [D1BW.1+D1BWI.1++],T5
        DGT       LSLS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DLGT      LSLS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DGT       LSRS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DLGT      LSRS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DGT       ASLS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DLGT      ASLS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DGT       ASRS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DLGT      ASRS      D1Re0,AC1.0,D1Re0 T5,[D1BR.1+D1BRI.1++]
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLR       LSLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLR       LSRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLR       ASLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLR       ASRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLR       LSLS      D0Re0,D0Re0,#0xf
        DLR       LSRS      D0Re0,D0Re0,#0xf
        DLR       ASLS      D0Re0,D0Re0,#0xf
        DLR       ASRS      D0Re0,D0Re0,#0xf
        DLR       LSLS      D0Re0,D0Re0,#0x10
        DLR       LSRS      D0Re0,D0Re0,#0x10
        DLR       ASLS      D0Re0,D0Re0,#0x10
        DLR       ASRS      D0Re0,D0Re0,#0x10
        DLR       LSLS      D0Re0,D0Re0,#0x1f
        DLR       LSRS      D0Re0,D0Re0,#0x1f
        DLR       ASLS      D0Re0,D0Re0,#0x1f
        DLR       ASRS      D0Re0,D0Re0,#0x1f
        DLR       LSLS      D0Re0,D0.7,#0xf
        DLR       LSRS      D0Re0,D0.7,#0xf
        DLR       ASLS      D0Re0,D0.7,#0xf
        DLR       ASRS      D0Re0,D0.7,#0xf
        DLR       LSLS      D0Re0,D0.7,#0x10
        DLR       LSRS      D0Re0,D0.7,#0x10
        DLR       ASLS      D0Re0,D0.7,#0x10
        DLR       ASRS      D0Re0,D0.7,#0x10
        DLR       LSLS      D0Re0,D0.7,#0x1f
        DLR       LSRS      D0Re0,D0.7,#0x1f
        DLR       ASLS      D0Re0,D0.7,#0x1f
        DLR       ASRS      D0Re0,D0.7,#0x1f
        DG        LSLS      D0Re0,AC0.0,#0xf
        DLB       LSLS      D0Re0,[D0AR.0],#0xf
        DB        LSRS      D0Re0,[D0AR.0],#0xf
        DLB       LSRS      D0Re0,[D0AR.0],#0xf
        DB        ASLS      D0Re0,[D0AR.0],#0xf
        DLB       ASLS      D0Re0,[D0AR.0],#0xf
        DB        ASRS      D0Re0,[D0AR.0],#0xf
        DLB       ASRS      D0Re0,[D0AR.0],#0xf
        DB        LSLS      D0Re0,[D0AR.0],#0x10
        DLB       LSLS      D0Re0,AC0.0,#0x10
        DB        LSRS      D0Re0,AC0.0,#0x10
        DLB       LSRS      D0Re0,AC0.0,#0x10
        DB        ASLS      D0Re0,AC0.0,#0x10
        DLB       ASLS      D0Re0,AC0.0,#0x10
        DB        ASRS      D0Re0,AC0.0,#0x10
        DLB       ASRS      D0Re0,AC0.0,#0x10
        DB        LSLS      D0Re0,AC0.0,#0x1f
        D         LSRS      D0Re0,[D0AR.0],#0x1f
        DL        LSRS      D0Re0,[D0AR.0],#0x1f
        D         ASLS      D0Re0,[D0AR.0],#0x1f
        DL        ASLS      D0Re0,[D0AR.0],#0x1f
        D         ASRS      D0Re0,[D0AR.0],#0x1f
        DL        ASRS      D0Re0,[D0AR.0],#0x1f
        D         LSLS      D0Re0,[D0AR.0++],#0xf
        D         LSRS      D0Re0,[D0AR.0++],#0xf
        D         ASLS      D0Re0,[D0AR.0++],#0xf
        D         ASRS      D0Re0,[D0AR.0++],#0xf
        D         LSLS      D0Re0,[D0AR.0++],#0x10
        D         LSRS      D0Re0,[D0AR.0++],#0x10
        D         ASLS      D0Re0,[D0AR.0++],#0x10
        D         ASRS      D0Re0,[D0AR.0++],#0x10
        D         LSLS      D0Re0,[D0AR.0++],#0x1f
        D         LSRS      D0Re0,[D0AR.0++],#0x1f
        D         ASLS      D0Re0,[D0AR.0++],#0x1f
        D         ASRS      D0Re0,[D0AR.0++],#0x1f
        D         LSLS      D0Re0,[D0AR.0+D0ARI.0++],#0xf
        D         LSRS      D0Re0,[D0AR.0+D0ARI.0++],#0xf
        D         ASLS      D0Re0,[D0AR.0+D0ARI.0++],#0xf
        D         ASRS      D0Re0,[D0AR.0+D0ARI.0++],#0xf
        D         LSLS      D0Re0,[D0AR.0+D0ARI.0++],#0x10
        D         LSRS      D0Re0,[D0AR.0+D0ARI.0++],#0x10
        D         ASLS      D0Re0,[D0AR.0+D0ARI.0++],#0x10
        D         ASRS      D0Re0,[D0AR.0+D0ARI.0++],#0x10
        D         LSLS      D0Re0,[D0AR.0+D0ARI.0++],#0x1f
        D         LSRS      D0Re0,[D0AR.0+D0ARI.0++],#0x1f
        D         ASLS      D0Re0,[D0AR.0+D0ARI.0++],#0x1f
        D         ASRS      D0Re0,[D0AR.0+D0ARI.0++],#0x1f
        D         LSLS      D0.7,D0Re0,#0xf
        D         LSRS      D0.7,D0Re0,#0xf
        D         ASLS      D0.7,D0Re0,#0xf
        D         ASRS      D0.7,D0Re0,#0xf
        D         LSLS      D0.7,D0Re0,#0x10
        D         LSRS      D0.7,D0Re0,#0x10
        D         ASLS      D0.7,D0Re0,#0x10
        D         ASRS      D0.7,D0Re0,#0x10
        D         LSLS      D0.7,D0Re0,#0x1f
        D         LSRS      D0.7,D0Re0,#0x1f
        D         ASLS      D0.7,D0Re0,#0x1f
        D         ASRS      D0.7,D0Re0,#0x1f
        D         LSLS      D0.7,D0.7,#0xf
        D         LSRS      D0.7,D0.7,#0xf
        D         ASLS      D0.7,D0.7,#0xf
        D         ASRS      D0.7,D0.7,#0xf
        D         LSLS      D0.7,D0.7,#0x10
        D         LSRS      D0.7,D0.7,#0x10
        D         ASLS      D0.7,D0.7,#0x10
        D         ASRS      D0.7,D0.7,#0x10
        D         LSLS      D0.7,D0.7,#0x1f
        D         LSRS      D0.7,D0.7,#0x1f
        D         ASLS      D0.7,D0.7,#0x1f
        D         ASRS      D0.7,D0.7,#0x1f
        D         LSLS      D0.7,[D0AR.0],#0xf
        DL        LSLS      D0.7,[D0AR.0],#0xf
        D         LSRS      D0.7,AC0.0,#0xf
        DL        LSRS      D0.7,AC0.0,#0xf
        D         ASLS      D0.7,AC0.0,#0xf
        DL        ASLS      D0.7,AC0.0,#0xf
        D         ASRS      D0.7,AC0.0,#0xf
        DL        ASRS      D0.7,AC0.0,#0xf
        D         LSLS      D0.7,AC0.0,#0x10
        DLR       LSLS      D0.7,[D0AR.0],#0x10
        DR        LSRS      D0.7,[D0AR.0],#0x10
        DLR       LSRS      D0.7,[D0AR.0],#0x10
        DR        ASLS      D0.7,[D0AR.0],#0x10
        DLR       ASLS      D0.7,[D0AR.0],#0x10
        DR        ASRS      D0.7,[D0AR.0],#0x10
        DLR       ASRS      D0.7,[D0AR.0],#0x10
        DR        LSLS      D0.7,[D0AR.0],#0x1f
        DLR       LSLS      D0.7,AC0.0,#0x1f
        DR        LSRS      D0.7,AC0.0,#0x1f
        DLR       LSRS      D0.7,AC0.0,#0x1f
        DR        ASLS      D0.7,AC0.0,#0x1f
        DLR       ASLS      D0.7,AC0.0,#0x1f
        DR        ASRS      D0.7,AC0.0,#0x1f
        DLR       ASRS      D0.7,AC0.0,#0x1f
        DB        LSLS      D0.7,[D0AR.0++],#0xf
        DB        LSRS      D0.7,[D0AR.0++],#0xf
        DB        ASLS      D0.7,[D0AR.0++],#0xf
        DB        ASRS      D0.7,[D0AR.0++],#0xf
        DB        LSLS      D0.7,[D0AR.0++],#0x10
        DB        LSRS      D0.7,[D0AR.0++],#0x10
        DB        ASLS      D0.7,[D0AR.0++],#0x10
        DB        ASRS      D0.7,[D0AR.0++],#0x10
        DB        LSLS      D0.7,[D0AR.0++],#0x1f
        DB        LSRS      D0.7,[D0AR.0++],#0x1f
        DB        ASLS      D0.7,[D0AR.0++],#0x1f
        DB        ASRS      D0.7,[D0AR.0++],#0x1f
        DB        LSLS      D0.7,[D0AR.0+D0ARI.0++],#0xf
        DB        LSRS      D0.7,[D0AR.0+D0ARI.0++],#0xf
        DB        ASLS      D0.7,[D0AR.0+D0ARI.0++],#0xf
        DB        ASRS      D0.7,[D0AR.0+D0ARI.0++],#0xf
        DB        LSLS      D0.7,[D0AR.0+D0ARI.0++],#0x10
        DB        LSRS      D0.7,[D0AR.0+D0ARI.0++],#0x10
        DB        ASLS      D0.7,[D0AR.0+D0ARI.0++],#0x10
        DB        ASRS      D0.7,[D0AR.0+D0ARI.0++],#0x10
        DB        LSLS      D0.7,[D0AR.0+D0ARI.0++],#0x1f
        DB        LSRS      D0.7,[D0AR.0+D0ARI.0++],#0x1f
        DB        ASLS      D0.7,[D0AR.0+D0ARI.0++],#0x1f
        DB        ASRS      D0.7,[D0AR.0+D0ARI.0++],#0x1f
        DB        LSLS      [D0AW.0],D0Re0,#0xf
        DB        LSRS      [D0AW.0],D0Re0,#0xf
        DB        ASLS      [D0AW.0],D0Re0,#0xf
        DB        ASRS      [D0AW.0],D0Re0,#0xf
        DB        LSLS      [D0AW.0],D0Re0,#0x10
        DB        LSRS      [D0AW.0],D0Re0,#0x10
        DB        ASLS      [D0AW.0],D0Re0,#0x10
        DB        ASRS      [D0AW.0],D0Re0,#0x10
        DB        LSLS      [D0AW.0],D0Re0,#0x1f
        DB        LSRS      [D0AW.0],D0Re0,#0x1f
        DB        ASLS      [D0AW.0],D0Re0,#0x1f
        DB        ASRS      [D0AW.0],D0Re0,#0x1f
        DB        LSLS      [D0AW.0],D0.7,#0xf
        DB        LSRS      [D0AW.0],D0.7,#0xf
        DB        ASLS      [D0AW.0],D0.7,#0xf
        DB        ASRS      [D0AW.0],D0.7,#0xf
        DB        LSLS      [D0AW.0],D0.7,#0x10
        DB        LSRS      [D0AW.0],D0.7,#0x10
        DB        ASLS      [D0AW.0],D0.7,#0x10
        DB        ASRS      [D0AW.0],D0.7,#0x10
        DB        LSLS      [D0AW.0],D0.7,#0x1f
        DB        LSRS      [D0AW.0],D0.7,#0x1f
        DB        ASLS      [D0AW.0],D0.7,#0x1f
        DB        ASRS      [D0AW.0],D0.7,#0x1f
        DR        LSLS      [D0AW.0],AC0.0,#0xf
        DLG       LSLS      [D0AW.0],[D0AR.0],#0xf
        DG        LSRS      [D0AW.0],[D0AR.0],#0xf
        DLG       LSRS      [D0AW.0],[D0AR.0],#0xf
        DG        ASLS      [D0AW.0],[D0AR.0],#0xf
        DLG       ASLS      [D0AW.0],[D0AR.0],#0xf
        DG        ASRS      [D0AW.0],[D0AR.0],#0xf
        DLG       ASRS      [D0AW.0],[D0AR.0],#0xf
        DG        LSLS      [D0AW.0],[D0AR.0],#0x10
        DLG       LSLS      [D0AW.0],AC0.0,#0x10
        DG        LSRS      [D0AW.0],AC0.0,#0x10
        DLG       LSRS      [D0AW.0],AC0.0,#0x10
        DG        ASLS      [D0AW.0],AC0.0,#0x10
        DLG       ASLS      [D0AW.0],AC0.0,#0x10
        DG        ASRS      [D0AW.0],AC0.0,#0x10
        DLG       ASRS      [D0AW.0],AC0.0,#0x10
        DG        LSLS      [D0AW.0],AC0.0,#0x1f
        DLB       LSLS      [D0AW.0],[D0AR.0],#0x1f
        DB        LSRS      [D0AW.0],[D0AR.0],#0x1f
        DLB       LSRS      [D0AW.0],[D0AR.0],#0x1f
        DB        ASLS      [D0AW.0],[D0AR.0],#0x1f
        DLB       ASLS      [D0AW.0],[D0AR.0],#0x1f
        DB        ASRS      [D0AW.0],[D0AR.0],#0x1f
        DLB       ASRS      [D0AW.0],[D0AR.0],#0x1f
        DLG       LSLS      [D0AW.0],[D0AR.0++],#0xf
        DLG       LSRS      [D0AW.0],[D0AR.0++],#0xf
        DLG       ASLS      [D0AW.0],[D0AR.0++],#0xf
        DLG       ASRS      [D0AW.0],[D0AR.0++],#0xf
        DLG       LSLS      [D0AW.0],[D0AR.0++],#0x10
        DLG       LSRS      [D0AW.0],[D0AR.0++],#0x10
        DLG       ASLS      [D0AW.0],[D0AR.0++],#0x10
        DLG       ASRS      [D0AW.0],[D0AR.0++],#0x10
        DLG       LSLS      [D0AW.0],[D0AR.0++],#0x1f
        DLG       LSRS      [D0AW.0],[D0AR.0++],#0x1f
        DLG       ASLS      [D0AW.0],[D0AR.0++],#0x1f
        DLG       ASRS      [D0AW.0],[D0AR.0++],#0x1f
        DLG       LSLS      [D0AW.0],[D0AR.0+D0ARI.0++],#0xf
        DLG       LSRS      [D0AW.0],[D0AR.0+D0ARI.0++],#0xf
        DLG       ASLS      [D0AW.0],[D0AR.0+D0ARI.0++],#0xf
        DLG       ASRS      [D0AW.0],[D0AR.0+D0ARI.0++],#0xf
        DLG       LSLS      [D0AW.0],[D0AR.0+D0ARI.0++],#0x10
        DLG       LSRS      [D0AW.0],[D0AR.0+D0ARI.0++],#0x10
        DLG       ASLS      [D0AW.0],[D0AR.0+D0ARI.0++],#0x10
        DLG       ASRS      [D0AW.0],[D0AR.0+D0ARI.0++],#0x10
        DLG       LSLS      [D0AW.0],[D0AR.0+D0ARI.0++],#0x1f
        DLG       LSRS      [D0AW.0],[D0AR.0+D0ARI.0++],#0x1f
        DLG       ASLS      [D0AW.0],[D0AR.0+D0ARI.0++],#0x1f
        DLG       ASRS      [D0AW.0],[D0AR.0+D0ARI.0++],#0x1f
        DLG       LSLS      [D0AW.0++],D0Re0,#0xf
        DLG       LSRS      [D0AW.0++],D0Re0,#0xf
        DLG       ASLS      [D0AW.0++],D0Re0,#0xf
        DLG       ASRS      [D0AW.0++],D0Re0,#0xf
        DLG       LSLS      [D0AW.0++],D0Re0,#0x10
        DLG       LSRS      [D0AW.0++],D0Re0,#0x10
        DLG       ASLS      [D0AW.0++],D0Re0,#0x10
        DLG       ASRS      [D0AW.0++],D0Re0,#0x10
        DLG       LSLS      [D0AW.0++],D0Re0,#0x1f
        DLG       LSRS      [D0AW.0++],D0Re0,#0x1f
        DLG       ASLS      [D0AW.0++],D0Re0,#0x1f
        DLG       ASRS      [D0AW.0++],D0Re0,#0x1f
        DLG       LSLS      [D0AW.0++],D0.7,#0xf
        DLG       LSRS      [D0AW.0++],D0.7,#0xf
        DLG       ASLS      [D0AW.0++],D0.7,#0xf
        DLG       ASRS      [D0AW.0++],D0.7,#0xf
        DLG       LSLS      [D0AW.0++],D0.7,#0x10
        DLG       LSRS      [D0AW.0++],D0.7,#0x10
        DLG       ASLS      [D0AW.0++],D0.7,#0x10
        DLG       ASRS      [D0AW.0++],D0.7,#0x10
        DLG       LSLS      [D0AW.0++],D0.7,#0x1f
        DLG       LSRS      [D0AW.0++],D0.7,#0x1f
        DLG       ASLS      [D0AW.0++],D0.7,#0x1f
        DLG       ASRS      [D0AW.0++],D0.7,#0x1f
        DB        LSLS      [D0AW.0++],[D0AR.0],#0xf
        DLB       LSLS      [D0AW.0++],AC0.0,#0xf
        DB        LSRS      [D0AW.0++],AC0.0,#0xf
        DLB       LSRS      [D0AW.0++],AC0.0,#0xf
        DB        ASLS      [D0AW.0++],AC0.0,#0xf
        DLB       ASLS      [D0AW.0++],AC0.0,#0xf
        DB        ASRS      [D0AW.0++],AC0.0,#0xf
        DLB       ASRS      [D0AW.0++],AC0.0,#0xf
        DB        LSLS      [D0AW.0++],AC0.0,#0x10
        D         LSRS      [D0AW.0++],[D0AR.0],#0x10
        DL        LSRS      [D0AW.0++],[D0AR.0],#0x10
        D         ASLS      [D0AW.0++],[D0AR.0],#0x10
        DL        ASLS      [D0AW.0++],[D0AR.0],#0x10
        D         ASRS      [D0AW.0++],[D0AR.0],#0x10
        DL        ASRS      [D0AW.0++],[D0AR.0],#0x10
        D         LSLS      [D0AW.0++],[D0AR.0],#0x1f
        DL        LSLS      [D0AW.0++],[D0AR.0],#0x1f
        D         LSRS      [D0AW.0++],AC0.0,#0x1f
        DL        LSRS      [D0AW.0++],AC0.0,#0x1f
        D         ASLS      [D0AW.0++],AC0.0,#0x1f
        DL        ASLS      [D0AW.0++],AC0.0,#0x1f
        D         ASRS      [D0AW.0++],AC0.0,#0x1f
        DL        ASRS      [D0AW.0++],AC0.0,#0x1f
        DR        LSLS      [D0AW.0++],[D0AR.0++],#0xf
        DR        LSRS      [D0AW.0++],[D0AR.0++],#0xf
        DR        ASLS      [D0AW.0++],[D0AR.0++],#0xf
        DR        ASRS      [D0AW.0++],[D0AR.0++],#0xf
        DR        LSLS      [D0AW.0++],[D0AR.0++],#0x10
        DR        LSRS      [D0AW.0++],[D0AR.0++],#0x10
        DR        ASLS      [D0AW.0++],[D0AR.0++],#0x10
        DR        ASRS      [D0AW.0++],[D0AR.0++],#0x10
        DR        LSLS      [D0AW.0++],[D0AR.0++],#0x1f
        DR        LSRS      [D0AW.0++],[D0AR.0++],#0x1f
        DR        ASLS      [D0AW.0++],[D0AR.0++],#0x1f
        DR        ASRS      [D0AW.0++],[D0AR.0++],#0x1f
        DR        LSLS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0xf
        DR        LSRS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0xf
        DR        ASLS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0xf
        DR        ASRS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0xf
        DR        LSLS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0x10
        DR        LSRS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0x10
        DR        ASLS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0x10
        DR        ASRS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0x10
        DR        LSLS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0x1f
        DR        LSRS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0x1f
        DR        ASLS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0x1f
        DR        ASRS      [D0AW.0++],[D0AR.0+D0ARI.0++],#0x1f
        DR        LSLS      [D0AW.0+D0AWI.0++],D0Re0,#0xf
        DR        LSRS      [D0AW.0+D0AWI.0++],D0Re0,#0xf
        DR        ASLS      [D0AW.0+D0AWI.0++],D0Re0,#0xf
        DR        ASRS      [D0AW.0+D0AWI.0++],D0Re0,#0xf
        DR        LSLS      [D0AW.0+D0AWI.0++],D0Re0,#0x10
        DR        LSRS      [D0AW.0+D0AWI.0++],D0Re0,#0x10
        DR        ASLS      [D0AW.0+D0AWI.0++],D0Re0,#0x10
        DR        ASRS      [D0AW.0+D0AWI.0++],D0Re0,#0x10
        DR        LSLS      [D0AW.0+D0AWI.0++],D0Re0,#0x1f
        DR        LSRS      [D0AW.0+D0AWI.0++],D0Re0,#0x1f
        DR        ASLS      [D0AW.0+D0AWI.0++],D0Re0,#0x1f
        DR        ASRS      [D0AW.0+D0AWI.0++],D0Re0,#0x1f
        DR        LSLS      [D0AW.0+D0AWI.0++],D0.7,#0xf
        DR        LSRS      [D0AW.0+D0AWI.0++],D0.7,#0xf
        DR        ASLS      [D0AW.0+D0AWI.0++],D0.7,#0xf
        DR        ASRS      [D0AW.0+D0AWI.0++],D0.7,#0xf
        DR        LSLS      [D0AW.0+D0AWI.0++],D0.7,#0x10
        DR        LSRS      [D0AW.0+D0AWI.0++],D0.7,#0x10
        DR        ASLS      [D0AW.0+D0AWI.0++],D0.7,#0x10
        DR        ASRS      [D0AW.0+D0AWI.0++],D0.7,#0x10
        DR        LSLS      [D0AW.0+D0AWI.0++],D0.7,#0x1f
        DR        LSRS      [D0AW.0+D0AWI.0++],D0.7,#0x1f
        DR        ASLS      [D0AW.0+D0AWI.0++],D0.7,#0x1f
        DR        ASRS      [D0AW.0+D0AWI.0++],D0.7,#0x1f
        D         LSLS      [D0AW.0+D0AWI.0++],AC0.0,#0xf
        DLR       LSLS      [D0AW.0+D0AWI.0++],[D0AR.0],#0xf
        DR        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0],#0xf
        DLR       LSRS      [D0AW.0+D0AWI.0++],[D0AR.0],#0xf
        DR        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0],#0xf
        DLR       ASLS      [D0AW.0+D0AWI.0++],[D0AR.0],#0xf
        DR        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0],#0xf
        DLR       ASRS      [D0AW.0+D0AWI.0++],[D0AR.0],#0xf
        DR        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0],#0x10
        DLR       LSLS      [D0AW.0+D0AWI.0++],AC0.0,#0x10
        DR        LSRS      [D0AW.0+D0AWI.0++],AC0.0,#0x10
        DLR       LSRS      [D0AW.0+D0AWI.0++],AC0.0,#0x10
        DR        ASLS      [D0AW.0+D0AWI.0++],AC0.0,#0x10
        DLR       ASLS      [D0AW.0+D0AWI.0++],AC0.0,#0x10
        DR        ASRS      [D0AW.0+D0AWI.0++],AC0.0,#0x10
        DLR       ASRS      [D0AW.0+D0AWI.0++],AC0.0,#0x10
        DR        LSLS      [D0AW.0+D0AWI.0++],AC0.0,#0x1f
        DLG       LSLS      [D0AW.0+D0AWI.0++],[D0AR.0],#0x1f
        DG        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0],#0x1f
        DLG       LSRS      [D0AW.0+D0AWI.0++],[D0AR.0],#0x1f
        DG        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0],#0x1f
        DLG       ASLS      [D0AW.0+D0AWI.0++],[D0AR.0],#0x1f
        DG        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0],#0x1f
        DLG       ASRS      [D0AW.0+D0AWI.0++],[D0AR.0],#0x1f
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0xf
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0xf
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0xf
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0xf
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0x10
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0x10
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0x10
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0x10
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0x1f
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0x1f
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0x1f
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0++],#0x1f
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0xf
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0xf
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0xf
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0xf
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x10
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x10
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x10
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x10
        DL        LSLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x1f
        DL        LSRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x1f
        DL        ASLS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x1f
        DL        ASRS      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x1f
        DL        LSLS      D1Re0,D1Re0,#0xf
        DL        LSRS      D1Re0,D1Re0,#0xf
        DL        ASLS      D1Re0,D1Re0,#0xf
        DL        ASRS      D1Re0,D1Re0,#0xf
        DL        LSLS      D1Re0,D1Re0,#0x10
        DL        LSRS      D1Re0,D1Re0,#0x10
        DL        ASLS      D1Re0,D1Re0,#0x10
        DL        ASRS      D1Re0,D1Re0,#0x10
        DL        LSLS      D1Re0,D1Re0,#0x1f
        DL        LSRS      D1Re0,D1Re0,#0x1f
        DL        ASLS      D1Re0,D1Re0,#0x1f
        DL        ASRS      D1Re0,D1Re0,#0x1f
        DL        LSLS      D1Re0,D1.7,#0xf
        DL        LSRS      D1Re0,D1.7,#0xf
        DL        ASLS      D1Re0,D1.7,#0xf
        DL        ASRS      D1Re0,D1.7,#0xf
        DL        LSLS      D1Re0,D1.7,#0x10
        DL        LSRS      D1Re0,D1.7,#0x10
        DL        ASLS      D1Re0,D1.7,#0x10
        DL        ASRS      D1Re0,D1.7,#0x10
        DL        LSLS      D1Re0,D1.7,#0x1f
        DL        LSRS      D1Re0,D1.7,#0x1f
        DL        ASLS      D1Re0,D1.7,#0x1f
        DL        ASRS      D1Re0,D1.7,#0x1f
        DLR       LSLS      D1Re0,AC1.0,#0xf
        DLR       LSRS      D1Re0,AC1.0,#0xf
        DLR       ASLS      D1Re0,AC1.0,#0xf
        DLR       ASRS      D1Re0,AC1.0,#0xf
        DLG       LSLS      D1Re0,AC1.0,#0x10
        DLG       LSRS      D1Re0,AC1.0,#0x10
        DLG       ASLS      D1Re0,AC1.0,#0x10
        DLG       ASRS      D1Re0,AC1.0,#0x10
        DLB       LSLS      D1Re0,AC1.0,#0x1f
        DLB       LSRS      D1Re0,AC1.0,#0x1f
        DLB       ASLS      D1Re0,AC1.0,#0x1f
        DLB       ASRS      D1Re0,AC1.0,#0x1f
        DLB       LSLS      D1Re0,[D1BR.1],#0xf
        DLB       LSRS      D1Re0,[D1BR.1],#0xf
        DLB       ASLS      D1Re0,[D1BR.1],#0xf
        DLB       ASRS      D1Re0,[D1BR.1],#0xf
        DLB       LSLS      D1Re0,[D1BR.1],#0x10
        DLB       LSRS      D1Re0,[D1BR.1],#0x10
        DLB       ASLS      D1Re0,[D1BR.1],#0x10
        DLB       ASRS      D1Re0,[D1BR.1],#0x10
        DLB       LSLS      D1Re0,[D1BR.1],#0x1f
        DLB       LSRS      D1Re0,[D1BR.1],#0x1f
        DLB       ASLS      D1Re0,[D1BR.1],#0x1f
        DLB       ASRS      D1Re0,[D1BR.1],#0x1f
        DLB       LSLS      D1Re0,[D1BR.1++],#0xf
        DLB       LSRS      D1Re0,[D1BR.1++],#0xf
        DLB       ASLS      D1Re0,[D1BR.1++],#0xf
        DLB       ASRS      D1Re0,[D1BR.1++],#0xf
        DLB       LSLS      D1Re0,[D1BR.1++],#0x10
        DLB       LSRS      D1Re0,[D1BR.1++],#0x10
        DLB       ASLS      D1Re0,[D1BR.1++],#0x10
        DLB       ASRS      D1Re0,[D1BR.1++],#0x10
        DLB       LSLS      D1Re0,[D1BR.1++],#0x1f
        DLB       LSRS      D1Re0,[D1BR.1++],#0x1f
        DLB       ASLS      D1Re0,[D1BR.1++],#0x1f
        DLB       ASRS      D1Re0,[D1BR.1++],#0x1f
        DLB       LSLS      D1Re0,[D1BR.1+D1BRI.1++],#0xf
        DLB       LSRS      D1Re0,[D1BR.1+D1BRI.1++],#0xf
        DLB       ASLS      D1Re0,[D1BR.1+D1BRI.1++],#0xf
        DLB       ASRS      D1Re0,[D1BR.1+D1BRI.1++],#0xf
        DLB       LSLS      D1Re0,[D1BR.1+D1BRI.1++],#0x10
        DLB       LSRS      D1Re0,[D1BR.1+D1BRI.1++],#0x10
        DLB       ASLS      D1Re0,[D1BR.1+D1BRI.1++],#0x10
        DLB       ASRS      D1Re0,[D1BR.1+D1BRI.1++],#0x10
        DLB       LSLS      D1Re0,[D1BR.1+D1BRI.1++],#0x1f
        DLB       LSRS      D1Re0,[D1BR.1+D1BRI.1++],#0x1f
        DLB       ASLS      D1Re0,[D1BR.1+D1BRI.1++],#0x1f
        DLB       ASRS      D1Re0,[D1BR.1+D1BRI.1++],#0x1f
        DLB       LSLS      D1.7,D1Re0,#0xf
        DLB       LSRS      D1.7,D1Re0,#0xf
        DLB       ASLS      D1.7,D1Re0,#0xf
        DLB       ASRS      D1.7,D1Re0,#0xf
        DLB       LSLS      D1.7,D1Re0,#0x10
        DLB       LSRS      D1.7,D1Re0,#0x10
        DLB       ASLS      D1.7,D1Re0,#0x10
        DLB       ASRS      D1.7,D1Re0,#0x10
        DLB       LSLS      D1.7,D1Re0,#0x1f
        DLB       LSRS      D1.7,D1Re0,#0x1f
        DLB       ASLS      D1.7,D1Re0,#0x1f
        DLB       ASRS      D1.7,D1Re0,#0x1f
        DLB       LSLS      D1.7,D1.7,#0xf
        DLB       LSRS      D1.7,D1.7,#0xf
        DLB       ASLS      D1.7,D1.7,#0xf
        DLB       ASRS      D1.7,D1.7,#0xf
        DLB       LSLS      D1.7,D1.7,#0x10
        DLB       LSRS      D1.7,D1.7,#0x10
        DLB       ASLS      D1.7,D1.7,#0x10
        DLB       ASRS      D1.7,D1.7,#0x10
        DLB       LSLS      D1.7,D1.7,#0x1f
        DLB       LSRS      D1.7,D1.7,#0x1f
        DLB       ASLS      D1.7,D1.7,#0x1f
        DLB       ASRS      D1.7,D1.7,#0x1f
        D         LSRS      D1.7,AC1.0,#0xf
        D         ASLS      D1.7,AC1.0,#0xf
        D         ASRS      D1.7,AC1.0,#0xf
        D         LSLS      D1.7,AC1.0,#0x10
        DR        LSRS      D1.7,AC1.0,#0x10
        DR        ASLS      D1.7,AC1.0,#0x10
        DR        ASRS      D1.7,AC1.0,#0x10
        DR        LSLS      D1.7,AC1.0,#0x1f
        DG        LSRS      D1.7,AC1.0,#0x1f
        DG        ASLS      D1.7,AC1.0,#0x1f
        DG        ASRS      D1.7,AC1.0,#0x1f
        DG        LSLS      D1.7,[D1BR.1],#0xf
        DG        LSRS      D1.7,[D1BR.1],#0xf
        DG        ASLS      D1.7,[D1BR.1],#0xf
        DG        ASRS      D1.7,[D1BR.1],#0xf
        DG        LSLS      D1.7,[D1BR.1],#0x10
        DG        LSRS      D1.7,[D1BR.1],#0x10
        DG        ASLS      D1.7,[D1BR.1],#0x10
        DG        ASRS      D1.7,[D1BR.1],#0x10
        DG        LSLS      D1.7,[D1BR.1],#0x1f
        DG        LSRS      D1.7,[D1BR.1],#0x1f
        DG        ASLS      D1.7,[D1BR.1],#0x1f
        DG        ASRS      D1.7,[D1BR.1],#0x1f
        DG        LSLS      D1.7,[D1BR.1++],#0xf
        DG        LSRS      D1.7,[D1BR.1++],#0xf
        DG        ASLS      D1.7,[D1BR.1++],#0xf
        DG        ASRS      D1.7,[D1BR.1++],#0xf
        DG        LSLS      D1.7,[D1BR.1++],#0x10
        DG        LSRS      D1.7,[D1BR.1++],#0x10
        DG        ASLS      D1.7,[D1BR.1++],#0x10
        DG        ASRS      D1.7,[D1BR.1++],#0x10
        DG        LSLS      D1.7,[D1BR.1++],#0x1f
        DG        LSRS      D1.7,[D1BR.1++],#0x1f
        DG        ASLS      D1.7,[D1BR.1++],#0x1f
        DG        ASRS      D1.7,[D1BR.1++],#0x1f
        DG        LSLS      D1.7,[D1BR.1+D1BRI.1++],#0xf
        DG        LSRS      D1.7,[D1BR.1+D1BRI.1++],#0xf
        DG        ASLS      D1.7,[D1BR.1+D1BRI.1++],#0xf
        DG        ASRS      D1.7,[D1BR.1+D1BRI.1++],#0xf
        DG        LSLS      D1.7,[D1BR.1+D1BRI.1++],#0x10
        DG        LSRS      D1.7,[D1BR.1+D1BRI.1++],#0x10
        DG        ASLS      D1.7,[D1BR.1+D1BRI.1++],#0x10
        DG        ASRS      D1.7,[D1BR.1+D1BRI.1++],#0x10
        DG        LSLS      D1.7,[D1BR.1+D1BRI.1++],#0x1f
        DG        LSRS      D1.7,[D1BR.1+D1BRI.1++],#0x1f
        DG        ASLS      D1.7,[D1BR.1+D1BRI.1++],#0x1f
        DG        ASRS      D1.7,[D1BR.1+D1BRI.1++],#0x1f
        DG        LSLS      [D1BW.1],D1Re0,#0xf
        DG        LSRS      [D1BW.1],D1Re0,#0xf
        DG        ASLS      [D1BW.1],D1Re0,#0xf
        DG        ASRS      [D1BW.1],D1Re0,#0xf
        DG        LSLS      [D1BW.1],D1Re0,#0x10
        DG        LSRS      [D1BW.1],D1Re0,#0x10
        DG        ASLS      [D1BW.1],D1Re0,#0x10
        DG        ASRS      [D1BW.1],D1Re0,#0x10
        DG        LSLS      [D1BW.1],D1Re0,#0x1f
        DG        LSRS      [D1BW.1],D1Re0,#0x1f
        DG        ASLS      [D1BW.1],D1Re0,#0x1f
        DG        ASRS      [D1BW.1],D1Re0,#0x1f
        DG        LSLS      [D1BW.1],D1.7,#0xf
        DG        LSRS      [D1BW.1],D1.7,#0xf
        DG        ASLS      [D1BW.1],D1.7,#0xf
        DG        ASRS      [D1BW.1],D1.7,#0xf
        DG        LSLS      [D1BW.1],D1.7,#0x10
        DG        LSRS      [D1BW.1],D1.7,#0x10
        DG        ASLS      [D1BW.1],D1.7,#0x10
        DG        ASRS      [D1BW.1],D1.7,#0x10
        DG        LSLS      [D1BW.1],D1.7,#0x1f
        DG        LSRS      [D1BW.1],D1.7,#0x1f
        DG        ASLS      [D1BW.1],D1.7,#0x1f
        DG        ASRS      [D1BW.1],D1.7,#0x1f
        DG        LSLS      [D1BW.1],AC1.0,#0xf
        DB        LSRS      [D1BW.1],AC1.0,#0xf
        DB        ASLS      [D1BW.1],AC1.0,#0xf
        DB        ASRS      [D1BW.1],AC1.0,#0xf
        DB        LSLS      [D1BW.1],AC1.0,#0x10
        DL        LSRS      [D1BW.1],AC1.0,#0x10
        DL        ASLS      [D1BW.1],AC1.0,#0x10
        DL        ASRS      [D1BW.1],AC1.0,#0x10
        DLR       LSLS      [D1BW.1],AC1.0,#0x1f
        DLR       LSRS      [D1BW.1],AC1.0,#0x1f
        DLR       ASLS      [D1BW.1],AC1.0,#0x1f
        DLR       ASRS      [D1BW.1],AC1.0,#0x1f
        DLR       LSLS      [D1BW.1],[D1BR.1],#0xf
        DLR       LSRS      [D1BW.1],[D1BR.1],#0xf
        DLR       ASLS      [D1BW.1],[D1BR.1],#0xf
        DLR       ASRS      [D1BW.1],[D1BR.1],#0xf
        DLR       LSLS      [D1BW.1],[D1BR.1],#0x10
        DLR       LSRS      [D1BW.1],[D1BR.1],#0x10
        DLR       ASLS      [D1BW.1],[D1BR.1],#0x10
        DLR       ASRS      [D1BW.1],[D1BR.1],#0x10
        DLR       LSLS      [D1BW.1],[D1BR.1],#0x1f
        DLR       LSRS      [D1BW.1],[D1BR.1],#0x1f
        DLR       ASLS      [D1BW.1],[D1BR.1],#0x1f
        DLR       ASRS      [D1BW.1],[D1BR.1],#0x1f
        DLR       LSLS      [D1BW.1],[D1BR.1++],#0xf
        DLR       LSRS      [D1BW.1],[D1BR.1++],#0xf
        DLR       ASLS      [D1BW.1],[D1BR.1++],#0xf
        DLR       ASRS      [D1BW.1],[D1BR.1++],#0xf
        DLR       LSLS      [D1BW.1],[D1BR.1++],#0x10
        DLR       LSRS      [D1BW.1],[D1BR.1++],#0x10
        DLR       ASLS      [D1BW.1],[D1BR.1++],#0x10
        DLR       ASRS      [D1BW.1],[D1BR.1++],#0x10
        DLR       LSLS      [D1BW.1],[D1BR.1++],#0x1f
        DLR       LSRS      [D1BW.1],[D1BR.1++],#0x1f
        DLR       ASLS      [D1BW.1],[D1BR.1++],#0x1f
        DLR       ASRS      [D1BW.1],[D1BR.1++],#0x1f
        DLR       LSLS      [D1BW.1],[D1BR.1+D1BRI.1++],#0xf
        DLR       LSRS      [D1BW.1],[D1BR.1+D1BRI.1++],#0xf
        DLR       ASLS      [D1BW.1],[D1BR.1+D1BRI.1++],#0xf
        DLR       ASRS      [D1BW.1],[D1BR.1+D1BRI.1++],#0xf
        DLR       LSLS      [D1BW.1],[D1BR.1+D1BRI.1++],#0x10
        DLR       LSRS      [D1BW.1],[D1BR.1+D1BRI.1++],#0x10
        DLR       ASLS      [D1BW.1],[D1BR.1+D1BRI.1++],#0x10
        DLR       ASRS      [D1BW.1],[D1BR.1+D1BRI.1++],#0x10
        DLR       LSLS      [D1BW.1],[D1BR.1+D1BRI.1++],#0x1f
        DLR       LSRS      [D1BW.1],[D1BR.1+D1BRI.1++],#0x1f
        DLR       ASLS      [D1BW.1],[D1BR.1+D1BRI.1++],#0x1f
        DLR       ASRS      [D1BW.1],[D1BR.1+D1BRI.1++],#0x1f
        DLR       LSLS      [D1BW.1++],D1Re0,#0xf
        DLR       LSRS      [D1BW.1++],D1Re0,#0xf
        DLR       ASLS      [D1BW.1++],D1Re0,#0xf
        DLR       ASRS      [D1BW.1++],D1Re0,#0xf
        DLR       LSLS      [D1BW.1++],D1Re0,#0x10
        DLR       LSRS      [D1BW.1++],D1Re0,#0x10
        DLR       ASLS      [D1BW.1++],D1Re0,#0x10
        DLR       ASRS      [D1BW.1++],D1Re0,#0x10
        DLR       LSLS      [D1BW.1++],D1Re0,#0x1f
        DLR       LSRS      [D1BW.1++],D1Re0,#0x1f
        DLR       ASLS      [D1BW.1++],D1Re0,#0x1f
        DLR       ASRS      [D1BW.1++],D1Re0,#0x1f
        DLR       LSLS      [D1BW.1++],D1.7,#0xf
        DLR       LSRS      [D1BW.1++],D1.7,#0xf
        DLR       ASLS      [D1BW.1++],D1.7,#0xf
        DLR       ASRS      [D1BW.1++],D1.7,#0xf
        DLR       LSLS      [D1BW.1++],D1.7,#0x10
        DLR       LSRS      [D1BW.1++],D1.7,#0x10
        DLR       ASLS      [D1BW.1++],D1.7,#0x10
        DLR       ASRS      [D1BW.1++],D1.7,#0x10
        DLR       LSLS      [D1BW.1++],D1.7,#0x1f
        DLR       LSRS      [D1BW.1++],D1.7,#0x1f
        DLR       ASLS      [D1BW.1++],D1.7,#0x1f
        DLR       ASRS      [D1BW.1++],D1.7,#0x1f
        DLG       LSLS      [D1BW.1++],AC1.0,#0xf
        DLG       LSRS      [D1BW.1++],AC1.0,#0xf
        DLG       ASLS      [D1BW.1++],AC1.0,#0xf
        DLG       ASRS      [D1BW.1++],AC1.0,#0xf
        DLB       LSLS      [D1BW.1++],AC1.0,#0x10
        DLB       LSRS      [D1BW.1++],AC1.0,#0x10
        DLB       ASLS      [D1BW.1++],AC1.0,#0x10
        DLB       ASRS      [D1BW.1++],AC1.0,#0x10
        D         LSRS      [D1BW.1++],AC1.0,#0x1f
        D         ASLS      [D1BW.1++],AC1.0,#0x1f
        D         ASRS      [D1BW.1++],AC1.0,#0x1f
        D         LSLS      [D1BW.1++],[D1BR.1],#0xf
        D         LSRS      [D1BW.1++],[D1BR.1],#0xf
        D         ASLS      [D1BW.1++],[D1BR.1],#0xf
        D         ASRS      [D1BW.1++],[D1BR.1],#0xf
        D         LSLS      [D1BW.1++],[D1BR.1],#0x10
        D         LSRS      [D1BW.1++],[D1BR.1],#0x10
        D         ASLS      [D1BW.1++],[D1BR.1],#0x10
        D         ASRS      [D1BW.1++],[D1BR.1],#0x10
        D         LSLS      [D1BW.1++],[D1BR.1],#0x1f
        D         LSRS      [D1BW.1++],[D1BR.1],#0x1f
        D         ASLS      [D1BW.1++],[D1BR.1],#0x1f
        D         ASRS      [D1BW.1++],[D1BR.1],#0x1f
        D         LSLS      [D1BW.1++],[D1BR.1++],#0xf
        D         LSRS      [D1BW.1++],[D1BR.1++],#0xf
        D         ASLS      [D1BW.1++],[D1BR.1++],#0xf
        D         ASRS      [D1BW.1++],[D1BR.1++],#0xf
        D         LSLS      [D1BW.1++],[D1BR.1++],#0x10
        D         LSRS      [D1BW.1++],[D1BR.1++],#0x10
        D         ASLS      [D1BW.1++],[D1BR.1++],#0x10
        D         ASRS      [D1BW.1++],[D1BR.1++],#0x10
        D         LSLS      [D1BW.1++],[D1BR.1++],#0x1f
        D         LSRS      [D1BW.1++],[D1BR.1++],#0x1f
        D         ASLS      [D1BW.1++],[D1BR.1++],#0x1f
        D         ASRS      [D1BW.1++],[D1BR.1++],#0x1f
        D         LSLS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0xf
        D         LSRS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0xf
        D         ASLS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0xf
        D         ASRS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0xf
        D         LSLS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0x10
        D         LSRS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0x10
        D         ASLS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0x10
        D         ASRS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0x10
        D         LSLS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0x1f
        D         LSRS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0x1f
        D         ASLS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0x1f
        D         ASRS      [D1BW.1++],[D1BR.1+D1BRI.1++],#0x1f
        D         LSLS      [D1BW.1+D1BWI.1++],D1Re0,#0xf
        D         LSRS      [D1BW.1+D1BWI.1++],D1Re0,#0xf
        D         ASLS      [D1BW.1+D1BWI.1++],D1Re0,#0xf
        D         ASRS      [D1BW.1+D1BWI.1++],D1Re0,#0xf
        D         LSLS      [D1BW.1+D1BWI.1++],D1Re0,#0x10
        D         LSRS      [D1BW.1+D1BWI.1++],D1Re0,#0x10
        D         ASLS      [D1BW.1+D1BWI.1++],D1Re0,#0x10
        D         ASRS      [D1BW.1+D1BWI.1++],D1Re0,#0x10
        D         LSLS      [D1BW.1+D1BWI.1++],D1Re0,#0x1f
        D         LSRS      [D1BW.1+D1BWI.1++],D1Re0,#0x1f
        D         ASLS      [D1BW.1+D1BWI.1++],D1Re0,#0x1f
        D         ASRS      [D1BW.1+D1BWI.1++],D1Re0,#0x1f
        D         LSLS      [D1BW.1+D1BWI.1++],D1.7,#0xf
        D         LSRS      [D1BW.1+D1BWI.1++],D1.7,#0xf
        D         ASLS      [D1BW.1+D1BWI.1++],D1.7,#0xf
        D         ASRS      [D1BW.1+D1BWI.1++],D1.7,#0xf
        D         LSLS      [D1BW.1+D1BWI.1++],D1.7,#0x10
        D         LSRS      [D1BW.1+D1BWI.1++],D1.7,#0x10
        D         ASLS      [D1BW.1+D1BWI.1++],D1.7,#0x10
        D         ASRS      [D1BW.1+D1BWI.1++],D1.7,#0x10
        D         LSLS      [D1BW.1+D1BWI.1++],D1.7,#0x1f
        D         LSRS      [D1BW.1+D1BWI.1++],D1.7,#0x1f
        D         ASLS      [D1BW.1+D1BWI.1++],D1.7,#0x1f
        D         ASRS      [D1BW.1+D1BWI.1++],D1.7,#0x1f
        D         LSLS      [D1BW.1+D1BWI.1++],AC1.0,#0xf
        DR        LSRS      [D1BW.1+D1BWI.1++],AC1.0,#0xf
        DR        ASLS      [D1BW.1+D1BWI.1++],AC1.0,#0xf
        DR        ASRS      [D1BW.1+D1BWI.1++],AC1.0,#0xf
        DR        LSLS      [D1BW.1+D1BWI.1++],AC1.0,#0x10
        DG        LSRS      [D1BW.1+D1BWI.1++],AC1.0,#0x10
        DG        ASLS      [D1BW.1+D1BWI.1++],AC1.0,#0x10
        DG        ASRS      [D1BW.1+D1BWI.1++],AC1.0,#0x10
        DG        LSLS      [D1BW.1+D1BWI.1++],AC1.0,#0x1f
        DB        LSRS      [D1BW.1+D1BWI.1++],AC1.0,#0x1f
        DB        ASLS      [D1BW.1+D1BWI.1++],AC1.0,#0x1f
        DB        ASRS      [D1BW.1+D1BWI.1++],AC1.0,#0x1f
        DB        LSLS      [D1BW.1+D1BWI.1++],[D1BR.1],#0xf
        DB        LSRS      [D1BW.1+D1BWI.1++],[D1BR.1],#0xf
        DB        ASLS      [D1BW.1+D1BWI.1++],[D1BR.1],#0xf
        DB        ASRS      [D1BW.1+D1BWI.1++],[D1BR.1],#0xf
        DB        LSLS      [D1BW.1+D1BWI.1++],[D1BR.1],#0x10
        DB        LSRS      [D1BW.1+D1BWI.1++],[D1BR.1],#0x10
        DB        ASLS      [D1BW.1+D1BWI.1++],[D1BR.1],#0x10
        DB        ASRS      [D1BW.1+D1BWI.1++],[D1BR.1],#0x10
        DB        LSLS      [D1BW.1+D1BWI.1++],[D1BR.1],#0x1f
        DB        LSRS      [D1BW.1+D1BWI.1++],[D1BR.1],#0x1f
        DB        ASLS      [D1BW.1+D1BWI.1++],[D1BR.1],#0x1f
        DB        ASRS      [D1BW.1+D1BWI.1++],[D1BR.1],#0x1f
        DB        LSLS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0xf
        DB        LSRS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0xf
        DB        ASLS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0xf
        DB        ASRS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0xf
        DB        LSLS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0x10
        DB        LSRS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0x10
        DB        ASLS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0x10
        DB        ASRS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0x10
        DB        LSLS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0x1f
        DB        LSRS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0x1f
        DB        ASLS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0x1f
        DB        ASRS      [D1BW.1+D1BWI.1++],[D1BR.1++],#0x1f
        DB        LSLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0xf
        DB        LSRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0xf
        DB        ASLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0xf
        DB        ASRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0xf
        DB        LSLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x10
        DB        LSRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x10
        DB        ASLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x10
        DB        ASRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x10
        DB        LSLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x1f
        DB        LSRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x1f
        DB        ASLS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x1f
        DB        ASRS      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#0x1f
        D         LSLS      D1Re0,D0Re0,D0Re0
        D         LSRS      TXSTAT,D0Re0,D0Re0
        D         ASRSNV    D0Re0,D0Re0,D0Re0
        D         LSLS      D1Re0,D0Re0,D0.7
        D         LSRS      TXSTAT,D0Re0,D0.7
        D         ASRSNV    D0Re0,D0Re0,D0.7
        D         LSLS      D1Re0,D0Re0,[D0AR.0]
        D         LSRS      TXSTAT,D0Re0,[D0AR.0]
        D         ASRSNV    D0Re0,D0Re0,[D0AR.0]
        D         LSLS      D1Re0,D0Re0,[D0AR.0++]
        D         LSRS      TXSTAT,D0Re0,[D0AR.0++]
        D         ASRSNV    D0Re0,D0Re0,[D0AR.0++]
        D         LSLS      D1Re0,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSRS      TXSTAT,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASRSNV    D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSLS      D1Re0,D0.7,D0Re0
        D         LSRS      TXSTAT,D0.7,D0Re0
        D         ASRSNV    D0Re0,D0.7,D0Re0
        D         LSLS      D1Re0,D0.7,D0.7
        D         LSRS      TXSTAT,D0.7,D0.7
        D         ASRSNV    D0Re0,D0.7,D0.7
        D         LSLS      D1Re0,D0.7,[D0AR.0]
        D         LSRS      TXSTAT,D0.7,[D0AR.0]
        D         ASRSNV    D0Re0,D0.7,[D0AR.0]
        D         LSLS      D1Re0,D0.7,[D0AR.0++]
        D         LSRS      TXSTAT,D0.7,[D0AR.0++]
        D         ASRSNV    D0Re0,D0.7,[D0AR.0++]
        D         LSLS      D1Re0,D0.7,[D0AR.0+D0ARI.0++]
        D         LSRS      TXSTAT,D0.7,[D0AR.0+D0ARI.0++]
        D         ASRSNV    D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        D         LSLSNV    D0Re0,AC0.0,D0Re0
        D         LSLS      TXSTAT,AC0.0,D0Re0
        D         LSRS      D1Re0,AC0.0,D0Re0
        D         ASLSNV    D0Re0,AC0.0,D0Re0
        D         ASLS      TXSTAT,AC0.0,D0Re0
        D         ASRS      D1Re0,AC0.0,D0Re0
        D         LSLSNV    D0Re0,AC0.0,D0.7
        D         LSLS      TXSTAT,AC0.0,D0.7
        D         LSRS      D1Re0,AC0.0,D0.7
        D         ASLSNV    D0Re0,AC0.0,D0.7
        D         ASLS      TXSTAT,AC0.0,D0.7
        D         ASRS      D1Re0,AC0.0,D0.7
        D         LSLSNV    D0Re0,AC0.0,[D0AR.0]
        D         LSLS      TXSTAT,AC0.0,[D0AR.0]
        D         LSRS      D1Re0,AC0.0,[D0AR.0]
        D         ASLSNV    D0Re0,AC0.0,[D0AR.0]
        D         ASLS      TXSTAT,AC0.0,[D0AR.0]
        D         ASRS      D1Re0,AC0.0,[D0AR.0]
        D         LSLSNV    D0Re0,AC0.0,[D0AR.0++]
        D         LSLS      TXSTAT,AC0.0,[D0AR.0++]
        D         LSRS      D1Re0,AC0.0,[D0AR.0++]
        D         ASLSNV    D0Re0,AC0.0,[D0AR.0++]
        D         ASLS      TXSTAT,AC0.0,[D0AR.0++]
        D         ASRS      D1Re0,AC0.0,[D0AR.0++]
        D         LSLSNV    D0Re0,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSLS      TXSTAT,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSRS      D1Re0,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASLSNV    D0Re0,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASLS      TXSTAT,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASRS      D1Re0,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSLS      D1Re0,[D0AR.0++],D0Re0
        D         LSRS      TXSTAT,[D0AR.0++],D0Re0
        D         ASRSNV    D0Re0,[D0AR.0++],D0Re0
        D         LSLS      D1Re0,[D0AR.0++],D0.7
        D         LSRS      TXSTAT,[D0AR.0++],D0.7
        D         ASRSNV    D0Re0,[D0AR.0++],D0.7
        D         LSLS      D1Re0,[D0AR.0++],[D0AR.0]
        D         LSRS      TXSTAT,[D0AR.0++],[D0AR.0]
        D         ASRSNV    D0Re0,[D0AR.0++],[D0AR.0]
        D         LSLS      D1Re0,[D0AR.0++],[D0AR.0++]
        D         LSRS      TXSTAT,[D0AR.0++],[D0AR.0++]
        D         ASRSNV    D0Re0,[D0AR.0++],[D0AR.0++]
        D         LSLS      D1Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSRS      TXSTAT,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASRSNV    D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSLS      D1Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         LSRS      TXSTAT,[D0AR.0+D0ARI.0++],D0Re0
        D         ASRSNV    D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        D         LSLS      D1Re0,[D0AR.0+D0ARI.0++],D0.7
        D         LSRS      TXSTAT,[D0AR.0+D0ARI.0++],D0.7
        D         ASRSNV    D0Re0,[D0AR.0+D0ARI.0++],D0.7
        D         LSLS      D1Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSRS      TXSTAT,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASRSNV    D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSLS      D1Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSRS      TXSTAT,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASRSNV    D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSLS      D1Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSRS      TXSTAT,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASRSNV    D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSRS      A0FrP,D0Re0,D0Re0
        D         ASRS      PCX,D0Re0,D0Re0
        D         ASLS      A1LbP,D0Re0,D0.7
        D         LSRS      A0FrP,D0Re0,[D0AR.0]
        D         ASRS      PCX,D0Re0,[D0AR.0]
        D         ASLS      A1LbP,D0Re0,[D0AR.0++]
        D         LSRS      A0FrP,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASRS      PCX,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASLS      A1LbP,D0.7,D0Re0
        D         LSRS      A0FrP,D0.7,D0.7
        D         ASRS      PCX,D0.7,D0.7
        D         ASLS      A1LbP,D0.7,[D0AR.0]
        D         LSRS      A0FrP,D0.7,[D0AR.0++]
        D         ASRS      PCX,D0.7,[D0AR.0++]
        D         ASLS      A1LbP,D0.7,[D0AR.0+D0ARI.0++]
        D         LSLS      A1LbP,AC0.0,D0Re0
        D         LSRS      PCX,AC0.0,D0Re0
        D         ASRS      A0FrP,AC0.0,D0Re0
        D         LSLS      A1LbP,AC0.0,D0.7
        D         LSRS      PCX,AC0.0,D0.7
        D         ASRS      A0FrP,AC0.0,D0.7
        D         LSLS      A1LbP,AC0.0,[D0AR.0]
        D         LSRS      PCX,AC0.0,[D0AR.0]
        D         ASRS      A0FrP,AC0.0,[D0AR.0]
        D         LSLS      A1LbP,AC0.0,[D0AR.0++]
        D         LSRS      PCX,AC0.0,[D0AR.0++]
        D         ASRS      A0FrP,AC0.0,[D0AR.0++]
        D         LSLS      A1LbP,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSRS      PCX,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASRS      A0FrP,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSRS      A0FrP,[D0AR.0++],D0Re0
        D         ASRS      PCX,[D0AR.0++],D0Re0
        D         ASLS      A1LbP,[D0AR.0++],D0.7
        D         LSRS      A0FrP,[D0AR.0++],[D0AR.0]
        D         ASRS      PCX,[D0AR.0++],[D0AR.0]
        D         ASLS      A1LbP,[D0AR.0++],[D0AR.0++]
        D         LSRS      A0FrP,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASRS      PCX,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASLS      A1LbP,[D0AR.0+D0ARI.0++],D0Re0
        D         LSRS      A0FrP,[D0AR.0+D0ARI.0++],D0.7
        D         ASRS      PCX,[D0AR.0+D0ARI.0++],D0.7
        D         ASLS      A1LbP,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSRS      A0FrP,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASRS      PCX,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASLS      A1LbP,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASRS      TXMASKI,D0Re0,D0Re0
        D         ASRS      TXMASKI,D0Re0,[D0AR.0]
        D         ASRS      TXMASKI,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASRS      TXMASKI,D0.7,D0.7
        D         ASRS      TXMASKI,D0.7,[D0AR.0++]
        D         LSRS      TXMASKI,AC0.0,D0Re0
        D         LSRS      TXMASKI,AC0.0,D0.7
        D         LSRS      TXMASKI,AC0.0,[D0AR.0]
        D         LSRS      TXMASKI,AC0.0,[D0AR.0++]
        D         LSRS      TXMASKI,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASRS      TXMASKI,[D0AR.0++],D0Re0
        D         ASRS      TXMASKI,[D0AR.0++],[D0AR.0]
        D         ASRS      TXMASKI,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASRS      TXMASKI,[D0AR.0+D0ARI.0++],D0.7
        D         ASRS      TXMASKI,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSLS      A0.7,D0Re0,D0Re0
        D         LSRS      TXGPIOO,D0Re0,D0Re0
        D         ASRSNV    D0.7,D0Re0,D0Re0
        D         LSLS      A0.7,D0Re0,D0.7
        D         LSRS      TXGPIOO,D0Re0,D0.7
        D         ASRSNV    D0.7,D0Re0,D0.7
        D         LSLS      A0.7,D0Re0,[D0AR.0]
        D         LSRS      TXGPIOO,D0Re0,[D0AR.0]
        D         ASRSNV    D0.7,D0Re0,[D0AR.0]
        D         LSLS      A0.7,D0Re0,[D0AR.0++]
        D         LSRS      TXGPIOO,D0Re0,[D0AR.0++]
        D         ASRSNV    D0.7,D0Re0,[D0AR.0++]
        D         LSLS      A0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSRS      TXGPIOO,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASRSNV    D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSLS      A0.7,D0.7,D0Re0
        D         LSRS      TXGPIOO,D0.7,D0Re0
        D         ASRSNV    D0.7,D0.7,D0Re0
        D         LSLS      A0.7,D0.7,D0.7
        D         LSRS      TXGPIOO,D0.7,D0.7
        D         ASRSNV    D0.7,D0.7,D0.7
        D         LSLS      A0.7,D0.7,[D0AR.0]
        D         LSRS      TXGPIOO,D0.7,[D0AR.0]
        D         ASRSNV    D0.7,D0.7,[D0AR.0]
        D         LSLS      A0.7,D0.7,[D0AR.0++]
        D         LSRS      TXGPIOO,D0.7,[D0AR.0++]
        D         ASRSNV    D0.7,D0.7,[D0AR.0++]
        D         LSLS      A0.7,D0.7,[D0AR.0+D0ARI.0++]
        D         LSRS      TXGPIOO,D0.7,[D0AR.0+D0ARI.0++]
        D         ASRSNV    D0.7,D0.7,[D0AR.0+D0ARI.0++]
        D         LSLSNV    D0.7,AC0.0,D0Re0
        D         LSLS      TXGPIOO,AC0.0,D0Re0
        D         LSRS      A0.7,AC0.0,D0Re0
        D         ASLSNV    D0.7,AC0.0,D0Re0
        D         ASLS      TXGPIOO,AC0.0,D0Re0
        D         ASRS      A0.7,AC0.0,D0Re0
        D         LSLSNV    D0.7,AC0.0,D0.7
        D         LSLS      TXGPIOO,AC0.0,D0.7
        D         LSRS      A0.7,AC0.0,D0.7
        D         ASLSNV    D0.7,AC0.0,D0.7
        D         ASLS      TXGPIOO,AC0.0,D0.7
        D         ASRS      A0.7,AC0.0,D0.7
        D         LSLSNV    D0.7,AC0.0,[D0AR.0]
        D         LSLS      TXGPIOO,AC0.0,[D0AR.0]
        D         LSRS      A0.7,AC0.0,[D0AR.0]
        D         ASLSNV    D0.7,AC0.0,[D0AR.0]
        D         ASLS      TXGPIOO,AC0.0,[D0AR.0]
        D         ASRS      A0.7,AC0.0,[D0AR.0]
        D         LSLSNV    D0.7,AC0.0,[D0AR.0++]
        D         LSLS      TXGPIOO,AC0.0,[D0AR.0++]
        D         LSRS      A0.7,AC0.0,[D0AR.0++]
        D         ASLSNV    D0.7,AC0.0,[D0AR.0++]
        D         ASLS      TXGPIOO,AC0.0,[D0AR.0++]
        D         ASRS      A0.7,AC0.0,[D0AR.0++]
        D         LSLSNV    D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSLS      TXGPIOO,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSRS      A0.7,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASLSNV    D0.7,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASLS      TXGPIOO,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASRS      A0.7,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSLS      A0.7,[D0AR.0++],D0Re0
        D         LSRS      TXGPIOO,[D0AR.0++],D0Re0
        D         ASRSNV    D0.7,[D0AR.0++],D0Re0
        D         LSLS      A0.7,[D0AR.0++],D0.7
        D         LSRS      TXGPIOO,[D0AR.0++],D0.7
        D         ASRSNV    D0.7,[D0AR.0++],D0.7
        D         LSLS      A0.7,[D0AR.0++],[D0AR.0]
        D         LSRS      TXGPIOO,[D0AR.0++],[D0AR.0]
        D         ASRSNV    D0.7,[D0AR.0++],[D0AR.0]
        D         LSLS      A0.7,[D0AR.0++],[D0AR.0++]
        D         LSRS      TXGPIOO,[D0AR.0++],[D0AR.0++]
        D         ASRSNV    D0.7,[D0AR.0++],[D0AR.0++]
        D         LSLS      A0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSRS      TXGPIOO,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASRSNV    D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSLS      A0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         LSRS      TXGPIOO,[D0AR.0+D0ARI.0++],D0Re0
        D         ASRSNV    D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         LSLS      A0.7,[D0AR.0+D0ARI.0++],D0.7
        D         LSRS      TXGPIOO,[D0AR.0+D0ARI.0++],D0.7
        D         ASRSNV    D0.7,[D0AR.0+D0ARI.0++],D0.7
        D         LSLS      A0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSRS      TXGPIOO,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASRSNV    D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSLS      A0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSRS      TXGPIOO,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASRSNV    D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSLS      A0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSRS      TXGPIOO,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASRSNV    D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSRS      A1.8,D0Re0,D0Re0
        D         LSRS      A1.8,D0Re0,D0.7
        D         LSRS      A1.8,D0Re0,[D0AR.0]
        D         LSRS      A1.8,D0Re0,[D0AR.0++]
        D         LSRS      A1.8,D0Re0,[D0AR.0+D0ARI.0++]
        D         LSRS      A1.8,D0.7,D0Re0
        D         LSRS      A1.8,D0.7,D0.7
        D         LSRS      A1.8,D0.7,[D0AR.0]
        D         LSRS      A1.8,D0.7,[D0AR.0++]
        D         LSRS      A1.8,D0.7,[D0AR.0+D0ARI.0++]
        D         LSLS      A1.8,AC0.0,D0Re0
        D         ASLS      A1.8,AC0.0,D0Re0
        D         LSLS      A1.8,AC0.0,D0.7
        D         ASLS      A1.8,AC0.0,D0.7
        D         LSLS      A1.8,AC0.0,[D0AR.0]
        D         ASLS      A1.8,AC0.0,[D0AR.0]
        D         LSLS      A1.8,AC0.0,[D0AR.0++]
        D         ASLS      A1.8,AC0.0,[D0AR.0++]
        D         LSLS      A1.8,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASLS      A1.8,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSRS      A1.8,[D0AR.0++],D0Re0
        D         LSRS      A1.8,[D0AR.0++],D0.7
        D         LSRS      A1.8,[D0AR.0++],[D0AR.0]
        D         LSRS      A1.8,[D0AR.0++],[D0AR.0++]
        D         LSRS      A1.8,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSRS      A1.8,[D0AR.0+D0ARI.0++],D0Re0
        D         LSRS      A1.8,[D0AR.0+D0ARI.0++],D0.7
        D         LSRS      A1.8,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSRS      A1.8,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         LSRS      A1.8,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASLSNV    D0.16,D0Re0,D0Re0
        D         ASLS      D1.16,D0Re0,D0.7
        D         ASLS      RA,D0Re0,[D0AR.0]
        D         ASRS      RA,D0Re0,[D0AR.0++]
        D         LSLS      RA,D0.7,D0Re0
        D         LSRS      RA,D0.7,D0.7
        DLNE      ASLSNE    D0.16,D0.7,[D0AR.0]
        D         ASLSNV    D0.16,D0.7,[D0AR.0++]
        D         ASLS      D1.16,D0.7,[D0AR.0+D0ARI.0++]
        DHNE      LSRSPL    D0.16,AC0.0,D0Re0
        D         ASLSNV    D0.16,AC0.0,D0Re0
        D         ASRS      RA,[D0AR.0],D0Re0
        D         LSRSNV    D0.16,AC0.0,D0.7
        D         ASLS      D1.16,[D0AR.0],D0.7
        D         ASRS      RA,AC0.0,D0.7
        D         LSRS      D1.16,AC0.0,[D0AR.0]
        D         ASLS      D1.16,AC0.0,[D0AR.0]
        DLNE      LSLSNE    D0.16,AC0.0,[D0AR.0++]
        D         LSRS      RA,[D0AR.0],[D0AR.0++]
        D         ASLS      RA,[D0AR.0],[D0AR.0++]
        D         LSLSNV    D0.16,AC0.0,[D0AR.0+D0ARI.0++]
        D         LSRS      RA,AC0.0,[D0AR.0+D0ARI.0++]
        D         ASLS      RA,AC0.0,[D0AR.0+D0ARI.0++]
        DLNE      ASLSNE    D0.16,[D0AR.0++],D0Re0
        D         ASLSNV    D0.16,[D0AR.0++],D0.7
        D         ASLS      D1.16,[D0AR.0++],[D0AR.0]
        D         ASLS      RA,[D0AR.0++],[D0AR.0++]
        D         ASRS      RA,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         LSLS      RA,[D0AR.0+D0ARI.0++],D0.7
        D         LSRS      RA,[D0AR.0+D0ARI.0++],[D0AR.0]
        DLLS      ASLSLS    D0.16,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASLSNV    D0.16,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSLS      RAM8X32,D0Re0,D0.7
        D         LSLS      RAM8X32,D0Re0,[D0AR.0++]
        D         LSLS      RAM8X32,D0.7,D0Re0
        D         LSLS      RAM8X32,D0.7,[D0AR.0]
        D         LSLS      RAM8X32,D0.7,[D0AR.0+D0ARI.0++]
        D         ASLS      RAM8X32,[D0AR.0],D0Re0
        D         ASLS      RAM8X32,[D0AR.0],D0.7
        D         ASLS      RAM8X32,[D0AR.0],[D0AR.0]
        D         ASLS      RAM8X32,[D0AR.0],[D0AR.0++]
        D         ASLS      RAM8X32,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSLS      RAM8X32,[D0AR.0++],D0.7
        D         LSLS      RAM8X32,[D0AR.0++],[D0AR.0++]
        D         LSLS      RAM8X32,[D0AR.0+D0ARI.0++],D0Re0
        D         LSLS      RAM8X32,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSLS      RAM8X32,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSLS      RAM8X,D0Re0,D0.7
        D         LSLS      RAM8X,D0Re0,[D0AR.0++]
        D         LSLS      RAM8X,D0.7,D0Re0
        D         LSLS      RAM8X,D0.7,[D0AR.0]
        D         LSLS      RAM8X,D0.7,[D0AR.0+D0ARI.0++]
        D         ASLS      RAM8X,[D0AR.0],D0Re0
        D         ASLS      RAM8X,[D0AR.0],D0.7
        D         ASLS      RAM8X,[D0AR.0],[D0AR.0]
        D         ASLS      RAM8X,[D0AR.0],[D0AR.0++]
        D         ASLS      RAM8X,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSLS      RAM8X,[D0AR.0++],D0.7
        D         LSLS      RAM8X,[D0AR.0++],[D0AR.0++]
        D         LSLS      RAM8X,[D0AR.0+D0ARI.0++],D0Re0
        D         LSLS      RAM8X,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSLS      RAM8X,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSLS      RABZ,D0Re0,D0.7
        D         LSLS      RABZ,D0Re0,[D0AR.0++]
        D         LSLS      RABZ,D0.7,D0Re0
        D         LSLS      RABZ,D0.7,[D0AR.0]
        D         LSLS      RABZ,D0.7,[D0AR.0+D0ARI.0++]
        D         ASLS      RABZ,[D0AR.0],D0Re0
        D         ASLS      RABZ,[D0AR.0],D0.7
        D         ASLS      RABZ,[D0AR.0],[D0AR.0]
        D         ASLS      RABZ,[D0AR.0],[D0AR.0++]
        D         ASLS      RABZ,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSLS      RABZ,[D0AR.0++],D0.7
        D         LSLS      RABZ,[D0AR.0++],[D0AR.0++]
        D         LSLS      RABZ,[D0AR.0+D0ARI.0++],D0Re0
        D         LSLS      RABZ,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSLS      RABZ,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSLS      RAWX,D0Re0,D0.7
        D         LSLS      RAWX,D0Re0,[D0AR.0++]
        D         LSLS      RAWX,D0.7,D0Re0
        D         LSLS      RAWX,D0.7,[D0AR.0]
        D         LSLS      RAWX,D0.7,[D0AR.0+D0ARI.0++]
        D         ASLS      RAWX,[D0AR.0],D0Re0
        D         ASLS      RAWX,[D0AR.0],D0.7
        D         ASLS      RAWX,[D0AR.0],[D0AR.0]
        D         ASLS      RAWX,[D0AR.0],[D0AR.0++]
        D         ASLS      RAWX,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         LSLS      RAWX,[D0AR.0++],D0.7
        D         LSLS      RAWX,[D0AR.0++],[D0AR.0++]
        D         LSLS      RAWX,[D0AR.0+D0ARI.0++],D0Re0
        D         LSLS      RAWX,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         LSLS      RAWX,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         ASLS      TXIDLECYC,D0Re0,D0Re0
        D         ASLS      TXIDLECYC,D0Re0,D0.7
        D         ASLS      TXIDLECYC,D0Re0,[D0AR.0]
        D         ASLS      TXIDLECYC,D0Re0,[D0AR.0++]
        D         ASLS      TXIDLECYC,D0Re0,[D0AR.0+D0ARI.0++]
        D         ASLS      TXIDLECYC,D0.7,D0Re0
        D         ASLS      TXIDLECYC,D0.7,D0.7
        D         ASLS      TXIDLECYC,D0.7,[D0AR.0]
        D         ASLS      TXIDLECYC,D0.7,[D0AR.0++]
        D         ASLS      TXIDLECYC,D0.7,[D0AR.0+D0ARI.0++]
        D         LSRS      TXIDLECYC,[D0AR.0],D0Re0
        D         ASRS      TXIDLECYC,[D0AR.0],D0Re0
        D         LSRS      TXIDLECYC,[D0AR.0],D0.7
        D         ASRS      TXIDLECYC,[D0AR.0],D0.7
        D         LSRS      TXIDLECYC,[D0AR.0],[D0AR.0]
        D         ASRS      TXIDLECYC,[D0AR.0],[D0AR.0]
        D         LSRS      TXIDLECYC,[D0AR.0],[D0AR.0++]
        D         ASRS      TXIDLECYC,[D0AR.0],[D0AR.0++]
        D         LSRS      TXIDLECYC,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASRS      TXIDLECYC,[D0AR.0],[D0AR.0+D0ARI.0++]
        D         ASLS      TXIDLECYC,[D0AR.0++],D0Re0
        D         ASLS      TXIDLECYC,[D0AR.0++],D0.7
        D         ASLS      TXIDLECYC,[D0AR.0++],[D0AR.0]
        D         ASLS      TXIDLECYC,[D0AR.0++],[D0AR.0++]
        D         ASLS      TXIDLECYC,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         ASLS      TXIDLECYC,[D0AR.0+D0ARI.0++],D0Re0
        D         ASLS      TXIDLECYC,[D0AR.0+D0ARI.0++],D0.7
        D         ASLS      TXIDLECYC,[D0AR.0+D0ARI.0++],[D0AR.0]
        D         ASLS      TXIDLECYC,[D0AR.0+D0ARI.0++],[D0AR.0++]
        D         ASLS      TXIDLECYC,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        D         LSLS      PC,D1Re0,D1Re0
        DLNE      ASLSNE    D1Re0,D1Re0,D1Re0
        D         ASRS      TXENABLE,D1Re0,D1Re0
        D         LSLS      PC,D1Re0,D1.7
        DLNE      ASLSNE    D1Re0,D1Re0,D1.7
        D         ASRS      TXENABLE,D1Re0,D1.7
        D         LSLS      PC,D1Re0,[D1BR.1]
        DLNE      ASLSNE    D1Re0,D1Re0,[D1BR.1]
        D         ASRS      TXENABLE,D1Re0,[D1BR.1]
        D         LSLS      PC,D1Re0,[D1BR.1++]
        DLNE      ASLSNE    D1Re0,D1Re0,[D1BR.1++]
        D         ASRS      TXENABLE,D1Re0,[D1BR.1++]
        D         LSLS      PC,D1Re0,[D1BR.1+D1BRI.1++]
        DLNE      ASLSNE    D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASRS      TXENABLE,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSLS      PC,D1.7,D1Re0
        DLNE      ASLSNE    D1Re0,D1.7,D1Re0
        D         ASRS      TXENABLE,D1.7,D1Re0
        D         LSLS      PC,D1.7,D1.7
        DLNE      ASLSNE    D1Re0,D1.7,D1.7
        D         ASRS      TXENABLE,D1.7,D1.7
        D         LSLS      PC,D1.7,[D1BR.1]
        DLNE      ASLSNE    D1Re0,D1.7,[D1BR.1]
        D         ASRS      TXENABLE,D1.7,[D1BR.1]
        D         LSLS      PC,D1.7,[D1BR.1++]
        DLNE      ASLSNE    D1Re0,D1.7,[D1BR.1++]
        D         ASRS      TXENABLE,D1.7,[D1BR.1++]
        D         LSLS      PC,D1.7,[D1BR.1+D1BRI.1++]
        DLNE      ASLSNE    D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        D         ASRS      TXENABLE,D1.7,[D1BR.1+D1BRI.1++]
        D         LSLS      PC,AC1.0,D1Re0
        DHNE      ASLSPL    D1Re0,AC1.0,D1Re0
        D         ASRS      TXENABLE,AC1.0,D1Re0
        D         LSLS      PC,AC1.0,D1.7
        DHNE      ASLSPL    D1Re0,AC1.0,D1.7
        D         ASRS      TXENABLE,AC1.0,D1.7
        D         LSLS      PC,AC1.0,[D1BR.1]
        DHNE      ASLSPL    D1Re0,AC1.0,[D1BR.1]
        D         ASRS      TXENABLE,AC1.0,[D1BR.1]
        D         LSLS      PC,AC1.0,[D1BR.1++]
        DHNE      ASLSPL    D1Re0,AC1.0,[D1BR.1++]
        D         ASRS      TXENABLE,AC1.0,[D1BR.1++]
        D         LSLS      PC,AC1.0,[D1BR.1+D1BRI.1++]
        DHNE      ASLSPL    D1Re0,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASRS      TXENABLE,AC1.0,[D1BR.1+D1BRI.1++]
        D         LSLS      PC,[D1BR.1],D1Re0
        DLNE      ASLSNE    D1Re0,[D1BR.1],D1Re0
        D         ASRS      TXENABLE,[D1BR.1],D1Re0
        D         LSLS      PC,[D1BR.1],D1.7
        DLNE      ASLSNE    D1Re0,[D1BR.1],D1.7
        D         ASRS      TXENABLE,[D1BR.1],D1.7
        D         LSLS      PC,[D1BR.1],[D1BR.1]
        DLNE      ASLSNE    D1Re0,[D1BR.1],[D1BR.1]
        D         ASRS      TXENABLE,[D1BR.1],[D1BR.1]
        D         LSLS      PC,[D1BR.1],[D1BR.1++]
        DLNE      ASLSNE    D1Re0,[D1BR.1],[D1BR.1++]
        D         ASRS      TXENABLE,[D1BR.1],[D1BR.1++]
        D         LSLS      PC,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLNE      ASLSNE    D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASRS      TXENABLE,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSLS      PC,[D1BR.1++],D1Re0
        DLNE      ASLSNE    D1Re0,[D1BR.1++],D1Re0
        D         ASRS      TXENABLE,[D1BR.1++],D1Re0
        D         LSLS      PC,[D1BR.1++],D1.7
        DLNE      ASLSNE    D1Re0,[D1BR.1++],D1.7
        D         ASRS      TXENABLE,[D1BR.1++],D1.7
        D         LSLS      PC,[D1BR.1++],[D1BR.1]
        DLNE      ASLSNE    D1Re0,[D1BR.1++],[D1BR.1]
        D         ASRS      TXENABLE,[D1BR.1++],[D1BR.1]
        D         LSLS      PC,[D1BR.1++],[D1BR.1++]
        DLNE      ASLSNE    D1Re0,[D1BR.1++],[D1BR.1++]
        D         ASRS      TXENABLE,[D1BR.1++],[D1BR.1++]
        D         LSLS      PC,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLNE      ASLSNE    D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASRS      TXENABLE,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSLS      PC,[D1BR.1+D1BRI.1++],D1Re0
        DLNE      ASLSNE    D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         ASRS      TXENABLE,[D1BR.1+D1BRI.1++],D1Re0
        D         LSLS      PC,[D1BR.1+D1BRI.1++],D1.7
        DLNE      ASLSNE    D1Re0,[D1BR.1+D1BRI.1++],D1.7
        D         ASRS      TXENABLE,[D1BR.1+D1BRI.1++],D1.7
        D         LSLS      PC,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLNE      ASLSNE    D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASRS      TXENABLE,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSLS      PC,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLNE      ASLSNE    D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASRS      TXENABLE,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSLS      PC,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLNE      ASLSNE    D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASRS      TXENABLE,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSRS      A1LbP,D1Re0,D1Re0
        D         LSLS      A0FrP,D1Re0,D1.7
        D         ASLS      PCX,D1Re0,D1.7
        D         LSRS      A1LbP,D1Re0,[D1BR.1]
        D         LSLS      A0FrP,D1Re0,[D1BR.1++]
        D         ASLS      PCX,D1Re0,[D1BR.1++]
        D         LSRS      A1LbP,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSLS      A0FrP,D1.7,D1Re0
        D         ASLS      PCX,D1.7,D1Re0
        D         LSRS      A1LbP,D1.7,D1.7
        D         LSLS      A0FrP,D1.7,[D1BR.1]
        D         ASLS      PCX,D1.7,[D1BR.1]
        D         LSRS      A1LbP,D1.7,[D1BR.1++]
        D         LSLS      A0FrP,D1.7,[D1BR.1+D1BRI.1++]
        D         ASLS      PCX,D1.7,[D1BR.1+D1BRI.1++]
        D         LSRS      A1LbP,AC1.0,D1Re0
        D         LSLS      A0FrP,AC1.0,D1.7
        D         ASLS      PCX,AC1.0,D1.7
        D         LSRS      A1LbP,AC1.0,[D1BR.1]
        D         LSLS      A0FrP,AC1.0,[D1BR.1++]
        D         ASLS      PCX,AC1.0,[D1BR.1++]
        D         LSRS      A1LbP,AC1.0,[D1BR.1+D1BRI.1++]
        D         LSLS      A0FrP,[D1BR.1],D1Re0
        D         ASLS      PCX,[D1BR.1],D1Re0
        D         LSRS      A1LbP,[D1BR.1],D1.7
        D         LSLS      A0FrP,[D1BR.1],[D1BR.1]
        D         ASLS      PCX,[D1BR.1],[D1BR.1]
        D         LSRS      A1LbP,[D1BR.1],[D1BR.1++]
        D         LSLS      A0FrP,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASLS      PCX,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSRS      A1LbP,[D1BR.1++],D1Re0
        D         LSLS      A0FrP,[D1BR.1++],D1.7
        D         ASLS      PCX,[D1BR.1++],D1.7
        D         LSRS      A1LbP,[D1BR.1++],[D1BR.1]
        D         LSLS      A0FrP,[D1BR.1++],[D1BR.1++]
        D         ASLS      PCX,[D1BR.1++],[D1BR.1++]
        D         LSRS      A1LbP,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSLS      A0FrP,[D1BR.1+D1BRI.1++],D1Re0
        D         ASLS      PCX,[D1BR.1+D1BRI.1++],D1Re0
        D         LSRS      A1LbP,[D1BR.1+D1BRI.1++],D1.7
        D         LSLS      A0FrP,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASLS      PCX,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSRS      A1LbP,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSLS      A0FrP,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASLS      PCX,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSLS      TXMASKI,D1Re0,D1.7
        D         LSLS      TXMASKI,D1Re0,[D1BR.1++]
        D         LSLS      TXMASKI,D1.7,D1Re0
        D         LSLS      TXMASKI,D1.7,[D1BR.1]
        D         LSLS      TXMASKI,D1.7,[D1BR.1+D1BRI.1++]
        D         LSLS      TXMASKI,AC1.0,D1.7
        D         LSLS      TXMASKI,AC1.0,[D1BR.1++]
        D         LSLS      TXMASKI,[D1BR.1],D1Re0
        D         LSLS      TXMASKI,[D1BR.1],[D1BR.1]
        D         LSLS      TXMASKI,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSLS      TXMASKI,[D1BR.1++],D1.7
        D         LSLS      TXMASKI,[D1BR.1++],[D1BR.1++]
        D         LSLS      TXMASKI,[D1BR.1+D1BRI.1++],D1Re0
        D         LSLS      TXMASKI,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSLS      TXMASKI,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSLS      A1.7,D1Re0,D1Re0
        DLNE      ASLSNE    D1.7,D1Re0,D1Re0
        D         ASRS      D0.7,D1Re0,D1Re0
        D         LSLS      A1.7,D1Re0,D1.7
        DLNE      ASLSNE    D1.7,D1Re0,D1.7
        D         ASRS      D0.7,D1Re0,D1.7
        D         LSLS      A1.7,D1Re0,[D1BR.1]
        DLNE      ASLSNE    D1.7,D1Re0,[D1BR.1]
        D         ASRS      D0.7,D1Re0,[D1BR.1]
        D         LSLS      A1.7,D1Re0,[D1BR.1++]
        DLNE      ASLSNE    D1.7,D1Re0,[D1BR.1++]
        D         ASRS      D0.7,D1Re0,[D1BR.1++]
        D         LSLS      A1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DLNE      ASLSNE    D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASRS      D0.7,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSLS      A1.7,D1.7,D1Re0
        DLNE      ASLSNE    D1.7,D1.7,D1Re0
        D         ASRS      D0.7,D1.7,D1Re0
        D         LSLS      A1.7,D1.7,D1.7
        DLNE      ASLSNE    D1.7,D1.7,D1.7
        D         ASRS      D0.7,D1.7,D1.7
        D         LSLS      A1.7,D1.7,[D1BR.1]
        DLNE      ASLSNE    D1.7,D1.7,[D1BR.1]
        D         ASRS      D0.7,D1.7,[D1BR.1]
        D         LSLS      A1.7,D1.7,[D1BR.1++]
        DLNE      ASLSNE    D1.7,D1.7,[D1BR.1++]
        D         ASRS      D0.7,D1.7,[D1BR.1++]
        D         LSLS      A1.7,D1.7,[D1BR.1+D1BRI.1++]
        DLNE      ASLSNE    D1.7,D1.7,[D1BR.1+D1BRI.1++]
        D         ASRS      D0.7,D1.7,[D1BR.1+D1BRI.1++]
        D         LSLS      A1.7,AC1.0,D1Re0
        DHNE      ASLSPL    D1.7,AC1.0,D1Re0
        D         ASRS      D0.7,AC1.0,D1Re0
        D         LSLS      A1.7,AC1.0,D1.7
        DHNE      ASLSPL    D1.7,AC1.0,D1.7
        D         ASRS      D0.7,AC1.0,D1.7
        D         LSLS      A1.7,AC1.0,[D1BR.1]
        DHNE      ASLSPL    D1.7,AC1.0,[D1BR.1]
        D         ASRS      D0.7,AC1.0,[D1BR.1]
        D         LSLS      A1.7,AC1.0,[D1BR.1++]
        DHNE      ASLSPL    D1.7,AC1.0,[D1BR.1++]
        D         ASRS      D0.7,AC1.0,[D1BR.1++]
        D         LSLS      A1.7,AC1.0,[D1BR.1+D1BRI.1++]
        DHNE      ASLSPL    D1.7,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASRS      D0.7,AC1.0,[D1BR.1+D1BRI.1++]
        D         LSLS      A1.7,[D1BR.1],D1Re0
        DLNE      ASLSNE    D1.7,[D1BR.1],D1Re0
        D         ASRS      D0.7,[D1BR.1],D1Re0
        D         LSLS      A1.7,[D1BR.1],D1.7
        DLNE      ASLSNE    D1.7,[D1BR.1],D1.7
        D         ASRS      D0.7,[D1BR.1],D1.7
        D         LSLS      A1.7,[D1BR.1],[D1BR.1]
        DLNE      ASLSNE    D1.7,[D1BR.1],[D1BR.1]
        D         ASRS      D0.7,[D1BR.1],[D1BR.1]
        D         LSLS      A1.7,[D1BR.1],[D1BR.1++]
        DLNE      ASLSNE    D1.7,[D1BR.1],[D1BR.1++]
        D         ASRS      D0.7,[D1BR.1],[D1BR.1++]
        D         LSLS      A1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLNE      ASLSNE    D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASRS      D0.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSLS      A1.7,[D1BR.1++],D1Re0
        DLNE      ASLSNE    D1.7,[D1BR.1++],D1Re0
        D         ASRS      D0.7,[D1BR.1++],D1Re0
        D         LSLS      A1.7,[D1BR.1++],D1.7
        DLNE      ASLSNE    D1.7,[D1BR.1++],D1.7
        D         ASRS      D0.7,[D1BR.1++],D1.7
        D         LSLS      A1.7,[D1BR.1++],[D1BR.1]
        DLNE      ASLSNE    D1.7,[D1BR.1++],[D1BR.1]
        D         ASRS      D0.7,[D1BR.1++],[D1BR.1]
        D         LSLS      A1.7,[D1BR.1++],[D1BR.1++]
        DLNE      ASLSNE    D1.7,[D1BR.1++],[D1BR.1++]
        D         ASRS      D0.7,[D1BR.1++],[D1BR.1++]
        D         LSLS      A1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLNE      ASLSNE    D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASRS      D0.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         LSLS      A1.7,[D1BR.1+D1BRI.1++],D1Re0
        DLNE      ASLSNE    D1.7,[D1BR.1+D1BRI.1++],D1Re0
        D         ASRS      D0.7,[D1BR.1+D1BRI.1++],D1Re0
        D         LSLS      A1.7,[D1BR.1+D1BRI.1++],D1.7
        DLNE      ASLSNE    D1.7,[D1BR.1+D1BRI.1++],D1.7
        D         ASRS      D0.7,[D1BR.1+D1BRI.1++],D1.7
        D         LSLS      A1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        DLNE      ASLSNE    D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASRS      D0.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSLS      A1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLNE      ASLSNE    D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASRS      D0.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         LSLS      A1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLNE      ASLSNE    D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASRS      D0.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASLS      A0.8,D1Re0,D1Re0
        D         ASLS      A0.8,D1Re0,D1.7
        D         ASLS      A0.8,D1Re0,[D1BR.1]
        D         ASLS      A0.8,D1Re0,[D1BR.1++]
        D         ASLS      A0.8,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASLS      A0.8,D1.7,D1Re0
        D         ASLS      A0.8,D1.7,D1.7
        D         ASLS      A0.8,D1.7,[D1BR.1]
        D         ASLS      A0.8,D1.7,[D1BR.1++]
        D         ASLS      A0.8,D1.7,[D1BR.1+D1BRI.1++]
        D         ASLS      A0.8,AC1.0,D1Re0
        D         ASLS      A0.8,AC1.0,D1.7
        D         ASLS      A0.8,AC1.0,[D1BR.1]
        D         ASLS      A0.8,AC1.0,[D1BR.1++]
        D         ASLS      A0.8,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASLS      A0.8,[D1BR.1],D1Re0
        D         ASLS      A0.8,[D1BR.1],D1.7
        D         ASLS      A0.8,[D1BR.1],[D1BR.1]
        D         ASLS      A0.8,[D1BR.1],[D1BR.1++]
        D         ASLS      A0.8,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASLS      A0.8,[D1BR.1++],D1Re0
        D         ASLS      A0.8,[D1BR.1++],D1.7
        D         ASLS      A0.8,[D1BR.1++],[D1BR.1]
        D         ASLS      A0.8,[D1BR.1++],[D1BR.1++]
        D         ASLS      A0.8,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASLS      A0.8,[D1BR.1+D1BRI.1++],D1Re0
        D         ASLS      A0.8,[D1BR.1+D1BRI.1++],D1.7
        D         ASLS      A0.8,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASLS      A0.8,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASLS      A0.8,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASLS      D0.16,D1Re0,D1Re0
        D         ASLS      RA,D1Re0,D1.7
        D         ASRS      RA,D1Re0,[D1BR.1]
        D         LSLS      RA,D1Re0,[D1BR.1+D1BRI.1++]
        D         LSRS      RA,D1.7,D1Re0
        DLLS      ASLSLS    D1.16,D1.7,D1.7
        D         ASLSNV    D1.16,D1.7,[D1BR.1]
        D         ASLS      D0.16,D1.7,[D1BR.1++]
        D         ASLS      RA,D1.7,[D1BR.1+D1BRI.1++]
        D         LSRS      D0.16,AC1.0,D1Re0
        D         ASRS      D0.16,AC1.0,D1Re0
        D         LSRS      D0.16,AC1.0,D1.7
        D         ASRS      D0.16,AC1.0,D1.7
        D         LSRS      D0.16,AC1.0,[D1BR.1]
        D         ASRS      D0.16,AC1.0,[D1BR.1]
        D         LSRS      D0.16,AC1.0,[D1BR.1++]
        D         ASRS      D0.16,AC1.0,[D1BR.1++]
        D         LSRS      D0.16,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASRS      D0.16,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASRS      RA,[D1BR.1],D1Re0
        D         LSLS      RA,[D1BR.1],[D1BR.1]
        D         LSRS      RA,[D1BR.1],[D1BR.1++]
        DECS      ASLSLE    D1.16,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASLSNV    D1.16,[D1BR.1++],D1Re0
        D         ASLS      D0.16,[D1BR.1++],D1.7
        D         ASLS      RA,[D1BR.1++],[D1BR.1]
        D         ASRS      RA,[D1BR.1++],[D1BR.1++]
        D         LSLS      RA,[D1BR.1+D1BRI.1++],D1Re0
        D         LSRS      RA,[D1BR.1+D1BRI.1++],D1.7
        DLLS      ASLSLS    D1.16,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASLSNV    D1.16,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASLS      D0.16,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSRS      RAM8X32,D1Re0,D1.7
        D         LSRS      RAM8X32,D1Re0,[D1BR.1++]
        D         LSRS      RAM8X32,D1.7,D1Re0
        D         LSRS      RAM8X32,D1.7,[D1BR.1]
        D         LSRS      RAM8X32,D1.7,[D1BR.1+D1BRI.1++]
        D         LSRS      RAM8X32,AC1.0,D1.7
        D         LSRS      RAM8X32,AC1.0,[D1BR.1++]
        D         LSRS      RAM8X32,[D1BR.1],D1Re0
        D         LSRS      RAM8X32,[D1BR.1],[D1BR.1]
        D         LSRS      RAM8X32,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSRS      RAM8X32,[D1BR.1++],D1.7
        D         LSRS      RAM8X32,[D1BR.1++],[D1BR.1++]
        D         LSRS      RAM8X32,[D1BR.1+D1BRI.1++],D1Re0
        D         LSRS      RAM8X32,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSRS      RAM8X32,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSRS      RAM8X,D1Re0,D1.7
        D         LSRS      RAM8X,D1Re0,[D1BR.1++]
        D         LSRS      RAM8X,D1.7,D1Re0
        D         LSRS      RAM8X,D1.7,[D1BR.1]
        D         LSRS      RAM8X,D1.7,[D1BR.1+D1BRI.1++]
        D         LSRS      RAM8X,AC1.0,D1.7
        D         LSRS      RAM8X,AC1.0,[D1BR.1++]
        D         LSRS      RAM8X,[D1BR.1],D1Re0
        D         LSRS      RAM8X,[D1BR.1],[D1BR.1]
        D         LSRS      RAM8X,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSRS      RAM8X,[D1BR.1++],D1.7
        D         LSRS      RAM8X,[D1BR.1++],[D1BR.1++]
        D         LSRS      RAM8X,[D1BR.1+D1BRI.1++],D1Re0
        D         LSRS      RAM8X,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSRS      RAM8X,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSRS      RABZ,D1Re0,D1.7
        D         LSRS      RABZ,D1Re0,[D1BR.1++]
        D         LSRS      RABZ,D1.7,D1Re0
        D         LSRS      RABZ,D1.7,[D1BR.1]
        D         LSRS      RABZ,D1.7,[D1BR.1+D1BRI.1++]
        D         LSRS      RABZ,AC1.0,D1.7
        D         LSRS      RABZ,AC1.0,[D1BR.1++]
        D         LSRS      RABZ,[D1BR.1],D1Re0
        D         LSRS      RABZ,[D1BR.1],[D1BR.1]
        D         LSRS      RABZ,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSRS      RABZ,[D1BR.1++],D1.7
        D         LSRS      RABZ,[D1BR.1++],[D1BR.1++]
        D         LSRS      RABZ,[D1BR.1+D1BRI.1++],D1Re0
        D         LSRS      RABZ,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSRS      RABZ,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSRS      RAWX,D1Re0,D1.7
        D         LSRS      RAWX,D1Re0,[D1BR.1++]
        D         LSRS      RAWX,D1.7,D1Re0
        D         LSRS      RAWX,D1.7,[D1BR.1]
        D         LSRS      RAWX,D1.7,[D1BR.1+D1BRI.1++]
        D         LSRS      RAWX,AC1.0,D1.7
        D         LSRS      RAWX,AC1.0,[D1BR.1++]
        D         LSRS      RAWX,[D1BR.1],D1Re0
        D         LSRS      RAWX,[D1BR.1],[D1BR.1]
        D         LSRS      RAWX,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         LSRS      RAWX,[D1BR.1++],D1.7
        D         LSRS      RAWX,[D1BR.1++],[D1BR.1++]
        D         LSRS      RAWX,[D1BR.1+D1BRI.1++],D1Re0
        D         LSRS      RAWX,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         LSRS      RAWX,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         ASLS      RAMX,D1Re0,D1Re0
        D         ASLS      RAMX,D1Re0,D1.7
        D         ASLS      RAMX,D1Re0,[D1BR.1]
        D         ASLS      RAMX,D1Re0,[D1BR.1++]
        D         ASLS      RAMX,D1Re0,[D1BR.1+D1BRI.1++]
        D         ASLS      RAMX,D1.7,D1Re0
        D         ASLS      RAMX,D1.7,D1.7
        D         ASLS      RAMX,D1.7,[D1BR.1]
        D         ASLS      RAMX,D1.7,[D1BR.1++]
        D         ASLS      RAMX,D1.7,[D1BR.1+D1BRI.1++]
        D         ASLS      RAMX,AC1.0,D1Re0
        D         ASLS      RAMX,AC1.0,D1.7
        D         ASLS      RAMX,AC1.0,[D1BR.1]
        D         ASLS      RAMX,AC1.0,[D1BR.1++]
        D         ASLS      RAMX,AC1.0,[D1BR.1+D1BRI.1++]
        D         ASLS      RAMX,[D1BR.1],D1Re0
        D         ASLS      RAMX,[D1BR.1],D1.7
        D         ASLS      RAMX,[D1BR.1],[D1BR.1]
        D         ASLS      RAMX,[D1BR.1],[D1BR.1++]
        D         ASLS      RAMX,[D1BR.1],[D1BR.1+D1BRI.1++]
        D         ASLS      RAMX,[D1BR.1++],D1Re0
        D         ASLS      RAMX,[D1BR.1++],D1.7
        D         ASLS      RAMX,[D1BR.1++],[D1BR.1]
        D         ASLS      RAMX,[D1BR.1++],[D1BR.1++]
        D         ASLS      RAMX,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ASLS      RAMX,[D1BR.1+D1BRI.1++],D1Re0
        D         ASLS      RAMX,[D1BR.1+D1BRI.1++],D1.7
        D         ASLS      RAMX,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         ASLS      RAMX,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         ASLS      RAMX,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         LSLS      TXSTAT,D0Re0,#0xf
        D         ASLSNV    D0Re0,D0Re0,#0xf
        D         ASRS      D1Re0,D0Re0,#0xf
        D         LSLS      TXSTAT,D0Re0,#0x10
        D         ASLSNV    D0Re0,D0Re0,#0x10
        D         ASRS      D1Re0,D0Re0,#0x10
        D         LSLS      TXSTAT,D0Re0,#0x1f
        D         ASLSNV    D0Re0,D0Re0,#0x1f
        D         ASRS      D1Re0,D0Re0,#0x1f
        D         LSLS      TXSTAT,D0.7,#0xf
        D         ASLSNV    D0Re0,D0.7,#0xf
        D         ASRS      D1Re0,D0.7,#0xf
        D         LSLS      TXSTAT,D0.7,#0x10
        D         ASLSNV    D0Re0,D0.7,#0x10
        D         ASRS      D1Re0,D0.7,#0x10
        D         LSLS      TXSTAT,D0.7,#0x1f
        D         ASLSNV    D0Re0,D0.7,#0x1f
        D         ASRS      D1Re0,D0.7,#0x1f
        D         LSLS      TXENABLE,AC0.0,#0xf
        DECS      LSRSLE    D0Re0,AC0.0,#0xf
        D         LSRS      PC,AC0.0,#0xf
        D         ASLS      TXENABLE,AC0.0,#0xf
        DECS      ASRSLE    D0Re0,AC0.0,#0xf
        D         ASRS      PC,AC0.0,#0xf
        D         LSLS      TXENABLE,AC0.0,#0x10
        DECS      LSRSLE    D0Re0,AC0.0,#0x10
        D         LSRS      PC,AC0.0,#0x10
        D         ASLS      TXENABLE,AC0.0,#0x10
        DECS      ASRSLE    D0Re0,AC0.0,#0x10
        D         ASRS      PC,AC0.0,#0x10
        D         LSLS      TXENABLE,AC0.0,#0x1f
        DECS      LSRSLE    D0Re0,AC0.0,#0x1f
        D         LSRS      PC,AC0.0,#0x1f
        D         ASLS      TXENABLE,AC0.0,#0x1f
        DECS      ASRSLE    D0Re0,AC0.0,#0x1f
        D         ASRS      PC,AC0.0,#0x1f
        D         LSLS      TXSTAT,[D0AR.0++],#0xf
        D         ASLSNV    D0Re0,[D0AR.0++],#0xf
        D         ASRS      D1Re0,[D0AR.0++],#0xf
        D         LSLS      TXSTAT,[D0AR.0++],#0x10
        D         ASLSNV    D0Re0,[D0AR.0++],#0x10
        D         ASRS      D1Re0,[D0AR.0++],#0x10
        D         LSLS      TXSTAT,[D0AR.0++],#0x1f
        D         ASLSNV    D0Re0,[D0AR.0++],#0x1f
        D         ASRS      D1Re0,[D0AR.0++],#0x1f
        D         LSLS      TXSTAT,[D0AR.0+D0ARI.0++],#0xf
        D         ASLSNV    D0Re0,[D0AR.0+D0ARI.0++],#0xf
        D         ASRS      D1Re0,[D0AR.0+D0ARI.0++],#0xf
        D         LSLS      TXSTAT,[D0AR.0+D0ARI.0++],#0x10
        D         ASLSNV    D0Re0,[D0AR.0+D0ARI.0++],#0x10
        D         ASRS      D1Re0,[D0AR.0+D0ARI.0++],#0x10
        D         LSLS      TXSTAT,[D0AR.0+D0ARI.0++],#0x1f
        D         ASLSNV    D0Re0,[D0AR.0+D0ARI.0++],#0x1f
        D         ASRS      D1Re0,[D0AR.0+D0ARI.0++],#0x1f
        D         LSRS      PCX,D0Re0,#0xf
        D         LSLS      A1LbP,D0Re0,#0x10
        D         ASRS      A0FrP,D0Re0,#0x10
        D         LSRS      PCX,D0Re0,#0x1f
        D         LSLS      A1LbP,D0.7,#0xf
        D         ASRS      A0FrP,D0.7,#0xf
        D         LSRS      PCX,D0.7,#0x10
        D         LSLS      A1LbP,D0.7,#0x1f
        D         ASRS      A0FrP,D0.7,#0x1f
        D         LSLS      PCX,AC0.0,#0xf
        D         ASLS      A0FrP,AC0.0,#0xf
        D         ASRS      A1LbP,AC0.0,#0xf
        D         LSLS      PCX,AC0.0,#0x10
        D         ASLS      A0FrP,AC0.0,#0x10
        D         ASRS      A1LbP,AC0.0,#0x10
        D         LSLS      PCX,AC0.0,#0x1f
        D         ASLS      A0FrP,AC0.0,#0x1f
        D         ASRS      A1LbP,AC0.0,#0x1f
        D         LSRS      PCX,[D0AR.0++],#0xf
        D         LSLS      A1LbP,[D0AR.0++],#0x10
        D         ASRS      A0FrP,[D0AR.0++],#0x10
        D         LSRS      PCX,[D0AR.0++],#0x1f
        D         LSLS      A1LbP,[D0AR.0+D0ARI.0++],#0xf
        D         ASRS      A0FrP,[D0AR.0+D0ARI.0++],#0xf
        D         LSRS      PCX,[D0AR.0+D0ARI.0++],#0x10
        D         LSLS      A1LbP,[D0AR.0+D0ARI.0++],#0x1f
        D         ASRS      A0FrP,[D0AR.0+D0ARI.0++],#0x1f
        D         LSRS      TXMASKI,D0Re0,#0x10
        D         LSRS      TXMASKI,D0.7,#0xf
        D         LSRS      TXMASKI,D0.7,#0x1f
        D         ASLS      TXMASKI,AC0.0,#0xf
        D         ASLS      TXMASKI,AC0.0,#0x10
        D         ASLS      TXMASKI,AC0.0,#0x1f
        D         LSRS      TXMASKI,[D0AR.0++],#0x10
        D         LSRS      TXMASKI,[D0AR.0+D0ARI.0++],#0xf
        D         LSRS      TXMASKI,[D0AR.0+D0ARI.0++],#0x1f
        D         LSLS      TXGPIOO,D0Re0,#0xf
        D         ASLSNV    D0.7,D0Re0,#0xf
        D         ASRS      A0.7,D0Re0,#0xf
        D         LSLS      TXGPIOO,D0Re0,#0x10
        D         ASLSNV    D0.7,D0Re0,#0x10
        D         ASRS      A0.7,D0Re0,#0x10
        D         LSLS      TXGPIOO,D0Re0,#0x1f
        D         ASLSNV    D0.7,D0Re0,#0x1f
        D         ASRS      A0.7,D0Re0,#0x1f
        D         LSLS      TXGPIOO,D0.7,#0xf
        D         ASLSNV    D0.7,D0.7,#0xf
        D         ASRS      A0.7,D0.7,#0xf
        D         LSLS      TXGPIOO,D0.7,#0x10
        D         ASLSNV    D0.7,D0.7,#0x10
        D         ASRS      A0.7,D0.7,#0x10
        D         LSLS      TXGPIOO,D0.7,#0x1f
        D         ASLSNV    D0.7,D0.7,#0x1f
        D         ASRS      A0.7,D0.7,#0x1f
        D         LSLS      D1.7,AC0.0,#0xf
        DECS      LSRSLE    D0.7,AC0.0,#0xf
        D         LSRS      A1.7,AC0.0,#0xf
        D         ASLS      D1.7,AC0.0,#0xf
        DECS      ASRSLE    D0.7,AC0.0,#0xf
        D         ASRS      A1.7,AC0.0,#0xf
        D         LSLS      D1.7,AC0.0,#0x10
        DECS      LSRSLE    D0.7,AC0.0,#0x10
        D         LSRS      A1.7,AC0.0,#0x10
        D         ASLS      D1.7,AC0.0,#0x10
        DECS      ASRSLE    D0.7,AC0.0,#0x10
        D         ASRS      A1.7,AC0.0,#0x10
        D         LSLS      D1.7,AC0.0,#0x1f
        DECS      LSRSLE    D0.7,AC0.0,#0x1f
        D         LSRS      A1.7,AC0.0,#0x1f
        D         ASLS      D1.7,AC0.0,#0x1f
        DECS      ASRSLE    D0.7,AC0.0,#0x1f
        D         ASRS      A1.7,AC0.0,#0x1f
        D         LSLS      TXGPIOO,[D0AR.0++],#0xf
        D         ASLSNV    D0.7,[D0AR.0++],#0xf
        D         ASRS      A0.7,[D0AR.0++],#0xf
        D         LSLS      TXGPIOO,[D0AR.0++],#0x10
        D         ASLSNV    D0.7,[D0AR.0++],#0x10
        D         ASRS      A0.7,[D0AR.0++],#0x10
        D         LSLS      TXGPIOO,[D0AR.0++],#0x1f
        D         ASLSNV    D0.7,[D0AR.0++],#0x1f
        D         ASRS      A0.7,[D0AR.0++],#0x1f
        D         LSLS      TXGPIOO,[D0AR.0+D0ARI.0++],#0xf
        D         ASLSNV    D0.7,[D0AR.0+D0ARI.0++],#0xf
        D         ASRS      A0.7,[D0AR.0+D0ARI.0++],#0xf
        D         LSLS      TXGPIOO,[D0AR.0+D0ARI.0++],#0x10
        D         ASLSNV    D0.7,[D0AR.0+D0ARI.0++],#0x10
        D         ASRS      A0.7,[D0AR.0+D0ARI.0++],#0x10
        D         LSLS      TXGPIOO,[D0AR.0+D0ARI.0++],#0x1f
        D         ASLSNV    D0.7,[D0AR.0+D0ARI.0++],#0x1f
        D         ASRS      A0.7,[D0AR.0+D0ARI.0++],#0x1f
        D         ASLS      A1.8,D0Re0,#0xf
        D         ASLS      A1.8,D0Re0,#0x10
        D         ASLS      A1.8,D0Re0,#0x1f
        D         ASLS      A1.8,D0.7,#0xf
        D         ASLS      A1.8,D0.7,#0x10
        D         ASLS      A1.8,D0.7,#0x1f
        D         LSRS      A0.8,AC0.0,#0xf
        D         ASRS      A0.8,AC0.0,#0xf
        D         LSRS      A0.8,AC0.0,#0x10
        D         ASRS      A0.8,AC0.0,#0x10
        D         LSRS      A0.8,AC0.0,#0x1f
        D         ASRS      A0.8,AC0.0,#0x1f
        D         ASLS      A1.8,[D0AR.0++],#0xf
        D         ASLS      A1.8,[D0AR.0++],#0x10
        D         ASLS      A1.8,[D0AR.0++],#0x1f
        D         ASLS      A1.8,[D0AR.0+D0ARI.0++],#0xf
        D         ASLS      A1.8,[D0AR.0+D0ARI.0++],#0x10
        D         ASLS      A1.8,[D0AR.0+D0ARI.0++],#0x1f
        D         LSRSNV    D0.16,D0Re0,#0xf
        D         ASRSNV    D0.16,D0Re0,#0xf
        D         LSRSNV    D0.16,D0Re0,#0x10
        D         ASRSNV    D0.16,D0Re0,#0x10
        D         LSRSNV    D0.16,D0Re0,#0x1f
        D         ASRSNV    D0.16,D0Re0,#0x1f
        D         LSRSNV    D0.16,D0.7,#0xf
        D         ASRSNV    D0.16,D0.7,#0xf
        D         LSRSNV    D0.16,D0.7,#0x10
        D         ASRSNV    D0.16,D0.7,#0x10
        D         LSRSNV    D0.16,D0.7,#0x1f
        D         ASRSNV    D0.16,D0.7,#0x1f
        D         LSLS      D1.16,AC0.0,#0xf
        D         LSRS      D1.16,AC0.0,#0xf
        D         ASLS      D1.16,AC0.0,#0xf
        D         ASRS      D1.16,AC0.0,#0xf
        D         LSLS      D1.16,AC0.0,#0x10
        D         LSRS      D1.16,AC0.0,#0x10
        D         ASLS      D1.16,AC0.0,#0x10
        D         ASRS      D1.16,AC0.0,#0x10
        D         LSLS      D1.16,AC0.0,#0x1f
        D         LSRS      D1.16,AC0.0,#0x1f
        D         ASLS      D1.16,AC0.0,#0x1f
        D         ASRS      D1.16,AC0.0,#0x1f
        D         LSRSNV    D0.16,[D0AR.0++],#0xf
        D         ASRSNV    D0.16,[D0AR.0++],#0xf
        D         LSRSNV    D0.16,[D0AR.0++],#0x10
        D         ASRSNV    D0.16,[D0AR.0++],#0x10
        D         LSRSNV    D0.16,[D0AR.0++],#0x1f
        D         ASRSNV    D0.16,[D0AR.0++],#0x1f
        D         LSRSNV    D0.16,[D0AR.0+D0ARI.0++],#0xf
        D         ASRSNV    D0.16,[D0AR.0+D0ARI.0++],#0xf
        D         LSRSNV    D0.16,[D0AR.0+D0ARI.0++],#0x10
        D         ASRSNV    D0.16,[D0AR.0+D0ARI.0++],#0x10
        D         LSRSNV    D0.16,[D0AR.0+D0ARI.0++],#0x1f
        D         ASRSNV    D0.16,[D0AR.0+D0ARI.0++],#0x1f
        D         LSRS      RAM8X32,D0Re0,#0x10
        D         LSRS      RAM8X32,D0.7,#0xf
        D         LSRS      RAM8X32,D0.7,#0x1f
        D         ASLS      RAM8X32,AC0.0,#0xf
        D         ASLS      RAM8X32,AC0.0,#0x10
        D         ASLS      RAM8X32,AC0.0,#0x1f
        D         LSRS      RAM8X32,[D0AR.0++],#0x10
        D         LSRS      RAM8X32,[D0AR.0+D0ARI.0++],#0xf
        D         LSRS      RAM8X32,[D0AR.0+D0ARI.0++],#0x1f
        D         LSRS      RAM8X,D0Re0,#0x10
        D         LSRS      RAM8X,D0.7,#0xf
        D         LSRS      RAM8X,D0.7,#0x1f
        D         ASLS      RAM8X,AC0.0,#0xf
        D         ASLS      RAM8X,AC0.0,#0x10
        D         ASLS      RAM8X,AC0.0,#0x1f
        D         LSRS      RAM8X,[D0AR.0++],#0x10
        D         LSRS      RAM8X,[D0AR.0+D0ARI.0++],#0xf
        D         LSRS      RAM8X,[D0AR.0+D0ARI.0++],#0x1f
        D         LSRS      RABZ,D0Re0,#0x10
        D         LSRS      RABZ,D0.7,#0xf
        D         LSRS      RABZ,D0.7,#0x1f
        D         ASLS      RABZ,AC0.0,#0xf
        D         ASLS      RABZ,AC0.0,#0x10
        D         ASLS      RABZ,AC0.0,#0x1f
        D         LSRS      RABZ,[D0AR.0++],#0x10
        D         LSRS      RABZ,[D0AR.0+D0ARI.0++],#0xf
        D         LSRS      RABZ,[D0AR.0+D0ARI.0++],#0x1f
        D         LSRS      RAWX,D0Re0,#0x10
        D         LSRS      RAWX,D0.7,#0xf
        D         LSRS      RAWX,D0.7,#0x1f
        D         ASLS      RAWX,AC0.0,#0xf
        D         ASLS      RAWX,AC0.0,#0x10
        D         ASLS      RAWX,AC0.0,#0x1f
        D         LSRS      RAWX,[D0AR.0++],#0x10
        D         LSRS      RAWX,[D0AR.0+D0ARI.0++],#0xf
        D         LSRS      RAWX,[D0AR.0+D0ARI.0++],#0x1f
        D         ASLS      RAMX,D0Re0,#0xf
        D         ASLS      RAMX,D0Re0,#0x10
        D         ASLS      RAMX,D0Re0,#0x1f
        D         ASLS      RAMX,D0.7,#0xf
        D         ASLS      RAMX,D0.7,#0x10
        D         ASLS      RAMX,D0.7,#0x1f
        D         LSRS      TXIDLECYC,AC0.0,#0xf
        D         ASRS      TXIDLECYC,AC0.0,#0xf
        D         LSRS      TXIDLECYC,AC0.0,#0x10
        D         ASRS      TXIDLECYC,AC0.0,#0x10
        D         LSRS      TXIDLECYC,AC0.0,#0x1f
        D         ASRS      TXIDLECYC,AC0.0,#0x1f
        D         ASLS      RAMX,[D0AR.0++],#0xf
        D         ASLS      RAMX,[D0AR.0++],#0x10
        D         ASLS      RAMX,[D0AR.0++],#0x1f
        D         ASLS      RAMX,[D0AR.0+D0ARI.0++],#0xf
        D         ASLS      RAMX,[D0AR.0+D0ARI.0++],#0x10
        D         ASLS      RAMX,[D0AR.0+D0ARI.0++],#0x1f
        D         LSLS      TXSTAT,D1Re0,#0xf
        D         ASLSNV    D1Re0,D1Re0,#0xf
        D         ASRS      D0Re0,D1Re0,#0xf
        D         LSLS      TXSTAT,D1Re0,#0x10
        D         ASLSNV    D1Re0,D1Re0,#0x10
        D         ASRS      D0Re0,D1Re0,#0x10
        D         LSLS      TXSTAT,D1Re0,#0x1f
        D         ASLSNV    D1Re0,D1Re0,#0x1f
        D         ASRS      D0Re0,D1Re0,#0x1f
        D         LSLS      TXSTAT,D1.7,#0xf
        D         ASLSNV    D1Re0,D1.7,#0xf
        D         ASRS      D0Re0,D1.7,#0xf
        D         LSLS      TXSTAT,D1.7,#0x10
        D         ASLSNV    D1Re0,D1.7,#0x10
        D         ASRS      D0Re0,D1.7,#0x10
        D         LSLS      TXSTAT,D1.7,#0x1f
        D         ASLSNV    D1Re0,D1.7,#0x1f
        D         ASRS      D0Re0,D1.7,#0x1f
        D         LSLS      TXSTAT,AC1.0,#0xf
        D         ASLSNV    D1Re0,AC1.0,#0xf
        D         ASRS      D0Re0,AC1.0,#0xf
        D         LSLS      TXSTAT,AC1.0,#0x10
        D         ASLSNV    D1Re0,AC1.0,#0x10
        D         ASRS      D0Re0,AC1.0,#0x10
        D         LSLS      TXSTAT,AC1.0,#0x1f
        D         ASLSNV    D1Re0,AC1.0,#0x1f
        D         ASRS      D0Re0,AC1.0,#0x1f
        D         LSLS      TXSTAT,[D1BR.1],#0xf
        DHCS      ASLSVS    D1Re0,[D1BR.1],#0xf
        D         ASRS      D0Re0,[D1BR.1],#0xf
        D         LSLS      TXSTAT,[D1BR.1],#0x10
        DHCS      ASLSVS    D1Re0,[D1BR.1],#0x10
        D         ASRS      D0Re0,[D1BR.1],#0x10
        D         LSLS      TXSTAT,[D1BR.1],#0x1f
        DHCS      ASLSVS    D1Re0,[D1BR.1],#0x1f
        D         ASRS      D0Re0,[D1BR.1],#0x1f
        D         LSLS      TXSTAT,[D1BR.1++],#0xf
        DHCS      ASLSVS    D1Re0,[D1BR.1++],#0xf
        D         ASRS      D0Re0,[D1BR.1++],#0xf
        D         LSLS      TXSTAT,[D1BR.1++],#0x10
        DHCS      ASLSVS    D1Re0,[D1BR.1++],#0x10
        D         ASRS      D0Re0,[D1BR.1++],#0x10
        D         LSLS      TXSTAT,[D1BR.1++],#0x1f
        DHCS      ASLSVS    D1Re0,[D1BR.1++],#0x1f
        D         ASRS      D0Re0,[D1BR.1++],#0x1f
        D         LSLS      TXSTAT,[D1BR.1+D1BRI.1++],#0xf
        DHCS      ASLSVS    D1Re0,[D1BR.1+D1BRI.1++],#0xf
        D         ASRS      D0Re0,[D1BR.1+D1BRI.1++],#0xf
        D         LSLS      TXSTAT,[D1BR.1+D1BRI.1++],#0x10
        DHCS      ASLSVS    D1Re0,[D1BR.1+D1BRI.1++],#0x10
        D         ASRS      D0Re0,[D1BR.1+D1BRI.1++],#0x10
        D         LSLS      TXSTAT,[D1BR.1+D1BRI.1++],#0x1f
        DHCS      ASLSVS    D1Re0,[D1BR.1+D1BRI.1++],#0x1f
        D         ASRS      D0Re0,[D1BR.1+D1BRI.1++],#0x1f
        D         LSRS      PCX,D1Re0,#0xf
        D         LSLS      A1LbP,D1Re0,#0x10
        D         ASRS      A0FrP,D1Re0,#0x10
        D         LSRS      PCX,D1Re0,#0x1f
        D         LSLS      A1LbP,D1.7,#0xf
        D         ASRS      A0FrP,D1.7,#0xf
        D         LSRS      PCX,D1.7,#0x10
        D         LSLS      A1LbP,D1.7,#0x1f
        D         ASRS      A0FrP,D1.7,#0x1f
        D         LSRS      PCX,AC1.0,#0xf
        D         LSLS      A1LbP,AC1.0,#0x10
        D         ASRS      A0FrP,AC1.0,#0x10
        D         LSRS      PCX,AC1.0,#0x1f
        D         LSLS      A1LbP,[D1BR.1],#0xf
        D         ASRS      A0FrP,[D1BR.1],#0xf
        D         LSRS      PCX,[D1BR.1],#0x10
        D         LSLS      A1LbP,[D1BR.1],#0x1f
        D         ASRS      A0FrP,[D1BR.1],#0x1f
        D         LSRS      PCX,[D1BR.1++],#0xf
        D         LSLS      A1LbP,[D1BR.1++],#0x10
        D         ASRS      A0FrP,[D1BR.1++],#0x10
        D         LSRS      PCX,[D1BR.1++],#0x1f
        D         LSLS      A1LbP,[D1BR.1+D1BRI.1++],#0xf
        D         ASRS      A0FrP,[D1BR.1+D1BRI.1++],#0xf
        D         LSRS      PCX,[D1BR.1+D1BRI.1++],#0x10
        D         LSLS      A1LbP,[D1BR.1+D1BRI.1++],#0x1f
        D         ASRS      A0FrP,[D1BR.1+D1BRI.1++],#0x1f
        D         LSRS      TXMASKI,D1Re0,#0x10
        D         LSRS      TXMASKI,D1.7,#0xf
        D         LSRS      TXMASKI,D1.7,#0x1f
        D         LSRS      TXMASKI,AC1.0,#0x10
        D         LSRS      TXMASKI,[D1BR.1],#0xf
        D         LSRS      TXMASKI,[D1BR.1],#0x1f
        D         LSRS      TXMASKI,[D1BR.1++],#0x10
        D         LSRS      TXMASKI,[D1BR.1+D1BRI.1++],#0xf
        D         LSRS      TXMASKI,[D1BR.1+D1BRI.1++],#0x1f
        D         LSLS      TXGPIOO,D1Re0,#0xf
        D         ASLSNV    D1.7,D1Re0,#0xf
        D         ASRS      A0.7,D1Re0,#0xf
        D         LSLS      TXGPIOO,D1Re0,#0x10
        D         ASLSNV    D1.7,D1Re0,#0x10
        D         ASRS      A0.7,D1Re0,#0x10
        D         LSLS      TXGPIOO,D1Re0,#0x1f
        D         ASLSNV    D1.7,D1Re0,#0x1f
        D         ASRS      A0.7,D1Re0,#0x1f
        D         LSLS      TXGPIOO,D1.7,#0xf
        D         ASLSNV    D1.7,D1.7,#0xf
        D         ASRS      A0.7,D1.7,#0xf
        D         LSLS      TXGPIOO,D1.7,#0x10
        D         ASLSNV    D1.7,D1.7,#0x10
        D         ASRS      A0.7,D1.7,#0x10
        D         LSLS      TXGPIOO,D1.7,#0x1f
        D         ASLSNV    D1.7,D1.7,#0x1f
        D         ASRS      A0.7,D1.7,#0x1f
        D         LSLS      TXGPIOO,AC1.0,#0xf
        D         ASLSNV    D1.7,AC1.0,#0xf
        D         ASRS      A0.7,AC1.0,#0xf
        D         LSLS      TXGPIOO,AC1.0,#0x10
        D         ASLSNV    D1.7,AC1.0,#0x10
        D         ASRS      A0.7,AC1.0,#0x10
        D         LSLS      TXGPIOO,AC1.0,#0x1f
        D         ASLSNV    D1.7,AC1.0,#0x1f
        D         ASRS      A0.7,AC1.0,#0x1f
        D         LSLS      TXGPIOO,[D1BR.1],#0xf
        DHCS      ASLSVS    D1.7,[D1BR.1],#0xf
        D         ASRS      A0.7,[D1BR.1],#0xf
        D         LSLS      TXGPIOO,[D1BR.1],#0x10
        DHCS      ASLSVS    D1.7,[D1BR.1],#0x10
        D         ASRS      A0.7,[D1BR.1],#0x10
        D         LSLS      TXGPIOO,[D1BR.1],#0x1f
        DHCS      ASLSVS    D1.7,[D1BR.1],#0x1f
        D         ASRS      A0.7,[D1BR.1],#0x1f
        D         LSLS      TXGPIOO,[D1BR.1++],#0xf
        DHCS      ASLSVS    D1.7,[D1BR.1++],#0xf
        D         ASRS      A0.7,[D1BR.1++],#0xf
        D         LSLS      TXGPIOO,[D1BR.1++],#0x10
        DHCS      ASLSVS    D1.7,[D1BR.1++],#0x10
        D         ASRS      A0.7,[D1BR.1++],#0x10
        D         LSLS      TXGPIOO,[D1BR.1++],#0x1f
        DHCS      ASLSVS    D1.7,[D1BR.1++],#0x1f
        D         ASRS      A0.7,[D1BR.1++],#0x1f
        D         LSLS      TXGPIOO,[D1BR.1+D1BRI.1++],#0xf
        DHCS      ASLSVS    D1.7,[D1BR.1+D1BRI.1++],#0xf
        D         ASRS      A0.7,[D1BR.1+D1BRI.1++],#0xf
        D         LSLS      TXGPIOO,[D1BR.1+D1BRI.1++],#0x10
        DHCS      ASLSVS    D1.7,[D1BR.1+D1BRI.1++],#0x10
        D         ASRS      A0.7,[D1BR.1+D1BRI.1++],#0x10
        D         LSLS      TXGPIOO,[D1BR.1+D1BRI.1++],#0x1f
        DHCS      ASLSVS    D1.7,[D1BR.1+D1BRI.1++],#0x1f
        D         ASRS      A0.7,[D1BR.1+D1BRI.1++],#0x1f
        D         ASLS      A1.8,D1Re0,#0xf
        D         ASLS      A1.8,D1Re0,#0x10
        D         ASLS      A1.8,D1Re0,#0x1f
        D         ASLS      A1.8,D1.7,#0xf
        D         ASLS      A1.8,D1.7,#0x10
        D         ASLS      A1.8,D1.7,#0x1f
        D         ASLS      A1.8,AC1.0,#0xf
        D         ASLS      A1.8,AC1.0,#0x10
        D         ASLS      A1.8,AC1.0,#0x1f
        D         ASLS      A1.8,[D1BR.1],#0xf
        D         ASLS      A1.8,[D1BR.1],#0x10
        D         ASLS      A1.8,[D1BR.1],#0x1f
        D         ASLS      A1.8,[D1BR.1++],#0xf
        D         ASLS      A1.8,[D1BR.1++],#0x10
        D         ASLS      A1.8,[D1BR.1++],#0x1f
        D         ASLS      A1.8,[D1BR.1+D1BRI.1++],#0xf
        D         ASLS      A1.8,[D1BR.1+D1BRI.1++],#0x10
        D         ASLS      A1.8,[D1BR.1+D1BRI.1++],#0x1f
        D         LSRSNV    D1.16,D1Re0,#0xf
        D         ASRSNV    D1.16,D1Re0,#0xf
        D         LSRSNV    D1.16,D1Re0,#0x10
        D         ASRSNV    D1.16,D1Re0,#0x10
        D         LSRSNV    D1.16,D1Re0,#0x1f
        D         ASRSNV    D1.16,D1Re0,#0x1f
        D         LSRSNV    D1.16,D1.7,#0xf
        D         ASRSNV    D1.16,D1.7,#0xf
        D         LSRSNV    D1.16,D1.7,#0x10
        D         ASRSNV    D1.16,D1.7,#0x10
        D         LSRSNV    D1.16,D1.7,#0x1f
        D         ASRSNV    D1.16,D1.7,#0x1f
        D         LSRSNV    D1.16,AC1.0,#0xf
        D         ASRSNV    D1.16,AC1.0,#0xf
        D         LSRSNV    D1.16,AC1.0,#0x10
        D         ASRSNV    D1.16,AC1.0,#0x10
        D         LSRSNV    D1.16,AC1.0,#0x1f
        D         ASRSNV    D1.16,AC1.0,#0x1f
        D         LSRSNV    D1.16,[D1BR.1],#0xf
        D         ASRSNV    D1.16,[D1BR.1],#0xf
        D         LSRSNV    D1.16,[D1BR.1],#0x10
        D         ASRSNV    D1.16,[D1BR.1],#0x10
        D         LSRSNV    D1.16,[D1BR.1],#0x1f
        D         ASRSNV    D1.16,[D1BR.1],#0x1f
        D         LSRSNV    D1.16,[D1BR.1++],#0xf
        D         ASRSNV    D1.16,[D1BR.1++],#0xf
        D         LSRSNV    D1.16,[D1BR.1++],#0x10
        D         ASRSNV    D1.16,[D1BR.1++],#0x10
        D         LSRSNV    D1.16,[D1BR.1++],#0x1f
        D         ASRSNV    D1.16,[D1BR.1++],#0x1f
        D         LSRSNV    D1.16,[D1BR.1+D1BRI.1++],#0xf
        D         ASRSNV    D1.16,[D1BR.1+D1BRI.1++],#0xf
        D         LSRSNV    D1.16,[D1BR.1+D1BRI.1++],#0x10
        D         ASRSNV    D1.16,[D1BR.1+D1BRI.1++],#0x10
        D         LSRSNV    D1.16,[D1BR.1+D1BRI.1++],#0x1f
        D         ASRSNV    D1.16,[D1BR.1+D1BRI.1++],#0x1f
        D         LSRS      RAM8X32,D1Re0,#0x10
        D         LSRS      RAM8X32,D1.7,#0xf
        D         LSRS      RAM8X32,D1.7,#0x1f
        D         LSRS      RAM8X32,AC1.0,#0x10
        D         LSRS      RAM8X32,[D1BR.1],#0xf
        D         LSRS      RAM8X32,[D1BR.1],#0x1f
        D         LSRS      RAM8X32,[D1BR.1++],#0x10
        D         LSRS      RAM8X32,[D1BR.1+D1BRI.1++],#0xf
        D         LSRS      RAM8X32,[D1BR.1+D1BRI.1++],#0x1f
        D         LSRS      RAM8X,D1Re0,#0x10
        D         LSRS      RAM8X,D1.7,#0xf
        D         LSRS      RAM8X,D1.7,#0x1f
        D         LSRS      RAM8X,AC1.0,#0x10
        D         LSRS      RAM8X,[D1BR.1],#0xf
        D         LSRS      RAM8X,[D1BR.1],#0x1f
        D         LSRS      RAM8X,[D1BR.1++],#0x10
        D         LSRS      RAM8X,[D1BR.1+D1BRI.1++],#0xf
        D         LSRS      RAM8X,[D1BR.1+D1BRI.1++],#0x1f
        D         LSRS      RABZ,D1Re0,#0x10
        D         LSRS      RABZ,D1.7,#0xf
        D         LSRS      RABZ,D1.7,#0x1f
        D         LSRS      RABZ,AC1.0,#0x10
        D         LSRS      RABZ,[D1BR.1],#0xf
        D         LSRS      RABZ,[D1BR.1],#0x1f
        D         LSRS      RABZ,[D1BR.1++],#0x10
        D         LSRS      RABZ,[D1BR.1+D1BRI.1++],#0xf
        D         LSRS      RABZ,[D1BR.1+D1BRI.1++],#0x1f
        D         LSRS      RAWX,D1Re0,#0x10
        D         LSRS      RAWX,D1.7,#0xf
        D         LSRS      RAWX,D1.7,#0x1f
        D         LSRS      RAWX,AC1.0,#0x10
        D         LSRS      RAWX,[D1BR.1],#0xf
        D         LSRS      RAWX,[D1BR.1],#0x1f
        D         LSRS      RAWX,[D1BR.1++],#0x10
        D         LSRS      RAWX,[D1BR.1+D1BRI.1++],#0xf
        D         LSRS      RAWX,[D1BR.1+D1BRI.1++],#0x1f
        D         ASLS      RAMX,D1Re0,#0xf
        D         ASLS      RAMX,D1Re0,#0x10
        D         ASLS      RAMX,D1Re0,#0x1f
        D         ASLS      RAMX,D1.7,#0xf
        D         ASLS      RAMX,D1.7,#0x10
        D         ASLS      RAMX,D1.7,#0x1f
        D         ASLS      RAMX,AC1.0,#0xf
        D         ASLS      RAMX,AC1.0,#0x10
        D         ASLS      RAMX,AC1.0,#0x1f
        D         ASLS      RAMX,[D1BR.1],#0xf
        D         ASLS      RAMX,[D1BR.1],#0x10
        D         ASLS      RAMX,[D1BR.1],#0x1f
        D         ASLS      RAMX,[D1BR.1++],#0xf
        D         ASLS      RAMX,[D1BR.1++],#0x10
        D         ASLS      RAMX,[D1BR.1++],#0x1f
        D         ASLS      RAMX,[D1BR.1+D1BRI.1++],#0xf
        D         ASLS      RAMX,[D1BR.1+D1BRI.1++],#0x10
        D         ASLS      RAMX,[D1BR.1+D1BRI.1++],#0x1f
        DUM       MUL       D0Re0,D0Re0,D0Re0
        D         MUL       D0Re0,D0Re0,A1.7
        DUM       MUL       D0Re0,D0Re0,A1.7
        DM        MUL       D0Re0,D0Re0,D1.7
        DLM       MUL       D0Re0,D0Re0,[D0AR.0]
        DLUM      MUL       D0Re0,D0Re0,RD
        DLUM      MUL       D0Re0,D0Re0,[D0AR.0++]
        DLUM      MUL       D0Re0,D0Re0,[D0AR.0+D0ARI.0++]
        DUM       MUL       D0Re0,D0Re0,A0.7
        DLUM      MUL       D0Re0,D0.7,D0Re0
        DM        MUL       D0Re0,D0.7,A1.7
        DLUM      MUL       D0Re0,D0.7,D0.7
        DUM       MUL       D0Re0,D0.7,D1.7
        DU        MUL       D0Re0,D0.7,RD
        DLM       MUL       D0Re0,D0.7,[D0AR.0++]
        DLM       MUL       D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        DUM       MUL       D0Re0,D0.7,A0FrP
        DLM       MUL       D0Re0,[D0AR.0],D0Re0
        DUM       MUL       D0Re0,[D0AR.0],A1LbP
        DU        MUL       D0Re0,[D0AR.0],A1.7
        DUM       MUL       D0Re0,[D0AR.0],D1Re0
        DM        MUL       D0Re0,[D0AR.0],RD
        DLUM      MUL       D0Re0,[D0AR.0],[D0AR.0]
        DLUM      MUL       D0Re0,[D0AR.0],[D0AR.0++]
        DLUM      MUL       D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        DUM       MUL       D0Re0,[D0AR.0],A0.7
        DLUM      MUL       D0Re0,[D0AR.0++],D0Re0
        DM        MUL       D0Re0,[D0AR.0++],A1.7
        DLUM      MUL       D0Re0,[D0AR.0++],D0.7
        DUM       MUL       D0Re0,[D0AR.0++],D1.7
        DU        MUL       D0Re0,[D0AR.0++],RD
        DLM       MUL       D0Re0,[D0AR.0++],[D0AR.0++]
        DLM       MUL       D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DUM       MUL       D0Re0,[D0AR.0++],A0FrP
        DLM       MUL       D0Re0,[D0AR.0+D0ARI.0++],D0Re0
        DUM       MUL       D0Re0,[D0AR.0+D0ARI.0++],A1LbP
        DU        MUL       D0Re0,[D0AR.0+D0ARI.0++],A1.7
        DUM       MUL       D0Re0,[D0AR.0+D0ARI.0++],D1Re0
        DM        MUL       D0Re0,[D0AR.0+D0ARI.0++],RD
        DLUM      MUL       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DLUM      MUL       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLUM      MUL       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DUM       MUL       D0Re0,[D0AR.0+D0ARI.0++],A0.7
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0FrP++],T6
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0FrP++],T6
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0FrP++],T6
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0FrP++],T6
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0FrP++],T6
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0FrP++],T6
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0FrP++],T6
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0FrP++],T6
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0FrP++]
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0FrP++]
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0FrP++]
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0FrP++]
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0FrP++]
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0FrP++]
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0FrP++]
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0FrP++]
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.7++],T6
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.7++],T6
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.7++],T6
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.7++],T6
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.7++],T6
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.7++],T6
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.7++],T6
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.7++],T6
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.7++]
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.7++]
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.7++]
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.7++]
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.7++]
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.7++]
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.7++]
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.7++]
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.8++],T6
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.8++],T6
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.8++],T6
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.8++],T6
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.8++],T6
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.8++],T6
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.8++],T6
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP+A0.8++],T6
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.8++]
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.8++]
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.8++]
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.8++]
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.8++]
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.8++]
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.8++]
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP+A0.8++]
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP],T6
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP],T6
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP],T6
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP],T6
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP],T6
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP],T6
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP],T6
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP],T6
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP]
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP]
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP]
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP]
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP]
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP]
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP]
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP]
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP++],T6
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP++],T6
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP++],T6
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP++],T6
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP++],T6
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP++],T6
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP++],T6
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP++],T6
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP++]
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP++]
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP++]
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP++]
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP++]
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP++]
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP++]
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP++]
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP--],T6
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP--],T6
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP--],T6
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP--],T6
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 [A0FrP--],T6
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP--],T6
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0FrP--],T6
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 [A0FrP--],T6
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP--]
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP--]
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP--]
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP--]
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP--]
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP--]
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP--]
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0FrP--]
        DLUM      MUL       D0.7,D0Re0,D0Re0
        DM        MUL       D0.7,D0Re0,A1.7
        DLUM      MUL       D0.7,D0Re0,D0.7
        DUM       MUL       D0.7,D0Re0,D1.7
        DU        MUL       D0.7,D0Re0,RD
        DLM       MUL       D0.7,D0Re0,[D0AR.0++]
        DLM       MUL       D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DUM       MUL       D0.7,D0Re0,A0FrP
        DZWT      MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0FrP++],T6
        DLZWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0FrP++],T6
        DUZWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0FrP++],T6
        DLUZWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0FrP++],T6
        DNWT      MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0FrP++],T6
        DLNWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0FrP++],T6
        DUNWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0FrP++],T6
        DLUNWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0FrP++],T6
        DZWT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0FrP++]
        DLZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0FrP++]
        DUZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0FrP++]
        DLUZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0FrP++]
        DNWT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0FrP++]
        DLNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0FrP++]
        DUNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0FrP++]
        DLUNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0FrP++]
        DLM       MUL       D0.7,D0.7,D0Re0
        DUM       MUL       D0.7,D0.7,A1LbP
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.7++],T6
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.7++],T6
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.7++],T6
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.7++],T6
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.7++],T6
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.7++],T6
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.7++],T6
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.7++],T6
        DU        MUL       D0.7,D0.7,A1.7
        DUM       MUL       D0.7,D0.7,D1Re0
        DM        MUL       D0.7,D0.7,RD
        DLUM      MUL       D0.7,D0.7,[D0AR.0]
        DLUM      MUL       D0.7,D0.7,[D0AR.0++]
        DLUM      MUL       D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.7++]
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.7++]
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.7++]
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.7++]
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.7++]
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.7++]
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.7++]
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.7++]
        DUM       MUL       D0.7,D0.7,A0.7
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.8++],T6
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.8++],T6
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.8++],T6
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.8++],T6
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.8++],T6
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.8++],T6
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.8++],T6
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 [A0.7+A0.8++],T6
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.8++]
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.8++]
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.8++]
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.8++]
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.8++]
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.8++]
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.8++]
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7+A0.8++]
        DLUM      MUL       D0.7,[D0AR.0],D0Re0
        DZWT      MUL       AC0.0,D0Re0,D0Re0 [A0.7],T6
        DLZWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7],T6
        DUZWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7],T6
        DLUZWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7],T6
        DNWT      MUL       AC0.0,D0Re0,D0Re0 [A0.7],T6
        DLNWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7],T6
        DUNWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7],T6
        DLUNWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7],T6
        DM        MUL       D0.7,[D0AR.0],A1.7
        DLUM      MUL       D0.7,[D0AR.0],D0.7
        DUM       MUL       D0.7,[D0AR.0],D1.7
        DU        MUL       D0.7,[D0AR.0],RD
        DLM       MUL       D0.7,[D0AR.0],[D0AR.0++]
        DLM       MUL       D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DZWT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7]
        DLZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7]
        DUZWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7]
        DLUZWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7]
        DNWT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7]
        DLNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7]
        DUNWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7]
        DLUNWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7]
        DUM       MUL       D0.7,[D0AR.0],A0FrP
        DLM       MUL       D0.7,[D0AR.0++],D0Re0
        DUM       MUL       D0.7,[D0AR.0++],A1LbP
        DMZWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7++],T6
        DLMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7++],T6
        DUMZWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7++],T6
        DLUMZWT   MUL       AC0.0,D0Re0,D0Re0 [A0.7++],T6
        DMNWT     MUL       AC0.0,D0Re0,D0Re0 [A0.7++],T6
        DLMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7++],T6
        DUMNWT    MUL       AC0.0,D0Re0,D0Re0 [A0.7++],T6
        DLUMNWT   MUL       AC0.0,D0Re0,D0Re0 [A0.7++],T6
        DU        MUL       D0.7,[D0AR.0++],A1.7
        DUM       MUL       D0.7,[D0AR.0++],D1Re0
        DM        MUL       D0.7,[D0AR.0++],RD
        DLU       MUL       D0.7,[D0AR.0++],[D0AR.0]
        DLU       MUL       D0.7,[D0AR.0++],[D0AR.0++]
        DLU       MUL       D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DMZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7++]
        DLMZT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7++]
        DUMZT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7++]
        DLUMZT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7++]
        DMNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7++]
        DLMNT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7++]
        DUMNT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7++]
        DLUMNT    MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7++]
        DU        MUL       D0.7,[D0AR.0++],A0.7
        DLU       MUL       D0.7,[D0AR.0+D0ARI.0++],D0Re0
        DM        MUL       D0.7,[D0AR.0+D0ARI.0++],D0.7
        DLU       MUL       D0.7,[D0AR.0+D0ARI.0++],D0.7
        DU        MUL       D0.7,[D0AR.0+D0ARI.0++],D1.7
        DU        MUL       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        MUL       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DL        MUL       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DU        MUL       D0.7,[D0AR.0+D0ARI.0++],A0FrP
        DZT       MUL       AC0.0,D0Re0,D0Re0 [A0.7--],T6
        DLZT      MUL       AC0.0,D0Re0,D0Re0 [A0.7--],T6
        DUZT      MUL       AC0.0,D0Re0,D0Re0 [A0.7--],T6
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 [A0.7--],T6
        DNT       MUL       AC0.0,D0Re0,D0Re0 [A0.7--],T6
        DLNT      MUL       AC0.0,D0Re0,D0Re0 [A0.7--],T6
        DUNT      MUL       AC0.0,D0Re0,D0Re0 [A0.7--],T6
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 [A0.7--],T6
        DZT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7--]
        DLZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7--]
        DUZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7--]
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7--]
        DNT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7--]
        DLNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7--]
        DUNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7--]
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.7--]
        DZT       MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0FrP++],T6
        DLZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0FrP++],T6
        DUZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0FrP++],T6
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0FrP++],T6
        DNT       MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0FrP++],T6
        DLNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0FrP++],T6
        DUNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0FrP++],T6
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0FrP++],T6
        DZT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0FrP++]
        DLZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0FrP++]
        DUZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0FrP++]
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0FrP++]
        DNT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0FrP++]
        DLNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0FrP++]
        DUNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0FrP++]
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0FrP++]
        DZT       MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.7++],T6
        DLZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.7++],T6
        DUZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.7++],T6
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.7++],T6
        DNT       MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.7++],T6
        DLNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.7++],T6
        DUNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.7++],T6
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.7++],T6
        DZT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.7++]
        DLZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.7++]
        DUZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.7++]
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.7++]
        DNT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.7++]
        DLNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.7++]
        DUNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.7++]
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.7++]
        DZT       MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.8++],T6
        DLZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.8++],T6
        DUZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.8++],T6
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.8++],T6
        DNT       MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.8++],T6
        DLNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.8++],T6
        DUNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.8++],T6
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 [A0.8+A0.8++],T6
        DZT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.8++]
        DLZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.8++]
        DUZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.8++]
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.8++]
        DNT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.8++]
        DLNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.8++]
        DUNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.8++]
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8+A0.8++]
        DZT       MUL       AC0.0,D0Re0,D0Re0 [A0.8],T6
        DLZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8],T6
        DUZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8],T6
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 [A0.8],T6
        DNT       MUL       AC0.0,D0Re0,D0Re0 [A0.8],T6
        DLNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8],T6
        DUNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8],T6
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 [A0.8],T6
        DZT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8]
        DLZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8]
        DUZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8]
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8]
        DNT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8]
        DLNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8]
        DUNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8]
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8]
        DZT       MUL       AC0.0,D0Re0,D0Re0 [A0.8++],T6
        DLZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8++],T6
        DUZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8++],T6
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 [A0.8++],T6
        DNT       MUL       AC0.0,D0Re0,D0Re0 [A0.8++],T6
        DLNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8++],T6
        DUNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8++],T6
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 [A0.8++],T6
        DZT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8++]
        DLZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8++]
        DUZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8++]
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8++]
        DNT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8++]
        DLNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8++]
        DUNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8++]
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8++]
        DZT       MUL       AC0.0,D0Re0,D0Re0 [A0.8--],T6
        DLZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8--],T6
        DUZT      MUL       AC0.0,D0Re0,D0Re0 [A0.8--],T6
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 [A0.8--],T6
        DNT       MUL       AC0.0,D0Re0,D0Re0 [A0.8--],T6
        DLNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8--],T6
        DUNT      MUL       AC0.0,D0Re0,D0Re0 [A0.8--],T6
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 [A0.8--],T6
        DZT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8--]
        DLZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8--]
        DUZT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8--]
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8--]
        DNT       MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8--]
        DLNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8--]
        DUNT      MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8--]
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 T6,[A0.8--]
        DM        MUL       [D0AW.0],D0Re0,D0Re0
        DUM       MUL       [D0AW.0],D0Re0,D0Re0
        DMP       MUL       AC0.0,D0Re0,D0Re0
        DUMP      MUL       AC0.0,D0Re0,D0Re0
        DM        MUL       [D0AW.0],D0Re0,A1LbP
        DMP       MUL       AC0.0,D0Re0,A1LbP
        DZ        MUL       AC0.0,D0Re0,D0.7
        DLM       MUL       [D0AW.0],D0Re0,D0.7
        DUMZ      MUL       AC0.0,D0Re0,D0.7
        DN        MUL       AC0.0,D0Re0,D0.7
        DLMP      MUL       AC0.0,D0Re0,D0.7
        DUMN      MUL       AC0.0,D0Re0,D0.7
        DM        MUL       [D0AW.0],D0Re0,D1Re0
        DMP       MUL       AC0.0,D0Re0,D1Re0
        DM        MUL       [D0AW.0],D0Re0,D1.7
        DMP       MUL       AC0.0,D0Re0,D1.7
        DZ        MUL       AC0.0,D0Re0,[D0AR.0]
        DLZ       MUL       AC0.0,D0Re0,RD
        DUM       MUL       [D0AW.0],D0Re0,[D0AR.0]
        DLUMZ     MUL       AC0.0,D0Re0,[D0AR.0]
        DMN       MUL       AC0.0,D0Re0,[D0AR.0]
        DLMN      MUL       AC0.0,D0Re0,[D0AR.0]
        DUMN      MUL       AC0.0,D0Re0,[D0AR.0]
        DLUMN     MUL       AC0.0,D0Re0,[D0AR.0]
        DLM       MUL       [D0AW.0],D0Re0,[D0AR.0++]
        DLUM      MUL       [D0AW.0],D0Re0,[D0AR.0++]
        DLMP      MUL       AC0.0,D0Re0,[D0AR.0++]
        DLUMP     MUL       AC0.0,D0Re0,[D0AR.0++]
        DLM       MUL       [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        DLUM      MUL       [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        DLMP      MUL       AC0.0,D0Re0,[D0AR.0+D0ARI.0++]
        DLUMP     MUL       AC0.0,D0Re0,[D0AR.0+D0ARI.0++]
        DUM       MUL       [D0AW.0],D0Re0,A0FrP
        DUMP      MUL       AC0.0,D0Re0,A0FrP
        DUM       MUL       [D0AW.0],D0Re0,A0.7
        DUMP      MUL       AC0.0,D0Re0,A0.7
        DLM       MUL       [D0AW.0],D0.7,D0Re0
        DLUM      MUL       [D0AW.0],D0.7,D0Re0
        DLMP      MUL       AC0.0,D0.7,D0Re0
        DLUMP     MUL       AC0.0,D0.7,D0Re0
        DUM       MUL       [D0AW.0],D0.7,A1LbP
        DUMP      MUL       AC0.0,D0.7,A1LbP
        DMZ       MUL       AC0.0,D0.7,D0.7
        DUZ       MUL       AC0.0,D0.7,D0.7
        DLUM      MUL       [D0AW.0],D0.7,D0.7
        DMN       MUL       AC0.0,D0.7,D0.7
        DUN       MUL       AC0.0,D0.7,D0.7
        DLUMP     MUL       AC0.0,D0.7,D0.7
        DUM       MUL       [D0AW.0],D0.7,D1Re0
        DUMP      MUL       AC0.0,D0.7,D1Re0
        DUM       MUL       [D0AW.0],D0.7,D1.7
        DUMP      MUL       AC0.0,D0.7,D1.7
        DMZ       MUL       AC0.0,D0.7,[D0AR.0]
        DU        MUL       [D0AW.0],D0.7,[D0AR.0]
        DLU       MUL       [D0AW.0],D0.7,[D0AR.0]
        DN        MUL       AC0.0,D0.7,[D0AR.0]
        DLN       MUL       AC0.0,D0.7,[D0AR.0]
        DUN       MUL       AC0.0,D0.7,[D0AR.0]
        DLUN      MUL       AC0.0,D0.7,[D0AR.0]
        DM        MUL       [D0AW.0],D0.7,[D0AR.0++]
        DUM       MUL       [D0AW.0],D0.7,[D0AR.0++]
        DMP       MUL       AC0.0,D0.7,[D0AR.0++]
        DUMP      MUL       AC0.0,D0.7,[D0AR.0++]
        DM        MUL       [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        DUM       MUL       [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        DMP       MUL       AC0.0,D0.7,[D0AR.0+D0ARI.0++]
        DUMP      MUL       AC0.0,D0.7,[D0AR.0+D0ARI.0++]
        DM        MUL       [D0AW.0],D0.7,A0FrP
        DMP       MUL       AC0.0,D0.7,A0FrP
        DM        MUL       [D0AW.0],D0.7,A0.7
        DMP       MUL       AC0.0,D0.7,A0.7
        DM        MUL       [D0AW.0],[D0AR.0],D0Re0
        DUM       MUL       [D0AW.0],[D0AR.0],D0Re0
        DMP       MUL       AC0.0,[D0AR.0],D0Re0
        DUMP      MUL       AC0.0,[D0AR.0],D0Re0
        DM        MUL       [D0AW.0],[D0AR.0],A1LbP
        DMP       MUL       AC0.0,[D0AR.0],A1LbP
        DZ        MUL       AC0.0,[D0AR.0],D0.7
        DLM       MUL       [D0AW.0],[D0AR.0],D0.7
        DUMZ      MUL       AC0.0,[D0AR.0],D0.7
        DN        MUL       AC0.0,[D0AR.0],D0.7
        DLMP      MUL       AC0.0,[D0AR.0],D0.7
        DUMN      MUL       AC0.0,[D0AR.0],D0.7
        DM        MUL       [D0AW.0],[D0AR.0],D1Re0
        DMP       MUL       AC0.0,[D0AR.0],D1Re0
        DM        MUL       [D0AW.0],[D0AR.0],D1.7
        DMP       MUL       AC0.0,[D0AR.0],D1.7
        DZ        MUL       AC0.0,[D0AR.0],[D0AR.0]
        DLZ       MUL       AC0.0,[D0AR.0],RD
        DUM       MUL       [D0AW.0],[D0AR.0],[D0AR.0]
        DLUMZ     MUL       AC0.0,[D0AR.0],[D0AR.0]
        DMN       MUL       AC0.0,[D0AR.0],[D0AR.0]
        DLMN      MUL       AC0.0,[D0AR.0],[D0AR.0]
        DUMN      MUL       AC0.0,[D0AR.0],[D0AR.0]
        DLUMN     MUL       AC0.0,[D0AR.0],[D0AR.0]
        DLM       MUL       [D0AW.0],[D0AR.0],[D0AR.0++]
        DLUM      MUL       [D0AW.0],[D0AR.0],[D0AR.0++]
        DLMP      MUL       AC0.0,[D0AR.0],[D0AR.0++]
        DLUMP     MUL       AC0.0,[D0AR.0],[D0AR.0++]
        DLM       MUL       [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLUM      MUL       [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLMP      MUL       AC0.0,[D0AR.0],[D0AR.0+D0ARI.0++]
        DLUMP     MUL       AC0.0,[D0AR.0],[D0AR.0+D0ARI.0++]
        DUM       MUL       [D0AW.0],[D0AR.0],A0FrP
        DUMP      MUL       AC0.0,[D0AR.0],A0FrP
        DUM       MUL       [D0AW.0],[D0AR.0],A0.7
        DUMP      MUL       AC0.0,[D0AR.0],A0.7
        DLM       MUL       [D0AW.0],[D0AR.0++],D0Re0
        DLUM      MUL       [D0AW.0],[D0AR.0++],D0Re0
        DLMP      MUL       AC0.0,[D0AR.0++],D0Re0
        DLUMP     MUL       AC0.0,[D0AR.0++],D0Re0
        DUM       MUL       [D0AW.0],[D0AR.0++],A1LbP
        DUMP      MUL       AC0.0,[D0AR.0++],A1LbP
        DMZ       MUL       AC0.0,[D0AR.0++],D0.7
        DUZ       MUL       AC0.0,[D0AR.0++],D0.7
        DLUM      MUL       [D0AW.0],[D0AR.0++],D0.7
        DMN       MUL       AC0.0,[D0AR.0++],D0.7
        DUN       MUL       AC0.0,[D0AR.0++],D0.7
        DLUMP     MUL       AC0.0,[D0AR.0++],D0.7
        DUM       MUL       [D0AW.0],[D0AR.0++],D1Re0
        DUMP      MUL       AC0.0,[D0AR.0++],D1Re0
        DUM       MUL       [D0AW.0],[D0AR.0++],D1.7
        DUMP      MUL       AC0.0,[D0AR.0++],D1.7
        DMZ       MUL       AC0.0,[D0AR.0++],[D0AR.0]
        DU        MUL       [D0AW.0],[D0AR.0++],[D0AR.0]
        DLU       MUL       [D0AW.0],[D0AR.0++],[D0AR.0]
        DN        MUL       AC0.0,[D0AR.0++],[D0AR.0]
        DLN       MUL       AC0.0,[D0AR.0++],[D0AR.0]
        DUN       MUL       AC0.0,[D0AR.0++],[D0AR.0]
        DLUN      MUL       AC0.0,[D0AR.0++],[D0AR.0]
        DM        MUL       [D0AW.0],[D0AR.0++],[D0AR.0++]
        DUM       MUL       [D0AW.0],[D0AR.0++],[D0AR.0++]
        DMP       MUL       AC0.0,[D0AR.0++],[D0AR.0++]
        DUMP      MUL       AC0.0,[D0AR.0++],[D0AR.0++]
        DM        MUL       [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DUM       MUL       [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DMP       MUL       AC0.0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DUMP      MUL       AC0.0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DM        MUL       [D0AW.0],[D0AR.0++],A0FrP
        DMP       MUL       AC0.0,[D0AR.0++],A0FrP
        DM        MUL       [D0AW.0],[D0AR.0++],A0.7
        DMP       MUL       AC0.0,[D0AR.0++],A0.7
        DM        MUL       [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DUM       MUL       [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DMP       MUL       AC0.0,[D0AR.0+D0ARI.0++],D0Re0
        DUMP      MUL       AC0.0,[D0AR.0+D0ARI.0++],D0Re0
        DM        MUL       [D0AW.0],[D0AR.0+D0ARI.0++],A1LbP
        DMP       MUL       AC0.0,[D0AR.0+D0ARI.0++],A1LbP
        DZ        MUL       AC0.0,[D0AR.0+D0ARI.0++],D0.7
        DLM       MUL       [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DUMZ      MUL       AC0.0,[D0AR.0+D0ARI.0++],D0.7
        DN        MUL       AC0.0,[D0AR.0+D0ARI.0++],D0.7
        DLMP      MUL       AC0.0,[D0AR.0+D0ARI.0++],D0.7
        DUMN      MUL       AC0.0,[D0AR.0+D0ARI.0++],D0.7
        DM        MUL       [D0AW.0],[D0AR.0+D0ARI.0++],D1Re0
        DMP       MUL       AC0.0,[D0AR.0+D0ARI.0++],D1Re0
        DM        MUL       [D0AW.0],[D0AR.0+D0ARI.0++],D1.7
        DMP       MUL       AC0.0,[D0AR.0+D0ARI.0++],D1.7
        DZ        MUL       AC0.0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DLZ       MUL       AC0.0,[D0AR.0+D0ARI.0++],RD
        DUM       MUL       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        DLUM      MUL       [D0AW.0],[D0AR.0+D0ARI.0++],RD
        DMP       MUL       AC0.0,[D0AR.0+D0ARI.0++],RD
        DLMP      MUL       AC0.0,[D0AR.0+D0ARI.0++],RD
        DUMP      MUL       AC0.0,[D0AR.0+D0ARI.0++],RD
        DLUMP     MUL       AC0.0,[D0AR.0+D0ARI.0++],RD
        DLZ       MUL       AC0.0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLUZ      MUL       AC0.0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLN       MUL       AC0.0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLUN      MUL       AC0.0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DLZ       MUL       AC0.0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLUZ      MUL       AC0.0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLN       MUL       AC0.0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLUN      MUL       AC0.0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DUZ       MUL       AC0.0,[D0AR.0+D0ARI.0++],A0FrP
        DUN       MUL       AC0.0,[D0AR.0+D0ARI.0++],A0FrP
        DUZ       MUL       AC0.0,[D0AR.0+D0ARI.0++],A0.7
        DUN       MUL       AC0.0,[D0AR.0+D0ARI.0++],A0.7
        DLZT      MUL       AC0.0,D0Re0,D0Re0 [D0AW.0],T6
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 [D0AW.0],T6
        DLNT      MUL       AC0.0,D0Re0,D0Re0 [D0AW.0],T6
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 [D0AW.0],T6
        DLZT      MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0]
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0]
        DLNT      MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0]
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0]
        DUM       MUL       [D0AW.0++],D0Re0,D0Re0
        D         MUL       [D0AW.0++],D0Re0,A1.7
        DUM       MUL       [D0AW.0++],D0Re0,A1.7
        DM        MUL       [D0AW.0++],D0Re0,D1.7
        DLM       MUL       [D0AW.0++],D0Re0,[D0AR.0]
        DM        MUL       [D0AW.0++],D0Re0,[D0AR.0++]
        DM        MUL       [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DM        MUL       [D0AW.0++],D0Re0,A0FrP
        DWT       MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T6
        DLWT      MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T6
        DUWT      MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T6
        DLUWT     MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T6
        DPWT      MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1LbP++],T6
        DLPWT     MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1LbP++],T6
        DUPWT     MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1LbP++],T6
        DLUPWT    MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1LbP++],T6
        DWT       MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1LbP++]
        DLWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1LbP++]
        DUWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1LbP++]
        DLUWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1LbP++]
        DPWT      MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1LbP++]
        DLPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1LbP++]
        DUPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1LbP++]
        DLUPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1LbP++]
        DM        MUL       [D0AW.0++],D0.7,D0Re0
        DM        MUL       [D0AW.0++],D0.7,A1LbP
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1.7++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1.7++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1.7++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1.7++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1.7++],T6
        DLM       MUL       [D0AW.0++],D0.7,D0.7
        DM        MUL       [D0AW.0++],D0.7,D1Re0
        D         MUL       [D0AW.0++],D0.7,RD
        DUM       MUL       [D0AW.0++],D0.7,RD
        DUM       MUL       [D0AW.0++],D0.7,[D0AR.0++]
        DUM       MUL       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1.7++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1.7++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1.7++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1.7++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1.7++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1.7++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1.7++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1.7++]
        DM        MUL       [D0AW.0++],D0.7,A0.7
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1.8++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1.8++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1.8++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1LbP+A1.8++],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1.8++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1.8++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1.8++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP+A1.8++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1.8++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1.8++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1.8++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP+A1.8++]
        DUM       MUL       [D0AW.0++],[D0AR.0],D0Re0
        DWT       MUL       D0Re0,D0Re0,D0Re0 [A1LbP],T6
        DLWT      MUL       D0Re0,D0Re0,D0Re0 [A1LbP],T6
        DUWT      MUL       D0Re0,D0Re0,D0Re0 [A1LbP],T6
        DLUWT     MUL       D0Re0,D0Re0,D0Re0 [A1LbP],T6
        DPWT      MUL       AC0.0,D0Re0,D0Re0 [A1LbP],T6
        DLPWT     MUL       AC0.0,D0Re0,D0Re0 [A1LbP],T6
        DUPWT     MUL       AC0.0,D0Re0,D0Re0 [A1LbP],T6
        DLUPWT    MUL       AC0.0,D0Re0,D0Re0 [A1LbP],T6
        D         MUL       [D0AW.0++],[D0AR.0],A1.7
        DUM       MUL       [D0AW.0++],[D0AR.0],A1.7
        DM        MUL       [D0AW.0++],[D0AR.0],D1.7
        DLM       MUL       [D0AW.0++],[D0AR.0],[D0AR.0]
        DM        MUL       [D0AW.0++],[D0AR.0],[D0AR.0++]
        DM        MUL       [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DWT       MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP]
        DLWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP]
        DUWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP]
        DLUWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP]
        DPWT      MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP]
        DLPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP]
        DUPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP]
        DLUPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP]
        DM        MUL       [D0AW.0++],[D0AR.0],A0FrP
        DM        MUL       [D0AW.0++],[D0AR.0++],D0Re0
        DM        MUL       [D0AW.0++],[D0AR.0++],A1LbP
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1LbP++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1LbP++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1LbP++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1LbP++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1LbP++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1LbP++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1LbP++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1LbP++],T6
        DLM       MUL       [D0AW.0++],[D0AR.0++],D0.7
        DM        MUL       [D0AW.0++],[D0AR.0++],D1Re0
        D         MUL       [D0AW.0++],[D0AR.0++],RD
        DUM       MUL       [D0AW.0++],[D0AR.0++],RD
        DUM       MUL       [D0AW.0++],[D0AR.0++],[D0AR.0++]
        DUM       MUL       [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP++]
        DM        MUL       [D0AW.0++],[D0AR.0++],A0.7
        DUM       MUL       [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        D         MUL       [D0AW.0++],[D0AR.0+D0ARI.0++],A1.7
        DUM       MUL       [D0AW.0++],[D0AR.0+D0ARI.0++],A1.7
        DM        MUL       [D0AW.0++],[D0AR.0+D0ARI.0++],D1.7
        DLM       MUL       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        DM        MUL       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DM        MUL       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DM        MUL       [D0AW.0++],[D0AR.0+D0ARI.0++],A0FrP
        DWT       MUL       D0Re0,D0Re0,D0Re0 [A1LbP--],T6
        DLWT      MUL       D0Re0,D0Re0,D0Re0 [A1LbP--],T6
        DUWT      MUL       D0Re0,D0Re0,D0Re0 [A1LbP--],T6
        DLUWT     MUL       D0Re0,D0Re0,D0Re0 [A1LbP--],T6
        DPWT      MUL       AC0.0,D0Re0,D0Re0 [A1LbP--],T6
        DLPWT     MUL       AC0.0,D0Re0,D0Re0 [A1LbP--],T6
        DUPWT     MUL       AC0.0,D0Re0,D0Re0 [A1LbP--],T6
        DLUPWT    MUL       AC0.0,D0Re0,D0Re0 [A1LbP--],T6
        DWT       MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP--]
        DLWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP--]
        DUWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP--]
        DLUWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1LbP--]
        DPWT      MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP--]
        DLPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP--]
        DUPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP--]
        DLUPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1LbP--]
        DZT       MUL       AC0.0,D0Re0,D0Re0 [D0AW.0++],T6
        DUZT      MUL       AC0.0,D0Re0,D0Re0 [D0AW.0++],T6
        DNT       MUL       AC0.0,D0Re0,D0Re0 [D0AW.0++],T6
        DUNT      MUL       AC0.0,D0Re0,D0Re0 [D0AW.0++],T6
        DZT       MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0++]
        DUZT      MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0++]
        DNT       MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0++]
        DUNT      MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0++]
        DM        MUL       [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DM        MUL       [D0AW.0+D0AWI.0++],D0Re0,A1LbP
        DLM       MUL       [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DM        MUL       [D0AW.0+D0AWI.0++],D0Re0,D1Re0
        D         MUL       [D0AW.0+D0AWI.0++],D0Re0,RD
        DUM       MUL       [D0AW.0+D0AWI.0++],D0Re0,RD
        DUM       MUL       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0++]
        DUM       MUL       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DM        MUL       [D0AW.0+D0AWI.0++],D0Re0,A0.7
        DUM       MUL       [D0AW.0+D0AWI.0++],D0.7,D0Re0
        D         MUL       [D0AW.0+D0AWI.0++],D0.7,A1.7
        DUM       MUL       [D0AW.0+D0AWI.0++],D0.7,A1.7
        DM        MUL       [D0AW.0+D0AWI.0++],D0.7,D1.7
        DLM       MUL       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DM        MUL       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DM        MUL       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        DM        MUL       [D0AW.0+D0AWI.0++],D0.7,A0FrP
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0],A1LbP
        DLM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0],D1Re0
        D         MUL       [D0AW.0+D0AWI.0++],[D0AR.0],RD
        DUM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0],RD
        DUM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0++]
        DUM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0],A0.7
        DUM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         MUL       [D0AW.0+D0AWI.0++],[D0AR.0++],A1.7
        DUM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0++],A1.7
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0++],D1.7
        DLM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0++],A0FrP
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A1LbP
        DLM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D1Re0
        D         MUL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],RD
        DUM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],RD
        DUM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0++]
        DUM       MUL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DM        MUL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],A0.7
        DLZT      MUL       AC0.0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T6
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T6
        DLNT      MUL       AC0.0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T6
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T6
        DLZT      MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0+D0ARI.0++]
        DLUZT     MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0+D0ARI.0++]
        DLNT      MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0+D0ARI.0++]
        DLUNT     MUL       AC0.0,D0Re0,D0Re0 T6,[D0AR.0+D0ARI.0++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1LbP++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1LbP++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1LbP++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1LbP++],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1LbP++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1LbP++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1LbP++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1LbP++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1LbP++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1LbP++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1LbP++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1LbP++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1.7++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1.7++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1.7++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1.7++],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1.7++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1.7++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1.7++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1.7++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1.7++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1.7++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1.7++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1.7++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1.8++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1.8++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1.8++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.7+A1.8++],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1.8++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1.8++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1.8++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7+A1.8++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1.8++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1.8++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1.8++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7+A1.8++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.7],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.7],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.7],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.7],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.7++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.7++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.7++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.7++],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.7--],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7--],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.7--],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.7--],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.7--],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7--],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.7--],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.7--],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7--]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7--]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7--]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.7--]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7--]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7--]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7--]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.7--]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1LbP++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1LbP++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1LbP++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1LbP++],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1LbP++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1LbP++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1LbP++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1LbP++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1LbP++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1LbP++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1LbP++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1LbP++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1.7++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1.7++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1.7++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1.7++],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1.7++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1.7++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1.7++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1.7++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1.7++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1.7++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1.7++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1.7++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1.8++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1.8++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1.8++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.8+A1.8++],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1.8++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1.8++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1.8++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8+A1.8++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1.8++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1.8++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1.8++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8+A1.8++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.8],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.8],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.8],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.8],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.8++],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8++],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8++],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.8++],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.8++],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8++],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8++],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.8++],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8++]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8++]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8++]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8++]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8++]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8++]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8++]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8++]
        DMWT      MUL       D0Re0,D0Re0,D0Re0 [A1.8--],T6
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8--],T6
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 [A1.8--],T6
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 [A1.8--],T6
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 [A1.8--],T6
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8--],T6
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 [A1.8--],T6
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 [A1.8--],T6
        DMWT      MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8--]
        DLMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8--]
        DUMWT     MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8--]
        DLUMWT    MUL       D0Re0,D0Re0,D0Re0 T6,[A1.8--]
        DMPWT     MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8--]
        DLMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8--]
        DUMPWT    MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8--]
        DLUMPWT   MUL       AC0.0,D0Re0,D0Re0 T6,[A1.8--]
        DUM       MUL       D1Re0,D1Re0,D1Re0
        D         MUL       D1Re0,D1Re0,A1.7
        DUM       MUL       D1Re0,D1Re0,A1.7
        DM        MUL       D1Re0,D1Re0,D0.7
        DM        MUL       D1Re0,D1Re0,A0FrP
        DUM       MUL       D1Re0,D1Re0,[D1BR.1]
        DUM       MUL       D1Re0,D1Re0,[D1BR.1++]
        DLM       MUL       D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        DM        MUL       D1Re0,D1.7,D1Re0
        DM        MUL       D1Re0,D1.7,A1LbP
        DLM       MUL       D1Re0,D1.7,D1.7
        DM        MUL       D1Re0,D1.7,D0Re0
        DM        MUL       D1Re0,D1.7,RD
        DM        MUL       D1Re0,D1.7,[D1BR.1]
        DM        MUL       D1Re0,D1.7,[D1BR.1++]
        D         MUL       D1Re0,D1.7,A0.7
        DUM       MUL       D1Re0,D1.7,A0.7
        DUM       MUL       D1Re0,[D1BR.1],D1Re0
        D         MUL       D1Re0,[D1BR.1],A1.7
        DUM       MUL       D1Re0,[D1BR.1],A1.7
        DM        MUL       D1Re0,[D1BR.1],D0.7
        DM        MUL       D1Re0,[D1BR.1],A0FrP
        DUM       MUL       D1Re0,[D1BR.1],[D1BR.1]
        DUM       MUL       D1Re0,[D1BR.1],[D1BR.1++]
        DLM       MUL       D1Re0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DM        MUL       D1Re0,[D1BR.1++],D1Re0
        DM        MUL       D1Re0,[D1BR.1++],A1LbP
        DLM       MUL       D1Re0,[D1BR.1++],D1.7
        DM        MUL       D1Re0,[D1BR.1++],D0Re0
        DM        MUL       D1Re0,[D1BR.1++],RD
        DM        MUL       D1Re0,[D1BR.1++],[D1BR.1]
        DM        MUL       D1Re0,[D1BR.1++],[D1BR.1++]
        D         MUL       D1Re0,[D1BR.1++],A0.7
        DUM       MUL       D1Re0,[D1BR.1++],A0.7
        DUM       MUL       D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         MUL       D1Re0,[D1BR.1+D1BRI.1++],A1.7
        DUM       MUL       D1Re0,[D1BR.1+D1BRI.1++],A1.7
        DM        MUL       D1Re0,[D1BR.1+D1BRI.1++],D0.7
        DM        MUL       D1Re0,[D1BR.1+D1BRI.1++],A0FrP
        DUM       MUL       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DUM       MUL       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DLM       MUL       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0FrP++],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0FrP++],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0FrP++],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0FrP++],T6
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0FrP++]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0FrP++]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0FrP++]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0FrP++]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0FrP++]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0FrP++]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0FrP++]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0FrP++]
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0.7++],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0.7++],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0.7++],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0.7++],T6
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0.7++]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0.7++]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0.7++]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0.7++]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0.7++]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0.7++]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0.7++]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0.7++]
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0.8++],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0.8++],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0.8++],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0FrP+A0.8++],T6
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0.8++]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0.8++]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0.8++]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP+A0.8++]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0.8++]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0.8++]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0.8++]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP+A0.8++]
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0FrP],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0FrP],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0FrP],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0FrP],T6
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP]
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0FrP++],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP++],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP++],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0FrP++],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0FrP++],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP++],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP++],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0FrP++],T6
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP++]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP++]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP++]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP++]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP++]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP++]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP++]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP++]
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0FrP--],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP--],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0FrP--],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0FrP--],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0FrP--],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP--],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0FrP--],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0FrP--],T6
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP--]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP--]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP--]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0FrP--]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP--]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP--]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP--]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0FrP--]
        DM        MUL       D1.7,D1Re0,D1Re0
        DM        MUL       D1.7,D1Re0,A1LbP
        DLM       MUL       D1.7,D1Re0,D1.7
        DM        MUL       D1.7,D1Re0,D0Re0
        DM        MUL       D1.7,D1Re0,RD
        DM        MUL       D1.7,D1Re0,[D1BR.1]
        DM        MUL       D1.7,D1Re0,[D1BR.1++]
        D         MUL       D1.7,D1Re0,A0.7
        DUM       MUL       D1.7,D1Re0,A0.7
        DMWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T6
        DLMWT     MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T6
        DUMWT     MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T6
        DLUMWT    MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T6
        DMPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0FrP++],T6
        DLMPWT    MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0FrP++],T6
        DUMPWT    MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0FrP++],T6
        DLUMPWT   MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0FrP++],T6
        DMWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0FrP++]
        DLMWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0FrP++]
        DUMWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0FrP++]
        DLUMWT    MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0FrP++]
        DMPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0FrP++]
        DLMPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0FrP++]
        DUMPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0FrP++]
        DLUMPWT   MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0FrP++]
        DUM       MUL       D1.7,D1.7,D1Re0
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0.7++],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0.7++],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0.7++],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0.7++],T6
        D         MUL       D1.7,D1.7,A1.7
        DUM       MUL       D1.7,D1.7,A1.7
        DM        MUL       D1.7,D1.7,D0.7
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0.7++]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0.7++]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0.7++]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0.7++]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0.7++]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0.7++]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0.7++]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0.7++]
        DM        MUL       D1.7,D1.7,A0FrP
        DUM       MUL       D1.7,D1.7,[D1BR.1]
        DUM       MUL       D1.7,D1.7,[D1BR.1++]
        DLM       MUL       D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0.8++],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0.8++],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0.8++],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0.7+A0.8++],T6
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0.8++]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0.8++]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0.8++]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7+A0.8++]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0.8++]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0.8++]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0.8++]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7+A0.8++]
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0.7],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0.7],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0.7],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0.7],T6
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7]
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0.7++],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7++],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7++],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0.7++],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0.7++],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7++],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7++],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0.7++],T6
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7++]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7++]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7++]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7++]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7++]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7++]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7++]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7++]
        DWT       MUL       D1Re0,D1Re0,D1Re0 [A0.7--],T6
        DLWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7--],T6
        DUWT      MUL       D1Re0,D1Re0,D1Re0 [A0.7--],T6
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 [A0.7--],T6
        DPWT      MUL       AC1.0,D1Re0,D1Re0 [A0.7--],T6
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7--],T6
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 [A0.7--],T6
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 [A0.7--],T6
        DWT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7--]
        DLWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7--]
        DUWT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7--]
        DLUWT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.7--]
        DPWT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7--]
        DLPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7--]
        DUPWT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7--]
        DLUPWT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.7--]
        DM        MUL       D1.7,[D1BR.1],D1Re0
        DM        MUL       D1.7,[D1BR.1],A1LbP
        DLM       MUL       D1.7,[D1BR.1],D1.7
        DM        MUL       D1.7,[D1BR.1],D0Re0
        DM        MUL       D1.7,[D1BR.1],RD
        D         MUL       D1.7,[D1BR.1],[D1BR.1]
        D         MUL       D1.7,[D1BR.1],[D1BR.1++]
        D         MUL       D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DUM       MUL       D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DU        MUL       D1.7,[D1BR.1++],D1Re0
        D         MUL       D1.7,[D1BR.1++],D1.7
        DUM       MUL       D1.7,[D1BR.1++],D1.7
        D         MUL       D1.7,[D1BR.1++],D0.7
        D         MUL       D1.7,[D1BR.1++],A0FrP
        DU        MUL       D1.7,[D1BR.1++],[D1BR.1]
        DU        MUL       D1.7,[D1BR.1++],[D1BR.1++]
        DL        MUL       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         MUL       D1.7,[D1BR.1+D1BRI.1++],D1Re0
        D         MUL       D1.7,[D1BR.1+D1BRI.1++],A1LbP
        DL        MUL       D1.7,[D1BR.1+D1BRI.1++],D1.7
        D         MUL       D1.7,[D1BR.1+D1BRI.1++],D0Re0
        D         MUL       D1.7,[D1BR.1+D1BRI.1++],RD
        D         MUL       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         MUL       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         MUL       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DUM       MUL       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0FrP++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0FrP++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0FrP++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0FrP++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0FrP++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0FrP++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0FrP++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0FrP++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0FrP++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0FrP++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0FrP++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0FrP++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0.7++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0.7++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0.7++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0.7++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0.7++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0.7++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0.7++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0.7++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0.7++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0.7++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0.7++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0.7++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0.8++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0.8++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0.8++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A0.8+A0.8++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0.8++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0.8++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0.8++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8+A0.8++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0.8++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0.8++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0.8++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8+A0.8++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A0.8],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A0.8],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A0.8],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A0.8],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A0.8++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A0.8++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A0.8++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A0.8++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A0.8--],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8--],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A0.8--],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A0.8--],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A0.8--],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8--],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A0.8--],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A0.8--],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8--]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8--]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8--]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A0.8--]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8--]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8--]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8--]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A0.8--]
        DUZ       MUL       AC1.0,D1Re0,D1Re0
        DLP       MUL       AC1.0,D1Re0,D1Re0
        DLUP      MUL       AC1.0,D1Re0,D1Re0
        DP        MUL       AC1.0,D1Re0,A1LbP
        DZ        MUL       AC1.0,D1Re0,D1.7
        DUMZ      MUL       AC1.0,D1Re0,D1.7
        DMP       MUL       AC1.0,D1Re0,D1.7
        DUP       MUL       AC1.0,D1Re0,D1.7
        DLUP      MUL       AC1.0,D1Re0,D1.7
        DP        MUL       AC1.0,D1Re0,D0Re0
        DZ        MUL       AC1.0,D1Re0,D0.7
        DUP       MUL       AC1.0,D1Re0,D0.7
        DUZ       MUL       AC1.0,D1Re0,RD
        DLP       MUL       AC1.0,D1Re0,RD
        DLUP      MUL       AC1.0,D1Re0,RD
        DP        MUL       AC1.0,D1Re0,A0FrP
        DZ        MUL       AC1.0,D1Re0,[D1BR.1]
        DP        MUL       AC1.0,D1Re0,[D1BR.1]
        DUP       MUL       AC1.0,D1Re0,[D1BR.1]
        DZ        MUL       AC1.0,D1Re0,[D1BR.1++]
        DP        MUL       AC1.0,D1Re0,[D1BR.1++]
        DUP       MUL       AC1.0,D1Re0,[D1BR.1++]
        DZ        MUL       AC1.0,D1Re0,[D1BR.1+D1BRI.1++]
        DUMZ      MUL       AC1.0,D1Re0,[D1BR.1+D1BRI.1++]
        DMP       MUL       AC1.0,D1Re0,[D1BR.1+D1BRI.1++]
        DUP       MUL       AC1.0,D1Re0,[D1BR.1+D1BRI.1++]
        DLUP      MUL       AC1.0,D1Re0,[D1BR.1+D1BRI.1++]
        DUZ       MUL       AC1.0,D1.7,D1Re0
        DLP       MUL       AC1.0,D1.7,D1Re0
        DLUP      MUL       AC1.0,D1.7,D1Re0
        DP        MUL       AC1.0,D1.7,A1LbP
        DZ        MUL       AC1.0,D1.7,D1.7
        DUMZ      MUL       AC1.0,D1.7,D1.7
        DMP       MUL       AC1.0,D1.7,D1.7
        DUP       MUL       AC1.0,D1.7,D1.7
        DLUP      MUL       AC1.0,D1.7,D1.7
        DP        MUL       AC1.0,D1.7,D0Re0
        DZ        MUL       AC1.0,D1.7,D0.7
        DUP       MUL       AC1.0,D1.7,D0.7
        DUZ       MUL       AC1.0,D1.7,RD
        DLP       MUL       AC1.0,D1.7,RD
        DLUP      MUL       AC1.0,D1.7,RD
        DP        MUL       AC1.0,D1.7,A0FrP
        DZ        MUL       AC1.0,D1.7,[D1BR.1]
        DP        MUL       AC1.0,D1.7,[D1BR.1]
        DUP       MUL       AC1.0,D1.7,[D1BR.1]
        DZ        MUL       AC1.0,D1.7,[D1BR.1++]
        DP        MUL       AC1.0,D1.7,[D1BR.1++]
        DUP       MUL       AC1.0,D1.7,[D1BR.1++]
        DZ        MUL       AC1.0,D1.7,[D1BR.1+D1BRI.1++]
        DUMZ      MUL       AC1.0,D1.7,[D1BR.1+D1BRI.1++]
        DMP       MUL       AC1.0,D1.7,[D1BR.1+D1BRI.1++]
        DUP       MUL       AC1.0,D1.7,[D1BR.1+D1BRI.1++]
        DLUP      MUL       AC1.0,D1.7,[D1BR.1+D1BRI.1++]
        DUZ       MUL       AC1.0,[D1BR.1],D1Re0
        DLP       MUL       AC1.0,[D1BR.1],D1Re0
        DLUP      MUL       AC1.0,[D1BR.1],D1Re0
        DP        MUL       AC1.0,[D1BR.1],A1LbP
        DZ        MUL       AC1.0,[D1BR.1],D1.7
        DUMZ      MUL       AC1.0,[D1BR.1],D1.7
        DMP       MUL       AC1.0,[D1BR.1],D1.7
        DUP       MUL       AC1.0,[D1BR.1],D1.7
        DLUP      MUL       AC1.0,[D1BR.1],D1.7
        DP        MUL       AC1.0,[D1BR.1],D0Re0
        DZ        MUL       AC1.0,[D1BR.1],D0.7
        DUP       MUL       AC1.0,[D1BR.1],D0.7
        DUZ       MUL       AC1.0,[D1BR.1],RD
        DLP       MUL       AC1.0,[D1BR.1],RD
        DLUP      MUL       AC1.0,[D1BR.1],RD
        DP        MUL       AC1.0,[D1BR.1],A0FrP
        DZ        MUL       AC1.0,[D1BR.1],[D1BR.1]
        DP        MUL       AC1.0,[D1BR.1],[D1BR.1]
        DUP       MUL       AC1.0,[D1BR.1],[D1BR.1]
        DZ        MUL       AC1.0,[D1BR.1],[D1BR.1++]
        DP        MUL       AC1.0,[D1BR.1],[D1BR.1++]
        DUP       MUL       AC1.0,[D1BR.1],[D1BR.1++]
        DZ        MUL       AC1.0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DUMZ      MUL       AC1.0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DMP       MUL       AC1.0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DUP       MUL       AC1.0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DLUP      MUL       AC1.0,[D1BR.1],[D1BR.1+D1BRI.1++]
        DUZ       MUL       AC1.0,[D1BR.1++],D1Re0
        DLP       MUL       AC1.0,[D1BR.1++],D1Re0
        DLUP      MUL       AC1.0,[D1BR.1++],D1Re0
        DP        MUL       AC1.0,[D1BR.1++],A1LbP
        DZ        MUL       AC1.0,[D1BR.1++],D1.7
        DUMZ      MUL       AC1.0,[D1BR.1++],D1.7
        DMP       MUL       AC1.0,[D1BR.1++],D1.7
        DUP       MUL       AC1.0,[D1BR.1++],D1.7
        DLUP      MUL       AC1.0,[D1BR.1++],D1.7
        DP        MUL       AC1.0,[D1BR.1++],D0Re0
        DZ        MUL       AC1.0,[D1BR.1++],D0.7
        DUP       MUL       AC1.0,[D1BR.1++],D0.7
        DUZ       MUL       AC1.0,[D1BR.1++],RD
        DLP       MUL       AC1.0,[D1BR.1++],RD
        DLUP      MUL       AC1.0,[D1BR.1++],RD
        DP        MUL       AC1.0,[D1BR.1++],A0FrP
        DZ        MUL       AC1.0,[D1BR.1++],[D1BR.1]
        DP        MUL       AC1.0,[D1BR.1++],[D1BR.1]
        DUP       MUL       AC1.0,[D1BR.1++],[D1BR.1]
        DZ        MUL       AC1.0,[D1BR.1++],[D1BR.1++]
        DP        MUL       AC1.0,[D1BR.1++],[D1BR.1++]
        DUP       MUL       AC1.0,[D1BR.1++],[D1BR.1++]
        DZ        MUL       AC1.0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DUMZ      MUL       AC1.0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DMP       MUL       AC1.0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DUP       MUL       AC1.0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DLUP      MUL       AC1.0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DUZ       MUL       AC1.0,[D1BR.1+D1BRI.1++],D1Re0
        DLP       MUL       AC1.0,[D1BR.1+D1BRI.1++],D1Re0
        DLUP      MUL       AC1.0,[D1BR.1+D1BRI.1++],D1Re0
        DP        MUL       AC1.0,[D1BR.1+D1BRI.1++],A1LbP
        DZ        MUL       AC1.0,[D1BR.1+D1BRI.1++],D1.7
        DUMZ      MUL       AC1.0,[D1BR.1+D1BRI.1++],D1.7
        DMP       MUL       AC1.0,[D1BR.1+D1BRI.1++],D1.7
        DUP       MUL       AC1.0,[D1BR.1+D1BRI.1++],D1.7
        DLUP      MUL       AC1.0,[D1BR.1+D1BRI.1++],D1.7
        DP        MUL       AC1.0,[D1BR.1+D1BRI.1++],D0Re0
        DZ        MUL       AC1.0,[D1BR.1+D1BRI.1++],D0.7
        DUP       MUL       AC1.0,[D1BR.1+D1BRI.1++],D0.7
        DUZ       MUL       AC1.0,[D1BR.1+D1BRI.1++],RD
        DLP       MUL       AC1.0,[D1BR.1+D1BRI.1++],RD
        DLUP      MUL       AC1.0,[D1BR.1+D1BRI.1++],RD
        DP        MUL       AC1.0,[D1BR.1+D1BRI.1++],A0FrP
        DZ        MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DP        MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DUP       MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DZ        MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DP        MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DUP       MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DZ        MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DUMZ      MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DMP       MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DUP       MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLUP      MUL       AC1.0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1LbP++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1LbP++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1LbP++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1LbP++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1LbP++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1LbP++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1LbP++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1LbP++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1LbP++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1LbP++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1LbP++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1LbP++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1.7++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1.7++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1.7++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1.7++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1.7++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1.7++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1.7++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1.7++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1.7++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1.7++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1.7++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1.7++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1.8++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1.8++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1.8++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1LbP+A1.8++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1.8++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1.8++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1.8++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP+A1.8++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1.8++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1.8++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1.8++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP+A1.8++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1LbP],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1LbP],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1LbP],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1LbP],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1LbP++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1LbP++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1LbP++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1LbP++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1LbP--],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP--],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1LbP--],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1LbP--],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1LbP--],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP--],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1LbP--],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1LbP--],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP--]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP--]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP--]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1LbP--]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP--]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP--]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP--]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1LbP--]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1LbP++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1LbP++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1LbP++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1LbP++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1LbP++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1LbP++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1LbP++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1LbP++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1LbP++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1LbP++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1LbP++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1LbP++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1.7++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1.7++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1.7++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1.7++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1.7++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1.7++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1.7++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1.7++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1.7++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1.7++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1.7++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1.7++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1.8++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1.8++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1.8++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.7+A1.8++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1.8++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1.8++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1.8++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7+A1.8++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1.8++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1.8++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1.8++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7+A1.8++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.7],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.7],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.7],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.7],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.7++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.7++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.7++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.7++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.7--],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7--],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.7--],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.7--],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.7--],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7--],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.7--],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.7--],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7--]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7--]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7--]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.7--]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7--]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7--]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7--]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.7--]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1LbP++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1LbP++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1LbP++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1LbP++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1LbP++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1LbP++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1LbP++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1LbP++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1LbP++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1LbP++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1LbP++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1LbP++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1.7++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1.7++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1.7++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1.7++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1.7++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1.7++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1.7++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1.7++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1.7++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1.7++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1.7++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1.7++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1.8++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1.8++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1.8++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.8+A1.8++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1.8++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1.8++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1.8++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8+A1.8++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1.8++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1.8++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1.8++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8+A1.8++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.8],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.8],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.8],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.8],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.8++],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8++],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8++],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.8++],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.8++],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8++],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8++],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.8++],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8++]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8++]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8++]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8++]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8++]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8++]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8++]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8++]
        DMT       MUL       D1Re0,D1Re0,D1Re0 [A1.8--],T6
        DLMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8--],T6
        DUMT      MUL       D1Re0,D1Re0,D1Re0 [A1.8--],T6
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 [A1.8--],T6
        DMPT      MUL       AC1.0,D1Re0,D1Re0 [A1.8--],T6
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8--],T6
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 [A1.8--],T6
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 [A1.8--],T6
        DMT       MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8--]
        DLMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8--]
        DUMT      MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8--]
        DLUMT     MUL       D1Re0,D1Re0,D1Re0 T6,[A1.8--]
        DMPT      MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8--]
        DLMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8--]
        DUMPT     MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8--]
        DLUMPT    MUL       AC1.0,D1Re0,D1Re0 T6,[A1.8--]
        DU        MUL       [D1BW.1],D1Re0,D1Re0
        D         MUL       [D1BW.1],D1Re0,D1.7
        DUM       MUL       [D1BW.1],D1Re0,D1.7
        D         MUL       [D1BW.1],D1Re0,D0.7
        D         MUL       [D1BW.1],D1Re0,A0FrP
        DU        MUL       [D1BW.1],D1Re0,[D1BR.1]
        DU        MUL       [D1BW.1],D1Re0,[D1BR.1++]
        DL        MUL       [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        D         MUL       [D1BW.1],D1.7,D1Re0
        D         MUL       [D1BW.1],D1.7,A1LbP
        DL        MUL       [D1BW.1],D1.7,D1.7
        D         MUL       [D1BW.1],D1.7,D0Re0
        D         MUL       [D1BW.1],D1.7,RD
        D         MUL       [D1BW.1],D1.7,[D1BR.1]
        D         MUL       [D1BW.1],D1.7,[D1BR.1++]
        D         MUL       [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DUM       MUL       [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DLT       MUL       D1Re0,D1Re0,D1Re0 [D1BW.1],T6
        DLUT      MUL       D1Re0,D1Re0,D1Re0 [D1BW.1],T6
        DLPT      MUL       AC1.0,D1Re0,D1Re0 [D1BW.1],T6
        DLUPT     MUL       AC1.0,D1Re0,D1Re0 [D1BW.1],T6
        DLT       MUL       D1Re0,D1Re0,D1Re0 T6,[D1BR.1]
        DLUT      MUL       D1Re0,D1Re0,D1Re0 T6,[D1BR.1]
        DLPT      MUL       AC1.0,D1Re0,D1Re0 T6,[D1BR.1]
        DLUPT     MUL       AC1.0,D1Re0,D1Re0 T6,[D1BR.1]
        DU        MUL       [D1BW.1],[D1BR.1],D1Re0
        D         MUL       [D1BW.1],[D1BR.1],D1.7
        DUM       MUL       [D1BW.1],[D1BR.1],D1.7
        D         MUL       [D1BW.1],[D1BR.1],D0.7
        D         MUL       [D1BW.1],[D1BR.1],A0FrP
        DU        MUL       [D1BW.1],[D1BR.1],[D1BR.1]
        DU        MUL       [D1BW.1],[D1BR.1],[D1BR.1++]
        DL        MUL       [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         MUL       [D1BW.1],[D1BR.1++],D1Re0
        D         MUL       [D1BW.1],[D1BR.1++],A1LbP
        DL        MUL       [D1BW.1],[D1BR.1++],D1.7
        D         MUL       [D1BW.1],[D1BR.1++],D0Re0
        D         MUL       [D1BW.1],[D1BR.1++],RD
        D         MUL       [D1BW.1],[D1BR.1++],[D1BR.1]
        D         MUL       [D1BW.1],[D1BR.1++],[D1BR.1++]
        D         MUL       [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DUM       MUL       [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DU        MUL       [D1BW.1],[D1BR.1+D1BRI.1++],D1Re0
        D         MUL       [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DUM       MUL       [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        D         MUL       [D1BW.1],[D1BR.1+D1BRI.1++],D0.7
        D         MUL       [D1BW.1],[D1BR.1+D1BRI.1++],A0FrP
        DU        MUL       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        DU        MUL       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        MUL       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         MUL       [D1BW.1++],D1Re0,D1Re0
        D         MUL       [D1BW.1++],D1Re0,A1LbP
        DL        MUL       [D1BW.1++],D1Re0,D1.7
        D         MUL       [D1BW.1++],D1Re0,D0Re0
        D         MUL       [D1BW.1++],D1Re0,RD
        D         MUL       [D1BW.1++],D1Re0,[D1BR.1]
        D         MUL       [D1BW.1++],D1Re0,[D1BR.1++]
        D         MUL       [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DUM       MUL       [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        DU        MUL       [D1BW.1++],D1.7,D1Re0
        D         MUL       [D1BW.1++],D1.7,D1.7
        DUM       MUL       [D1BW.1++],D1.7,D1.7
        D         MUL       [D1BW.1++],D1.7,D0.7
        D         MUL       [D1BW.1++],D1.7,A0FrP
        DU        MUL       [D1BW.1++],D1.7,[D1BR.1]
        DU        MUL       [D1BW.1++],D1.7,[D1BR.1++]
        DL        MUL       [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DT        MUL       D1Re0,D1Re0,D1Re0 [D1BW.1++],T6
        DUT       MUL       D1Re0,D1Re0,D1Re0 [D1BW.1++],T6
        DPT       MUL       AC1.0,D1Re0,D1Re0 [D1BW.1++],T6
        DUPT      MUL       AC1.0,D1Re0,D1Re0 [D1BW.1++],T6
        DT        MUL       D1Re0,D1Re0,D1Re0 T6,[D1BR.1++]
        DUT       MUL       D1Re0,D1Re0,D1Re0 T6,[D1BR.1++]
        DPT       MUL       AC1.0,D1Re0,D1Re0 T6,[D1BR.1++]
        DUPT      MUL       AC1.0,D1Re0,D1Re0 T6,[D1BR.1++]
        D         MUL       [D1BW.1++],[D1BR.1],D1Re0
        D         MUL       [D1BW.1++],[D1BR.1],A1LbP
        DL        MUL       [D1BW.1++],[D1BR.1],D1.7
        D         MUL       [D1BW.1++],[D1BR.1],D0Re0
        D         MUL       [D1BW.1++],[D1BR.1],RD
        D         MUL       [D1BW.1++],[D1BR.1],[D1BR.1]
        D         MUL       [D1BW.1++],[D1BR.1],[D1BR.1++]
        D         MUL       [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DUM       MUL       [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DU        MUL       [D1BW.1++],[D1BR.1++],D1Re0
        D         MUL       [D1BW.1++],[D1BR.1++],D1.7
        DUM       MUL       [D1BW.1++],[D1BR.1++],D1.7
        D         MUL       [D1BW.1++],[D1BR.1++],D0.7
        D         MUL       [D1BW.1++],[D1BR.1++],A0FrP
        DU        MUL       [D1BW.1++],[D1BR.1++],[D1BR.1]
        DU        MUL       [D1BW.1++],[D1BR.1++],[D1BR.1++]
        DL        MUL       [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         MUL       [D1BW.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         MUL       [D1BW.1++],[D1BR.1+D1BRI.1++],A1LbP
        DL        MUL       [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        D         MUL       [D1BW.1++],[D1BR.1+D1BRI.1++],D0Re0
        D         MUL       [D1BW.1++],[D1BR.1+D1BRI.1++],RD
        D         MUL       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         MUL       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        D         MUL       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DUM       MUL       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DU        MUL       [D1BW.1+D1BWI.1++],D1Re0,D1Re0
        D         MUL       [D1BW.1+D1BWI.1++],D1Re0,D1.7
        DUM       MUL       [D1BW.1+D1BWI.1++],D1Re0,D1.7
        D         MUL       [D1BW.1+D1BWI.1++],D1Re0,D0.7
        D         MUL       [D1BW.1+D1BWI.1++],D1Re0,A0FrP
        DU        MUL       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        DU        MUL       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        DL        MUL       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        D         MUL       [D1BW.1+D1BWI.1++],D1.7,D1Re0
        D         MUL       [D1BW.1+D1BWI.1++],D1.7,A1LbP
        DL        MUL       [D1BW.1+D1BWI.1++],D1.7,D1.7
        D         MUL       [D1BW.1+D1BWI.1++],D1.7,D0Re0
        D         MUL       [D1BW.1+D1BWI.1++],D1.7,RD
        D         MUL       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        D         MUL       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1++]
        D         MUL       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DUM       MUL       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DLT       MUL       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T6
        DLUT      MUL       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T6
        DLPT      MUL       AC1.0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T6
        DLUPT     MUL       AC1.0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T6
        DLT       MUL       D1Re0,D1Re0,D1Re0 T6,[D1BR.1+D1BRI.1++]
        DLUT      MUL       D1Re0,D1Re0,D1Re0 T6,[D1BR.1+D1BRI.1++]
        DLPT      MUL       AC1.0,D1Re0,D1Re0 T6,[D1BR.1+D1BRI.1++]
        DLUPT     MUL       AC1.0,D1Re0,D1Re0 T6,[D1BR.1+D1BRI.1++]
        DU        MUL       [D1BW.1+D1BWI.1++],[D1BR.1],D1Re0
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        DUM       MUL       [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1],D0.7
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1],A0FrP
        DU        MUL       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        DU        MUL       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1++]
        DL        MUL       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1++],D1Re0
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1++],A1LbP
        DL        MUL       [D1BW.1+D1BWI.1++],[D1BR.1++],D1.7
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1++],D0Re0
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1++],RD
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DUM       MUL       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        DU        MUL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1Re0
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        DUM       MUL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D0.7
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],A0FrP
        DU        MUL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        DU        MUL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        MUL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         MUL       D0Re0,D0Re0,#0x7fff
        D         MUL       D0.7,D0.7,#-0x8000
        D         MUL       [D0AW.0],[D0AR.0],#-0x1
        D         MUL       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],#0x7fff
        D         MUL       D1Re0,D1Re0,#-0x8000
        D         MUL       D1.7,D1.7,#-0x1
        D         MUL       [D1BW.1++],[D1BR.1++],#0x7fff
        D         MUL       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],#-0x8000
        DL        FFB       D0Re0,D0Re0
        DL        ABS       D0Re0,D0Re0
        D         NMIN      D0Re0,D0Re0,D0.7
        DL        CMP       D0Re0,[D0AR.0]
        DL        CMP       D0Re0,[D0AR.0++]
        DL        CMP       D0Re0,[D0AR.0+D0ARI.0++]
        D         CMP       D0Re0,A0.7
        D         MAX       D0Re0,D0.7,D0Re0
        D         CMP       D0.7,D0.7
        DL        NMIN      D0Re0,D0.7,D0.7
        D         NMIN      D0Re0,D0.7,[D0AR.0]
        D         NMIN      D0Re0,D0.7,[D0AR.0++]
        D         NMIN      D0Re0,D0.7,[D0AR.0+D0ARI.0++]
        D         NORM      D0Re0,[D0AR.0]
        DL        MIN       D0Re0,[D0AR.0],D0Re0
        D         MIN       D0Re0,[D0AR.0],D0.7
        D         CMP       [D0AR.0],[D0AR.0]
        DL        NMIN      D0Re0,[D0AR.0],[D0AR.0]
        DL        NMIN      D0Re0,[D0AR.0],[D0AR.0++]
        DL        NMIN      D0Re0,[D0AR.0],[D0AR.0+D0ARI.0++]
        DL        NORM      D0Re0,[D0AR.0++]
        DL        NMIN      D0Re0,[D0AR.0++],D0Re0
        DL        MIN       D0Re0,[D0AR.0++],D0.7
        D         MIN       D0Re0,[D0AR.0++],[D0AR.0]
        D         MIN       D0Re0,[D0AR.0++],[D0AR.0++]
        D         MIN       D0Re0,[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         CMP       [D0AR.0+D0ARI.0++],D0Re0
        D         ABS       D0Re0,[D0AR.0+D0ARI.0++]
        D         CMP       [D0AR.0+D0ARI.0++],A1.7
        D         CMP       [D0AR.0+D0ARI.0++],D1Re0
        DL        MIN       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        MIN       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DL        MIN       D0Re0,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DWT       FFB       D0Re0,D0Re0 [A0FrP+A0FrP++],T7
        DLWT      NORM      D0Re0,D0Re0 [A0FrP+A0FrP++],T7
        DWT       NMIN      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T7
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 [A0FrP+A0FrP++],T7
        DLWT      CMP       D0Re0,D0Re0 T7,[A0FrP+A0FrP++]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP+A0FrP++]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP+A0FrP++]
        DWT       FFB       D0Re0,D0Re0 [A0FrP+A0.7++],T7
        DLWT      NORM      D0Re0,D0Re0 [A0FrP+A0.7++],T7
        DWT       NMIN      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T7
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 [A0FrP+A0.7++],T7
        DLWT      CMP       D0Re0,D0Re0 T7,[A0FrP+A0.7++]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP+A0.7++]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP+A0.7++]
        DWT       FFB       D0Re0,D0Re0 [A0FrP+A0.8++],T7
        DLWT      NORM      D0Re0,D0Re0 [A0FrP+A0.8++],T7
        DWT       NMIN      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T7
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 [A0FrP+A0.8++],T7
        DLWT      CMP       D0Re0,D0Re0 T7,[A0FrP+A0.8++]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP+A0.8++]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP+A0.8++]
        DWT       FFB       D0Re0,D0Re0 [A0FrP],T7
        DLWT      NORM      D0Re0,D0Re0 [A0FrP],T7
        DWT       NMIN      D0Re0,D0Re0,D0Re0 [A0FrP],T7
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 [A0FrP],T7
        DLWT      CMP       D0Re0,D0Re0 T7,[A0FrP]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP]
        DWT       FFB       D0Re0,D0Re0 [A0FrP++],T7
        DLWT      NORM      D0Re0,D0Re0 [A0FrP++],T7
        DWT       NMIN      D0Re0,D0Re0,D0Re0 [A0FrP++],T7
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 [A0FrP++],T7
        DLWT      CMP       D0Re0,D0Re0 T7,[A0FrP++]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP++]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP++]
        DWT       FFB       D0Re0,D0Re0 [A0FrP--],T7
        DLWT      NORM      D0Re0,D0Re0 [A0FrP--],T7
        DWT       NMIN      D0Re0,D0Re0,D0Re0 [A0FrP--],T7
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 [A0FrP--],T7
        DLWT      CMP       D0Re0,D0Re0 T7,[A0FrP--]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP--]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A0FrP--]
        DL        NORM      D0.7,D0Re0
        DL        NMIN      D0.7,D0Re0,D0Re0
        D         MAX       D0.7,D0Re0,[D0AR.0]
        DL        MIN       D0.7,D0Re0,[D0AR.0++]
        DL        NMIN      D0.7,D0Re0,[D0AR.0+D0ARI.0++]
        DLWT      CMP       D0Re0,D0Re0 [A0.7+A0FrP++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A0.7+A0FrP++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A0.7+A0FrP++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A0.7+A0FrP++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.7+A0FrP++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.7+A0FrP++]
        D         NMIN      D0.7,D0.7,D0Re0
        DWT       FFB       D0Re0,D0Re0 [A0.7+A0.7++],T7
        DLWT      NORM      D0Re0,D0Re0 [A0.7+A0.7++],T7
        DWT       ABS       D0Re0,D0Re0 [A0.7+A0.7++],T7
        DLT       MAX       D0Re0,D0Re0,D0Re0 [A0.7+A0.7++],T7
        D         MAX       D0.7,D0.7,[D0AR.0]
        DL        MIN       D0.7,D0.7,[D0AR.0++]
        DL        NMIN      D0.7,D0.7,[D0AR.0+D0ARI.0++]
        DLWT      CMP       D0Re0,D0Re0 T7,[A0.7+A0.7++]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A0.7+A0.7++]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A0.7+A0.7++]
        DWT       FFB       D0Re0,D0Re0 [A0.7+A0.8++],T7
        DLWT      NORM      D0Re0,D0Re0 [A0.7+A0.8++],T7
        DWT       NMIN      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T7
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 [A0.7+A0.8++],T7
        DLWT      CMP       D0Re0,D0Re0 T7,[A0.7+A0.8++]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A0.7+A0.8++]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A0.7+A0.8++]
        DL        NORM      D0.7,[D0AR.0]
        DL        NMIN      D0.7,[D0AR.0],D0Re0
        DLWT      CMP       D0Re0,D0Re0 [A0.7],T7
        D         MAX       D0.7,[D0AR.0],D0.7
        DL        MIN       D0.7,[D0AR.0],D0.7
        DL        NMIN      D0.7,[D0AR.0],D0.7
        DL        NMIN      D0.7,[D0AR.0],[D0AR.0]
        D         MAX       D0.7,[D0AR.0],[D0AR.0+D0ARI.0++]
        DWT       FFB       D0Re0,D0Re0 T7,[A0.7]
        DLWT      NORM      D0Re0,D0Re0 T7,[A0.7]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.7]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.7]
        D         NMIN      D0.7,[D0AR.0++],D0Re0
        DWT       FFB       D0Re0,D0Re0 [A0.7++],T7
        DLWT      NORM      D0Re0,D0Re0 [A0.7++],T7
        DWT       ABS       D0Re0,D0Re0 [A0.7++],T7
        DLT       MAX       D0Re0,D0Re0,D0Re0 [A0.7++],T7
        D         MAX       D0.7,[D0AR.0++],[D0AR.0]
        DL        MIN       D0.7,[D0AR.0++],[D0AR.0++]
        DL        NMIN      D0.7,[D0AR.0++],[D0AR.0+D0ARI.0++]
        DLWT      CMP       D0Re0,D0Re0 T7,[A0.7++]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A0.7++]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A0.7++]
        DL        NORM      D0.7,[D0AR.0+D0ARI.0++]
        DL        NMIN      D0.7,[D0AR.0+D0ARI.0++],D0Re0
        D         MAX       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0]
        DL        MIN       D0.7,[D0AR.0+D0ARI.0++],[D0AR.0++]
        DL        NMIN      D0.7,[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLWT      CMP       D0Re0,D0Re0 [A0.7--],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A0.7--],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A0.7--],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A0.7--]
        DLWT      NORM      D0Re0,D0Re0 T7,[A0.7--]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.7--]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.7--]
        DLWT      CMP       D0Re0,D0Re0 [A0.8+A0FrP++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A0.8+A0FrP++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A0.8+A0FrP++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A0.8+A0FrP++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8+A0FrP++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8+A0FrP++]
        DLWT      CMP       D0Re0,D0Re0 [A0.8+A0.7++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A0.8+A0.7++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A0.8+A0.7++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A0.8+A0.7++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8+A0.7++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8+A0.7++]
        DLWT      CMP       D0Re0,D0Re0 [A0.8+A0.8++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A0.8+A0.8++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A0.8+A0.8++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A0.8+A0.8++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8+A0.8++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8+A0.8++]
        DLWT      CMP       D0Re0,D0Re0 [A0.8],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A0.8],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A0.8],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A0.8]
        DLWT      NORM      D0Re0,D0Re0 T7,[A0.8]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8]
        DLWT      CMP       D0Re0,D0Re0 [A0.8++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A0.8++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A0.8++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A0.8++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A0.8++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8++]
        DLWT      CMP       D0Re0,D0Re0 [A0.8--],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A0.8--],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A0.8--],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A0.8--]
        DLWT      NORM      D0Re0,D0Re0 T7,[A0.8--]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8--]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A0.8--]
        D         NMIN      [D0AW.0],D0Re0,D0Re0
        DL        MIN       [D0AW.0],D0Re0,D0.7
        DL        NMIN      [D0AW.0],D0Re0,[D0AR.0]
        D         MAX       [D0AW.0],D0Re0,[D0AR.0+D0ARI.0++]
        DL        NORM      [D0AW.0],D0.7
        DL        NMIN      [D0AW.0],D0.7,D0Re0
        D         MAX       [D0AW.0],D0.7,[D0AR.0]
        DL        MIN       [D0AW.0],D0.7,[D0AR.0++]
        DL        NMIN      [D0AW.0],D0.7,[D0AR.0+D0ARI.0++]
        D         NMIN      [D0AW.0],[D0AR.0],D0Re0
        DL        MIN       [D0AW.0],[D0AR.0],D0.7
        DL        NMIN      [D0AW.0],[D0AR.0],[D0AR.0]
        D         MAX       [D0AW.0],[D0AR.0],[D0AR.0+D0ARI.0++]
        DL        NORM      [D0AW.0],[D0AR.0++]
        DL        NMIN      [D0AW.0],[D0AR.0++],D0Re0
        D         MAX       [D0AW.0],[D0AR.0++],[D0AR.0]
        DL        MIN       [D0AW.0],[D0AR.0++],[D0AR.0++]
        DL        NMIN      [D0AW.0],[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         NMIN      [D0AW.0],[D0AR.0+D0ARI.0++],D0Re0
        DL        MIN       [D0AW.0],[D0AR.0+D0ARI.0++],D0.7
        DL        NMIN      [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0]
        D         MAX       [D0AW.0],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLT       CMP       D0Re0,D0Re0 [D0AW.0],T7
        DLT       MAX       D0Re0,D0Re0,D0Re0 [D0AW.0],T7
        DLT       NORM      D0Re0,D0Re0 T7,[D0AR.0]
        DLT       NMIN      D0Re0,D0Re0,D0Re0 T7,[D0AR.0]
        D         NMIN      [D0AW.0++],D0Re0,D0Re0
        DL        MIN       [D0AW.0++],D0Re0,D0.7
        DL        NMIN      [D0AW.0++],D0Re0,[D0AR.0]
        D         MAX       [D0AW.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DWT       FFB       D0Re0,D0Re0 [A1LbP+A1LbP++],T7
        DLWT      NORM      D0Re0,D0Re0 [A1LbP+A1LbP++],T7
        DWT       NMIN      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T7
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 [A1LbP+A1LbP++],T7
        DLWT      CMP       D0Re0,D0Re0 T7,[A1LbP+A1LbP++]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A1LbP+A1LbP++]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A1LbP+A1LbP++]
        DL        NORM      [D0AW.0++],D0.7
        DL        NMIN      [D0AW.0++],D0.7,D0Re0
        DLWT      CMP       D0Re0,D0Re0 [A1LbP+A1.7++],T7
        D         MAX       [D0AW.0++],D0.7,D0.7
        DL        MIN       [D0AW.0++],D0.7,D0.7
        DL        NMIN      [D0AW.0++],D0.7,D0.7
        DL        NMIN      [D0AW.0++],D0.7,[D0AR.0]
        D         MAX       [D0AW.0++],D0.7,[D0AR.0+D0ARI.0++]
        DWT       FFB       D0Re0,D0Re0 T7,[A1LbP+A1.7++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1LbP+A1.7++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1LbP+A1.7++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1LbP+A1.7++]
        DLWT      CMP       D0Re0,D0Re0 [A1LbP+A1.8++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1LbP+A1.8++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1LbP+A1.8++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1LbP+A1.8++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1LbP+A1.8++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1LbP+A1.8++]
        D         NMIN      [D0AW.0++],[D0AR.0],D0Re0
        DWT       FFB       D0Re0,D0Re0 [A1LbP],T7
        DLWT      NORM      D0Re0,D0Re0 [A1LbP],T7
        DWT       ABS       D0Re0,D0Re0 [A1LbP],T7
        DLT       MAX       D0Re0,D0Re0,D0Re0 [A1LbP],T7
        D         MAX       [D0AW.0++],[D0AR.0],[D0AR.0]
        DL        MIN       [D0AW.0++],[D0AR.0],[D0AR.0++]
        DL        NMIN      [D0AW.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DLWT      CMP       D0Re0,D0Re0 T7,[A1LbP]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A1LbP]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A1LbP]
        DL        NORM      [D0AW.0++],[D0AR.0++]
        DL        NMIN      [D0AW.0++],[D0AR.0++],D0Re0
        DLWT      CMP       D0Re0,D0Re0 [A1LbP++],T7
        D         MAX       [D0AW.0++],[D0AR.0++],D0.7
        DL        MIN       [D0AW.0++],[D0AR.0++],D0.7
        DL        NMIN      [D0AW.0++],[D0AR.0++],D0.7
        DL        NMIN      [D0AW.0++],[D0AR.0++],[D0AR.0]
        D         MAX       [D0AW.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        DWT       FFB       D0Re0,D0Re0 T7,[A1LbP++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1LbP++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1LbP++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1LbP++]
        D         NMIN      [D0AW.0++],[D0AR.0+D0ARI.0++],D0Re0
        DL        MIN       [D0AW.0++],[D0AR.0+D0ARI.0++],D0.7
        DL        NMIN      [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        D         MAX       [D0AW.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DWT       FFB       D0Re0,D0Re0 [A1LbP--],T7
        DLWT      NORM      D0Re0,D0Re0 [A1LbP--],T7
        DWT       NMIN      D0Re0,D0Re0,D0Re0 [A1LbP--],T7
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 [A1LbP--],T7
        DLWT      CMP       D0Re0,D0Re0 T7,[A1LbP--]
        DWT       MAX       D0Re0,D0Re0,D0Re0 T7,[A1LbP--]
        DLWT      MAX       D0Re0,D0Re0,D0Re0 T7,[A1LbP--]
        DLT       CMP       D0Re0,D0Re0 [D0AW.0++],T7
        DLT       MAX       D0Re0,D0Re0,D0Re0 [D0AW.0++],T7
        DLT       NORM      D0Re0,D0Re0 T7,[D0AR.0++]
        DLT       NMIN      D0Re0,D0Re0,D0Re0 T7,[D0AR.0++]
        D         NMIN      [D0AW.0+D0AWI.0++],D0Re0,D0Re0
        DL        MIN       [D0AW.0+D0AWI.0++],D0Re0,D0.7
        DL        NMIN      [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0]
        D         MAX       [D0AW.0+D0AWI.0++],D0Re0,[D0AR.0+D0ARI.0++]
        DL        NORM      [D0AW.0+D0AWI.0++],D0.7
        DL        NMIN      [D0AW.0+D0AWI.0++],D0.7,D0Re0
        D         MAX       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0]
        DL        MIN       [D0AW.0+D0AWI.0++],D0.7,[D0AR.0++]
        DL        NMIN      [D0AW.0+D0AWI.0++],D0.7,[D0AR.0+D0ARI.0++]
        D         NMIN      [D0AW.0+D0AWI.0++],[D0AR.0],D0Re0
        DL        MIN       [D0AW.0+D0AWI.0++],[D0AR.0],D0.7
        DL        NMIN      [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0]
        D         MAX       [D0AW.0+D0AWI.0++],[D0AR.0],[D0AR.0+D0ARI.0++]
        DL        NORM      [D0AW.0+D0AWI.0++],[D0AR.0++]
        DL        NMIN      [D0AW.0+D0AWI.0++],[D0AR.0++],D0Re0
        D         MAX       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0]
        DL        MIN       [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0++]
        DL        NMIN      [D0AW.0+D0AWI.0++],[D0AR.0++],[D0AR.0+D0ARI.0++]
        D         NMIN      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0Re0
        DL        MIN       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],D0.7
        DL        NMIN      [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0]
        D         MAX       [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++],[D0AR.0+D0ARI.0++]
        DLT       CMP       D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T7
        DLT       MAX       D0Re0,D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T7
        DLT       NORM      D0Re0,D0Re0 T7,[D0AR.0+D0ARI.0++]
        DLT       NMIN      D0Re0,D0Re0,D0Re0 T7,[D0AR.0+D0ARI.0++]
        DLWT      CMP       D0Re0,D0Re0 [A1.7+A1LbP++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.7+A1LbP++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.7+A1LbP++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.7+A1LbP++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7+A1LbP++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7+A1LbP++]
        DLWT      CMP       D0Re0,D0Re0 [A1.7+A1.7++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.7+A1.7++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.7+A1.7++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.7+A1.7++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7+A1.7++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7+A1.7++]
        DLWT      CMP       D0Re0,D0Re0 [A1.7+A1.8++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.7+A1.8++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.7+A1.8++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.7+A1.8++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7+A1.8++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7+A1.8++]
        DLWT      CMP       D0Re0,D0Re0 [A1.7],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.7],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.7],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.7]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.7]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7]
        DLWT      CMP       D0Re0,D0Re0 [A1.7++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.7++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.7++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.7++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.7++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7++]
        DLWT      CMP       D0Re0,D0Re0 [A1.7--],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.7--],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.7--],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.7--]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.7--]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7--]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.7--]
        DLWT      CMP       D0Re0,D0Re0 [A1.8+A1LbP++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.8+A1LbP++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.8+A1LbP++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.8+A1LbP++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8+A1LbP++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8+A1LbP++]
        DLWT      CMP       D0Re0,D0Re0 [A1.8+A1.7++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.8+A1.7++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.8+A1.7++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.8+A1.7++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8+A1.7++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8+A1.7++]
        DLWT      CMP       D0Re0,D0Re0 [A1.8+A1.8++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.8+A1.8++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.8+A1.8++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.8+A1.8++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8+A1.8++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8+A1.8++]
        DLWT      CMP       D0Re0,D0Re0 [A1.8],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.8],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.8],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.8]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.8]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8]
        DLWT      CMP       D0Re0,D0Re0 [A1.8++],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.8++],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.8++],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.8++]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.8++]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8++]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8++]
        DLWT      CMP       D0Re0,D0Re0 [A1.8--],T7
        DWT       MAX       D0Re0,D0Re0,D0Re0 [A1.8--],T7
        DLWT      MAX       D0Re0,D0Re0,D0Re0 [A1.8--],T7
        DWT       FFB       D0Re0,D0Re0 T7,[A1.8--]
        DLWT      NORM      D0Re0,D0Re0 T7,[A1.8--]
        DWT       NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8--]
        DLWT      NMIN      D0Re0,D0Re0,D0Re0 T7,[A1.8--]
        D         MAX       D1Re0,D1Re0,D1Re0
        D         CMP       D1Re0,D1.7
        DL        NMIN      D1Re0,D1Re0,D1.7
        D         MAX       D1Re0,D1Re0,[D1BR.1]
        D         MAX       D1Re0,D1Re0,[D1BR.1++]
        D         MIN       D1Re0,D1Re0,[D1BR.1+D1BRI.1++]
        D         NORM      D1Re0,D1.7
        DL        MIN       D1Re0,D1.7,D1Re0
        D         MIN       D1Re0,D1.7,D1.7
        D         CMP       D1.7,RD
        DL        MAX       D1Re0,D1.7,[D1BR.1]
        DL        MAX       D1Re0,D1.7,[D1BR.1++]
        DL        MIN       D1Re0,D1.7,[D1BR.1+D1BRI.1++]
        DL        NORM      D1Re0,[D1BR.1]
        DL        NMIN      D1Re0,[D1BR.1],D1Re0
        DL        MIN       D1Re0,[D1BR.1],D1.7
        DL        CMP       [D1BR.1],[D1BR.1]
        DL        CMP       [D1BR.1],[D1BR.1++]
        D         CMP       [D1BR.1],A0.7
        D         CMP       [D1BR.1++],D1Re0
        D         ABS       D1Re0,[D1BR.1++]
        D         CMP       [D1BR.1++],A1.7
        D         CMP       [D1BR.1++],D0Re0
        D         NMIN      D1Re0,[D1BR.1++],[D1BR.1]
        D         NMIN      D1Re0,[D1BR.1++],[D1BR.1++]
        D         MAX       D1Re0,[D1BR.1++],[D1BR.1+D1BRI.1++]
        DL        CMP       [D1BR.1+D1BRI.1++],D1Re0
        DL        MAX       D1Re0,[D1BR.1+D1BRI.1++],D1Re0
        D         MAX       D1Re0,[D1BR.1+D1BRI.1++],D1.7
        D         CMP       [D1BR.1+D1BRI.1++],A0FrP
        DL        NMIN      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1]
        DL        NMIN      D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1++]
        DL        MAX       D1Re0,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DLT       CMP       D1Re0,D1Re0 [A0FrP+A0FrP++],T7
        DT        MAX       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T7
        DLT       MAX       D1Re0,D1Re0,D1Re0 [A0FrP+A0FrP++],T7
        DT        FFB       D1Re0,D1Re0 T7,[A0FrP+A0FrP++]
        DLT       NORM      D1Re0,D1Re0 T7,[A0FrP+A0FrP++]
        DT        NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP+A0FrP++]
        DLT       NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP+A0FrP++]
        DLT       CMP       D1Re0,D1Re0 [A0FrP+A0.7++],T7
        DT        MAX       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T7
        DLT       MAX       D1Re0,D1Re0,D1Re0 [A0FrP+A0.7++],T7
        DT        FFB       D1Re0,D1Re0 T7,[A0FrP+A0.7++]
        DLT       NORM      D1Re0,D1Re0 T7,[A0FrP+A0.7++]
        DT        NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP+A0.7++]
        DLT       NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP+A0.7++]
        DLT       CMP       D1Re0,D1Re0 [A0FrP+A0.8++],T7
        DT        MAX       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T7
        DLT       MAX       D1Re0,D1Re0,D1Re0 [A0FrP+A0.8++],T7
        DT        FFB       D1Re0,D1Re0 T7,[A0FrP+A0.8++]
        DLT       NORM      D1Re0,D1Re0 T7,[A0FrP+A0.8++]
        DT        NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP+A0.8++]
        DLT       NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP+A0.8++]
        DLT       CMP       D1Re0,D1Re0 [A0FrP],T7
        DT        MAX       D1Re0,D1Re0,D1Re0 [A0FrP],T7
        DLT       MAX       D1Re0,D1Re0,D1Re0 [A0FrP],T7
        DT        FFB       D1Re0,D1Re0 T7,[A0FrP]
        DLT       NORM      D1Re0,D1Re0 T7,[A0FrP]
        DT        NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP]
        DLT       NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP]
        DLT       CMP       D1Re0,D1Re0 [A0FrP++],T7
        DT        MAX       D1Re0,D1Re0,D1Re0 [A0FrP++],T7
        DLT       MAX       D1Re0,D1Re0,D1Re0 [A0FrP++],T7
        DT        FFB       D1Re0,D1Re0 T7,[A0FrP++]
        DLT       NORM      D1Re0,D1Re0 T7,[A0FrP++]
        DT        NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP++]
        DLT       NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP++]
        DLT       CMP       D1Re0,D1Re0 [A0FrP--],T7
        DT        MAX       D1Re0,D1Re0,D1Re0 [A0FrP--],T7
        DLT       MAX       D1Re0,D1Re0,D1Re0 [A0FrP--],T7
        DT        FFB       D1Re0,D1Re0 T7,[A0FrP--]
        DLT       NORM      D1Re0,D1Re0 T7,[A0FrP--]
        DT        NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP--]
        DLT       NMIN      D1Re0,D1Re0,D1Re0 T7,[A0FrP--]
        D         ABS       D1.7,D1Re0
        D         NMIN      D1.7,D1Re0,D1.7
        DL        MAX       D1.7,D1Re0,[D1BR.1]
        D         MIN       D1.7,D1Re0,[D1BR.1+D1BRI.1++]
        DT        FFB       D1Re0,D1Re0 [A0.7+A0FrP++],T7
        DLT       NORM      D1Re0,D1Re0 [A0.7+A0FrP++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A0.7+A0FrP++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A0.7+A0FrP++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A0.7+A0FrP++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A0.7+A0FrP++]
        DL        FFB       D1.7,D1.7
        DL        ABS       D1.7,D1.7
        DLT       CMP       D1Re0,D1Re0 [A0.7+A0.7++],T7
        DWT       MIN       D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T7
        DWT       NMIN      D1Re0,D1Re0,D1Re0 [A0.7+A0.7++],T7
        DLWT      ABS       D1Re0,D1Re0 [A0.7+A0.7++],T7
        DT        NORM      D1Re0,D1Re0 T7,[A0.7+A0.7++]
        DT        MIN       D1Re0,D1Re0,D1Re0 T7,[A0.7+A0.7++]
        DLT       MIN       D1Re0,D1Re0,D1Re0 T7,[A0.7+A0.7++]
        D         MIN       D1.7,D1.7,[D1BR.1]
        D         NMIN      D1.7,D1.7,[D1BR.1++]
        DL        MAX       D1.7,D1.7,[D1BR.1+D1BRI.1++]
        DLT       CMP       D1Re0,D1Re0 [A0.7+A0.8++],T7
        DT        MAX       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T7
        DLT       MAX       D1Re0,D1Re0,D1Re0 [A0.7+A0.8++],T7
        DT        FFB       D1Re0,D1Re0 T7,[A0.7+A0.8++]
        DLT       NORM      D1Re0,D1Re0 T7,[A0.7+A0.8++]
        DT        NMIN      D1Re0,D1Re0,D1Re0 T7,[A0.7+A0.8++]
        DLT       NMIN      D1Re0,D1Re0,D1Re0 T7,[A0.7+A0.8++]
        DLT       CMP       D1Re0,D1Re0 [A0.7],T7
        DT        MAX       D1Re0,D1Re0,D1Re0 [A0.7],T7
        DLT       MAX       D1Re0,D1Re0,D1Re0 [A0.7],T7
        DT        FFB       D1Re0,D1Re0 T7,[A0.7]
        DLT       NORM      D1Re0,D1Re0 T7,[A0.7]
        DT        NMIN      D1Re0,D1Re0,D1Re0 T7,[A0.7]
        DLT       NMIN      D1Re0,D1Re0,D1Re0 T7,[A0.7]
        DLT       CMP       D1Re0,D1Re0 [A0.7++],T7
        DT        MAX       D1Re0,D1Re0,D1Re0 [A0.7++],T7
        DLT       MAX       D1Re0,D1Re0,D1Re0 [A0.7++],T7
        DT        FFB       D1Re0,D1Re0 T7,[A0.7++]
        DLT       NORM      D1Re0,D1Re0 T7,[A0.7++]
        DT        NMIN      D1Re0,D1Re0,D1Re0 T7,[A0.7++]
        DLT       NMIN      D1Re0,D1Re0,D1Re0 T7,[A0.7++]
        DLT       CMP       D1Re0,D1Re0 [A0.7--],T7
        DT        MAX       D1Re0,D1Re0,D1Re0 [A0.7--],T7
        DLT       MAX       D1Re0,D1Re0,D1Re0 [A0.7--],T7
        DT        FFB       D1Re0,D1Re0 T7,[A0.7--]
        DLT       NORM      D1Re0,D1Re0 T7,[A0.7--]
        DT        NMIN      D1Re0,D1Re0,D1Re0 T7,[A0.7--]
        DLT       NMIN      D1Re0,D1Re0,D1Re0 T7,[A0.7--]
        D         ABS       D1.7,[D1BR.1]
        D         NMIN      D1.7,[D1BR.1],D1.7
        DL        MAX       D1.7,[D1BR.1],[D1BR.1]
        D         MIN       D1.7,[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        FFB       D1.7,[D1BR.1++]
        DL        ABS       D1.7,[D1BR.1++]
        D         MIN       D1.7,[D1BR.1++],[D1BR.1]
        D         NMIN      D1.7,[D1BR.1++],[D1BR.1++]
        DL        MAX       D1.7,[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ABS       D1.7,[D1BR.1+D1BRI.1++]
        D         NMIN      D1.7,[D1BR.1+D1BRI.1++],D1.7
        DL        MAX       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1]
        D         MIN       D1.7,[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DT        FFB       D1Re0,D1Re0 [A0.8+A0FrP++],T7
        DLT       NORM      D1Re0,D1Re0 [A0.8+A0FrP++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A0.8+A0FrP++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A0.8+A0FrP++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8+A0FrP++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8+A0FrP++]
        DT        FFB       D1Re0,D1Re0 [A0.8+A0.7++],T7
        DLT       NORM      D1Re0,D1Re0 [A0.8+A0.7++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A0.8+A0.7++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A0.8+A0.7++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8+A0.7++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8+A0.7++]
        DT        FFB       D1Re0,D1Re0 [A0.8+A0.8++],T7
        DLT       NORM      D1Re0,D1Re0 [A0.8+A0.8++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A0.8+A0.8++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A0.8+A0.8++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8+A0.8++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8+A0.8++]
        DT        FFB       D1Re0,D1Re0 [A0.8],T7
        DLT       NORM      D1Re0,D1Re0 [A0.8],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A0.8],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A0.8],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A0.8]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8]
        DT        FFB       D1Re0,D1Re0 [A0.8++],T7
        DLT       NORM      D1Re0,D1Re0 [A0.8++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A0.8++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A0.8++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A0.8++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8++]
        DT        FFB       D1Re0,D1Re0 [A0.8--],T7
        DLT       NORM      D1Re0,D1Re0 [A0.8--],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A0.8--],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A0.8--],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A0.8--]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8--]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A0.8--]
        DT        FFB       D1Re0,D1Re0 [A1LbP+A1LbP++],T7
        DLT       NORM      D1Re0,D1Re0 [A1LbP+A1LbP++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1LbP+A1LbP++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1LbP+A1LbP++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP+A1LbP++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP+A1LbP++]
        DT        FFB       D1Re0,D1Re0 [A1LbP+A1.7++],T7
        DLT       NORM      D1Re0,D1Re0 [A1LbP+A1.7++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1LbP+A1.7++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1LbP+A1.7++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP+A1.7++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP+A1.7++]
        DT        FFB       D1Re0,D1Re0 [A1LbP+A1.8++],T7
        DLT       NORM      D1Re0,D1Re0 [A1LbP+A1.8++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1LbP+A1.8++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1LbP+A1.8++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP+A1.8++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP+A1.8++]
        DT        FFB       D1Re0,D1Re0 [A1LbP],T7
        DLT       NORM      D1Re0,D1Re0 [A1LbP],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1LbP],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1LbP],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1LbP]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP]
        DT        FFB       D1Re0,D1Re0 [A1LbP++],T7
        DLT       NORM      D1Re0,D1Re0 [A1LbP++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1LbP++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1LbP++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1LbP++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP++]
        DT        FFB       D1Re0,D1Re0 [A1LbP--],T7
        DLT       NORM      D1Re0,D1Re0 [A1LbP--],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1LbP--],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1LbP--],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1LbP--]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP--]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1LbP--]
        DT        FFB       D1Re0,D1Re0 [A1.7+A1LbP++],T7
        DLT       NORM      D1Re0,D1Re0 [A1.7+A1LbP++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.7+A1LbP++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.7+A1LbP++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7+A1LbP++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7+A1LbP++]
        DT        FFB       D1Re0,D1Re0 [A1.7+A1.7++],T7
        DLT       NORM      D1Re0,D1Re0 [A1.7+A1.7++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.7+A1.7++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.7+A1.7++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7+A1.7++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7+A1.7++]
        DT        FFB       D1Re0,D1Re0 [A1.7+A1.8++],T7
        DLT       NORM      D1Re0,D1Re0 [A1.7+A1.8++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.7+A1.8++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.7+A1.8++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7+A1.8++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7+A1.8++]
        DT        FFB       D1Re0,D1Re0 [A1.7],T7
        DLT       NORM      D1Re0,D1Re0 [A1.7],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.7],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.7],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.7]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7]
        DT        FFB       D1Re0,D1Re0 [A1.7++],T7
        DLT       NORM      D1Re0,D1Re0 [A1.7++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.7++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.7++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.7++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7++]
        DT        FFB       D1Re0,D1Re0 [A1.7--],T7
        DLT       NORM      D1Re0,D1Re0 [A1.7--],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.7--],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.7--],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.7--]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7--]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.7--]
        DT        FFB       D1Re0,D1Re0 [A1.8+A1LbP++],T7
        DLT       NORM      D1Re0,D1Re0 [A1.8+A1LbP++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.8+A1LbP++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.8+A1LbP++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8+A1LbP++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8+A1LbP++]
        DT        FFB       D1Re0,D1Re0 [A1.8+A1.7++],T7
        DLT       NORM      D1Re0,D1Re0 [A1.8+A1.7++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.8+A1.7++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.8+A1.7++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8+A1.7++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8+A1.7++]
        DT        FFB       D1Re0,D1Re0 [A1.8+A1.8++],T7
        DLT       NORM      D1Re0,D1Re0 [A1.8+A1.8++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.8+A1.8++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.8+A1.8++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8+A1.8++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8+A1.8++]
        DT        FFB       D1Re0,D1Re0 [A1.8],T7
        DLT       NORM      D1Re0,D1Re0 [A1.8],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.8],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.8],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.8]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8]
        DT        FFB       D1Re0,D1Re0 [A1.8++],T7
        DLT       NORM      D1Re0,D1Re0 [A1.8++],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.8++],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.8++],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.8++]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8++]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8++]
        DT        FFB       D1Re0,D1Re0 [A1.8--],T7
        DLT       NORM      D1Re0,D1Re0 [A1.8--],T7
        DT        NMIN      D1Re0,D1Re0,D1Re0 [A1.8--],T7
        DLT       NMIN      D1Re0,D1Re0,D1Re0 [A1.8--],T7
        DLT       CMP       D1Re0,D1Re0 T7,[A1.8--]
        DT        MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8--]
        DLT       MAX       D1Re0,D1Re0,D1Re0 T7,[A1.8--]
        DL        FFB       [D1BW.1],D1Re0
        DL        ABS       [D1BW.1],D1Re0
        D         MIN       [D1BW.1],D1Re0,[D1BR.1]
        D         NMIN      [D1BW.1],D1Re0,[D1BR.1++]
        DL        MAX       [D1BW.1],D1Re0,[D1BR.1+D1BRI.1++]
        D         ABS       [D1BW.1],D1.7
        D         NMIN      [D1BW.1],D1.7,D1.7
        DL        MAX       [D1BW.1],D1.7,[D1BR.1]
        D         MIN       [D1BW.1],D1.7,[D1BR.1+D1BRI.1++]
        DT        NORM      D1Re0,D1Re0 [D1BW.1],T7
        DLT       MIN       D1Re0,D1Re0,D1Re0 [D1BW.1],T7
        DLT       FFB       D1Re0,D1Re0 T7,[D1BR.1]
        DLT       ABS       D1Re0,D1Re0 T7,[D1BR.1]
        D         ABS       [D1BW.1],[D1BR.1]
        D         NMIN      [D1BW.1],[D1BR.1],D1.7
        DL        MAX       [D1BW.1],[D1BR.1],[D1BR.1]
        D         MIN       [D1BW.1],[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        FFB       [D1BW.1],[D1BR.1++]
        DL        ABS       [D1BW.1],[D1BR.1++]
        D         MIN       [D1BW.1],[D1BR.1++],[D1BR.1]
        D         NMIN      [D1BW.1],[D1BR.1++],[D1BR.1++]
        DL        MAX       [D1BW.1],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ABS       [D1BW.1],[D1BR.1+D1BRI.1++]
        D         NMIN      [D1BW.1],[D1BR.1+D1BRI.1++],D1.7
        DL        MAX       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         MIN       [D1BW.1],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DL        FFB       [D1BW.1++],D1Re0
        DL        ABS       [D1BW.1++],D1Re0
        D         MIN       [D1BW.1++],D1Re0,[D1BR.1]
        D         NMIN      [D1BW.1++],D1Re0,[D1BR.1++]
        DL        MAX       [D1BW.1++],D1Re0,[D1BR.1+D1BRI.1++]
        D         ABS       [D1BW.1++],D1.7
        D         NMIN      [D1BW.1++],D1.7,D1.7
        DL        MAX       [D1BW.1++],D1.7,[D1BR.1]
        D         MIN       [D1BW.1++],D1.7,[D1BR.1+D1BRI.1++]
        DT        NORM      D1Re0,D1Re0 [D1BW.1++],T7
        DLT       MIN       D1Re0,D1Re0,D1Re0 [D1BW.1++],T7
        DLT       FFB       D1Re0,D1Re0 T7,[D1BR.1++]
        DLT       ABS       D1Re0,D1Re0 T7,[D1BR.1++]
        D         ABS       [D1BW.1++],[D1BR.1]
        D         NMIN      [D1BW.1++],[D1BR.1],D1.7
        DL        MAX       [D1BW.1++],[D1BR.1],[D1BR.1]
        D         MIN       [D1BW.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        FFB       [D1BW.1++],[D1BR.1++]
        DL        ABS       [D1BW.1++],[D1BR.1++]
        D         MIN       [D1BW.1++],[D1BR.1++],[D1BR.1]
        D         NMIN      [D1BW.1++],[D1BR.1++],[D1BR.1++]
        DL        MAX       [D1BW.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ABS       [D1BW.1++],[D1BR.1+D1BRI.1++]
        D         NMIN      [D1BW.1++],[D1BR.1+D1BRI.1++],D1.7
        DL        MAX       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         MIN       [D1BW.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        DL        FFB       [D1BW.1+D1BWI.1++],D1Re0
        DL        ABS       [D1BW.1+D1BWI.1++],D1Re0
        D         MIN       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1]
        D         NMIN      [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1++]
        DL        MAX       [D1BW.1+D1BWI.1++],D1Re0,[D1BR.1+D1BRI.1++]
        D         ABS       [D1BW.1+D1BWI.1++],D1.7
        D         NMIN      [D1BW.1+D1BWI.1++],D1.7,D1.7
        DL        MAX       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1]
        D         MIN       [D1BW.1+D1BWI.1++],D1.7,[D1BR.1+D1BRI.1++]
        DT        NORM      D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T7
        DLT       MIN       D1Re0,D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T7
        DLT       FFB       D1Re0,D1Re0 T7,[D1BR.1+D1BRI.1++]
        DLT       ABS       D1Re0,D1Re0 T7,[D1BR.1+D1BRI.1++]
        D         ABS       [D1BW.1+D1BWI.1++],[D1BR.1]
        D         NMIN      [D1BW.1+D1BWI.1++],[D1BR.1],D1.7
        DL        MAX       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1]
        D         MIN       [D1BW.1+D1BWI.1++],[D1BR.1],[D1BR.1+D1BRI.1++]
        DL        FFB       [D1BW.1+D1BWI.1++],[D1BR.1++]
        DL        ABS       [D1BW.1+D1BWI.1++],[D1BR.1++]
        D         MIN       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1]
        D         NMIN      [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1++]
        DL        MAX       [D1BW.1+D1BWI.1++],[D1BR.1++],[D1BR.1+D1BRI.1++]
        D         ABS       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++]
        D         NMIN      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],D1.7
        DL        MAX       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1]
        D         MIN       [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++],[D1BR.1+D1BRI.1++]
        D         CMP       D0Re0,#-0x8000
        D         CMP       D0.7,#-0x1
        D         CMP       [D0AR.0++],#0x7fff
        D         CMP       [D0AR.0+D0ARI.0++],#-0x8000
        D         CMP       D1Re0,#-0x1
        D         CMP       [D1BR.1],#0x7fff
        D         CMP       [D1BR.1++],#-0x8000
        D         CMP       [D1BR.1+D1BRI.1++],#-0x1
        D         TST       D0Re0,D1Re0
        DL        TST       D0Re0,[D0AR.0+D0ARI.0++]
        DL        TST       D0.7,D0.7
        D         TST       D0.7,[D0AR.0+D0ARI.0++]
        D         TST       [D0AR.0],A1.7
        DL        TST       [D0AR.0],[D0AR.0++]
        D         TST       [D0AR.0++],D0.7
        D         TST       [D0AR.0++],[D0AR.0++]
        D         TST       [D0AR.0+D0ARI.0++],A1LbP
        DL        TST       [D0AR.0+D0ARI.0++],[D0AR.0]
        DWT       TST       D0Re0,D0Re0 [A0FrP+A0FrP++],T7
        DWT       TST       D0Re0,D0Re0 [A0FrP+A0.7++],T7
        DWT       TST       D0Re0,D0Re0 [A0FrP+A0.8++],T7
        DWT       TST       D0Re0,D0Re0 [A0FrP],T7
        DWT       TST       D0Re0,D0Re0 [A0FrP++],T7
        DWT       TST       D0Re0,D0Re0 [A0FrP--],T7
        DWT       TST       D0Re0,D0Re0 [A0.7+A0FrP++],T7
        DWT       TST       D0Re0,D0Re0 [A0.7+A0.7++],T7
        DWT       TST       D0Re0,D0Re0 [A0.7+A0.8++],T7
        DWT       TST       D0Re0,D0Re0 [A0.7],T7
        DWT       TST       D0Re0,D0Re0 [A0.7++],T7
        DWT       TST       D0Re0,D0Re0 [A0.7--],T7
        DWT       TST       D0Re0,D0Re0 [A0.8+A0FrP++],T7
        DWT       TST       D0Re0,D0Re0 [A0.8+A0.7++],T7
        DWT       TST       D0Re0,D0Re0 [A0.8+A0.8++],T7
        DWT       TST       D0Re0,D0Re0 [A0.8],T7
        DWT       TST       D0Re0,D0Re0 [A0.8++],T7
        DWT       TST       D0Re0,D0Re0 [A0.8--],T7
        DLT       TST       D0Re0,D0Re0 [D0AW.0],T7
        DWT       TST       D0Re0,D0Re0 T7,[A1LbP+A1LbP++]
        DWT       TST       D0Re0,D0Re0 T7,[A1LbP+A1.7++]
        DWT       TST       D0Re0,D0Re0 T7,[A1LbP+A1.8++]
        DWT       TST       D0Re0,D0Re0 T7,[A1LbP]
        DWT       TST       D0Re0,D0Re0 T7,[A1LbP++]
        DWT       TST       D0Re0,D0Re0 T7,[A1LbP--]
        DLT       TST       D0Re0,D0Re0 [D0AW.0+D0AWI.0++],T7
        DWT       TST       D0Re0,D0Re0 T7,[A1.7+A1LbP++]
        DWT       TST       D0Re0,D0Re0 T7,[A1.7+A1.7++]
        DWT       TST       D0Re0,D0Re0 T7,[A1.7+A1.8++]
        DWT       TST       D0Re0,D0Re0 T7,[A1.7]
        DWT       TST       D0Re0,D0Re0 T7,[A1.7++]
        DWT       TST       D0Re0,D0Re0 T7,[A1.7--]
        DWT       TST       D0Re0,D0Re0 T7,[A1.8+A1LbP++]
        DWT       TST       D0Re0,D0Re0 T7,[A1.8+A1.7++]
        DWT       TST       D0Re0,D0Re0 T7,[A1.8+A1.8++]
        DWT       TST       D0Re0,D0Re0 T7,[A1.8]
        DWT       TST       D0Re0,D0Re0 T7,[A1.8++]
        DWT       TST       D0Re0,D0Re0 T7,[A1.8--]
        DL        TST       D1Re0,D1.7
        DL        TST       D1Re0,[D1BR.1++]
        D         TST       D1.7,A1.7
        D         TST       D1.7,[D1BR.1++]
        D         TST       [D1BR.1],D1.7
        DL        TST       [D1BR.1],[D1BR.1]
        D         TST       [D1BR.1++],A1LbP
        D         TST       [D1BR.1++],[D1BR.1]
        DL        TST       [D1BR.1+D1BRI.1++],D1Re0
        D         TST       [D1BR.1+D1BRI.1++],A0FrP
        DT        TST       D1Re0,D1Re0 [A0FrP+A0FrP++],T7
        DT        TST       D1Re0,D1Re0 [A0FrP+A0.7++],T7
        DT        TST       D1Re0,D1Re0 [A0FrP+A0.8++],T7
        DT        TST       D1Re0,D1Re0 [A0FrP],T7
        DT        TST       D1Re0,D1Re0 [A0FrP++],T7
        DT        TST       D1Re0,D1Re0 [A0FrP--],T7
        DT        TST       D1Re0,D1Re0 [A0.7+A0FrP++],T7
        DT        TST       D1Re0,D1Re0 [A0.7+A0.7++],T7
        DT        TST       D1Re0,D1Re0 [A0.7+A0.8++],T7
        DT        TST       D1Re0,D1Re0 [A0.7],T7
        DT        TST       D1Re0,D1Re0 [A0.7++],T7
        DT        TST       D1Re0,D1Re0 [A0.7--],T7
        DT        TST       D1Re0,D1Re0 [A0.8+A0FrP++],T7
        DT        TST       D1Re0,D1Re0 [A0.8+A0.7++],T7
        DT        TST       D1Re0,D1Re0 [A0.8+A0.8++],T7
        DT        TST       D1Re0,D1Re0 [A0.8],T7
        DT        TST       D1Re0,D1Re0 [A0.8++],T7
        DT        TST       D1Re0,D1Re0 [A0.8--],T7
        DT        TST       D1Re0,D1Re0 [A1LbP+A1LbP++],T7
        DT        TST       D1Re0,D1Re0 [A1LbP+A1.7++],T7
        DT        TST       D1Re0,D1Re0 [A1LbP+A1.8++],T7
        DT        TST       D1Re0,D1Re0 [A1LbP],T7
        DT        TST       D1Re0,D1Re0 [A1LbP++],T7
        DT        TST       D1Re0,D1Re0 [A1LbP--],T7
        DT        TST       D1Re0,D1Re0 [A1.7+A1LbP++],T7
        DT        TST       D1Re0,D1Re0 [A1.7+A1.7++],T7
        DT        TST       D1Re0,D1Re0 [A1.7+A1.8++],T7
        DT        TST       D1Re0,D1Re0 [A1.7],T7
        DT        TST       D1Re0,D1Re0 [A1.7++],T7
        DT        TST       D1Re0,D1Re0 [A1.7--],T7
        DT        TST       D1Re0,D1Re0 [A1.8+A1LbP++],T7
        DT        TST       D1Re0,D1Re0 [A1.8+A1.7++],T7
        DT        TST       D1Re0,D1Re0 [A1.8+A1.8++],T7
        DT        TST       D1Re0,D1Re0 [A1.8],T7
        DT        TST       D1Re0,D1Re0 [A1.8++],T7
        DT        TST       D1Re0,D1Re0 [A1.8--],T7
        DT        TST       D1Re0,D1Re0 [D1BW.1],T7
        DT        TST       D1Re0,D1Re0 [D1BW.1+D1BWI.1++],T7
        D         TST       D0Re0,#-0x1
        D         TST       [D0AR.0],#0x7fff
        D         TST       [D0AR.0++],#-0x8000
        D         TST       [D0AR.0+D0ARI.0++],#-0x1
        D         TST       D1.7,#0x7fff
        D         TST       [D1BR.1],#-0x8000
        D         TST       [D1BR.1++],#-0x1
        D         T0        #0x0, #0x0, #0x0, #0x0
        D         TF        #0x0, #0x0, #0x0, #0xf
        D         TF        #0x0, #0x3, #0x1f, #0x1f
        D         TF        #0x1f, #0x1f, #0x1f, #0x1f
        DL        MOV       D0AR.0,#-0x8000
        D         MOV       D0AW.0,#-0x8000
        DL        MOV       D1BR.1,#0x7fff
        D         MOV       D1BW.1,#0
        DL        MOV       D1BW.1,#-0x1
        DL        MOV       D0ARI.0,#-0x8000
        D         MOV       D0AWI.0,#-0x8000
        DL        MOV       D1BRI.1,#0x7fff
        D         MOV       D1BWI.1,#0
        DL        MOV       D1BWI.1,#-0x1
        DL        MOV       AC1.0,#0x7fff
        DL        MOV       AC1.0,#-0x8000
        DL        MOV       D0Re0,D0AW.0
        DL        MOV       D0Re0,D0AWI.0
        DL        MOV       D0.7,D0AW.0
        DL        MOV       D0.7,D0AWI.0
        DL        MOV       D0.16,D0AW.0
        DL        MOV       D0.16,D0AWI.0
        D         T7        #0x0, #0x0, #0x0
        D         GETD      D0AR.0,[A0FrP+A0FrP++]
        D         SETD      [A0FrP+A0.7++],T0
        D         GETL      D1AR.0,D0AR.0,[A0FrP+A0.7++]
        D         GETD      D0AR.0,[A0FrP+A0.8++]
        D         SETD      [A0FrP],T0
        D         GETL      D1AR.0,D0AR.0,[A0FrP]
        D         GETD      D0AR.0,[A0FrP++]
        D         SETD      [A0FrP--],T0
        D         GETL      D1AR.0,D0AR.0,[A0FrP--]
        D         GETD      D0AR.0,[A0.7+A0FrP++]
        D         SETD      [A0.7+A0.7++],T0
        D         GETL      D1AR.0,D0AR.0,[A0.7+A0.7++]
        D         GETD      D0AR.0,[A0.7+A0.8++]
        D         SETD      [A0.7],T0
        D         GETL      D1AR.0,D0AR.0,[A0.7]
        D         GETD      D0AR.0,[A0.7++]
        D         SETD      [A0.7--],T0
        D         GETL      D1AR.0,D0AR.0,[A0.7--]
        D         GETD      D0AR.0,[A0.8+A0FrP++]
        D         SETD      [A0.8+A0.7++],T0
        D         GETL      D1AR.0,D0AR.0,[A0.8+A0.7++]
        D         GETD      D0AR.0,[A0.8+A0.8++]
        D         SETD      [A0.8],T0
        D         GETL      D1AR.0,D0AR.0,[A0.8]
        D         GETD      D0AR.0,[A0.8++]
        D         SETD      [A0.8--],T0
        D         GETL      D1AR.0,D0AR.0,[A0.8--]
        D         GETD      D0AR.0,[A1LbP+A1LbP++]
        D         SETD      [A1LbP+A1.7++],T0
        D         GETL      D1AR.0,D0AR.0,[A1LbP+A1.7++]
        D         GETD      D0AR.0,[A1LbP+A1.8++]
        D         SETD      [A1LbP],T0
        D         GETL      D1AR.0,D0AR.0,[A1LbP]
        D         GETD      D0AR.0,[A1LbP++]
        D         SETD      [A1LbP--],T0
        D         GETL      D1AR.0,D0AR.0,[A1LbP--]
        D         GETD      D0AR.0,[A1.7+A1LbP++]
        D         SETD      [A1.7+A1.7++],T0
        D         GETL      D1AR.0,D0AR.0,[A1.7+A1.7++]
        D         GETD      D0AR.0,[A1.7+A1.8++]
        D         SETD      [A1.7],T0
        D         GETL      D1AR.0,D0AR.0,[A1.7]
        D         GETD      D0AR.0,[A1.7++]
        D         SETD      [A1.7--],T0
        D         GETL      D1AR.0,D0AR.0,[A1.7--]
        D         GETD      D0AR.0,[A1.8+A1LbP++]
        D         SETD      [A1.8+A1.7++],T0
        D         GETL      D1AR.0,D0AR.0,[A1.8+A1.7++]
        D         GETD      D0AR.0,[A1.8+A1.8++]
        D         SETD      [A1.8],T0
        D         GETL      D1AR.0,D0AR.0,[A1.8]
        D         GETD      D0AR.0,[A1.8++]
        D         SETD      [A1.8--],T0
        D         GETL      D1AR.0,D0AR.0,[A1.8--]
        D         GETD      T1,[A0FrP+A0.7++]
        D         GETD      T1,[A0FrP]
        D         GETD      T1,[A0FrP--]
        D         GETD      T1,[A0.7+A0.7++]
        D         GETD      T1,[A0.7]
        D         GETD      T1,[A0.7--]
        D         GETD      T1,[A0.8+A0.7++]
        D         GETD      T1,[A0.8]
        D         GETD      T1,[A0.8--]
        D         GETD      T1,[A1LbP+A1.7++]
        D         GETD      T1,[A1LbP]
        D         GETD      T1,[A1LbP--]
        D         GETD      T1,[A1.7+A1.7++]
        D         GETD      T1,[A1.7]
        D         GETD      T1,[A1.7--]
        D         GETD      T1,[A1.8+A1.7++]
        D         GETD      T1,[A1.8]
        D         GETD      T1,[A1.8--]
        D         SETD      [A0FrP+A0.7++],D0AW.0
        D         SETL      [A0FrP+A0.8++],T2
        D         GETD      T2,[A0FrP]
        D         SETD      [A0FrP--],D0AW.0
        D         SETL      [A0.7+A0FrP++],T2
        D         GETD      T2,[A0.7+A0.7++]
        D         SETD      [A0.7],D0AW.0
        D         SETL      [A0.7++],T2
        D         GETD      T2,[A0.7--]
        D         SETD      [A0.8+A0.7++],D0AW.0
        D         SETL      [A0.8+A0.8++],T2
        D         GETD      T2,[A0.8]
        D         SETD      [A0.8--],D0AW.0
        D         SETL      [A1LbP+A1LbP++],T2
        D         GETD      T2,[A1LbP+A1.7++]
        D         SETD      [A1LbP],D0AW.0
        D         SETL      [A1LbP++],T2
        D         GETD      T2,[A1LbP--]
        D         SETD      [A1.7+A1.7++],D0AW.0
        D         SETL      [A1.7+A1.8++],T2
        D         GETD      T2,[A1.7]
        D         SETD      [A1.7--],D0AW.0
        D         SETL      [A1.8+A1LbP++],T2
        D         GETD      T2,[A1.8+A1.7++]
        D         SETD      [A1.8],D0AW.0
        D         SETL      [A1.8++],T2
        D         GETD      T2,[A1.8--]
        D         GETD      T3,[A0FrP+A0.7++]
        D         GETD      T3,[A0FrP]
        D         GETD      T3,[A0FrP--]
        D         GETD      T3,[A0.7+A0.7++]
        D         GETD      T3,[A0.7]
        D         GETD      T3,[A0.7--]
        D         GETD      T3,[A0.8+A0.7++]
        D         GETD      T3,[A0.8]
        D         GETD      T3,[A0.8--]
        D         GETD      T3,[A1LbP+A1.7++]
        D         GETD      T3,[A1LbP]
        D         GETD      T3,[A1LbP--]
        D         GETD      T3,[A1.7+A1.7++]
        D         GETD      T3,[A1.7]
        D         GETD      T3,[A1.7--]
        D         GETD      T3,[A1.8+A1.7++]
        D         GETD      T3,[A1.8]
        D         GETD      T3,[A1.8--]
        D         SETD      [A0FrP+A0.7++],D0BR.0
        D         SETL      [A0FrP+A0.8++],T4
        D         GETD      T4,[A0FrP]
        D         SETD      [A0FrP--],D0BR.0
        D         SETL      [A0.7+A0FrP++],T4
        D         GETD      T4,[A0.7+A0.7++]
        D         SETD      [A0.7],D0BR.0
        D         SETL      [A0.7++],T4
        D         GETD      T4,[A0.7--]
        D         SETD      [A0.8+A0.7++],D0BR.0
        D         SETL      [A0.8+A0.8++],T4
        D         GETD      T4,[A0.8]
        D         SETD      [A0.8--],D0BR.0
        D         SETL      [A1LbP+A1LbP++],T4
        D         GETD      T4,[A1LbP+A1.7++]
        D         SETD      [A1LbP],D0BR.0
        D         SETL      [A1LbP++],T4
        D         GETD      T4,[A1LbP--]
        D         SETD      [A1.7+A1.7++],D0BR.0
        D         SETL      [A1.7+A1.8++],T4
        D         GETD      T4,[A1.7]
        D         SETD      [A1.7--],D0BR.0
        D         SETL      [A1.8+A1LbP++],T4
        D         GETD      T4,[A1.8+A1.7++]
        D         SETD      [A1.8],D0BR.0
        D         SETL      [A1.8++],T4
        D         GETD      T4,[A1.8--]
        D         SETD      [A0FrP+A0.7++],D1BR.1
        D         SETL      [A0FrP+A0.8++],T5
        D         GETD      T5,[A0FrP]
        D         SETD      [A0FrP--],D1BR.1
        D         SETL      [A0.7+A0FrP++],T5
        D         GETD      T5,[A0.7+A0.7++]
        D         SETD      [A0.7],D1BR.1
        D         SETL      [A0.7++],T5
        D         GETD      T5,[A0.7--]
        D         SETD      [A0.8+A0.7++],D1BR.1
        D         SETL      [A0.8+A0.8++],T5
        D         GETD      T5,[A0.8]
        D         SETD      [A0.8--],D1BR.1
        D         SETL      [A1LbP+A1LbP++],T5
        D         GETD      T5,[A1LbP+A1.7++]
        D         SETD      [A1LbP],D1BR.1
        D         SETL      [A1LbP++],T5
        D         GETD      T5,[A1LbP--]
        D         SETD      [A1.7+A1.7++],D1BR.1
        D         SETL      [A1.7+A1.8++],T5
        D         GETD      T5,[A1.7]
        D         SETD      [A1.7--],D1BR.1
        D         SETL      [A1.8+A1LbP++],T5
        D         GETD      T5,[A1.8+A1.7++]
        D         SETD      [A1.8],D1BR.1
        D         SETL      [A1.8++],T5
        D         GETD      T5,[A1.8--]
        D         GETD      T6,[A0FrP+A0.7++]
        D         GETD      T6,[A0FrP]
        D         GETD      T6,[A0FrP--]
        D         GETD      T6,[A0.7+A0.7++]
        D         GETD      T6,[A0.7]
        D         GETD      T6,[A0.7--]
        D         GETD      T6,[A0.8+A0.7++]
        D         GETD      T6,[A0.8]
        D         GETD      T6,[A0.8--]
        D         GETD      T6,[A1LbP+A1.7++]
        D         GETD      T6,[A1LbP]
        D         GETD      T6,[A1LbP--]
        D         GETD      T6,[A1.7+A1.7++]
        D         GETD      T6,[A1.7]
        D         GETD      T6,[A1.7--]
        D         GETD      T6,[A1.8+A1.7++]
        D         GETD      T6,[A1.8]
        D         GETD      T6,[A1.8--]
        D         SETD      [A0FrP+A0.7++],D1BW.1
        D         SETL      [A0FrP+A0.8++],T7
        D         GETD      T7,[A0FrP]
        D         SETD      [A0FrP--],D1BW.1
        D         SETL      [A0.7+A0FrP++],T7
        D         GETD      T7,[A0.7+A0.7++]
        D         SETD      [A0.7],D1BW.1
        D         SETL      [A0.7++],T7
        D         GETD      T7,[A0.7--]
        D         SETD      [A0.8+A0.7++],D1BW.1
        D         SETL      [A0.8+A0.8++],T7
        D         GETD      T7,[A0.8]
        D         SETD      [A0.8--],D1BW.1
        D         SETL      [A1LbP+A1LbP++],T7
        D         GETD      T7,[A1LbP+A1.7++]
        D         SETD      [A1LbP],D1BW.1
        D         SETL      [A1LbP++],T7
        D         GETD      T7,[A1LbP--]
        D         SETD      [A1.7+A1.7++],D1BW.1
        D         SETL      [A1.7+A1.8++],T7
        D         GETD      T7,[A1.7]
        D         SETD      [A1.7--],D1BW.1
        D         SETL      [A1.8+A1LbP++],T7
        D         GETD      T7,[A1.8+A1.7++]
        D         SETD      [A1.8],D1BW.1
        D         SETL      [A1.8++],T7
        D         GETD      T7,[A1.8--]
        D         SETD      [A0FrP+A0.7++],D0ARI.0
        D         SETL      [A0FrP+A0.8++],T8
        D         GETD      T8,[A0FrP]
        D         SETD      [A0FrP--],D0ARI.0
        D         SETL      [A0.7+A0FrP++],T8
        D         GETD      T8,[A0.7+A0.7++]
        D         SETD      [A0.7],D0ARI.0
        D         SETL      [A0.7++],T8
        D         GETD      T8,[A0.7--]
        D         SETD      [A0.8+A0.7++],D0ARI.0
        D         SETL      [A0.8+A0.8++],T8
        D         GETD      T8,[A0.8]
        D         SETD      [A0.8--],D0ARI.0
        D         SETL      [A1LbP+A1LbP++],T8
        D         GETD      T8,[A1LbP+A1.7++]
        D         SETD      [A1LbP],D0ARI.0
        D         SETL      [A1LbP++],T8
        D         GETD      T8,[A1LbP--]
        D         SETD      [A1.7+A1.7++],D0ARI.0
        D         SETL      [A1.7+A1.8++],T8
        D         GETD      T8,[A1.7]
        D         SETD      [A1.7--],D0ARI.0
        D         SETL      [A1.8+A1LbP++],T8
        D         GETD      T8,[A1.8+A1.7++]
        D         SETD      [A1.8],D0ARI.0
        D         SETL      [A1.8++],T8
        D         GETD      T8,[A1.8--]
        D         GETD      T9,[A0FrP+A0.7++]
        D         GETD      T9,[A0FrP]
        D         GETD      T9,[A0FrP--]
        D         GETD      T9,[A0.7+A0.7++]
        D         GETD      T9,[A0.7]
        D         GETD      T9,[A0.7--]
        D         GETD      T9,[A0.8+A0.7++]
        D         GETD      T9,[A0.8]
        D         GETD      T9,[A0.8--]
        D         GETD      T9,[A1LbP+A1.7++]
        D         GETD      T9,[A1LbP]
        D         GETD      T9,[A1LbP--]
        D         GETD      T9,[A1.7+A1.7++]
        D         GETD      T9,[A1.7]
        D         GETD      T9,[A1.7--]
        D         GETD      T9,[A1.8+A1.7++]
        D         GETD      T9,[A1.8]
        D         GETD      T9,[A1.8--]
        D         SETD      [A0FrP+A0.7++],D0AWI.0
        D         SETL      [A0FrP+A0.8++],TA
        D         GETD      TA,[A0FrP]
        D         SETD      [A0FrP--],D0AWI.0
        D         SETL      [A0.7+A0FrP++],TA
        D         GETD      TA,[A0.7+A0.7++]
        D         SETD      [A0.7],D0AWI.0
        D         SETL      [A0.7++],TA
        D         GETD      TA,[A0.7--]
        D         SETD      [A0.8+A0.7++],D0AWI.0
        D         SETL      [A0.8+A0.8++],TA
        D         GETD      TA,[A0.8]
        D         SETD      [A0.8--],D0AWI.0
        D         SETL      [A1LbP+A1LbP++],TA
        D         GETD      TA,[A1LbP+A1.7++]
        D         SETD      [A1LbP],D0AWI.0
        D         SETL      [A1LbP++],TA
        D         GETD      TA,[A1LbP--]
        D         SETD      [A1.7+A1.7++],D0AWI.0
        D         SETL      [A1.7+A1.8++],TA
        D         GETD      TA,[A1.7]
        D         SETD      [A1.7--],D0AWI.0
        D         SETL      [A1.8+A1LbP++],TA
        D         GETD      TA,[A1.8+A1.7++]
        D         SETD      [A1.8],D0AWI.0
        D         SETL      [A1.8++],TA
        D         GETD      TA,[A1.8--]
        D         GETD      TB,[A0FrP+A0.7++]
        D         GETD      TB,[A0FrP]
        D         GETD      TB,[A0FrP--]
        D         GETD      TB,[A0.7+A0.7++]
        D         GETD      TB,[A0.7]
        D         GETD      TB,[A0.7--]
        D         GETD      TB,[A0.8+A0.7++]
        D         GETD      TB,[A0.8]
        D         GETD      TB,[A0.8--]
        D         GETD      TB,[A1LbP+A1.7++]
        D         GETD      TB,[A1LbP]
        D         GETD      TB,[A1LbP--]
        D         GETD      TB,[A1.7+A1.7++]
        D         GETD      TB,[A1.7]
        D         GETD      TB,[A1.7--]
        D         GETD      TB,[A1.8+A1.7++]
        D         GETD      TB,[A1.8]
        D         GETD      TB,[A1.8--]
        D         GETD      TC,[A0FrP+A0.7++]
        D         GETD      TC,[A0FrP]
        D         GETD      TC,[A0FrP--]
        D         GETD      TC,[A0.7+A0.7++]
        D         GETD      TC,[A0.7]
        D         GETD      TC,[A0.7--]
        D         GETD      TC,[A0.8+A0.7++]
        D         GETD      TC,[A0.8]
        D         GETD      TC,[A0.8--]
        D         GETD      TC,[A1LbP+A1.7++]
        D         GETD      TC,[A1LbP]
        D         GETD      TC,[A1LbP--]
        D         GETD      TC,[A1.7+A1.7++]
        D         GETD      TC,[A1.7]
        D         GETD      TC,[A1.7--]
        D         GETD      TC,[A1.8+A1.7++]
        D         GETD      TC,[A1.8]
        D         GETD      TC,[A1.8--]
        D         SETD      [A0FrP+A0.7++],D1BRI.1
        D         SETL      [A0FrP+A0.8++],TD
        D         GETD      TD,[A0FrP]
        D         SETD      [A0FrP--],D1BRI.1
        D         SETL      [A0.7+A0FrP++],TD
        D         GETD      TD,[A0.7+A0.7++]
        D         SETD      [A0.7],D1BRI.1
        D         SETL      [A0.7++],TD
        D         GETD      TD,[A0.7--]
        D         SETD      [A0.8+A0.7++],D1BRI.1
        D         SETL      [A0.8+A0.8++],TD
        D         GETD      TD,[A0.8]
        D         SETD      [A0.8--],D1BRI.1
        D         SETL      [A1LbP+A1LbP++],TD
        D         GETD      TD,[A1LbP+A1.7++]
        D         SETD      [A1LbP],D1BRI.1
        D         SETL      [A1LbP++],TD
        D         GETD      TD,[A1LbP--]
        D         SETD      [A1.7+A1.7++],D1BRI.1
        D         SETL      [A1.7+A1.8++],TD
        D         GETD      TD,[A1.7]
        D         SETD      [A1.7--],D1BRI.1
        D         SETL      [A1.8+A1LbP++],TD
        D         GETD      TD,[A1.8+A1.7++]
        D         SETD      [A1.8],D1BRI.1
        D         SETL      [A1.8++],TD
        D         GETD      TD,[A1.8--]
        D         GETD      TE,[A0FrP+A0.7++]
        D         GETD      TE,[A0FrP]
        D         GETD      TE,[A0FrP--]
        D         GETD      TE,[A0.7+A0.7++]
        D         GETD      TE,[A0.7]
        D         GETD      TE,[A0.7--]
        D         GETD      TE,[A0.8+A0.7++]
        D         GETD      TE,[A0.8]
        D         GETD      TE,[A0.8--]
        D         GETD      TE,[A1LbP+A1.7++]
        D         GETD      TE,[A1LbP]
        D         GETD      TE,[A1LbP--]
        D         GETD      TE,[A1.7+A1.7++]
        D         GETD      TE,[A1.7]
        D         GETD      TE,[A1.7--]
        D         GETD      TE,[A1.8+A1.7++]
        D         GETD      TE,[A1.8]
        D         GETD      TE,[A1.8--]
        D         SETD      [A0FrP+A0.7++],D1BWI.1
        D         SETL      [A0FrP+A0.8++],TF
        D         GETD      TF,[A0FrP]
        D         SETD      [A0FrP--],D1BWI.1
        D         SETL      [A0.7+A0FrP++],TF
        D         GETD      TF,[A0.7+A0.7++]
        D         SETD      [A0.7],D1BWI.1
        D         SETL      [A0.7++],TF
        D         GETD      TF,[A0.7--]
        D         SETD      [A0.8+A0.7++],D1BWI.1
        D         SETL      [A0.8+A0.8++],TF
        D         GETD      TF,[A0.8]
        D         SETD      [A0.8--],D1BWI.1
        D         SETL      [A1LbP+A1LbP++],TF
        D         GETD      TF,[A1LbP+A1.7++]
        D         SETD      [A1LbP],D1BWI.1
        D         SETL      [A1LbP++],TF
        D         GETD      TF,[A1LbP--]
        D         SETD      [A1.7+A1.7++],D1BWI.1
        D         SETL      [A1.7+A1.8++],TF
        D         GETD      TF,[A1.7]
        D         SETD      [A1.7--],D1BWI.1
        D         SETL      [A1.8+A1LbP++],TF
        D         GETD      TF,[A1.8+A1.7++]
        D         SETD      [A1.8],D1BWI.1
        D         SETL      [A1.8++],TF
        D         GETD      TF,[A1.8--]
        D         GETL      AC0.0,AC1.0,[A0FrP+A0FrP++]
        D         GETL      AC0.0,AC1.0,[A0FrP+A0.7++]
        D         GETL      AC0.0,AC1.0,[A0FrP+A0.8++]
        D         GETL      AC0.0,AC1.0,[A0FrP]
        D         GETL      AC0.0,AC1.0,[A0FrP++]
        D         GETL      AC0.0,AC1.0,[A0FrP--]
        D         GETL      AC0.0,AC1.0,[A0.7+A0FrP++]
        D         GETL      AC0.0,AC1.0,[A0.7+A0.7++]
        D         GETL      AC0.0,AC1.0,[A0.7+A0.8++]
        D         GETL      AC0.0,AC1.0,[A0.7]
        D         GETL      AC0.0,AC1.0,[A0.7++]
        D         GETL      AC0.0,AC1.0,[A0.7--]
        D         GETL      AC0.0,AC1.0,[A0.8+A0FrP++]
        D         GETL      AC0.0,AC1.0,[A0.8+A0.7++]
        D         GETL      AC0.0,AC1.0,[A0.8+A0.8++]
        D         GETL      AC0.0,AC1.0,[A0.8]
        D         GETL      AC0.0,AC1.0,[A0.8++]
        D         GETL      AC0.0,AC1.0,[A0.8--]
        D         GETL      AC0.0,AC1.0,[A1LbP+A1LbP++]
        D         GETL      AC0.0,AC1.0,[A1LbP+A1.7++]
        D         GETL      AC0.0,AC1.0,[A1LbP+A1.8++]
        D         GETL      AC0.0,AC1.0,[A1LbP]
        D         GETL      AC0.0,AC1.0,[A1LbP++]
        D         GETL      AC0.0,AC1.0,[A1LbP--]
        D         GETL      AC0.0,AC1.0,[A1.7+A1LbP++]
        D         GETL      AC0.0,AC1.0,[A1.7+A1.7++]
        D         GETL      AC0.0,AC1.0,[A1.7+A1.8++]
        D         GETL      AC0.0,AC1.0,[A1.7]
        D         GETL      AC0.0,AC1.0,[A1.7++]
        D         GETL      AC0.0,AC1.0,[A1.7--]
        D         GETL      AC0.0,AC1.0,[A1.8+A1LbP++]
        D         GETL      AC0.0,AC1.0,[A1.8+A1.7++]
        D         GETL      AC0.0,AC1.0,[A1.8+A1.8++]
        D         GETL      AC0.0,AC1.0,[A1.8]
        D         GETL      AC0.0,AC1.0,[A1.8++]
        D         GETL      AC0.0,AC1.0,[A1.8--]
        DH        GETL      AC0.0,AC1.0,[A0FrP+A0FrP++]
        DH        GETL      AC0.0,AC1.0,[A0FrP+A0.7++]
        DH        GETL      AC0.0,AC1.0,[A0FrP+A0.8++]
        DH        GETL      AC0.0,AC1.0,[A0FrP]
        DH        GETL      AC0.0,AC1.0,[A0FrP++]
        DH        GETL      AC0.0,AC1.0,[A0FrP--]
        DH        GETL      AC0.0,AC1.0,[A0.7+A0FrP++]
        DH        GETL      AC0.0,AC1.0,[A0.7+A0.7++]
        DH        GETL      AC0.0,AC1.0,[A0.7+A0.8++]
        DH        GETL      AC0.0,AC1.0,[A0.7]
        DH        GETL      AC0.0,AC1.0,[A0.7++]
        DH        GETL      AC0.0,AC1.0,[A0.7--]
        DH        GETL      AC0.0,AC1.0,[A0.8+A0FrP++]
        DH        GETL      AC0.0,AC1.0,[A0.8+A0.7++]
        DH        GETL      AC0.0,AC1.0,[A0.8+A0.8++]
        DH        GETL      AC0.0,AC1.0,[A0.8]
        DH        GETL      AC0.0,AC1.0,[A0.8++]
        DH        GETL      AC0.0,AC1.0,[A0.8--]
        DH        GETL      AC0.0,AC1.0,[A1LbP+A1LbP++]
        DH        GETL      AC0.0,AC1.0,[A1LbP+A1.7++]
        DH        GETL      AC0.0,AC1.0,[A1LbP+A1.8++]
        DH        GETL      AC0.0,AC1.0,[A1LbP]
        DH        GETL      AC0.0,AC1.0,[A1LbP++]
        DH        GETL      AC0.0,AC1.0,[A1LbP--]
        DH        GETL      AC0.0,AC1.0,[A1.7+A1LbP++]
        DH        GETL      AC0.0,AC1.0,[A1.7+A1.7++]
        DH        GETL      AC0.0,AC1.0,[A1.7+A1.8++]
        DH        GETL      AC0.0,AC1.0,[A1.7]
        DH        GETL      AC0.0,AC1.0,[A1.7++]
        DH        GETL      AC0.0,AC1.0,[A1.7--]
        DH        GETL      AC0.0,AC1.0,[A1.8+A1LbP++]
        DH        GETL      AC0.0,AC1.0,[A1.8+A1.7++]
        DH        GETL      AC0.0,AC1.0,[A1.8+A1.8++]
        DH        GETL      AC0.0,AC1.0,[A1.8]
        DH        GETL      AC0.0,AC1.0,[A1.8++]
        DH        GETL      AC0.0,AC1.0,[A1.8--]
        DH        TF        #0x0, #0x0, #0x0, #0x3
        DH        TF        #0x0, #0x0, #0x1f, #0x1f
        DH        TF        #0x7, #0x1f, #0x1f, #0x1f
        D         RTDW      D1Re0,D1Re0
        DL        XSDB      D1Re0,D1Re0
        DL        XSDSW     D1Re0,D1Re0
        D         RTDW      D1Re0,D1.7
        DL        XSDB      D1Re0,D1.7
        DL        XSDSW     D1Re0,D1.7
        D         RTDSW     D0Re0,[D0AR.0]
        D         XSDW      D0Re0,[D0AR.0++]
        DL        XSDSB     D0Re0,[D0AR.0++]
        D         RTDSW     D0Re0,[D0AR.0+D0ARI.0++]
        D         XSDW      D1Re0,[D1BR.1]
        DL        XSDSB     D1Re0,[D1BR.1]
        D         RTDSW     D1Re0,[D1BR.1++]
        D         XSDW      D1Re0,[D1BR.1+D1BRI.1++]
        DL        XSDSB     D1Re0,[D1BR.1+D1BRI.1++]
        D         RTDW      D1.7,D1Re0
        DL        XSDB      D1.7,D1Re0
        DL        XSDSW     D1.7,D1Re0
        D         RTDW      D1.7,D1.7
        DL        XSDB      D1.7,D1.7
        DL        XSDSW     D1.7,D1.7
        D         RTDSW     D0.7,[D0AR.0]
        D         XSDW      D0.7,[D0AR.0++]
        DL        XSDSB     D0.7,[D0AR.0++]
        D         RTDSW     D0.7,[D0AR.0+D0ARI.0++]
        D         XSDW      D1.7,[D1BR.1]
        DL        XSDSB     D1.7,[D1BR.1]
        D         RTDSW     D1.7,[D1BR.1++]
        D         XSDW      D1.7,[D1BR.1+D1BRI.1++]
        DL        XSDSB     D1.7,[D1BR.1+D1BRI.1++]
        D         RTDSW     [D0AW.0],D0Re0
        D         XSDW      [D0AW.0],D0.7
        DL        XSDSB     [D0AW.0],D0.7
        D         RTDSW     [D0AW.0],[D0AR.0]
        D         XSDW      [D0AW.0],[D0AR.0++]
        DL        XSDSB     [D0AW.0],[D0AR.0++]
        D         RTDSW     [D0AW.0],[D0AR.0+D0ARI.0++]
        D         XSDW      [D0AW.0++],D0Re0
        DL        XSDSB     [D0AW.0++],D0Re0
        D         RTDSW     [D0AW.0++],D0.7
        D         XSDW      [D0AW.0++],[D0AR.0]
        DL        XSDSB     [D0AW.0++],[D0AR.0]
        D         RTDSW     [D0AW.0++],[D0AR.0++]
        D         XSDW      [D0AW.0++],[D0AR.0+D0ARI.0++]
        DL        XSDSB     [D0AW.0++],[D0AR.0+D0ARI.0++]
        D         RTDSW     [D0AW.0+D0AWI.0++],D0Re0
        D         XSDW      [D0AW.0+D0AWI.0++],D0.7
        DL        XSDSB     [D0AW.0+D0AWI.0++],D0.7
        D         RTDSW     [D0AW.0+D0AWI.0++],[D0AR.0]
        D         XSDW      [D0AW.0+D0AWI.0++],[D0AR.0++]
        DL        XSDSB     [D0AW.0+D0AWI.0++],[D0AR.0++]
        D         RTDSW     [D0AW.0+D0AWI.0++],[D0AR.0+D0ARI.0++]
        D         XSDW      [D1BW.1],D1Re0
        DL        XSDSB     [D1BW.1],D1Re0
        D         RTDSW     [D1BW.1],D1.7
        D         XSDW      [D1BW.1],[D1BR.1]
        DL        XSDSB     [D1BW.1],[D1BR.1]
        D         RTDSW     [D1BW.1],[D1BR.1++]
        D         XSDW      [D1BW.1],[D1BR.1+D1BRI.1++]
        DL        XSDSB     [D1BW.1],[D1BR.1+D1BRI.1++]
        D         RTDSW     [D1BW.1++],D1Re0
        D         XSDW      [D1BW.1++],D1.7
        DL        XSDSB     [D1BW.1++],D1.7
        D         RTDSW     [D1BW.1++],[D1BR.1]
        D         XSDW      [D1BW.1++],[D1BR.1++]
        DL        XSDSB     [D1BW.1++],[D1BR.1++]
        D         RTDSW     [D1BW.1++],[D1BR.1+D1BRI.1++]
        D         XSDW      [D1BW.1+D1BWI.1++],D1Re0
        DL        XSDSB     [D1BW.1+D1BWI.1++],D1Re0
        D         RTDSW     [D1BW.1+D1BWI.1++],D1.7
        D         XSDW      [D1BW.1+D1BWI.1++],[D1BR.1]
        DL        XSDSB     [D1BW.1+D1BWI.1++],[D1BR.1]
        D         RTDSW     [D1BW.1+D1BWI.1++],[D1BR.1++]
        D         XSDW      [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++]
        DL        XSDSB     [D1BW.1+D1BWI.1++],[D1BR.1+D1BRI.1++]
        DB        MMOVD     RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D1Re0++]
        DB        MMOVD     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D1Re0++]
        DB        MMOVD     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[A0FrP++]
        DB        MMOVD     RAM8X32,RAM8X32,[D0.7++]
        DB        MMOVD     RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D0.7++]
        DB        MMOVD     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D0.7++]
        DB        MMOVD     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D0.7++]
        DB        MMOVD     RAM8X32,RAM8X32,RAM8X32,RAM8X32,[A0.8++]
        DB        MMOVD     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[A0.8++]
        DB        MMOVD     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D1.16++]
        DB        MMOVD     RAM8X,RAM8X,[D1Re0++]
        DB        MMOVD     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[D1Re0++]
        DB        MMOVD     RAM8X,RAM8X,RAM8X,[A0FrP++]
        DB        MMOVD     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[A0FrP++]
        DB        MMOVD     RAM8X,RAM8X,RAM8X,[D0.7++]
        DB        MMOVD     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[D0.7++]
        DB        MMOVD     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[D0.7++]
        DB        MMOVD     RAM8X,RAM8X,[A0.8++]
        DB        MMOVD     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[A0.8++]
        DB        MMOVD     RAM8X,RAM8X,RAM8X,[D1.16++]
        DB        MMOVD     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[D1.16++]
        DBH       MMOVD     RABZ,RABZ,RABZ,RABZ,[D1Re0++]
        DBH       MMOVD     RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,[D1Re0++]
        DBH       MMOVD     RABZ,RABZ,RABZ,RABZ,RABZ,[A0FrP++]
        DBH       MMOVD     RABZ,RABZ,[D0.7++]
        DBH       MMOVD     RABZ,RABZ,RABZ,RABZ,[D0.7++]
        DBH       MMOVD     RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,[D0.7++]
        DBH       MMOVD     RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,[D0.7++]
        DBH       MMOVD     RABZ,RABZ,RABZ,RABZ,[A0.8++]
        DBH       MMOVD     RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,[A0.8++]
        DBH       MMOVD     RABZ,RABZ,RABZ,RABZ,RABZ,[D1.16++]
        DW        MMOVD     RAMX,RAMX,[D1Re0++]
        DW        MMOVD     RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,[D1Re0++]
        DW        MMOVD     RAMX,RAMX,RAMX,[A0FrP++]
        DW        MMOVD     RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,[A0FrP++]
        DW        MMOVD     RAMX,RAMX,RAMX,[D0.7++]
        DW        MMOVD     RAMX,RAMX,RAMX,RAMX,RAMX,[D0.7++]
        DW        MMOVD     RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,[D0.7++]
        DW        MMOVD     RAMX,RAMX,[A0.8++]
        DW        MMOVD     RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,[A0.8++]
        DW        MMOVD     RAMX,RAMX,RAMX,[D1.16++]
        DW        MMOVD     RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,[D1.16++]
        DB        MMOVL     RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D1Re0++]
        DB        MMOVL     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D1Re0++]
        DB        MMOVL     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[A0FrP++]
        DB        MMOVL     RAM8X32,RAM8X32,[D0.7++]
        DB        MMOVL     RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D0.7++]
        DB        MMOVL     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D0.7++]
        DB        MMOVL     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D0.7++]
        DB        MMOVL     RAM8X32,RAM8X32,RAM8X32,RAM8X32,[A0.8++]
        DB        MMOVL     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[A0.8++]
        DB        MMOVL     RAM8X32,RAM8X32,RAM8X32,RAM8X32,RAM8X32,[D1.16++]
        DB        MMOVL     RAM8X,RAM8X,[D1Re0++]
        DB        MMOVL     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[D1Re0++]
        DB        MMOVL     RAM8X,RAM8X,RAM8X,[A0FrP++]
        DB        MMOVL     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[A0FrP++]
        DB        MMOVL     RAM8X,RAM8X,RAM8X,[D0.7++]
        DB        MMOVL     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[D0.7++]
        DB        MMOVL     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[D0.7++]
        DB        MMOVL     RAM8X,RAM8X,[A0.8++]
        DB        MMOVL     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[A0.8++]
        DB        MMOVL     RAM8X,RAM8X,RAM8X,[D1.16++]
        DB        MMOVL     RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,RAM8X,[D1.16++]
        DBH       MMOVL     RABZ,RABZ,RABZ,RABZ,[D1Re0++]
        DBH       MMOVL     RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,[D1Re0++]
        DBH       MMOVL     RABZ,RABZ,RABZ,RABZ,RABZ,[A0FrP++]
        DBH       MMOVL     RABZ,RABZ,[D0.7++]
        DBH       MMOVL     RABZ,RABZ,RABZ,RABZ,[D0.7++]
        DBH       MMOVL     RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,[D0.7++]
        DBH       MMOVL     RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,[D0.7++]
        DBH       MMOVL     RABZ,RABZ,RABZ,RABZ,[A0.8++]
        DBH       MMOVL     RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,RABZ,[A0.8++]
        DBH       MMOVL     RABZ,RABZ,RABZ,RABZ,RABZ,[D1.16++]
        DW        MMOVL     RAMX,RAMX,[D1Re0++]
        DW        MMOVL     RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,[D1Re0++]
        DW        MMOVL     RAMX,RAMX,RAMX,[A0FrP++]
        DW        MMOVL     RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,[A0FrP++]
        DW        MMOVL     RAMX,RAMX,RAMX,[D0.7++]
        DW        MMOVL     RAMX,RAMX,RAMX,RAMX,RAMX,[D0.7++]
        DW        MMOVL     RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,[D0.7++]
        DW        MMOVL     RAMX,RAMX,[A0.8++]
        DW        MMOVL     RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,[A0.8++]
        DW        MMOVL     RAMX,RAMX,RAMX,[D1.16++]
        DW        MMOVL     RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,RAMX,[D1.16++]
        DW        MMOVD     D0Re0,D0Ar6,D0Ar4,RD
        DW        MMOVD     D0Re0,D0Ar6,D0Ar4,D0.7,RD
        DW        MMOVD     A1LbP,A1.2,A1.3,RD
        DW        MMOVD     D1Ar5,D1Ar3,D1.7,RD
        DW        MMOVD     A1LbP,A1.2,A1.8,RD
        DW        MMOVD     A1LbP,A1.2,A1.7,A1.8,RD
        DW        MMOVD     A1.2,A1.7,RD
        DW        MMOVD     A1.2,A1.3,A1.8,RD
        DW        MMOVD     A1.3,A1.8,RD
        DW        MMOVL     D0Re0,D0Ar4,RD
        DW        MMOVL     D0Re0,D0Ar4,D0.7,RD
        DW        MMOVL     A1LbP,A1.3,RD
        DW        MMOVL     A1LbP,A1.2,A1.7,RD
        DW        MMOVL     A1LbP,A1.8,RD
        DW        MMOVL     A1LbP,A1.7,A1.8,RD
        DW        MMOVL     A1.2,A1.3,RD
        DW        MMOVL     A1.2,A1.8,RD
        DW        MMOVL     A1.3,A1.7,RD
